/*
 * Created on 03 mai. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.bonita;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.Document;

//Bonita imports
import hero.interfaces.*;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSession;


/**
 * The aim of this class is to transform Bonita activity data to a xml Document
 */
public class BonitaActivityAdapter {

	private ArrayList projectKeys;
	private ArrayList activityKeys;

	/**
	 * Default class constructor 
	 */
	public BonitaActivityAdapter(){};	
	
	/**
	 * Provides a Document output for Bonita data
	 * @param theHashMap : The table containing the properties.
	 * @param theProjectName : The project concerned.
	 * @param theProjectName the activity concerned in the project
	 * @return Document document with the activity information : project name & activity name & activity properties.
	 * @throws IOExecption
	 */
	private Document generateBonitaDataDocument(HashMap projectHashMap, HashMap activityHashMap, String theProjectName, String theActivityName) 
		throws IOException{
		
		//DOM
		Document document = null;
		
		//a parser to generate the xml document from the dataProcess
		DocumentParser parser = new DocumentParser(); 
		
		//Create the dataProcess structure with bonita key value
		DataActivity data = new DataActivity(theProjectName, theActivityName, projectHashMap, activityHashMap);
			
		//Generate the xml document representing the dataProcess
		if( data != null )
			document = parser.createDocument(data, projectKeys, activityKeys);	
			
		return document;
	}
	
	
	
	
	/**
	 * Get the Bonita activity data identified by its name and put them in a DOM 
	 * @param theProjectName the name of the project we have to get process informations
	 * @param theActivityName the name of the activityname concerned
	 * @return the document that represents bonita activity data
	 * @throws LoginException
	 * @throws IOException
	 */
	public Document getActivityData(String theProjectName, String theActivityName, String theProjectVersion) 
		throws IOException{
		
		//SET PROJECT AND ACTIVITY DATA IN HASHMAPS
		
		//H map of project data where 'Bonita key - value' will be put
		HashMap projectMap = new HashMap();
		
		//H map of activity data where 'Bonita key - value' will be put
		HashMap activityMap = new HashMap();
			
		//The local home
		ProjectSessionHome projecth = null;
		
		//The bean
		ProjectSession project  = null;
		ProjectSession model  = null;
		
		//keys arrayLists
		projectKeys = new ArrayList();
		activityKeys = new ArrayList();
		
		try{
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();
			
			//Project creation
			project = projecth.create();
			
			model = projecth.create(); // isAdmin of the model ?
			int i = theProjectName.indexOf("_instance");
			model.initModelWithVersion(theProjectName.substring(0, i),theProjectVersion);
			//model.initProject(theProjectName.substring(0, i));
			//ORIG: model.initProject(theProjectName.substring(0, i));
			
			
			//Project initialisation
			project.initProject(theProjectName);
			//ORIG: project.initProject(theProjectName);
			
			if (!project.isUserInNodeRole(theActivityName) && !model.isAdminOfProject())
				throw new Exception("You are not allowed to execute this activity ! ");
			
			//properties collection
			Collection collectionProperties;
			//colelction iterator
			Iterator it;	
			//The current key of the property value object
			String currentKey = "";
			//The current value of the property value object
			String currentValue  = "";
			String possibleValue = "";
			
			
			//GET PROJECT PROPERTIES
			//Project Property value of Bonita
			BnProjectPropertyValue pv = null;
			collectionProperties = project.getProperties();
			//For all the keys we have to retrieve the associated value
			it = collectionProperties.iterator();
			
			while (it.hasNext())
			{
				//Getting the values
				pv = (BnProjectPropertyValue)it.next();
			
				//Retrieve the key
				currentKey   = pv.getTheKey();
				currentValue = pv.getTheValue();
				if (currentValue==null) currentValue = "";
				//And the value or possible values
 		                Collection possibleValues =  pv.getPossibleValues();
 		                if (possibleValues != null)
		                {
					//Pb with the StringTokenizer if ""
					if (currentValue.equals("")) currentValue=" ";
					//For all the keys we have to retrieve the associated value
					Iterator itValues = possibleValues.iterator();
					// each possible values are added after the value
					while (itValues.hasNext())
					{
	        		       	   	// we add a pipe | after each possible values
		        	       	   	currentValue = currentValue + "|" + (String)itValues.next();
		        	       	}
		                }
				//Adding key to collection to mantain order
				projectKeys.add(currentKey);
				//Adding key and value in a hashMap
				projectMap.put(currentKey, currentValue);
			}
					
			//GET ACTIVITY PROPERTIES
			//Project Property value of Bonita
			BnNodePropertyValue npv = null;
			collectionProperties = project.getNodeProperties(theActivityName);
			
			//For all the keys we have to retrieve the associated value
			it = collectionProperties.iterator();
			
			while (it.hasNext()){
				//Getting the values
				npv = (BnNodePropertyValue)it.next();
			
				//Retrieve the key
				currentKey = npv.getTheKey();
				currentValue = npv.getTheValue();
				if (currentValue==null) currentValue = "";
				//And the value or possible values
 		                Collection possibleValues =  npv.getPossibleValues();
 		                if (possibleValues != null)
		                {
					//Pb with the StringTokenizer if ""
					if (currentValue.equals("")) currentValue=" ";
					//For all the keys we have to retrieve the associated value
					Iterator itValues = possibleValues.iterator();
					// each possible values are added after the value
					while (itValues.hasNext())
					{
		        	       	   	currentValue = currentValue + "|" + (String)itValues.next();
		        	       	}
		                }
				//Adding key to collection to mantain order
				activityKeys.add(currentKey);
				//Adding key and value in a hashMap
				activityMap.put(currentKey, currentValue);
				currentValue = "";
			}
			project.remove();
			model.remove();
		}catch(Exception e){
			try{
				if (project != null)
					project.remove();
			}catch(Exception re){
				System.out.println(re + " " + e.getMessage());
			}
			throw new IOException (e.getMessage()); 
		}

		
		//Document generated
		return this.generateBonitaDataDocument(projectMap, activityMap, theProjectName, theActivityName);
	}
}


