/*
 * Created on 19 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.bonita;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.Document;

import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionUtil;


/**
 * The aim of this class is to transform data from a Bonita project to a xml Document
 */
public class BonitaProjectAdapter {

	private ArrayList keys;


	/**
	 * Default class constructor 
	 */
	public BonitaProjectAdapter(){};
	
	
	

		
	/**
	 * Get the bonita properties as key value hash map
	 * @param theProjectName : The project concerned.
	 * @return HashMap the table containing the properties.
	 */
	private HashMap getHBonitaData(String theProjectName, String theProjectVersion){
				
		//Hash map where 'Bonita key - value' will be put
		HashMap hMap = new HashMap();
			
		//The local home
		ProjectSessionHome projecth = null;
		
		//The bean
		ProjectSession project  = null;
		
		//keys arrayList
		keys = new ArrayList();
		
		try{
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();
			
			//Project creation
			project = projecth.create();			
					  
			project.initModelWithVersion(theProjectName, theProjectVersion);
			project.executeProcessHook();
			
			//We are now able to acess Bonita api and get properties
			Collection collectionProperties = project.getProperties();
			
			//Number of bean instances responding
			
			//Property value of Bonita
			BnProjectPropertyValue pv = null;
			
			//The current key of the property value object
			String currentKey = "";
			
			//The current value of the property value object
			String currentValue  = "";
			String possibleValue = "";

			//For all the keys we have to retrieve the associated value
			Iterator it = collectionProperties.iterator();
			
			while (it.hasNext())
			{
				//Getting the values
				pv = (BnProjectPropertyValue)it.next();
			
				//Retrieve the key
				currentKey   = pv.getTheKey();
				currentValue = pv.getTheValue();
				if (currentValue==null) currentValue = "";
				//And the value or possible values
 		           Collection possibleValues =  pv.getPossibleValues();
 		           if (possibleValues != null)
		           {
					//Pb with the StringTokenizer if ""
					if (currentValue.equals("")) currentValue=" ";
					//For all the keys we have to retrieve the associated value
					Iterator itValues = possibleValues.iterator();
					// each possible values are added after the value
					while (itValues.hasNext())
					{
	        		       	   	// we add a pipe | after each possible values
		        	       	   	currentValue = currentValue + "|" + (String)itValues.next();
		        	       	}
		                }
				//Adding key to collection to mantain order
				keys.add(currentKey);
				//Adding key and value in a hashMap
				hMap.put(currentKey, currentValue);
			}
			project.remove();
			//Returns the hash map
			return hMap;
		}
		catch(Exception e){
			try{
				if (project != null)
					project.remove();
			}catch(Exception re){
				System.out.println(re + " " + e.getMessage());
			}
			System.out.println(e + " " + e.getMessage());
			return null;
		}
	}
	
	
	

	
	/**
	 * Provides a Document output for Bonita data
	 * @param theHashMap : The table containing the properties.
	 * @param theProjectName : The project concerned.
	 * @return Document document with the process information : name & properties.
	 * @throws IOExecption
	 */
	private Document generateBonitaDataDocument(HashMap theHashMap, String theProjectName) throws IOException{
		
		//DOM
		Document document = null;
		
		//a parser to generate the xml document from the dataProcess
		DocumentParser parser = new DocumentParser(); 
		
		//Create the dataProcess structure with bonita key value
		DataProject data = new DataProject(theProjectName, theHashMap);
			
		//Generate the xml document representing the dataProcess
		if( data != null )
			document = parser.createDocument(data, keys);	
			
		return document;
	}
	
	
	
	
	/**
	 * Get the Bonita project data identified by its name and put them in a DOM 
	 * @param theProjectName : The name of the project we have to get process informations
	 * @return the document that represents bonita data
	 * @throws LoginException
	 * @throws IOException
	 */
	public Document getProjectData(String theProjectName, String theProjectVersion) throws IOException{
		
		//H map of Bonita data
		HashMap bonitaMap = new HashMap();
		
		//Set Bonita data in H map
		bonitaMap = this.getHBonitaData(theProjectName, theProjectVersion);
		
		//Document generated
		return this.generateBonitaDataDocument(bonitaMap, theProjectName);
	}
}


