/*
 * Created on 21 avr. 2004
 *
 * This class creates a Bonita process instance
 * thanks to the process parameters given in a XML file
 *
 */
package mc.formgenerator.bonita;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Collection;
import java.util.Set;
import java.util.Vector;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import hero.interfaces.*;
import hero.util.HeroException;

/**
 * Given instance document with properties, initializes a project
 */
public class BonitaProjectExecutor {



	/**
	 * Default class constructor
	 *
	 */
	public BonitaProjectExecutor() {
	}



	/**
	 * Instanciate a process thanks to its data which are in a XML file.
	 * @param fichier XML file with the process information.
	 * @throws MalformedURLException
	 * @throws SAXException
	 * @throws IOException
	 * @throws LoginException
	 */
	public void startBonitaProject(File fichier, Vector req_parameters, boolean formsExists, String version)
		throws MalformedURLException, SAXException, IOException, Exception {


		//**********************************************************************
		//						FILE --> DOCUMENT
		//**********************************************************************

		//A parser is needed to read an XML file
		DOMParser parser = new DOMParser();

		//the parser parse the XML file
		parser.parse(fichier.toURL().toString());

		//get a Document which represents the File. It's to normalyse the data manipulated
		Document document = parser.getDocument();



		//'parse' transforms a Document into a DataProcess
		DocumentParser parse = new DocumentParser();

		DataProject data = parse.parseProject(document, req_parameters, formsExists);

		//wants the process name
		String name = data.getProjectName();

		//wants the process properties
		HashMap table = data.getProjectProperties();

		//start the process
		instanciateProject(name, version, table);

	}






	/**
	 * Instanciate a process thanks to its data which are in a Document.
	 * @param document Document with the process information.
	 * @throws MalformedURLException
	 * @throws SAXException
	 * @throws IOException
	 * @throws LoginException
	 */
	public String startBonitaProject(Document document, Vector req_parameters, boolean formsExists, String version )
		throws MalformedURLException, SAXException, IOException, Exception {

		String instance = null;

		//'parse' transforms a Document into a DataProcess
		DocumentParser parse = new DocumentParser();

		DataProject data = parse.parseProject(document, req_parameters, formsExists);
		
		//get the process name & version
		String name = data.getProjectName();

		//get the process properties
		HashMap table = data.getProjectProperties();

		//start the process thanks to its name, version and properties
		instance = instanciateProject(name, version, table);
		return instance;
	}





	/**
	 * Instanciate the process thanks to its name and its parameters which are in a XML file
	 * @param processName Name of the process to instanciate.
	 * @param table HashMap with 'key value' representing the process properties. 'Key' is the property name and 'Value' is its value.
	 * @throws LoginException
	 */
	private String instanciateProject(String processName, String processVersion, HashMap table) throws Exception {

		//The local home
		ProjectSessionHome projecth = null;

		//The bean
		ProjectSession project = null;

		//The instance name
		String instance = null;

		try {

			//**************************************************************************
			//					INSTANCIATE THE PROCESS
			//**************************************************************************

			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();

			//Creation of the project
			project = projecth.create();

			//Project instanciation
			if (processVersion==null)
				instance = project.instantiateProject(processName,new Hashtable(table));
			else
				instance = project.instantiateProject(processName,processVersion,new Hashtable(table));
			//project.initProject(processName);

			project.remove();

		} catch (Exception e) {
			try{
				if (project != null)
					project.remove();
			}catch(Exception re){
				System.out.println(re + " " + e.getMessage());
			}
			//System.out.println(e.getMessage());
			throw new Exception(e.getMessage());
		}
		return instance;
	}



}
