/*
 * Created on 1 april 2005
 *
 */
package mc.formgenerator.servlets.bonita;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import mc.formgenerator.servlets.util.FormGeneratorConstant;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Attr;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;


/**
 * This class is a facade class to operate with XForm files.
 *
 */
public class RepositoryFacade {


	/**
	 * Attribute which stores the repository document
	 */
	private Document xmlDocument;
	private String path;


	/**
	 * Default class constructor
	 */
	public RepositoryFacade(String xmlPath) {
		DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();

		try {
			//get file and parse it to xmlDocument
			File xmlFile = new File(xmlPath);
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			this.xmlDocument = documentBuilder.parse(xmlFile);
			this.path = xmlPath;
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}



	/**
	 * Returns the path of the XForm file for a project.
	 * @param projectName: the project's name
	 * @param projectVersion: the project's version
	 * @return String formpath.
	 * @throws Exception
	 */
	public String getProjectFormPath (String projectName, String projectVersion){

		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			return null;
		}
		else {
			//search projectName
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				//get project name
				NodeList elementsName = projectElement.getElementsByTagName("projectname");
				Text nameElement = (Text)elementsName.item(0).getFirstChild();
				String name = nameElement.getNodeValue();

				if(name.equals(projectName)){
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {
        					//if project name has been found in repository, lookup its xform filename
        					NodeList elementsXformFile = projectElement.getElementsByTagName("projectxformfile");
        					Text xformpathElement = (Text)elementsXformFile.item(0).getFirstChild();
        					if(xformpathElement!=null){
        						//this project has an xform file associated, then return file path
        						String path = File.separator + "web" + File.separator + "xforms" + File.separator + projectName + File.separator + projectVersion + File.separator + xformpathElement.getNodeValue();
        //debug
        //System.out.println("RepositoryFacade ====> Project "+ projectName + " has been found in repository. XForm file: " + path);
        						return (path);
        					}else return null;
                    }
                }

			}
		}

		return null;
	}


	/**
	 * Updates project name.
	 * @param oldProjectName: the old project's name
     * @param newProjectName: the new project's name
     * @param projectVersion: the project's version
	 */
	public void updateProjectName (String oldProjectName, String newProjectName, String projectVersion) throws Exception{

		//get "project" nodes
		Element root = this.xmlDocument.getDocumentElement();
		NodeList nodeList = root.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			System.out.println("WARNING: RepositoryFacade ==> repository.xml has no projects");
		}
		else {
			//search projectName
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				//get project name
				NodeList elementsName = projectElement.getElementsByTagName("projectname");
				Text nameElement = (Text)elementsName.item(0).getFirstChild();
				String name = nameElement.getNodeValue();

				if(name.equals(oldProjectName)){
				    //get project version
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {
        					//if project name & version has been found in repository, change name
        					nameElement.setNodeValue(newProjectName);
                    }
				}
			}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();
	}

	/**
	 * Remove project form path from repository.
	 * @param projectName: the project's name
     * @param projectVersion: the project's version
     * Warning: This method is not used by proed at now !
	 */
	public void removeProjectFormPath (String projectName, String projectVersion) throws Exception{
		//get "project" nodes
		Element root = this.xmlDocument.getDocumentElement();
		NodeList nodeList = root.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			System.out.println("WARNING: RepositoryFacade ==> repository.xml has no projects");
		}
		else {
			//search projectName
			for (int i=0;i<nodeList.getLength();i++){
				Node projectElement = nodeList.item(i);
				//get project name
				NodeList elementsName = ((Element)projectElement).getElementsByTagName("projectname");
				Text nameElement = (Text)elementsName.item(0).getFirstChild();
				String name = nameElement.getNodeValue();

				if(name.equals(projectName)){
                    NodeList elementsVersion = ((Element)projectElement).getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {
        					//if project name has been found in repository, remove project node
        					root.removeChild(projectElement);
        //debug
        //System.out.println("RepositoryFacade ====> Project "+ projectName + " has been removed from repository.xml");
        					break;
                    }
				}
			}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();
	}


	/**
	 * Returns the path of the XForm file for an activity.
	 * @param projectName: the project's name
	 * @param activityName: the activity's name
     * @param projectVersion: the project's version
	 * @return String formpath.
	 * @throws Exception
	 */
	public String getActivityFormPath (String projectName, String activityName, String projectVersion){

		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			return null;
		}
		else {
			//search project name
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				NodeList pElementsName = projectElement.getElementsByTagName("projectname");
				Text pNameElement = (Text)pElementsName.item(0).getFirstChild();
				String pName = pNameElement.getNodeValue();

				if(pName.equals(projectName)){
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {

        					//if project name & version  has been found in repository, lookup its xform filename
        					NodeList elementsActivities = projectElement.getElementsByTagName("activities");
        					Element activitiesElement = (Element)elementsActivities.item(0);
        					//get "activity" nodes
        					NodeList activitiesNodeList = activitiesElement.getElementsByTagName("activity");
        					int activitiesSize = activitiesNodeList.getLength();
        					if (activitiesSize == 0) {
        						return null;
        					}
        					else {
        						for (int j=0;j<activitiesNodeList.getLength();j++){
        							Element activityElement = (Element)activitiesNodeList.item(j);
        							NodeList aElementsName = activityElement.getElementsByTagName("activityname");
        							Text aNameElement = (Text)aElementsName.item(0).getFirstChild();
        							String aName = aNameElement.getNodeValue();

        							if(aName.equals(activityName)){
        								//if activity name has been found in repository, lookup its xform filename
        								NodeList elementsXformFile = projectElement.getElementsByTagName("activityxformfile");
        								Text xformpathElement = (Text)elementsXformFile.item(0).getFirstChild();
        								if(xformpathElement!=null){
        									//this activity has an xform file associated, then return filename
        									String path = File.separator + "web" + File.separator + "xforms" + File.separator + projectName + File.separator + projectVersion + File.separator + activityName + File.separator + xformpathElement.getNodeValue();
        									return (path);
        								}else return null;
        							}
        						}
        					}
                    }
				}
			}
		}
		return null;
	}


	/**
	 * Updates activiy name in repository.
	 * @param projectName: the project's name
     * @param projectVersion: the project's version
	 * @param activityName: the activity's name
	 */
	public void updateActivityName (String projectName, String projectVersion, String oldActivityName, String newActivityName) throws Exception{
		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			System.out.println("WARNING: RepositoryFacade ==> Project " + projectName + "not found");
		}
		else {
			//search project name
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				NodeList pElementsName = projectElement.getElementsByTagName("projectname");
				Text pNameElement = (Text)pElementsName.item(0).getFirstChild();
				String pName = pNameElement.getNodeValue();

				if(pName.equals(projectName)){
				    //get project version
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {
        					//if project name has been found in repository, lookup its xform filename
        					NodeList elementsActivities = projectElement.getElementsByTagName("activities");
        					Element activitiesElement = (Element)elementsActivities.item(0);
        					//get "activity" nodes
        					NodeList activitiesNodeList = activitiesElement.getElementsByTagName("activity");
        					int activitiesSize = activitiesNodeList.getLength();
        					if (activitiesSize == 0) {
        						System.out.println("WARNING: RepositoryFacade ==> Activity " + oldActivityName + "not found");
        						break;
        					}
        					else {
        						for (int j=0;j<activitiesNodeList.getLength();j++){
        							Element activityElement = (Element)activitiesNodeList.item(j);
        							NodeList aElementsName = activityElement.getElementsByTagName("activityname");
        							Text aNameElement = (Text)aElementsName.item(0).getFirstChild();
        							String aName = aNameElement.getNodeValue();

        							if(aName.equals(oldActivityName)){
        								//if activity name has been found in repository, change name
        								aNameElement.setNodeValue(newActivityName);
        //System.out.println("RepositoryFacade ==> Activity " + oldActivityName + "updated to " + newActivityName);
        								break;
        							}
        						}
        					}
                    }
				}
			}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();
	}


	/**
	 * Removes activiy form path from repository.
	 * @param projectName: the project's name
     * @param projectVersion: the project's version
	 * @param activityName: the activity's name
	 */
	public void removeActivityFormPath (String projectName, String projectVersion, String activityName) throws Exception{
		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
						System.out.println("WARNING: RepositoryFacade ==> Project " + projectName + "not found");
		}
		else {
			//search project name
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				NodeList pElementsName = projectElement.getElementsByTagName("projectname");
				Text pNameElement = (Text)pElementsName.item(0).getFirstChild();
				String pName = pNameElement.getNodeValue();

				if(pName.equals(projectName)){
				    //if project name has been found
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                        //if project version has been found
                        if (version.equals(projectVersion)) {
        					//if project name & version has been found in repository, lookup its xform filename
        					NodeList elementsActivities = projectElement.getElementsByTagName("activities");
        					Element activitiesElement = (Element)elementsActivities.item(0);
        					//get "activity" nodes
        					NodeList activitiesNodeList = activitiesElement.getElementsByTagName("activity");
        					int activitiesSize = activitiesNodeList.getLength();
        					if (activitiesSize == 0) {
        							System.out.println("WARNING: RepositoryFacade ==> Activity " + activityName + "not found");
        						break;
        					}
        					else {
        						for (int j=0;j<activitiesNodeList.getLength();j++){
        							Element activityElement = (Element)activitiesNodeList.item(j);
        							NodeList aElementsName = activityElement.getElementsByTagName("activityname");
        							Text aNameElement = (Text)aElementsName.item(0).getFirstChild();
        							String aName = aNameElement.getNodeValue();

        							if(aName.equals(activityName)){
        								//if activity name has been found in repository, remove activity node
        								activitiesElement.removeChild(activityElement);
        //System.out.println("RepositoryFacade ==> Activity " + activityName + "removed from repository");
        								break;
        							}
        						}
        					}
                }
			}
		}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();
	}


	/**
	 * Sets the path of the XForm file for a project.
	 * @param projectName: the project's name
     * @param projectVersion: the project's version
	 * @throws Exception
	 */
	public void setProjectFormPath (String projectName, String projectVersion) throws Exception {
		String formPath = null;

		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
        // case No project
		if (size == 0) {
			//if does not exist any project in repository create new project node
			Element root = this.xmlDocument.getDocumentElement();
			Element project = this.xmlDocument.createElement("project");
			//projectname element
			Element projectname = this.xmlDocument.createElement("projectname");
			Text projectnameText = this.xmlDocument.createTextNode(projectName);
			projectname.appendChild(projectnameText);
			project.appendChild(projectname);
			//MBL: projectversion element
            Element projectversion = this.xmlDocument.createElement("projectversion");
            Text projectversionText = this.xmlDocument.createTextNode(projectVersion);
            projectversion.appendChild(projectversionText);
            project.appendChild(projectversion);

			Element activities = this.xmlDocument.createElement("activities");
			project.appendChild(activities);
			Element projectxformfile = this.xmlDocument.createElement("projectxformfile");
			Text projectxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
			projectxformfile.appendChild(projectxformfileText);
			project.appendChild(projectxformfile);
			root.appendChild(project);
		}
		else {
			//search projectName
			boolean exist = false;
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				//get project name
				NodeList elementsName = projectElement.getElementsByTagName("projectname");
				Text nameElement = (Text)elementsName.item(0).getFirstChild();
				String name = nameElement.getNodeValue();

				if(name.equals(projectName)){
				    //get project version
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {

					NodeList elementsXformfile = projectElement.getElementsByTagName("projectxformfile");
					setStringValue(elementsXformfile.item(0),"xform.xhtml");
					//Text xformFileElement = (Text)elementsXformfile.item(0).getFirstChild();
					//xformFileElement.setNodeValue("xform.xhtml");
					exist = true;
                    }
				}

			}
			//if project name has not been found in repository, create new project node
			if(!exist){
				Element root = this.xmlDocument.getDocumentElement();
				Element project = this.xmlDocument.createElement("project");
                //projectname element
				Element projectname = this.xmlDocument.createElement("projectname");
				Text projectnameText = this.xmlDocument.createTextNode(projectName);
				projectname.appendChild(projectnameText);
				project.appendChild(projectname);
                //MBL: projectversion element
                Element projectversion = this.xmlDocument.createElement("projectversion");
                Text projectversionText = this.xmlDocument.createTextNode(projectVersion);
                projectversion.appendChild(projectversionText);
                project.appendChild(projectversion);

				Element activities = this.xmlDocument.createElement("activities");
				project.appendChild(activities);
				Element projectxformfile = this.xmlDocument.createElement("projectxformfile");
				Text projectxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
				projectxformfile.appendChild(projectxformfileText);
				project.appendChild(projectxformfile);
				root.appendChild(project);
			}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();


		///String xml = toPrettyXmlString(this.xmlDocument);
		///BufferedWriter br = new BufferedWriter(new OutputStreamWriter(output));
		//br.write(xml,0,xml.length());
		//br.close();
	}

	/**
	 * Sets the path of the XForm file for an activity.
	 * @param projectName: the project's name
     * @param projectVersion: the project's version
	 * @param activityName: the activity's name
	 * @throws Exception
	 */
	public void setActivityFormPath (String projectName, String projectVersion, String activityName) throws Exception {
		String formPath = null;

		//get "project" nodes
		NodeList nodeList = this.xmlDocument.getElementsByTagName("project");
		int size = nodeList.getLength();
		if (size == 0) {
			//if does not exist any project in repository create new project node
			Element root = this.xmlDocument.getDocumentElement();
			Element project = this.xmlDocument.createElement("project");
			//projectname
			Element projectname = this.xmlDocument.createElement("projectname");
			Text projectnameText = this.xmlDocument.createTextNode(projectName);
			projectname.appendChild(projectnameText);
			project.appendChild(projectname);
			//projectversion element
            Element projectversion = this.xmlDocument.createElement("projectversion");
            Text projectversionText = this.xmlDocument.createTextNode(projectVersion);
            projectversion.appendChild(projectversionText);
            project.appendChild(projectversion);
			//activities
			Element activities = this.xmlDocument.createElement("activities");
			//activity
			Element activity = this.xmlDocument.createElement("activity");
			Element activityname = this.xmlDocument.createElement("activityname");
			Text activitynameText = this.xmlDocument.createTextNode(activityName);
			activityname.appendChild(activitynameText);
			activity.appendChild(activityname);
			Element activityxformfile = this.xmlDocument.createElement("activityxformfile");
			Text activityxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
			activityxformfile.appendChild(activityxformfileText);
			activity.appendChild(activityxformfile);
			activities.appendChild(activity);
			project.appendChild(activities);
			//end acitivity
			//project xformfile
			Element projectxformfile = this.xmlDocument.createElement("projectxformfile");
			project.appendChild(projectxformfile);
			root.appendChild(project);
		}
		else {
			//search project name
			boolean projectExist = false;
			boolean activityExist = false;
			for (int i=0;i<nodeList.getLength();i++){
				Element projectElement = (Element)nodeList.item(i);
				NodeList pElementsName = projectElement.getElementsByTagName("projectname");
				Text pNameElement = (Text)pElementsName.item(0).getFirstChild();
				String pName = pNameElement.getNodeValue();

				if(pName.equals(projectName)){

                    // if project version has been found
                    NodeList elementsVersion = projectElement.getElementsByTagName("projectversion");
                    Text versionElement = (Text)elementsVersion.item(0).getFirstChild();
                    String version = versionElement.getNodeValue();

                    if (version.equals(projectVersion)) {



            					//if project name has been found in repository, lookup its activities
            					NodeList elementsActivities = projectElement.getElementsByTagName("activities");
            					Element activitiesElement = (Element)elementsActivities.item(0);
            					//get "activity" nodes
            					NodeList activitiesNodeList = activitiesElement.getElementsByTagName("activity");
            					int activitiesSize = activitiesNodeList.getLength();
            					if (activitiesSize == 0) {
            						//if does not exist any activity in repository create new activity node
            						Element activity = this.xmlDocument.createElement("activity");
            						Element activityname = this.xmlDocument.createElement("activityname");
            						Text activitynameText = this.xmlDocument.createTextNode(activityName);
            						activityname.appendChild(activitynameText);
            						activity.appendChild(activityname);
            						Element activityxformfile = this.xmlDocument.createElement("activityxformfile");
            						Text activityxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
            						activityxformfile.appendChild(activityxformfileText);
            						activity.appendChild(activityxformfile);
            						activitiesElement.appendChild(activity);
            						activityExist = true;
            					}else {
            						for (int j=0;j<activitiesNodeList.getLength();j++){
            							Element activityElement = (Element)activitiesNodeList.item(j);
            							NodeList aElementsName = activityElement.getElementsByTagName("activityname");
            							Text aNameElement = (Text)aElementsName.item(0).getFirstChild();
            							String aName = aNameElement.getNodeValue();

            							if(aName.equals(activityName)){
            								activityExist = true;
            							}
            						}
            						if(!activityExist){
            							//if does not exist this activity in repository create new activity node
            							Element activity = this.xmlDocument.createElement("activity");
            							Element activityname = this.xmlDocument.createElement("activityname");
            							Text activitynameText = this.xmlDocument.createTextNode(activityName);
            							activityname.appendChild(activitynameText);
            							activity.appendChild(activityname);
            							Element activityxformfile = this.xmlDocument.createElement("activityxformfile");
            							Text activityxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
            							activityxformfile.appendChild(activityxformfileText);
            							activity.appendChild(activityxformfile);
            							activitiesElement.appendChild(activity);
            							activityExist = true;
            						}
            					}
            					projectExist = true;
                    }
            		}
			}
			if(!projectExist){
				//if does not exist this project in repository create new project node
				Element root = this.xmlDocument.getDocumentElement();
				Element project = this.xmlDocument.createElement("project");
				//projectname
				Element projectname = this.xmlDocument.createElement("projectname");
				Text projectnameText = this.xmlDocument.createTextNode(projectName);
				projectname.appendChild(projectnameText);
				project.appendChild(projectname);
				//projectversion element
                Element projectversion = this.xmlDocument.createElement("projectversion");
                Text projectversionText = this.xmlDocument.createTextNode(projectVersion);
                projectversion.appendChild(projectversionText);
                project.appendChild(projectversion);
				//activities
				Element activities = this.xmlDocument.createElement("activities");
				//activity
				Element activity = this.xmlDocument.createElement("activity");
				Element activityname = this.xmlDocument.createElement("activityname");
				Text activitynameText = this.xmlDocument.createTextNode(activityName);
				activityname.appendChild(activitynameText);
				activity.appendChild(activityname);
				Element activityxformfile = this.xmlDocument.createElement("activityxformfile");
				Text activityxformfileText = this.xmlDocument.createTextNode("xform.xhtml");
				activityxformfile.appendChild(activityxformfileText);
				activity.appendChild(activityxformfile);
				activities.appendChild(activity);
				project.appendChild(activities);
				//end acitivity
				//project xformfile
				Element projectxformfile = this.xmlDocument.createElement("projectxformfile");
				project.appendChild(projectxformfile);
				root.appendChild(project);
			}
		}
		//save repository file
//System.out.println("RepositoryFacade ==> SAVE PATH = " + this.path);
		FileOutputStream output = new FileOutputStream(this.path);
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
		of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = new XMLSerializer(output,of);
		serializer.asDOMSerializer();
		serializer.serialize(this.xmlDocument.getDocumentElement() );
		output.close();

		//FileOutputStream output = new FileOutputStream(this.path);
		//String xml = toPrettyXmlString(this.xmlDocument);
		//BufferedWriter br = new BufferedWriter(new OutputStreamWriter(output));
		//br.write(xml,0,xml.length());
		//br.close();
	}

	private static String toPrettyXmlString(Document doc) {
		StringBuffer buffer = new StringBuffer(200);
		toPrettyXmlString(doc.getDocumentElement(), 0, buffer);
		return buffer.toString();
	}

	private static void toPrettyXmlString(Node elmnt, int level, StringBuffer buffer)
	{
		String piece;
		int i;

		buffer.append("<?xml version=\"1.0\" encoding=\"" + FormGeneratorConstant.DEBUG_OUTPUT_FORMAT_ENCODING + "\"?>");
		buffer.append("\n");

		for (i = 0; i < level; i++)
			buffer.append("    ");

		buffer.append("<");


		buffer.append(elmnt.getNodeName());
		NamedNodeMap atts = elmnt.getAttributes();
		Node node;

		for (i = 0; i < atts.getLength(); i++) {
			node = atts.item(i);
			piece = " " + node.getNodeName() + "=\""
				+ node.getNodeValue() + "\"";

			if ((buffer.length() - (buffer.toString().lastIndexOf('\n') + 1))
				+ piece.length() > 78) {
				buffer.append("\n");
				for (int j = 0; j < (level + 1); j++)
					buffer.append("    ");
			}

			buffer.append(piece);
		}

		// Process child nodes:

		NodeList kids = elmnt.getChildNodes();

		int n = kids.getLength();

		Node kid;

		boolean needIndent = false;

		buffer.append(n < 1 ? "/>" : ">");

		//if (n > 1) xml += "\n";
		if (!(elmnt.getFirstChild() instanceof Text)) {
			//			xml += "\n";
		}

		for (i = 0; i < n; i++) {
			kid = kids.item(i);

			if (kid instanceof Element) {
				needIndent = true;
				buffer.append("\n");
				toPrettyXmlString((Element)kid, level + 1, buffer);
			} else if (kid instanceof Text) {
				buffer.append(kid.getNodeValue());
			} else if (kid instanceof Comment) {
				if (i > 0 && kids.item(i - 1) instanceof Text) {
					if (kids.item(i - 1).getNodeValue().trim().length() < 1) {
						buffer.append(kids.item(i - 1).getNodeValue());
					}
				}
				buffer.append("<!--" + kid.getNodeValue() + "-->");
			} else {
//System.out.println(kid.getClass().getName());
			}
		}

		// End-tag:
		if (n > 0) {
			if (n > 1 || (n == 1 && !(elmnt.getFirstChild() instanceof Text)))
				buffer.append("\n");

			if (needIndent)
				for (i = 0; i < level; i++)
					buffer.append("    ");

			buffer.append("</");

			buffer.append(elmnt.getNodeName() + ">");
		}
	}

	private static void setStringValue(Node node, String value) {
		if (node == null) {
			System.out.println("WARNING:  No node .....");
		} else if (node instanceof Attr) {
			node.setNodeValue(value);
		} else {
			Node n;
			NodeList nodes = node.getChildNodes();

			for (int i = 0; i < nodes.getLength(); i++) {
				n = nodes.item(i);

				if (n instanceof Text) {
					n.setNodeValue(value);
					return ;
				}
			}

			Document document = node.getOwnerDocument();
			Text text = document.createTextNode(value);
			node.appendChild(text);
		}
	}

}
