/*
 * Created on September 2004, the 16th
 * By the Message Center Team
 */
package mc.formgenerator.servlets.bonita;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

import javax.servlet.http.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import mc.formgenerator.servlets.util.FormGeneratorConstant;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Attr;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;

/**
 * Servlet for XForm Editor to get or write XForm documents
 *
 */
public class ServletXformEditor extends HttpServlet {

	private RepositoryFacade repositoryFacade = null;
	private String contextPath = null;
	private String projectName = null;
	private String projectVersion = null;
	private String activityName = null;
	private String lang1 = null;
	private String lang2 = null;
	private String lang3 = null;


	/**
	 * Post request
	 * @throws IOException
	 */
	public void doPost(HttpServletRequest req, HttpServletResponse res) throws IOException  {

		//Get the session of the user.
		//HttpSession session = req.getSession(true);

		// set content-type and charset for the request/response
		res.setContentType("text/html; charset=" + FormGeneratorConstant.CHARACTER_ENCODING);
		req.setCharacterEncoding(FormGeneratorConstant.CHARACTER_ENCODING);

		//get parameters
		String xmlRequest = req.getParameter("xml");
		this.projectName = req.getParameter("projectname");
		this.projectVersion = req.getParameter("projectversion");
		this.activityName = req.getParameter("activityname");
		this.lang1 = req.getParameter("lang1");
		this.lang2 = req.getParameter("lang2");
		this.lang3 = req.getParameter("lang3");

		this.contextPath = getServletConfig().getServletContext().getRealPath("");

		PrintWriter out = res.getWriter( );

		try{
			//Create the repository facade corresponding to this controller
			this.repositoryFacade = new RepositoryFacade(getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator+ "repository.xml");
			if ((repositoryFacade == null))
				System.out.println("ERROR: ServletXformEditor - Invalid session");
			else{
				if (xmlRequest.equals("")){
					Document xmlResponse = createXmlResponse(xmlRequest);
					//System.out.println("ServletXformEditor ==> XML Response = " + toXmlString(xmlResponse));
					String xmlResponseString = toXmlString(xmlResponse);
					//clean response
					//xmlResponseString = xmlResponseString.replaceAll("\n","");
					//xmlResponseString = xmlResponseString.replaceAll("\t","");
					//System.out.println("ServletXformEditor ==> CLEANED XML Response = " + xmlResponseString);
					out.print(xmlResponseString);
				}else{
					//build documents, save them and update repository
					String response = writeDocuments(xmlRequest);
					out.print(response);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,	e.getMessage());
		}
	}


	//	lookup in repository and build xmlresponse
	/**  17-05-2005
	 *   Lookup in repository and build xmlresponse
	 *   <p/>
	 *
	 *   @param xmlRequest xml string with xform and languages documents.
	 */
	public Document createXmlResponse(String xmlRequest) {

	 	Document xmlResponse = null;	//XML Document with XForm and Languages
		try{
			DOMImplementation domImpl = new DOMImplementationImpl();

			//XForm Document creation with the root element called servletResponse
//System.out.println("ServletXformEditor ==> Creating xform document...");
			xmlResponse = domImpl.createDocument(null, "servletResponse" , null);
			Element root = xmlResponse.getDocumentElement();

			String realProjectName = this.projectName.replaceFirst("_instance.+","");
			String formPath = null;
			String langPath = null;
			String projectLangPath = null;
			//lookup for a formPath
			if(this.activityName.equals("")){
				//working with project
				formPath = repositoryFacade.getProjectFormPath(realProjectName, this.projectVersion);
				langPath = File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.projectVersion + File.separator;
			}else{
				//working with activity
				formPath = repositoryFacade.getActivityFormPath(realProjectName,activityName,this.projectVersion);
				//versionning xforms : adding the projectVersion
                langPath = File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.projectVersion + File.separator + this.activityName + File.separator;

				//langPath = File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.activityName + File.separator;
				//if exist xform file for activity's project, get project languages path
				String tmp = repositoryFacade.getProjectFormPath(realProjectName, this.projectVersion);
				if(tmp != null){
					projectLangPath = File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.projectVersion + File.separator;
				}
			}

			//documentBuilder to get documents
			DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			if(formPath != null){
				//XForm file exist. Get files, parse them to xml Documents and build xmlResponse

				//xform
				File xformFile = new File(this.contextPath + formPath);
				Document xformDocument = documentBuilder.parse(xformFile);
				//Element xform = xmlResponse.createElement("xform");
				Node html = xformDocument.getDocumentElement();
				root.appendChild(xmlResponse.importNode(html, true));

				//lang1
				File lang1File = new File(this.contextPath + langPath + this.lang1 + ".xml");
				Document lang1Document = documentBuilder.parse(lang1File);
				//Element lang1 = xmlResponse.createElement("lang1");
				Node lang1 = lang1Document.getDocumentElement();
				if(projectLangPath!=null){
					//We are working with an activity whose project has xform file.
					//Include project name into language xml.
					//If string project name exist, modify it (maybe it has been changed!).
					File lang1ProjectFile = new File(this.contextPath + projectLangPath + this.lang1 + ".xml");
					Document lang1ProjectDocument = documentBuilder.parse(lang1ProjectFile);
					Node lang1Project = lang1ProjectDocument.getDocumentElement();
					NodeList projectNodeList = lang1Project.getChildNodes();
					int i = 0;
					int m = projectNodeList.getLength();
					boolean ok = false;
					Node projectStringsNode = projectNodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(projectStringsNode,"id") != null){
							if((getAttribute(projectStringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//get old project label node and change it to new value
								Node oldprojectStringsNode = getNodeById(lang1,realProjectName);
								if(oldprojectStringsNode != null){
									setStringValue(oldprojectStringsNode,getStringValue(projectStringsNode));
									ok = true;
								}
							}else{
								i++;
								projectStringsNode = projectNodeList.item(i);
							}
						}else{
							i++;
							projectStringsNode = projectNodeList.item(i);
						}
					}
					if(!ok){
						System.out.println("ServletXformEditor ==> ERROR: project name not found in " + this.lang1 + ".");
					}
				}
				root.appendChild(xmlResponse.importNode(lang1, true));

				//lang2
				File lang2File = new File(this.contextPath + langPath + this.lang2 + ".xml");
				Document lang2Document = documentBuilder.parse(lang2File);
				//Element lang2 = xmlResponse.createElement("lang2");
				Node lang2 = lang2Document.getDocumentElement();
				if(projectLangPath!=null){
					//We are working with an activity whose project has xform file.
					//Include project name into language xml.
					//If string project name exist, modify it (maybe it has been changed!).
					File lang2ProjectFile = new File(this.contextPath + projectLangPath + this.lang2 + ".xml");
					Document lang2ProjectDocument = documentBuilder.parse(lang2ProjectFile);
					Node lang2Project = lang2ProjectDocument.getDocumentElement();
					NodeList projectNodeList = lang2Project.getChildNodes();
					int i = 0;
					int m = projectNodeList.getLength();
					boolean ok = false;
					Node projectStringsNode = projectNodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(projectStringsNode,"id") != null){
							if((getAttribute(projectStringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//get old project label node and change it to new value
								Node oldprojectStringsNode = getNodeById(lang2,realProjectName);
								if(oldprojectStringsNode != null){
									setStringValue(oldprojectStringsNode,getStringValue(projectStringsNode));
									ok = true;
								}
							}else{
								i++;
								projectStringsNode = projectNodeList.item(i);
							}
						}else{
							i++;
							projectStringsNode = projectNodeList.item(i);
						}
					}
					if(!ok){
						System.out.println("ServletXformEditor ==> ERROR: project name not found in " + this.lang1 + ".");
					}
				}
				root.appendChild(xmlResponse.importNode(lang2, true));

				//lang3
				File lang3File = new File(this.contextPath + langPath + this.lang3 + ".xml");
				Document lang3Document = documentBuilder.parse(lang3File);
				//Element lang3 = xmlResponse.createElement("lang3");
				Node lang3 = lang3Document.getDocumentElement();
				if(projectLangPath!=null){
					//We are working with an activity whose project has xform file.
					//Include project name into language xml.
					//If string project name exist, modify it (maybe it has been changed!).
					File lang3ProjectFile = new File(this.contextPath + projectLangPath + this.lang3 + ".xml");
					Document lang3ProjectDocument = documentBuilder.parse(lang3ProjectFile);
					Node lang3Project = lang3ProjectDocument.getDocumentElement();
					NodeList projectNodeList = lang3Project.getChildNodes();
					int i = 0;
					int m = projectNodeList.getLength();
					boolean ok = false;
					Node projectStringsNode = projectNodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(projectStringsNode,"id") != null){
							if((getAttribute(projectStringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//get old project label node and change it to new value
								Node oldprojectStringsNode = getNodeById(lang3,realProjectName);
								if(oldprojectStringsNode != null){
									setStringValue(oldprojectStringsNode,getStringValue(projectStringsNode));
									ok = true;
								}
							}else{
								i++;
								projectStringsNode = projectNodeList.item(i);
							}
						}else{
							i++;
							projectStringsNode = projectNodeList.item(i);
						}
					}
					if(!ok){
						System.out.println("ServletXformEditor ==> ERROR: project name not found in " + this.lang1 + ".");
					}
				}
				root.appendChild(xmlResponse.importNode(lang3, true));
				//formPath = "/web/xforms/"+formPath;

			}else{
				//XForm file does not exist
				//we must pass language files with project properties labels
				//lang1
				Element lang1 = xmlResponse.createElement("strings");
				if(projectLangPath!=null){
					//include project name into language xml
					File lang1ProjectFile = new File(this.contextPath + projectLangPath + this.lang1 + ".xml");
					Document lang1ProjectDocument = documentBuilder.parse(lang1ProjectFile);
					Node lang1Project = lang1ProjectDocument.getDocumentElement();
					NodeList nodeList = lang1Project.getChildNodes();
					//20-06-2005 copy childs
					for(int i=0;i<nodeList.getLength();i++){
						lang1.appendChild(xmlResponse.importNode(nodeList.item(i),true));
					}
					/*
					int i = 0;
					int m = nodeList.getLength();
					boolean ok = false;
					Node stringsNode = nodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(stringsNode,"id") != null){
							if((getAttribute(stringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//lang1.appendChild(stringsNode.cloneNode(true));
								lang1.appendChild(xmlResponse.importNode(stringsNode, true));
								ok = true;
							}else{
								i++;
								stringsNode = nodeList.item(i);
							}
						}else{
							i++;
							stringsNode = nodeList.item(i);
						}
					}
					if(!ok){
						Element projectLabel = xmlResponse.createElement("string");
						projectLabel.setAttribute("id",realProjectName);
						Text projectLabelText = xmlResponse.createTextNode(realProjectName);
						projectLabel.appendChild(projectLabelText);
						lang1.appendChild(projectLabel);
					}
					*/

				}else{
					Element projectLabel = xmlResponse.createElement("string");
					projectLabel.setAttribute("id",realProjectName);
					Text projectLabelText = xmlResponse.createTextNode(realProjectName);
					projectLabel.appendChild(projectLabelText);
					lang1.appendChild(projectLabel);
				}
				root.appendChild(lang1);

				//lang2
				Element lang2 = xmlResponse.createElement("strings");
				if(projectLangPath!=null){
					//include project name into language xml
					//if string project name exist, modify it (maybe it has been changed)
					File lang2ProjectFile = new File(this.contextPath + projectLangPath + this.lang2 + ".xml");
					Document lang2ProjectDocument = documentBuilder.parse(lang2ProjectFile);
					Node lang2Project = lang2ProjectDocument.getDocumentElement();
					NodeList nodeList = lang2Project.getChildNodes();
					//20-06-2005 copy childs
					for(int i=0;i<nodeList.getLength();i++){
						lang2.appendChild(xmlResponse.importNode(nodeList.item(i),true));
					}
					/*
					int i = 0;
					int m = nodeList.getLength();
					boolean ok = false;
					Node stringsNode = nodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(stringsNode,"id") != null){
							if((getAttribute(stringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//lang2.appendChild(stringsNode.cloneNode(true));
								lang2.appendChild(xmlResponse.importNode(stringsNode, true));
								ok = true;
							}else{
								i++;
								stringsNode = nodeList.item(i);
							}
						}else{
							i++;
							stringsNode = nodeList.item(i);
						}
					}
					if(!ok){
						Element projectLabel = xmlResponse.createElement("string");
						projectLabel.setAttribute("id",realProjectName);
						Text projectLabelText = xmlResponse.createTextNode(realProjectName);
						projectLabel.appendChild(projectLabelText);
						lang2.appendChild(projectLabel);
					}
					*/

				}else{
					Element projectLabel = xmlResponse.createElement("string");
					projectLabel.setAttribute("id",realProjectName);
					Text projectLabelText = xmlResponse.createTextNode(realProjectName);
					projectLabel.appendChild(projectLabelText);
					lang2.appendChild(projectLabel);
				}
				root.appendChild(lang2);

				//lang3
				Element lang3 = xmlResponse.createElement("strings");
				if(projectLangPath!=null){
					//include project name into language xml
					//if string project name exist, modify it (maybe it has been changed)
					File lang3ProjectFile = new File(this.contextPath + projectLangPath + this.lang3 + ".xml");
					Document lang3ProjectDocument = documentBuilder.parse(lang3ProjectFile);
					Node lang3Project = lang3ProjectDocument.getDocumentElement();
					NodeList nodeList = lang3Project.getChildNodes();
					//20-06-2005 copy childs
					for(int i=0;i<nodeList.getLength();i++){
						lang3.appendChild(xmlResponse.importNode(nodeList.item(i),true));
					}
					/*
					int i = 0;
					int m = nodeList.getLength();
					boolean ok = false;
					Node stringsNode = nodeList.item(i);
					while(i<m && !ok){
						if(getAttribute(stringsNode,"id") != null){
							if((getAttribute(stringsNode,"id")).equalsIgnoreCase(realProjectName)){
								//lang3.appendChild(stringsNode.cloneNode(true));
								lang3.appendChild(xmlResponse.importNode(stringsNode, true));
								ok = true;
							}else{
								i++;
								stringsNode = nodeList.item(i);
							}
						}else{
							i++;
							stringsNode = nodeList.item(i);
						}
					}
					if(!ok){
						Element projectLabel = xmlResponse.createElement("string");
						projectLabel.setAttribute("id",realProjectName);
						Text projectLabelText = xmlResponse.createTextNode(realProjectName);
						projectLabel.appendChild(projectLabelText);
						lang3.appendChild(projectLabel);
					}
					*/

				}else{
					Element projectLabel = xmlResponse.createElement("string");
					projectLabel.setAttribute("id",realProjectName);
					Text projectLabelText = xmlResponse.createTextNode(realProjectName);
					projectLabel.appendChild(projectLabelText);
					lang3.appendChild(projectLabel);
				}
				root.appendChild(lang3);

			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return xmlResponse;
	}


	/**  17-05-2005
	 *   This method builds xform and language documents and saves them
	 *   in application/web/xforms/project-activity_name
	 *   <p/>
	 *
	 *   @param xmlRequest xml input Document.
	 */
	public String writeDocuments(String xmlRequest) {
		Document xformDocument = null;
		Document instance = null;
		Document lang1Document = null;
		Document lang2Document = null;
		Document lang3Document = null;
		String realProjectName = this.projectName.replaceFirst("_instance.+","");
		//versionning for xforms
        String projectVersion = this.projectVersion;
//System.out.println("ServletXformEditor ==> Writing xform documents...");
		try{
			//parse string to document
			DOMParser parser = new DOMParser();
			parser.parse(new InputSource(new StringReader(xmlRequest)));
			Document xmlDocument = parser.getDocument();
			Element root = xmlDocument.getDocumentElement();
			NodeList rootChilds = root.getChildNodes();

			//build documents
			DOMImplementation domImpl = new DOMImplementationImpl();
			Element tmpRoot = null;	//to build documents
			NodeList tmpRootChilds = null; //to build documents
			int i;

			//build xform document
			xformDocument = domImpl.createDocument(null, "html" , null);
			Element xformRoot = xformDocument.getDocumentElement();
			tmpRoot = (Element)rootChilds.item(0);
			//set root attributes
			NamedNodeMap map = tmpRoot.getAttributes();
			for(i=0;i<map.getLength();i++){
				xformRoot.setAttribute(map.item(i).getNodeName(),map.item(i).getNodeValue());
			}
			//copy childs
			tmpRootChilds = tmpRoot.getChildNodes();
			for(i=0;i<tmpRootChilds.getLength();i++){
				xformRoot.appendChild(xformDocument.importNode(tmpRootChilds.item(i),true));
			}

			//build instance document
			instance = domImpl.createDocument(null, "process" , null);
			Element instanceRoot = instance.getDocumentElement();
			tmpRoot = (Element)rootChilds.item(1);
			//copy childs
			tmpRootChilds = tmpRoot.getChildNodes();
			for(i=0;i<tmpRootChilds.getLength();i++){
				instanceRoot.appendChild(instance.importNode(tmpRootChilds.item(i),true));
			}

			//build lang1 document
			lang1Document = domImpl.createDocument(null, "strings" , null);
			Element lang1Root = lang1Document.getDocumentElement();
			tmpRoot = (Element)rootChilds.item(2);
			//copy childs
			tmpRootChilds = tmpRoot.getChildNodes();
			for(i=0;i<tmpRootChilds.getLength();i++){
				lang1Root.appendChild(lang1Document.importNode(tmpRootChilds.item(i),true));
			}

			//build lang2 document
			lang2Document = domImpl.createDocument(null, "strings" , null);
			Element lang2Root = lang2Document.getDocumentElement();
			tmpRoot = (Element)rootChilds.item(3);
			//copy childs
			tmpRootChilds = tmpRoot.getChildNodes();
			for(i=0;i<tmpRootChilds.getLength();i++){
				lang2Root.appendChild(lang2Document.importNode(tmpRootChilds.item(i),true));
			}

			//build lang3 document
			lang3Document = domImpl.createDocument(null, "strings" , null);
			Element lang3Root = lang3Document.getDocumentElement();
			tmpRoot = (Element)rootChilds.item(4);
			//copy childs
			tmpRootChilds = tmpRoot.getChildNodes();
			for(i=0;i<tmpRootChilds.getLength();i++){
				lang3Root.appendChild(lang3Document.importNode(tmpRootChilds.item(i),true));
			}

			//save documents
			//adding projectVersion sub directory
			String path = null;
			if (this.activityName.equals("")){
				this.repositoryFacade.setProjectFormPath(realProjectName, projectVersion);
				File directory = new File(this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName);
				directory.mkdir();
                //create the sub directory for the project version
                directory = new File(this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion);
                directory.mkdir();
				path = this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator;
//debug
//System.out.println("ServletXformEditor:writeDocuments() - path = " + path);
			}else{
				this.repositoryFacade.setActivityFormPath(realProjectName, projectVersion, this.activityName);
				File directory = new File(this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator + this.activityName);
				directory.mkdirs();
				//create the sub directory for the project version
                directory = new File(this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator + this.activityName);
                directory.mkdir();

				path = this.contextPath + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator + this.activityName + File.separator;
			}


			FileOutputStream output = null;
			OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
			of.setIndent(1);
			of.setIndenting(true);
			XMLSerializer serializer = null;
			//BufferedWriter br = null;
			String xml = null;
			//xform
			output = new FileOutputStream(path + "xform.xhtml");
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(xformDocument.getDocumentElement() );
			output.close();
			//xml = toPrettyXmlString(xformDocument);
			//br = new BufferedWriter(new OutputStreamWriter(output));
			//br.write(xml,0,xml.length());
			//br.close();
			//instance
			output = new FileOutputStream(path + "instance.xml");
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(instance.getDocumentElement() );
			output.close();
			//lang1
			output = new FileOutputStream(path + this.lang1 + ".xml");
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(lang1Document.getDocumentElement() );
			output.close();
			//xml = toPrettyXmlString(lang1Document);
			//br = new BufferedWriter(new OutputStreamWriter(output));
			//br.write(xml,0,xml.length());
			//br.close();
			//lang2
			output = new FileOutputStream(path + this.lang2 + ".xml");
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(lang2Document.getDocumentElement() );
			output.close();
			//xml = toPrettyXmlString(lang2Document);
			//br = new BufferedWriter(new OutputStreamWriter(output));
			//br.write(xml,0,xml.length());
			//br.close();
			//lang3
			output = new FileOutputStream(path + this.lang3 + ".xml");
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(lang3Document.getDocumentElement() );
			output.close();
			//xml = toPrettyXmlString(lang3Document);
			//br = new BufferedWriter(new OutputStreamWriter(output));
			//br.write(xml,0,xml.length());
			//br.close();
		}catch(Exception e){
			e.printStackTrace();
		}

		return("ok");
	}


	private static String toXmlString(Document document) {
		try {
			OutputFormat outputFormat = new OutputFormat(document,FormGeneratorConstant.DEBUG_OUTPUT_FORMAT_ENCODING,true);
			outputFormat.setIndenting(true);
			outputFormat.setIndent(1);
			outputFormat.setPreserveSpace(true);
			StringWriter writer = new StringWriter();
			DOMSerializer serializer = new XMLSerializer(writer, outputFormat);
			serializer.serialize(document);
			writer.close();
			return writer.toString();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}

	private static String toPrettyXmlString(Document doc) {
		StringBuffer buffer = new StringBuffer(200);
		toPrettyXmlString(doc.getDocumentElement(), 0, buffer);
		return buffer.toString();
	}

	private static void toPrettyXmlString(Node elmnt, int level,
		StringBuffer buffer)
	{
		String piece;
		int i;

		buffer.append("<?xml version=\"1.0\" encoding=\"" + FormGeneratorConstant.DEBUG_OUTPUT_FORMAT_ENCODING + "\"?>");
		buffer.append("\n");

		for (i = 0; i < level; i++)
			buffer.append("    ");

		buffer.append("<");


		buffer.append(elmnt.getNodeName());
		NamedNodeMap atts = elmnt.getAttributes();
		Node node;

		for (i = 0; i < atts.getLength(); i++) {
			node = atts.item(i);
			piece = " " + node.getNodeName() + "=\""
				+ node.getNodeValue() + "\"";

			if ((buffer.length() - (buffer.toString().lastIndexOf('\n') + 1))
				+ piece.length() > 78) {
				buffer.append("\n");
				for (int j = 0; j < (level + 1); j++)
					buffer.append("    ");
			}

			buffer.append(piece);
		}

		// Process child nodes:

		NodeList kids = elmnt.getChildNodes();

		int n = kids.getLength();

		Node kid;

		boolean needIndent = false;

		buffer.append(n < 1 ? "/>" : ">");

		//if (n > 1) xml += "\n";
		if (!(elmnt.getFirstChild() instanceof Text)) {
			//			xml += "\n";
		}

		for (i = 0; i < n; i++) {
			kid = kids.item(i);

			if (kid instanceof Element) {
				needIndent = true;
				buffer.append("\n");
				toPrettyXmlString((Element)kid, level + 1, buffer);
			} else if (kid instanceof Text) {
				buffer.append(kid.getNodeValue());
			} else if (kid instanceof Comment) {
				if (i > 0 && kids.item(i - 1) instanceof Text) {
					if (kids.item(i - 1).getNodeValue().trim().length() < 1) {
						buffer.append(kids.item(i - 1).getNodeValue());
					}
				}
				buffer.append("<!--" + kid.getNodeValue() + "-->");
			} else {
				System.out.println(kid.getClass().getName());
			}
		}

		// End-tag:
		if (n > 0) {
			if (n > 1 || (n == 1 && !(elmnt.getFirstChild() instanceof Text)))
				buffer.append("\n");

			if (needIndent)
				for (i = 0; i < level; i++)
					buffer.append("    ");

			buffer.append("</");

			buffer.append(elmnt.getNodeName() + ">");
		}
	}

	private static String getAttribute(Node node, String attrName) {
		if (node instanceof Element) {
			if (((Element)node).getAttributeNode(attrName) == null) {
				return null;
			} else {
				return ((Element)node).getAttribute(attrName);
			}
		}

		NamedNodeMap map = node.getAttributes();

		if (map == null)
			return null;

		Node value = map.getNamedItem(attrName);

		return value != null ? value.getNodeValue() : null;
	}

	private static String getStringValue(Node node) {
		if (node == null) {
			return null;
		} else if (node instanceof Attr) {
			return node.getNodeValue();
		} else {
			Node n;
			NodeList nodes = node.getChildNodes();

			for (int i = 0; i < nodes.getLength(); i++) {
				n = nodes.item(i);

				if (n instanceof Text) {
					return n.getNodeValue();
				}
			}
		}

		return null;
	}

	private static void setStringValue(Node node, String value) {
		if (node == null) {
			System.out.println("Warning!  No node...");
		} else if (node instanceof Attr) {
			node.setNodeValue(value);
		} else {
			Node n;
			NodeList nodes = node.getChildNodes();

			for (int i = 0; i < nodes.getLength(); i++) {
				n = nodes.item(i);

				if (n instanceof Text) {
					n.setNodeValue(value);
					return ;
				}
			}

			Document document = node.getOwnerDocument();
			Text text = document.createTextNode(value);
			node.appendChild(text);
		}
	}

	private static Node getNodeById(Node lang1, String realProjectName){
		Node projectNameNode = null;

		NodeList nodeList = lang1.getChildNodes();
		int i = 0;
		int m = nodeList.getLength();
		Node stringsNode = nodeList.item(i);
		while(i<m){
			if(getAttribute(stringsNode,"id") != null){
				if((getAttribute(stringsNode,"id")).equalsIgnoreCase(realProjectName)){
					return nodeList.item(i);
				}else{
					i++;
					stringsNode = nodeList.item(i);
				}
			}else{
				i++;
				stringsNode = nodeList.item(i);
			}
		}
		return null;
	}

}
