<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
	xmlns:xhtml="http://www.w3.org/2002/06/xhtml2"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xforms="http://www.w3.org/2002/xforms"
	xmlns:xlink="http://www.w3.org/1999/xlink"
	xmlns:chiba="http://chiba.sourceforge.net/xforms"
	exclude-result-prefixes="xhtml xforms chiba xlink">

	<!-- ######################## PARAMS ############################## -->
	<xsl:param name="host-url"
		select="'http://@baseurl.host@:@baseurl.port@'" />
	<xsl:param name="debug-enabled" select="'true'" />
	<!-- ### this url will be used to build the form action attribute ### -->
	<xsl:param name="action-url"
		select="'http://@baseurl.host@:@baseurl.port@/chiba-@chiba-version@/XFormsServlet'" />
	<xsl:param name="scripted" select="'false'" />
	<!-- ### specifies the parameter prefix for repeat selectors ### -->
	<xsl:param name="selector-prefix" select="'s_'" />
	<xsl:param name="form-id" select="'chiba-form'" />
	<xsl:param name="form-name" select="//xhtml:title" />
	<!-- change this to your ShowAttachmentServlet -->
	<!-- ### this parameter is used when the Adapter wants to specify the CSS to use ### -->
	<xsl:param name="css-file" select="''" />
	<!-- ### contains the full user-agent string as received from the servlet ### -->
	<xsl:param name="user-agent" select="'default'" />

	<!-- ######################## VARIABLES ############################## -->
	<xsl:variable name="data-prefix" select="'d_'" />
	<xsl:variable name="trigger-prefix" select="'t_'" />
	<xsl:variable name="uses-upload"
		select="boolean(//*/xforms:upload)" />
	<xsl:variable name="show-attachment-action"
		select="'http://localhost:8080/chiba-1.0.0/ShowAttachmentServlet'" />
	<xsl:variable name="remove-upload-prefix" select="'ru_'" />
	<!-- ### the CSS stylesheet to use ### -->
	<xsl:variable name="default-css" select="'styles/styles.css'" />
	<xsl:variable name="mozilla-css"
		select="'styles/mozilla-xforms.css'" />
	<xsl:variable name="ie-css" select="'styles/ie-xforms.css'" />


	<!-- ####################################################################################################### -->
	<!-- ##################################### TEMPLATES ####################################################### -->
	<!-- ####################################################################################################### -->

	<xsl:template match="/">
		<xsl:choose>
			<xsl:when test="$debug-enabled='true'">
				<xsl:message>
					********** INFO: XSLT MESSAGES ARE ENABLED
				</xsl:message>
				<xsl:message>RECEIVED PARAMETERS...</xsl:message>
				<xsl:message>
					css-file=
					<xsl:value-of select="$css-file" />
				</xsl:message>
				<xsl:message>
					data-prefix=
					<xsl:value-of select="$data-prefix" />
				</xsl:message>
				<xsl:message>
					trigger-prefix=
					<xsl:value-of select="$trigger-prefix" />
				</xsl:message>
				<xsl:message>
					user-agent=
					<xsl:value-of select="$user-agent" />
				</xsl:message>
				<xsl:message>
					scripting=
					<xsl:value-of select="$scripted" />
				</xsl:message>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$scripted='true'">
					<xsl:message>
						********** INFO: XSLT MESSAGES ARE DISABLED
					</xsl:message>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="$scripted='true'">
				<xsl:message>----------------</xsl:message>
				<xsl:message>INFO: scripting is on</xsl:message>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$debug-enabled='true'">
					<xsl:message>INFO: scripting is off</xsl:message>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="xhtml:head">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>head</xsl:message>
		</xsl:if>
		<head>
			<title>
				<xsl:value-of select="$form-name" />
			</title>
			<xsl:call-template name="getCSS" />

			<!-- copy initial model for scripting -->
			<!-- todo: import current instance into container before passing to this stylesheet -->
			<xsl:if test="$scripted='true'">
				<script type="text/xml" id="xforms-model">
					<xsl:for-each select="xforms:model">
						<xsl:copy-of select="." />
					</xsl:for-each>
				</script>
			</xsl:if>

			<!-- :olli hack -->
			<!-- import slider JavaScript -->
			<script type="text/javascript" src="scripts/slider.js"></script>
			<!-- :olli hack ende -->
			<!-- script to open windows -->
			<script type="text/javascript">
				var newwindow; function openwindow(url,name) {
				newwindow=window.open(url,name,'height=400,width=750');
				if (window.focus) {newwindow.focus()} }
			</script>
		</head>
	</xsl:template>

	<!-- copy unmatched mixed markup, comments, whitespace, and text -->
	<!-- ### copy elements from the xhtml2 namespace to html (without any namespace) by re-creating the     ### -->
	<!-- ### elements. Other Elements are just copied.                                                      ### -->
	<xsl:template match="*|@*|text()">
		<xsl:choose>
			<xsl:when
				test="namespace-uri(.)='http://www.w3.org/2002/06/xhtml2'">
				<xsl:element name="{local-name(.)}">
					<xsl:apply-templates select="*|@*|text()" />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates select="*|@*|text()" />
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xhtml:html">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>html</xsl:message>
		</xsl:if>
		<html>
			<xsl:apply-templates />
		</html>
	</xsl:template>

	<xsl:template match="xhtml:link">
		<xsl:copy-of select="." />
	</xsl:template>

	<xsl:template match="xhtml:body">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>body</xsl:message>
		</xsl:if>
		<body class="content">
			<xsl:copy-of select="@*" />
			<xsl:element name="form">
				<xsl:attribute name="name">
					<xsl:value-of select="$form-id" />
				</xsl:attribute>
				<xsl:attribute name="action">
					<xsl:value-of select="$action-url" />
				</xsl:attribute>
				<xsl:attribute name="method">POST</xsl:attribute>
				<xsl:attribute name="enctype">
					application/x-www-form-urlencoded
				</xsl:attribute>
				<xsl:if test="$uses-upload">
					<xsl:attribute name="enctype">
						multipart/form-data
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="$scripted='true'">
					<xsl:attribute name="onsubmit">
						javascript:submit();
					</xsl:attribute>
				</xsl:if>

				<!-- provide a first submit which does not map to any xforms:trigger -->
				<input type="image" name="dummy"
					style="width:0pt;height:0pt;" value="dummy" />
				<xsl:apply-templates />
			</xsl:element>
		</body>
	</xsl:template>

	<xsl:template match="xhtml:span">
		<span>
			<xsl:copy-of select="@xhtml:class" />
			<xsl:apply-templates />
		</span>
	</xsl:template>

	<!-- ### skip explicitely disabled control ### -->
	<!--    <xsl:template match="*[chiba:data/@chiba:enabled='false']">-->
	<!--        <xsl:if test="$debug-enabled='true'">-->
	<!--            <xsl:message>skipping disabled control</xsl:message>-->
	<!--        </xsl:if>-->
	<!--    </xsl:template>-->

	<!-- ### skip chiba:data elements ### -->
	<xsl:template match="chiba:data" />

	<!-- ### skip model section ### -->
	<xsl:template match="xforms:model" />

	<!-- ######################################################################################################## -->
	<!-- #####################################  CONTROLS ######################################################## -->
	<!-- ######################################################################################################## -->

	<!-- ### handle xforms:input ### -->
	<xsl:template match="xforms:input">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:output ### -->
	<xsl:template match="xforms:output">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:range ### -->
	<xsl:template match="xforms:range">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:secret ### -->
	<xsl:template match="xforms:secret">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:select ### -->
	<xsl:template match="xforms:select">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:select1 ### -->
	<xsl:template match="xforms:select1">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:submit ### -->
	<xsl:template match="xforms:submit">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:trigger ### -->
	<xsl:template match="xforms:trigger">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:textarea ### -->
	<xsl:template match="xforms:textarea">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle xforms:upload ### -->
	<xsl:template match="xforms:upload">
		<xsl:call-template name="buildControl" />
	</xsl:template>

	<!-- ### handle label ### -->
	<xsl:template match="xforms:label">
		<xsl:variable name="group-id"
			select="ancestor::xforms:group[1]/@id" />
		<xsl:variable name="img" select="@xforms:src" />

		<xsl:choose>
			<!--
				<xsl:when test="name(..)='xforms:group'">
				<xsl:apply-templates/>
				</xsl:when>
			-->
			<xsl:when test="name(..)='xforms:item'">
				<span id="{@id}" class="label">
					<xsl:apply-templates />
				</span>
			</xsl:when>
			<!-- suppress trigger labels - they are handle by the control itself -->
			<xsl:when test="parent::xforms:trigger"
				xmlns:xforms="http://www.w3.org/2002/xforms">
			</xsl:when>
			<!-- if there's an output child -->
			<xsl:when test="self::xforms:output"
				xmlns:xforms="http://www.w3.org/2002/xforms">
				<xsl:apply-templates select="xforms:output" />
			</xsl:when>
			<!-- if there's a src attribute pointing to some image file the image is linked in -->
			<xsl:when
				test="boolean($img) and ( contains($img,'.gif') or contains($img,'.jpg') or contains($img,'.png') )">
				<img src="{$img}" id="{@id}-label" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- ### handle hint ### -->
	<xsl:template match="xforms:hint">
		<!--  already handled by individual controls in html-form-controls.xsl -->
	</xsl:template>

	<!-- ### handle help ### -->
	<!-- ### only reacts on help elements with a 'src' attribute and interprets it as html href ### -->
	<xsl:template match="xforms:help">
		?
		<!--
			<span style="font-color:blue;vertical-align:top;padding-left:2px;font-weight:bold">
			<xsl:if test="@xforms:src">
			<a href="{@xforms:src}">?</a>
			</xsl:if>
			</span>
		-->
		<!--        <img src="images/kasten_blau.gif"/>-->
		<!-- this implementation renders a button to display a javascript message -->
		<!--        <img src="images/help.gif" onClick="javascript:xf_help('{normalize-space(.)}');return true;"/>-->
	</xsl:template>

	<!-- ### handle explicitely enabled alert ### -->
	<!--    <xsl:template match="xforms:alert[../chiba:data/@chiba:valid='false']">-->
	<xsl:template match="xforms:alert">
		<span id="{../@id}-alert" class="alert">
			<xsl:value-of select="." />
		</span>
	</xsl:template>

	<!-- ### handle extensions ### -->
	<xsl:template match="xforms:extension">
		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="chiba:selector"></xsl:template>


	<!-- ########################## ACTIONS ####################################################### -->
	<!-- these templates serve no real purpose here but are shown for reference what may be over-   -->
	<!-- written by customized stylesheets importing this one. -->
	<!-- ########################## ACTIONS ####################################################### -->

	<!-- action nodes are simply copied to output without any modification -->
	<xsl:template match="xforms:action">
		<xsl:apply-templates />
	</xsl:template>
	<xsl:template match="xforms:dispatch" />
	<xsl:template match="xforms:rebuild" />
	<xsl:template match="xforms:recalculate" />
	<xsl:template match="xforms:revalidate" />
	<xsl:template match="xforms:refresh" />
	<xsl:template match="xforms:setfocus" />
	<xsl:template match="xforms:load" />
	<xsl:template match="xforms:setvalue" />
	<xsl:template match="xforms:send" />
	<xsl:template match="xforms:reset" />
	<xsl:template match="xforms:message" />
	<xsl:template match="xforms:toggle" />
	<xsl:template match="xforms:insert" />
	<xsl:template match="xforms:delete" />
	<xsl:template match="xforms:setindex" />

	<!-- ####################################################################################################### -->
	<!-- This stylesheet handles the XForms UI constructs [XForms 1.0, Chapter 9]'group', 'repeat' and           -->
	<!-- 'switch' and offers some standard interpretations for the appearance attribute.                         -->
	<!-- author: joern turner                                                                                    -->
	<!-- ####################################################################################################### -->

	<!-- ############################################ PARAMS ################################################### -->
	<!-- ##### should be declared in html4.xsl ###### -->
	<!-- ############################################ VARIABLES ################################################ -->
	<!-- ### this url will be used to call CMS ### -->
	<xsl:param name="cms-url"
		select="'http://@baseurl.host@:@baseurl.port@/soacms'" />
	<xsl:param name="host" select="'localhost'" />
	<xsl:param name="port" select="9000" />

	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"
		indent="yes" doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN" />

	<!-- ### transcodes the XHMTL namespaced elements to HTML ### -->
	<xsl:namespace-alias stylesheet-prefix="xhtml"
		result-prefix="#default" />

	<xsl:preserve-space elements="*" />
	<xsl:strip-space elements="xforms:action" />

	<!-- ####################################################################################################### -->
	<!-- #################################### GROUPS ########################################################### -->
	<!-- ####################################################################################################### -->

	<!--
		processing of groups and repeats is handled with a computational pattern (as mentioned in Michael Kay's XSLT
		Programmers Reference) in this stylesheet, that means that when a group or repeat is found its children will
		be processed with for-each. this top-down approach seems to be more adequate for transforming XForms markup
		than to follow a rule-based pattern. Also note that whenever nodesets of XForms controls are processed the
		call template 'buildControl' is used to handle the control. In contrast to apply-templates a call-template
		preserves the position() of the control inside its parent nodeset and this can be valuable information for
		annotating controls with CSS classes that refer to their parent.
	-->
	<!-- ###################################### MINIMAL GROUP ################################################## -->
	<!-- handle 'minimal' group - this is the default for groups and only annotates CSS to labels + controls and
		outputs them in a kind of flow-layout -->
	<xsl:template match="xforms:group[@xforms:appearance='minimal']">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>found minimal group</xsl:message>
		</xsl:if>

		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<xsl:variable name="id" select="@id" />

		<div
			class="{normalize-space(concat('minimal-group',' ',$group-css))}"
			id="{$id}">
			<xsl:for-each select="*">
				<xsl:choose>

					<!-- **** handle group label ***** -->
					<xsl:when test="self::xforms:label">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								handling group label ...
							</xsl:message>
						</xsl:if>
						<span id="{$id}-label"
							class="minimal-group-label">
							<xsl:apply-templates select="." />
						</span>
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								handled group label ...
							</xsl:message>
						</xsl:if>
					</xsl:when>

					<!-- **** handle group alert ***** -->
					<xsl:when test="self::xforms:alert">
						<xsl:apply-templates select="xforms:alert" />
					</xsl:when>

					<!-- **** handle sub group ***** -->
					<xsl:when test="self::xforms:group">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>found group</xsl:message>
						</xsl:if>
						<xsl:apply-templates select="." />
					</xsl:when>

					<!-- **** handle repeat ***** -->
					<xsl:when test="self::xforms:repeat">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>found repeat</xsl:message>
						</xsl:if>
						<xsl:apply-templates select="." />
					</xsl:when>

					<!-- **** handle switch ***** -->
					<xsl:when test="self::xforms:switch">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>found switch</xsl:message>
						</xsl:if>
						<xsl:apply-templates select="." />
					</xsl:when>

					<!-- **** handle chiba:data element ***** -->
					<xsl:when test="self::chiba:data"
						xmlns:chiba="http://chiba.sourceforge.net/xforms">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								ignoring chiba data element
							</xsl:message>
						</xsl:if>
					</xsl:when>

					<!-- **** handle trigger + submit ***** -->
					<xsl:when
						test="self::xforms:trigger or self::xforms:submit">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								handling trigger:
								<xsl:value-of select="xforms:label" />
							</xsl:message>
						</xsl:if>
						<xsl:variable name="css">
							<xsl:call-template name="assembleClasses" />
						</xsl:variable>
						<span class="{$css}" id="{@id}">
							<xsl:call-template name="buildControl" />
						</span>
					</xsl:when>

					<!-- **** handle xforms control ***** -->
					<xsl:when test="self::xforms:*">

						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								handling control label
							</xsl:message>
							<xsl:message>
								<xsl:value-of select="name()" />
								-
								<xsl:value-of select="xforms:label" />
							</xsl:message>
						</xsl:if>

						<xsl:variable name="css">
							<xsl:call-template name="assembleClasses" />
						</xsl:variable>
						<xsl:variable name="label-class">
							<xsl:call-template name="labelClasses" />
						</xsl:variable>
						<span id="{@id}" class="{$css}">
							<span id="{@id}-label"
								class="{$label-class}">
								<xsl:apply-templates
									select="xforms:label" />
							</span>
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handling control
								</xsl:message>
								<xsl:message>
									<xsl:value-of select="name()" />
								</xsl:message>
							</xsl:if>
							<xsl:call-template name="buildControl" />
						</span>
					</xsl:when>

					<!-- **** handle all other ***** -->
					<xsl:otherwise>
						<xsl:call-template
							name="handle-foreign-elements" />
					</xsl:otherwise>

				</xsl:choose>
			</xsl:for-each>
		</div>
	</xsl:template>


	<!-- ###################################### COMPACT GROUP ################################################## -->
	<xsl:template match="xforms:group[@xforms:appearance='compact']">
		<xsl:if test="$debug-enabled='yes'">
			<xsl:message>
				found compact group
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="id" select="@id" />
		<xsl:variable name="control-count"
			select="count(./*/xforms:label)" />
		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<table
			class="{normalize-space(concat('compact-group',' ',$group-css))}"
			id="{$id}">
			<!-- ***** build caption with column labels ***** -->
			<tr>
				<td colspan="{$control-count}" id="{$id}-label"
					class="compact-group-label">
					<xsl:apply-templates select="xforms:label" />
				</td>
			</tr>
			<tr>
				<xsl:for-each select="./*/xforms:label">
					<xsl:variable name="label-class">
						<xsl:call-template name="labelClasses" />
					</xsl:variable>
					<td id="{../@id}-label" class="{$label-class}">
						<xsl:apply-templates
							select="self::node()[not(name(..)='xforms:trigger' or name(..)='xforms:submit')]" />
					</td>
				</xsl:for-each>
			</tr>
			<tr>
				<xsl:for-each select="*">
					<xsl:choose>

						<!-- **** handle group label ***** -->
						<xsl:when test="self::xforms:label">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									ignoring group label ...
								</xsl:message>
							</xsl:if>
						</xsl:when>

						<!-- **** handle group alert ***** -->
						<xsl:when test="self::xforms:alert">
							<xsl:apply-templates select="xforms:alert" />
						</xsl:when>

						<!-- **** handle sub group ***** -->
						<xsl:when test="self::xforms:group">
							<td colspan="{$control-count}">
								<xsl:apply-templates select="." />
							</td>
						</xsl:when>

						<!-- **** handle repeat ***** -->
						<xsl:when test="self::xforms:repeat">
							<td colspan="{$control-count}">
								<xsl:apply-templates select="." />
							</td>
						</xsl:when>

						<!-- **** handle switch ***** -->
						<xsl:when test="self::xforms:switch">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>found switch</xsl:message>
							</xsl:if>
							<td colspan="{$control-count}">
								<xsl:apply-templates select="." />
							</td>
						</xsl:when>

						<!-- **** handle trigger + submit ***** -->
						<xsl:when
							test="self::xforms:trigger or self::xforms:submit">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handling trigger:
									<xsl:value-of select="xforms:label" />
								</xsl:message>
							</xsl:if>
							<xsl:variable name="css">
								<xsl:call-template
									name="assembleClasses" />
							</xsl:variable>
							<td class="{$css}" id="{@id}">
								<xsl:call-template name="buildControl" />
							</td>
						</xsl:when>

						<!-- **** handle xforms control ***** -->
						<xsl:when test="self::xforms:*">
							<xsl:variable name="css">
								<xsl:call-template
									name="assembleClasses" />
							</xsl:variable>
							<td id="{@id}" class="{$css}">
								<xsl:if test="$debug-enabled='true'">
									<xsl:message>
										handling control
									</xsl:message>
									<xsl:message>
										<xsl:value-of select="name()" />
									</xsl:message>
								</xsl:if>
								<xsl:call-template name="buildControl" />
							</td>
						</xsl:when>

						<!-- **** handle chiba:data element ***** -->
						<xsl:when test="self::chiba:data"
							xmlns:xforms="http://chiba.sourceforge.net/xforms">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									ignoring chiba data element
								</xsl:message>
							</xsl:if>
						</xsl:when>

						<!-- **** handle all other ***** -->
						<xsl:otherwise>
							<xsl:call-template
								name="handle-foreign-elements" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</tr>
		</table>
	</xsl:template>

	<!-- ###################################### FULL GROUP ################################################## -->
	<!-- handle group with apprearance 'full' - will render controls in a two-column table with labels on
		the left side. -->
	<xsl:template match="xforms:group" name="full-group">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				found full group (the default)
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="id" select="@id" />
		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<!--<table class="{normalize-space(concat('full-group',' ',$group-css))}" id="{$id}" border="5">-->
		<table class="top" id="{$id}" style="width:99%">
			<!-- handling group children -->
			<xsl:for-each select="*">
				<xsl:if test="$debug-enabled='true'">
					<xsl:message>
						*=
						<xsl:value-of select="." />
						...
					</xsl:message>
				</xsl:if>
				<xsl:choose>

					<!-- ***** build caption with project labels ***** -->
					<xsl:when test="self::xforms:label">
						<xsl:if test="@id='projectname'">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handling project label ...
								</xsl:message>
							</xsl:if>
							<!-- <tr> -->
							<tr class="panelTitle">
								<!--<td colspan="2" id="{$id}-label" class="full-group-label">-->
								<td id="{$id}-label" align="center"
									class="contentTitle" colspan="4">
									<xsl:apply-templates select="." />
								</td>
							</tr>
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handled project label ...
								</xsl:message>
							</xsl:if>
						</xsl:if>
						<xsl:if test="@id='activityname'">
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handling activity label ...
								</xsl:message>
							</xsl:if>
							<!-- <tr> -->
							<tr class="gridRowOdd">
								<!--<td colspan="2" id="{$id}-label" class="full-group-label">-->
								<td id="{$id}-label" align="center"
									class="gridRowOdd" colspan="4">
									<xsl:apply-templates select="." />
								</td>
							</tr>
							<xsl:if test="$debug-enabled='true'">
								<xsl:message>
									handled activity label ...
								</xsl:message>
							</xsl:if>
						</xsl:if>
					</xsl:when>

					<!-- **** handle group alert ***** -->
					<xsl:when test="self::xforms:alert">
						<xsl:apply-templates select="xforms:alert" />
					</xsl:when>

					<!-- **** handle sub group ***** -->
					<xsl:when test="self::xforms:group">
						<tr>
							<td colspan="2">
								<xsl:apply-templates select="." />
							</td>
						</tr>
					</xsl:when>

					<!-- **** handle repeat ***** -->
					<xsl:when test="self::xforms:repeat">
						<tr>
							<td colspan="2">
								<xsl:apply-templates select="." />
							</td>
						</tr>
					</xsl:when>

					<!-- **** handle switch ***** -->
					<xsl:when test="self::xforms:switch">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>found switch</xsl:message>
						</xsl:if>
						<tr>
							<td colspan="2">
								<xsl:apply-templates select="." />
							</td>
						</tr>
					</xsl:when>

					<!-- **** handle trigger + submit ***** -->
					<xsl:when
						test="self::xforms:trigger or self::xforms:submit">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								handling trigger:
								<xsl:value-of select="xforms:label" />
							</xsl:message>
						</xsl:if>
						<tr>
							<td align="center" colspan="4">
								<table align="center"
									cellpadding="30">
									<tr>
										<xsl:variable name="css">
											<xsl:call-template
												name="assembleClasses" />
										</xsl:variable>
										<!-- <td class="{$css}" id="{@id}" colspan="2">-->
										<td id="{@id}" align="center">
											<xsl:call-template
												name="buildControl" />
										</td>
									</tr>
								</table>
							</td>
						</tr>
					</xsl:when>

					<!-- **** handle xforms control ***** -->
					<xsl:when test="self::xforms:*">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								element ->
								<xsl:value-of select="name(.)" />
							</xsl:message>
						</xsl:if>

						<!-- 
							<xsl:variable name="extattr">
							<xsl:choose>
							<xsl:when test="@xforms:extattr='project'">project</xsl:when>
							<xsl:otherwise>activity</xsl:otherwise>
							</xsl:choose>
							</xsl:variable>-->

						<!-- <tr> -->
						<xsl:if
							test="chiba:data/@chiba:enabled='true'">

							<tr class="gridRowOdd {@xforms:extattr}">
								<xsl:variable name="css">
									<xsl:call-template
										name="assembleClasses" />
								</xsl:variable>
								<xsl:variable name="label-class">
									<xsl:call-template
										name="labelClasses" />
								</xsl:variable>
								<!--<td id="{@id}-label" class="{$label-class}" test1="test1">-->
								<td id="{@id}-label" align="left"
									class="gridRowOdd {$css}" style="width:50%">
									<xsl:if
										test="$debug-enabled='true'">
										<xsl:message>
											handling control label
										</xsl:message>
										<xsl:message>
											<xsl:value-of
												select="name()" />
											-
											<xsl:value-of
												select="xforms:label" />
										</xsl:message>
									</xsl:if>
									<xsl:apply-templates
										select="xforms:label" />
								</td>

								<!--
									<xsl:variable name="cssa">
									<xsl:call-template name="assembleClasses"/>
									</xsl:variable>
								-->
								<!-- <td id="{@id}" class="{$css}" test2="test2">-->
								<td id="{@id}" align="left"
									class="gridRowOdd {$css}" style="width:50%">
									<xsl:if
										test="$debug-enabled='true'">
										<xsl:message>
											handling control
										</xsl:message>
										<xsl:message>
											<xsl:value-of
												select="name()" />
										</xsl:message>
									</xsl:if>
									<xsl:call-template
										name="buildControl" />
									<xsl:if
										test="@xforms:content='yes'">
										<xsl:if
											test="chiba:data/@chiba:readonly='true'">
											<!--<a href="{$host-url}/soacms/front/workflow/viewDocument.do?documentUUID={chiba:data/text()}" target="SOACMS">CMS</a>-->
											<a
												href="javascript:openwindow('{$host-url}/soacms/front/workflow/viewDocument.do?documentUUID={chiba:data/text()}','CMS');">
												CMS
											</a>
										</xsl:if>
										<xsl:if
											test="chiba:data/@chiba:readonly='false'">
											<!--<a href="{$host-url}/soacms/back/workflow/editDocument.do?documentUUID={chiba:data/text()}" target="SOACMS">CMS</a>-->
											<a
												href="javascript:openwindow('{$host-url}/soacms/back/workflow/editDocument.do?documentUUID={chiba:data/text()}','CMS');">
												CMS
											</a>
										</xsl:if>
									</xsl:if>
								</td>
							</tr>
						</xsl:if>
					</xsl:when>

					<!-- **** handle chiba:data element ***** -->
					<xsl:when test="self::chiba:data"
						xmlns:xforms="http://chiba.sourceforge.net/xforms">
						<xsl:if test="$debug-enabled='true'">
							<xsl:message>
								ignoring chiba data element
							</xsl:message>
						</xsl:if>
					</xsl:when>

					<!-- **** handle all other ***** -->
					<xsl:otherwise>
						<xsl:call-template
							name="handle-foreign-elements" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</table>
	</xsl:template>

	<!-- ###################################### MULTI-COLUMN GROUP ############################################## -->
	<!-- ### based upon compact apprearance the mulit-column group puts child groups into columns of a        ### -->
	<!-- ### table and allow groups to appear side by side.                                                   ### -->
	<!-- ### ATTENTION: ONLY GROUPS ARE PROCESSED BY THIS TEMPLATE - EVERYTHING ELSE IS IGNORED               ### -->
	<!-- ######################################################################################################## -->
	<xsl:template
		match="xforms:group[@xforms:appearance='multi-column']">
		<xsl:if test="$debug-enabled='yes'">
			<xsl:message>
				found multi-column group
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="id" select="@id" />
		<xsl:variable name="group-count" select="count(./xforms:group)" />
		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<table
			class="{normalize-space(concat('multi-column-group',' ',$group-css))}"
			id="{$id}">
			<!-- ***** build caption with column labels ***** -->
			<tr>
				<td colspan="{$group-count}" width="100%"
					id="{$id}-label" class="multi-column-group-label">
					<xsl:apply-templates select="xforms:label" />
				</td>
			</tr>
			<tr>
				<xsl:for-each select="xforms:group">
					<td>
						<xsl:apply-templates select="." />
					</td>
				</xsl:for-each>
			</tr>
		</table>
	</xsl:template>



	<!-- ####################################### GROUP HELPER ################################################### -->
	<xsl:template name="handle-foreign-elements">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				handling element:
				<xsl:value-of select="name()" />
			</xsl:message>
		</xsl:if>
		<xsl:copy>
			<xsl:copy-of select="@*" />
			<xsl:apply-templates />
		</xsl:copy>
	</xsl:template>


	<!-- ######################################################################################################## -->
	<!-- ####################################### REPEAT ######################################################### -->
	<!-- ######################################################################################################## -->

	<!-- ### handle repeat with 'minimal' appearance ### -->
	<xsl:template match="xforms:repeat">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				found minimal repeat
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<table class="{concat('minimal-repeat',' ',$group-css)}"
			id="{@id}">
			<xsl:if test="$scripted='true'">
				<!-- clone repeat prototype -->
				<!-- style attribute for safety in case CSS file is not there -->
				<tr class="repeat-prototype"
					onclick="setRepeatIndex('{@id}');" style="display:none;">
					<xsl:for-each
						select="chiba:data/xforms:group[@chiba:transient]">
						<xsl:call-template name="processMinimalChilds" />
					</xsl:for-each>
				</tr>
			</xsl:if>

			<xsl:variable name="outermost-id"
				select="ancestor-or-self::xforms:repeat/@id" />
			<xsl:variable name="repeat-id" select="@id" />

			<!-- ***** loop repeat entries ***** -->
			<xsl:for-each select="xforms:group[@chiba:transient]">
				<xsl:if test="$debug-enabled='true'">
					<xsl:message>
						found
						<xsl:value-of select="name()" />
						...
					</xsl:message>
					<xsl:message>
						found
						<xsl:value-of select="xforms:label" />
						...
					</xsl:message>
				</xsl:if>

				<xsl:choose>
					<xsl:when test="@chiba:selected='true'">
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item repeat-index"
									onclick="setRepeatIndex('{$repeat-id}');">
									<xsl:call-template
										name="processMinimalChilds" />
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item repeat-index">
									<td
										class="minimal-repeat-selector">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" checked="checked" />
									</td>
									<xsl:call-template
										name="processMinimalChilds" />
								</tr>
							</xsl:otherwise>
						</xsl:choose>
						<!--
							<tr class="repeat-item repeat-index" onclick="setRepeatIndex('{$repeat-id}');">
							<xsl:if test="not($scripted='true')">
							<td class="minimal-repeat-selector">
							<input type="radio" name="{$selector-prefix}{$outermost-id}" value="{$repeat-id}:{@chiba:position}" checked="checked"/>
							</td>
							</xsl:if>
							<xsl:for-each select="*">
							<xsl:variable name="css">
							<xsl:call-template name="assembleClasses"/>
							</xsl:variable>
							<xsl:variable name="label-class">
							<xsl:call-template name="labelClasses"/>
							</xsl:variable>
							<td id="{@id}" class="{$css}">
							<span id="{@id}-label" class="{$label-class}">
							<xsl:apply-templates select="./xforms:label"/>
							</span>
							
							<xsl:call-template name="buildControl"/>
							</td>
							</xsl:for-each>
							</tr>
						-->
					</xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item"
									onclick="setRepeatIndex('{$repeat-id}');">
									<xsl:call-template
										name="processMinimalChilds" />
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item">
									<td
										class="minimal-repeat-selector">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" />
									</td>
									<xsl:call-template
										name="processMinimalChilds" />
								</tr>
							</xsl:otherwise>
						</xsl:choose>
						<!--
							<tr class="repeat-item" onclick="setRepeatIndex('{$repeat-id}');">
							<xsl:if test="not($scripted='true')">
							<td class="minimal-repeat-selector">
							<input type="radio" name="{$selector-prefix}{$outermost-id}" value="{$repeat-id}:{@chiba:position}"/>
							</td>
							</xsl:if>
							<xsl:for-each select="*">
							<xsl:variable name="css">
							<xsl:call-template name="assembleClasses"/>
							</xsl:variable>
							<xsl:variable name="label-class">
							<xsl:call-template name="labelClasses"/>
							</xsl:variable>
							<td id="{@id}" class="{$css}">
							<span id="{@id}-label" class="{$label-class}">
							<xsl:apply-templates select="./xforms:label"/>
							</span>
							
							<xsl:call-template name="buildControl"/>
							</td>
							</xsl:for-each>
							</tr>
						-->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</table>
	</xsl:template>

	<!-- ### used by minimal repeat ### -->
	<xsl:template name="processMinimalChilds">
		<xsl:for-each select="*">
			<xsl:variable name="css">
				<xsl:call-template name="assembleClasses" />
			</xsl:variable>
			<xsl:variable name="label-class">
				<xsl:call-template name="labelClasses" />
			</xsl:variable>
			<td id="{@id}" class="{$css}">
				<span id="{@id}-label" class="{$label-class}">
					<xsl:apply-templates select="./xforms:label" />
				</span>

				<xsl:call-template name="buildControl" />
			</td>
		</xsl:for-each>
	</xsl:template>

	<!-- ### handle repeat with 'compact' appearance ### -->
	<xsl:template match="xforms:repeat[@xforms:appearance='compact']"
		priority="1">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				found compact repeat
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="repeat" select="." />
		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>
		<table id="{@id}"
			class="{concat('compact-repeat',' ',$group-css)}" border="0">
			<xsl:if test="$scripted='true'">
				<!-- clone repeat prototype -->
				<!-- style attribute for safety in case CSS file is not there -->
				<tr class="repeat-prototype"
					onclick="setRepeatIndex('{@id}');" style="display:none;">
					<xsl:for-each
						select="chiba:data/xforms:group[@chiba:transient]">
						<xsl:call-template name="processCompactChilds" />
					</xsl:for-each>
				</tr>
			</xsl:if>
			<tr class="compact-repeat-label">
				<xsl:if test="not($scripted='true')">
					<!-- ***** build empty selector cell ***** -->
					<td>&#160;</td>
				</xsl:if>
				<!-- ***** build header ***** -->
				<xsl:for-each select="xforms:group[1]/*/xforms:label">
					<xsl:variable name="label-class">
						<xsl:call-template name="labelClasses" />
					</xsl:variable>
					<td id="{../@id}-label" class="{$label-class}">
						<xsl:apply-templates
							select="self::node()[not(name(..)='xforms:trigger' or name(..)='xforms:submit')]" />
					</td>
				</xsl:for-each>
			</tr>

			<xsl:variable name="outermost-id"
				select="ancestor-or-self::xforms:repeat/@id" />
			<xsl:variable name="repeat-id" select="@id" />

			<xsl:for-each select="xforms:group[@chiba:transient]">
				<xsl:choose>
					<xsl:when test="@chiba:selected='true'">
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item repeat-index"
									onclick="setRepeatIndex('{$repeat/@id}');">
									<xsl:call-template
										name="processCompactChilds" />
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item repeat-index">
									<td class="selector-cell">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" checked="checked" />
									</td>
									<xsl:call-template
										name="processCompactChilds" />
								</tr>
							</xsl:otherwise>
						</xsl:choose>

						<!--
							<tr class="repeat-item repeat-index" onclick="setRepeatIndex('{$repeat/@id}');">
							<xsl:if test="not($scripted='true')">
							<td class="selector-cell">
							<input type="radio" name="{$selector-prefix}{$outermost-id}" value="{$repeat-id}:{@chiba:position}" checked="checked"/>
							</td>
							</xsl:if>
							<xsl:for-each select="*">
							<xsl:variable name="css">
							<xsl:call-template name="assembleClasses"/>
							</xsl:variable>
							<td id="{@id}" class="{$css}">
							<xsl:call-template name="buildControl"/>
							</td>
							</xsl:for-each>
							</tr>
						-->
					</xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item"
									onclick="setRepeatIndex('{$repeat/@id}');">
									<xsl:call-template
										name="processCompactChilds" />
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item">
									<td class="selector-cell">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" />
									</td>
									<xsl:call-template
										name="processCompactChilds" />
								</tr>
							</xsl:otherwise>
						</xsl:choose>
						<!--
							<tr class="repeat-item" onclick="setRepeatIndex('{$repeat/@id}');">
							<xsl:if test="not($scripted='true')">
							<td class="selector-cell">
							<input type="radio" name="{$selector-prefix}{$outermost-id}" value="{$repeat-id}:{@chiba:position}"/>
							</td>
							</xsl:if>
							<xsl:for-each select="*">
							<xsl:variable name="css">
							<xsl:call-template name="assembleClasses"/>
							</xsl:variable>
							<td id="{@id}" class="{$css}">
							<xsl:call-template name="buildControl"/>
							</td>
							</xsl:for-each>
							</tr>
						-->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</table>
	</xsl:template>

	<!-- ### used by compact repeat ### -->
	<xsl:template name="processCompactChilds">
		<xsl:for-each select="*">
			<xsl:variable name="css">
				<xsl:call-template name="assembleClasses" />
			</xsl:variable>
			<td id="{@id}" class="{$css}">
				<xsl:call-template name="buildControl" />
			</td>
		</xsl:for-each>
	</xsl:template>

	<!-- ### handle repeat with 'full' appearance ### -->
	<xsl:template match="xforms:repeat[@xforms:appearance='full']">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				found full repeat
				<xsl:value-of select="xforms:label" />
				...
			</xsl:message>
		</xsl:if>

		<xsl:variable name="repeat" select="." />
		<xsl:variable name="group-css">
			<xsl:call-template name="assembleClasses" />
		</xsl:variable>

		<table
			class="{normalize-space(concat('full-repeat',' ',$group-css))}"
			id="{@id}">
			<xsl:if test="$scripted='true'">
				<!-- clone repeat prototype -->
				<!-- style attribute for safety in case CSS file is not there -->
				<tr class="repeat-prototype"
					onclick="setRepeatIndex('{@id}');" style="display:none;">
					<xsl:for-each
						select="chiba:data/xforms:group[@chiba:transient]">
						<td>
							<xsl:call-template name="full-group" />
						</td>
					</xsl:for-each>
				</tr>
			</xsl:if>

			<xsl:variable name="outermost-id"
				select="ancestor-or-self::xforms:repeat/@id" />
			<xsl:variable name="repeat-id" select="@id" />

			<!-- ***** loop repeat entries ***** -->
			<xsl:for-each select="xforms:group[@chiba:transient]">
				<xsl:choose>
					<xsl:when test="@chiba:selected='true'">
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item repeat-index"
									onclick="setRepeatIndex('{$repeat-id}');">
									<td>
										<xsl:call-template
											name="full-group" />
									</td>
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item repeat-index">
									<td class="selector-cell">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" checked="checked" />
									</td>
									<td>
										<xsl:call-template
											name="full-group" />
									</td>
								</tr>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="$scripted='true'">
								<tr class="repeat-item"
									onclick="setRepeatIndex('{$repeat-id}');">
									<td>
										<xsl:call-template
											name="full-group" />
									</td>
								</tr>
							</xsl:when>
							<xsl:otherwise>
								<tr class="repeat-item">
									<td class="selector-cell">
										<input type="radio"
											name="{$selector-prefix}{$outermost-id}"
											value="{$repeat-id}:{@chiba:position}" />
									</td>
									<td>
										<xsl:call-template
											name="full-group" />
									</td>
								</tr>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
		</table>
	</xsl:template>

	<!-- ### handle repeats attribute on foreign elements ### -->
	<xsl:template
		match="*[@xforms:repeat-bind]|*[@xforms:repeat-nodeset]">
		<xsl:apply-templates />
	</xsl:template>


	<!-- ######################################################################################################## -->
	<!-- ####################################### SWITCH ######################################################### -->
	<!-- ######################################################################################################## -->

	<!-- ### handle xforms:switch ### -->
	<xsl:template match="xforms:switch">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>handling switch</xsl:message>
		</xsl:if>
		<xsl:apply-templates />
	</xsl:template>

	<!-- ### handle selected xforms:case ### -->
	<xsl:template match="xforms:case[@xforms:selected='true']">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>handling selected case</xsl:message>
		</xsl:if>
		<xsl:apply-templates />
	</xsl:template>

	<!-- ### skip unselected xforms:case ### -->
	<xsl:template match="xforms:case">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>handling unselected case</xsl:message>
		</xsl:if>
	</xsl:template>

	<!-- ***** builds a string containing the correct css-classes reflecting UI-states like
		readonly/readwrite, enabled/disabled, valid/invalid ***** -->
	<xsl:template name="assembleClasses">

		<xsl:variable name="authorClasses">
			<xsl:call-template name="collectExistingClasses" />
		</xsl:variable>

		<!-- only execute if there's a data element which is e.g. not the case for unbound groups -->
		<xsl:variable name="pseudoClasses">
			<xsl:if test="chiba:data">
				<xsl:variable name="valid">
					<xsl:choose>
						<xsl:when
							test="string-length(chiba:data) = 0 and chiba:data/@chiba:visited='false'">
							valid
						</xsl:when>
						<xsl:when
							test="chiba:data/@chiba:valid='true'">
							valid
						</xsl:when>
						<xsl:otherwise>invalid</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:variable name="readonly">
					<xsl:choose>
						<xsl:when
							test="chiba:data/@chiba:readonly='true'">
							readonly
						</xsl:when>
						<xsl:otherwise>readwrite</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:variable name="required">
					<xsl:choose>
						<xsl:when
							test="chiba:data/@chiba:required='true'">
							required
						</xsl:when>
						<xsl:otherwise>optional</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:variable name="enabled">
					<xsl:choose>
						<xsl:when
							test="chiba:data/@chiba:enabled='true'">
							enabled
						</xsl:when>
						<xsl:otherwise>disabled</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<!-- <xsl:variable name="extattr">
					<xsl:choose>
					<xsl:when test="chiba:data/@int:extattr='project'">project</xsl:when>
					<xsl:otherwise>activity</xsl:otherwise>
					</xsl:choose>
					</xsl:variable>         
				-->
				<!-- <xsl:value-of select="concat(' ',$valid,' ',$readonly,' ',$required, ' ', $enabled, ' ', $extattr)"/>-->
				<xsl:value-of
					select="concat(' ',$valid,' ',$readonly,' ',$required, ' ', $enabled)" />
			</xsl:if>
		</xsl:variable>

		<xsl:value-of
			select="normalize-space(concat(local-name(),' ',$authorClasses,$pseudoClasses))" />

	</xsl:template>

	<!-- ####################################################################################################### -->
	<!-- #####################################  HELPER TEMPLATES '############################################## -->
	<!-- ####################################################################################################### -->

	<xsl:template name="buildControl">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				buildControl:
				<xsl:value-of select="name(.)" />
			</xsl:message>
		</xsl:if>
		<xsl:choose>
			<xsl:when test="local-name()='input'">
				<xsl:call-template name="input" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='output'">
				<xsl:call-template name="output" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='range'">
				<xsl:call-template name="range" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='secret'">
				<xsl:call-template name="secret" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='select'">
				<xsl:call-template name="select" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='select1'">
				<xsl:call-template name="select1" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='submit'">
				<xsl:call-template name="submit" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='trigger'">
				<xsl:call-template name="trigger" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='textarea'">
				<xsl:call-template name="textarea" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='upload'">
				<xsl:call-template name="upload" />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='repeat'">
				<xsl:apply-templates select="." />
			</xsl:when>
			<xsl:when test="local-name()='group'">
				<xsl:apply-templates select="." />
				<xsl:apply-templates select="xforms:help" />
				<xsl:apply-templates select="xforms:alert" />
			</xsl:when>
			<xsl:when test="local-name()='switch'">
				<xsl:apply-templates select="." />
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="labelClasses">

		<!-- only execute if there's a data element which is e.g. not the case for unbound groups -->
		<xsl:choose>
			<xsl:when test="self::chiba:data"
				xmlns:chiba="http://chiba.sourceforge.net/xforms">
				<xsl:variable name="enabled">
					<xsl:choose>
						<xsl:when
							test="chiba:data/@chiba:enabled='true'">
							enabled
						</xsl:when>
						<xsl:otherwise>disabled</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:value-of select="concat('label ',$enabled)" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'label'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="collectExistingClasses">
		<xsl:variable name="classes">
			<xsl:choose>
				<xsl:when test="@class">
					<xsl:value-of select="@class" />
				</xsl:when>
				<xsl:otherwise></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="$classes" />
	</xsl:template>

	<!-- build input control -->
	<xsl:template name="input">

		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				###repeat-id:
				<xsl:value-of select="$repeat-id" />
			</xsl:message>
			<xsl:message>
				###has repeat-id:
				<xsl:value-of
					select="boolean(string-length($repeat-id) > 0)" />
			</xsl:message>
			<xsl:message>
				###position:
				<xsl:value-of select="position()" />
			</xsl:message>
		</xsl:if>

		<xsl:element name="input">
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($data-prefix,$id)" />
			</xsl:attribute>
			<xsl:attribute name="type">text</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="chiba:data/text()" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(xforms:hint)" />
			</xsl:attribute>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<xsl:call-template name="assembleRepeatClasses">
				<xsl:with-param name="repeat-id" select="$repeat-id" />
				<xsl:with-param name="pos" select="$pos" />
				<xsl:with-param name="classes" select="'value'" />
			</xsl:call-template>
			<xsl:if test="$scripted='true'">
				<xsl:attribute name="onchange">
					javascript:setXFormsValue('
					<xsl:value-of select="$id" />
					');
				</xsl:attribute>
			</xsl:if>
		</xsl:element>

		<xsl:call-template name="handleRequired" />
		<xsl:call-template name="handleInteger" />
	</xsl:template>

	<!-- build output -->
	<xsl:template name="output">

		<xsl:variable name="css" select="@class" />
		<xsl:choose>
			<xsl:when test="@xforms:appearance='minimal'">
				<xsl:value-of select="chiba:data/text()" />
			</xsl:when>
			<xsl:when test="@xforms:appearance='image'">
				<xsl:element name="img">
					<xsl:attribute name="id">
						<xsl:value-of select="concat(@id,'-value')" />
					</xsl:attribute>
					<xsl:if test="$css">
						<xsl:attribute name="class">
							<xsl:value-of select="$css" />
						</xsl:attribute>
					</xsl:if>
					<xsl:attribute name="src">
						<xsl:value-of select="chiba:data/text()" />
					</xsl:attribute>
				</xsl:element>
			</xsl:when>
			<xsl:when test="@xforms:appearance='anchor'">
				<xsl:element name="a">
					<xsl:attribute name="id">
						<xsl:value-of select="concat(@id,'-value')" />
					</xsl:attribute>
					<xsl:if test="$css">
						<xsl:attribute name="class">
							<xsl:value-of select="$css" />
						</xsl:attribute>
					</xsl:if>
					<xsl:attribute name="href">
						<xsl:value-of select="chiba:data/text()" />
					</xsl:attribute>
					<xsl:value-of select="chiba:data/text()" />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="span">
					<xsl:attribute name="id">
						<xsl:value-of select="concat(@id,'-value')" />
					</xsl:attribute>
					<xsl:if test="$css">
						<xsl:attribute name="class">
							<xsl:value-of select="$css" />
						</xsl:attribute>
					</xsl:if>
					<xsl:value-of select="chiba:data/text()" />
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- build range -->
	<xsl:template name="range">
		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />
		<xsl:variable name="start" select="@xforms:start" />
		<xsl:variable name="end" select="@xforms:end" />
		<xsl:variable name="step" select="@xforms:step" />
		<xsl:variable name="showInput">
			<xsl:choose>
				<xsl:when test="@xforms:appearance='full'">
					true
				</xsl:when>
				<xsl:otherwise>false</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>


		<xsl:if test="$debug-enabled='true'">
			<xsl:message>WARN: range not supported yet</xsl:message>
		</xsl:if>


		<xsl:element name="script">
			<xsl:attribute name="language">JavaScript</xsl:attribute>
			createSlider2('
			<xsl:value-of select="$form-id" />
			', '
			<xsl:value-of select="concat($id,'-value')" />
			', '
			<xsl:value-of select="concat($data-prefix,$id)" />
			', '
			<xsl:value-of select="$start" />
			', '
			<xsl:value-of select="$end" />
			', '
			<xsl:value-of select="$step" />
			',
			<xsl:value-of select="$showInput" />
			, "", ""); setSlider('
			<xsl:value-of select="concat($data-prefix,$id)" />
			', '
			<xsl:value-of select="chiba:data/text()" />
			');
		</xsl:element>

		<xsl:call-template name="handleRequired" />
	</xsl:template>

	<!-- build secret control -->
	<xsl:template name="secret">
		<xsl:param name="maxlength" />

		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:element name="input">
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($data-prefix,$id)" />
			</xsl:attribute>
			<xsl:attribute name="type">password</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="chiba:data/text()" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(./xforms:hint)" />
			</xsl:attribute>
			<xsl:if test="$maxlength">
				<xsl:attribute name="maxlength">
					<xsl:value-of select="$maxlength" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<xsl:call-template name="assembleRepeatClasses">
				<xsl:with-param name="repeat-id" select="$repeat-id" />
				<xsl:with-param name="pos" select="$pos" />
				<xsl:with-param name="classes" select="'value'" />
			</xsl:call-template>
			<xsl:if test="$scripted='true'">
				<xsl:attribute name="onchange">
					javascript:setXFormsValue('
					<xsl:value-of select="$id" />
					');
				</xsl:attribute>
			</xsl:if>
		</xsl:element>

		<xsl:call-template name="handleRequired" />
		<xsl:call-template name="handleInteger" />
	</xsl:template>

	<xsl:template name="select">

		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:variable name="parent" select="." />
		<xsl:choose>
			<xsl:when test="chiba:data/@chiba:readonly='true'">
				<xsl:element name="textarea">
					<xsl:attribute name="cols">20</xsl:attribute>
					<xsl:attribute name="rows">3</xsl:attribute>
					<xsl:attribute name="readonly">
						readonly
					</xsl:attribute>
					<!--  <xsl:if test="chiba:data/@chiba:readonly='true'">
						<xsl:attribute name="disabled">disabled</xsl:attribute>
						</xsl:if>-->
					<xsl:call-template name="build-items-selected">
						<xsl:with-param name="value"
							select="chiba:data/text()" />
						<xsl:with-param name="parent" select="$parent" />
					</xsl:call-template>
				</xsl:element>


			</xsl:when>
			<xsl:otherwise>

				<xsl:choose>

					<xsl:when test="@xforms:appearance='compact'">

						<xsl:element name="select">
							<xsl:attribute name="id">
								<xsl:value-of
									select="concat($id,'-value')" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of
									select="concat($data-prefix,$id)" />
							</xsl:attribute>
							<xsl:attribute name="title">
								<xsl:value-of
									select="normalize-space(./xforms:hint)" />
							</xsl:attribute>
							<xsl:attribute name="multiple">
								true
							</xsl:attribute>
							<xsl:attribute name="size">5</xsl:attribute>
							<xsl:if
								test="chiba:data/@chiba:readonly='true'">
								<xsl:attribute name="disabled">
									disabled
								</xsl:attribute>
							</xsl:if>
							<xsl:attribute name="class">
								value
							</xsl:attribute>
							<xsl:call-template
								name="assembleRepeatClasses">
								<xsl:with-param name="repeat-id"
									select="$repeat-id" />
								<xsl:with-param name="pos"
									select="$pos" />
								<xsl:with-param name="classes"
									select="'value'" />
							</xsl:call-template>
							<xsl:if test="$scripted='true'">
								<xsl:attribute name="onchange">
									javascript:setXFormsValue('
									<xsl:value-of select="$id" />
									');
								</xsl:attribute>
							</xsl:if>
							<xsl:call-template name="build-items">
								<xsl:with-param name="value"
									select="chiba:data/text()" />
								<xsl:with-param name="parent"
									select="$parent" />
							</xsl:call-template>
						</xsl:element>
						<!-- create hidden parameter for deselection -->
						<input type="hidden"
							name="{concat($data-prefix,$id)}" value="" />
					</xsl:when>
					<xsl:when test="@xforms:appearance='full'">
						<xsl:call-template name="build-checkboxes">
							<xsl:with-param name="id" select="$id" />
							<xsl:with-param name="name"
								select="concat($data-prefix,$id)" />
							<xsl:with-param name="parent"
								select="$parent" />
						</xsl:call-template>
						<!-- create hidden parameter for identification and deselection -->
						<input type="hidden" id="{$id}-value"
							name="{concat($data-prefix,$id)}" value="" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:element name="select">
							<xsl:attribute name="id">
								<xsl:value-of
									select="concat($id,'-value')" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of
									select="concat($data-prefix,$id)" />
							</xsl:attribute>
							<xsl:attribute name="title">
								<xsl:value-of
									select="normalize-space(./xforms:hint)" />
							</xsl:attribute>
							<xsl:attribute name="multiple">
								true
							</xsl:attribute>
							<xsl:attribute name="size">3</xsl:attribute>
							<xsl:if
								test="chiba:data/@chiba:readonly='true'">
								<xsl:attribute name="disabled">
									disabled
								</xsl:attribute>
							</xsl:if>

							<xsl:call-template
								name="assembleRepeatClasses">
								<xsl:with-param name="repeat-id"
									select="$repeat-id" />
								<xsl:with-param name="pos"
									select="$pos" />
								<xsl:with-param name="classes"
									select="'value'" />
							</xsl:call-template>
							<xsl:if test="$scripted='true'">
								<xsl:attribute name="onchange">
									javascript:setXFormsValue('
									<xsl:value-of select="$id" />
									');
								</xsl:attribute>
							</xsl:if>
							<xsl:call-template name="build-items">
								<xsl:with-param name="value"
									select="chiba:data/text()" />
								<xsl:with-param name="parent"
									select="$parent" />
							</xsl:call-template>
						</xsl:element>
						<!-- create hidden parameter for deselection -->
						<input type="hidden"
							name="{concat($data-prefix,$id)}" value="" />
					</xsl:otherwise>

				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>


		<xsl:call-template name="handleRequired" />
	</xsl:template>

	<xsl:template name="select1">

		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:variable name="parent" select="." />

		<!--       <xsl:choose>
			<xsl:when test="chiba:data/@chiba:readonly='true'">
			<xsl:element name="textarea">
			<xsl:attribute name="cols">20</xsl:attribute>
			<xsl:attribute name="rows">3</xsl:attribute>
			<xsl:attribute name="readonly">readonly</xsl:attribute>
			
			<xsl:call-template name="build-items-selected">
			<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
			</xsl:element>
			
			
			</xsl:when>
			<xsl:otherwise> -->
		<xsl:choose>
			<xsl:when test="@xforms:appearance='compact'">
				<xsl:element name="select">
					<xsl:attribute name="id">
						<xsl:value-of select="concat($id,'-value')" />
					</xsl:attribute>
					<xsl:attribute name="name">
						<xsl:value-of select="concat($data-prefix,$id)" />
					</xsl:attribute>
					<xsl:attribute name="size">5</xsl:attribute>
					<xsl:attribute name="title">
						<xsl:value-of
							select="normalize-space(./xforms:hint)" />
					</xsl:attribute>

					<xsl:call-template name="assembleRepeatClasses">
						<xsl:with-param name="repeat-id"
							select="$repeat-id" />
						<xsl:with-param name="pos" select="$pos" />
						<xsl:with-param name="classes" select="'value'" />
					</xsl:call-template>
					<xsl:if test="chiba:data/@chiba:readonly='true'">
						<xsl:attribute name="disabled">
							disabled
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="$scripted='true'">
						<xsl:attribute name="onchange">
							javascript:setXFormsValue('
							<xsl:value-of select="$id" />
							');
						</xsl:attribute>
					</xsl:if>
					<xsl:call-template name="build-items">
						<xsl:with-param name="parent" select="$parent" />
					</xsl:call-template>
				</xsl:element>
				<!-- create hidden parameter for deselection -->
				<input type="hidden" name="{concat($data-prefix,$id)}"
					value="" />
			</xsl:when>
			<xsl:when test="@xforms:appearance='full'">
				<xsl:call-template name="build-radiobuttons">
					<xsl:with-param name="id" select="$id" />
					<xsl:with-param name="name"
						select="concat($data-prefix,$id)" />
					<xsl:with-param name="parent" select="$parent" />
				</xsl:call-template>
				<!-- create hidden parameter for identification and deselection -->
				<input type="hidden" id="{$id}-value"
					name="{concat($data-prefix,$id)}" value="" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="select">
					<xsl:attribute name="id">
						<xsl:value-of select="concat($id,'-value')" />
					</xsl:attribute>
					<xsl:attribute name="name">
						<xsl:value-of select="concat($data-prefix,$id)" />
					</xsl:attribute>
					<xsl:attribute name="size">1</xsl:attribute>
					<xsl:attribute name="title">
						<xsl:value-of
							select="normalize-space(./xforms:hint)" />
					</xsl:attribute>
					<xsl:call-template name="assembleRepeatClasses">
						<xsl:with-param name="repeat-id"
							select="$repeat-id" />
						<xsl:with-param name="pos" select="$pos" />
						<xsl:with-param name="classes" select="'value'" />
					</xsl:call-template>
					<xsl:if test="chiba:data/@chiba:readonly='true'">
						<xsl:attribute name="disabled">
							disabled
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="$scripted='true'">
						<xsl:attribute name="onchange">
							javascript:setXFormsValue('
							<xsl:value-of select="$id" />
							');
						</xsl:attribute>
					</xsl:if>
					<xsl:call-template name="build-items">
						<xsl:with-param name="parent" select="$parent" />
					</xsl:call-template>
				</xsl:element>
				<!-- create hidden parameter for deselection -->
				<input type="hidden" name="{concat($data-prefix,$id)}"
					value="" />
			</xsl:otherwise>
		</xsl:choose>
		<!-- </xsl:otherwise>
			</xsl:choose> -->
		<xsl:call-template name="handleRequired" />
	</xsl:template>

	<!-- build submit -->
	<xsl:template name="submit">
		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:element name="input">
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:choose>
				<xsl:when test="$scripted='true'">
					<xsl:attribute name="type">button</xsl:attribute>
					<xsl:attribute name="onclick">
						javascript:activate('
						<xsl:value-of select="$id" />
						');
					</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="type">submit</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($trigger-prefix,$id)" />
			</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="xforms:label" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(xforms:hint)" />
			</xsl:attribute>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<!--            <xsl:if test="chiba:data/@chiba:enabled='false'">-->
			<!--                <xsl:attribute name="disabled">true</xsl:attribute>-->
			<!--            </xsl:if>-->
			<xsl:call-template name="assembleRepeatClasses">
				<xsl:with-param name="repeat-id" select="$repeat-id" />
				<xsl:with-param name="pos" select="$pos" />
				<xsl:with-param name="classes" select="'value'" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<!-- build trigger -->
	<!-- ### please note that triggers are always submit buttons cause this stylesheet assumes no javascript ### -->
	<xsl:template name="trigger">
		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:element name="input">
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($trigger-prefix,$id)" />
			</xsl:attribute>
			<xsl:choose>
				<xsl:when test="$scripted='true'">
					<xsl:attribute name="type">button</xsl:attribute>
					<xsl:attribute name="onclick">
						javascript:activate('
						<xsl:value-of select="$id" />
						');
					</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="type">submit</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:attribute name="value">
				<xsl:value-of select="xforms:label" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(xforms:hint)" />
			</xsl:attribute>
			<xsl:call-template name="assembleRepeatClasses">
				<xsl:with-param name="repeat-id" select="$repeat-id" />
				<xsl:with-param name="pos" select="$pos" />
				<xsl:with-param name="classes" select="'value'" />
			</xsl:call-template>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<!--            <xsl:if test="chiba:data/@chiba:enabled='false'">-->
			<!--                <xsl:attribute name="disabled">true</xsl:attribute>-->
			<!--            </xsl:if>-->
			<xsl:if test="@xforms:accesskey">
				<xsl:attribute name="accesskey">
					<xsl:value-of select="@xforms:accesskey" />
				</xsl:attribute>
				<xsl:attribute name="title">
					<xsl:value-of select="normalize-space(xforms:hint)" />
					- KEY: [ALT]+
					<xsl:value-of select="@xforms:accesskey" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if
				test="contains(@xforms:src,'.gif') or contains(@xforms:src,'.jpg') or contains(@xforms:src,'.png')">
				<img src="{@xforms:src}" id="{@id}-label" />
			</xsl:if>

		</xsl:element>

	</xsl:template>

	<!-- build textarea control -->
	<xsl:template name="textarea">
		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:element name="textarea">
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($data-prefix,$id)" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(./xforms:hint)" />
			</xsl:attribute>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<xsl:call-template name="assembleRepeatClasses">
				<xsl:with-param name="repeat-id" select="$repeat-id" />
				<xsl:with-param name="pos" select="$pos" />
				<xsl:with-param name="classes" select="'value'" />
			</xsl:call-template>
			<xsl:if test="$scripted='true'">
				<xsl:attribute name="onchange">
					javascript:setXFormsValue('
					<xsl:value-of select="$id" />
					');
				</xsl:attribute>
			</xsl:if>
			<xsl:value-of select="chiba:data/text()" />
		</xsl:element>

		<xsl:call-template name="handleRequired" />
		<xsl:call-template name="handleInteger" />
	</xsl:template>

	<!-- build upload control -->
	<xsl:template name="upload">
		<!-- the stylesheet using this template has to take care, that form enctype is set to 'multipart/form-data' -->
		<xsl:variable name="repeat-id"
			select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
		<xsl:variable name="pos" select="position()" />
		<xsl:variable name="id" select="@id" />

		<xsl:choose>
			<xsl:when
				test="not(xforms:filename/chiba:data/text() = '')">
				<span class="upload-label label">
					<a
						href="{concat($show-attachment-action,'?id=',$id)}"
						class="upload-show">
						<xsl:value-of
							select="xforms:filename/chiba:data/text()" />
					</a>
				</span>
				<!-- form handler must know about the remove-upload-prefix and handle it -->
				<input type="submit" class="upload-remove value"
					name="{concat($remove-upload-prefix,$id)}" value="Remove" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="input">
					<xsl:attribute name="id">
						<xsl:value-of select="concat($id,'-value')" />
					</xsl:attribute>
					<xsl:attribute name="name">
						<xsl:value-of select="concat($data-prefix,$id)" />
					</xsl:attribute>
					<xsl:attribute name="type">file</xsl:attribute>
					<xsl:attribute name="value"></xsl:attribute>
					<xsl:attribute name="title">
						<xsl:value-of
							select="normalize-space(xforms:hint)" />
					</xsl:attribute>
					<xsl:if test="chiba:data/@chiba:readonly='true'">
						<xsl:attribute name="disabled">
							disabled
						</xsl:attribute>
					</xsl:if>

					<xsl:call-template name="assembleRepeatClasses">
						<xsl:with-param name="repeat-id"
							select="$repeat-id" />
						<xsl:with-param name="pos" select="$pos" />
						<xsl:with-param name="classes" select="'value'" />
					</xsl:call-template>

					<!-- Content types accepted, from mediatype xforms:upload attribute
						to accept input attribute -->
					<xsl:attribute name="accept">
						<xsl:value-of
							select="translate(normalize-space(@xforms:mediatype),' ',',')" />
					</xsl:attribute>
					<xsl:if test="$scripted='true'">
						<xsl:choose>
							<xsl:when test="@xforms:onchange">
								<xsl:attribute name="onchange">
									<xsl:value-of
										select="@xforms:onchange" />
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="onchange">
									javascript:upload('
									<xsl:value-of select="$id" />
									');
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</xsl:element>
				<xsl:if test="xforms:filename">
					<input type="hidden" id="{xforms:filename/@id}"
						value="{xforms:filename/chiba:data}" />
				</xsl:if>
				<xsl:if test="@chiba:destination">
					<!-- create hidden parameter for destination -->
					<input type="hidden" id="{$id}-destination"
						value="{@chiba:destination}" />
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:call-template name="handleRequired" />
	</xsl:template>

	<!-- ########## builds indexed classname for styling repeats rendered as tables ########## -->
	<xsl:template name="assembleRepeatClasses">
		<xsl:param name="repeat-id" />
		<xsl:param name="pos" />
		<xsl:param name="classes" />
		<xsl:choose>
			<xsl:when test="boolean(string-length($repeat-id) > 0)">
				<xsl:attribute name="class">
					<xsl:value-of
						select="concat($repeat-id,'-',$pos,' ',$classes)" />
				</xsl:attribute>
			</xsl:when>
			<xsl:when test="boolean(string-length(@class) > 0)">
				<xsl:attribute name="class">
					<xsl:value-of select="concat(@class, ' ',$classes)" />
				</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class">
					<xsl:value-of select="$classes" />
				</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- handles required/optional property -->
	<xsl:template name="handleRequired">
		<xsl:choose>
			<xsl:when test="chiba:data/@chiba:required='true'">
				<span id="{@id}-required" class="required-symbol">
					*
				</span>
			</xsl:when>
			<xsl:otherwise>
				<span id="{@id}-required" class="required-symbol"></span>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- handles integer/string property -->
	<xsl:template name="handleInteger">
		<xsl:choose>
			<xsl:when test="chiba:data/@chiba:type='integer'">
				<span id="{@id}-integer" class="integer-symbol">
					1234...
				</span>
			</xsl:when>
			<xsl:otherwise>
				<span id="{@id}-string" class="string-symbol"></span>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- ######################################################################################################## -->
	<!-- ########################################## HELPER TEMPLATES FOR SELECT, SELECT1 ######################## -->
	<!-- ######################################################################################################## -->

	<xsl:template name="build-items">
		<xsl:param name="parent" />

		<!-- add an empty item, cause otherwise deselection is not possible -->
		<!--
			<option value="">
			<xsl:if test="string-length($parent/chiba:data/text()) = 0">
			<xsl:attribute name="selected">selected</xsl:attribute>
			</xsl:if>
			</option>
		-->
		<!-- todo: handle xforms:choice -->
		<xsl:variable name="items"
			select="$parent//xforms:item[not(ancestor::chiba:data)]" />
		<xsl:for-each select="$items">
			<option id="{@id}-value" value="{xforms:value}"
				title="{xforms:hint}">
				<xsl:if test="@xforms:selected='true'">
					<xsl:attribute name="selected">
						selected
					</xsl:attribute>
				</xsl:if>
				<xsl:value-of select="xforms:label" />
			</option>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="build-items-selected">
		<xsl:param name="parent" />
		<!-- add an empty item, cause otherwise deselection is not possible -->
		<!--
			<option value="">
			<xsl:if test="string-length($parent/chiba:data/text()) = 0">
			<xsl:attribute name="selected">selected</xsl:attribute>
			</xsl:if>
			</option>
		-->
		<xsl:variable name="newline">
			<xsl:text>
</xsl:text>
		</xsl:variable>
		<xsl:variable name="items"
			select="$parent//xforms:item[not(ancestor::chiba:data)]" />
		<xsl:for-each select="$items">
			<xsl:if test="@xforms:selected='true'">

				<xsl:value-of select="xforms:label" />
				<xsl:value-of select="$newline" />

			</xsl:if>

		</xsl:for-each>
	</xsl:template>

	<!-- overwrite/change this template, if you don't like the way labels are rendered for checkboxes -->
	<xsl:template name="build-checkboxes">
		<xsl:param name="id" />
		<xsl:param name="name" />
		<xsl:param name="parent" />

		<!-- todo: handle xforms:choice -->
		<xsl:variable name="items"
			select="$parent//xforms:item[not(ancestor::chiba:data)]" />
		<xsl:for-each select="$items">
			<xsl:variable name="title">
				<xsl:choose>
					<xsl:when test="xforms:hint">
						<xsl:value-of select="xforms:hint" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$parent/xforms:hint" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<input id="{@id}-value" class="value" type="checkbox"
				name="{$name}" value="{xforms:value}" title="{$title}">
				<xsl:if
					test="$parent/chiba:data/@chiba:readonly='true'">
					<xsl:attribute name="disabled">
						disabled
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="@xforms:selected='true'">
					<xsl:attribute name="checked">
						checked
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="$scripted='true'">
					<xsl:attribute name="onclick">
						javascript:setXFormsValue('
						<xsl:value-of select="$parent/@id" />
						');
					</xsl:attribute>
				</xsl:if>
			</input>
			<span id="{@id}-label" class="label">
				<xsl:if
					test="$parent/chiba:data/@chiba:readonly='true'">
					<xsl:attribute name="disabled">
						disabled
					</xsl:attribute>
				</xsl:if>
				<xsl:apply-templates select="xforms:label" />
			</span>
		</xsl:for-each>
	</xsl:template>

	<!-- overwrite/change this template, if you don't like the way labels are rendered for checkboxes -->
	<xsl:template name="build-radiobuttons">
		<xsl:param name="id" />
		<xsl:param name="name" />
		<xsl:param name="parent" />

		<!-- todo: handle xforms:choice -->
		<xsl:variable name="items"
			select="$parent//xforms:item[not(ancestor::chiba:data)]" />
		<xsl:for-each select="$items">
			<xsl:variable name="title">
				<xsl:choose>
					<xsl:when test="xforms:hint">
						<xsl:value-of select="xforms:hint" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$parent/xforms:hint" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<input id="{@id}-value" class="value" type="radio"
				name="{$name}" value="{xforms:value}" title="{$title}">
				<xsl:if
					test="$parent/chiba:data/@chiba:readonly='true'">
					<xsl:attribute name="disabled">
						disabled
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="@xforms:selected='true'">
					<xsl:attribute name="checked">
						checked
					</xsl:attribute>
				</xsl:if>
				<xsl:if test="$scripted='true'">
					<xsl:attribute name="onclick">
						javascript:setXFormsValue('
						<xsl:value-of select="$parent/@id" />
						');
					</xsl:attribute>
				</xsl:if>
			</input>
			<span id="{@id}-label" class="label">
				<xsl:if
					test="$parent/chiba:data/@chiba:readonly='true'">
					<xsl:attribute name="disabled">
						disabled
					</xsl:attribute>
				</xsl:if>
				<xsl:apply-templates select="xforms:label" />
			</span>
		</xsl:for-each>
	</xsl:template>

	<!-- chooses the CSS stylesheet to use -->
	<xsl:template name="getCSS">
		<xsl:if test="$debug-enabled='true'">
			<xsl:message>
				user agent:
				<xsl:value-of select="$user-agent" />
			</xsl:message>
		</xsl:if>

		<xsl:choose>
			<!-- if the 'css-file' parameter has been set this takes precedence -->
			<xsl:when test="string-length($css-file) > 0">
				<link rel="stylesheet" type="text/css"
					href="{$css-file}" />
			</xsl:when>
			<!-- if there's a stylesheet linked plainly, then take this stylesheet. -->
			<xsl:when test="xhtml:link">
				<link rel="stylesheet" type="text/css"
					href="{xhtml:link/@href}" />
			</xsl:when>
			<!-- if there's an chiba:style statement block....
				Attention: the user-agent must match (case-sensitive) to output a link element. Otherwise no stylesheet will be linked.
			-->
			<xsl:when test="chiba:style">
				<xsl:choose>
					<xsl:when test="contains($user-agent,'IE')">
						<link rel="stylesheet" type="text/css"
							href="{.//chiba:useragent[@test='IE']/@href}" />
					</xsl:when>
					<xsl:when test="contains($user-agent,'Mozilla')">
						<link rel="stylesheet" type="text/css"
							href="{.//chiba:useragent[@test='Mozilla']/@href}" />
					</xsl:when>
					<xsl:otherwise>
						<link rel="stylesheet" type="text/css"
							href="{$default-css}" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<!--  if nothings present standard stylesheets for Mozilla and IE are choosen. -->
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="contains($user-agent,'IE')">
						<link rel="stylesheet" type="text/css"
							href="{$ie-css}" />
					</xsl:when>
					<xsl:when test="contains($user-agent,'Mozilla')">
						<link rel="stylesheet" type="text/css"
							href="{$mozilla-css}" />
					</xsl:when>
					<xsl:otherwise>
						<link rel="stylesheet" type="text/css"
							href="{$default-css}" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="xhtml:style">
			<style type="text/css">
				<xsl:value-of select="xhtml:style" />
			</style>
		</xsl:if>
	</xsl:template>

	<xsl:template name="selectorName">
		<xsl:variable name="repeat-id"
			select="ancestor-or-self::xforms:repeat/@id"
			xmlns:xforms="http://www.w3.org/2002/xforms" />
		<xsl:value-of select="concat($selector-prefix, $repeat-id)" />
	</xsl:template>

	<!-- build image trigger / submit -->
	<xsl:template name="image-trigger">
		<xsl:element name="input">
			<xsl:variable name="id" select="@id" />
			<xsl:variable name="repeat-id"
				select="ancestor::*[name(.)='xforms:repeat'][1]/@id" />
			<xsl:attribute name="id">
				<xsl:value-of select="concat($id,'-value')" />
			</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="concat($trigger-prefix,$id)" />
			</xsl:attribute>
			<xsl:attribute name="type">image</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="xforms:label" />
			</xsl:attribute>
			<xsl:attribute name="title">
				<xsl:value-of select="normalize-space(xforms:hint)" />
			</xsl:attribute>
			<xsl:attribute name="src">
				<xsl:value-of select="xforms:label/@xlink:href" />
			</xsl:attribute>
			<xsl:attribute name="class">value</xsl:attribute>
			<xsl:if test="chiba:data/@chiba:readonly='true'">
				<xsl:attribute name="disabled">disabled</xsl:attribute>
			</xsl:if>
			<xsl:if test="$scripted='true'">
				<xsl:attribute name="onclick">
					javascript:activate('
					<xsl:value-of select="$id" />
					');
				</xsl:attribute>
			</xsl:if>
		</xsl:element>

	</xsl:template>

</xsl:stylesheet>
