/*
*
* JMSServicesClient.java - 
* Copyright (C) 2004 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package hero.client.events;

import javax.jms.TopicConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.jms.TopicPublisher;
import javax.jms.TextMessage;
import javax.jms.ObjectMessage;
import javax.jms.JMSException;
import javax.jms.Session;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.naming.NamingException;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.io.InputStreamReader;
import java.util.Hashtable;
import java.util.Enumeration;


public class JMSServicesClient {

    private static JMSServicesClient jms = null; // Singleton pattern

	private static String                  topicName = "testTopic";
	private static Context                 jndiContext = null;
	private static TopicConnectionFactory  topicConnectionFactory = null;
	private static TopicSession            topicSession = null;
	private static Topic                   topic = null;
	private static TopicConnection         topicConnection = null;
	private static TopicSubscriber         topicSubscriber = null;
	private static TopicPublisher          topicPublisher = null;
	private static TextMessage             message = null;
	private static ObjectMessage           omessage = null;
	private static InputStreamReader       inputStreamReader = null;

    public JMSServicesClient() throws Exception{             

        /* 
         * Create a JNDI InitialContext object if none exists yet.
         */

        try {

        jndiContext = new InitialContext();
        } catch (NamingException e) {
	    throw new Exception(e.getMessage(),e);
        }

        /*
         * Look up connection factory and topic.  If either does
         * not exist, exit.
         */


        try {
             topicConnectionFactory = (TopicConnectionFactory) jndiContext.lookup("JTCF");
	     topic = (Topic) jndiContext.lookup(topicName);            
        } catch (NamingException e) {
            System.out.println("JNDI lookup failed: " 
                + e.toString());
	    throw new Exception(e.getMessage(),e);
	    //            System.exit(1);
        }

	/*
	 * Create connection.
	 * Create session from connection; false means session is
	 * not transacted.
	 */
	 

	try { 
	    topicConnection = topicConnectionFactory.createTopicConnection();
	    topicSession = topicConnection.createTopicSession(false,Session.AUTO_ACKNOWLEDGE);   
	} catch (JMSException e) {
	    System.out.println("Exception occurred: " + e.toString());
	}

    }
    
    /**
     * Returns the instance of the JMSServices
     * Implementation of the Singleton pattern.
    */
    
    public static JMSServicesClient getInstance() throws Exception {
        if (jms == null) {
           jms = new JMSServicesClient();
        }   
        return jms;

    }

    /**
     * Create subscriber.
     * Register message listener (TextListener).
     * Receive text messages from topic.
     */ 

    public void createSubscription(MessageListener listener, String evt) throws Exception {

	try {
	    topicSubscriber = topicSession.createSubscriber(topic, evt,false);
	    topicSubscriber.setMessageListener(listener);
	    topicConnection.start();
	} catch (JMSException e) {e.printStackTrace();
	  System.out.println("Exception occurred: " 
			     + e.toString());
	}
    }


    public static Hashtable getEvt(Message  message) throws JMSException
    {
	Enumeration keys = message.getPropertyNames();
	Hashtable evt = new Hashtable();
	
	while (keys.hasMoreElements())
	{
	    String key = (String)keys.nextElement();
	    evt.put(key,message.getObjectProperty(key));
	}
	return evt;
    }

    public static java.io.Serializable getEvtObject(Message  message) throws JMSException
    {
	return (((ObjectMessage)message).getObject());
    }
    
    public void closeConnection() {
	
	if (topicConnection != null) {
	    try {
		topicConnection.close();
	    } catch (JMSException e) {}
	}
    }
	
}
