/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BonitaActivityView.java,v 1.1 2004/07/30 14:57:57 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.client.grapheditor;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;

import com.jgraph.JGraph;
import com.jgraph.graph.CellMapper;
import com.jgraph.graph.CellViewRenderer;
import com.jgraph.graph.GraphConstants;
import com.jgraph.graph.VertexRenderer;
import com.jgraph.graph.VertexView;
import java.util.Map;
import javax.swing.ImageIcon;
import java.awt.Toolkit;

public class BonitaActivityView extends VertexView {

	public ActivityRenderer renderer;

	public BonitaActivityView(Object arg0, JGraph arg1, CellMapper arg2) {
		super(arg0, arg1, arg2);
		renderer = new ActivityRenderer((BonitaActivityCell) arg0);
	}

	public CellViewRenderer getRenderer() {
		return renderer;
	}

	public static class ActivityRenderer extends VertexRenderer {

		private BonitaActivityCell activity;

		public ActivityRenderer(BonitaActivityCell rb) {
			activity = rb;
		}

		public void paint(Graphics g) {

			Graphics2D g2 = (Graphics2D) g;
			Dimension d = getSize();
			boolean tmp = selected;

			// ??
			try {
				setBorder(null);
				setOpaque(false);
				setText("");
				selected = false;
				super.paint(g);
			} finally {
				selected = tmp;
			}

			// choice stroke style
			if (selected) {
				g2.setStroke(GraphConstants.SELECTION_STROKE);
			} else {
				g2.setStroke(new BasicStroke(1));
			}

			int y = ((getHeight() - 19) / 2) + 1;

			Color contour = Color.decode("#3E6AAB");
			Map props = activity.getAttributes();	

			// draw box
			if (!selected) {
				g2.setColor(getBackground());
				if (((String)props.get("subProcess")).equals("yes"))
					g2.fillOval(4, 4, getWidth() - 9, getHeight() - 9);
				else
					g2.fillRoundRect(4, 4, getWidth() - 9, getHeight() - 9, 10, 10);
			}
			g2.setColor(contour);
					
			if (((String)props.get("subProcess")).equals("yes"))
				g2.drawOval(4, 4, getWidth() - 9, getHeight() - 9);
			else
			    g2.drawRoundRect(4, 4, getWidth() - 9, getHeight() - 9, 10, 10);

			// Set icon path
			Map propers = activity.getAttributes();	
			String iteration = (String)propers.get("iterate");
			if (!iteration.equals("false")) {
				String iconPath=null; 
			   if (iteration.equals("from"))
			     iconPath = "images/from.gif";
			   else
			     iconPath = "images/to.gif";
			// draw icon
			  try {
				java.net.URL iconUrl = Thread.currentThread().getContextClassLoader().getResource(iconPath);
				//g2.drawImage(IconManager.getIcon(iconPath),11,y,19,19,Frame.getFrame().getWorkflowGraph());
				g2.drawImage(Toolkit.getDefaultToolkit().getImage(iconUrl),5,y,10,10,Frame.getFrame().getWorkflowGraph());
			  } catch(Exception e){
				System.out.println("Unable to load image " + iconPath);				
			  }
			}
					
			// draw name
			g2.setColor(Color.WHITE);
			if (((String)props.get("subProcess")).equals("yes"))
				g2.setFont(g2.getFont().deriveFont((float) (12)));
			else
			   g2.setFont(g2.getFont().deriveFont((float) (11)));
			g2.setFont(g2.getFont().deriveFont(Font.ITALIC));
			
			String name = (String)props.get("name");
			g2.drawString(name, 11, y + 12);
		}
	}

}
