/*
 * @(#)Spring.java	1.0 1/1/02
 *
 * Copyright (C) 2001 Gaudenz Alder
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package hero.client.grapheditor;

import com.jgraph.JGraph;
import com.jgraph.graph.*;
import java.awt.Rectangle;
import java.awt.Point;
import java.util.*;

/**
 *  Class to implement Kamada and Kawai's spring algorithm
 *                      with modifications).
 */

public class Spring {

static java.util.ResourceBundle resource = java.util.ResourceBundle.getBundle("resources.Traduction")/*#BundleType=List*/;

    protected WFGraph graph;

    /** Reference the array of vertices we work on */
    private CellView[] vertices = null;

   private static Rectangle rect;
   private static double xmax,xmin,ymax,ymin;
   private static int orderedList[];	// this is used to enable a quick lookup
					// of nodes' array values (which might
					// differ from their index values)
   private static int N;  // nodes in graph; will be set in compute function
   private static int EDGES;  // edges in graph; set in compute function
   private static long D[][]; //the graphical distances between all pairs
   private static double K[][]; // spring strengths
   private static double Ko;    // spring constant
   private static double L[][]; // ideal spring lengths
   private static double epsilon;
   private static double delta[];
   private static int MAX_TIMES_REPOSITIONED = 10; // constant to limit number
						  // of times thru "inner" loop
   private static int numTimesRepositioned = 0;  // counts number of times
						 // thru "inner" loop
   private static int NUM_TIMES_MOVED[];	// counts number of times each
						// vertex has been selected to
						// move

   private static int HY_SIZE = 10;	//  these variables control the E
   private static int HY_PERCENTAGE = 5;//   stopping condition and how it
   private static double E,E_HY[];	//   keeps track of its information
   private static int COUNTER = 0;

   private static int mv = -1;  // the node chosen to move at each iteration
   private static double partial_x,partial_y,partial_xx,
		partial_xy,partial_yx,partial_yy;
   private static boolean connected;

   private int DEBUG = 3;

   public Spring() {
    this(3);
  }

  public Spring(int debug) {
    DEBUG = debug;
  }

   public String compute(WFGraph G)
     {
      this.graph = G;
      double tempValue,maxValue,maxDel; int maxTimes = 0;
      int mvi,i;
      int v,u; //DefaultVertex v,u;

      rect = G.fromScreen(G.getBounds());

      // Init array of vertices
      Initialize(G);

      // if graph not entirely contained in drawing area, change drawing area?
      // change graph?? (want to make sure that graph is inside the drawing
      // area before starting....)  For now (with the "non-bouncing
      // moving-function" used), this doesn't need to be enforced)

      // Also check to make sure that graph is connected, or else we need to
      // exit here
      /*
         for(i=0;i<G.getModel().getVertexCount();i++)
            if (connected == false)
              {
               //System.out.println("Error:  This algorithm should not be run on a non-connected graph!");
               return "Error:  This algorithm should not be run on a non-connected graph!";
              }
      */
      //  and undirected (maybe a prompt dialog box to ask if it should be made
      // undirected or if we should cancel the algorithm) but for now just exit
      // here.
         if (false) //(G.isDirected())
           {
            //System.out.println("Error:  This algorithm should not be run on an undirected graph.");
            return resource.getString("spring.error");
           }


      // Done with primary checking start main iterative loop:
      //int DEBUG = 3;
      while(DEBUG-->0)
        {
         // if any pair of vertices share the same position, move them away
         CheckPositions(G);

         // determine the next vertex to move according to TempFunction

         maxDel = 0;
         for (int index = 0; index < N; index++) {
          v = index; //G.getModel().getVertex(index);

           delta[v]=Math.sqrt(findDelta2(G,v));
           if (delta[v] > maxDel)
              maxDel = delta[v];
           if (NUM_TIMES_MOVED[v] > maxTimes)
              maxTimes = NUM_TIMES_MOVED[v];
          //System.out.println("Delta v="+delta[v]+" v="+v);
         }

         mv = 0; mvi = 0;
         maxValue = 0;
         v = mv;
         for (int index = 1; index < N; index++)
         {
            v = index;
            if (maxTimes != 0)
               tempValue = TempFunction
	         (delta[v]/maxDel,
	         1 - NUM_TIMES_MOVED[v]);
            else
               // the first time, base the decision only upon the delta
               tempValue = delta[v]/maxDel;
            if (tempValue > maxValue)
              {
               mv = v; mvi = mv;
               maxValue = tempValue;
              }
           }


         // see if any stopping conditions are met
         // DEBUG HERE
         //System.out.println(System.currentTimeMillis()+" testing conditions");
         if (delta[mvi] < epsilon)
           {
            //System.out.println("mvi="+mvi+" epsilon="+epsilon+" delta[mvi]="+delta[mvi]);
            break;
           }
         if (COUNTER > HY_SIZE)
            if (E_HY[COUNTER%HY_SIZE] * HY_PERCENTAGE/100.0  > (E = findE(G)))
              {
               //System.out.println("Quitting because of history");
               break;
              }

         // can't quit yet, so put the E in the history array and get ready to
	 // find a new position for vertex 'mv'

         E_HY[COUNTER%HY_SIZE] = E;

         numTimesRepositioned = 0;

         while ( (delta[mvi] > epsilon)
			 && (numTimesRepositioned < MAX_TIMES_REPOSITIONED) )
           {
            //MoveToNewPosition(G);  // this function simulates elastic collision
				   //  instead of hitting & sticking
            //MoveToNewPosition1(G); // this function "hits and sticks"
            MoveToNewPosition2(G); // this function doesn't care
            delta[mvi] = Math.sqrt(findDelta2(G,mv));
            numTimesRepositioned++;
           }//end "inner" while

         NUM_TIMES_MOVED[mvi]++;

         //if redraw is on ???

              //update.update(false);

         // if the cool stop button (that isn't implemented) has been pressed
              // redraw
              // break;

        }//end "outer" while
      return null;

     }//end main compute function





   private void Initialize(JGraph G)
     {
      //CellView v,u;
      int i;
      EDGES = 0;
      // Create array of vertices amd count edges
      Object[] cells = G.getRoots();
      LinkedList list = new LinkedList();
      for (int idx = 0; idx < cells.length; idx++)
	if (graph.isVertex(cells[idx]))
          list.add(cells[idx]);
        else if (graph.isEdge(cells[idx]))
          EDGES++;

      // Create array
      vertices = graph.getView().getMapping(list.toArray());
      N = vertices.length; //G.numberOfNodes();

      makeOrderedList(G);
      NUM_TIMES_MOVED = new int[N];
      for (i=0;i<N;i++)
         NUM_TIMES_MOVED[i] = 0;
      EDGES=0;
      /*
      for (v = G.firstNode(); v !=null; v = G.nextNode(v))
        {
         for (u = G.nextNode(v); u !=null; u = G.nextNode(u))
            if (v.hasChild(u)) EDGES++;
        }
      */
      //EDGES=G.getModel().getEdgeCount();
      epsilon = 0.5*(N+EDGES);
      findDistances(G);
      find_l_and_k(G);
      delta = new double[N];
      E_HY = new double[HY_SIZE];
      xmin = rect.x;  xmax = xmin + rect.width;
      ymin = rect.y;  ymax = ymin + rect.height;
     }

   private void findDistances(JGraph G)
     {
      int s,u,v,p;
      int i,j,vi,pi,si,ui;
      boolean done[];
      Queue queue;

      queue = new Queue();
      int vertexCount = vertices.length; //G.getModel().getVertexCount();
      done = new boolean[vertexCount];
      D = new long[vertexCount][vertexCount];


      /* Initialize the distance matrix to 0s */
      for (i=0;i<vertexCount;i++)
         for (j=i;j<vertexCount;j++)
            { D[i][j] = 0; D[j][i] = 0; }


      /* For each node, do Dijstra... */
      for (int index = 0; index < vertexCount; index++) {

         /* Initialize the done array to false */
         for (i=0;i<vertexCount;i++)
            done[i] = false;

         done[index] = true;

         // Create array of neighbours
         CellView[] adj = getVerticesFor(G, index);

         // Insert all (index,n) pairs
         for (int idx2 = 0; idx2 < adj.length; idx2++) {
            v = getIndexOf(adj[idx2]);
            queue.push(v);
            queue.push(index);
            //System.out.println("Pushed "+v+", "+index);
            done[v] = true;
         }
         while (!(queue.isEmpty()))
           {
            v = queue.pop();
            p = queue.pop();
            //System.out.println("Popped "+v+", "+p);
            D[index][v] = D[p][index] + 1;
            D[v][index] = D[index][v];
            //System.out.println("p="+p+" v="+v+" index="+index);

           for (int idx3 = 0; idx3 < vertexCount; idx3++) {

             // Create array of neighbours
             CellView[] adj2 = getVerticesFor(G, idx3);

             for (int idx2 = 0; idx2 < adj2.length; idx2++) {
                 u = getIndexOf(adj2[idx2]);
                 if (!(done[u]))
                 {
                  queue.push(u);
                  queue.push(v);
                  done[u] = true;
                 }
               }
             }
           }
        }/* for s*/

   /* Change the appropriate remaining 0's to infinities and set value of the
      boolean variable connected */
   connected = true;
   for (i=0;i<N;i++)
      for (j=i+1;j<N;j++)
         if (D[i][j]==0)
           {
            connected = false;
            D[i][j] = Long.MAX_VALUE;
            D[j][i] = Long.MAX_VALUE;
           }
     }//end findDistances function

     public int getIndexOf(CellView v) {
      for (int i = 0; i < vertices.length; i++)
        if (vertices[i] == v)
          return i;
      //System.out.println("Illegal Argument in getIndexOf: Vertex not found "+v);
      return 0;
     }

     public CellView[] getVerticesFor(JGraph G, int index) {
      Set set = DefaultGraphModel.getEdges(G.getModel(), new Object[]{vertices[index].getCell()});
       Object[] obj = set.toArray();
       if (obj != null) {
         CellView[] adj = new CellView[obj.length];
         for (int i = 0; i < obj.length; i++)
	  adj[i] = (CellView) graph.getView().getMapping(graph.getNeighbour(obj[i], vertices[index]), false);
         return adj;
       } else
        return new CellView[0];
    }

   private void makeOrderedList(JGraph G)
     {
      int highestIndex,i;
      CellView v;

      orderedList = new int[vertices.length];
      for (int index = 0; index < vertices.length; index++) {
        //s = vertices(index);
        orderedList[index] = index; //G.getIndexFromNode(v);
      }
     }//end makeOrderedList function

   private static int enume(int Index)
     {
      int i;

      for (i=0;  ;i++)
         if (orderedList[i] == Index)
            return i;
     }//end  enum function

   private void find_l_and_k(JGraph G)
     {
      if(D.length == 0)
	return;

      int i,j;
      long diam;
      double Lo;

      Ko = 0;
      int vertexCount = vertices.length;
      L = new double[vertexCount][vertexCount];
      K = new double[vertexCount][vertexCount];
      /* Find the diameter of the graph */
      diam = D[0][0];
      for (i=0;i<vertexCount;i++)
         for (j=i+1;j<vertexCount;j++)
           {
            if ((diam < D[i][j]) && (D[i][j] < Long.MAX_VALUE))
               diam = D[i][j];
            Ko+= D[i][j];
           }
      Ko = Ko/vertexCount;

      for (i=0;i<vertexCount;i++)
         for (j=i+1;j<vertexCount;j++)
           {
            /* Lo is desired edge length */
        /*    Lo = avgEdgeLength(G);  */         /* this Lo makes it grow */
        /*  Lo = Math.sqrt(findArea(G))/diam;*/  /* this Lo makes it shrink */
            Lo = Math.sqrt(rect.width*rect.height/4)/diam;
            L[i][j] = Lo*D[i][j];
            L[j][i] = L[i][j];
            if (D[i][j] < Long.MAX_VALUE) {
               /*  K[i][j] = Ko/(D[i][j]*D[i][j]);*/
              K[i][j] = Ko*Ko/(D[i][j]*D[i][j]);
              //System.out.println("K.i.j="+K[i][j]+" (if)");
            } else {
               K[i][j] = 0;
              //System.out.println("K="+K[i][j]+" (else) D="+D[i][j]+" i="+i+" j="+j);
            }
            K[j][i] = K[i][j];
           }
     }//end find_l_and_k function





   private double findDelta2(JGraph G,int v)
     {
      findPartials(G,v);
      double delta = (partial_x*partial_x+partial_y*partial_y);
      //System.out.println("v="+v+" delta="+delta);
      return delta;
     }





   private void findPartials(JGraph G,int v)
     {
      int u;
      int vi,ui;
      double dx,dy,dd;


      vi = v;
      partial_x=0;
      partial_y=0;
      partial_xx=0;
      partial_xy=0;
      partial_yx=0;
      partial_yy=0;
      for (u = 0; u<N; u++)
         if (v!=u)
           {
            ui = u;
            dx = vertices[v].getBounds().x - vertices[u].getBounds().x;
            dy = vertices[v].getBounds().y - vertices[u].getBounds().y;
            dd = Math.sqrt(dx*dx+dy*dy);
            partial_x+= K[vi][ui]*(dx-L[vi][ui]*dx/dd);
            partial_y+= K[vi][ui]*(dy-L[vi][ui]*dy/dd);
            partial_xx+= K[vi][ui]*(1-L[vi][ui]*dy*dy/(dd*dd*dd));
            partial_xy+= K[vi][ui]*(L[vi][ui]*dx*dy/(dd*dd*dd));
            partial_yx+= K[vi][ui]*(L[vi][ui]*dy*dx/(dd*dd*dd));
            partial_yy+= K[vi][ui]*(1-L[vi][ui]*dx*dx/(dd*dd*dd));
            //System.out.println("k.vi.ui="+K[vi][ui]);
           }
     }

   private static double TempFunction(double del, double times)
     {
      return (0.5 * del  +  0.5 * times);
     }





   private double findE(JGraph G)
     {
      int u,v;
      int vi,ui;
      double dx,dy,dd,dif;
      double total=0;

      for (v = 0; v <N; v++)
        {
         vi = v;
         for (u = v; u<N; u++)
           {
            ui = u;
            dx = vertices[v].getBounds().x - vertices[u].getBounds().x;
            dy = vertices[v].getBounds().y - vertices[u].getBounds().y;
            dd = Math.sqrt(dx*dx+dy*dy);
            dif = dd - L[vi][ui];
            total+= K[vi][ui]*dif*dif;
           }
        }
      return total/2;
     }






   private void MoveToNewPosition(JGraph G)
     {
      double A,B,C,D,E,F,dx,dy;
      double xpos,ypos,xfrac,yfrac;

      xpos = vertices[mv].getBounds().x;
      ypos = vertices[mv].getBounds().y;

      int mvi = mv;
      findPartials(G,mv);
            A = partial_xx;
            B = partial_xy;
            C = -(partial_x);
            D = partial_yx;
            E = partial_yy;
            F = -(partial_y);
      dy = (A*F-C*D)/(A*E-B*D);
      dx = (C*E-B*F)/(A*E-B*D);

      while ( (dy != 0) && (dx != 0) )
        {
         if (xpos + dx < xmin)
            xfrac = (xpos - xmin)/dx;
         else if (xpos + dx > xmax)
            xfrac = (xmax - xpos)/dx;
         else
            xfrac = 1;


         if (ypos + dy < ymin)
            yfrac = (ypos - ymin)/dy;
         else if (ypos + dy > ymax)
            yfrac = (ymax - ypos)/dy;
         else
            yfrac = 1;
//double x = xpos+dx;
//double y = ypos+dy;

         if (xfrac < yfrac)
           {
            xpos += dx * xfrac;
            dx *= (xfrac - 1);
            ypos += dy * xfrac;
            dy *= (1 - xfrac);
           }
         else if (yfrac < xfrac)
           {
            ypos += dy * yfrac;
            dy *= (yfrac - 1);
            xpos += dx * yfrac;
            dx *= (1 - yfrac);
           }
         else
           {
            xpos += dx;
            dx = 0;
            ypos += dy;
            dy = 0;
           }
        }//end while

      if (xpos < 0) xpos = 0;
      if (ypos < 0) ypos = 0;
      Point p = G.snap(new Point((int) xpos, (int) ypos));
      vertices[mv].getBounds().setLocation(p);
      vertices[mv].getBounds().setLocation(new Point((int) xpos,(int) ypos));
     }

   private void MoveToNewPosition1(JGraph G)
     {
      double A,B,C,D,E,F,dx,dy;
      double xpos,ypos,xfrac,yfrac;

      xpos = vertices[mv].getBounds().x;
      ypos = vertices[mv].getBounds().y;



      int mvi = mv;
      findPartials(G,mv);
            A = partial_xx;
            B = partial_xy;
            C = -(partial_x);
            D = partial_yx;
            E = partial_yy;
            F = -(partial_y);
      dy = (A*F-C*D)/(A*E-B*D);
      dx = (C*E-B*F)/(A*E-B*D);

         if (xpos + dx < xmin)
            xfrac = (xpos - xmin)/dx;
         else if (xpos + dx > xmax)
            xfrac = (xmax - xpos)/dx;
         else
            xfrac = 1;


         if (ypos + dy < ymin)
            yfrac = (ypos - ymin)/dy;
         else if (ypos + dy > ymax)
            yfrac = (ymax - ypos)/dy;
         else
            yfrac = 1;

         if (xfrac < yfrac)
           {
            yfrac = xfrac;
           }
         else if (yfrac < xfrac)
           {
            xfrac = yfrac;
           }
double x = xpos+dx;
double y = ypos+dy;

            xpos += dx*xfrac;
            dx = 0;
            ypos += dy*yfrac;
            dy = 0;

      if (xpos < 0) xpos = 0;
      if (ypos < 0) ypos = 0;
      Point p = G.snap(new Point((int) xpos, (int) ypos));
      vertices[mv].getBounds().setLocation(p);
     }



   private void MoveToNewPosition2(JGraph G)
     {
      double A,B,C,D,E,F,dx,dy;
      double xpos,ypos,xfrac,yfrac;

      xpos = vertices[mv].getBounds().x;
      ypos = vertices[mv].getBounds().y;




      int mvi = mv;
      findPartials(G,mv);
            A = partial_xx;
            B = partial_xy;
            C = -(partial_x);
            D = partial_yx;
            E = partial_yy;
            F = -(partial_y);
      dy = (A*F-C*D)/(A*E-B*D);
      dx = (C*E-B*F)/(A*E-B*D);

      if (xpos < 0) xpos = 0;
      if (ypos < 0) ypos = 0;

      Point p = G.snap(new Point((int) (xpos+dx), (int) (ypos+dy)));
      vertices[mv].getBounds().setLocation(p);
     }



   private void CheckPositions(JGraph G)
     {
      int v,u;
      int vi,ui;
      double rand;
      boolean OK = false;

      //while(!OK)
       // {
         OK = true;
         for (v = 0; v < N; v++)
            for (u = v; u<N; u++)
               if ((vertices[v].getBounds().x == vertices[u].getBounds().x)
                  && (vertices[v].getBounds().y == vertices[u].getBounds().y))
                 {
                  vi = v;
                  ui = u;
                  rand = (1.4*Math.random() - 1) * L[vi][ui];
                  Point tmp = new Point(vertices[u].getBounds().x + (int) rand , vertices[u].getBounds().y);
                  vertices[u].getBounds().setLocation(tmp);
                  rand = (1.4*Math.random() - 1) * L[vi][ui];
                  tmp = new Point(vertices[u].getBounds().x + (int) rand , vertices[u].getBounds().y);
                  vertices[u].getBounds().setLocation(tmp);
                  OK = false;
                 }
        //}
     }


  }//end class KK
