package hero.client.grapheditor;

/**
 * WFMenuBar.java
 *
 *
 * Created: Fri Aug 09 16:01:06 2002
 *
 * @version 1.0
 */

import java.awt.Color;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

public class WFMenuBar extends javax.swing.JMenuBar {
static java.util.ResourceBundle resource = java.util.ResourceBundle.getBundle("resources.Traduction")/*#BundleType=List*/;
	Frame frame;

	public WFMenuBar(final Frame frame, final WFManager manager) {
		this.frame = frame;
		boolean buttonState = manager.getPersistence().hasPermission();
		ClassLoader cl = frame.getClass().getClassLoader();

		JMenu menu;
		JMenuItem mi;

		menu = new JMenu(resource.getString("wfmenubar.file")); // File
		menu.setBackground(new Color(153, 153, 255));
		add(menu);

		// New
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/new.png")));
		mi.setText(resource.getString("wfmenubar.new"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newProject();
			}
		});
		menu.add(mi);

		// Clone
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/clone.gif")));
		mi.setText(resource.getString("wfmenubar.clone"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.cloneProject();
			}
		});
		menu.add(mi);

		// Open
		mi = new JMenuItem(new ImageIcon(cl.getResource("images/open.png")));
		mi.setText(resource.getString("wfmenubar.open"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.openProject();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		JMenu export = new JMenu(resource.getString("wfmenubar.export"));
		export.setBackground(new Color(153, 153, 255));
		export.setIcon(new ImageIcon(cl.getResource("images/export.gif")));
		// ImageIO Writers
		AbstractAction a;

		String[] s = javax.imageio.ImageIO.getWriterFormatNames();
		for (int i = 0; i < s.length; i++) {
			final String format = s[i];
			if (format.toUpperCase().equals(format)) {
				a = new AbstractAction(format) {
					public void actionPerformed(ActionEvent e) {
						String name =
							saveDialog(resource.getString("wfmenubar.saveas") + format.toUpperCase());
						if (manager.getGraph().getModel().getRootCount() > 0) {
							if (name != null) {
								try {
									BufferedImage im = manager.getGraphImage();
									if (im != null) {
										File f = new File(name);
										ImageIO.write(im, format, f);
										JOptionPane.showMessageDialog(
											null,
											resource.getString("wfmenubar.successexport"));
									} else
										error("");
								} catch (IOException ex) {
									error(ex.toString());
								}
							}
						} else
							error("");
					}
				};
				JMenuItem imageFormat = new JMenuItem(a);
				imageFormat.setBackground(new Color(153, 153, 255));
				export.add(imageFormat);
			}
		}

		menu.addSeparator();

		AbstractAction b = new AbstractAction("VRML") {
			public void actionPerformed(ActionEvent e) {
				String name = saveDialog(resource.getString("wfmenubar.saveasvrm"));
				if (manager.getGraph().getModel().getRootCount() > 0) {
					if (name != null) {
						try {
							String data = manager.getGraphVRML();
							if (data != null) {
								File f = new File(name + ".wrl");
								FileOutputStream fout = new FileOutputStream(f);
								fout.write(data.getBytes());
								fout.close();
								JOptionPane.showMessageDialog(
									null,
									resource.getString("wfmenubar.successexport"));
							} else
								error("");
						} catch (IOException ex) {
							error(ex.toString());
						}
					}
				} else
					error("");
			}
		};
		JMenuItem vrmlFormat = new JMenuItem(b);
		vrmlFormat.setBackground(new Color(153, 153, 255));
		export.add(vrmlFormat);

		menu.add(export);

		menu.addSeparator();


		mi = new JMenuItem(resource.getString("wfmenubar.exit")); // - Exit
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/exit.png")));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.exit();
			}
		});
		menu.add(mi);

		menu = new JMenu(resource.getString("wfmenubar.edit")); //Edit
		menu.setBackground(new Color(153, 153, 255));
		add(menu);

		mi = new JMenuItem(resource.getString("wfmenubar.copy")); // - Copy	
		mi.setBackground(new Color(153, 153, 255));
		//	mi.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Copy.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.copySelection();
			}
		});
		menu.add(mi);
		mi = new JMenuItem(resource.getString("wfmenubar.paste")); // - Paste
		mi.setBackground(new Color(153, 153, 255));
		//	mi.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Paste.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.pasteSelection();
			}
		});
		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.delete")); // - Delete
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		mi.setIcon(new ImageIcon(cl.getResource("images/Delete.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.deleteSelection();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		mi = new JMenuItem(resource.getString("wfmenubar.selectall"), 'A'); // - Select all
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.selectAll();
			}
		});
		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.deselectall"), 'D'); // - Deselect all
		mi.setBackground(new Color(153, 153, 255));

		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.clearSelection();
			}
		});
		menu.add(mi);

		menu = new JMenu(resource.getString("wfmenubar.format")); //Format
		menu.setBackground(new Color(153, 153, 255));
		add(menu);

		final JCheckBoxMenuItem menuItem =
			new JCheckBoxMenuItem(resource.getString("wfmenubar.autolay"));
		menu.add(menuItem);
		menu.setBackground(new Color(153, 153, 255));
		menuItem.setState(true);
		menuItem.setBackground(new Color(153, 153, 255));
		menuItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.automaticLayout();
			}
		});
		menu.add(menuItem);

		mi = new JMenuItem(resource.getString("wfmenubar.circlelay"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/Arrange.gif")));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.setLayout1();
			}
		});

		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.expandlay"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/Layout.gif")));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.setLayout2();
			}
		});

		menu.add(mi);

		menu = new JMenu(resource.getString("wfmenubar.user")); //Users
		menu.setEnabled(buttonState);
		menu.setBackground(new Color(153, 153, 255));
		add(menu);
		mi = (JMenuItem) menu.add(new JMenuItem(resource.getString("wfmenubar.newuser")));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/User.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newUser();
			}
		});
		menu.add(mi);

		mi = (JMenuItem) menu.add(new JMenuItem(resource.getString("wfmenubar.projuser")));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/User2.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.usersInProject();
			}
		});
		menu.add(mi);

		menu.addSeparator();

		mi = (JMenuItem) menu.add(new JMenuItem(resource.getString("wfmenubar.addrole")));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Role2.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.newRole();
			}
		});
		menu.add(mi);
		mi = (JMenuItem) menu.add(new JMenuItem(resource.getString("wfmenubar.addrole2")));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_J, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/Role.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.addUserToRole();
			}
		});
		menu.add(mi);

		mi = (JMenuItem) menu.add(new JMenuItem(resource.getString("wfmenubar.addmapper")));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/rolemapper.gif")));
		mi.setEnabled(buttonState);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.addMapperToRole();
			}
		});
		menu.add(mi);

		menu = new JMenu(resource.getString("wfmenubar.zoom")); //Zoom
		menu.setBackground(new Color(153, 153, 255));
		add(menu);

		mi = new JMenuItem(resource.getString("wfmenubar.fit"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.fitWindow();
			}
		});
		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.zoom100"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/Zoom100.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoom(100);
			}
		});
		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.zoomin"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/ZoomIn.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoomIn();
			}
		});
		menu.add(mi);

		mi = new JMenuItem(resource.getString("wfmenubar.zoomout"));
		mi.setBackground(new Color(153, 153, 255));
		mi.setIcon(new ImageIcon(cl.getResource("images/ZoomOut.gif")));
		mi.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.zoomOut();
			}
		});
		menu.add(mi);
		/*
			menu = new JMenu("Help");	    	//HELP
			add(menu);
		*/

		menu = new JMenu(resource.getString("wfmenubar.info")); //Info
		menu.setBackground(new Color(153, 153, 255));
		add(menu);

		mi = new JMenuItem(resource.getString("wfmenubar.about")); // - Copy	
		mi.setBackground(new Color(153, 153, 255));
		//	mi.setAccelerator(KeyStroke.getKeyStroke( KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		mi.setIcon(new ImageIcon(cl.getResource("images/toolbar-info.png")));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.Info();
			}
		});
		menu.add(mi);

	}

	/* Show a dialog with the given error message. */
	public void error(String message) {
		JOptionPane.showMessageDialog(this,message,resource.getString("wfmenubar.info"),JOptionPane.ERROR_MESSAGE);
	}
	/* Show a file save dialog and return the filename. */
	public String saveDialog(String message) {
		return dialog(message, FileDialog.SAVE);
	}

	/* Show a file load dialog and return the filename. */
	public String loadDialog(String message) {
		return dialog(message, FileDialog.LOAD);
	}

	/* Open a dialog and return the filename. Returns null if cancelled. */
	protected String dialog(String message, int mode) {
		FileDialog f = new FileDialog(frame, message, mode);
		f.show();
		if (f.getFile() != null)
			return f.getDirectory() + f.getFile();
		else
			return null;
	}

}
