package hero.client.grapheditor;

/*
*
* WFPersistence.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.Constants;

import java.util.Iterator;
import java.util.Collection;
import java.util.Date;

import javax.swing.JOptionPane;
import hero.util.BonitaClient;
import hero.util.ProjectNodes;
import hero.net.ProjectSession.StrutsNodeValue;
import javax.swing.ImageIcon;
import java.text.SimpleDateFormat;

public class WFPersistence {

    //	private String userAcces = "all";
	private String projectName;
	public boolean permission = false;
	private String endpoint = null;
	private String url = null;
	private String userPassword = null;
	public BonitaClient soapclient;
	private ProjectNodes pn = new ProjectNodes();
	public final static String imageBase = "images/";
	public final static ImageIcon icon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource(imageBase + "icon.png"));

	public WFPersistence(String managerProject, BonitaClient soapclient) throws Exception{
			this.soapclient = soapclient;
			soapclient.openProject(managerProject);
			projectName = managerProject;
			permission = isInRole(this.getUser(), Constants.ADMIN);
	}

	public boolean openProject(String projectName) throws Exception {
		soapclient.openProject(projectName);
		this.projectName = projectName;		
		this.setProjectNodes();
		return true;
	}

	public boolean createProject(String projectName) throws Exception {

		if (exists(projectName)) {
			return false;
		}

		soapclient.createProject(projectName);
		//		userAcces = "All";
		this.projectName = projectName;
		return true;

	}
	
	public boolean createModel(String modelName) throws Exception {

		if (exists(modelName)) {
			return false;
		}

		soapclient.createModel(modelName);
		//		userAcces = "All";
		this.projectName = modelName;
		return true;

	}

	public boolean cloneProject(String projectSrc, String projectDest)
		throws Exception {
		if (exists(projectDest)) {
			return false;
		}
		soapclient.cloneProject(projectSrc, projectDest);
		//		userAcces = "All";
		this.projectName = projectDest;
		return true;
	}
	
	public boolean cloneSubProject(String projectSrc, String projectDest)
			throws Exception {
			soapclient.cloneProject(projectSrc, projectDest);
			return true;
		}

	public String getUser() throws Exception {
		return (soapclient.getUser());
	}

	public boolean hasPermission() {
		return permission;
	}

	private boolean exists(String name) throws Exception {
		String prj;
		Collection projects = soapclient.getProjects();
		Iterator i = projects.iterator();
		while (i.hasNext()) {
			prj = (String) i.next();
			if (prj.equalsIgnoreCase(name)) {
				return true;
			}
		}
		return false;
	}

	public Object[] getProjects() throws Exception {
		Collection projects = soapclient.getProjects();
		return (projects.toArray());
	}

	public boolean containsNode(String value) throws Exception {
		Collection nodes = soapclient.getNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			String nodeName = (String) i.next();
			if (value.equalsIgnoreCase(nodeName)) {
				return true;
			}
		}
		return false;
	}
	
	public boolean containsProject(String value) throws Exception {
		Collection projects = soapclient.getProjects();
		Iterator i = projects.iterator();
		while (i.hasNext()) {
			String projectName = (String) i.next();
			if (value.equalsIgnoreCase(projectName)) {
				return true;
			}
		}
		return false;
	}

	public Object[] getNodes() throws Exception {
		Collection nodes = soapclient.getNodes();
		return (nodes.toArray());
	}

	public Object[] getEdges() throws Exception {
		Collection nodes = soapclient.getEdges();
		return (nodes.toArray());
	}

	public void addNode(String name, String type) throws Exception {
		soapclient.addNode(name, type);		
	}
	
	public void addNodeSubProcess(String name, String projectName) throws Exception {
		soapclient.addNodeSubProcess(name, projectName);
	}

	public boolean deleteNode(String node) throws Exception {
		soapclient.deleteNode(node);
		return true;
	}
	
	public boolean deleteProject(String project) throws Exception {
		soapclient.deleteProject(project);
		return true;
	}

	public void deleteEdges(Object[] edges) throws Exception {
		soapclient.deleteEdges(edges);		
	}

	public String addEdge(String src, String dest) throws Exception {
		return soapclient.addEdge(src, dest);
	}

	public String setEdgeCondition(String edgeName, String condition)
		throws Exception {
		soapclient.setEdgeCondition(edgeName, condition);
		return edgeName;
	}
	
	public String getEdgeCondition(String edgeName)
		throws Exception {
		return (soapclient.getEdgeCondition(edgeName));
	}

	public String getState(String name) throws Exception {
		int state = soapclient.getNodeState(name);
		switch (state) {
			case Constants.Nd.INITIAL :
				return ("Initial");
			case Constants.Nd.READY :
				return ("READY");
			case Constants.Nd.ANTICIPABLE :
				return ("Anticipable");
			case Constants.Nd.ANTICIPATING :
				return ("Anticipating");
			case Constants.Nd.EXECUTING :
				return ("Executing");
			case Constants.Nd.EXECUTED :
				return ("Executed");
			case Constants.Nd.TERMINATED :
				return ("Terminated");
			case Constants.Nd.ANT_SUSPENDED :
				return ("Ant_suspended");
			case Constants.Nd.EXEC_SUSPENDED :
				return ("Exec_suspended");
		}
		return ("");
	}

	public int getNodeState(String name) throws Exception {
		/*	Object[] nodes = this.pn.getNodes();
			for (int i=0;i<nodes.length;i++)
			{		
			    StrutsNodeValue node = (StrutsNodeValue) nodes[i];
				if ((node.getName()).equals(name))
					return(new Integer(node.getState()).intValue());
			}			
		return(0);*/
		return(soapclient.getNodeState(name));
	}
	
	public int getNodeType(String name) throws Exception {
				/*Object[] nodes = this.pn.getNodes();
				for (int i=0;i<nodes.length;i++)
				{		
					StrutsNodeValue node = (StrutsNodeValue) nodes[i];
					if ((node.getName()).equals(name))
						return(new Integer(node.getType()).intValue());
				}			
			return(0);*/
		return(soapclient.getNodeType(name));
		}

	public String getEdgeInNode(String name) throws Exception {
		return (soapclient.getEdgeInNode(name));
	}

	public String getEdgeOutNode(String name) throws Exception {
		return (soapclient.getEdgeOutNode(name));
	}

	public boolean isActive(String name)
		throws Exception { //Only used by WFmanager.updateStatus
		int state = soapclient.getNodeState(name);
		switch (state) {
			case Constants.Nd.INITIAL :
				return false;
			case Constants.Nd.READY :
				return false;
			case Constants.Nd.ANTICIPABLE :
				return false;
			case Constants.Nd.ANTICIPATING :
				return true;
			case Constants.Nd.EXECUTING :
				return true;
			case Constants.Nd.EXECUTED :
				return false;
			case Constants.Nd.TERMINATED :
				return true;
			case Constants.Nd.ANT_SUSPENDED :
				return false;
			case Constants.Nd.EXEC_SUSPENDED :
				return false;
		}
		return false;
	}

	public String getType(String name) throws Exception {
		return (soapclient.getType(name));
	}

	public String getRole(String name) throws Exception {
		return (soapclient.getRole(name));
	}

	public void setNodeRole(String node, String newRole) throws Exception {
		soapclient.setNodeRole(node, newRole);
		this.setProjectNodeRole(node,newRole);
	}

	public String getDeadline(String name) throws Exception {
		this.setProjectNodes();
		return (soapclient.getDeadline(name));
	}

	public void setDeadline(String name, long dl) throws Exception {
		soapclient.setDeadline(name, dl);		
	}

	public String getNodeInterHookValue(String nodeName, String hookName)
		throws Exception {
		return (soapclient.getNodeInterHookValue(nodeName, hookName));
	}
	
	public String getProjectInterHookValue(String hookName)
		throws Exception {
		return (soapclient.getProjectInterHookValue(hookName));
	}

	public String getNodeDescription(String nodeName) throws Exception {
		return (soapclient.getNodeDescription(nodeName));
	}

	public void setNodeDescription(String nodeName, String description)
		throws Exception {
		soapclient.setNodeDescription(nodeName, description);
		this.setProjectNodeDescription(nodeName,description);
	}

	public void setNodeTraditional(String nodeName)
		throws Exception {
		soapclient.setNodeTraditional(nodeName);
	}
	
	public void setNodeAnticipable(String nodeName)
		throws Exception {
		soapclient.setNodeAnticipable(nodeName);
	}

	public boolean getNodeAnticipable(String nodeName)
		throws Exception {
		return(soapclient.getNodeAnticipable(nodeName));
	}

	public void setNodeInterHookValue(String nodeName,String hookName,String value)
		throws Exception {
		soapclient.setNodeInterHookValue(nodeName, hookName, value);
	}
	
	public void setProjectInterHookValue(String hookName,String value)
		throws Exception {
		soapclient.setProjectInterHookValue(hookName, value);
	}

	public Object[] getRoles() throws Exception {
		Collection roles = soapclient.getRoles();
		return (roles.toArray());
	}

	public Object[] getUsersInProject() throws Exception {
		Collection users = soapclient.getUsersInProject();
		return (users.toArray());
	}

	public Object[] getAllUsers() throws Exception {
		Collection users = soapclient.getAllUsers();
		return (users.toArray());
	}

	public void addUser(String name) throws Exception {
		soapclient.addUser(name);
	}

	public void addRole(String name, String desc) throws Exception {
		soapclient.addRole(name, desc);
	}

	public void addNodeProperty(String nodeName,String key,String value,boolean propagate)
		throws Exception {
		soapclient.addNodeProperty(nodeName, key, value, propagate);
	}

	public void addProjectProperty(String key,String value)
		throws Exception {
		soapclient.addProjectProperty(key, value);
	}

	public void setUserRole(String name, String role) throws Exception {
		soapclient.setUserRole(name, role);
	}
	
	public void addRoleMapper(String role, String mapper, String type) throws Exception {
		soapclient.addRoleMapper(role,mapper,type);
	}
	
	public void addPerformerAssigment(String node, String performerName, String type, String property) throws Exception {
		soapclient.addNodePerformerAssigment(node,performerName,type,property);
	}
	
	public void setEditNode(String node, String role, String description, Date deadline) throws Exception {
		soapclient.setEditNode(node, role, description, deadline.getTime());
		this.setProjectNodeRole(node,role);
		this.setProjectNodeDescription(node,description);
		this.setProjectNodeDeadline(node,deadline.getTime());
	}

	public void addNodeInterHook(String nodeName,String hookName,String event,String value)
		throws Exception {
		soapclient.addNodeInterHook(nodeName, hookName, event, value);
	}
	
	public void addProjectInterHook(String hookName,String event,String value)
		throws Exception {
		soapclient.addProjectInterHook(hookName, event, value);
	}

	public void addIteration(String from,String to, String condition)
		throws Exception {
		soapclient.addIteration(from, to, condition);
	}

	public Collection getIterations()
		throws Exception {
		return(soapclient.getIterations());
	}


	public boolean isInRole(String name, String role) throws Exception {
		return (soapclient.isInRole(name, role));
	}

	public String getProjectName() {
		return this.projectName;
	}

	public String getStateString(int state) throws Exception {
		switch (state) {
			case Constants.Nd.INITIAL :
				return ("INITIAL");
			case Constants.Nd.READY :
				return ("READY");
			case Constants.Nd.ANTICIPABLE :
				return ("ANTICIPABLE");
			case Constants.Nd.ANTICIPATING :
				return ("ANTICIPATING");
			case Constants.Nd.EXECUTING :
				return ("EXECUTING");
			case Constants.Nd.EXECUTED :
				return ("EXECUTED");
			case Constants.Nd.TERMINATED :
				return ("TERMINATED");
			case Constants.Nd.ANT_SUSPENDED :
				return ("ANT_SUSPENDED");
			case Constants.Nd.EXEC_SUSPENDED :
				return ("EXEC_SUSPENDED");
		}
		return ("");
	}
	
	public int getStateInt(String state) throws Exception {
		if (state.equals("INITIAL"))
			return Constants.Nd.INITIAL;
		if (state.equals("READY"))
			return Constants.Nd.READY;
		if (state.equals("ANTICIPABLE"))
			return Constants.Nd.ANTICIPABLE;
		if (state.equals("ANTICIPATING"))
			return Constants.Nd.ANTICIPATING;
		if (state.equals("EXECUTING"))
			return Constants.Nd.EXECUTING;
		if (state.equals("TERMINATED"))
			return Constants.Nd.TERMINATED;
		return (-1);
	}	
	

	public String getTypeString(int type) throws Exception {
		switch (type) {
			case Constants.Nd.AND_JOIN_NODE :
				return ("And Join");
			case Constants.Nd.OR_JOIN_NODE :
				return ("Or Join");
			case Constants.Nd.AND_JOIN_AUTOMATIC_NODE :
				return ("And Join Automatic");
			case Constants.Nd.OR_JOIN_AUTOMATIC_NODE :
				return ("Or Join Automatic");
		}
		return ("");
	}

	public void setProjectNodes() throws Exception {
		this.pn.setNodes(soapclient.getProjectNodes());		
	}
	
	public Object[] getProjectNodes() throws Exception {
			return(soapclient.getProjectNodes());		
		}
		
	public Object[] getProjectNodesArray() throws Exception {
		return(this.pn.getNodes());		
	}

	public Collection getProperties() throws Exception {
		return(soapclient.getProperties());		
	}

	public void windowMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg);
	}
	
	// Local Model operations
	
	public StrutsNodeValue getProjectNode(String nodeName) throws Exception {
		/*Collection nodes = this.pn.getNodes();		
		Iterator i = nodes.iterator();
		while (i.hasNext()) {*/
		Object[] nodes = this.pn.getNodes();
		for (int i=0;i<nodes.length;i++)
		{		
		StrutsNodeValue node = (StrutsNodeValue)nodes[i];
			if ((node.getName()).equals(nodeName))
				return node;
		}
		return null;
		//return(soapclient.getStrutsNode(nodeName));	
	}
		
	public void setProjectNodeState(String nodeName, int state) throws Exception {		
			Object[] nodes = this.pn.getNodes();
			for (int i=0;i<nodes.length;i++)
			{		
			StrutsNodeValue node = (StrutsNodeValue) nodes[i];
				if ((node.getName()).equals(nodeName))
					node.setState(this.getStateString(state));
			}			
		}
		
	public void setProjectNodeRole(String nodeName, String role) throws Exception {		
			Object[] nodes = this.pn.getNodes();
			for (int i=0;i<nodes.length;i++)
			{		
			  StrutsNodeValue node = (StrutsNodeValue) nodes[i];
			  if ((node.getName()).equals(nodeName))
					node.setRole(role);
			  }			
			}		

	public void setProjectNodeDescription(String nodeName, String description) throws Exception {		
			Object[] nodes = this.pn.getNodes();
			for (int i=0;i<nodes.length;i++)
			{		
			    StrutsNodeValue node = (StrutsNodeValue) nodes[i];
				if ((node.getName()).equals(nodeName))
					node.setDescription(description);
			}			
		}
		
	public void setProjectNodeDeadline(String nodeName, long deadline) throws Exception {		
				Object[] nodes = this.pn.getNodes();
				for (int i=0;i<nodes.length;i++)
				{		
				StrutsNodeValue node = (StrutsNodeValue) nodes[i];
					if ((node.getName()).equals(nodeName)){
					  SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
					  node.setDeadline(df.format(df.parse(new java.sql.Date(deadline).toString())));
					}
				}			
			}
		
} // WFPersistence

