/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: Sample1CreateProcessModel.java,v 1.3 2004/11/10 17:10:50 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.client.samples;

import javax.security.auth.login.LoginContext;
import hero.client.test.SimpleCallbackHandler;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import hero.interfaces.Constants;


public class Sample1CreateProcessModel {
	
	static public void main(String[] args) throws Exception{
	  // User Admin login
	  char[] password={'t','o','t','o'};
	  SimpleCallbackHandler handler = new SimpleCallbackHandler("admin",password);
	  LoginContext lc = new LoginContext("TestClient", handler);
	  lc.login();	 

	  // Creation of Bonita Project "Order Processing " 
	  
	  // Collaboration Project Session Bean Creation using Remote Interface
	  ProjectSessionHome prjHome= (ProjectSessionHome) ProjectSessionUtil.getHome();
	  ProjectSession prjSession = prjHome.create();
	  
	  // Order Service project creation

	  prjSession.initModel("Order Processing");
	  
	  //Process Model creation by user admin	  
	  // Activities creation : non anticipative : traditional execution mode)
	  
	  // First activity is created : Receive Order
	  prjSession.addNode("Receive Order",Constants.Nd.AND_JOIN_NODE);
	  prjSession.setNodeTraditional("Receive Order");
	  
	  // Set node properties : customer_name, product_name, items
	  prjSession.setNodeProperty("Receive Order","once_more","OK",true);
	  prjSession.setNodeProperty("Receive Order","customer_name","",true);
	  prjSession.setNodeProperty("Receive Order","product_name","",true);
	  prjSession.setNodeProperty("Receive Order","items","",true);
	  
	  // Second Activity creation : Check Stock AutomaticAndJoin activity
	  prjSession.addNode("Check Stock",Constants.Nd.AND_JOIN_AUTOMATIC_NODE );
	  prjSession.setNodeTraditional("Check Stock");
	  
	  
	  // Activities connection : Set edges between nodes
	  prjSession.addEdge("Receive Order","Check Stock");
	 
	  // Set node hook
	  // This hook (Check Stock Hook)verifies if the user has sent a correct order and set a new property "stock_status" with the value ok or nok
	  String scriptHook =
			  "import hero.interfaces.*;\n"
				  + "import hero.interfaces.BnNodeLocal;\n"
				  + "beforeTerminate (Object b,Object n) {\n"
				  + "hero.interfaces.ProjectSessionLocalHome pHome = (hero.interfaces.ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome(); \n"
				  + "hero.interfaces.ProjectSessionLocal subProcess = pHome.create(); \n"
	  			  + "subProcess.initModel(n.getBnProject().getName()); \n"
	  			  + "if (customer_name != null && product_name != null && items != null) \n"
	  			  + "{ System.out.println(\"(Order Processing : Check Stock) Stock Status = OK\"); \n"
	  			  + " subProcess.setNodeProperty(n.getName(), \"stock_status\",\"ok\",true);\n "
	  			  + "	}\n"
	  			  + " else \n"
	  			  + "{ System.out.println(\"(Order Processing : Check Stock) Stock Status = NOK\");\n "
	  			  + "subProcess.setNodeProperty(n.getName(), \"stock_status\",\"nok\",true); \n"
	  			  + " }\n"
				  + "}";
			
	  prjSession.addNodeInterHook("Check Stock","Check Stock Hook",Constants.Nd.BEFORETERMINATE,Constants.Hook.BSINTERACTIVE,scriptHook);

	// "Accept Order" subProcess activity creation, attached to "Customer Service" project
	  prjSession.addNodeSubProcess("Accept Order","Customer Service");

	// Adds Edge between "Check Stock" and "Accept Order"
	// and adds a condition to this new edge
	String edgeStockOrder = prjSession.addEdge("Check Stock","Accept Order");
	String condition = "stock_status.equals(\"ok\")";
	prjSession.setEdgeCondition(edgeStockOrder,condition);
	
	// Add two activities connected with "Accept Order", once selected depending on the edge conditions
	// First
	prjSession.addNode("Ship & Report",Constants.Nd.AND_JOIN_AUTOMATIC_NODE );
	prjSession.setNodeTraditional("Ship & Report");
	
	// Set node hook
	// This hook (Ship & Report Hook)just print a message
	  String scriptHookSR =
			  "import hero.interfaces.*;\n"
				  + "import hero.interfaces.BnNodeLocal;\n"
				  + "beforeTerminate (Object b,Object n) {\n"
				  + "hero.interfaces.ProjectSessionLocalHome pHome = (hero.interfaces.ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome(); \n"
				  + "hero.interfaces.ProjectSessionLocal subProcess = pHome.create(); \n"
	  			  + "subProcess.initModel(n.getBnProject().getName()); \n"
	  			  + "System.out.println(\"(Order Processing : Ship & Report) That's OK\"); \n"
	  			  + "}" ;
				  			  
	prjSession.addNodeInterHook("Ship & Report","Ship Report Hook",Constants.Nd.BEFORETERMINATE,Constants.Hook.BSINTERACTIVE,scriptHookSR);

	
	String edgeOrderReport = prjSession.addEdge("Accept Order","Ship & Report");
	String condition1 = "partial_sales_status.equals(\"ok\")";
	prjSession.setEdgeCondition(edgeOrderReport,condition1);
		
	// second
	prjSession.addNode("Cancel Order",Constants.Nd.AND_JOIN_AUTOMATIC_NODE );
	prjSession.setNodeTraditional("Cancel Order");
	
	// Set node hook
	// This hook (Cancel Order Hook)just print a message
	  String scriptHookCO =
			  "import hero.interfaces.*;\n"
				  + "import hero.interfaces.BnNodeLocal;\n"
				  + "beforeTerminate (Object b,Object n) {\n"
				  + "hero.interfaces.ProjectSessionLocalHome pHome = (hero.interfaces.ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome(); \n"
				  + "hero.interfaces.ProjectSessionLocal subProcess = pHome.create(); \n"
	  			  + "subProcess.initModel(n.getBnProject().getName()); \n"
	  			  + "System.out.println(\"(Order Processing : Cancel Order) That's NOT OK\"); \n" 
	  			  + "}" ;
			
	prjSession.addNodeInterHook("Cancel Order","Cancel Order Hook",Constants.Nd.BEFORETERMINATE,Constants.Hook.BSINTERACTIVE,scriptHookCO);

	
	
	
	
	String edgeOrderCancel = prjSession.addEdge("Accept Order","Cancel Order");
	String condition2 = "partial_sales_status.equals(\"nok\")";
	prjSession.setEdgeCondition(edgeOrderCancel,condition2);
	
	  // Creates two roles for this process : agent and customer 
	  prjSession.addRole("agent","Customer Service agent");
	  prjSession.addRole("customer","Sales order customer");
	  
	  // set roles for activities
	  prjSession.setNodeRole("Accept Order","agent");
	  prjSession.setNodeRole("Check Stock","agent");
	  prjSession.setNodeRole("Ship & Report","agent");
	  prjSession.setNodeRole("Cancel Order","agent");
	  prjSession.setNodeRole("Receive Order","customer");  


	  // Adding iteration from Receive Order and Ship & report
	  // note : this is just as a sample : it is not a good design 
	  // when an iteration includes nodes
	  // with many output nodes, as Accept Order is
	  String iterCondition="once_more.equals(\"OK\")" ;
	  prjSession.addIteration("Ship & Report","Receive Order",iterCondition);
	}
	  
	
}


