/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: Sample1CreateSubProcessModel.java,v 1.3 2004/11/10 17:10:50 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.client.samples;
import javax.security.auth.login.LoginContext;
import hero.client.test.SimpleCallbackHandler;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import hero.interfaces.Constants;

public class Sample1CreateSubProcessModel {
	
	static public void main(String[] args) throws Exception{
	  // User Admin login
	  char[] password={'t','o','t','o'};
	  SimpleCallbackHandler handler = new SimpleCallbackHandler("admin",password);
	  LoginContext lc = new LoginContext("TestClient", handler);
	  lc.login();
	  	  
	  // Creation of Bonita Project "Customer Service " 
	  // It will be used as Sub Process in our complete Order Processing Project
	  
	  // Collaboration Project Session Bean Creation using Remote Interface
	  ProjectSessionHome prjHome= (ProjectSessionHome) ProjectSessionUtil.getHome();
	  ProjectSession prjSession = prjHome.create();
	  
	  // Customer Service project creation
	  prjSession.initModel("Customer Service");
	  
	  //Process Model creation by user admin	  
	  // Activities creation : non anticipative : traditional execution mode)
	  // Two activities are created : "Ask Customer" and "Notify Sales"
 	  prjSession.addNode("Ask Customer",Constants.Nd.AND_JOIN_NODE);
	  prjSession.setNodeTraditional("Ask Customer");
	  
	  prjSession.addNode("Notify Sales",Constants.Nd.AND_JOIN_NODE);
	  prjSession.setNodeTraditional("Notify Sales");
	  
	  
	  // Activities connection : Set edges between nodes
	  prjSession.addEdge("Ask Customer","Notify Sales");
	 
	 
	  // Set node hook
	  // First hook is in charge of printing information
	  String scriptHook1 =
			  "import hero.interfaces.*;\n"
				  + "import hero.interfaces.BnNodeLocal;\n"
				  + "afterStart (Object b,Object n) {\n\n\n"
				  + "System.out.println(\"customer name: \"+ customer_name);"
				  + "System.out.println(\"product name: \"+ product_name);"
				  + "System.out.println(\"items : \"+ items );"
				  + "}";
	  prjSession.addNodeInterHook("Ask Customer","order data",Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,scriptHook1);
	  // second hook is in charge of creating a new property containing the user order confirmation choice : 
	  // partial_sales_status
	  String scriptHook2 = 
	  		" import hero.interfaces.* ;"
	  			+ "import hero.interfaces.BnNodeLocal;\n"
	  			+ "afterTerminate (Object b,Object n) {\n\n\n"
	  			 	+ "System.out.println(\"partial_sales_status = OK\"); \n"
	  				+ "hero.interfaces.ProjectSessionLocalHome pHome = (hero.interfaces.ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome(); \n"
	  				+ "hero.interfaces.ProjectSessionLocal subProcess = pHome.create(); \n"
	  				+ "subProcess.initModel(n.getBnProject().getName()); \n"
	  			 	+ "subProcess.setProperty(\"partial_sales_status\",\"ok\"); \n"
	  			+"}" ;
	  prjSession.addNodeInterHook("Notify Sales","Set property",Constants.Nd.AFTERTERMINATE,Constants.Hook.BSINTERACTIVE,scriptHook2);	  			
	  
	  
	  // Add roles to this process
	  prjSession.addRole("agent","Customer Service agent");
	  prjSession.addRole("customer","Sales order customer");

	  // specify role for each node
	  prjSession.setNodeRole("Ask Customer","customer");
	  prjSession.setNodeRole("Notify Sales","agent");  


	  // Use Mappers for the role resolution
	  prjSession.addRoleMapper("agent","hero.mapper.Sample1CustomAgentGroupMembers",Constants.Mapper.CUSTOM);
	  prjSession.addRoleMapper("customer","hero.mapper.Sample1CustomCustomerGroupMembers",Constants.Mapper.CUSTOM);
	}
	  
	
}


