package hero.client.samples;

/*
*
* Inscription.java - 
* Copyright (C) 2004 Ecoo Team
* Anne.Geron@bull.net
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


import javax.security.auth.login.LoginContext;
import hero.client.test.SimpleCallbackHandler;

import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserRegistration;
import hero.interfaces.UserRegistrationHome;
import hero.interfaces.UserRegistrationUtil;
import hero.interfaces.BnRoleLocal;


import hero.interfaces.Constants;

import java.util.*;

public class SampleProjectApi {
	
	static public void main(String[] args) throws Exception{
	
		  // User registration interface creation
		  UserRegistrationHome urHome = UserRegistrationUtil.getHome();
		  UserRegistration userReg=urHome.create();
		  
		  try{
			  // User "jack" (customer) creation in Bonita database
		     userReg.userCreate("jack","jack","miguel.valdes-faura@ext.bull.net");
		     // User "john" (service customer) creation in Bonita database		 
		     userReg.userCreate("john","john","miguel.valdes-faura@ext.bull.net");
		  }catch(Exception e){} // Maybe user exists 
		  
		  userReg.remove();
		  
          // User Admin login
          char[] password={'t','o','t','o'};
          SimpleCallbackHandler handler = new SimpleCallbackHandler("admin",password);
          LoginContext lc = new LoginContext("TestClient", handler);
          lc.login();          

         // Project Session Bean Creation using Remote Interface
          ProjectSessionHome prjHome= (ProjectSessionHome) ProjectSessionUtil.getHome();
          ProjectSession prjSession = prjHome.create();
                            
          //***************************************************************/
          //************** API Documentation - Sample 1 *******************/
          //************** Initiating ProjectSessionBean  *****************/
          //***************************************************************/

          //Process creation by user admin
	  
	  System.out.println("Original Process creation ");
          prjSession.initModel("Original Process");
                // if "Original Process" does not exists, it is created.
                // Process definition : see following sections
                // adding activities, edges, ...
                //

	  System.out.println("Original Process cloning into Clone Process ");
          try {
               prjSession.initProject("Original Process", "Clone Process");
          } catch(Exception e) {System.out.println(e);} //Maybe project does not exists

	  System.out.println("Original Process : current Process");
          prjSession.initModel("Original Process");

          /***************************************************************/
          /************** API Documentation - Sample 2 *******************/
          //************** Project properties            *****************/
          /***************************************************************/

        String processName = prjSession.getName() ;
        System.out.println("Current Process : " + processName) ;

        try {
               String parentName = prjSession.getParent();
                System.out.println("Parent Process : " + parentName) ;
          } catch(Exception e) {System.out.println(e);} //Maybe there is no parent


        try {
               String creatorName = prjSession.getCreator();
                System.out.println("Process Creator : " + creatorName) ;
          } catch(Exception e) {System.out.println(e);} //Maybe there is a problem

        try {
                prjSession.setProperty("userId","user1");
                prjSession.setProperty("recordId","1111");
                prjSession.setProperty("orderId","0001");
          } catch(Exception e) {System.out.println(e);} //Maybe there is a problem


          // First way to get properties values
	  System.out.println("First way to access proprerty values : ");

          Collection properties = prjSession.getProperties() ;
          Iterator i = properties.iterator();
          while (i.hasNext())
          {
                hero.interfaces.BnProjectPropertyValue property = (hero.interfaces.BnProjectPropertyValue)i.next();       
                try {
                        String propertyKeyName = property.getTheKey();
                        String propertyValue = (String)property.getTheValue();
                        System.out.println("Property (Key, Value) : " + propertyKeyName + "/" + propertyValue);
                } catch(Exception e) {System.out.println(e);} //Maybe there is a problem
          }


          // Second way to get properties values
	  System.out.println("Second  way to access proprerty values : ");
          properties = prjSession.getPropertiesKey() ;
          i = properties.iterator();
          while (i.hasNext())
          {
                String propertyKey = (String)i.next();
                try {
                        hero.interfaces.BnProjectPropertyValue propertyValue = prjSession.getProperty(propertyKey);
                        System.out.println("Property (Key, Value) : " + i + "/" + propertyValue);
                } catch(Exception e) {System.out.println(e);} //Maybe there is a problem
          }


        //Deleting Property
        try {
           prjSession.deleteProperty("orderId");
        } catch(Exception e) {System.out.println(e);} //Maybe there is a problem

        //Verification

	  System.out.println("Properties after one deletion : ");
          Collection propertiesLeft = prjSession.getPropertiesKey() ;
          Iterator j = properties.iterator();
          while (j.hasNext())
          {
                String propertyLeftKey = (String)j.next();
                try {
                        hero.interfaces.BnProjectPropertyValue propertyValue = prjSession.getProperty(propertyLeftKey);
                        System.out.println("Property (Key, Value) : " + i + "/" + propertyValue);

                } catch(Exception e) {System.out.println(e);} //Maybe there is a problem
          }



          /***************************************************************/
          /************** API Documentation - Sample 3 *******************/
          //************** Activities in Project       *****************/
          /***************************************************************/

	  System.out.println("Activities creation ... ");
          try {
          	prjSession.addNode("Activity 1",Constants.Nd.AND_JOIN_NODE);
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong
          try {
          	prjSession.addNode("Activity 2",Constants.Nd.AND_JOIN_NODE);
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong
          try {
          	prjSession.addNode("Activity 3",Constants.Nd.AND_JOIN_NODE);
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("Activity 3 definition ... ");
          try {
          	Date dateLim = new Date(2005,05,02) ;
	  	prjSession.setNodeDeadline("Activity 3",dateLim.getTime()) ;
	  	prjSession.setNodeDescription("Activity 3","Activity 3 Description") ;
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("Setting Activities types");
          try {
          	prjSession.setNodeTraditional("Activity 1");
          	prjSession.setNodeAutomatic("Activity 2");
          	prjSession.setNodeTraditional("Activity 3");
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("Setting node properties which will not be propagated to other nodes");
          try {
	  	prjSession.setNodeProperty("Activity 1","color","blue",false);
	  	System.out.println("Setting node properties which will be propagated to other nodes");
	  	prjSession.setNodeProperty("Activity 1","price","expensive",true);
	  	prjSession.setNodeProperty("Activity 1","shape","square");
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong
	
	  System.out.println("Adding edges between activities");
          try {
	  	prjSession.addEdge("Activity 1","Activity 2");
	  	prjSession.addEdge("Activity 2","Activity 3");
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("Getting names of all the nodes in the project");
          Collection nodesNames = prjSession.getNodesNames() ;
          j = nodesNames.iterator();
          while (j.hasNext())
          {
                String nodeName = (String)j.next();
                System.out.println("Node :  " + nodeName + " (anticipable : " + prjSession.getNodeAnticipable(nodeName) + " )");
                Collection nodeProperties = prjSession.getNodeProperties(nodeName) ;
                Iterator k = nodeProperties.iterator() ;
                while (k.hasNext())
                {                	
                		hero.interfaces.BnNodePropertyValue nodeProperty = (hero.interfaces.BnNodePropertyValue)k.next();
               	     	try {
               	     		String nodePropertyKeyName = nodeProperty.getTheKey();
               	     		String nodePropertyValue = nodeProperty.getTheValue();
               	     		System.out.println(" --> Property (Key, Value) : " + nodePropertyKeyName + "/" + nodePropertyValue);
               	     	} catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong                               
                }
          }

	  System.out.println("Node deletion");
  	  try {
		prjSession.deleteNode("Activity 3") ;
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("Node deletion verification");
	  try {
            nodesNames = prjSession.getNodesNames() ;
            j = nodesNames.iterator();
            while (j.hasNext())
            {
                String nodeName = (String)j.next();
                System.out.println("Node :  " + nodeName ); }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong


	  System.out.println("Adding a role in the project (before instantiation) ");
	  try {
		  prjSession.addRole("Customer","customer") ;
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("  Getting role names of the project ");
	  try {
            Collection rolesNames = prjSession.getRolesNames() ;
            j = rolesNames.iterator();
            while (j.hasNext())
            {
                String roleName = (String)j.next();            
                System.out.println("Role :  " + roleName ); }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

          /***************************************************************/
          /*              "Original Process" instantiation               */
          /***************************************************************/
	  System.out.println("Creating project new intance");
          try {
               prjSession.instantiateProject("Original Process");
          } catch(Exception e) {System.out.println(" --> " + e);} //Maybe project does not exists
        // The new instance becomes the current project

        processName = prjSession.getName() ;
        System.out.println("Current Process : " + processName) ;

	  System.out.println("  Getting role names of the instance ");
	  try {
            Collection rolesNames = prjSession.getRolesNames() ;
            j = rolesNames.iterator();
            while (j.hasNext())
            {
                String roleName = (String)j.next();
                System.out.println("Role :  " + roleName ); }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong
	  
          /***************************************************************/
          /************** API Documentation - Sample 4 *******************/
          //************** Users in Project       *****************/
          /***************************************************************/

	  System.out.println("  Getting users names of the project ");
	  try {
            Collection usersNames = prjSession.getUsers() ;
            j = usersNames.iterator();
            while (j.hasNext())
            {
                String userName = (String)j.next();
                System.out.println("User :  " + userName ); }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("  Adding John in the project ");
	  try {
		prjSession.addUser("john") ;
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong
	  
        processName = prjSession.getName() ;
        System.out.println("  --> Current Process : " + processName + " contains john :" + prjSession.containsUser("john") ) ;

          /***************************************************************/
          /************** API Documentation - Sample 5 *******************/
          //************** Roles in Project       *****************/
          /***************************************************************/

	  System.out.println("Adding a Custumer role for john in the current project ");
	  try {
		  prjSession.setUserRole("john","Customer") ;
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("  Getting role names of the project ");
	  try {
            Collection rolesNames = prjSession.getRolesNames() ;
            j = rolesNames.iterator();
            while (j.hasNext())
            {
                String roleName = (String)j.next();
                System.out.println("Role :  " + roleName ); }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("  Getting role names for john user in this project ");
	  try {
            Collection johnRolesNames = prjSession.getRolesNames() ;
            j = johnRolesNames.iterator();
            while (j.hasNext())
            {
                String johnRoleName = (String)j.next();
                System.out.println("John role :  " + johnRoleName ); 
	    }
	  } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	  System.out.println("  Setting role names for an activites of this project ");
	  try {
             Collection prjNodesNames = prjSession.getNodesNames() ;
	     j = prjNodesNames.iterator() ;
	     while (j.hasNext())
	       {
                String prjNodeName = (String)j.next();
		System.out.println("Node --> " + prjNodeName ) ;
		System.out.println("  --> Getting the actuel role names for Activities "); 
		try {
        		System.out.println("      Activity role :  " + prjSession.getNodeRoleName(prjNodeName)); 
		} catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

               	System.out.println("  --> Setting activities new roles " ); 
		try {
			prjSession.setNodeRole(prjNodeName,"admin") ;
			// prjSession.setNodeRole("Activity 2","Customer") ;
		} catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

		System.out.println("  --> Getting the new role names for Activities "); 
		try {
        		System.out.println("      Activity role :  " + prjSession.getNodeRoleName(prjNodeName)); 
		} catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong

	      }
	   } catch(Exception e) {System.out.println(" --> " + e);} //Maybe something is wrong



	} //End Main

} // End SampleProjectApi Class

