/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
--------------------------------------------------------------------------
 * Class AbortSchema.java created by Jordi Anguela - June 2006
--------------------------------------------------------------------------
 */

package hero.client.samples.iterations.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class AbortSchema {
	static public void main(String[] args) throws Exception {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
			ProjectSession ps = projectSessionh.create();
			System.out.print("\n   Creating AbortSchema model");			
			ps.initModel("AbortSchema");
			
			// Set Properties
			ps.setProperty("abort", "false");
			
			// Set Nodes
			ps.addNode("Start", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("A1", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("A2", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("B1", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("B2", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("Abort", Constants.Nd.OR_JOIN_NODE);
	
			ps.setNodeTraditional("Start");
			ps.setNodeTraditional("A1");
			ps.setNodeTraditional("A2");
			ps.setNodeTraditional("B1");
			ps.setNodeTraditional("B2");
			ps.setNodeTraditional("Abort");
	
			// Set Edges
			ps.addEdge("Start", "A1");
			ps.addEdge("Start", "B1");
			String fromA1toA2 = ps.addEdge("A1", "A2");
			String fromB1toB2 = ps.addEdge("B1", "B2");
			String fromA1toAbort = ps.addEdge("A1", "Abort");
			String fromA2toAbort = ps.addEdge("A2", "Abort");
			String fromB1toAbort = ps.addEdge("B1", "Abort");
			String fromB2toAbort = ps.addEdge("B2", "Abort");
			
			// Set transition conditions
			ps.setEdgeCondition(fromA1toA2, "abort.equals(\"false\")");
			ps.setEdgeCondition(fromB1toB2, "abort.equals(\"false\")");
			ps.setEdgeCondition(fromA1toAbort, "abort.equals(\"true\")");
			ps.setEdgeCondition(fromA2toAbort, "abort.equals(\"true\")");
			ps.setEdgeCondition(fromB1toAbort, "abort.equals(\"true\")");
			ps.setEdgeCondition(fromB2toAbort, "abort.equals(\"true\")");
	
			// Iteration
			ps.addIteration("Abort", "Start", "abort.equals(\"true\")");
			
			// Set activity roles
			ps.addRole("Administrator", "The system administator role");
			ps.setNodeRole("Start", "Administrator");
			ps.setNodeRole("A1", "Administrator");
			ps.setNodeRole("A2", "Administrator");
			ps.setNodeRole("B1", "Administrator");
			ps.setNodeRole("B2", "Administrator");
			ps.setNodeRole("Abort", "Administrator");
			
			// Set role mapper
			ps.addRoleMapper("Administrator", "AdministratorMapper", Constants.Mapper.PROPERTIES);
	
			// Checks if the model is correctly defined
			ps.checkModelDefinition();

			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		}
	}
}
