/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
--------------------------------------------------------------------------
 * Class ArbitraryCycles.java created by Jordi Anguela - June 2006
--------------------------------------------------------------------------
 */
package hero.client.samples.iterations.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class ArbitraryCycles {
	static public void main(String[] args) throws Exception {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
			
			ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
			ProjectSession ps = projectSessionh.create();
			System.out.print("\n   Creating ArbitraryCycles model");
			ps.initModel("ArbitraryCycles");
			
			// Set Properties
			ps.setProperty("condition1", "50");	// % to do 1st iteration			
			ps.setProperty("condition2", "50");	// % to do 2nd iteration
			ps.setProperty("randomNum", "0");	// random number who decides if we iterate or not
	
			// Set Nodes
			ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
			ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
			ps.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
			ps.addNode("E", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
			ps.addNode("F", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	
			ps.setNodeTraditional("A");
			ps.setNodeTraditional("B");
			ps.setNodeTraditional("C");
			ps.setNodeTraditional("D");
			ps.setNodeTraditional("E");
			ps.setNodeTraditional("F");
		
			// Set Edges
			ps.addEdge("A", "B");
			ps.addEdge("B", "C");
			ps.addEdge("C", "D");
			String fromDtoE = ps.addEdge("D", "E");
			String fromEtoF = ps.addEdge("E", "F");
		
			// Set transition conditions
			ps.setEdgeCondition(fromDtoE, "(new Integer(randomNum).intValue() >= new Integer(condition1).intValue())");
			ps.setEdgeCondition(fromEtoF, "(new Integer(randomNum).intValue() >= new Integer(condition2).intValue())");
	
			// Set Hook
			String generateRandomScript =
				  "import hero.interfaces.*;\n"
				+ "import java.util.Random; \n"
				+ "beforeTerminate(Object b,Object n) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			    + "  String name = n.getName(); \n"
			    + "  Random generator = new Random(System.currentTimeMillis()); \n"
			    + "  int newRnd = generator.nextInt(100); \n"
			    + "  prjSession.setProperty(\"randomNum\", Integer.toString(newRnd)); \n"
			    + "  System.out.print(\" --> BeforeTerminate, node '\" + name + \"' new random value : \" + newRnd); \n"
				+ "  BnProjectPropertyValue conditionProp; \n"
			    + "  if (name.equals(\"D\")) { \n"
			    + "    conditionProp = prjSession.getProperty(\"condition1\"); \n"
			    + "    if (newRnd < Integer.parseInt(conditionProp.getTheValue())) { \n"
			    + "      System.out.println(\" --> ITERATE --> next activity 'B'\"); \n"
			    + "    } else { \n"
			    + "      System.out.println(\" --> DON'T ITERATE --> next activity 'E'\"); \n"
			    + "    } \n"
			    + "  } else { \n"
			    + "    conditionProp = prjSession.getProperty(\"condition2\"); \n"
			    + "    if (newRnd < Integer.parseInt(conditionProp.getTheValue())) { \n"
			    + "      System.out.println(\" --> ITERATE --> next activity 'C'\"); \n"
			    + "    } else { \n"
			    + "      System.out.println(\" --> DON'T ITERATE --> next activity 'F'\"); \n"
			    + "    } \n"
			    + "  } \n"
				+ "} \n";
			ps.addNodeInterHook("D", "Generates random number and saves it in 'randomNum' var", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, generateRandomScript);			
			ps.addNodeInterHook("E", "Generates random number and saves it in 'randomNum' var", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, generateRandomScript);			
	
			// Set Iterations
			ps.addIteration("D", "B", "(new Integer(randomNum).intValue() < new Integer(condition1).intValue())");
			ps.addIteration("E", "C", "(new Integer(randomNum).intValue() < new Integer(condition2).intValue())");
	
			// Set activity roles
			ps.addRole("Administrator", "The system administator role");
			ps.setNodeRole("A", "Administrator");
			//ps.setNodeRole("B", "Administrator");  // uncomment if you want to execute them manually
			//ps.setNodeRole("C", "Administrator");
			//ps.setNodeRole("D", "Administrator");
			//ps.setNodeRole("E", "Administrator");
			//ps.setNodeRole("F", "Administrator");
	
			// Set role mapper
			ps.addRoleMapper("Administrator", "AdministratorMapper", Constants.Mapper.PROPERTIES);
			
			// Checks if the model is correctly defined
			ps.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		}
	}
}
