/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
--------------------------------------------------------------------------
 * Class ExtraEntryPoint.java created by Jordi Anguela - June 2006
--------------------------------------------------------------------------
 */

package hero.client.samples.iterations.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class ExtraEntryPoint {
	static public void main(String[] args) throws Exception {
		try {		
			System.out.println(" ****** ExtraEntryPoint 1.0 ******* ");
			System.out.println(" *        Entry                   * ");
			System.out.println(" *          |_____ repeat=false   * ");
			System.out.println(" *        A----->B----->C         * ");
			System.out.println(" *        |      |                * ");
			System.out.println(" *        |_ _ _ |                * ");
			System.out.println(" *      repeat=true               * ");
			System.out.println(" ********************************** ");
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
			ProjectSession ps = projectSessionh.create();
			System.out.print("\n   Creating ExtraEntryPoint model");
			ps.initModel("ExtraEntryPoint");
			
			// Set Properties
			ps.setProperty("repeat", "true");
			
			// Set Nodes
			ps.addNode("A", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("B", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("C", Constants.Nd.AND_JOIN_NODE);
			ps.addNode("Entry", Constants.Nd.AND_JOIN_NODE);
	
			ps.setNodeTraditional("A");
			ps.setNodeTraditional("B");
			ps.setNodeTraditional("C");
			ps.setNodeTraditional("Entry");
	
			// Set Edges
			ps.addEdge("A", "B");
			String fromBtoC = ps.addEdge("B", "C");
			ps.addEdge("Entry", "B");
			
			ps.setEdgeCondition(fromBtoC, "repeat.equals(\"false\")");
	
			// Set Iteration
			ps.addIteration("B", "A", "repeat.equals(\"true\")");
			
			// Set activity roles
			ps.addRole("Administrator", "The system administator role");
			ps.setNodeRole("A", "Administrator");
			ps.setNodeRole("B", "Administrator");
			ps.setNodeRole("C", "Administrator");
			ps.setNodeRole("Entry", "Administrator");
			
			// Set role mapper
			ps.addRoleMapper("Administrator", "AdministratorMapper", Constants.Mapper.PROPERTIES);
	
			// Checks if the model is correctly defined
			ps.checkModelDefinition();

			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		}			
	}
}
