/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
--------------------------------------------------------------------------
 * Class MultiIterationsNode.java created by Jordi Anguela - June 2006
--------------------------------------------------------------------------
 */
package hero.client.samples.iterations.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class MultiIterationsNode {
	static public void main(String[] args) throws Exception {
		
		System.out.println(" ****   MultiIterationsNode    **** ");
		System.out.println(" *                                * ");
		System.out.println(" *          path=A                * ");
		System.out.println(" *     - - - - - - - -            * ");
		System.out.println(" *     |       path=B |           * ");
		System.out.println(" *     |      - - - - |           * ");
		System.out.println(" *     |      |      ||path=D     * ");
		System.out.println(" *     A----->B----->C----->D     * ");
		System.out.println(" *                  |_|           * ");
		System.out.println(" *                 path=C         * ");
		System.out.println(" *                                * ");
		System.out.println(" ********************************** ");

		// User Admin login
		char[] password = { 't', 'o', 't', 'o' };
		SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
		LoginContext lc = new LoginContext("TestClient", handler);
		lc.login();

		// Project Session Bean Creation using Remote Interface
		ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
		ProjectSession prjSession = prjHome.create();

		// Process creation by user admin
		System.out.println("\nMultiIterationsNode model creation");
		prjSession.initModel("MultiIterationsNode");
		
		System.out.println("\nActivities creation");
		try {
			prjSession.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 

		System.out.println("Setting Activities types");
		try {
			prjSession.setNodeTraditional("A");
			prjSession.setNodeTraditional("B");
			prjSession.setNodeTraditional("C");
			prjSession.setNodeTraditional("D");
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 
		
		System.out.println("Adding project properties");
		try {
			prjSession.setProperty("path", "A"); // Decides which iteration we have to take			
		} catch (Exception e) {
			System.out.println(e);
		}

		System.out.println("Adding edges between activities");
		String fromCtoD = "";  // Exit conditions from the iterations
		try {
			prjSession.addEdge("A", "B");
			prjSession.addEdge("B", "C");
			fromCtoD = prjSession.addEdge("C", "D");
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 

		System.out.println("Adding 'C' edge condition");
		try {
			prjSession.setEdgeCondition(fromCtoD, "path.equals(\"D\") || path.equals(\"d\")");
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 		

		System.out.println("Adding iterations: between C--->A & C--->B & C--->C");
		try {
			prjSession.addIteration("C", "A", "path.equals(\"A\") || path.equals(\"a\")");
			prjSession.addIteration("C", "B", "path.equals(\"B\") || path.equals(\"b\")");
			prjSession.addIteration("C", "C", "path.equals(\"C\") || path.equals(\"c\")");
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 		

		System.out.println("Settind activity roles and the role mapper");
		try {
			prjSession.addRole("Administrator", "The system administator role");
			prjSession.setNodeRole("A", "Administrator");
			prjSession.setNodeRole("B", "Administrator");
			prjSession.setNodeRole("C", "Administrator");
			prjSession.setNodeRole("D", "Administrator");

			prjSession.addRoleMapper("Administrator", "AdministratorMapper", Constants.Mapper.PROPERTIES);
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 		
		
		System.out.println("Checking model definition");
		try {
			prjSession.checkModelDefinition();
		} catch (Exception e) {
			System.out.println(" --> " + e);
		} 
		
		System.out.println(" --- FINISHED! ---");
	}
}
