/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 04 : Exclusive Choice
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * Activity evaluate_claim is followed by either pay_damage or
 * contact_customer. 
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern04ExclusiveChoice {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 04 - Exclusive Choice");
			prjSession.initModel("Pattern 04 - Exclusive Choice");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("evaluate_claim", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("pay_damage", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_customer", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("evaluate_claim");
			prjSession.setNodeTraditional("pay_damage");
			prjSession.setNodeTraditional("contact_customer");

			// Add transitions between activities
			String fromEvaluateToPay = prjSession.addEdge("evaluate_claim", "pay_damage");
			String fromEvaluateToContact = prjSession.addEdge("evaluate_claim", "contact_customer");
			
			// Add a node property and set transition conditions
			prjSession.setNodeProperty("evaluate_claim", "pay", "true", false);
			prjSession.setEdgeCondition(fromEvaluateToPay, "pay.equals(\"true\")");
			prjSession.setEdgeCondition(fromEvaluateToContact, "pay.equals(\"false\")");
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("evaluate_claim", "Executor");
			prjSession.setNodeRole("pay_damage", "Executor");
			prjSession.setNodeRole("contact_customer", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
