/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 06 : Multi Choice
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * After executing the activity evaluate_damage the activity more than one
 * activity can be executed. In this example activities contact_police and
 * contact_hospital are chosen.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern06MultiChoice {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 06 - Multi Choice");
			prjSession.initModel("Pattern 06 - Multi Choice");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("evaluate_damage", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_fire_department", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_police", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_hospital", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("evaluate_damage");
			prjSession.setNodeTraditional("contact_fire_department");
			prjSession.setNodeTraditional("contact_police");
			prjSession.setNodeTraditional("contact_hospital");
			
			// Add transitions between activities
			String toFireDep  = prjSession.addEdge("evaluate_damage", "contact_fire_department");
			String toPolice   = prjSession.addEdge("evaluate_damage", "contact_police");
			String toHospital = prjSession.addEdge("evaluate_damage", "contact_hospital");
			
			// Add a Project Property and set transition conditions
			prjSession.setNodeProperty("evaluate_damage", "call_fire_department", "false", false);
			prjSession.setNodeProperty("evaluate_damage", "call_police", "true", false);
			prjSession.setNodeProperty("evaluate_damage", "call_hospital", "true", false);
			prjSession.setEdgeCondition(toFireDep, "call_fire_department.equals(\"true\")");
			prjSession.setEdgeCondition(toPolice, "call_police.equals(\"true\")");
			prjSession.setEdgeCondition(toHospital, "call_hospital.equals(\"true\")");

			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("evaluate_damage", "Executor");
			prjSession.setNodeRole("contact_fire_department", "Executor");
			prjSession.setNodeRole("contact_police", "Executor");
			prjSession.setNodeRole("contact_hospital", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
