/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 07 : Synchronizing Merge
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * The idea of this simple example is create a new activity (contact_both)
 * that executes both contact_fire_department and contact_insurance 
 * activities.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern07SynchronizingMerge {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 07 - Synchronizing Merge");
			prjSession.initModel("Pattern 07 - Synchronizing Merge");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("evaluate_damage", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_fire_department", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_insurance", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_both", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("submit_report", hero.interfaces.Constants.Nd.OR_JOIN_NODE);
			prjSession.setNodeTraditional("evaluate_damage");
			prjSession.setNodeTraditional("contact_fire_department");
			prjSession.setNodeTraditional("contact_insurance");
			prjSession.setNodeTraditional("contact_both");
			prjSession.setNodeTraditional("submit_report");
			
			// Add transitions between activities
			String toFireDep  = prjSession.addEdge("evaluate_damage", "contact_fire_department");
			String toInsurance   = prjSession.addEdge("evaluate_damage", "contact_insurance");
			String toBoth = prjSession.addEdge("evaluate_damage", "contact_both");
			prjSession.addEdge("contact_fire_department", "submit_report");
			prjSession.addEdge("contact_insurance", "submit_report");
			prjSession.addEdge("contact_both", "submit_report");
			
			// Add Project Properties and set transition conditions
			prjSession.setProperty("call_fire_department", "true");
			prjSession.setProperty("call_insurance", "true");
			prjSession.setEdgeCondition(toFireDep, "(call_fire_department.equals(\"true\") && call_insurance.equals(\"false\"))");
			prjSession.setEdgeCondition(toInsurance, "(call_insurance.equals(\"true\") && call_fire_department.equals(\"false\"))");
			prjSession.setEdgeCondition(toBoth, "(call_insurance.equals(\"true\") && call_fire_department.equals(\"true\"))");

			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("evaluate_damage", "Executor");
			prjSession.setNodeRole("contact_fire_department", "Executor");
			prjSession.setNodeRole("contact_insurance", "Executor");
			prjSession.setNodeRole("contact_both", "Executor");
			prjSession.setNodeRole("submit_report", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]"); 
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
