/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 08 : Multi Merge
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * Two activities audit_application and process_application running in
 * parallel which are both followed by an activity close_case. Use of 
 * SubProcesses to do the same activities multiple times. 
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern08MultiMerge {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 08 - Multi Merge");
			prjSession.initModel("Pattern 08 - Multi Merge");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("audit_application", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("process_aplication", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("audit_application");
			prjSession.setNodeTraditional("process_aplication");
			
			// SubProcess definition
			ProjectSession prjSession2 = prjHome.create();
			prjSession2.initModel("Close case (SubProcess)");
			prjSession2.addNode("prepare_report",Constants.Nd.AND_JOIN_NODE);
			prjSession2.addNode("archive_case",Constants.Nd.AND_JOIN_NODE);
			prjSession2.setNodeTraditional("prepare_report");
			prjSession2.setNodeTraditional("archive_case");
			prjSession2.addEdge("prepare_report", "archive_case");
			prjSession2.addRole("Executor", "Rol that enables to execute the activities inside the SubProcess");
			prjSession2.setUserRole("admin", "Executor");
			prjSession2.setNodeRole("prepare_report", "Executor");
			prjSession2.setNodeRole("archive_case", "Executor");
			prjSession2.remove();
			
			// SubProcess instantiation (one instance for each path)
			prjSession.addNodeSubProcess("close_case1", "Close case (SubProcess)");
			prjSession.addNodeSubProcess("close_case2", "Close case (SubProcess)");
			
			// Add transitions between activities
			prjSession.addEdge("audit_application", "close_case1");
			prjSession.addEdge("process_aplication", "close_case2");
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("audit_application", "Executor");
			prjSession.setNodeRole("process_aplication", "Executor");
			prjSession.setNodeRole("close_case1", "Executor");
			prjSession.setNodeRole("close_case2", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
