/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 09 : Discriminator
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * Two activities audit_application and process_application running in
 * parallel which are both followed by an activity close_case. Use of 
 * SubProcesses to do the same activities multiple times. 
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern09Discriminator {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 09 - Discriminator");
			prjSession.initModel("Pattern 09 - Discriminator");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("create_request", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("consult_db1", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("consult_db2", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("process_answer", hero.interfaces.Constants.Nd.OR_JOIN_NODE);
			prjSession.setNodeTraditional("create_request");
			prjSession.setNodeTraditional("consult_db1");
			prjSession.setNodeTraditional("consult_db2");
			prjSession.setNodeTraditional("process_answer");
			
			// Add transitions between activities
			prjSession.addEdge("create_request", "consult_db1");
			prjSession.addEdge("create_request", "consult_db2");
			prjSession.addEdge("consult_db1", "process_answer");
			prjSession.addEdge("consult_db2", "process_answer");
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("create_request", "Executor");
			prjSession.setNodeRole("consult_db1", "Executor");
			prjSession.setNodeRole("consult_db2", "Executor");
			prjSession.setNodeRole("process_answer", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
