/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 10 : Arbitrary Cycles (Extra Exit Point)
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * Classical example of approval workflow where the user request a demand
 * to be approved for another person. If the demand is approved then 
 * processes the demand, if not, then the flow is redirect to the first 
 * user again to able to change his request. 
--------------------------------------------------------------------------
*/
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern10ArbitraryCycles1ExtraExitPoint {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 10 - Arbitrary Cycles (Extra Exit Point)");
			prjSession.initModel("Pattern 10 - Arbitrary Cycles (Extra Exit Point) 666");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("request_demand", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("approve_demand", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("process_demand", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("cancel_demand666", hero.interfaces.Constants.Nd.OR_JOIN_NODE); 
			prjSession.setNodeTraditional("request_demand");
			prjSession.setNodeTraditional("approve_demand");
			prjSession.setNodeTraditional("process_demand");
			prjSession.setNodeTraditional("cancel_demand666");
			
			// Add transitions between activities
			String fromDemandToapprove_demand = prjSession.addEdge("request_demand", "approve_demand");
			String fromDemandToCancel = prjSession.addEdge("request_demand", "cancel_demand666");
			String fromapprove_demandToProcess = prjSession.addEdge("approve_demand", "process_demand");
			String fromapprove_demandToCancel = prjSession.addEdge("approve_demand", "cancel_demand666");
			
			// Add Project Properties and set transition conditions
			prjSession.setProperty("approved", "false");
			prjSession.setProperty("cancel", "false");
			prjSession.setProperty("iterations", "1");
			prjSession.setEdgeCondition(fromDemandToapprove_demand, "cancel.equals(\"false\")");
			prjSession.setEdgeCondition(fromDemandToCancel, "cancel.equals(\"true\")");
			prjSession.setEdgeCondition(fromapprove_demandToProcess, "(approved.equals(\"true\") && cancel.equals(\"false\"))");
			prjSession.setEdgeCondition(fromapprove_demandToCancel, "cancel.equals(\"true\")");

			// Add iteration
			prjSession.addIteration("approve_demand", "request_demand", "(approved.equals(\"false\") && cancel.equals(\"false\"))");
			
			// Add Hook to approve_demand activity to iterate twice and then accept the demand
			String approve_demandHookScript =
				  "import hero.interfaces.*;\n"
				+ "beforeStart(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  BnProjectPropertyValue propIterations = prjSession.getProperty(\"iterations\"); \n"
				+ "  int it= Integer.parseInt(propIterations.getTheValue()); \n"
				+ "  if ( it > 0 ) { \n"
				+ "    it--; \n"
				+ "    prjSession.setProperty(\"iterations\", Integer.toString(it)); \n"
				+ "  } else { \n "
				+ "    prjSession.setProperty(\"approved\", \"true\"); \n"
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("approve_demand", "approve_demand Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, approve_demandHookScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("request_demand", "Executor");
			prjSession.setNodeRole("approve_demand", "Executor");
			prjSession.setNodeRole("process_demand", "Executor");
			prjSession.setNodeRole("cancel_demand666", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
