/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 13 : Multiple Instances With a Priori Design Time Knowledge
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * Similar to pattern12 but now the sub process create are synchronized
 * with another activity. This example simulates an approval workflow 
 * where three different users are asked for an authorization. After that
 * 'send_requisition' activity is executed.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern13MultipleInstancesWithDesignTimeKnowledge {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 13 - Multiple Instances With a Priori Design Time Knowledge");
			prjSession.initModel("Pattern 13 - Multiple Instances With a Priori Design Time Knowledge");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("hazardous_materials_requisition", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("send_requesition", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("hazardous_materials_requisition");
			prjSession.setNodeTraditional("send_requesition");
			
			// Add Project Properties and set transition conditions
			//prjSession.setProperty("delegate", "3");
			
			// SubProcess definition
			ProjectSession prjSession2 = prjHome.create();
			prjSession2.initModel("Authorize requisition (SubProcess)");
			prjSession2.addNode("authorize",Constants.Nd.AND_JOIN_NODE);
			prjSession2.setNodeTraditional("authorize");
			prjSession2.addRole("Executor", "Rol that enables to execute the activities inside the SubProcess");
			prjSession2.setUserRole("admin", "Executor");
			prjSession2.setNodeRole("authorize", "Executor");
			prjSession2.remove();

			// SubProcess instantiation
			prjSession.addNodeSubProcess("authorization1", "Authorize requisition (SubProcess)");
			prjSession.addNodeSubProcess("authorization2", "Authorize requisition (SubProcess)");
			prjSession.addNodeSubProcess("authorization3", "Authorize requisition (SubProcess)");
			prjSession.addEdge("hazardous_materials_requisition", "authorization1");
			prjSession.addEdge("hazardous_materials_requisition", "authorization2");
			prjSession.addEdge("hazardous_materials_requisition", "authorization3");
			prjSession.addEdge("authorization1", "send_requesition");
			prjSession.addEdge("authorization2", "send_requesition");
			prjSession.addEdge("authorization3", "send_requesition");
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("hazardous_materials_requisition", "Executor");
			prjSession.setNodeRole("authorization1", "Executor");
			prjSession.setNodeRole("authorization2", "Executor");
			prjSession.setNodeRole("authorization3", "Executor");
			prjSession.setNodeRole("send_requesition", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
