/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 15 : Multiple Instances Without a Priori Runtime Knowledge
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * This example is based on the previous pattern but know we have an extra
 * activity called 'add_books' that will add two new sub 
 * processes.
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern15MultipleInstancesWithoutRuntimeKnowledge {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 15 - Multiple Instances Without a Priori Runtime Knowledge");
			prjSession.initModel("Pattern 15 - Multiple Instances Without a Priori Runtime Knowledge");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("order_books", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("add_books", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("shipping_process", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("send_bill", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("order_books");
			prjSession.setNodeTraditional("shipping_process");
			prjSession.setNodeTraditional("send_bill");
			prjSession.addEdge("shipping_process", "send_bill");
			
			// Add Project Properties : we want to have 3 instances of the subprocess in total. 
			// One created in the model and the other two created during runtime. 
			prjSession.setProperty("number_of_sub_processes", "2");
			
			// SubProcess definition
			ProjectSession prjSession2 = prjHome.create();
			prjSession2.initModel("Check book avaliability (SubProcess) 2");
			prjSession2.addNode("check_avaliability",Constants.Nd.AND_JOIN_NODE);
			prjSession2.setNodeTraditional("check_avaliability");
			String update_stockHookScript =
				  "import hero.interfaces.*;\n"
				+ "beforeStart(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  BnProjectPropertyValue bookNameProp = prjSession.getProperty(\"book_name\"); \n"
				+ "  System.out.println(\"   ---> check_avaliability for : \" + bookNameProp.getTheValue()); \n"
				+ "}";			
			prjSession2.addNodeInterHook("check_avaliability", "check_avaliability Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, update_stockHookScript);			
			prjSession2.addRole("Executor", "Rol that enables to execute the activities inside the SubProcess");
			prjSession2.setUserRole("admin", "Executor");
			prjSession2.setNodeRole("check_avaliability", "Executor");
			prjSession2.remove();
			
			// SubProcess instantiation
			prjSession.addNodeSubProcess("check_book0", "Check book avaliability (SubProcess) 2");
			prjSession.setNodeProperty("check_book0", "book_name", "check_book0", true); 
			prjSession.addEdge("order_books", "check_book0");
			prjSession.addEdge("check_book0", "shipping_process");
			
			// Add Hook to 'Order books' activity to create a SubProces for each book 
			String createSubProcessesScript =
				  "import hero.interfaces.*;\n"
				+ "beforeStart(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  BnProjectPropertyValue numberOfSubProcessesProp = prjSession.getProperty(\"number_of_sub_processes\"); \n"
				+ "  int numberOfSubProcesses = Integer.parseInt(numberOfSubProcessesProp.getTheValue()); \n"
				+ "  for (int i = 1; i <= numberOfSubProcesses; i++) { \n"
				+ "    String nodeName = \"check_book\" + Integer.toString(i); \n"
				+ "    prjSession.addNodeSubProcess(nodeName, \"Check book avaliability (SubProcess) 2\"); \n"
				+ "    prjSession.setNodeProperty(nodeName, \"book_name\", nodeName, true); \n"
				+ "    prjSession.addEdge(nodeName, \"shipping_process\"); \n"
				+ "    prjSession.setNodeRole(nodeName, \"Executor\"); \n"
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("check_book0", "check_book0 Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, createSubProcessesScript);
			
			// Add Hook to 'add_books' activity to create two more sub proceses 
			String createSubProcesses2Script =
				  "import hero.interfaces.*;\n"
				+ "afterStart(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  for (int i = 3; i <= 4; i++) { \n"
				+ "    String nodeName = \"check_book\" + Integer.toString(i); \n"
				+ "    prjSession.addNodeSubProcess(nodeName, \"Check book avaliability (SubProcess) 2\"); \n"
				+ "    prjSession.setNodeProperty(nodeName, \"book_name\", nodeName, true); \n"
				+ "    prjSession.addEdge(nodeName, \"shipping_process\"); \n"
				+ "    prjSession.setNodeRole(nodeName, \"Executor\"); \n"
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("add_books", "add_books Hook", hero.interfaces.Constants.Nd.AFTERSTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, createSubProcesses2Script);
			
			// Add Hook to 'shipping_process' activity to cancel the generation of more sub proceses when it starts 
			String cancelScript =
				  "import hero.interfaces.*; \n"
				+ "beforeStart(Object engine, Object currentNode) { \n"
				+ "  UserSessionHome userHome = UserSessionUtil.getHome(); \n"
				+ "  UserSession userSession = userHome.create(); \n"
				+ "  userSession.cancelActivity(currentNode.getBnProject().getName(), \"add_books\"); \n "
				+ "}";
			prjSession.addNodeInterHook("shipping_process", "shipping_process Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, cancelScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("order_books", "Executor");
			prjSession.setNodeRole("add_books", "Executor");
			prjSession.setNodeRole("check_book0", "Executor");			
			prjSession.setNodeRole("shipping_process", "Executor");
			prjSession.setNodeRole("send_bill", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
