/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 18 : Milestone
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * In this case we have 3 activities: 'A', 'B' and 'C'. A is only enabled 
 * only if 'B' has been executed and 'C' has not been executed yet. 
 * A trasition from 'B' to 'A' is used to enable 'A' when 'B' is finnishes
 * its execution
 * 'C' has a hook that cancels 'A' if it is still ready to disable it.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern18Milestone {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 18 - Milestone");
			prjSession.initModel("Pattern 18 - Milestone");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("B");
			prjSession.setNodeTraditional("A");
			prjSession.setNodeTraditional("C");
	
			// Add transitions between activities
			prjSession.addEdge("B", "C");
			prjSession.addEdge("B", "A");
			
			// Add Hook to 'C' to cancel the 'A' if it has not executed yet 
			String cancelScript =
				  "import hero.interfaces.*; \n"
				+ "beforeStart(Object engine, Object currentNode) { \n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  if (prjSession.getNodeState(\"A\") == hero.interfaces.Constants.Nd.READY) { \n"
				+ "    UserSessionHome userHome = UserSessionUtil.getHome(); \n"
				+ "    UserSession userSession = userHome.create(); \n"
				+ "    userSession.cancelActivity(currentNode.getBnProject().getName(), \"A\"); \n "
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("C", "C Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, cancelScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("B", "Executor");
			prjSession.setNodeRole("A", "Executor");
			prjSession.setNodeRole("C", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
