/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern Extra 01 : Explicit Termination
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * This example uses a special activity called 'BonitaEnd' that 
 * terminates explicitly the execution of the process when it is reached.
 * We will use the same example as in pattern05 - Simple Merge but now
 * it is not necessary to execute both paths to terminate.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class PatternExtra1ExplicitTermination {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern Extra 1 - Explicit Termination");
			prjSession.initModel("Pattern Extra 1 - Explicit Termination");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("pay_damage", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("contact_customer", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("archive_claim", hero.interfaces.Constants.Nd.OR_JOIN_NODE);
			prjSession.addNode("after_archive", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("BonitaEnd", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
			prjSession.setNodeTraditional("pay_damage");
			prjSession.setNodeTraditional("contact_customer");
			prjSession.setNodeTraditional("archive_claim");
			prjSession.setNodeTraditional("after_archive");
			prjSession.setNodeTraditional("BonitaEnd");
			
			// Add transitions between activities
			prjSession.addEdge("pay_damage", "archive_claim");
			prjSession.addEdge("contact_customer", "archive_claim");
			prjSession.addEdge("archive_claim", "after_archive");
			prjSession.addEdge("after_archive", "BonitaEnd");

			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("pay_damage", "Executor");
			prjSession.setNodeRole("contact_customer", "Executor");
			prjSession.setNodeRole("archive_claim", "Executor");
			prjSession.setNodeRole("after_archive", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
