/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern Extra 2 : Cancel Path
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * This example shows a cancel in cascade. A hook is lunched before 
 * terminating 'split' activity  cancelling 'A1'. Then, activities 'A2'
 * and 'A3' are automaticaly cancelled due to they are reachable.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class PatternExtra2CancelPath {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern Extra 2 - Cancel Path");
			prjSession.initModel("Pattern Extra 2 - Cancel Path");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("split", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("A1", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("A2", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("A3", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("B1", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("split");
			prjSession.setNodeTraditional("A1");
			prjSession.setNodeTraditional("A2");
			prjSession.setNodeTraditional("A3");
			prjSession.setNodeTraditional("B1");
	
			// Add transitions between activities
			prjSession.addEdge("split", "A1");
			prjSession.addEdge("A1", "A2");
			prjSession.addEdge("A2", "A3");
			prjSession.addEdge("split", "B1");

			// Add Hook to 'split' to cancel the 'A1'. 'A2' and 'A3' activities will be cancelled automaticaly 
			String cancelScript =
				  "import hero.interfaces.*; \n"
				+ "afterTerminate(Object engine, Object currentNode) { \n"
				+ "  UserSessionHome userHome = UserSessionUtil.getHome(); \n"
				+ "  UserSession userSession = userHome.create(); \n"
				+ "  userSession.cancelActivity(currentNode.getBnProject().getName(), \"A1\"); \n "
				+ "}";
			prjSession.addNodeInterHook("split", "split Hook", hero.interfaces.Constants.Nd.AFTERTERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, cancelScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("split", "Executor");
			prjSession.setNodeRole("A1", "Executor");
			prjSession.setNodeRole("A2", "Executor");
			prjSession.setNodeRole("A3", "Executor");
			prjSession.setNodeRole("B1", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
