package hero.client.test;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: AdminSessionTests.java,v 1.3 2006/03/31 14:27:31 mvaldes Exp $
*
--------------------------------------------------------------------------
*/
import javax.security.auth.login.LoginContext;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.UserRegistration;
import hero.interfaces.UserRegistrationHome;
import hero.interfaces.UserRegistrationUtil;

import hero.interfaces.AdminSessionHome;
import hero.interfaces.AdminSession;
import hero.interfaces.AdminSessionUtil;

import java.util.Collection;
import junit.framework.TestCase;
import junit.framework.TestSuite;


public class AdminSessionTests extends TestCase {

    private ProjectSessionHome projectsesh;

    public AdminSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(AdminSessionTests.class);
    }

    public void setUp() throws Exception {
    }

   public void testGetModels() throws Exception {
	UserRegistrationHome urHome = UserRegistrationUtil.getHome();
	UserRegistration userReg=urHome.create();
	userReg.userCreate("userAdmin1","userAdmin1","miguel.valdes-faura@ext.bull.net");
	userReg.userCreate("userAdmin2","userAdmin2","miguel.valdes-faura@ext.bull.net");
	
	SimpleCallbackHandler handler =	new SimpleCallbackHandler("userAdmin1", "userAdmin1".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
   	
	projectsesh=ProjectSessionUtil.getHome();
	ProjectSession p = projectsesh.create();
	p.initProject("project1");	
	p.initModel("project2");
		
	handler =	new SimpleCallbackHandler("userAdmin2", "userAdmin2".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p2 = projectsesh.create();
	p2.initModel("project3");
	p2.initModel("project4");
	
	AdminSessionHome aHome = AdminSessionUtil.getHome();
	AdminSession adminsession = aHome.create();
	
	Collection models = adminsession.getModels();
	assertTrue("Error in getModels, "+models.size(), models.size() == 2);
	
	handler =	new SimpleCallbackHandler("userAdmin1", "userAdmin1".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
    }
   
   public void testInstances() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	ProjectSession p = projectsesh.create();
	p.initProject("project5");	
	p.initModel("project6");
	
	p.instantiateProject("project6");
	p.instantiateProject("project6");
		
	SimpleCallbackHandler handler =	new SimpleCallbackHandler("userAdmin2", "userAdmin2".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p2 = projectsesh.create();
	p2.initModel("project7");
	p2.initModel("project8");
	
	p2.instantiateProject("project7");
	p2.instantiateProject("project7");
	p2.instantiateProject("project7");
	
	handler =	new SimpleCallbackHandler("userAdmin1", "userAdmin1".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p3 = projectsesh.create();
	p3.instantiateProject("project8");
	p3.instantiateProject("project8");
	
	handler =	new SimpleCallbackHandler("userAdmin2", "userAdmin2".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p4 = projectsesh.create();
	p4.instantiateProject("project8");
	
	AdminSessionHome aHome = AdminSessionUtil.getHome();
	AdminSession adminsession = aHome.create();
	
	Collection instances = adminsession.getInstances();
	assertTrue("Error in getInstances, "+instances.size(), instances.size() == 6);
	
	handler =	new SimpleCallbackHandler("userAdmin1", "userAdmin1".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
    }
    
   public void testActivities() throws Exception {
	projectsesh=ProjectSessionUtil.getHome();
	ProjectSession p = projectsesh.create();
	p.initProject("project9");
	p.addNode("node1",1);
	
	p.initModel("project10");
	p.addNode("node1",1);
	
	p.instantiateProject("project10");
	p.addUser("userAdmin2");
	p.instantiateProject("project10");
		
	SimpleCallbackHandler handler =	new SimpleCallbackHandler("userAdmin2", "userAdmin2".toCharArray());
	LoginContext lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p2 = projectsesh.create();
	p2.initModel("project11");
	p2.addNode("node1",1);
	p2.addNode("node2",1);
	
	p2.initModel("project12");
	p2.addNode("node1",1);
	
	p2.instantiateProject("project11");
	p2.instantiateProject("project11");
	p2.instantiateProject("project11");
	
	handler =	new SimpleCallbackHandler("userAdmin1", "userAdmin1".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p3 = projectsesh.create();
	p3.instantiateProject("project12");
	p3.instantiateProject("project12");
	
	handler =	new SimpleCallbackHandler("userAdmin2", "userAdmin2".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
	
	ProjectSession p4 = projectsesh.create();
	p4.instantiateProject("project12");
	
	AdminSessionHome aHome = AdminSessionUtil.getHome();
	AdminSession adminsession = aHome.create();
	
	ProjectSession p5 = projectsesh.create();
	p5.initProject("project13");
	p5.addNode("node1",1);
	
	Collection activities = adminsession.getActivities();
	assertTrue("Error in getInstances, "+activities.size(), activities.size() == 10);
		
	handler =	new SimpleCallbackHandler("admin", "toto".toCharArray());
	lc = new LoginContext("TestClient", handler);
	lc.login();
    }   
}
