package hero.client.test;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: IterationTests.java,v 1.7 2006/08/17 15:30:53 mvaldes Exp $
*
--------------------------------------------------------------------------
*/

import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class IterationTests extends TestCase {
    public IterationTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(IterationTests.class);
    }

    public void setUp() throws Exception {
    }

    /**
     * Tests the correct exit from the iterations 
     * Uses: Automatic and Non automatic anticipable activities
	 * 
	 * @throws Exception
     */
	public void testLeaveIteration() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("LeaveIteration");
		
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node8");
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node9");

		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");
		
		ps.setProperty("iterate","true");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		ps.checkModelDefinition();
		
		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("LeaveIteration");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		usr.startActivity(project,"node0");
		assertTrue("Error in LeaveIteration, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in LeaveIteration, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.ANTICIPABLE);
		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in LeaveIteration, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in LeaveIteration, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);

		int index = 0;
		while (index < 2)	{
			usr.startActivity(project,"node1");		
			assertTrue("Error in LeaveIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in LeaveIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
			assertTrue("Error in LeaveIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in LeaveIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in LeaveIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in LeaveIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node3");
			usr.startActivity(project,"node5");
			usr.startActivity(project,"node4");
			
			if (index == 0) {
				assertTrue("Error in LeaveIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPABLE);
				usr.startActivity(project,"node2");
				assertTrue("Error in LeaveIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPATING);
			}
			
			usr.terminateActivity(project,"node3");
			usr.terminateActivity(project,"node5");
			usr.terminateActivity(project,"node4");			
			assertTrue("Error in LeaveIteration, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
			assertTrue("Error in LeaveIteration, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
			assertTrue("Error in LeaveIteration, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);

			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");	
			assertTrue("Error in LeaveIteration, terminateActivity node6: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in LeaveIteration, terminateActivity node6: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.INITIAL);
			index++;
		}
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in LeaveIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in LeaveIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in LeaveIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in LeaveIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in LeaveIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in LeaveIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
		assertTrue("Error in LeaveIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in LeaveIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
		usr.startActivity(project,"node3");
		usr.startActivity(project,"node5");
		usr.startActivity(project,"node4");

		// Finally, it will leave throw node2 and node8 activities. node6 and node7 will become DEAD
		ps.setProperty("iterate","false");
		
		usr.terminateActivity(project,"node3");
		usr.terminateActivity(project,"node5");
		usr.terminateActivity(project,"node4");
			
		assertTrue("Error in LeaveIteration, terminateActivity node3 | node4, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.DEAD);
		assertTrue("Error in LeaveIteration, terminateActivity node3 | node4, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.DEAD);

		assertTrue("Error in LeaveIteration, terminateActivity node3, after iterate: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in LeaveIteration, terminateActivity node3, after iterate: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.TERMINATED);

		assertTrue("Error in LeaveIteration, terminateActivity node6, after iterate: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node2");
		assertTrue("Error in LeaveIteration, startActivity node2, after iterate: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.EXECUTING);

		usr.terminateActivity(project,"node2"); 
	}

    /**
     * Tests that the iteration finalizes correctly (based on testLeaveIteration)
     * Uses: Automatic and Non automatic anticipable activities
	 * 
	 * @throws Exception
     */
	public void testFinalizeIteration() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("FinalizeIteration");
		
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node8");
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node9");

		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");
		
		ps.setProperty("iterate","true");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("FinalizeIteration");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		usr.startActivity(project,"node0");
		assertTrue("Error in FinalizeIteration, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in FinalizeIteration, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.ANTICIPABLE);
		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in FinalizeIteration, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);

		int index = 0;
		while (index < 2)	{
			usr.startActivity(project,"node1");		
			assertTrue("Error in FinalizeIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in FinalizeIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
			assertTrue("Error in FinalizeIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node3");
			usr.startActivity(project,"node5");
			usr.startActivity(project,"node4");
			
			if (index == 0) {
				assertTrue("Error in FinalizeIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPABLE);
				usr.startActivity(project,"node2");
				assertTrue("Error in FinalizeIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPATING);
			}
			
			usr.terminateActivity(project,"node3");
			usr.terminateActivity(project,"node5");
			usr.terminateActivity(project,"node4");			
			assertTrue("Error in FinalizeIteration, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
			assertTrue("Error in FinalizeIteration, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
			assertTrue("Error in FinalizeIteration, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);

			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");	
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.INITIAL);
			index++;
		}
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in FinalizeIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in FinalizeIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in FinalizeIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in FinalizeIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
		usr.startActivity(project,"node3");
		usr.startActivity(project,"node5");
		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node3");
		usr.terminateActivity(project,"node5");
		usr.terminateActivity(project,"node4");
		assertTrue("Error in FinalizeIteration, terminateActivity node3, after iterate: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in FinalizeIteration, terminateActivity node3, after iterate: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
		assertTrue("Error in FinalizeIteration, terminateActivity node4, after iterate: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
		
		// Finally, it will finalize throw node7.  Node2, node8 and node9 will become DEAD
		ps.setProperty("iterate","false");
		
		usr.startActivity(project,"node6");
		usr.terminateActivity(project,"node6");			
		assertTrue("Error in FinalizeIteration, terminateActivity node6, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node6, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node7");
		usr.terminateActivity(project,"node7");			
	}
	
    /**
     * Tests that the iteration finalizes correctly with OR activities (based on testFinalizeIteration)
     * Uses: Automatic and Non automatic anticipable activities
	 * 
	 * @throws Exception
     */
	public void testFinalizeIterationORjoin() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("FinalizeIterationORjoin");
		
		ps.addNode("node0",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node8");
		ps.addNode("node9",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node9");

		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");

		ps.setProperty("iterate","true");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		
		ps.addIteration("node6","node1","iterate.equals(\"true\")");

		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("FinalizeIterationORjoin");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		usr.startActivity(project,"node0");
		assertTrue("Error in FinalizeIteration, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in FinalizeIteration, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.ANTICIPABLE);
		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in FinalizeIteration, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);

		int index = 0;
		while (index < 2)	{
			usr.startActivity(project,"node1");		
			assertTrue("Error in FinalizeIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in FinalizeIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
			assertTrue("Error in FinalizeIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.ANTICIPABLE);
			assertTrue("Error in FinalizeIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);
			assertTrue("Error in FinalizeIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node3");
			usr.startActivity(project,"node5");
			usr.startActivity(project,"node4");
			
			if (index == 0) {
				assertTrue("Error in FinalizeIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPABLE);
				usr.startActivity(project,"node2");
				assertTrue("Error in FinalizeIteration, startActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.ANTICIPATING);
			} 
			
			usr.terminateActivity(project,"node3");
			assertTrue("Error in FinalizeIteration, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
			assertTrue("Error in FinalizeIteration, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
			assertTrue("Error in FinalizeIteration, terminateActivity node3: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");		
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);  
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in FinalizeIteration, terminateActivity node6: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.INITIAL);
			index++;
		}
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in FinalizeIteration, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in FinalizeIteration, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in FinalizeIteration, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in FinalizeIteration, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
			
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);
		assertTrue("Error in FinalizeIteration, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
		usr.startActivity(project,"node3");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in FinalizeIteration, terminateActivity node3, after iterate: node8 failed", ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in FinalizeIteration, terminateActivity node3, after iterate: node9 failed", ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");
		assertTrue("Error in FinalizeIteration, terminateActivity node4, after iterate: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
		assertTrue("Error in FinalizeIteration, terminateActivity node4, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.READY);
		
		// Finally, it will finalize throw node7.  Node2, node8 and node9 are DEAD
		ps.setProperty("iterate","false");
		
		usr.startActivity(project,"node6");
		usr.terminateActivity(project,"node6");			
		assertTrue("Error in FinalizeIteration, terminateActivity node6, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in FinalizeIteration, terminateActivity node6, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node7");
		usr.terminateActivity(project,"node7");			
	}
	
	/**
	 * Tests a iteration over the same activity
	 * Uses: Non automatic traditional activities
	 * 
	 * @throws Exception
	 */
	public void testRepeatActivity() throws Exception {
		ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("RepeatActivity");
		
		ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("A");
		ps.setNodeTraditional("B");
		ps.setNodeTraditional("C");
		
		ps.setProperty("repeat", "true");			
		ps.setProperty("repetitions", "3");			
	
		ps.addEdge("A", "B");
		String fromBtoC = ps.addEdge("B", "C");
		ps.setEdgeCondition(fromBtoC, "repeat.equals(\"false\")");
	
		String repeatScript =
			  "import hero.interfaces.*;\n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			+ "  BnProjectPropertyValue repetitionsProp = prjSession.getProperty(\"repetitions\"); \n"
			+ "  int repetitions = Integer.parseInt(repetitionsProp.getTheValue()); \n"
			+ "  if (repetitions == 0) { \n"
			+ "    prjSession.setProperty(\"repeat\", \"false\"); \n"
			+ "  } else { \n "
			+ "    repetitions--; \n"
			+ "    prjSession.setProperty(\"repetitions\", Integer.toString(repetitions)); \n"
			+ "  } \n"
			+ "} \n";
		ps.addNodeInterHook("B", "Repeat hook", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, repeatScript);			

		ps.addIteration("B", "B", "repeat.equals(\"true\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("RepeatActivity");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		usr.startActivity(project, "A");
		usr.terminateActivity(project, "A");
		assertTrue("Error in RepeatActivity, terminateActivity 'A': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in RepeatActivity, terminateActivity 'A': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);
		
		int repetitions = 0;
		while (repetitions < 3) {
			usr.startActivity(project, "B");
			usr.terminateActivity(project, "B");
			assertTrue("Error in RepeatActivity, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);
			assertTrue("Error in RepeatActivity, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.INITIAL);
			repetitions++;
		}
		usr.startActivity(project, "B");
		usr.terminateActivity(project, "B");
		assertTrue("Error in RepeatActivity, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in RepeatActivity, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.READY);

		usr.startActivity(project, "C");
		usr.terminateActivity(project, "C");
	}
	
	/**
	 * Tests a iteration inside another iteration
	 * Uses: Non automatic traditional activities
	 * 
	 * @throws Exception
	 */
	public void testIterationInsideIteration() throws Exception {
		ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("IterationInsideIteration");
		
		ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("A");
		ps.setNodeTraditional("B");
		ps.setNodeTraditional("C");
		ps.setNodeTraditional("D");
		
		ps.setProperty("condition1", "true");			
		ps.setProperty("condition2", "true");			
		ps.setProperty("repetitions1", "3");			
		ps.setProperty("repetitions2", "3");			
	
		ps.addEdge("A", "B");
		String fromBtoC = ps.addEdge("B", "C");
		String fromCtoD = ps.addEdge("C", "D");
		ps.setEdgeCondition(fromBtoC, "condition1.equals(\"false\")");
		ps.setEdgeCondition(fromCtoD, "condition2.equals(\"false\")");
	
		String repeat1Script =
			  "import hero.interfaces.*;\n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			+ "  BnProjectPropertyValue repetitions1Prop = prjSession.getProperty(\"repetitions1\"); \n"
			+ "  int repetitions1 = Integer.parseInt(repetitions1Prop.getTheValue()); \n"
			+ "  repetitions1--; \n"
			+ "  if (repetitions1 == 0) { \n"
			+ "    prjSession.setProperty(\"repetitions1\", \"3\"); \n"
			+ "    prjSession.setProperty(\"condition1\", \"false\"); \n"
			+ "  } else { \n "
			+ "    prjSession.setProperty(\"repetitions1\", Integer.toString(repetitions1)); \n"
			+ "  } \n"
			+ "} \n";
		ps.addNodeInterHook("B", "Repeat1 hook", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, repeat1Script);			
		ps.addIteration("B", "B", "condition1.equals(\"true\")");

		String repeat2Script =
			  "import hero.interfaces.*;\n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			+ "  BnProjectPropertyValue repetitions2Prop = prjSession.getProperty(\"repetitions2\"); \n"
			+ "  int repetitions2 = Integer.parseInt(repetitions2Prop.getTheValue()); \n"
			+ "  repetitions2--; \n"
			+ "  if (repetitions2 == 0) { \n"
			+ "    prjSession.setProperty(\"condition2\", \"false\"); \n"
			+ "  } else { \n "
			+ "    prjSession.setProperty(\"repetitions2\", Integer.toString(repetitions2)); \n"
			+ "    prjSession.setProperty(\"condition1\", \"true\"); \n"
			+ "  } \n"
			+ "} \n";
		ps.addNodeInterHook("C", "Repeat2 hook", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, repeat2Script);			
		ps.addIteration("C", "A", "condition2.equals(\"true\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("IterationInsideIteration");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		int repetitions1 = 0;
		int repetitions2 = 0;
		while (repetitions2 < 3) {
			usr.startActivity(project, "A");
			usr.terminateActivity(project, "A");
			assertTrue("Error in IterationInsideIteration, terminateActivity 'A': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in IterationInsideIteration, terminateActivity 'A': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);
		
			repetitions1 = 0;
			while (repetitions1 < 2) {
				usr.startActivity(project, "B");
				usr.terminateActivity(project, "B");
				assertTrue("Error in IterationInsideIteration, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);
				assertTrue("Error in IterationInsideIteration, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.INITIAL);
				repetitions1++;
			}
			usr.startActivity(project, "B");
			usr.terminateActivity(project, "B");
			assertTrue("Error in IterationInsideIteration, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in IterationInsideIteration, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.READY);
			assertTrue("Error in IterationInsideIteration, terminateActivity 'B': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.INITIAL);

			usr.startActivity(project, "C");
			usr.terminateActivity(project, "C");
			repetitions2++;
		}		
		assertTrue("Error in IterationInsideIteration, terminateActivity 'C': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in IterationInsideIteration, terminateActivity 'C': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in IterationInsideIteration, terminateActivity 'C': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in IterationInsideIteration, terminateActivity 'C': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.READY);
		
		usr.startActivity(project, "D");
		usr.terminateActivity(project, "D");
	}

	/**
	 * Tests that we can loop throw two crossed iterations as we want
	 * Uses: Non automatic traditional activities
	 * 
	 * @throws Exception
	 */
	public void testManualArbitraryCycles() throws Exception {
		ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ManualArbitraryCycles");
		
		ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("E", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("F", hero.interfaces.Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("A");
		ps.setNodeTraditional("B");
		ps.setNodeTraditional("C");
		ps.setNodeTraditional("D");
		ps.setNodeTraditional("E");
		ps.setNodeTraditional("F");
		
		ps.setProperty("condition1", "true");	// first condition			
		ps.setProperty("condition2", "true");	// second iteration
		ps.setProperty("iterations", "2");		// number of iterations
	
		ps.addEdge("A", "B");
		ps.addEdge("B", "C");
		ps.addEdge("C", "D");
		String fromDtoE = ps.addEdge("D", "E");
		String fromEtoF = ps.addEdge("E", "F");
	
		ps.setEdgeCondition(fromDtoE, "condition1.equals(\"false\")");
		ps.setEdgeCondition(fromEtoF, "condition2.equals(\"false\")");
		
		String dScript =
			  "import hero.interfaces.*;\n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			+ "  BnProjectPropertyValue condition1Prop = prjSession.getProperty(\"condition1\"); \n"
			+ "  String condition1 = condition1Prop.getTheValue(); \n"
			+ "  if (condition1.equals(\"true\")) { \n"
			+ "    prjSession.setProperty(\"condition1\", \"false\"); \n"
			+ "  } else { \n "
			+ "    prjSession.setProperty(\"condition1\", \"true\"); \n"
			+ "  } \n"
			+ "} \n";
		ps.addNodeInterHook("D", "D script", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, dScript);			

		String eScript =
			  "import hero.interfaces.*;\n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
			+ "  BnProjectPropertyValue iterationsProp = prjSession.getProperty(\"iterations\"); \n"
			+ "  int iterations = Integer.parseInt(iterationsProp.getTheValue()); \n"
			+ "  if (iterations == 0) { \n"
			+ "    prjSession.setProperty(\"condition2\", \"false\"); \n"
			+ "  } else { \n "
			+ "    iterations--; \n"
			+ "    prjSession.setProperty(\"iterations\", Integer.toString(iterations)); \n"
			+ "  } \n"
			+ "} \n";
		ps.addNodeInterHook("E", "E script", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, eScript);			

		ps.addIteration("D", "B", "condition1.equals(\"true\")");
		ps.addIteration("E", "C", "condition2.equals(\"true\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ManualArbitraryCycles");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		usr.startActivity(project, "A");
		usr.terminateActivity(project, "A");
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);

		usr.startActivity(project, "B");
		usr.terminateActivity(project, "B");
		assertTrue("Error in ArbitraryCycles, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.READY);

		usr.startActivity(project, "C");
		usr.terminateActivity(project, "C");
		assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.READY);

		int iterations = 0;
		while (iterations < 2) {
			usr.startActivity(project, "D");
			usr.terminateActivity(project, "D");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'E' failed", ps.getNodeValue("E").getState() == Constants.Nd.READY);

			usr.startActivity(project, "E");
			usr.terminateActivity(project, "E");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.READY);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'E' failed", ps.getNodeValue("E").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'F' failed", ps.getNodeValue("F").getState() == Constants.Nd.INITIAL);

			usr.startActivity(project, "C");
			usr.terminateActivity(project, "C");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.READY);

			usr.startActivity(project, "D");
			usr.terminateActivity(project, "D");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.READY);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'D': 'E' failed", ps.getNodeValue("E").getState() == Constants.Nd.INITIAL);
			
			usr.startActivity(project, "B");
			usr.terminateActivity(project, "B");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'B': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'B': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.READY);

			usr.startActivity(project, "C");
			usr.terminateActivity(project, "C");
			assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ArbitraryCycles, terminateActivity 'C': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.READY);
			
			iterations++;
		}
		usr.startActivity(project, "D");
		usr.terminateActivity(project, "D");

		usr.startActivity(project, "E");
		usr.terminateActivity(project, "E");
		
		assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'E': 'E' failed", ps.getNodeValue("E").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project, "F");
		usr.terminateActivity(project, "F");
	}
	
	/**
	 * Tests arbitrary cycles with automatic activities using random values to evaluate the conditions
	 * Uses: Automaticand non automatic traditional activities
	 * 
	 * @throws Exception
	 */
	public void testAutomaticArbitraryCycles() throws Exception {
		ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("AutomaticArbitraryCycles");
		
		ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("E", hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("F", hero.interfaces.Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("A");
		ps.setNodeTraditional("B");
		ps.setNodeTraditional("C");
		ps.setNodeTraditional("D");
		ps.setNodeTraditional("E");
		ps.setNodeTraditional("F");
		
		ps.setProperty("condition1", "50");	// % to do 1st iteration			
		ps.setProperty("condition2", "50");	// % to do 2nd iteration
		ps.setProperty("randomNum", "0");	// random number who decides if we iterate or not
	
		ps.addEdge("A", "B");
		ps.addEdge("B", "C");
		ps.addEdge("C", "D");
		String fromDtoE = ps.addEdge("D", "E");
		String fromEtoF = ps.addEdge("E", "F");
	
		ps.setEdgeCondition(fromDtoE, "(new Integer(randomNum).intValue() >= new Integer(condition1).intValue())");
		ps.setEdgeCondition(fromEtoF, "(new Integer(randomNum).intValue() >= new Integer(condition2).intValue())");
		String generateRandomScript =
			  "import hero.interfaces.*;\n"
			+ "import java.util.Random; \n"
			+ "beforeTerminate(Object b,Object n) {\n\n\n"
			+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
			+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
			+ "  prjSession.initModel(n.getBnProject().getName()); \n"				
		    + "  Random generator = new Random(System.currentTimeMillis()); \n"
		    + "  int newRnd = generator.nextInt(100); \n"
		    + "  prjSession.setProperty(\"randomNum\", Integer.toString(newRnd)); \n"
			+ "} \n";
		ps.addNodeInterHook("D", "Generates random number and saves it in 'randomNum' var", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, generateRandomScript);			
		ps.addNodeInterHook("E", "Generates random number and saves it in 'randomNum' var", Constants.Nd.BEFORETERMINATE, Constants.Hook.BSINTERACTIVE, generateRandomScript);			

		ps.addIteration("D", "B", "(new Integer(randomNum).intValue() < new Integer(condition1).intValue())");
		ps.addIteration("E", "C", "(new Integer(randomNum).intValue() < new Integer(condition2).intValue())");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("AutomaticArbitraryCycles");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		usr.startActivity(project, "A");
		usr.terminateActivity(project, "A");
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'A' failed", ps.getNodeValue("A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'B' failed", ps.getNodeValue("B").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'C' failed", ps.getNodeValue("C").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'D' failed", ps.getNodeValue("D").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ArbitraryCycles, terminateActivity 'A': 'E' failed", ps.getNodeValue("E").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project, "F");
		usr.terminateActivity(project, "F");
	}
	
	/** 
	 * Tests all constrains related with iteration definition model <br>
	 * All restrictions defined in hero.session.ProjectSessionBean checkModelDefinition() and related with iterations should be tested here <br>
	 * To do that, it forces to throw all the possible Exceptions<br>
	 * <br>
	 * 		****  MultiIterationsLeaving  ****<br>
	 *      *                                *<br>
	 *      *          path=A                *<br>
	 *      *     - - - - - - - -            *<br>
	 *      *     |       path=B |           *<br>
	 *      *     |      - - - - |           *<br>
	 *      *     |      |      ||path=D     *<br>
	 *      *     A----->B----->C----->D     *<br>
	 *      *                  |_|           *<br>
	 *      *                 path=C         *<br>
	 *      *                                *<br>
	 *      **********************************<br>
	 * <br>     
	 * Uses: Non automatic traditional activities<br>
	 *      
	 * @throws Exception
	 */
	public void testIterationsConstrains() throws Exception {
		ProjectSessionHome projectSessionh = ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("MultiIterationsLeaving");

		ps.addNode("A", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("B", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("C", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
		ps.addNode("D", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
	
		ps.setNodeTraditional("A");
		ps.setNodeTraditional("B");
		ps.setNodeTraditional("C");
		ps.setNodeTraditional("D");

		ps.setProperty("path", "A"); // Decides which iteration we have to take			
		
		// ----- hero.session.ProjectSessionBean.addIteration checks: -----
		// - checks if fromIteration node is reachable from toIteration node
		ps.addEdge("A", "B");
		//ps.addEdge("B", "C"); // We don't add this edge to force the Exception
		String fromCtoD = ps.addEdge("C", "D");
		ps.setEdgeCondition(fromCtoD, "path.equals(\"D\")");
		try {
			ps.addIteration("C", "A", "path.equals(\"A\")");
			assertTrue("Error in testIterationsConstrains, it's imposible to have and iteration here", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}
		
		ps.addEdge("B", "C"); // We don't add this edge to force the Exception
		ps.addIteration("C", "A", "path.equals(\"A\")");

		// - checks that there is not another iteration between from-->to nodes. (we cannot define 2 iterations over the same nodes)
		try {
			ps.addIteration("C", "A", "path.equals(\"B\")");
			assertTrue("Error in testIterationsConstrains, we cannot define 2 iterations over the same nodes", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}		
		ps.addIteration("C", "B", "path.equals(\"B\")");
		ps.addIteration("C", "C", "path.equals(\"C\")");

		ps.checkModelDefinition();
		
		// - tests with nodes that doesn't exist
		try {
			ps.addIteration("D", "zzz", "true");
			assertTrue("Error in testIterationsConstrains, cannot create an iteration with nodes that doesn't exit", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}		
		try {
			ps.addIteration("zzz", "D", "true");
			assertTrue("Error in testIterationsConstrains, cannot create an iteration with nodes that doesn't exit", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}		
		// ----- end addIteration checks ------
		
		// ----- hero.session.ProjectSessionBean.addEdge checks: -----
		// - checks if the new edge creates a new cycle (we cannot put an edge from D to A because it creates a cycle)
		try {
			ps.addEdge("C", "D");
			assertTrue("Error in testIterationsConstrains, this edge creates a cycle", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}		
		// ----- end addEdge checks ------
			
		// ----- hero.session.ProjectSessionBean.checkModelDefinition checks: -----
		// - checkIterationConstrains
		// 		· checks if iteration's conditions are not empty (ex1: cond2.equals(""))
		try {
			ps.addIteration("D", "D", "");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, iteration's condition cannot be \"\" (empty)", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
			ps.deleteIteration("D", "D");
		}		

		//		· (NOTE: it is allowed to have "true" as condition -> in order to have an infinite process)
		try {
			ps.addIteration("D", "D", "\"true\"");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, it is posible to have an iteration condition = \"true\"", true);
			ps.deleteIteration("D", "D");
		} catch (Exception e) { // NOT Expected execution
			System.out.println(e);
		}
		
		//		· checks if exists a path between from--->to
		try {
			ps.addIteration("D", "C", "\"true\"");
			ps.deleteEdge(fromCtoD); // delete the edge to force the error (we don't have path from C to D) 
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, it's impossible to have a iteration because these nodes are not connected", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
			ps.deleteIteration("D", "C");
			fromCtoD = ps.addEdge("C", "D");
			ps.setEdgeCondition(fromCtoD, "path.equals(\"D\")");	
		}		
			
		//		· if we have more than one iteration starting in the same node -> checks that the iterations conditions are different (ex: new path condition != path1 condition)
		try {
			ps.addIteration("D", "A", "path.equals(\"A\")");
			ps.addIteration("D", "B", "path.equals(\"A\")");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, iterations conditions must be different", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
			ps.deleteIteration("D", "A");
			ps.deleteIteration("D", "B");
		}		

		// - checkMandatoryIterationConditions
		// 		· checks mandatory conditions over the out edges of the nodes that create and extra exit point from the iteration (ex: condition in edge C--->D)
		// 			throws an HeroException if these edges don't have a condition
		try {
			ps.deleteEdge(fromCtoD); // delete the edge to delete the condition
			fromCtoD = ps.addEdge("C", "D");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, condition in edge C--->D has to exist", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}		

		// 			throws an HeroException if the condition is empty (equals("")) 
		try {
			ps.setEdgeCondition(fromCtoD, "");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, condition in edge C--->D cannot be \"\" (empty)", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}
		
		// 			throws an HeroException if the condition is "true" (equals("true"))
		try {
			ps.deleteEdge(fromCtoD);
			fromCtoD = ps.addEdge("C", "D");
			ps.setEdgeCondition(fromCtoD, "true");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, condition in edge C--->D cannot be \"true\"", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}
				 
		//		· checks if mandatory conditions over the out edges from the node that starts the iteration are different from the iteration's condition (ex: condition in edge C--->D & condition of iteration C--->A)
		// 			throws an HeroException if the iteration's condition and out edge's condition are equal 
		try {
			ps.deleteEdge(fromCtoD);
			fromCtoD = ps.addEdge("C", "D");
			ps.setEdgeCondition(fromCtoD, "path.equals(\"A\")");
			ps.checkModelDefinition();
			assertTrue("Error in testIterationsConstrains, condition in edge C--->D cannot be equal to another iteration condition", false);
		} catch (Exception e) { // Expected execution
			System.out.println(e);
		}
		// ----- end checkModelDefinition checks ------
	}
	
	/**
	 * It tests extra entry points incoming to an iteration<br>
	 * Uses: node2 = AND_JOIN and true conditions<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsSimple1() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsSimple1");

		// Properties
		ps.setProperty("condA", "true");
		ps.setProperty("condB", "true");
		ps.setProperty("condC", "true");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		
		// add Edges
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		String from3to4 = ps.addEdge("node3","node4");

		String fromAto2 = ps.addEdge("extraA","node2");
		String fromBto2 = ps.addEdge("extraB","node2");
		String fromCto2 = ps.addEdge("extraC","node2");

		ps.setEdgeCondition(from3to4, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto2, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto2, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto2, "condC.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node3", "node1", "iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsSimple1");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		usr.startActivity(project,"extraA");		
		usr.startActivity(project,"extraB");		
		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraA");
		usr.terminateActivity(project,"extraB");		
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsSimple1, extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		int index = 0;
		while (index < 2) {
			usr.startActivity(project,"node1");		
			usr.terminateActivity(project,"node1");
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node1: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node1: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node2");		
			usr.terminateActivity(project,"node2");
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node2: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);
	
			usr.startActivity(project,"node3");		
			usr.terminateActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
			index++;
		}
		usr.startActivity(project,"node1");		
		usr.terminateActivity(project,"node1");
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node1: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node1: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node2");		
		usr.terminateActivity(project,"node2");
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node2: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node3");
		ps.setProperty("iterate", "false");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple1, terminateActivity node3: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");
	}
	
	/**
	 * It tests extra entry points incoming to an iteration<br>
	 * Uses: node2 = AND_JOIN and a false condition<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsSimple2() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsSimple2");

		// Properties
		ps.setProperty("condA", "true");
		ps.setProperty("condB", "true");
		ps.setProperty("condC", "false");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.AND_JOIN_NODE);

		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		
		// add Edges
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		String from3to4 = ps.addEdge("node3","node4");

		String fromAto2 = ps.addEdge("extraA","node2");
		String fromBto2 = ps.addEdge("extraB","node2");
		String fromCto2 = ps.addEdge("extraC","node2");

		ps.setEdgeCondition(from3to4, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto2, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto2, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto2, "condC.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node3", "node1", "iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsSimple2");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		usr.startActivity(project,"extraA");		
		usr.startActivity(project,"extraB");		
		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraA");
		usr.terminateActivity(project,"extraB");		
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsSimple2, extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple2, extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple2, extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple2, node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsSimple2, node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsSimple2, node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.DEAD);
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsSimple2, startActivity node1: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		usr.terminateActivity(project,"node1");
	}

	private void incrementAndTestNodeProperty(ProjectSession ps, String node, String prop, int expectedNewValue) throws Exception {
		BnNodePropertyValue propValue = ps.getNodeProperty(node, prop);
		int counter = Integer.parseInt(propValue.getTheValue());
		counter++;
		if (counter != expectedNewValue) {
			throw new Exception("Error in testExtraEntryPointsSimple3, counter in " + node + " should be " + expectedNewValue);
		}
		ps.setNodeProperty(node, prop, Integer.toString(counter));
	}
	
	/**
	 * It tests extra entry points incoming to an iteration<br>
	 * Also tests node properties propagation : when it iterates takes the previous value.<br>
	 *    nodeProperties are not propagated through the iteration<br>
	 * 
	 * Uses: node2 = OR_JOIN and has a true in edge from an extra entry<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsSimple3() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsSimple3");

		// Properties
		ps.setProperty("cond0", "true");
		ps.setProperty("condA", "true");
		ps.setProperty("condB", "false");
		ps.setProperty("condC", "false");
		ps.setProperty("iterate", "true");
		
		// add Nodes
		ps.addNode("initial", Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node0", Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1", Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2", Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node3", Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4", Constants.Nd.AND_JOIN_NODE);
		// extra entry points
		ps.addNode("extraA", Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraB", Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraC", Constants.Nd.OR_JOIN_NODE);

		ps.setNodeTraditional("initial");
		ps.setNodeTraditional("node0");
		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");

		// Node properties
		ps.setNodeProperty("extraA", "counter", "0", true);

		// add Edges
		String from0to1 = ps.addEdge("node0","node1");
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		String from3to4 = ps.addEdge("node3","node4");

		String fromAto2 = ps.addEdge("extraA","node2");
		String fromBto2 = ps.addEdge("extraB","node2");
		String fromCto2 = ps.addEdge("extraC","node2");

		ps.setEdgeCondition(from3to4, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from0to1, "cond0.equals(\"true\")");
		ps.setEdgeCondition(fromAto2, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto2, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto2, "condC.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node3", "node0", "iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsSimple3");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		usr.startActivity(project,"extraB");		
		usr.terminateActivity(project,"extraB");		
		assertTrue("Error in ExtraEntryPointsSimple3, extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);

		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsSimple3, extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		this.incrementAndTestNodeProperty(ps, "extraA", "counter", 1);
		usr.startActivity(project,"extraA");		
		usr.terminateActivity(project,"extraA");
		assertTrue("Error in ExtraEntryPointsSimple3, extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.READY);
		
		int index = 0;
		while (index < 2) {
			this.incrementAndTestNodeProperty(ps, "node2", "counter", 2 + index);
			usr.startActivity(project,"node2");		
			usr.terminateActivity(project,"node2");
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node2: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);
	
			this.incrementAndTestNodeProperty(ps, "node3", "counter", 3 + index);
			usr.startActivity(project,"node3");		
			usr.terminateActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node0 failed", ps.getNodeValue("node0").getState() == Constants.Nd.READY);
			try {
				assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.INITIAL);
				assertTrue("Error in ExtraEntryPointsSimple3, node1 should not be instantiated yet", false);
			} catch (Exception e) {} // Expected execution
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: initial failed", ps.getNodeValue("initial").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
			index++;
		}
		this.incrementAndTestNodeProperty(ps, "node2", "counter", 4);
		usr.startActivity(project,"node2");		
		usr.terminateActivity(project,"node2");
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node2: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node2: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);

		this.incrementAndTestNodeProperty(ps, "node3", "counter", 5);
		usr.startActivity(project,"node3");
		ps.setProperty("iterate", "false");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: initial failed", ps.getNodeValue("initial").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node0 failed", ps.getNodeValue("node0").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.READY);

		this.incrementAndTestNodeProperty(ps, "node4", "counter", 6);
		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");
		
		try {
			this.incrementAndTestNodeProperty(ps, "node0", "counter", 6);
			assertTrue("Error in ExtraEntryPointsSimple3, node node0 : 'counter' nodeProperty it would not have to be defined here", false);
		} catch (Exception e) {} // expected behaviour
		// Execute other activities to finish the process
		usr.startActivity(project,"node0");		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node0: node0 failed", ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node0: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		
		try {
			this.incrementAndTestNodeProperty(ps, "node1", "counter", 7);
			assertTrue("Error in ExtraEntryPointsSimple3, node node1 : 'counter' nodeProperty it would not have to be defined here", false);
		} catch (Exception e) {} // expected behaviour
		usr.startActivity(project,"node1");		
		usr.terminateActivity(project,"node1");
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: initial failed", ps.getNodeValue("initial").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: node0 failed", ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple3, terminateActivity node1: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		
		try {
			this.incrementAndTestNodeProperty(ps, "initial", "counter", 0);
			assertTrue("Error in ExtraEntryPointsSimple3, node initial : 'counter' nodeProperty it would not have to be defined here", false);
		} catch (Exception e) {} // expected behaviour
		usr.startActivity(project,"initial");		
		usr.terminateActivity(project,"initial"); 
	}
	
	/**
	 * It tests extra entry points incoming to an iteration<br>
	 * Uses: node2 = OR_JOIN_AUTOMATIC and has a true condition from an extra entry<br>
	 * 		 in this case it will start node2 automaticly before node1 due to setIterationPathInitial()
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsSimple4() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsSimple4");

		// Properties
		ps.setProperty("condA", "false");
		ps.setProperty("condB", "true");
		ps.setProperty("condC", "false");
		ps.setProperty("cond1", "false");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node1",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.addNode("node2",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.OR_JOIN_NODE);

		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		
		// add Edges
		String from1to2 = ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		String from3to4 = ps.addEdge("node3","node4");
		String fromAto2 = ps.addEdge("extraA","node2");
		String fromBto2 = ps.addEdge("extraB","node2");
		String fromCto2 = ps.addEdge("extraC","node2");

		ps.setEdgeCondition(from1to2, "cond1.equals(\"true\")");
		ps.setEdgeCondition(from3to4, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto2, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto2, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto2, "condC.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node3", "node1", "iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsSimple4");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		assertTrue("Error in ExtraEntryPointsSimple4 node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraC: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraC: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		usr.startActivity(project,"extraA");		
		usr.terminateActivity(project,"extraA");
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraA: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraA: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		usr.startActivity(project,"extraB");		
		usr.terminateActivity(project,"extraB");		
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraB: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraB: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity extraB: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);

		int index = 0;
		while (index < 2) {
			usr.startActivity(project,"node3");		
			usr.terminateActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
			index++;
		}

		usr.startActivity(project,"node3");
		ps.setProperty("iterate", "false");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node3 failed", ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: node4 failed", ps.getNodeValue("node4").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraB failed", ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple4, terminateActivity node3: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");
	}
	
	/**
	 * It tests extra entry points incoming to an iteration<br>
	 * Uses: node2 = OR_JOIN and all the conditions are false<br>
	 * 		 it propagates a CANCEL
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsSimple5() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsSimple5");

		// Properties
		ps.setProperty("condA", "false");
		ps.setProperty("condB", "false");
		ps.setProperty("condC", "false");
		ps.setProperty("cond1", "false");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node1",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.addNode("node2",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.OR_JOIN_NODE);

		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		
		// add Edges
		String from1to2 = ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		String from3to4 = ps.addEdge("node3","node4");
		String fromAto2 = ps.addEdge("extraA","node2");
		String fromBto2 = ps.addEdge("extraB","node2");
		String fromCto2 = ps.addEdge("extraC","node2");

		ps.setEdgeCondition(from1to2, "cond1.equals(\"true\")");
		ps.setEdgeCondition(from3to4, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto2, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto2, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto2, "condC.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node3", "node1", "iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsSimple5");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		
		assertTrue("Error in ExtraEntryPointsSimple5 node1 failed", ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsSimple5, terminateActivity extraC: extraC failed", ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple5, terminateActivity extraC: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		usr.startActivity(project,"extraA");		
		usr.terminateActivity(project,"extraA");
		assertTrue("Error in ExtraEntryPointsSimple5, terminateActivity extraA: extraA failed", ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsSimple5, terminateActivity extraA: node2 failed", ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		
		usr.startActivity(project,"extraB");		
		usr.terminateActivity(project,"extraB");
	}
	
	/**
	 * Based on testFinalizeIteration() (iterates 3 times and then finalizes the iteration)<br>
	 * Now with more entry points. Normal behaviour.<br>
	 * Uses: Automatic and non automatic traditional activities<br>
	 *       Extra entry points = AND_JOIN (node3 & node5B)<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsBig1() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsBig1");

		// Properties
		ps.setProperty("condA", "true");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5A",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5B",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraA0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);

		ps.setNodeTraditional("node0");
		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("node5");
		ps.setNodeTraditional("node5A");
		ps.setNodeTraditional("node5B");
		ps.setNodeTraditional("node6");
		ps.setNodeTraditional("node7");
		ps.setNodeTraditional("node8");
		ps.setNodeTraditional("node9");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		ps.setNodeTraditional("extraA0");
		ps.setNodeTraditional("extraB0");
		ps.setNodeTraditional("extraB1");
		ps.setNodeTraditional("extraC0");
		ps.setNodeTraditional("extraC1");
		ps.setNodeTraditional("extraC2");
		ps.setNodeTraditional("extraZ0");
		ps.setNodeTraditional("extraZ1");
		ps.setNodeTraditional("extraZ2");
		
		// add Edges
		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		ps.addEdge("node5","node5A");
		ps.addEdge("node5A","node5B");
		ps.addEdge("node5B","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");

		String fromAto3 = ps.addEdge("extraA","node3");
		ps.addEdge("extraB","node3");
		ps.addEdge("extraC","node3");
		ps.addEdge("extraA0","extraA");
		ps.addEdge("extraB0","extraB");
		ps.addEdge("extraB1","extraB");
		ps.addEdge("extraC0","extraC");
		ps.addEdge("extraC1","extraC");
		ps.addEdge("extraC2","extraC");
		ps.addEdge("extraZ0","node5B");
		ps.addEdge("extraZ1","node5B");
		ps.addEdge("extraZ2","node5B");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto3, "condA.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsBig1");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		assertTrue("Error in ExtraEntryPointsBig1, extraA0 failed",ps.getNodeValue("extraA0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraB0 failed",ps.getNodeValue("extraB0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraB1 failed",ps.getNodeValue("extraB1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraC0 failed",ps.getNodeValue("extraC0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraC1 failed",ps.getNodeValue("extraC1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraC2 failed",ps.getNodeValue("extraC2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraZ0 failed",ps.getNodeValue("extraZ0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraZ1 failed",ps.getNodeValue("extraZ1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraZ2 failed",ps.getNodeValue("extraZ2").getState() == Constants.Nd.TERMINATED);
		
		usr.startActivity(project,"extraA");		
		usr.startActivity(project,"extraB");		
		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraC");		
		usr.terminateActivity(project,"extraB");		
		usr.terminateActivity(project,"extraA");		
		assertTrue("Error in ExtraEntryPointsBig1, extraA failed",ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraB failed",ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, extraC failed",ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		
		usr.startActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.INITIAL);
		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		
		int index = 0;
		while (index < 2)	{
			usr.startActivity(project,"node1");		
			assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node3");
			usr.terminateActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);

			usr.startActivity(project,"node5");
			usr.terminateActivity(project,"node5");
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node4");
			usr.terminateActivity(project,"node4");			
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);

			try{
				usr.startActivity(project, "node2");
				assertTrue("Error in ExtraEntryPointsBig1, startActivity node2: node2 should be failed",false);	
			}catch(Exception e){}
			try{
				usr.terminateActivity(project, "node2");
				assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node2: node2 should be failed",false);	
			}catch(Exception e){}

			usr.startActivity(project,"node5A");
			usr.terminateActivity(project,"node5A");
			
			usr.startActivity(project,"node5B");
			usr.terminateActivity(project,"node5B");		
			
			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");			
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node5A failed",ps.getNodeValue("node5A").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraA failed",ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraB failed",ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraC failed",ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraA0 failed",ps.getNodeValue("extraA0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraB0 failed",ps.getNodeValue("extraB0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraB1 failed",ps.getNodeValue("extraB1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraC0 failed",ps.getNodeValue("extraC0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraC1 failed",ps.getNodeValue("extraC1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraC2 failed",ps.getNodeValue("extraC2").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraZ0 failed",ps.getNodeValue("extraZ0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraZ1 failed",ps.getNodeValue("extraZ1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6: extraZ2 failed",ps.getNodeValue("extraZ2").getState() == Constants.Nd.TERMINATED);
			
			index++;
		}
		usr.startActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig1, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
		
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node3");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node5");
		usr.terminateActivity(project,"node5");
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");			
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node5A");
		usr.terminateActivity(project,"node5A");
		
		usr.startActivity(project,"node5B");
		usr.terminateActivity(project,"node5B");		
		
		// Finally, it will finalize throw node7.  Node2, node8 and node9 will become DEAD
		ps.setProperty("iterate","false");

		usr.startActivity(project,"node6");
		usr.terminateActivity(project,"node6");			
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig1, terminateActivity node6, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node7");
		usr.terminateActivity(project,"node7");			
	}

	/**
	 * Based on testExtraEntryPointsBig1()<br>
	 * Now extraA node has a false condition that CANCELS some nodes inside the iteration<br>
	 * Uses: Automatic and non automatic traditional activities<br>
	 *       Extra entry points : node3 = AND_JOIN_AUTOMATIC -- node5B = AND_JOIN<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsBig2() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsBig2");

		// Properties
		ps.setProperty("condA", "false");
		ps.setProperty("iterate", "true");

		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5A",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5B",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraA0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);

		ps.setNodeTraditional("node0");
		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("node5");
		ps.setNodeTraditional("node5A");
		ps.setNodeTraditional("node5B");
		ps.setNodeTraditional("node6");
		ps.setNodeTraditional("node7");
		ps.setNodeTraditional("node8");
		ps.setNodeTraditional("node9");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		ps.setNodeTraditional("extraA0");
		ps.setNodeTraditional("extraB0");
		ps.setNodeTraditional("extraB1");
		ps.setNodeTraditional("extraC0");
		ps.setNodeTraditional("extraC1");
		ps.setNodeTraditional("extraC2");
		ps.setNodeTraditional("extraZ0");
		ps.setNodeTraditional("extraZ1");
		ps.setNodeTraditional("extraZ2");
		
		// add Edges
		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		ps.addEdge("node5","node5A");
		ps.addEdge("node5A","node5B");
		ps.addEdge("node5B","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");

		String fromAto3 = ps.addEdge("extraA","node3");
		ps.addEdge("extraB","node3");
		ps.addEdge("extraC","node3");
		ps.addEdge("extraA0","extraA");
		ps.addEdge("extraB0","extraB");
		ps.addEdge("extraB1","extraB");
		ps.addEdge("extraC0","extraC");
		ps.addEdge("extraC1","extraC");
		ps.addEdge("extraC2","extraC");
		ps.addEdge("extraZ0","node5B");
		ps.addEdge("extraZ1","node5B");
		ps.addEdge("extraZ2","node5B");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto3, "condA.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsBig2");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		assertTrue("Error in ExtraEntryPointsBig2, extraA0 failed",ps.getNodeValue("extraA0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraB0 failed",ps.getNodeValue("extraB0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraB1 failed",ps.getNodeValue("extraB1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraC0 failed",ps.getNodeValue("extraC0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraC1 failed",ps.getNodeValue("extraC1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraC2 failed",ps.getNodeValue("extraC2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraZ0 failed",ps.getNodeValue("extraZ0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraZ1 failed",ps.getNodeValue("extraZ1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, extraZ2 failed",ps.getNodeValue("extraZ2").getState() == Constants.Nd.TERMINATED);
		
		usr.startActivity(project,"extraA");		
		usr.terminateActivity(project,"extraA");		
		assertTrue("Error in ExtraEntryPointsBig2, extraA failed",ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"extraB");	
		usr.terminateActivity(project,"extraB");		
		assertTrue("Error in ExtraEntryPointsBig2, extraB failed",ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsBig2, extraC failed",ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);

		usr.startActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.INITIAL);
	
		usr.terminateActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig2, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
		
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
		
		try {
			usr.startActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsBig2, startActivity node3 is not possible", false);
		} catch(Exception e) {};
		try {
			usr.terminateActivity(project,"node3");
			assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node3 is not possible", false);
		} catch(Exception e) {};
		
		assertTrue("Error in ExtraEntryPointsBig2, node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node5");
		usr.terminateActivity(project,"node5");
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");			
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node6, after iterate: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node6, after iterate: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node5A");
		usr.terminateActivity(project,"node5A");
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node5A: node5A failed",ps.getNodeValue("node5A").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node5A: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node5B");
		usr.terminateActivity(project,"node5B");
		
		try {
			usr.startActivity(project,"node6");
			assertTrue("Error in ExtraEntryPointsBig2, startActivity node6 is not possible", false);
		} catch(Exception e) {};
		try {
			usr.terminateActivity(project,"node6");			
			assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node6 is not possible", false);
		} catch(Exception e) {};
		try {
			usr.startActivity(project,"node7");
			assertTrue("Error in ExtraEntryPointsBig2, startActivity node7 is not possible", false);
		} catch(Exception e) {};
		try {
			usr.terminateActivity(project,"node7");			
			assertTrue("Error in ExtraEntryPointsBig2, terminateActivity node7 is not possible", false);
		} catch(Exception e) {};		
	}
	
	/**
	 * Based on testExtraEntryPointsBig1() (iterates 3 times and then finalizes the iteration)<br>
	 * Uses: Automatic and non automatic traditional activities<br>
	 *       Extra entry points : node3 = OR_JOIN -- node5B = OR_JOIN_AUTOMATIC<br>
	 * Now: it will iterate extraB will be true. Edges comming from extraA, extraC, node1 will be false<br>
	 * 		extra5B will start before extra5A because extraZ2 will be executed before extra5A<br>
	 * 
	 * @throws Exception
	 */
	public void testExtraEntryPointsBig3() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ExtraEntryPointsBig3");

		// Properties
		ps.setProperty("iterate", "true");
		ps.setProperty("condA", "false");
		ps.setProperty("condB", "true");
		ps.setProperty("condC", "false");
		ps.setProperty("condZ0", "false");
		ps.setProperty("condZ1", "false");
		ps.setProperty("condZ2", "true");
		ps.setProperty("cond1", "false");

		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.OR_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5A",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node5B",Constants.Nd.OR_JOIN_AUTOMATIC_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		// extra entry points
		ps.addNode("extraA",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraB",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraC",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("extraA0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraB1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraC2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ0",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ1",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.addNode("extraZ2",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);

		ps.setNodeTraditional("node0");
		ps.setNodeTraditional("node1");
		ps.setNodeTraditional("node2");
		ps.setNodeTraditional("node3");
		ps.setNodeTraditional("node4");
		ps.setNodeTraditional("node5");
		ps.setNodeTraditional("node5A");
		ps.setNodeTraditional("node5B");
		ps.setNodeTraditional("node6");
		ps.setNodeTraditional("node7");
		ps.setNodeTraditional("node8");
		ps.setNodeTraditional("node9");
		ps.setNodeTraditional("extraA");
		ps.setNodeTraditional("extraB");
		ps.setNodeTraditional("extraC");
		ps.setNodeTraditional("extraA0");
		ps.setNodeTraditional("extraB0");
		ps.setNodeTraditional("extraB1");
		ps.setNodeTraditional("extraC0");
		ps.setNodeTraditional("extraC1");
		ps.setNodeTraditional("extraC2");
		ps.setNodeTraditional("extraZ0");
		ps.setNodeTraditional("extraZ1");
		ps.setNodeTraditional("extraZ2");
		
		// add Edges
		ps.addEdge("node0","node1");
		String from1to3 = ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		String from3to6 = ps.addEdge("node3","node6");
		String from3to8 = ps.addEdge("node3","node8");
		String from4to2 = ps.addEdge("node4","node2");
		String from4to6 = ps.addEdge("node4","node6");
		ps.addEdge("node5","node4");
		ps.addEdge("node5","node6");
		ps.addEdge("node5","node5A");
		ps.addEdge("node5A","node5B");
		ps.addEdge("node5B","node6");
		String from6to7 = ps.addEdge("node6","node7");
		ps.addEdge("node8","node9");

		String fromAto3 = ps.addEdge("extraA","node3");
		String fromBto3 = ps.addEdge("extraB","node3");
		String fromCto3 = ps.addEdge("extraC","node3");
		ps.addEdge("extraA0","extraA");
		ps.addEdge("extraB0","extraB");
		ps.addEdge("extraB1","extraB");
		ps.addEdge("extraC0","extraC");
		ps.addEdge("extraC1","extraC");
		ps.addEdge("extraC2","extraC");
		String fromZ0to5B = ps.addEdge("extraZ0","node5B");
		String fromZ1to5B = ps.addEdge("extraZ1","node5B");
		String fromZ2to5B = ps.addEdge("extraZ2","node5B");

		// set conditions to exit from the iteration (they are mandatory)
		ps.setEdgeCondition(from3to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from4to6, "iterate.equals(\"true\")");
		ps.setEdgeCondition(from3to8, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from4to2, "iterate.equals(\"false\")");
		ps.setEdgeCondition(from6to7, "iterate.equals(\"false\")");
		ps.setEdgeCondition(fromAto3, "condA.equals(\"true\")");
		ps.setEdgeCondition(fromBto3, "condB.equals(\"true\")");
		ps.setEdgeCondition(fromCto3, "condC.equals(\"true\")");
		ps.setEdgeCondition(fromZ0to5B, "condZ0.equals(\"true\")");
		ps.setEdgeCondition(fromZ1to5B, "condZ1.equals(\"true\")");
		ps.setEdgeCondition(fromZ2to5B, "condZ2.equals(\"true\")");
		ps.setEdgeCondition(from1to3, "cond1.equals(\"true\")");
		
		// set Iteration
		ps.addIteration("node6","node1","iterate.equals(\"true\")");
		
		// checks if model is defined correctly
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		ps.setUserRole("admin","InitialRole");
		String project = ps.instantiateProject("ExtraEntryPointsBig3");
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();

		assertTrue("Error in ExtraEntryPointsBig3, extraA0 failed",ps.getNodeValue("extraA0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraB0 failed",ps.getNodeValue("extraB0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraB1 failed",ps.getNodeValue("extraB1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraC0 failed",ps.getNodeValue("extraC0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraC1 failed",ps.getNodeValue("extraC1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraC2 failed",ps.getNodeValue("extraC2").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraZ0 failed",ps.getNodeValue("extraZ0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraZ1 failed",ps.getNodeValue("extraZ1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraZ2 failed",ps.getNodeValue("extraZ2").getState() == Constants.Nd.TERMINATED);
		// Executed before node5A because it is an OR_JOIN_AUTOMATIC and condZ2 is true
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.TERMINATED);
		
		usr.startActivity(project,"extraA");		
		usr.startActivity(project,"extraB");		
		usr.startActivity(project,"extraC");
		usr.terminateActivity(project,"extraA");		
		usr.terminateActivity(project,"extraB");		
		usr.terminateActivity(project,"extraC");		
		assertTrue("Error in ExtraEntryPointsBig3, extraA failed",ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraB failed",ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, extraC failed",ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.INITIAL);
		
		usr.terminateActivity(project,"node0");
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node0: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node0: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		
		int index = 0;
		while (index < 2)	{
			assertTrue("Error in ExtraEntryPointsBig3, node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);		

			// Executed before node1 because it is an OR_JOIN and condB is true
			usr.startActivity(project,"node3");		
			usr.terminateActivity(project,"node3");		
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);		
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);
			
			usr.startActivity(project,"node1");		
			assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
			assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			
			usr.terminateActivity(project,"node1");		
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
						
			try {
				usr.startActivity(project,"node3");
				assertTrue("Error in ExtraEntryPointsBig3, startActivity node3 is not possible", false);
			} catch(Exception e) {};
			try {
				usr.terminateActivity(project,"node3");;			
				assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3 is not possible", false);
			} catch(Exception e) {};
			
			usr.startActivity(project,"node5");
			usr.terminateActivity(project,"node5");
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node5: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node5: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);
			
			usr.startActivity(project,"node4");
			usr.terminateActivity(project,"node4");			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.READY);

			try{
				usr.startActivity(project, "node2");
				assertTrue("Error in ExtraEntryPointsBig3, startActivity node2: node2 should be failed",false);	
			}catch(Exception e){}
			try{
				usr.terminateActivity(project, "node2");
				assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node2: node2 should be failed",false);	
			}catch(Exception e){}

			assertTrue("Error in ExtraEntryPointsBig3, before startActivity node5A: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.TERMINATED);			
			usr.startActivity(project,"node5A");
			usr.terminateActivity(project,"node5A");
			assertTrue("Error in ExtraEntryPointsBig3, after terminateActivity node5A: node5A failed",ps.getNodeValue("node5A").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig3, after terminateActivity node5A: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.TERMINATED);			
		
			usr.startActivity(project,"node6");
			usr.terminateActivity(project,"node6");			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node0 failed",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.READY);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node5A failed",ps.getNodeValue("node5A").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.TERMINATED); // It is an OR_JOIN_AUTOMATIC and has a true edge incoming
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.INITIAL);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraA failed",ps.getNodeValue("extraA").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraB failed",ps.getNodeValue("extraB").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraC failed",ps.getNodeValue("extraC").getState() == Constants.Nd.TERMINATED);			
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraA0 failed",ps.getNodeValue("extraA0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraB0 failed",ps.getNodeValue("extraB0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraB1 failed",ps.getNodeValue("extraB1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraC0 failed",ps.getNodeValue("extraC0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraC1 failed",ps.getNodeValue("extraC1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraC2 failed",ps.getNodeValue("extraC2").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraZ0 failed",ps.getNodeValue("extraZ0").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraZ1 failed",ps.getNodeValue("extraZ1").getState() == Constants.Nd.TERMINATED);
			assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: extraZ2 failed",ps.getNodeValue("extraZ2").getState() == Constants.Nd.TERMINATED);
			
			index++;
		}
		
		usr.startActivity(project,"node3");
		usr.terminateActivity(project,"node3");
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3: node8 failed",ps.getNodeValue("node8").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node3: node9 failed",ps.getNodeValue("node9").getState() == Constants.Nd.DEAD);

		usr.startActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig3, startActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
		
		usr.terminateActivity(project,"node1");		
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node1 failed",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);			
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node3 failed",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node5 failed",ps.getNodeValue("node5").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node5");
		usr.terminateActivity(project,"node5");
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node1: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.READY);

		usr.startActivity(project,"node4");
		usr.terminateActivity(project,"node4");			
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node4 failed",ps.getNodeValue("node4").getState() == Constants.Nd.TERMINATED);			
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node2 failed",ps.getNodeValue("node2").getState() == Constants.Nd.DEAD);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node4: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.READY);

		// Finally, it will finalize throw node7.  Node2, node8 and node9 will become DEAD
		ps.setProperty("iterate","false");

		usr.startActivity(project,"node6");
		usr.terminateActivity(project,"node6");			
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node6 failed",ps.getNodeValue("node6").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in ExtraEntryPointsBig3, terminateActivity node6: node7 failed",ps.getNodeValue("node7").getState() == Constants.Nd.READY);
		
		usr.startActivity(project,"node7");
		usr.terminateActivity(project,"node7");		
		
		assertTrue("Error in ExtraEntryPointsBig3, before startActivity node5A: node5B failed",ps.getNodeValue("node5B").getState() == Constants.Nd.TERMINATED);			
		usr.startActivity(project,"node5A");
		usr.terminateActivity(project,"node5A");
	}
	
	
	// ---------------------------------------------------------------------- //
	// -------------------------   SUBPROCESSES   --------------------------- //
	// ---------------------------------------------------------------------- //

	/**
	 * Tests that an iteration with a subprocess inside: node->subProc->node
	 * Uses: Non automatic anticipable activities
	 * 
	 * @throws Exception
	 */
	public void testSubProcessesIteration() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		
		ProjectSession ps1 = projectSessionh.create();
		ps1.initModel("oneSubProject");
		ps1.addNode("test1",Constants.Nd.AND_JOIN_NODE);
		ps1.addNode("test2",Constants.Nd.AND_JOIN_NODE);
		ps1.addEdge("test1","test2");
		
		ProjectSession ps = projectSessionh.create();
		ps.initModel("oneTestProject");
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNodeSubProcess("node2","oneSubProject");
				
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		
		ps.setNodeProperty("node1","prop","ok",true);
		ps.addRoleMapper("InitialRole","roleMapperTest",Constants.Mapper.PROPERTIES);
		
		ps.addIteration("node3","node1","prop.equals(\"ok\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		String instName = ps.instantiateProject("oneTestProject");
		UserSessionHome userSessionh = UserSessionUtil.getHome();
		UserSession us = userSessionh.create();
		assertTrue("Error in roleMapper",us.getToDoList(instName).contains("node1"));
		int index = 0;
		String actName=null;
		while (index<3)
		{
			us.startActivity(instName,"node1");
			us.terminateActivity(instName,"node1");
			Collection inst = us.getProjectInstancesNames("oneSubProject");
			Iterator instances = inst.iterator();
			actName = (String)instances.next();			
			assertTrue("Error in Iteration, subProcess is not reinitialized",us.getNode(actName,"test1").getState()==Constants.Nd.READY);
			
			us.startActivity(actName,"test1");
			us.terminateActivity(actName,"test1");
			us.startActivity(actName,"test2");
			us.terminateActivity(actName,"test2");
							
			us.startActivity(instName,"node3");
			us.terminateActivity(instName,"node3");
			assertTrue("Error in Iteration",us.getToDoList(instName).contains("node1"));
			assertTrue("Error in Iteration, subProcess is not reinitialized",us.getToDoList(actName).isEmpty());
			assertTrue("Error in Iteration, subProcess is not reinitialized",us.getNode(actName,"test1").getState()==Constants.Nd.INITIAL);
			index++;
		}
		
		us.startActivity(instName,"node1");
		us.terminateActivity(instName,"node1");
		Collection inst = us.getProjectInstancesNames("oneSubProject");
		Iterator instances = inst.iterator();
		actName = (String)instances.next();			
		
		us.startActivity(actName,"test1");
		us.terminateActivity(actName,"test1");
		us.startActivity(actName,"test2");
		us.terminateActivity(actName,"test2");
			
		us.startActivity(instName,"node3");
		
		ps.setNodeProperty("node3","prop","nok",true);
		
		us.terminateActivity(instName,"node3");
		try {
			us.getToDoList(instName);
		    assertTrue("Should have throw FinderException",false);
		} catch (Exception e) {
		    //expected behavior
		}
		try {
			us.getToDoList(actName);
		    assertTrue("Should have throw FinderException",false);
		} catch (Exception e) {
		    //expected behavior
		}
	}
	
	/**
	 * Tests iterations between 2 subprocesses
	 * Uses: Non automatic anticipable activities
	 * 
	 * @throws Exception
	 */
	public void testSubProcessesIterationOnlySubProcesses() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		 
		ProjectSession ps1 = projectSessionh.create();
		ps1.initModel("SubProject1");
		ps1.addNode("test1",Constants.Nd.AND_JOIN_NODE);
		ps1.addNode("test2",Constants.Nd.AND_JOIN_NODE);
		ps1.addEdge("test1","test2");
		
		ProjectSession ps2 = projectSessionh.create();
		ps2.initModel("SubProject2");
		ps2.addNode("test1",Constants.Nd.AND_JOIN_NODE);
		ps2.addNode("test2",Constants.Nd.AND_JOIN_NODE);
		ps2.addEdge("test1","test2");
		
		ProjectSession ps = projectSessionh.create();
		ps.initModel("MyProjectIte");
		ps.addNodeSubProcess("node1","SubProject1");
		ps.addNodeSubProcess("node2bis","SubProject2");
		
		ps.addEdge("node1","node2bis");
		
		ps.setNodeProperty("node1","prop","ok",true);
		ps.addRoleMapper("InitialRole","roleMapperTest",Constants.Mapper.PROPERTIES);
		
		ps.addIteration("node2bis","node1","prop.equals(\"ok\")");
		
		ps.checkModelDefinition();

		// ******************************   MAIN   ******************************
		String instName = ps.instantiateProject("MyProjectIte");
		UserSessionHome userSessionh = UserSessionUtil.getHome();
		UserSession us = userSessionh.create();
		
		Collection inst = us.getProjectInstancesNames("SubProject1");
		Iterator instances = inst.iterator();
		String actName = (String)instances.next();	
				
		int index = 0;
		while (index<3)
		{
			us.startActivity(actName,"test1");
			us.terminateActivity(actName,"test1");
			us.startActivity(actName,"test2");
			us.terminateActivity(actName,"test2");

			inst = us.getProjectInstancesNames("SubProject2");
			instances = inst.iterator();
			actName = (String)instances.next();
			
			us.startActivity(actName,"test1");
			us.terminateActivity(actName,"test1");
			us.startActivity(actName,"test2");
			us.terminateActivity(actName,"test2");
									
			inst = us.getProjectInstancesNames("SubProject1");
			instances = inst.iterator();
			actName = (String)instances.next();
			
			index++;
		}
		
		us.startActivity(actName,"test1");
		us.terminateActivity(actName,"test1");
		us.startActivity(actName,"test2");
		us.terminateActivity(actName,"test2");
					
		inst = us.getProjectInstancesNames("SubProject2");
		instances = inst.iterator();
		String actName2 = (String)instances.next();
		
		us.startActivity(actName2,"test1");
		us.terminateActivity(actName2,"test1");
		us.startActivity(actName2,"test2");
		
		ps.initProject(actName2); // Special case for subprocess iteration
		ps.setProperty("prop","nok");
		us.terminateActivity(actName2,"test2");
		
		assertTrue("Error in SubProcess 1",us.getProjectInstancesNames("SubProject1").size()==0);
		assertTrue("Error in SubProcess 2",us.getProjectInstancesNames("SubProject2").size()==0);
		assertTrue("Error in SubProcess 3",us.getProjectInstancesNames("MyProjectIte").size()==0);
	    	
								
	}
	
}
