package hero.client.test;

/*
*
* NodeTests.java - 
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnRoleMapperValue;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectHome;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnUser;
import hero.interfaces.BnUserHome;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.util.DuplicatedEdgeException;
import hero.util.EventConstants;
import hero.util.HeroException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;

public class ProjectSessionTests extends TestCase {
    private BnProjectHome projecth;
    private ProjectSessionHome projectSessionh;

    public ProjectSessionTests(String testname) {
	super(testname);
    }
    
    public static TestSuite suite() {
	return new TestSuite(ProjectSessionTests.class);
    }

    public void setUp() throws Exception {
    }

	public void testCreateProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsession");
	BnProjectValue pv=lProject.getDetails();
	BnUserLightValue[] uv=pv.getBnUsers();
	int i=0;
	while (i<uv.length) {
	    if (uv[i].getName().equals("admin"))
		break;
	    i++;
	}
	assertTrue("Admin is in the projet",i<uv.length);
    }

    public void testCloneProjectSession() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectsource");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");
	lProject = lHome.create();
	lProject.initProject("projectsource","tralara");	
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==3);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("tralara"));	            
    }
    
   public void testImportProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectS");
	lProject.addNode("nclone",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c1",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("c2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone","admin");
	lProject.setNodeRole("c1","admin");
	lProject.setNodeRole("c2","admin");
	String edge1=lProject.addEdge("c1","c2");

	lProject = lHome.create();
	lProject.initProject("pDest");
	
	lProject.addNode("nclone2",Constants.Nd.AND_JOIN_NODE);
	lProject.setNodeRole("nclone2","admin");
	lProject.importProject("projectS", EventConstants.DEFAULTPROJECTVERSION, "pDest");
	BnProjectValue pv=lProject.getDetails();
	assertTrue("Clone failure1",pv.getBnNodes().length==4);	  
	assertTrue("Clone failure2",pv.getName().equalsIgnoreCase("pDest"));	            
    }

    public void testAddUserToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectadduser");	
	BnUserHome uHome = BnUserUtil.getHome();
	BnUserValue pdata=new BnUserValue();
	pdata.setName("bidochon");
	pdata.setPassword("bidochon");
	pdata.setEmail("bidochon@loria.fr");
	BnUser lUser = uHome.create(pdata);
	lProject.addUser("bidochon");
	pdata.setName("marcel");
	pdata.setPassword("marcel");
	pdata.setEmail("marcel@loria.fr");
	lUser = uHome.create(pdata);
	lProject.addUser("marcel");
	pdata.setName("germaine");
	pdata.setPassword("germaine");
	pdata.setEmail("germaine@loria.fr");
	lUser = uHome.create(pdata);
	lProject.addUser("germaine");
	assertTrue(lProject.containsUser("bidochon"));
    }

    public void testAddNodeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddnode");	
	lProject.addNode("thenode",1);
	BnNodeValue nv=lProject.getNodeValue("thenode");
	assertTrue("Creator not correctly set",nv.getCreator().equals("admin"));
    }

    public void testAddEdgeToProject() throws Exception {
	ProjectSessionHome lHome=ProjectSessionUtil.getHome();
	ProjectSession lProject = lHome.create();
	lProject.initProject("projectaddedge");		
	lProject.addNode("thenode",Constants.Nd.AND_JOIN_NODE);
	lProject.addNode("thenode2",Constants.Nd.AND_JOIN_NODE);
	String ename=lProject.addEdge("thenode","thenode2");
	BnEdgeValue ev=lProject.getEdgeValue(ename);
	BnNodeValue nv=lProject.getNodeValue("thenode2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("bad node state test3",nv.getState()==Constants.Nd.INITIAL);    }

    public void testProjectSessionAddUser() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst");		

	try {
	    pss.addUser("badUser");
	    assertTrue("Should have throw EJBException",false);
	} catch (Exception e) {
	    //expected behavior
	}
    }	

    public void testProjectSessionAddNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("pst2");	
	pss.addNode("test",Constants.Nd.AND_JOIN_NODE);
	BnNodeValue pv=pss.getNodeValue("test");

	assertTrue("Creator not correctly set",pv.getCreator().equals("admin"));
    }

    public void testProjectSessionAddEdgesState() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("w");		

	pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	String edname=pss.addEdge("node1","node2");
	BnEdgeValue ev=pss.getEdgeValue(edname);
	BnNodeValue nv=pss.getNodeValue("node2");
	assertTrue("addEdge failed a1",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a1/n2",nv.getState()==Constants.Nd.INITIAL);
	edname=pss.addEdge("node2","node3");
	ev=pss.getEdgeValue(edname);
	BnNodeValue nv2=pss.getNodeValue("node2");
	BnNodeValue nv3=pss.getNodeValue("node3");
	assertTrue("addEdge failed a2",ev.getState()==Constants.Ed.INITIAL);
	assertTrue("addEdge failed a2/nv2",nv2.getState()==Constants.Nd.INITIAL);
	assertTrue("addEdge failed a2/nv3",nv3.getState()==Constants.Nd.INITIAL);
    }

    public void testcheckDuplicateEdge() throws Exception {
    
	try {
	    ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss = projectSessionh.create();
		pss.initProject("wde");		    
	    
	    pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	    pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node2");
	    
	    pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node2","node3");
	    
	    pss.addNode("node4",Constants.Nd.AND_JOIN_NODE);
	    pss.addEdge("node1","node4");
	    
	    pss.addEdge("node1","node2");
	    assertTrue("Should have thrown a DuplicatedEdgeException",false);
	}catch(DuplicatedEdgeException e){}
    }

    public void testDynamicProject() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n1","admin");

	UserSession usr=usersh.create();

	usr.startActivity("dynamic","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	String edge2=pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getNodeValue("n3").getState()==Constants.Nd.INITIAL);

	usr.startActivity("dynamic","n2");

	assertTrue("dynamic change failed - n1.1",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.1",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

	pss.deleteEdge(edge2);

	assertTrue("dynamic change failed - n1.2",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.2",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.2",pss.getNodeValue("n3").getState()==Constants.Nd.READY);

	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1.3",pss.getNodeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.3",pss.getNodeValue("n2").getState()==Constants.Nd.ANTICIPATING);
	assertTrue("dynamic change failed - n3.3",pss.getNodeValue("n3").getState()==Constants.Nd.ANTICIPABLE);

    }
	
    public void TestDynamicProject2() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();

	ProjectSession pss = projectSessionh.create();
	pss.initProject("dynamic2");	

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	
	UserSession usr=usersh.create();

	usr.startActivity("dynamic2","n1");

	pss.addNode("n3",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n2","n3");

	assertTrue("dynamic change failed - n1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2",pss.getEdgeValue("n2").getState()==Constants.Nd.ANTICIPABLE);
	assertTrue("dynamic change failed - n3",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);

	pss.addNode("n4",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("n1","n4");
	pss.deleteEdge(edge1);

	assertTrue("dynamic change failed - n1.1",pss.getEdgeValue("n1").getState()==Constants.Nd.EXECUTING);
	assertTrue("dynamic change failed - n2.1",pss.getEdgeValue("n2").getState()==Constants.Nd.READY);
	assertTrue("dynamic change failed - n3.1",pss.getEdgeValue("n3").getState()==Constants.Nd.INITIAL);
	assertTrue("dynamic change failed - n4.1",pss.getEdgeValue("n4").getState()==Constants.Nd.ANTICIPABLE);
    }

    public void testDeleteNode() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deleteNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");

	try {
	    pss.deleteNode("n2");
	} catch (HeroException he) { 
	    assertTrue("Deletion Failed",false); 
	}
	
	UserSession usr=usersh.create();

	usr.startActivity("deleteNode","n1");
	try {
	    pss.deleteNode("n1");
	    assertTrue("Deletion Should have Failed",false); 
	} catch (HeroException he) { // Normal Behavior
	}
    }	


    public void testPropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.addNode("n1",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n2",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n1","n2");
	pss.setNodeRole("n1","admin");
	pss.setNodeRole("n2","admin");
	pss.setNodeProperty("n1","kpn1","vpn1");
	pss.setNodeProperty("n2","kpn2","vpn2");
	pss.setNodeProperty("n2","kpn2","vpn22");
	
	pss.deleteNode("n2");
	
    }	

    public void testDeletePropertyNode() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.deleteNodeProperty("n1","kpn1");
	
    }	

    public void testPropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("maintenant","ara");
	pss.setProperty("deja","ahora");
	pss.setProperty("deja","ja");
	
    }	
    public void testDeletePropertyProject() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.deleteProperty("maintenant");	
    }	


    public void testNodePropertyPropagate() throws Exception {
	UserSessionHome usersh=UserSessionUtil.getHome();
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("n5",Constants.Nd.AND_JOIN_NODE);
	pss.addNode("n6",Constants.Nd.AND_JOIN_NODE);
	String edge1=pss.addEdge("n5","n6");
	pss.setNodeRole("n5","admin");
	pss.setNodeRole("n6","admin");
	pss.setNodeProperty("n5","value","15");
	pss.setNodeProperty("n5","value2","10");
	pss.setNodeProperty("n6","value","25");

	UserSession usr=usersh.create();
	usr.startActivity("propertyNode","n5");
	usr.terminateActivity("propertyNode","n5");
    }

  public void testGetPropertiesNode() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");	

	pss.addNode("p1",Constants.Nd.AND_JOIN_NODE);

	pss.setNodeProperty("p1","k1","10");
	
	Collection c=pss.getNodeProperties("p1");
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnNodePropertyValue p=(BnNodePropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    assertTrue("Get BnNode Properties",key.equalsIgnoreCase("k1"));
	    
	}
    }
  public void testGetProperties() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("propertyNode");		

	pss.setProperty("pk1","xxx");
	
	Collection c=pss.getProperties();
        
	for(Iterator i=c.iterator();i.hasNext();){
	    BnProjectPropertyValue p=(BnProjectPropertyValue)i.next();
	    String key=p.getTheKey();
	    String value=p.getTheValue();
	    if (value.equals("xxx"))
	    	assertTrue("Get Properties",key.equals("pk1"));
	}
    }

 public void testNodeDeadline() throws Exception {

	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("deadlineNode");	
		
	pss.addNode("d1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("d1","admin");
	pss.setNodeDeadline("d1",System.currentTimeMillis()+(long)10000);	
    }


   public void testJavaHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss=projectSessionh.create();
	pss.initProject("hookProject");
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.addNodeHook("h1","hero.hook.TestHook",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.JAVA);
	pss.setNodeRole("h1","admin");

	usr.startActivity("hookProject","h1");
	usr.terminateActivity("hookProject","h1");	
	usr.startActivity("hookProject","h2");
	pss.deleteNodeHook("h1","hero.hook.TestHook");
	usr.terminateActivity("hookProject","h2");

   }
   
   public void testJavaHookInstance() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss=projectSessionh.create();
	pss.initModel("hookProjectInst");	
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.addNodeHook("h1","hero.hook.TestHook",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.JAVA);
	pss.setNodeRole("h1","admin");
	
	String instName = pss.instantiateProject("hookProjectInst");

	usr.startActivity(instName,"h1");
	usr.terminateActivity(instName,"h1");	
	usr.startActivity(instName,"h2");
	usr.terminateActivity(instName,"h2");
   }
   
   public void testProjectJavaHook() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession usr=usersh.create();
		ProjectSession pss=projectSessionh.create();
		pss.initModel("ProjectJavahook");
		pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
		pss.addEdge("h1","h2");						
		pss.addHook("hero.hook.TestProcessHook",hero.interfaces.Constants.Pj.ONINSTANTIATE,Constants.Hook.JAVA);		
		pss.instantiateProject("ProjectJavahook");		
	   }

   public void testNodeInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNode");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook test, node: \"+n.getName());"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNode","h1");
	usr.terminateActivity("hookInterNode","h1");	
   }

   public void testNodeInterHookProperty() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
    UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterNodeP");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeProperty("h1","testp1","testv1",true);
	pss.setNodeProperty("h1","testp2","testv2",true);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnNodee Hook properties tests, node: \"+n.getName());\n"
	    + "System.out.println(\"property \"+testp2);\n"
	    + "}";
	pss.addNodeInterHook("h1","interTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterNodeP","h1");
	usr.terminateActivity("hookInterNodeP","h1");
   }
    
    public void testProjectInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("hookInterProject");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	usr.startActivity("hookInterProject","h1");
	usr.terminateActivity("hookInterProject","h1");	
   }

    public void testDuplicatedInterHook() throws Exception {
	ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	UserSessionHome usersh=UserSessionUtil.getHome();
	UserSession usr=usersh.create();
	ProjectSession pss = projectSessionh.create();
	pss.initProject("dupliHookInter");		
	pss.addNode("h2",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h2","admin");
	pss.addNode("h1",Constants.Nd.AND_JOIN_NODE);
	pss.setNodeRole("h1","admin");
	String edge1=pss.addEdge("h1","h2");
	String script =
	    "import hero.interfaces.BnProjectLocal;\n"
	    + "import hero.interfaces.BnNodeLocal;\n"
	    + "afterStart (Object b,Object n) {\n\n\n"
	    + "System.out.println(\"InteractiveBnProjectt Hook test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
	    + "}";
	pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	try {
	    pss.addInterHook("projectInterTest",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	    assertTrue("should have failed", false);
	} catch (HeroException h) {
	    // normal behavior
	}
	usr.startActivity("dupliHookInter","h1");
	usr.terminateActivity("dupliHookInter","h1");	
   }
    
   public void testSubProcess() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  ProjectSession pss = projectSessionh.create();
	  pss.initProject("SubProject");
	  pss.addNode("subNode1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("subNode2",Constants.Nd.AND_JOIN_NODE);
	  pss.addRoleMapper("InitialRole","mapper1",Constants.Mapper.PROPERTIES);
	  pss.addEdge("subNode1","subNode2");	
	  
	  ProjectSession pss1 = projectSessionh.create();
	  pss1 = projectSessionh.create();
	  pss1.initProject("ParentProject");	
	  pss1.addRoleMapper("InitialRole","mapper1",Constants.Mapper.PROPERTIES);
	  pss1.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss1.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss1.addNodeSubProcess("nodeSub","SubProject");
	  pss1.addEdge("node1","nodeSub");
	  pss1.addEdge("nodeSub","node2");
	  
	  UserSessionHome usersh=UserSessionUtil.getHome();
	  UserSession usr=usersh.create();
	  usr.startActivity("ParentProject","node1");
	  usr.terminateActivity("ParentProject","node1");	  	  
	  
	  assertTrue("SubProcess Error", pss1.getNodeState("nodeSub")==Constants.Nd.EXECUTING);
	
	  // Terminate all SubProcess activities
	  
	  usr.startActivity("SubProject","subNode1");
	  usr.startActivity("SubProject","subNode2");
	  usr.terminateActivity("SubProject","subNode1");
	  usr.terminateActivity("SubProject","subNode2"); 	
	  assertTrue("SubProcess Error 3", pss1.getNodeState("nodeSub")==Constants.Nd.TERMINATED);
	  }
	  
	/*public void testSimpleInstantiation() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  ProjectSession pss = projectSessionh.create();
	  pss.initModel("Instantiation");
	  pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node4",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	  pss.addEdge("node1","node2");
	  pss.addEdge("node2","node3");
	  pss.addEdge("node2","node4");
	  String script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"Instantiation test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	  pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	  pss.addUser("admin2");
	  pss.setUserRole("admin2","InitialRole");
	  pss.setNodeRole("node1","admin");
	  
	  ProjectSession pss1 = projectSessionh.create();	  
	  String instName=pss1.instantiateProject("Instantiation");
	  assertTrue("Instantiation Error, instance name error", pss1.getName().equals(instName));
	  assertTrue("Instantiation Error, role error in node1", pss1.getNodeRoleName("node1").equals("admin"));
	  Collection users = pss1.getUsers();
	  assertTrue("Instantiation Error, user admin2 does not exist", users.contains("admin2"));
	  assertTrue("Instantiation Error, type error in node1", pss1.getNodeType("node1")== Constants.Nd.AND_JOIN_NODE);	  
	}
	
	public void testSubProcessInstantiation() throws Exception {

	  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	  
	  ProjectSession psSub = projectSessionh.create();
	  psSub.initProject("node5");
	  psSub.addNode("subNode1",Constants.Nd.AND_JOIN_NODE);
	  psSub.addNode("subNode2",Constants.Nd.AND_JOIN_NODE);
	  psSub.addEdge("subNode1","subNode2");
	  
	  String script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"SPInstantiation test, SubProcess: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	  psSub.addNodeInterHook("subNode2","subNode2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	  psSub.addUser("admin2");
	  psSub.setNodeRole("subNode1","admin");
	  
	  ProjectSession pss = projectSessionh.create();
	  pss.initModel("SPInstantiation");
	  pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node2",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node3",Constants.Nd.AND_JOIN_NODE);
	  pss.addNode("node4",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	  pss.addNodeSubProcess("node5","SPInstantiation");
	  pss.addEdge("node1","node2");
	  pss.addEdge("node2","node3");
	  pss.addEdge("node2","node4");
	  pss.addEdge("node3","node5");
	  script =
			"import hero.interfaces.BnProjectLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
	  		+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"SPInstantiation test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	 pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	 pss.addUser("admin2");
	 pss.setUserRole("admin2","InitialRole");
	 pss.setNodeRole("node1","admin");
	 ProjectSession pss1 = projectSessionh.create();
	 String instName=pss1.instantiateProject("SPInstantiation");
	 assertTrue("Instantiation Error, instance name error", pss1.getName().equals(instName));
	 assertTrue("Instantiation Error, role error in node1", pss1.getNodeRoleName("node1").equals("admin"));
	 Collection users = pss1.getUsers();
	 assertTrue("Instantiation Error, user admin2 does not exist", users.contains("admin2"));
	 assertTrue("Instantiation Error, type error in node1", pss1.getNodeType("node1")== Constants.Nd.AND_JOIN_NODE);
	}
	
	public void testGetProjectNameOfInstance() throws Exception {
	    ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
	    ProjectSession ps = projectSessionh.create();
		ps.initModel("ofInstance");
		ProjectSession pss1 = projectSessionh.create();
		String instanceName = pss1.instantiateProject("ofInstance");
		String projectName = pss1.getProjectNameOfInstance(instanceName);
		assertTrue("Error in getProjectNameOfInstance",projectName.equals("ofInstance"));
		try {
			projectName = pss1.getProjectNameOfInstance("ofInstance");
		 } catch (Exception io) { 
			assertTrue("getProjectNameOfInstance error when is not a valid instance",true); 
		 }		
	}*/
	
	public void testGetParent() throws Exception {
		// Test when we creates a subProcess activity within a project
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		
		ps = projectSessionh.create();
		ps.initModel("getParent");
				
		ProjectSession ps2 = projectSessionh.create();
		
		ps2.initModel("getParent2");
		
		ps.addNodeSubProcess("node","getParent2");
					
		ps.initProject("getParent");
		String p= ps.getParent();
		assertTrue("Error in getParent 2 when add subProcess",p.equals("getParent"));
				
		ps2.initProject("getParent2");
		String p2 = ps2.getParent();
		assertTrue("Error in getParent when add subProcess",p2.equals("getParent2"));
		
				// Test when we creates an instance of a project
		String instance=ps.instantiateProject("getParent");
		String pInstance=ps.getParent();
		assertTrue("Error in getParent when instantiate",pInstance.equals(ps.getName()));
		
	}
	
	public void testUnsetUserRole() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("unset");
		ProjectSession ps2 = projectSessionh.create();
		ps2.initProject("unset2");
		
		ps.addUser("admin2");
		ps.setUserRole("admin2","InitialRole");
		ps2.addUser("admin2");
		ps2.setUserRole("admin2","InitialRole");

		ps.unsetUserRole("admin2","InitialRole");
		Collection names = ps.getUserRolesInProjectNames("admin2");
		
		assertTrue("Error in unset user roles: InitialRole",!names.contains("InitialRole"));
		assertTrue("Error in unset user roles: admin",names.contains("admin2"));
	}
	
	public void testMapper() throws Exception {

		  ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		  ProjectSession pss = projectSessionh.create();
		  String projectName = "TestMapperProperties5";
		  pss.initModel(projectName);
		  
		  String role1="Admintata";
		  pss.addRole(role1, "role pour activity 1");
		  pss.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		  pss.setNodeRole("node1",role1);
		  	  
	      //  add mapper
		  pss.addRoleMapper(role1,"mapper1",Constants.Mapper.PROPERTIES);
		  
		  java.util.Collection co = null;
		  co = pss.getRoleMappers();
		  java.util.Iterator it =co.iterator();
		  String mapperName ="";
		  int mapperType = 100;
		  while (it.hasNext()){
			BnRoleMapperValue mv = (BnRoleMapperValue)it.next();
			mapperName = mv.getName();	
			mapperType = mv.getType();		  
		  }
		  System.out.println("mapperName = " + mapperName);
		  assertTrue("Error retreiving the created mapper name","mapper1".equals(mapperName));
		  assertTrue("Error retreiving the created mapper id", Constants.Mapper.PROPERTIES == mapperType);
		  
		  // Execution of the mapper at instantiation
		  pss.instantiateProject(projectName);
		 
	      String creator = pss.getCreator();
		  co = pss.getUserRolesInProjectNames(creator);
	      assertTrue("Role Admintata has not been set to the creator", co.contains(creator));
		}

	public void testLazyInstantiation() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("LazyProject");
		
		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.OR_JOIN_NODE);
		ps.setNodeTraditional("node4");
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		
		//	add Edges
		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		
		ps.addRoleMapper("InitialRole","roleMapperTest",Constants.Mapper.PROPERTIES);

		String instName = ps.instantiateProject("LazyProject");
		assertTrue("Error in number of nodes after instantiation, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==1);
		
		UserSessionHome userSessionh = UserSessionUtil.getHome();
		UserSession us = userSessionh.create();
		assertTrue("Error in roleMapper",us.getToDoList(instName).contains("node0"));
	}
	
	public void testLazyInstantiationexecution() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("LazyProjectExec");
		
		// add Nodes
		ps.addNode("node0",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node1",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node2",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node3",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node4",Constants.Nd.OR_JOIN_NODE);
		ps.setNodeTraditional("node4");
		ps.addNode("node5",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node6",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node7",Constants.Nd.AND_JOIN_NODE);
		ps.addNode("node8",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node8");
		ps.addNode("node9",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		ps.setNodeTraditional("node9");
		
		//	add Edges
		ps.addEdge("node0","node1");
		ps.addEdge("node1","node3");
		ps.addEdge("node1","node4");
		ps.addEdge("node1","node5");
		ps.addEdge("node3","node6");
		ps.addEdge("node4","node6");
		ps.addEdge("node4","node2");
		ps.addEdge("node5","node6");
		ps.addEdge("node6","node7");
		ps.addEdge("node3","node8");
		ps.addEdge("node8","node9");
		ps.addEdge("node5","node4");
		
		String instName = ps.instantiateProject("LazyProjectExec");
		UserSessionHome userSessionh = UserSessionUtil.getHome();
		UserSession us = userSessionh.create();
		us.startActivity(instName,"node0");
		assertTrue("Error in state of node0 after start node0",ps.getNodeValue("node0").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in state of node1 after start node0",ps.getNodeValue("node1").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in state of node3 after start node0",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node4 after start node0",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node5 after start node0",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in number of nodes after start node0, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==5);
		us.terminateActivity(instName,"node0");
		assertTrue("Error in state of node0 after terminate node0",ps.getNodeValue("node0").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in state of node1 after terminate node0",ps.getNodeValue("node1").getState() == Constants.Nd.READY);
		assertTrue("Error in state of node3 after terminate node0",ps.getNodeValue("node3").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node4 after terminate node0",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node5 after terminate node0",ps.getNodeValue("node5").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in number of nodes after terminate node0, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==5);
		us.startActivity(instName,"node1");
		assertTrue("Error in state of node1 after start node1",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in state of node3 after start node1",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in state of node4 after start node1",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node5 after start node1",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPABLE);
		assertTrue("Error in state of node6 after start node1",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node8 after start node1",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in number of nodes after start node1, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==7);
		us.startActivity(instName,"node3");
		us.startActivity(instName,"node5");
		assertTrue("Error in state of node1 after start node3 and node5",ps.getNodeValue("node1").getState() == Constants.Nd.EXECUTING);
		assertTrue("Error in state of node3 after start node3 and node5",ps.getNodeValue("node3").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("Error in state of node4 after start node3 and node5",ps.getNodeValue("node4").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node5 after start node3 and node5",ps.getNodeValue("node5").getState() == Constants.Nd.ANTICIPATING);
		assertTrue("Error in state of node6 after start node3 and node5",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node8 after start node3 and node5",ps.getNodeValue("node8").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in number of nodes after start node3 and node5, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==7);
		us.terminateActivity(instName,"node1");
		us.terminateActivity(instName,"node3");
		us.terminateActivity(instName,"node5");
		assertTrue("Error in state of node1 after terminate node1,node3 and node5",ps.getNodeValue("node1").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in state of node2 after terminate node1,node3 and node5",ps.getNodeValue("node2").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node3 after terminate node1,node3 and node5",ps.getNodeValue("node3").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in state of node4 after terminate node1,node3 and node5",ps.getNodeValue("node4").getState() == Constants.Nd.READY);
		assertTrue("Error in state of node5 after terminate node1,node3 and node5",ps.getNodeValue("node5").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in state of node6 after terminate node1,node3 and node5",ps.getNodeValue("node6").getState() == Constants.Nd.INITIAL);
		assertTrue("Error in state of node8 after terminate node1,node3 and node5",ps.getNodeValue("node8").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in state of node9 after terminate node1,node3 and node5",ps.getNodeValue("node9").getState() == Constants.Nd.TERMINATED);
		assertTrue("Error in number of nodes after start node3 and node5, number of nodes:"+ps.getNodesNames().size(),ps.getNodesNames().size()==9);
		us.startActivity(instName,"node4");
		us.terminateActivity(instName,"node4");
		
		us.startActivity(instName,"node6");
		us.terminateActivity(instName,"node6");
		
		us.startActivity(instName,"node7");
		us.terminateActivity(instName,"node7");
		
		try{
			us.terminate(instName);
			assertTrue("Error in terminate project, terminate project should be failed",false);	
		}catch(Exception e){}
		
		us.startActivity(instName,"node2");
		us.terminateActivity(instName,"node2");
	}
	
	public void testUnsetUser() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("unsetUser");
		ProjectSession ps2 = projectSessionh.create();
		ps2.initProject("unsetUser2");
		
		ps.addUser("admin2");
		assertTrue("Error in add user: ",ps.getUsers().contains("admin2"));
		
		ps2.addUser("admin2");
		assertTrue("Error in add user (2): ",ps2.getUsers().contains("admin2"));
		
		ps.unsetUser("admin2");
		assertTrue("Error in unset user: ",!ps.getUsers().contains("admin2"));
		assertTrue("Error in add user (3): ",ps2.getUsers().contains("admin2"));
		
		ps2.unsetUser("admin2");
		assertTrue("Error in unset user (2): ",!ps2.getUsers().contains("admin2"));
	}
	
	public void testGetUsersRole() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("getUsersRole");				
		ps.addUser("admin2");
		
		ps.addRole("roleTest","");
		ps.setUserRole("admin2","roleTest");
		
		assertTrue("Error in getUsersRole (1): ",ps.getUsersRole("roleTest").contains("admin2"));
		assertTrue("Error in getUsersRole (1): ",!ps.getUsersRole("roleTest").contains("admin"));		
	}
	
	public void testPropertyPossibleValues() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initProject("ProjectPossibleValues");
		ArrayList values = new ArrayList();
		values.add("value1");
		values.add("value2");
		
		ps.setPropertyPossibleValues("key1",values);
		try{
			ps.setProperty("key1","value3");		
			assertTrue("Error setProperty should be failed",false);	
		}catch(Exception e){}		
		ps.setProperty("key1","value2");		
	}
	
	public void testNodePropertyPossibleValues() throws Exception {
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession user = usersh.create();
		
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("ProjectNodesPossibleValues");
		ps.addNode("node1",1);
		ps.addNode("node2",1);
		ps.addNode("node3",1);
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		ps.setNodeRole("node1",Constants.ADMIN);
		ps.setNodeRole("node2",Constants.ADMIN);
		ps.setNodeRole("node3",Constants.ADMIN);
		ArrayList values = new ArrayList();
		values.add("value1");
		values.add("value2");
		
		ps.setNodePropertyPossibleValues("node1","key1",values);
		try{
			ps.setNodeProperty("node1","key1","value3");
			assertTrue("Error setNodeProperty should be failed",false);
		}catch(Exception e){}
		ps.setNodeProperty("node1","key1","value2",true);
		String instName=ps.instantiateProject("ProjectNodesPossibleValues");
		
		user.startActivity(instName,"node1");
		user.terminateActivity(instName,"node1");
		
		BnNodePropertyValue pv = ps.getNodeProperty("node2","key1");
		
		assertTrue("Error testNodeProperty after instantiation", pv.getPossibleValues().size()==2);		
		
		user.startActivity(instName,"node2");
		user.terminateActivity(instName,"node2");
		
		pv = ps.getNodeProperty("node3","key1");
		
		assertTrue("Error testNodeProperty after instantiation 2", pv.getPossibleValues().size()==2);
	}	
	
	public void testUpdateNodePropertyPossibleValues() throws Exception {
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession user = usersh.create();
		
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("UpdateProjectNodesPossibleValues");
		ps.addNode("node1",1);
		ps.addNode("node2",1);
		ps.addNode("node3",1);
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		ps.setNodeRole("node1",Constants.ADMIN);
		ps.setNodeRole("node2",Constants.ADMIN);
		ps.setNodeRole("node3",Constants.ADMIN);
		ArrayList values = new ArrayList();
		values.add("value1");
		values.add("value2");
		
		ps.setNodePropertyPossibleValues("node1","key1",values);
		try{
			ps.setNodeProperty("node1","key1","value3");
			assertTrue("Error setNodeProperty should be failed",false);
		}catch(Exception e){}
		ps.setNodeProperty("node1","key1","value2",true);
		String instName=ps.instantiateProject("UpdateProjectNodesPossibleValues");
		
		user.startActivity(instName,"node1");
		user.terminateActivity(instName,"node1");
		
		BnNodePropertyValue pv = ps.getNodeProperty("node2","key1");
		
		assertTrue("Error testNodeProperty after instantiation", pv.getPossibleValues().size()==2);

		values = new ArrayList();
		values.add("value3");
		values.add("value4");
		values.add("value5");
		ArrayList ardf = new ArrayList();
		ardf.add("value3");
		ps.updateNodePropertyPossibleValues("node2","key1",values,ardf);
		
		pv = ps.getNodeProperty("node2","key1");
		
		assertTrue("Error testNodeProperty after instantiation", pv.getPossibleValues().size()==3);
		
		user.startActivity(instName,"node2");
		user.terminateActivity(instName,"node2");
		
		pv = ps.getNodeProperty("node3","key1");
		
		assertTrue("Error testNodeProperty after instantiation 2", pv.getPossibleValues().size()==3);
	}	
	
	public void testUpdatePropertyPossibleValues() throws Exception {
		UserSessionHome usersh=UserSessionUtil.getHome();
		UserSession user = usersh.create();
		
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession ps = projectSessionh.create();
		ps.initModel("UpdateProjectPossibleValues");
		ps.addNode("node1",1);
		ps.addNode("node2",1);
		ps.addNode("node3",1);
		ps.addEdge("node1","node2");
		ps.addEdge("node2","node3");
		ps.setNodeRole("node1",Constants.ADMIN);
		ps.setNodeRole("node2",Constants.ADMIN);
		ps.setNodeRole("node3",Constants.ADMIN);
		ArrayList values = new ArrayList();
		values.add("value1");
		values.add("value2");
		
		ps.setPropertyPossibleValues("key1",values);
		try{
			ps.setProperty("key1","value3");
			assertTrue("Error setNodeProperty should be failed",false);
		}catch(Exception e){}
		ps.setProperty("key1","value2");
		String instName=ps.instantiateProject("UpdateProjectPossibleValues");
		
		user.startActivity(instName,"node1");
		user.terminateActivity(instName,"node1");
		
		BnProjectPropertyValue pv = ps.getProperty("key1");
		
		assertTrue("Error testProperty after instantiation", pv.getPossibleValues().size()==2);

		values = new ArrayList();
		values.add("value3");
		values.add("value4");
		values.add("value5");
		ArrayList ardf = new ArrayList();
		ardf.add("value3");
		ps.updatePropertyPossibleValues("key1",values,ardf);
		
		pv = ps.getProperty("key1");
		
		assertTrue("Error testProperty after instantiation", pv.getPossibleValues().size()==3);
		
		user.startActivity(instName,"node2");
		user.terminateActivity(instName,"node2");
		
		pv = ps.getProperty("key1");
		
		assertTrue("Error testProperty after instantiation 2", pv.getPossibleValues().size()==3);
	}	
	
	public void testAddDeleteInitiatorMapper() throws Exception {
	  ProjectSessionHome pHome=ProjectSessionUtil.getHome();
	  ProjectSession lProject = pHome.create();
	  lProject.initModel("InitiatorModel");
	  lProject.addInitiatorMapper("hero.initiatorMapper.CustomGroupMembers",Constants.InitiatorMapper.CUSTOM);
	  String instName = lProject.instantiateProject("InitiatorModel");
	  UserSessionHome uHome=UserSessionUtil.getHome();
	  UserSession user = uHome.create();
	  user.removeProject(instName);
	  user.removeProject("InitiatorModel");
	  assertTrue("Error testAddDeleteInitiator", lProject.existingProject("InitiatorProject")==false);
	}
	
	public void testSimpleVersioningTest() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss=projectSessionh.create();
		
		pss.initModelWithVersion("Approval_workflow_versioned","1.0");
		pss.addNode("Approval",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("Approval2",Constants.Nd.AND_JOIN_NODE);
		pss.addEdge("Approval","Approval2");
		
		pss.initModelWithVersion("Approval_workflow_versioned","1.1");
		pss.addNode("Approval",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("Approval2",Constants.Nd.AND_JOIN_NODE);
		pss.addEdge("Approval","Approval2");
		
		String instName = pss.instantiateProject("Approval_workflow_versioned","1.1");
		
		assertTrue("Error instance version",pss.getDetails().getVersion().equals("1.1"));
		
		UserSessionHome userSessionh=UserSessionUtil.getHome();
		UserSession uss=userSessionh.create();
		uss.startActivity(instName,"Approval");
		assertTrue("Error Approval activity state",pss.getNodeState("Approval") == Constants.Nd.EXECUTING);
		
		uss.terminateActivity(instName,"Approval");
		assertTrue("Error Approval activity state",pss.getNodeState("Approval") == Constants.Nd.TERMINATED);
		
		uss.startActivity(instName,"Approval2");
		assertTrue("Error Approval2 activity state",pss.getNodeState("Approval2") == Constants.Nd.EXECUTING);
		assertTrue("Error Approval2 model version",pss.getNodeValue("Approval2").getBnProject().getVersion().equals("1.1"));
		
		uss.terminateActivity(instName,"Approval2");
		
		String instName2 = pss.instantiateProject("Approval_workflow_versioned","1.0");
		
		assertTrue("Error instance version 2",pss.getDetails().getVersion().equals("1.0"));
		
		uss.startActivity(instName2,"Approval");
		assertTrue("Error Approval activity state 2",pss.getNodeState("Approval") == Constants.Nd.EXECUTING);
		
		uss.terminateActivity(instName2,"Approval");
		assertTrue("Error Approval activity state 2",pss.getNodeState("Approval") == Constants.Nd.TERMINATED);
		
		uss.startActivity(instName2,"Approval2");
		assertTrue("Error Approval2 activity state 2",pss.getNodeState("Approval2") == Constants.Nd.EXECUTING);
		assertTrue("Error Approval2 model version 2",pss.getNodeValue("Approval2").getBnProject().getVersion().equals("1.0"));
		
		uss.terminateActivity(instName2,"Approval2");
		
		instName = pss.instantiateProject("Approval_workflow_versioned","1.1");
		uss.removeInstance(instName);
		
		instName = pss.instantiateProject("Approval_workflow_versioned","1.0");
		uss.removeInstance(instName);
		
		try{ 
			uss.removeInstance(instName);
			assertTrue("Should have throw EJBException",false);
		} catch (Exception e) {
		    //expected behavior
		}
		
		uss.removeProject("Approval_workflow_versioned","1.0");
		uss.removeProject("Approval_workflow_versioned","1.1");
	}

	public void testDynamicEnumerationPropagation() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss=projectSessionh.create();
		
		pss.initModelWithVersion("DynamicEnumPropagation","1.0");
		pss.addNode("start",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("appro3",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("appro3OK",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		pss.addNode("approv3bis",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		pss.addNode("End",Constants.Nd.AND_JOIN_NODE);
		
		pss.addNodeHook("appro3OK","hero.hook.UpdateEnumDyn", Constants.Nd.BEFORETERMINATE, Constants.Hook.JAVA);
		
		pss.addEdge("start","appro3");
		pss.addEdge("appro3","appro3OK");
		pss.addEdge("appro3OK","approv3bis");
		pss.addEdge("appro3OK","End");
		
		// Set node properties
		Collection cas = new ArrayList();
		cas.add("cas1");
		cas.add("cas2");
		cas.add("cas3");
		pss.setNodePropertyPossibleValues("start","cas",cas);
		pss.setNodeProperty("start","cas","cas3",true);
		
		Collection approv = new ArrayList();
		approv.add("yes");
		approv.add("no");
		pss.setNodePropertyPossibleValues("start","approv",approv);
		pss.setNodeProperty("start","approv","yes",true);
		
		Collection enumdyn = new ArrayList();
		enumdyn.add("default");
		pss.setNodePropertyPossibleValues("start","enumdyn",enumdyn);
		pss.setNodeProperty("start","enumdyn","default",true);
		
		pss.setNodeProperty("appro3","mykey","myvalue");
		
		String instName = pss.instantiateProject("DynamicEnumPropagation","1.0");
		
		assertTrue("Error instance version",pss.getDetails().getVersion().equals("1.0"));
		
		UserSessionHome userSessionh=UserSessionUtil.getHome();
		UserSession uss=userSessionh.create();
		
		uss.startActivity(instName,"start");
		assertTrue("Error start activity state",pss.getNodeState("start") == Constants.Nd.EXECUTING);
		
		uss.terminateActivity(instName,"start");
		assertTrue("Error start activity state",pss.getNodeState("start") == Constants.Nd.TERMINATED);
		
		uss.startActivity(instName,"appro3");
		assertTrue("Error appro3 activity state",pss.getNodeState("appro3") == Constants.Nd.EXECUTING);
		assertTrue("Error appro3 model version",pss.getNodeValue("appro3").getBnProject().getVersion().equals("1.0"));
		
		uss.terminateActivity(instName,"appro3");
		assertTrue("Error End activity state",pss.getNodeState("End") == Constants.Nd.READY);		
		
	}
	
	public void testPropertiesPropagation() throws Exception {
		ProjectSessionHome projectSessionh=ProjectSessionUtil.getHome();
		ProjectSession pss=projectSessionh.create();
		
		pss.initModelWithVersion("PropertiesPropagation","1.0");
		pss.addNode("appro3OK",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		pss.addNode("approv3bis",Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
		pss.addNode("MyManual",Constants.Nd.AND_JOIN_NODE);
		pss.addNode("End",Constants.Nd.AND_JOIN_NODE);
		
		pss.addNodeHook("appro3OK","hero.hook.UpdateEnumDyn", Constants.Nd.BEFORETERMINATE, Constants.Hook.JAVA);
		pss.addNodeHook("MyManual","hero.hook.UpdateEnumDyn2", Constants.Nd.BEFORETERMINATE, Constants.Hook.JAVA);
		pss.addNodeHook("End","hero.hook.UpdateEnumDyn3", Constants.Nd.ONREADY, Constants.Hook.JAVA);
		
		pss.addEdge("appro3OK","approv3bis");
		pss.addEdge("appro3OK","MyManual");
		pss.addEdge("MyManual","End");
		
		// Set node properties
		Collection cas = new ArrayList();
		cas.add("cas1");
		cas.add("cas2");
		cas.add("cas3");
		pss.setNodePropertyPossibleValues("appro3OK","cas",cas);
		pss.setNodeProperty("appro3OK","cas","cas3",true);
				
		pss.setNodeProperty("appro3OK","testProp","yes",true);
		
		Collection enumdyn = new ArrayList();
		enumdyn.add("default");
		pss.setNodePropertyPossibleValues("appro3OK","enumdyn",enumdyn);
		pss.setNodeProperty("appro3OK","enumdyn","default",true);
		
		String instName = pss.instantiateProject("PropertiesPropagation","1.0");
		
		assertTrue("Error instance version",pss.getDetails().getVersion().equals("1.0"));
		assertTrue("Error start activity state 1 ",pss.getNodeState("appro3OK") == Constants.Nd.TERMINATED);
		assertTrue("Error start activity state 2 ",pss.getNodeState("approv3bis") == Constants.Nd.TERMINATED);
		
		assertTrue("Error value property enumdyn ",pss.getNodeProperty("approv3bis","enumdyn").getTheValue().equals("miguel"));
		
		UserSessionHome userSessionh=UserSessionUtil.getHome();
		UserSession uss=userSessionh.create();
		
		uss.startActivity(instName,"MyManual");
		assertTrue("Error start activity state",pss.getNodeState("MyManual") == Constants.Nd.EXECUTING);
		assertTrue("Error value property testprop ",pss.getNodeProperty("MyManual","testProp").getTheValue().equals("yes"));
		
		uss.terminateActivity(instName,"MyManual");
		assertTrue("Error start activity state",pss.getNodeState("MyManual") == Constants.Nd.TERMINATED);
		
		assertTrue("Error End activity state",pss.getNodeState("End") == Constants.Nd.READY);		
		assertTrue("Error value property enumdyn ",pss.getNodeProperty("MyManual","enumdyn").getTheValue().equals("new1"));
		assertTrue("Error value property enumdyn ",pss.getNodeProperty("End","enumdyn").getTheValue().equals("newValue"));
		assertTrue("Error value property testprop ",pss.getNodeProperty("End","testProp").getTheValue().equals("yes"));
	}

	
}
