/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: ModelCreationPerfTest.java,v 1.2 2005/03/18 14:51:23 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.client.test.perf;

import javax.security.auth.login.LoginContext;
import hero.client.test.SimpleCallbackHandler;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.interfaces.UserRegistration;
import hero.interfaces.UserRegistrationHome;
import hero.interfaces.UserRegistrationUtil;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import hero.interfaces.Constants;
import java.util.Collection;
import java.util.Iterator;
import java.util.Timer;

public class ModelCreationPerfTest {
	
	static public void main(String[] args) throws Exception{

	long startTime = System.currentTimeMillis();
	
	  char[] password={'t','o','t','o'};
	  SimpleCallbackHandler handler = new SimpleCallbackHandler("admin",password);
	  LoginContext lc = new LoginContext("TestClient", handler);
	  lc.login();
		
	ProjectSessionHome pHome = ProjectSessionUtil.getHome();
	try{

	System.out.println("  --> Model Creation "); 
	ProjectSession psSub = pHome.create();
	psSub.initModel("node5Sub");
	psSub.addNode("subNode1", Constants.Nd.AND_JOIN_NODE);
	psSub.addNode("subNode2", Constants.Nd.AND_JOIN_NODE);
	psSub.setNodeRole("subNode1", "admin");
	psSub.setNodeRole("subNode2", "admin");
	psSub.addEdge("subNode1", "subNode2");
	String script =
		"import hero.interfaces.BnInstanceLocal;\n"
				+ "import hero.interfaces.BnNodeLocal;\n"
				+ "afterStart (Object b,Object n) {\n\n\n"
				+ "System.out.println(\"SPInstantiation test, SubProcess: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
				+ "}";
	psSub.addNodeInterHook("subNode2","subNode2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	psSub.addUser("admin2");
	psSub.setNodeRole("subNode1", "admin");
	psSub.setNodeRole("subNode2", "admin");
	
	ProjectSession pss = pHome.create();
	pss.initModel("Stress");
	pss.addNode("node1", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node2", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node3", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node4", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	//pss.addNodeSubProcess("node5Sub","Stress");
	pss.addNode("node5Sub",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("node1", "node2");
	pss.addEdge("node2", "node3");
	pss.addEdge("node2", "node4");
	pss.addEdge("node3", "node5Sub");
	script =
		"import hero.interfaces.BnInstanceLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"Stress test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	pss.addUser("admin2");
	pss.setUserRole("admin2", "InitialRole");
	pss.setNodeRole("node1", "admin");
	pss.setNodeRole("node2", "admin");
	pss.setNodeRole("node3", "admin");
	pss.setNodeRole("node4", "admin");
	pss.setNodeRole("node5Sub", "admin");
	System.out.println("  --> Stress Model totally created "); 
	psSub.remove();
	pss.remove() ;
	}catch(Exception e){System.out.println(" --> " + e);}

	long time = System.currentTimeMillis();
	long tmp = (time - startTime)/1000;
	long h = tmp/3600;
	long m = (tmp-3600*h)/60;
	long s = tmp-3600*h-60*m;
	System.out.println("  --> Stress Model system time elapsed : " + h + ":"+ m + ":" + s );
	} // End Main/
} // End Class
