/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: InstancesTests.java,v 1.7 2005/01/14 17:14:39 mvaldes Exp $
 *
--------------------------------------------------------------------------
 */

package hero.client.test.stress;

import javax.security.auth.login.LoginContext;
import hero.client.test.SimpleCallbackHandler;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;
import hero.interfaces.UserRegistration;
import hero.interfaces.UserRegistrationHome;
import hero.interfaces.UserRegistrationUtil;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import hero.interfaces.Constants;
import java.util.Collection;
import java.util.Iterator;

public class InstancesTests extends TestCase {
 
	public InstancesTests(String testname) {
	super(testname);
	}
    
	public static TestSuite suite() {
	return new TestSuite(InstancesTests.class);
	}

	public void setUp() throws Exception {
	}
    
	public void testInstances() throws Exception {
	ProjectSessionHome pHome = ProjectSessionUtil.getHome();
	ProjectSession psSub = pHome.create();
	try{
		
	psSub.initModel("node5Sub");
	psSub.addNode("subNode1", Constants.Nd.AND_JOIN_NODE);
	psSub.addNode("subNode2", Constants.Nd.AND_JOIN_NODE);
	psSub.setNodeRole("subNode1", "admin");
	psSub.setNodeRole("subNode2", "admin");
	psSub.addEdge("subNode1", "subNode2");
	String script =
		"import hero.interfaces.BnInstanceLocal;\n"
				+ "import hero.interfaces.BnNodeLocal;\n"
				+ "afterStart (Object b,Object n) {\n\n\n"
				+ "System.out.println(\"SPInstantiation test, SubProcess: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
				+ "}";
	psSub.addNodeInterHook("subNode2","subNode2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	psSub.addUser("admin2");
	psSub.setNodeRole("subNode1", "admin");
	psSub.setNodeRole("subNode2", "admin");
	
	ProjectSession pss = pHome.create();
	pss.initModel("Stress");
	pss.addNode("node1", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node2", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node3", Constants.Nd.AND_JOIN_NODE);
	pss.addNode("node4", Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	pss.addNodeSubProcess("node5Sub","Stress");
	//pss.addNode("node5Sub",Constants.Nd.AND_JOIN_NODE);
	pss.addEdge("node1", "node2");
	pss.addEdge("node2", "node3");
	pss.addEdge("node2", "node4");
	pss.addEdge("node3", "node5Sub");
	script =
		"import hero.interfaces.BnInstanceLocal;\n"
			+ "import hero.interfaces.BnNodeLocal;\n"
			+ "afterStart (Object b,Object n) {\n\n\n"
			+ "System.out.println(\"Stress test, node: \"+n.getName()+\" project: \"+(n.getBnProject()).getName());"
			+ "}";
	pss.addNodeInterHook("node2","node2",hero.interfaces.Constants.Nd.AFTERSTART,Constants.Hook.BSINTERACTIVE,script);
	pss.addUser("admin2");
	pss.setUserRole("admin2", "InitialRole");
	pss.setNodeRole("node1", "admin");
	pss.setNodeRole("node2", "admin");
	pss.setNodeRole("node3", "admin");
	pss.setNodeRole("node4", "admin");
	pss.setNodeRole("node5Sub", "admin");
	ProjectSession pss1 = pHome.create();
	pss1.instantiateProject("Stress");
	assertTrue("Instantiation Error, role error in node1",pss1.getNodeRoleName("node1").equals("admin"));
	Collection users = pss1.getUsers();
	assertTrue("Instantiation Error, user admin2 does not exist",users.contains("admin2"));
	assertTrue("Instantiation Error, type error in node1",pss1.getNodeType("node1") == Constants.Nd.AND_JOIN_NODE);
	assertTrue("Instantiation Error, type error in node4",pss1.getNodeType("node4") == Constants.Nd.AND_JOIN_AUTOMATIC_NODE);
	}catch(Exception e){}
	int global = 0;
	for (global=0;global<100;global++)
	{
		UserRegistrationHome urHome = UserRegistrationUtil.getHome();
		UserRegistration userReg=urHome.create();
		userReg.userCreate("user"+global,"user"+global,"miguel.valdes-faura@ext.bull.net");
		userReg.remove();
		String uName= "user"+global;
		char[] password=uName.toCharArray();
		SimpleCallbackHandler handler = new SimpleCallbackHandler(uName,password);
		LoginContext lc = new LoginContext("TestClient", handler);
		lc.login();

		int i=0;
		for (i=0;i<10;i++){
			ProjectSession ps = pHome.create();
			ps.instantiateProject("Stress");
			//Thread.sleep(100);
			ps.remove();
		}
		
		UserSessionHome uHome = UserSessionUtil.getHome();
		UserSession us = uHome.create();
		Collection instances = us.getInstancesListNames();
		assertTrue("getInstancesList error",instances.size()>=1);
		Iterator index = instances.iterator();
		UserSession usUser = uHome.create();
		while (index.hasNext())
		{
			String name = (String)index.next();
			Collection task = usUser.getToDoList(name);
			Iterator it = task.iterator();
			while (it.hasNext())
			{
			  String activity = (String)it.next();			  
			  usUser.startActivity(name,activity);
			  if (activity.matches(".*_instance.*"))
				 this.executeSubProcess(activity);

			  usUser.terminateActivity(name,activity);
			  task = usUser.getToDoList(name);
			  it = task.iterator();
			}
		}
	}
   }

   public void executeSubProcess(String subProcess) throws Exception{
	 UserSessionHome uHome = UserSessionUtil.getHome();
	 UserSession us = uHome.create();
	 us.startActivity(subProcess,"subNode1");
	 us.terminateActivity(subProcess,"subNode1");
	 us.startActivity(subProcess,"subNode2");
	 us.terminateActivity(subProcess,"subNode2");
   }
   
	/*public void testInstancesInst() throws Exception {
	UserSessionHome uHome = UserSessionUtil.getHome();
	UserSession us = uHome.create();
	Collection instances = us.getInstancesListNames();
	assertTrue("getInstancesList error",instances.size()>=1);
	Iterator index = instances.iterator();
	UserSession usUser = uHome.create();
	while (index.hasNext())
	{
		String name = (String)index.next();
		Collection task = usUser.getToDoList(name);
		Iterator it = task.iterator();
		while (it.hasNext())
		{
		  String activity = (String)it.next();
		  usUser.startActivity(name,activity);
		  if (activity.matches(".*_instance.*"))
			 this.executeSubProcess(activity);

		  usUser.terminateActivity(name,activity);
		  task = usUser.getToDoList(name);
		  it = task.iterator();
		}
	}
  }*/
   /*public void testUser() throws Exception {
   int global =0;
   for (global=0;global<50000;global++)
	{
		UserRegistrationHome urHome = UserRegistrationUtil.getHome();
		UserRegistration userReg=urHome.create();
		userReg.userCreate("user"+global,"user"+global,"miguel.valdes-faura@ext.bull.net");
		userReg.remove();
	}   
   }*/
}
