package hero.entity;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.Constants;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceValue;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaProjectValue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;


/*
* 02/01/2002 - 15:24:07
*
* AndEdgeState.java -
* Copyright (C) 2002 Ecoo Team
* charoy@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

public class AndEdgeState extends EdgeState implements java.io.Serializable {

    private static AndEdgeState nd;

    public AndEdgeState() {
    }

    public static AndEdgeState getInstance() {
	if (nd==null) {
	    nd=new AndEdgeState();
	}
	return nd;
    }

    public int state(BnNodeLocal node) {
		Collection edgesInstance = new ArrayList();
		Collection edgesModel = new ArrayList();

		try{
			edgesInstance = BonitaServiceValue.getNode(node.getBnProject(),node.getName()).getInEdges();
		}catch (Exception e) {e.printStackTrace();}

		// Instances consistency (may be some activities are not yet created in the instance)
		String nodeName = node.getName();
		if (isInstance(node.getName()))
			nodeName = getModel(nodeName);

		try{
			String modelName = this.getModel(node.getBnProject().getName());
			String modelVersion = node.getBnProject().getVersion();
			BonitaProjectValue model = BonitaProjectLocator.getInstance().getModelValue(modelName, modelVersion);
			if (node.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				edgesModel = BonitaServiceValue.getNode(model.getName(),model.getVersion(), nodeName).getInEdges();
			else
				edgesModel = BonitaServiceValue.getNodeFromCache(model,nodeName).getInEdges();
		}catch (Exception e) {edgesModel=new ArrayList();}

		if ((edgesInstance.size() > 0) || (edgesModel.size() > 0)) {
			edgesInstance = concatEdges(edgesInstance, edgesModel);
		}

		// If some of the edges state is DEAD in an AND -> return CANCEL over the next node
		Iterator it = edgesInstance.iterator();
		while (it.hasNext()) {
			BonitaEdgeValue edge = (BonitaEdgeValue)it.next();
		    if ((edge.getState() == Constants.Ed.DEAD))	{
			    return Constants.Nd.CANCEL;
			}
		}

		it = edgesInstance.iterator();
		int result = Constants.Nd.ACTIVE;
		while (it.hasNext()) {
			BonitaEdgeValue edge = (BonitaEdgeValue)it.next();
		    if (edge.getState() == Constants.Ed.INITIAL) {
		    	result=Constants.Nd.EDGEINITIAL;
		    	break;
		    }
		    if ((edge.getState() == Constants.Ed.ANTICIPATING)) {
		    	result=Constants.Nd.ANTACTIVE;
		    }
		}
		return result;
    }

	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}

	/**
	 * Concatenates model edges that doesn't exist in the instances edges (without repetition)
	 *
	 * @param edgesInstance
	 * @param edgesModel
	 * @return
	 */
	private Collection concatEdges(Collection edgesInstance, Collection edgesModel) {
		Collection result = new ArrayList();
		boolean found = false;

		result = edgesInstance;

		Iterator it = edgesModel.iterator();
		while (it.hasNext()) {
			found = false;
			BonitaEdgeValue el = (BonitaEdgeValue)it.next();
			String nodeName = el.getInNode();
			Iterator instanceEdgesIt = edgesInstance.iterator();
			while(instanceEdgesIt.hasNext()) {
				BonitaEdgeValue insEd = (BonitaEdgeValue) instanceEdgesIt.next();
				String node = insEd.getInNode();
				if (this.isInstance(node)) {
					node = this.getModel(node);
				}
				if (nodeName.equals(node)) {
					found = true;
					break;
				}
			}
			if (!found) {
				result.add(el);
			}
		}
		return result;
	}
}
