package hero.entity;

import hero.interfaces.BnAgentEdgePK;
import hero.interfaces.BnAgentEdgeValue;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;

import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnAgentEdge
 *
 * @author Miguel Valdes
 * @version $Revision: 1.9 $
 *
 * @ejb:bean name="BnAgentEdge"
 *           display-name="BnAgentEdge Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnAgentEdge"
 *           local-jndi-name="ejb/hero/BnAgentEdge_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:pk
 *      class="hero.interfaces.BnAgentEdgePK"
 *
 * @ejb:transaction type="Supports"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnAgentEdge"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(a) FROM BnAgentEdge a"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnAgentEdge findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Remote"
 *     query="SELECT OBJECT(a) FROM BnAgentEdge a WHERE a.name = ?1 AND a.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnAgentEdgeLocal findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Local"
 *     query="SELECT OBJECT(a) FROM BnAgentEdge a WHERE a.name = ?1 AND a.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb.value-object
 *    match="*"
 *    name="BnAgentEdge"
 *
 * @jboss.persistence
 *    table-name="BnAgentEdge"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/

public abstract class BnAgentEdgeBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnAgentEdge's id.
   *
   * @return Returns an int representing the id of this BnAgentEdge.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnAgentEdge's id.
   *
   * @param pId The id of this BnAgentEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setId( String pId );

   /**
   * Retrieve the BnAgentEdge's name.
   *
   * @return Returns a string representing the name of this BnAgentEdge.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the BnAgentEdge's name.
   *
   * @param pName The name of this BnAgentEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setName( String pName );

    /**
     * @return  the project of the BnAgentEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="AgentEdges" role-name="is-edge-of" target-ejb="BnProject" target-role-name="Illness-Has-Patient"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectIdFK"
     *
     **/
    public abstract BnProjectLocal getBnProject();

    /**
     * Set the BnProject Name.
     *
     * @param pName The Name of this BnAgentEdge.  Is set at creation time.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject(BnProjectLocal pProject);

    /**
     * @return  the state of the BnAgentEdge
     * @ejb:interface-method view-type="local"
     * @ejb:persistence column-name="state"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract int getState();

    /**
     * Set the State value.
     *
     * @param pstate The new state of this BnAgentEdge.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setState(int state);

    /**
     * @return  the In BnAgent of the BnAgentEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InAgent" role-name="has-node-in" target-ejb="BnAgent"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnAgentLightValue"
     *       aggregate-name="InAgent"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "agentEdgeIdFK"
     **/
    public abstract BnAgentLocal getInBnAgent();

   /**
   * Set the BnProject Name.
   *
   * @param pAgent The In BnAgent of the BnAgentEdge
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setInBnAgent(BnAgentLocal pAgent);

    /**
     * @return  the Out BnNode of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutNodeInAgentEdge" role-name="has-node-out" target-ejb="BnNode"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnNodeLightValue"
     *       aggregate-name="OutNodeInAgentEdge"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "nodeAgentEdgeIdFK"
     **/
    public abstract BnNodeLocal getOutBnNode();

   /**
   * Set the BnProject Name.
   *
   * @param pNode The Name of this BnEdge.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutBnNode(BnNodeLocal pNode);

    /**
     * @return Return the creation date of the BnAgentEdge
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     *
     **/

    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnAgentEdge
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnAgentEdgeValue getBnAgentEdgeValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnAgentEdgeValue(BnAgentEdgeValue v);


   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnAgentEdge based on the supplied BnAgentEdge Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnAgentEdgePK ejbCreate()
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
	 this.setName("BnAgentEdge"+this.getId());
     this.setId(hero.interfaces.BnAgentEdgeUtil.generateGUID(this));
     return null;
   }

   /**
   * Create a BnAgentEdge based on the supplied BnAgentEdge Value Object. (c'est merdique)
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnAgentEdgePK ejbCreate(BnAgentEdgeValue pAgentEdge)
      throws
         CreateException
   {
       return null;
   }

   public void ejbPostCreate(BnAgentEdgeValue pAgentEdge)
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnAgentEdgeValue(pAgentEdge);
   }

   public void ejbPostCreate( )
   {
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
