package hero.entity;

import hero.interfaces.BnAuthRoleData;
import hero.interfaces.BnAuthRolePK;
import hero.interfaces.BnAuthRoleValue;

import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnAuthRole (for authorization of ejb method)
 *
 * @author Fran?ois Charoy
 * @version $Revision: 1.11 $
 *
 * @ejb:bean name="BnAuthRole"
 *           display-name="BnAuthRole Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnAuthRole"
 *           local-jndi-name="ejb/hero/BnAuthRole_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * The following tag is required to make JBoss guarantee database consistency
 * when removing entities involved in relations.
 * @ejb:transaction type="Required"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnAuthRole"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(a) FROM BnAuthRole a"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnAuthRole findByName(java.lang.String pName)" result-type-mapping="Remote"
 *     query="SELECT OBJECT(n) FROM BnAuthRole n WHERE n.name = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnAuthRoleLocal findByName(java.lang.String pName)" result-type-mapping="Local"
 *     query="SELECT OBJECT(n) FROM BnAuthRole n WHERE n.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnAuthRole"
 *
 * @jboss.persistence
 *    table-name="BnAuthRole"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnAuthRoleBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnAuthRole's id.
   *
   * @return Returns an int representing the id of this BnAuthRole.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * 	key-jdbc-name="BnUser_id"
   * @ejb:transaction type="Supports"
   *
   **/
   public abstract String getId();

   /**
   * Set the BnAuthRole's id.
   *
   * @param pId The id of this BnAuthRole. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );

    /**
     * Retrieve the BnAuthRole's Name.
     *
     * @return Returns an int representing the Name of this BnAuthRole.
     * @ejb:interface-method view-type="both"
     *
     * @ejb:persistence column-name="name"
     * @ejb:transaction type="Supports"
     **/
    public abstract String getName();

    /**
     * Set the BnAuthRole's Name.
     *
     * @param pName The Name of this BnAuthRole.  Is set at creation time.
     * @ejb:transaction type="Required"
     **/
    public abstract void setName( String pName );


    /**
     * Retrieve the BnAuthRole's Group.
     *
     * @return Returns an int representing the Group of this BnAuthRole.
     *
     * @ejb:persistence column-name="authrolegroup"
     * @ejb:transaction type="Supports"
     **/
    public abstract String getBnRoleGroup();

    /**
     * Set the BnAuthRole's Group.
     *
     * @param pGroup the Group of this BnAuthRole
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnRoleGroup( String pGroup );


    /**
     * @returns the users of the role
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="AuthRoles" role-name="is-authrole-of" multiple="yes"
     * @ejb:transaction type="Supports"
     *
     * @jonas.ejb-relation
     * 	  pk-composite="true"
     *    ejb-relation-name="AuthRoles"
     *    ejb-relationship-role-name1="is-authrole-of"
     *    jdbc-table-name="BNAUTHROLE_BNUSER"
     *    foreign-key-jdbc-name1="BnUser_id"
     *
     * @jboss.relation-table
     *      table-name="BNAUTHROLE_BNUSER"
     *      create-table="true"
     *      remove-table="false"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "BnUser_id"
     */
    public abstract Collection getBnUsers() ;

    /**
     * Set the AuthRoles of this project
     *
     * @param pUsers users of the  role
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnUsers( Collection pUsers);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnAuthRoleValue getBnAuthRoleValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnAuthRoleValue(BnAuthRoleValue v);

   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnAuthRole based on the supplied BnAuthRole Value Object.
   *
   * @param pAuthRole The data used to create the BnAuthRole.
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnAuthRolePK ejbCreate( BnAuthRoleValue pAuthRole )
      throws
         EJBException,
         CreateException
   {
       this.setId(hero.interfaces.BnAuthRoleUtil.generateGUID(this));
       return null;
   }

   public void ejbPostCreate( BnAuthRoleValue pAuthRole )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnAuthRoleValue(pAuthRole);
   }

   
   public void ejbPostCreate( BnAuthRoleData pAuthRole )
   {
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }
}
