package hero.entity;

import hero.interfaces.BnEdgePK;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnEdgeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;

import java.sql.Date;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnEdge
 *
 * @author Fran?ois Charoy
 * @version $Revision: 1.14 $
 *
 * @ejb:bean name="BnEdge"
 *           display-name="BnEdge Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnEdge"
 *           local-jndi-name="ejb/hero/BnEdge_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 *
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnEdge"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(a) FROM BnEdge a"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnEdge findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Remote"
 *     query="SELECT OBJECT(n) FROM BnEdge n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnEdgeLocal findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Local"
 *     query="SELECT OBJECT(e) FROM BnEdge e WHERE e.name = ?1 AND e.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnEdge"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnEdgeLight"
 *
 * @jboss.persistence
 *    table-name="BnEdge"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnEdgeBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnEdge's id.
   *
   * @return Returns an int representing the id of this BnEdge.
   *
   * @ejb.value-object
   *    match="light"
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @ejb:interface-method view-type="local"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnEdge's id.
   *
   * @param pId The id of this BnEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setId( String pId );

   /**
   * Retrieve the BnEdge's name.
   *
   * @return Returns a string representing the name of this BnEdge.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the BnEdge's name.
   *
   * @param pName The name of this BnEdge. Is set at creation time.
   * @ejb:transaction type="Required"
  **/
   public abstract void setName( String pName );

    /**
     * @return  the project of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Edges" role-name="is-edge-of" target-ejb="BnProject" cascade-delete="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectEdgeIdFK"
     *
     **/
    public abstract BnProjectLocal getBnProject();

    /**
     * Set the BnProject Name.
     *
     * @param pName The Name of this BnEdge.  Is set at creation time.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject(BnProjectLocal pProject);

    /**
     * @ejb.value-object
     *    match="light"
     * @return  the state of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:persistence column-name="state"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract int getState();

    /**
     * Set the State value.
     *
     * @param pstate The new state of this BnEdge.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setState(int state);

    /**
     * @ejb.value-object
     *    match="light"
     * @return  the condition of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:persistence column-name="edgecondition"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract String getCondition();

    /**
     * Set the Condition value.
     *
     * @param condition The new state of this BnEdge.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCondition(String condition);

    /**
     * @return  the In BnNode of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InBnNode" role-name="has-node-in" target-ejb="BnNode" cascade-delete="yes"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnNodeValue"
     *       aggregate-name="InBnNode"
     * @ejb:transaction type="Supports"
     *
     * @jonas.field-mapping
     *    foreign-key-jdbc-name="fk_innode"
     *    key-jdbc-name="id"
     *
     *  @jboss.relation
     *        fk-column="fk_innode"
     *        related-pk-field="id"
     *
     **/
    public abstract BnNodeLocal getInBnNode();

   /**
   * Set the BnProject Name.
   *
   * @param pNode The Name of this BnEdge.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setInBnNode(BnNodeLocal pNode);

    /**
     * @return  the Out BnNode of the BnEdge
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutBnNode" role-name="has-node-out" target-ejb="BnNode" cascade-delete="yes"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnNodeValue"
     *       aggregate-name="OutBnNode"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "NodeEdgeIdFK"
     **/
    public abstract BnNodeLocal getOutBnNode();

   /**
   * Set the BnProject Name.
   *
   * @param pNode The Name of this BnEdge.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutBnNode(BnNodeLocal pNode);
    /**
     * @return Return the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     *
     **/

    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @ejb.value-object
     *    match="light"
     * @return Return the modification date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="modificationdate"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Date getModificationDate();

    /**
     * @param pDate the creation date of the BnEdge
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setModificationDate(Date pDate);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnEdgeValue getBnEdgeValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnEdgeValue(BnEdgeValue v);

	/**
	  * @ejb.interface-method
	  * @ejb:transaction type="Supports"
	  */
	 public abstract BnEdgeLightValue getBnEdgeLightValue();

	 /**
	  * @ejb.interface-method
	  * @ejb:transaction type="Required"
	  */
	 public abstract void setBnEdgeLightValue(BnEdgeLightValue v);

   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnEdge based on the supplied BnEdge Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnEdgePK ejbCreate()
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
   	this.setId(hero.interfaces.BnEdgeUtil.generateGUID(this));
    this.setName("edge"+this.getId());
    return null;
   }

   /**
   * Create a BnEdge based on the supplied BnEdge Value Object. (c'est merdique)
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnEdgePK ejbCreate(BnEdgeValue pEdge)
      throws
         CreateException
   {
       this.setId(hero.interfaces.BnEdgeUtil.generateGUID(this));

       return null;
   }

   public void ejbPostCreate(BnEdgeValue pEdge)
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnEdgeValue(pEdge);
   }


   public void ejbPostCreate( )
   {
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
