/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnInitiatorMapperBean.java,v 1.6 2006/08/21 09:56:42 brice Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnInitiatorMapperPK;
import hero.interfaces.BnInitiatorMapperValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnInitiatorMapper
 *
 * @author Marc Blachon
 * @version $Revision: 1.6 $
 *
 * @ejb:bean name="BnInitiatorMapper"
 *           display-name="BnInitiatorMapper Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnInitiatorMapper"
 *           local-jndi-name="ejb/hero/BnInitiatorMapper_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnInitiatorMapper"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(n) FROM BnInitiatorMapper n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByProject(java.lang.String ProjectId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnInitiatorMapper p Where p.bnProject.id = ?1"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.InitiatorMapperLocal findByName(java.lang.String name)" view-type="local"
 *     query="SELECT OBJECT(p) From BnInitiatorMapper p Where p.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnInitiatorMapper"
 *
 * @jboss.persistence
 *    table-name="BnInitiatorMapper"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnInitiatorMapperBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnInitiatorMapper's id.
   *
   * @return Returns an int representing the id of this BnInitiatorMapper.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the Mapper's id.
   *
   * @param pId The id of this Mapper. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );


   /** Retrieve the project associated with the mapper.
	*
	* @return Returns a String representing the Name of the project.
	* @ejb:interface-method view-type="both"
	* @ejb:relation name="mapper-project" project-name="mapper2project" cascade-delete="yes" role-name="has-project"
	* @ejb:transaction type="Supports"
	* 
    * @jonas.field-mapping
    *    foreign-key-jdbc-name="fk_project"
    *    key-jdbc-name="id"
	*
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectInitiatorIdFK"
	**/
    public abstract BnProjectLocal getBnProject();


    /**
     * Set the BnProject.
     *
     * @ejb:interface-method view-type="local"
     * @param pProject The BnProject of this InitiatorMapper.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject( BnProjectLocal pProject );

   /**
   * Retrieve the Name value.
   *
   * @return Returns an String representing the Mapper Name value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the Name value.
   * @ejb:interface-method view-type="both"
   * @param key The Name of this Mapper.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String mapperName );


   /**
   * Retrieve the Type value.
   *
   * @return Returns an int representing the Type value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();

   /**
   * Set the Type value.
   * @ejb:interface-method view-type="both"
   * @param value The Type value of this Mapper.
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int mapperType );

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnInitiatorMapperValue getBnInitiatorMapperValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnInitiatorMapperValue(BnInitiatorMapperValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a Mapper based on the supplied Mapper Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnInitiatorMapperPK ejbCreate(BnInitiatorMapperValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setId(hero.interfaces.BnInitiatorMapperUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnInitiatorMapperValue prp )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnInitiatorMapperValue(prp);
   }

   
   /**
   * Create a Mapper based on the supplied  parameters.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnInitiatorMapperPK ejbCreate(String mapperName, int mapperType)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setName(mapperName);
      this.setType(mapperType);
      this.setId(hero.interfaces.BnInitiatorMapperUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }
}
