/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnIterationBean.java,v 1.12 2006/08/21 09:56:42 brice Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnIterationPK;
import hero.interfaces.BnIterationValue;
import hero.interfaces.BnIterationLightValue;
import hero.interfaces.InvalidValueException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;


 /**
 * The Entity bean represents a BnIteration
 *
 * @ejb:bean name="BnIteration"
 *           display-name="Iteration"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnIteration"
 *           local-jndi-name="ejb/hero/BnIteration_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnIteration"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(p) FROM BnIteration p"
 *     transaction-type="Supports"
 *     
 * @ejb:finder signature="java.util.Collection findByProject(java.lang.String projectId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnIterationLocal findByFromTo(java.lang.String from, java.lang.String to, java.lang.String projectId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?3 AND p.fromNode=?1 AND p.toNode=?2"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByFrom(java.lang.String projectId, java.lang.String from)" view-type="local"
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1 AND p.fromNode=?2"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByTo(java.lang.String projectId, java.lang.String to)" view-type="local"
 *     query="SELECT OBJECT(p) From BnIteration p Where p.bnProject.id = ?1 AND p.toNode=?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnIteration"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnIterationLight"
 *
 * @jboss.persistence
 *    table-name="BnIteration"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnIterationBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnIteration  id.
   *
   * @return Returns an int representing the id of this BnIteration.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the Iteration id.
   *
   * @param pId The id of this NodeIteration. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );


    /** Retrieve the BnProject.
     *
     * @return Returns a BnProjectLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="projectIte" role-name="iteration-by" cascade-delete="yes"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectIterationIdFK"
     **/
    public abstract BnProjectLocal getBnProject();

    /**
     * Set the BnProject.
     *
     * @ejb:interface-method view-type="local"
     * @param pProject The BnProject of this Property.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject( BnProjectLocal pProject );


   /**
   * Retrieve the from node.
   *
   * @return Returns an String representing the from node value of this Iteration.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="fromnode"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getFromNode();

   /**
   * Set the from node value.
   * @ejb:interface-method view-type="both"
   * @param from The from node of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setFromNode( String from );

   /**
   * Retrieve the to node.
   *
   * @return Returns an String representing the to node value of this Iteration.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="tonode"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getToNode();

   /**
   * Set the to node value.
   * @ejb:interface-method view-type="both"
   * @param to The to node value of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setToNode( String to );

   /**
   * Retrieve the condition value.
   *
   * @return Returns an String representing the condition value of this Iteration.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="iterationcondition"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getCondition();

   /**
   * Set the condition value.
   * @ejb:interface-method view-type="both"
   * @param condition The condition value of this Iteration.
   * @ejb:transaction type="Required"
   **/
   public abstract void setCondition( String condition );

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnIterationValue getBnIterationValue();

	/**
	 * @ejb.interface-method
	 * @ejb:transaction type="Supports"
	 */
	public abstract BnIterationLightValue getBnIterationLightValue();


    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnIterationValue(BnIterationValue v);

	/**
	 * @ejb.interface-method
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnIterationLightValue(BnIterationLightValue v);




   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a NodeIteration based on the supplied Property Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnIterationPK ejbCreate(BnIterationLightValue nd )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setId(hero.interfaces.BnIterationUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnIterationLightValue nd )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnIterationLightValue(nd);
   }

   
   /**
   * Create a NodeIteration based on the supplied  key  and value.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnIterationPK ejbCreate(String from, String to, String condition)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setFromNode(from);
       this.setToNode(to);
	   this.setCondition(condition);
	   this.setId(hero.interfaces.BnIterationUtil.generateGUID(this));

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
