/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnNextNumberBean.java,v 1.8 2006/08/21 09:56:41 brice Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.InvalidValueException;
import hero.interfaces.BnNextNumberPK;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;


 /**
 * The Entity bean represents a BnNextNumber
 *
 * @ejb:bean name="BnNextNumber"
 *           display-name="NextNumber"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNextNumber"
 *           local-jndi-name="ejb/hero/BnNextNumber_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnNextNumber"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(p) FROM BnNextNumber p"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="hero.interfaces.BnNextNumberLocal findByName(java.lang.String name )" result-type-mapping="Local"
 *     query="SELECT Object(n) From BnNextNumber as n Where n.name = ?1"
 *     transaction-type="Supports"
 *
 * @jboss.persistence
 *    table-name="BnNextNumber"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnNextNumberBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnNextNumber id.
   *
   * @return Returns an int representing the id of this BnNextNumber.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnNextNumber id.
   *
   * @param pId The id of this BnNextNumber.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );

   /**
   * Retrieve the name value.
   *
   * @return Returns an String representing the name
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the name value.
   * @ejb:interface-method view-type="both"
   * @param name The name value.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String name );

   /**
    * Retrieve the max value.
    *
    * @return Returns an int representing the max
    * @ejb:interface-method view-type="both"
    *
    * @ejb:persistence column-name="maxnumber"
    * @ejb:transaction type="Supports"
    **/
    public abstract int getMax();

    /**
    * Set the max value.
    * @ejb:interface-method view-type="both"
    * @param max The max value.
    * @ejb:transaction type="Required"
    **/
    public abstract void setMax( int max );


   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnNextNumber based on the supplied name.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNextNumberPK ejbCreate(String name)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
	   this.setId(hero.interfaces.BnNextNumberUtil.generateGUID(this));
	   this.setName(name);
	   this.setMax(1);

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
