package hero.entity;

import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodePK;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnNodePerformerAssignLocal;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.InvalidValueException;
import hero.util.BonitaServiceLocator;
import hero.util.BonitaServiceException;
import hero.util.HeroException;

import java.util.ArrayList;
import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnUser
 *
 * @author Fran?s Charoy
 * @version $Revision: 1.37 $
 *
 * @ejb:bean name="BnNode"
 *           display-name="BnNode Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNode"
 *           local-jndi-name="ejb/hero/BnNode_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk method-intf="both"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnNode"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(n) FROM BnNode n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnNode findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Remote"
 *     query="SELECT OBJECT(n) FROM BnNode n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnNodeLocal findByName(java.lang.String pName, java.lang.String projectId )" view-type="both" result-type-mapping="Local"
 *     query="SELECT OBJECT(n) FROM BnNode n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReady(java.lang.String projectId, java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE n.bnProject.id = ?1
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND u.name = ?2
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?2)"
 *    transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserExecuting(java.lang.String projectId, java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.bnProject.id = ?1
 *                              AND (n.state = 5 OR n.state =6)
 *                              AND (n.type <> 5)
 *                              AND n.executor = ?2"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllProjects(java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE u.name = ?1
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND ru.id=u.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                          
 *    transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserExecutingAllProjects(java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.executor = ?1
 *                              AND (n.type <> 5)
 * 				                AND (n.state = 5 OR n.state =6)"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserTerminatedAllProjects(java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.executor = ?1
 * 				                AND n.state = 10"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllProjectsByProperty(java.lang.String userName, java.lang.String key, java.lang.String value)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, IN (p.bnProperties) AS prop, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND prop.theKey = ?2
 * 								AND prop.theValue = ?3
 * 			  					AND (prop.bnProject.name = n.bnProject.name)
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                   
 *    transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllActivitiesByProperty(java.lang.String userName, java.lang.String key, java.lang.String value)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, IN (n.bnProperties) AS prop, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND prop.theKey = ?2
 * 								AND prop.theValue = ?3
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                   
 *    transaction-type="Supports"
 *
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllProjectsByTwoPropertiesAND(java.lang.String userName, java.lang.String key1, java.lang.String value1, java.lang.String key2, java.lang.String value2)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, BnProjectProperty AS prop1, BnProjectProperty AS prop2, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND prop1.theKey = ?2
 * 								AND prop1.theValue = ?3
 * 								AND prop2.theKey = ?4
 * 								AND prop2.theValue = ?5
 * 			  					AND (prop1.bnProject.name = n.bnProject.name)
 * 								AND (prop2.bnProject.name = n.bnProject.name)
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                   
 *    transaction-type="Supports"
 *
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllProjectsByTwoPropertiesOR(java.lang.String userName, java.lang.String key1, java.lang.String value1, java.lang.String key2, java.lang.String value2)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, In (n.bnProject.bnProperties) As prop, IN (n.bnProject.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND ((prop.theKey = ?2 AND prop.theValue = ?3) OR (prop.theKey = ?4 AND prop.theValue = ?5))
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                   
 *    transaction-type="Supports"
 *
 *
  * @ejb.finder
 *     signature="Collection findUserReadyAllProjectsByThreePropertiesAND(java.lang.String userName, java.lang.String key1, java.lang.String value1, java.lang.String key2, java.lang.String value2, java.lang.String key3, java.lang.String value3)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, BnProjectProperty AS prop1, BnProjectProperty AS prop2, BnProjectProperty AS prop3, IN (p.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND prop1.theKey = ?2
 * 								AND prop1.theValue = ?3
 * 								AND prop2.theKey = ?4
 * 								AND prop2.theValue = ?5
 * 								AND prop3.theKey = ?6
 * 								AND prop3.theValue = ?7
 * 			  					AND (prop1.bnProject.name = n.bnProject.name)
 * 								AND (prop2.bnProject.name = n.bnProject.name)
 * 								AND (prop3.bnProject.name = n.bnProject.name)
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                
 *    transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReadyAllProjectsByThreePropertiesOR(java.lang.String userName, java.lang.String key1, java.lang.String value1, java.lang.String key2, java.lang.String value2, java.lang.String key3, java.lang.String value3)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, In (n.bnProject.bnProperties) As prop, IN (n.bnProject.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE u.name = ?1
 * 			        			AND ((prop.theKey = ?2 AND prop.theValue = ?3) OR (prop.theKey = ?4 AND prop.theValue = ?5) OR (prop.theKey = ?6 AND prop.theValue = ?7))
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?1)"                                   
 *    transaction-type="Supports"
 *
 *
 * @ejb.finder
 *     signature="Collection findUserExecutingAllProjectsByProperty(java.lang.String userName, java.lang.String key, java.lang.String value)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnProject AS p, IN (p.bnProperties) AS prop
 *                              WHERE n.executor = ?1
 * 								AND prop.theKey = ?2
 * 								AND prop.theValue = ?3
 * 								AND (prop.bnProject.name = n.bnProject.name)
 *                              AND (n.type <> 5)
 * 								AND (n.state = 5 OR n.state =6)"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivities (java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE u.name = ?1 
 *                              AND r.name = ?2
 * 								AND (n.bnProject.type = ?3 OR n.bnProject.type = ?4)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterAll (int state, java.lang.String executor, java.lang.String performer, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.executor = ?2
 *                              AND n.activityPerformer = ?3
 *                              AND n.bnProject.name LIKE ?4
 *                              AND u.name = ?5 
 *                              AND r.name = ?6
 * 								AND (n.bnProject.type = ?7 OR n.bnProject.type = ?8)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoState (java.lang.String executor, java.lang.String performer, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.executor = ?1
 *                              AND n.activityPerformer = ?2
 *                              AND n.bnProject.name LIKE ?3
 *                              AND u.name = ?4 
 *                              AND r.name = ?5
 * 								AND (n.bnProject.type = ?6 OR n.bnProject.type = ?7)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoExecutor (int state, java.lang.String performer, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.activityPerformer = ?2
 *                              AND n.bnProject.name LIKE ?3
 *                              AND u.name = ?4 
 *                              AND r.name = ?5
 * 								AND (n.bnProject.type = ?6 OR n.bnProject.type = ?7)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoPerformer (int state, java.lang.String executor, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.executor = ?2
 *                              AND n.bnProject.name LIKE ?3
 *                              AND u.name = ?4 
 *                              AND r.name = ?5
 * 								AND (n.bnProject.type = ?6 OR n.bnProject.type = ?7)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoProcess (int state, java.lang.String executor, java.lang.String performer, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.executor = ?2
 *                              AND n.activityPerformer = ?3
 *                              AND u.name = ?4 
 *                              AND r.name = ?5
 * 								AND (n.bnProject.type = ?6 OR n.bnProject.type = ?7)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoExecutor (java.lang.String performer, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.activityPerformer = ?1
 *                              AND n.bnProject.name LIKE ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoExecutorNoPerformer (java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND u.name = ?2 
 *                              AND r.name = ?3
 * 								AND (n.bnProject.type = ?4 OR n.bnProject.type = ?5)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoProcess (java.lang.String executor, java.lang.String performer, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.executor = ?1
 *                              AND n.activityPerformer = ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoExecutorNoPerformerNoProcess (int state, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND u.name = ?2 
 *                              AND r.name = ?3
 * 								AND (n.bnProject.type = ?4 OR n.bnProject.type = ?5)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoPerformerNoProcess (java.lang.String executor, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.executor = ?1
 *                              AND u.name = ?2 
 *                              AND r.name = ?3
 * 								AND (n.bnProject.type = ?4 OR n.bnProject.type = ?5)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoExecutorNoProcess (java.lang.String performer, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.activityPerformer = ?1
 *                              AND u.name = ?2 
 *                              AND r.name = ?3
 * 								AND (n.bnProject.type = ?4 OR n.bnProject.type = ?5)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoStateNoPerformer (java.lang.String executor, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.executor = ?1
 *                              AND n.bnProject.name LIKE ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoExecutorNoProcess (int state, java.lang.String performer, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.activityPerformer = ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoExecutorNoPerformer (int state, java.lang.String process, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.bnProject.name LIKE ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findAdminActivitiesFilterNoPerformerNoProcess (int state, java.lang.String executor, java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.state = ?1
 *                              AND n.executor = ?2
 *                              AND u.name = ?3 
 *                              AND r.name = ?4
 * 								AND (n.bnProject.type = ?5 OR n.bnProject.type = ?6)
 *                              AND r.bnProject.id = n.bnProject.id
 *                              AND ru.id=u.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserReadyModelActivity(java.lang.String model, java.lang.String node, java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND u.name = ?3
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND n.name = ?2
 *                              AND r.id = n.bnRole.id
 *                              AND ru.id=u.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?3)"
 *    transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserTerminatedModelActivity(java.lang.String model, java.lang.String node, java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND n.name = ?2
 *                              AND n.executor = ?3
 * 				                AND n.state = 10"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserModelReady(java.lang.String model,java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, IN (n.bnProject.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND u.name = ?2
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserModelTerminated(java.lang.String model,java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND n.executor = ?2
 *                              AND n.state = 10"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserInstancesProjectNodes(java.lang.String model,java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n
 *                              WHERE n.bnProject.name LIKE ?1
 *                              AND n.bnProject.creator = ?2
 *                              AND (n.state = 1 OR n.state =3 OR n.state=6)"
 *     transaction-type="Supports"
 *
 * @ejb.finder
 *     signature="Collection findUserInstanceNodes(java.lang.String instance,java.lang.String userName)"
 *     view-type="local" result-type-mapping="Local"
 *     query="SELECT DISTINCT OBJECT(n) FROM BnNode AS n, IN (n.bnProject.bnUsers) AS u, IN (u.bnRoles) AS r
 *                              WHERE n.bnProject.name = ?1
 *                              AND u.name = ?2
 *                              AND (n.state = 1 OR n.state =3)
 *                              AND r.id = n.bnRole.id
 *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?2)"
 *     transaction-type="Supports"
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnNode"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnNodeLight"
 *
 * @jboss.persistence
 *    table-name="BnNode"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnNodeBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnNode's id.
   *
   * @return Returns an int representing the id of this BnNode.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:interface-method view-type="local"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   *   	key-jdbc-name="fk_node"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnNode's id.
   *
   * @param pId The id of this BnNode. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );

   /**
   * Retrieve the BnNode's type.
   *
   * @return Returns an int representing the type of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();

   /**
   * Set the BnNode's type.
   *
   * @param pType The type of this BnNode. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int pType );

   /**
   * Retrieve the BnNode's state.
   *
   * @return Returns an int representing the state of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="state"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getState();

   /**
   * Set the BnNode's state.
   *
   * @param pState The type of this BnNode. Is set at creation time.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setState( int pState );

   /**
   * Retrieve the BnNode's anticipation mode.
   *
   * @return Returns a boolean indicating if the node is anticipable
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="anticipable"
   * @ejb:transaction type="Supports"
   **/
   public abstract boolean getAnticipable();

   /**
   * Set the BnNode's Anticipable mode.
   *
   * @param pAnticipate The mode of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setAnticipable( boolean pAnticipable);

   /**
   * Retrieve the BnNode's Name.
   *
   * @return Returns an int representing the Name of this BnNode.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the BnNode's Name.
   *
   * @param pName The Name of this BnNode.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String pName );

	/**
	* Retrieve the BnNode's Reference.
	*
	* @return Returns an int representing the Reference pointed by this Node (only for subProcesses)
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="Reference"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getReference();

	/**
	 * Set the BnNode's Reference.
	 *
	 * @param nReference  Reference pointed by this Node (only for subProcesses)
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 */
	public abstract void setReference(String nReference);
   
   
   /**
   * Retrieve the BnNode's Description.
   *
   * @return Returns the description of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="description"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getDescription();

   /**
   * Set the BnNode's Description.
   *
   * @param pDescription The Description of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setDescription( String pDescription );

   /**
   * Retrieve the BnNode's activityPerformer.
   *
   * @return Returns the activityPerformer of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="activityPerformer"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getActivityPerformer();

   /**
   * Set the BnNode's activityPerformer.
   *
   * @param pActivityPerformer The ActivityPerformer of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setActivityPerformer( String pActivityPerformer );

   /**
   * Retrieve the BnNode's StartDate.
   *
   * @return Returns the start date of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="startdate"
   * @ejb:transaction type="Supports"
   **/
   public abstract Date getStartDate();

   /**
   * Set the BnNode's StartDate.
   *
   * @param pStartDate The StartDate of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setStartDate( Date pStartDate );

   /**
   * Retrieve the BnNode's EndDate.
   *
   * @return Returns the end date of the node
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="enddate"
   * @ejb:transaction type="Supports"
   **/
   public abstract Date getEndDate();

   /**
   * Set the BnNode's EndDate.
   *
   * @param pEndDate The end date of this BnNode.
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   **/
   public abstract void setEndDate( Date pEndDate );

   /**
    * Retrieve the BnNode's Deadlines.
    *
    * @return Returns the collection of deadlines of the node
    *
    * @ejb:interface-method view-type="both"
    * @ejb.value-object
    *    match="light"
    *
    * @ejb:persistence column-name="deadlines"
    * @ejb:transaction type="Supports"
    *
    * @jboss.relation-table create-table="true" remove-table="false"
    **/
    public abstract Collection getDeadlines();

    /**
    * Set the BnNode's Deadlines.
    *
    * @param pDeadlines The Collection of Deadlines of this BnNode.
    * @ejb:interface-method view-type="both"
    * @ejb:transaction type="Required"
    **/
    public abstract void setDeadlines( Collection pDeadlines );

    /**
     * Retrieve the BnNode's RelativeDeadlines.
     *
     * @return Returns the collection of relative deadlines of the node
     *
     * @ejb:interface-method view-type="both"
     * @ejb.value-object
     *    match="light"
     *
     * @ejb:persistence column-name="relativedeadlines"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     **/
     public abstract Collection getRelativeDeadlines();

     /**
     * Set the BnNode's Relative Deadlines.
     *
     * @param pDeadlines The Collection of Relative Deadlines of this BnNode.
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     **/
     public abstract void setRelativeDeadlines( Collection pDeadlines );


    /** Retrieve the BnNode's Creator.
     * @return Returns the node creator
     *
     * @ejb:interface-method view-type="both"
     * @ejb.value-object
     *    match="light"
     *
     * @ejb:persistence column-name="creator"
     * @ejb:transaction type="Supports"
     **/
    public abstract String getCreator();

    /**
     * Set the BnNode's Creator.
     *
     * @param pCreator The creator of this BnNode.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCreator( String pCreator );

    /** Retrieve the BnNode's Executor.
     * @return Returns the node executor
     *
     * @ejb:interface-method view-type="both"
     * @ejb.value-object
     *    match="light"
     *
     * @ejb:persistence column-name="executor"
     * @ejb:transaction type="Supports"
     **/

    public abstract String getExecutor();

    /**
     * Set the BnNode's Executor.
     *
     * @ejb:interface-method view-type="local"
     * @param pCreator The creator of this BnNode.  Is set at creation time.
     * @ejb:transaction type="Required"
     **/
    public abstract void setExecutor( String pExecutor );

    /** Retrieve the BnNode's BnRole.
     *
     * @return Returns a String representing the ole associated to the BnNode.
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="NodeRole" role-name="has-role" cascade-delete="yes"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnRoleValue"
     *       aggregate-name="BnRole"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "roleNodeIdFK"
     **/
    public abstract BnRoleLocal getBnRole();

    /**
     * Set the BnNode's Creator.
     *
     * @param pCreator The creator of this BnNode.  Is set at creation time.
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnRole(BnRoleLocal role);

	/** Retrieve the performerAssign for the node.
	 *
	 * @return Returns the node performerAssign.
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="performerAssign-node" role-name="node2performerAssign"
	 * @ejb:value-object match="*"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodePerformerAssignValue"
	 *       aggregate-name="BnNodePerformerAssign"
	 * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "nodePerformerNodeIdFK"
	 **/
   public abstract BnNodePerformerAssignLocal getBnNodePerformerAssign() ;

   /**
	* Set the Nodes of this performerAssign
	*
	* @param pPerformerAssign performerAssign of the  node
	* @ejb:transaction type="Required"
	**/
   public abstract void setBnNodePerformerAssign( BnNodePerformerAssignLocal pPerformerAssign);

    /**
     * @return  the project of the BnNode
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Nodes" role-name="is-node-of" cascade-delete="yes"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnProjectLightValue"
     *       aggregate-name="BnProject"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectNodeIdFK"
     **/
    public abstract BnProjectLocal getBnProject();

   /**
   * Set the BnProject Name.
   *
   * @param pName The Name of this BnNode.  Is set at creation time.
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnProject(BnProjectLocal pProject);

    /**
     * @return  the outgoing edges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="InBnNode" role-name="has-edges-in" target-multiple="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     **/
    public abstract Collection getOutBnEdges();

   /**
   * Set the Outgoing edges of the node
   *
   * @param pEdges the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setOutBnEdges(Collection pEdges);

    /**
     * @return  the ingoing edges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutBnNode" role-name="has-edges-out" target-multiple="yes"
     * @ejb:transaction type="Supports"
     *
     **/
    public abstract Collection getInBnEdges();

   /**
   * Set the Ingoing edges of the node
   *
   * @param pEdges the ingoing edges of the node
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
    public abstract void setInBnEdges(Collection pEdges);

    /**
     * @return  the ingoing AgentEdges of the node
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="OutNodeInAgentEdge" role-name="has-edges-out" target-multiple="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     **/
    public abstract Collection getInBnAgentEdges();

    /**
     * Set the Ingoing AgentEdges of the node
     *
     * @param pAgentEdges the ingoing AgentEdges of the node
     * @ejb:transaction type="Required"
     **/
    public abstract void setInBnAgentEdges(Collection pAgentEdges);

    /**
     * @return Return the creation date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     * @ejb.value-object
	 *      match="light"
     *
     **/
    public abstract Date getCreationDate();

    /**
     * @param pDate the creation date of the BnNode
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     *
     **/
    public abstract void setCreationDate(Date pDate);

    /**
     * @return  the properties of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodeprp" role-name="is-property-of"
     * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodePropertyValue"
	 *       aggregate-name="BnPropertie"
	 *       members="hero.interfaces.BnNodePropertyLocal"
	 *       members-name="BnPropertie"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     **/
    public abstract Collection getBnProperties();

   /**
   * Set the properties of the node
   * @ejb:interface-method view-type="both"
   * @param pPrp the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnProperties(Collection pPrp);

   /**
     * @return  the Hooks of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodehook" role-name="is-hook-of"
     * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodeHookValue"
	 *       aggregate-name="BnHook"
	 *       members="hero.interfaces.BnNodeHookLocal"
	 *       members-name="BnHook"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     **/
    public abstract Collection getBnHooks();

   /**
   * Set the Hooks of the node
   * @ejb:interface-method view-type="both"
   * @param pHook the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnHooks(Collection pHook);


   /**
     * @return  the Hooks of the node
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="nodeinterhook" role-name="is-interhook-of"
     * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnNodeInterHookValue"
	 *       aggregate-name="BnInterHook"
	 *       members="hero.interfaces.BnNodeInterHookLocal"
	 *       members-name="BnInterHook"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     **/
    public abstract Collection getBnInterHooks();

   /**
   * Set the Hooks of the node
   * @ejb:interface-method view-type="both"
   * @param pHook the outgoing edges of the node
   * @ejb:transaction type="Required"
   **/
    public abstract void setBnInterHooks(Collection pHook);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodeValue getBnNodeValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnNodeValue(BnNodeValue v);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodeLightValue getBnNodeLightValue();

	/**
	 * @ejb.interface-method
	 * @ejb:transaction type="Required"
	 */
	public abstract void setBnNodeLightValue(BnNodeLightValue v);

   /**
     * @return true if the BnNode is executing
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */
    public boolean isExecuting() {
	return (this.getState()==hero.interfaces.Constants.Nd.EXECUTING ||
		this.getState()==hero.interfaces.Constants.Nd.ANTICIPATING);
    }

   /**
     * @return true if the BnNode is TErminated
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */
    public boolean isTerminated() {
	return (this.getState()==hero.interfaces.Constants.Nd.TERMINATED ||
		this.getState()==hero.interfaces.Constants.Nd.FINISHED);
    }

   /**
     * @return true if the BnNode is Cancelled
     * @ejb.interface-method view-type="local"
     * @ejb:transaction type="Supports"
     */
    public boolean isCancelled() {
	return (this.getState()==hero.interfaces.Constants.Nd.DEAD);
    }

	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"    
     *     query="SELECT DISTINCT n.bnProject.name FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
     *                   WHERE u.name = ?1 
     *                   AND r.name = ?2
     * 					 AND n.bnProject.type = ?3
     *                   AND r.bnProject.id = n.bnProject.id
     *                   AND ru.id=u.id"
	 */
	public abstract Collection ejbSelectAdminActivitiesByProcess(java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetAdminActivitiesByProcess(String userName, String role, String instance, String cooperative) throws HeroException {
		try {
			return (ejbSelectAdminActivitiesByProcess(userName, role, instance, cooperative));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"    
     *     query="SELECT DISTINCT n.executor FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
     *                   WHERE u.name = ?1 
     *                   AND r.name = ?2
     * 					 AND (n.bnProject.type = ?3 OR n.bnProject.type = ?4)
     *                   AND r.bnProject.id = n.bnProject.id
     *                   AND ru.id=u.id"
	 */
	public abstract Collection ejbSelectAdminActivitiesByExecutor(java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetAdminActivitiesByExecutor(String userName, String role, String instance, String cooperative) throws HeroException {
		try {
			return (ejbSelectAdminActivitiesByExecutor(userName, role, instance, cooperative));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"    
     *     query="SELECT DISTINCT n.activityPerformer FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
     *                   WHERE u.name = ?1 
     *                   AND r.name = ?2
     * 					 AND (n.bnProject.type = ?3 OR n.bnProject.type = ?4)
     *                   AND r.bnProject.id = n.bnProject.id
     *                   AND ru.id=u.id"
	 */
	public abstract Collection ejbSelectAdminActivitiesByPerformer(java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetAdminActivitiesByPerformer(String userName, String role, String instance, String cooperative) throws HeroException {
		try {
			return (ejbSelectAdminActivitiesByPerformer(userName, role, instance, cooperative));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"    
     *     query="SELECT DISTINCT n.state FROM BnNode AS n, BnRole as r, IN (n.bnProject.bnUsers) AS u, IN (r.bnUsers) as ru
     *                   WHERE u.name = ?1 
     *                   AND r.name = ?2
     * 					 AND (n.bnProject.type = ?3 OR n.bnProject.type = ?4)
     *                   AND r.bnProject.id = n.bnProject.id
     *                   AND ru.id=u.id"
	 */
	public abstract Collection ejbSelectAdminActivitiesByState(java.lang.String userName, java.lang.String role ,java.lang.String instance, java.lang.String cooperative) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetAdminActivitiesByState(String userName, String role, String instance, String cooperative) throws HeroException {
		try {
			return (ejbSelectAdminActivitiesByState(userName, role, instance, cooperative));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnNode based on the supplied BnNode Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePK ejbCreate(BnNodeValue nd )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
	   try {
		  BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
		  this.setId(hero.interfaces.BnNodeUtil.generateGUID(this));
		  this.setDeadlines(new ArrayList());
		  this.setRelativeDeadlines(new ArrayList());

	   } catch (BonitaServiceException e) {
		  throw new CreateException(e.getMessage());
	   }
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnNodeValue nd )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnNodeValue(nd);
   }

   
   /**
   * Create a BnNode based on the supplied Name and type.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePK ejbCreate(String name, int type )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(name);
       this.setType(type);
       this.setState(hero.interfaces.Constants.Nd.READY);
       this.setAnticipable(true);
       this.setDeadlines(new ArrayList());
	   this.setRelativeDeadlines(new ArrayList());
	   this.setId(hero.interfaces.BnNodeUtil.generateGUID(this));

       // This is only possible in CMPs. Otherwise return a valid PK.
       return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
