/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnNodePerformerAssignBean.java,v 1.10 2006/08/21 09:56:41 brice Exp $
 *
--------------------------------------------------------------------------
 */

package hero.entity;

import hero.interfaces.BnNodePerformerAssignPK;
import hero.interfaces.BnNodePerformerAssignValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.InvalidValueException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnNodePerformerAssign
 *
 * @author Marc Blachon
 * @version $Revision: 1.10 $
 *
 * @ejb:bean name="BnNodePerformerAssign"
 *           display-name="BnNode Performer Assign Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnNodePerformerAssign"
 *           local-jndi-name="ejb/hero/BnNodePerformerAssign_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnNodePerformerAssign"
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(n) FROM BnNodePerformerAssign n"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByNode(java.lang.String NodeId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnNodePerformerAssign p Where p.bnNode.id = ?1"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.NodeMapperLocal findByName(java.lang.String name)" view-type="local"
 *     query="SELECT OBJECT(p) From BnNodePerformerAssign p Where p.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnNodePerformerAssign"
 *
 * @jboss.persistence
 *    table-name="BnNodePerformerAssign"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnNodePerformerAssignBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnNodePerformerAssign's id.
   *
   * @return Returns an int representing the id of this BnNodePerformerAssign.
   *
   * @ejb:pk-field
   *
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the PerformerAssign's id.
   *
   * @param pId The id of this PerformerAssign. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );


   /** Retrieve the node associated with the PerformerAssign.
	*
	* @return Returns a String representing the Name of the node.
	* @ejb:interface-method view-type="both"
	* @ejb:relation name="performerAssign-node" role-name="performerAssign2node" cascade-delete="yes"
	* @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "bnNodePerformerIdFK"
	*
	**/
    public abstract BnNodeLocal getBnNode();


    /**
     * Set the BnNode.
     *
     * @ejb:interface-method view-type="local"
     * @param pNode The BnNode of this PerformerAssign.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnNode( BnNodeLocal pNode );

   /**
   * Retrieve the Name value.
   *
   * @return Returns an String representing the PerformerAssign Name value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the Name value.
   * @ejb:interface-method view-type="both"
   * @param key The Name of this PerformerAssign.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String performerAssignName );


   /**
   * Retrieve the Type value.
   *
   * @return Returns an int representing the Type value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="type"
   * @ejb:transaction type="Supports"
   **/
   public abstract int getType();

   /**
   * Set the Type value.
   * @ejb:interface-method view-type="both"
   * @param value The Type value of this PerformerAssign.
   * @ejb:transaction type="Required"
   **/
   public abstract void setType( int performerAssignType );

   /**
   * Retrieve the Name of the property.
   *
   * @return Returns an String representing the property Name value.
   *
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="property"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getPropertyName();

   /**
   * Set the property name value.
   * @ejb:interface-method view-type="both"
   * @param propertyName The Name of the property containing the user.
   * @ejb:transaction type="Required"
   **/
   public abstract void setPropertyName( String propertyName );

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnNodePerformerAssignValue getBnNodePerformerAssignValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnNodePerformerAssignValue(BnNodePerformerAssignValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a PerformerAssign based on the supplied PerformerAssign Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePerformerAssignPK ejbCreate(BnNodePerformerAssignValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setId(hero.interfaces.BnNodePerformerAssignUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnNodePerformerAssignValue prp )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnNodePerformerAssignValue(prp);
   }

   
   /**
   * Create a PerformerAssign based on the supplied  parameters.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnNodePerformerAssignPK ejbCreate(String performerAssignName, int performerAssignType, String propertyName)
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setName(performerAssignName);
       this.setType(performerAssignType);
       this.setPropertyName(propertyName);
       this.setId(hero.interfaces.BnNodePerformerAssignUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
