package hero.entity;

import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectPK;
import hero.interfaces.BnProjectValue;
import hero.interfaces.InvalidValueException;
import hero.interfaces.SequenceGeneratorLocal;
import hero.interfaces.SequenceGeneratorLocalHome;
import hero.interfaces.SequenceGeneratorUtil;
import hero.interfaces.BnInitiatorMapperLocal;
import hero.util.HeroException;
import hero.util.values.BonitaProjectValue;

import java.util.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import org.apache.log4j.Category;

/**
 * The Entity bean represents a BnProject
 *
 * @author Fran?ois Charoy
 * @version $Revision: 1.46 $
 *
 * @ejb.bean name="BnProject"
 *           display-name="BnProject Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnProject"
 *           local-jndi-name="ejb/hero/BnProject_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb.permission unchecked="yes"
 * @ejb:pk  class="hero.interfaces.BnProjectPK" generate="true" method-intf="remote"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 *
 * @jonas.shared false
 *
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnProject"
 *
 * @ejb.finder signature="java.util.Collection findAll()"
 *     query="SELECT object(p) FROM BnProject p"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="java.util.Collection findModels(java.lang.String type, java.lang.String status)"
 *     query="SELECT object(p) FROM BnProject as p WHERE p.type = ?1 AND p.status =?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.finder signature="hero.interfaces.BnProject findByName(java.lang.String pName )" result-type-mapping="Remote"
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1"
 *     transaction-type="Supports"
 * @ejb.finder signature="hero.interfaces.BnProjectLocal findByName(java.lang.String pName )"  result-type-mapping="Local"
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.finder signature="hero.interfaces.BnProject findByNameVersion(java.lang.String pName, java.lang.String pVersion )" result-type-mapping="Remote"
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1 AND p.version = ?2"
 *     transaction-type="Supports"
 * @ejb.finder signature="hero.interfaces.BnProjectLocal findByNameVersion(java.lang.String pName, java.lang.String pVersion )"  result-type-mapping="Local"
 *     query="SELECT Object(p) From BnProject as p Where p.name = ?1 AND p.version = ?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.finder signature="java.util.Collection findByUser(java.lang.String userName )"
 *     query="SELECT OBJECT(p) From BnProject as p, in (p.bnUsers) as u Where u.name = ?1"
 *     transaction-type="Supports"
 *
 *
 *     
 * @ejb.finder signature="java.util.Collection findUserInstancesByProjectVersion(java.lang.String projectName, java.lang.String version,java.lang.String userName )"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject AS p, IN (p.bnUsers) u
 * 				 WHERE p.name LIKE ?1
 * 				 AND p.version = ?2
 * 				 AND u.name = ?3"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="java.util.Collection findProjectsByProperty(java.lang.String key, java.lang.String value )"
 *     query="SELECT OBJECT(p) From BnProject as p, BnProjectProperty as prop
 * 					 Where prop.theKey = ?1
 * 					 AND prop.theValue = ?2
 * 					 AND p.id = prop.bnProject.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="java.util.Collection findByAdmin(java.lang.String userName, java.lang.String roleName, java.lang.String type)"
 *     query="SELECT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where u.name = ?1
 * 					AND r.name = ?2
 * 					AND p.type = ?3
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *
 * @ejb.finder signature="java.util.Collection findByAdminFiltersAll(int state, java.lang.String creator, java.lang.String user, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?3
 * 					AND r.name = ?4
 * 					AND pp.theKey = ?5
 * 					AND pp.theValue = ?6
 * 					AND p.type = ?9
 *  				AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoProp(int state, java.lang.String creator, java.lang.String user, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?3
 * 					AND r.name = ?4
 * 					AND p.type = ?7
 *  				AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoRole(int state, java.lang.String creator, java.lang.String user, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?3
 * 					AND pp.theKey = ?4
 * 					AND pp.theValue = ?5
 * 					AND r.name = ?7
 * 					AND p.type = ?8
 *  				AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoUser(int state, java.lang.String creator, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?6
 * 					AND r.name = ?3
 * 					AND pp.theKey = ?4
 * 					AND pp.theValue = ?5
 * 					AND p.type = ?8
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreator(int state, java.lang.String user, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND pp.theKey = ?4
 * 					AND pp.theValue = ?5
 * 					AND p.type = ?8
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoState(java.lang.String creator, java.lang.String user, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.creator = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND pp.theKey = ?4
 * 					AND pp.theValue = ?5
 * 					AND p.type = ?8
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoRoleNoProp(int state, java.lang.String creator, java.lang.String user, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?3
 * 					AND r.name = ?5
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoUserNoProp(int state, java.lang.String creator, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND r.name = ?3
 * 					AND u.name = ?4
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoUserNoRole(int state, java.lang.String creator, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?5
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND r.name = ?6
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoProp(java.lang.String creator, java.lang.String user, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.creator = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoRole(java.lang.String creator, java.lang.String user, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.creator = ?1
 * 					AND u.name = ?2
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND r.name = ?6
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoRole(int state, java.lang.String user, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND u.name = ?2
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND r.name = ?5
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreator(java.lang.String user, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where u.name = ?1
 * 					AND r.name = ?2
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoUserNoRoleNoProp(int state, java.lang.String creator, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND p.creator = ?2
 * 					AND u.name = ?3
 * 					AND r.name = ?4
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoRoleNoProp(int state, java.lang.String user, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?4
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoUserNoProp(int state, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND u.name = ?3
 * 					AND r.name = ?2
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoUserNoRole(int state, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnProperties) as pp, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND pp.theKey = ?2
 * 					AND pp.theValue = ?3
 * 					AND u.name = ?4
 * 					AND r.name = ?5
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoRoleNoProp(java.lang.String creator, java.lang.String user, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.creator = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?4
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoUserNoProp(java.lang.String creator, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.creator = ?1
 * 					AND u.name = ?3
 * 					AND r.name = ?2
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoUserNoRole(java.lang.String creator, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.creator = ?1
 * 					AND u.name = ?4
 * 					AND pp.theKey = ?2
 * 					AND pp.theValue = ?3
 * 					AND r.name = ?5
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoProp(java.lang.String user, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where u.name = ?1
 * 					AND r.name = ?2
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoRole(java.lang.String user, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where u.name = ?1
 * 					AND pp.theKey = ?2
 * 					AND pp.theValue = ?3
 * 					AND r.name = ?5
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoUser(java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where u.name = ?4
 * 					AND r.name = ?1
 * 					AND pp.theKey = ?2
 * 					AND pp.theValue = ?3
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoUserNoRoleNoProp(int state, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND p.type = ?4
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 *     
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoUserNoRoleNoProp(java.lang.String creator, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.creator = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND p.type = ?4
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoRoleNoProp(java.lang.String user, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where u.name = ?1
 * 					AND r.name = ?3
 * 					AND p.type = ?4
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoUserNoProp(java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where u.name = ?2
 * 					AND r.name = ?1
 * 					AND p.type = ?4
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoCreatorNoUserNoRole(java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where u.name = ?3
 * 					AND pp.theKey = ?1
 * 					AND pp.theValue = ?2
 * 					AND r.name = ?4
 * 					AND p.type = ?5
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoUser(int state, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.state = ?1
 * 					AND r.name = ?2
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 *  * @ejb.finder signature="java.util.Collection findByAdminFiltersNoCreatorNoProp(int state, java.lang.String user, java.lang.String role, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r
 * 					Where p.state = ?1
 * 					AND u.name = ?2
 * 					AND r.name = ?3
 * 					AND p.type = ?6
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 *  * @ejb.finder signature="java.util.Collection findByAdminFiltersNoStateNoUser(java.lang.String creator, java.lang.String role, java.lang.String propkey, java.lang.String propval, java.lang.String userName, java.lang.String userRole, java.lang.String type)"
 *     query="SELECT DISTINCT OBJECT(p) From BnProject as p, in (p.bnRoles) as pr, in (p.bnUsers) as u, in (u.bnRoles) as r, in (p.bnProperties) as pp
 * 					Where p.creator = ?1
 * 					AND u.name = ?5
 * 					AND pp.theKey = ?3
 * 					AND pp.theValue = ?4
 * 					AND r.name = ?2
 * 					AND p.type = ?7
 * 					AND r.id = pr.id"
 *     transaction-type="Supports"
 * 
 * @ejb.value-object
 *    match="*"
 *    name="BnProject"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnProjectLight"
 *
 * @ejb:transaction type="Required"
 * 
 * @jboss.persistence
 *    table-name="BnProject"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 *
 * @version $Revision: 1.46 $
 * @author $Author: dparisek $
 **/
public abstract class BnProjectBean implements EntityBean {

	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category log =	Category.getInstance(BnProjectBean.class);
	public EntityContext mContext;


	// -------------------------------------------------------------------------
	// Properties (Getters/Setters)
	// -------------------------------------------------------------------------

	/**
	* Retrieve the BnProject's id.
	*
	* @return Returns an int representing the id of this BnUser.
	*
	* @ejb:pk-field
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="id"
	* @jonas.cmp-field-jdbc-mapping
	*    field-name="id"
	*    jdbc-field-name="id"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getId();

	/**
	* Set the BnProject's id.
	*
	* @param pId The id of this BnUser. Is set at creation time.
	* @ejb:transaction type="Required"
	**/
	public abstract void setId(String pId);

	/**
    * Get parent of this project (or the name of the project if is not a child)
    *
    * @return Returns a String representing the parent name
    *
    * @ejb:interface-method view-type="both"
    * @ejb.value-object
    *      match="light"
    *
    * @ejb:persistence column-name="parent"
    * @ejb:transaction type="Supports"
    **/

    public abstract String getParent();

    /**
    * Set the parent of this project (or the name of the project if is not a child)
    *
    * @param parent of this project. Is set at creation time.
    * @ejb:interface-method view-type="both"
    * @ejb:transaction type="Required"
    **/
    public abstract void setParent(String parent);

	/**
	* Retrieve the BnProject's Name.
	*
	* @return Returns an int representing the Name of this BnProject.
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="name"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getName();

	/**
	 * Set the BnProject's Name.
	 *
	 * @param pName The Name of this BnProject. Is set at creation time.
	 * @ejb:transaction type="Required"
	 */
	public abstract void setName(String pName);

	/**
	* Retrieve the BnProject's Version.
	*
	* @return Returns an int representing the Version of this BnProject.
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="version"
	* @ejb:transaction type="Supports"
	**/
	public abstract String getVersion();

	/**
	 * Set the BnProject's Version.
	 *
	 * @param pVersion The Version of this BnProject. Is set at creation time.
	 * @ejb:transaction type="Required"
	 */
	public abstract void setVersion(String pVersion);

	/**
	 * Get the status of this project (Active/Hidden)
	 *
	 * @return Returns a String representing the status name
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb.value-object match="light"
	 *
	 * @ejb:persistence column-name="status"
	 * @ejb:transaction type="Supports"
	 */

	public abstract String getStatus();

	/**
	 * Set the status of this project (Active/Hidden)
	 *
	 * @param status of this project. Is set at creation time.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 */
	public abstract void setStatus(String status);

	/**
	 * Get the type of this project (model/cooperative/instance)
	 *
	 * @return Returns a String representing the type name
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb.value-object match="light"
	 *
	 * @ejb:persistence column-name="type"
	 * @ejb:transaction type="Supports"
	 */

	public abstract String getType();

	/**
	 * Set the type of this project (model/cooperative/instance)
	 *
	 * @param type of this project. Is set at creation time.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 */
	public abstract void setType(String type);

	/**
	 * Retrieve the BnProject Creator.
	 *
	 * @return Returns a string representing the name of the creator.
	 *
	 * @ejb.interface-method view-type="both"
	 * @ejb.value-object match="light"
	 *
	 * @ejb:persistence column-name="creator"
	 * @ejb:transaction type="Supports"
	 */
	public abstract String getCreator();

	/**
	* Set the Creator Name.
	*
	* @param pCreator The Creator of this project
	* @ejb:transaction type="Required"
	**/
	public abstract void setCreator(String pCreator);

	/**
	* @return Returns the State of this BnProject
	*
	* @ejb:interface-method view-type="both"
	* @ejb.value-object
	*      match="light"
	*
	* @ejb:persistence column-name="state"
	* @ejb:transaction type="Supports"
	**/
	public abstract int getState();

	/**
	* Specify the State of this BnProject
	*
	* @param pState State of this BnProject
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setState(int pState);

	/**
	 * @return Return the nodes of the project
	 *
	 * @ejb.interface-method view-type="both"
	 * @ejb.relation name="Nodes" role-name="has-nodes"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnNodeValue"
	 *       compose-name="BnNode"
	 *       members="hero.interfaces.BnNodeLocal"
	 *       members-name="BnNode"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 **/
	public abstract Collection getBnNodes();

	/**
	* Specify the Nodes of this BnProject
	*
	* @param pNodes Nodes of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnNodes(Collection pNodes);

	/**
	 * @return Return the edges of the project
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="Edges" role-name="has-edges"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnEdgeValue"
	 *       compose-name="BnEdge"
	 *       members="hero.interfaces.BnEdgeLocal"
	 *       members-name="BnEdge"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 **/
	public abstract Collection getBnEdges();

	/**
	* Specify the Edges of this BnProject
	*
	* @param pEdges Edges of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnEdges(Collection pEdges);

	/**
	 * @return Return the Agents of the project
	 *
	 * @ejb.interface-method view-type="local"
	 * @ejb.relation name="Agents" role-name="has-agents"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnAgentValue"
	 *       compose-name="BnAgent"
	 *       members="hero.interfaces.BnAgentLocal"
	 *       members-name="BnAgent"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 **/
	public abstract Collection getBnAgents();

	/**
	* Specify the Agents of this BnProject
	*
	* @param pAgents Agents of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnAgents(Collection pAgents);

	/**
	 * @return Return the agentedges of the project
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="AgentEdges" role-name="has-agentedges"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnAgentEdgeValue"
	 *       compose-name="BnAgentEdge"
	 *       members="hero.interfaces.BnAgentEdgeLocal"
	 *       members-name="BnAgentEdge"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
	 **/
	public abstract Collection getBnAgentEdges();

	/**
	* Specify the AgentEdges of this BnProject
	*
	* @param pAgentEdges AgentEdges of this BnProject
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnAgentEdges(Collection pAgentEdges);

	/**
	 * @return Return the users of the project
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="Users" role-name="has-users" target-multiple="yes"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnUserLightValue"
	 *       aggregate-name="BnUser"
	 *       members="hero.interfaces.BnUserLocal"
	 *       members-name="BnUser"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectUserIdFK"
	 **/
	public abstract Collection getBnUsers();

	/**
	* Specify the Users of this BnProject
	*
	* @param pUsers Users of this BnProject
	*
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnUsers(Collection pUsers);

	/**
	 * @return Return the roles of the project
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:relation name="ProjectRoles" role-name="has-roles" target-multiple="yes"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnRoleValue"
	 *       aggregate-name="BnRole"
	 *       members="hero.interfaces.BnRoleLocal"
	 *       members-name="BnRole"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 **/
	public abstract Collection getBnRoles();

	/**
	* Specify the Roles of this BnProject
	*
	* @param pRoles Roles of this BnProject
	*
	* @ejb:interface-method view-type="local"
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnRoles(Collection pRoles);

    /** Retrieve the mapper (for the initiator) of the project.
     *
     * @return Returns the project initiator mapper.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="mapper-project" project-name="project2mapper" role-name="has-intiatorMapper"
     * @ejb:value-object match="*"
     *       relation="external"
     *       aggregate="hero.interfaces.BnInitiatorMapperValue"
     *       aggregate-name="BnInitiatorMapper"
     *
     * @jonas.field-mapping
     *    key-jdbc-name="id"
     *
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectMapperIdFK"
     **/
public abstract BnInitiatorMapperLocal getBnInitiatorMapper();

/**
    * Set the Project of this (initiator) mapper
    *
    * @param pInitiatorMapper initiatorMapper of the  project
    * @ejb:transaction type="Required"
    **/
public abstract void setBnInitiatorMapper( BnInitiatorMapperLocal pInitiatorMapper);

	/**
	 * @return Return the creation date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="creationdate"
	 * @ejb.value-object
	 *      match="light"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public abstract Date getCreationDate();

	/**
	 * @param pDate the creation date of the BnProject
	 * @ejb:interface-method view-type="both"
	 * @ejb.value-object
	 *      match="light"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setCreationDate(Date pDate);

	/**
	 * @return Return the date in which the BnProject was finished
	 * @ejb:interface-method view-type="both"
	 * @ejb:persistence column-name="enddate"
	 * @ejb:transaction type="Supports"
	 * @ejb.value-object
	 *      match="light"
	 *
	 **/
	public abstract Date getEndDate();

	/**
	 * @param the date in which the BnProject was finished
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public abstract void setEndDate(Date pDate);

	/**
	 * @param nodeName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnNodeLocal getBnNode(String nodeName) throws HeroException {
		BnNodeLocalHome nodehome;
		BnNodeLocal node;
		try {
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			node = nodehome.findByName(nodeName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnNode with this name");
		}

		return node;
	}

	/**
	 * @param AgentName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnAgentLocal getBnAgent(String agentName) throws HeroException {
		BnAgentLocalHome agenthome;
		BnAgentLocal agent;
		try {
			agenthome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			agent = agenthome.findByName(agentName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnAgent with this name");
		}
		return agent;
	}

	/**
	 * @param edgeName name of the node to return
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public BnEdgeLocal getBnEdge(String edgeName) throws HeroException {
		BnEdgeLocalHome edgehome;
		BnEdgeLocal edge;
		try {
			edgehome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			edge = edgehome.findByName(edgeName, this.getId());
		} catch (FinderException fe) {
			throw new HeroException("No BnEdge with this name");
		}

		return edge;
	}

	/**
	 * @return  the properties of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projectprp" role-name="is-property-of"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnProjectPropertyValue"
	 *       compose-name="BnPropertie"
	 *       members="hero.interfaces.BnProjectPropertyLocal"
	 *       members-name="BnPropertie"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 **/
	public abstract Collection getBnProperties();

	/**
	* Set the properties of the project
	* @ejb:interface-method view-type="both"
	* @param pPrp the properties of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnProperties(Collection pPrp);


	/**
	 * @return  the iterations of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projectIte" role-name="is-iteration-of"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnIterationLightValue"
	 *       compose-name="BnIteration"
	 *       members="hero.interfaces.BnIterationLocal"
	 *       members-name="BnIteration"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 **/
	public abstract Collection getBnIterations();

	/**
	* Set the iterations of the project
	* @ejb:interface-method view-type="both"
	* @param ite the iterations of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnIterations(Collection ite);

	/**
	 * @return  the Hooks of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projecthook" role-name="is-hook-of"
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnProjectHookValue"
	 *       compose-name="BnHook"
	 *       members="hero.interfaces.BnProjectHookLocal"
	 *       members-name="BnHook"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 **/
	public abstract Collection getBnHooks();

	/**
	* Set the Hooks of the project
	* @ejb:interface-method view-type="both"
	* @param pHooks the Hooks of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnHooks(Collection pHooks);

	/**
	 * @return  the Interactives Hooks of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="projectinterhook" role-name="is-hook-of"
	 *
	 * @ejb.value-object
	 *       match="normal"
	 *       type="Collection"
	 *       relation="external"
	 *       compose="hero.interfaces.BnProjectInterHookValue"
	 *       compose-name="BnInterHook"
	 *       members="hero.interfaces.BnProjectInterHookLocal"
	 *       members-name="BnInterHook"
	 * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
	 *
	 **/
	public abstract Collection getBnInterHooks();

	/**
	* Set the Interactives Hooks of the project
	* @ejb:interface-method view-type="both"
	* @param pHooks the Hooks of the project
	* @ejb:transaction type="Required"
	**/
	public abstract void setBnInterHooks(Collection pHooks);

	/**
	 * @ejb.interface-method
	 * @ejb:transaction type="Supports"
	 */
	public abstract BnProjectValue getBnProjectValue();

	/**
	 * @returns the light value of the bean (without relationships)
	 * @ejb.interface-method view-type=both
	 */
	public abstract BnProjectLightValue getBnProjectLightValue();

	/**
	 * @ejb.interface-method
	 * @ejb:transaction type="Supports"
	 */
	public abstract void setBnProjectValue(BnProjectValue v);

	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT u.name FROM BnProject AS p, IN (p.bnUsers) as u
 	 *				Where p.type = ?3"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectModelUserList(String userName, String roleName, String type) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetModelUserList(String userName, String roleName, String type) throws HeroException {
		try {
			return (ejbSelectModelUserList(userName, roleName, type));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT p.creator FROM BnProject AS p
	 * 				Where p.type = ?3"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectModelCreatorList(String userName, String roleName, String type) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetModelCreatorList(String userName, String roleName, String type) throws HeroException {
		try {
			return (ejbSelectModelCreatorList(userName, roleName, type));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT pr.name FROM BnProject AS p, IN (p.bnRoles) as pr
 	 *				Where p.type = ?3"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectModelRoleList(String userName, String roleName, String type) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetModelRoleList(String userName, String roleName, String type) throws HeroException {
		try {
			return (ejbSelectModelRoleList(userName, roleName, type));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT pp.theKey FROM BnProject AS p, in (p.bnProperties) as pp
	 *				Where p.type = ?3"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectModelPropertyKeyList(String userName, String roleName, String type) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetModelPropertyKeyList(String userName, String roleName, String type) throws HeroException {
		try {
			return (ejbSelectModelPropertyKeyList(userName, roleName, type));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT p.state FROM BnProject AS p
 	 *				Where p.type = ?3"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectModelStateList(String userName, String roleName, String type) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetModelStateList(String userName, String roleName, String type) throws HeroException {
		try {
			return (ejbSelectModelStateList(userName, roleName, type));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT n.name FROM BnNode AS n, IN (n.bnProject.bnUsers) AS u, IN (u.bnRoles) AS r
	 *                              WHERE n.bnProject.name LIKE ?1
	 * 				                AND (n.state = 1 OR n.state = 3)
	 * 							    AND u.name = ?2
	 *                              AND r.id = n.bnRole.id
     *                              AND (n.activityPerformer IS NULL OR n.activityPerformer = ?2)"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectUserModelReady(String projectName, String userName) throws FinderException;

	/**
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT n.name FROM BnNode AS n
	 *                              WHERE n.bnProject.name LIKE ?1
	 *                              AND n.executor = ?2
	 *                              AND n.state = 10"
	 *     transaction-type="Supports"
	 *     */
	public abstract Collection ejbSelectUserModelTerminated(String projectName, String userName) throws FinderException;

	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT p.name FROM BnProject AS p
	 *                             WHERE p.type = 'Instance'
	 * 							   AND p.creator = ?1
	 * 				               AND (p.state = 0 OR p.state=1)"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract Collection ejbSelectUserInstancesProject(String userName) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetUserInstancesProject(String userName) throws HeroException {
		try {
			return (ejbSelectUserInstancesProject(userName));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 *
	 * @ejb.select
	 *	  result-type-mapping="Local"
	 *     query="SELECT DISTINCT p.version FROM BnProject AS p
	 *                             WHERE p.type = 'Instance'
	 * 							   AND p.creator = ?1
	 * 				               AND (p.state = 0 OR p.state=1)
	 * 							   AND p.name = ?2"
	 *     transaction-type="Supports"
	 *
	 */
	public abstract String ejbSelectUserModelVersion(String userName, String project) throws FinderException;

	/**
	 * @ejb.home-method view-type="local"
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String ejbHomeGetUserModelVersion(String userName, String project) throws HeroException {
		try {
			return (ejbSelectUserModelVersion(userName, project));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}
	
	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetUserModelReady(String projectName, String userName) throws HeroException {
		try {
			return (ejbSelectUserModelReady(projectName, userName));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * @ejb.home-method view-type="local"
	 * @param projectName name of the project
	 * @param userName name of the user
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection ejbHomeGetUserModelTerminated(String projectName, String userName) throws HeroException {
		try {
			return (ejbSelectUserModelTerminated(projectName, userName));
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		}
	}

	// -------------------------------------------------------------------------
	// Framework Callbacks
	// -------------------------------------------------------------------------

	/**
	* Create a BnProject
	*
	* @throws InvalidValueException If one of the values are not correct,
	*                               this will not roll back the transaction
	*                               because the caller has the chance to
	*                               fix the problem and try again
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnProjectPK ejbCreate()
		throws InvalidValueException, EJBException, CreateException {
			// Each title must have a unique id to identify itself within the DB
			this.setId(hero.interfaces.BnProjectUtil.generateGUID(this));

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	/**
	* Create a BnProject based on project name and creator.
	*
	* @param pName the name of the project
	* @param pCreator the creator of the project
	*
	* @throws EJBException If no new unique ID could be retrieved this will
	*                      rollback the transaction because there is no
	*                      hope to try again
	* @throws CreateException Because we have to do so (EJB spec.)
	*
	* @ejb:create-method view-type="both"
	* @ejb:transaction type="Required"
	**/
	public BnProjectPK ejbCreate(String pName, String pVersion, String creator)
		throws InvalidValueException, EJBException, CreateException {
		log.debug("ejbCreate: pName=" + pName + " creator=" + creator);
			try{
				SequenceGeneratorLocalHome sgHome = SequenceGeneratorUtil.getLocalHome();
				SequenceGeneratorLocal sg = sgHome.create();

				this.setId(new Integer(sg.getNextNumber("BnProject")).toString());

				if (pName.matches(".*_instance.*"))
					this.setName(pName+this.getId());
				else
					this.setName(pName);
				this.setCreator(creator);
				this.setVersion(pVersion);


			}catch ( NamingException ne ) {
				throw new CreateException( "Naming lookup failure: " + ne.getMessage() );
			}catch ( HeroException re ) {
				throw new CreateException(re.getMessage() );
			}

		// This is only possible in CMPs. Otherwise return a valid PK.
		return null;
	}

	public void setEntityContext(EntityContext lContext) {
		mContext = lContext;
	}

	public void unsetEntityContext() {
		mContext = null;
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

	public void ejbLoad() {
	}

	public void ejbStore() {
	}

	public void ejbRemove() throws RemoveException {
	}
}
