package hero.entity;

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectPropertyPK;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.InvalidValueException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

import java.util.Collection;

/**
 * The Entity bean represents a BnProjectProperty
 *
 * @author Oriol Montala
 * @version $Revision: 1.10 $
 *
 * @ejb:bean name="BnProjectProperty"
 *           display-name="Property Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnProjectProperty"
 *           local-jndi-name="ejb/hero/BnProjectProperty_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnProjectProperty"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(p) FROM BnProjectProperty p"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByProject(java.lang.String projectId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnProjectProperty p Where p.bnProject.id = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByTheKey(java.lang.String projectId, java.lang.String pkey)" view-type="local"
 *     query="SELECT OBJECT(p) From BnProjectProperty p Where p.bnProject.id = ?1 AND p.theKey=?2"
 *     transaction-type="Supports"
 *
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnProjectProperty"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnProjectPropertyLight"
 *
 * @jboss.persistence
 *    table-name="BnProjectProperty"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnProjectPropertyBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnProjectProperty's id.
   *
   * @return Returns an int representing the id of this BnProjectProperty.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the Property's id.
   *
   * @param pId The id of this Property. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );



    /** Retrieve the BnProject.
     *
     * @return Returns a BnProjectLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="projectprp" role-name="property-by" cascade-delete="yes"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectPropertyIdFK"
     **/
    public abstract BnProjectLocal getBnProject();

    /**
     * Set the BnProject.
     *
     * @ejb:interface-method view-type="local"
     * @param pProject The BnProject of this Property.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject( BnProjectLocal pProject );

   /**
   * Retrieve the Key value.
   *
   * @return Returns an String representing the Key value of this Property.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="thekey"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheKey();

   /**
   * Set the key value.
   * @ejb:interface-method view-type="both"
   * @param key The Key of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheKey( String key );

   /**
   * Retrieve the Value value.
   *
   * @return Returns an String representing the Value value of this Property.
   * @ejb.value-object
   *    match="light"
   * @ejb:interface-method view-type="both"
   *
   * @ejb:persistence column-name="thevalue"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheValue();

   /**
   * Set the Value value.
   * @ejb:interface-method view-type="both"
   * @param value The Value value of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheValue( String value );

   /**
    * Retrieve the Value value.
    *
    * @return Returns a Collection representing possibles values of Property.
    * @ejb.value-object
    *    match="light"
    * @ejb:interface-method view-type="both"
    *
    * @ejb:persistence column-name="possiblevalues"
    * @ejb:transaction type="Supports"
    *
    * @jboss.relation-table create-table="true" remove-table="false"
    **/
    public abstract Collection getPossibleValues();

    /**
    * Set the possible values of the property
    * @ejb:interface-method view-type="both"
    * @param values possibles values of the property
    * @ejb:transaction type="Required"
    **/
    public abstract void setPossibleValues( Collection value );

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnProjectPropertyValue getBnProjectPropertyValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnProjectPropertyValue(BnProjectPropertyValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a Property based on the supplied Property Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnProjectPropertyPK ejbCreate(BnProjectPropertyValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setId(hero.interfaces.BnProjectPropertyUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnProjectPropertyValue prp )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnProjectPropertyValue(prp);
   }
   
   /**
   * Create a Property based on the supplied  key  and value.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnProjectPropertyPK ejbCreate(String key, String value )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setTheKey(key);
       this.setTheValue(value);
       this.setId(hero.interfaces.BnProjectPropertyUtil.generateGUID(this));

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   /**
    * Create a Property based on the supplied  key  and value.
    *
    *
    * @throws InvalidValueException If one of the values are not correct,
    *                               this will not roll back the transaction
    *                               because the caller has the chance to
    *                               fix the problem and try again
    * @throws EJBException If no new unique ID could be retrieved this will
    *                      rollback the transaction because there is no
    *                      hope to try again
    * @throws CreateException Because we have to do so (EJB spec.)
    *
    * @ejb:create-method view-type="both"
    **/
    public BnProjectPropertyPK ejbCreate(String key, Collection values )
       throws
          InvalidValueException,
          EJBException,
          CreateException
    {
        this.setTheKey(key);
        this.setPossibleValues(values);
        this.setId(hero.interfaces.BnProjectPropertyUtil.generateGUID(this));

       // This is only possible in CMPs. Otherwise return a valid PK.
       return null;
    }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
