package hero.entity;

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnRoleData;
import hero.interfaces.BnRolePK;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnRoleMapperLocal;

import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnRole
 *
 * @author Fran?ois Charoy
 * @version $Revision: 1.13 $
 *
 * @ejb:bean name="BnRole"
 *           display-name="BnRole Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnRole"
 *           local-jndi-name="ejb/hero/BnRole_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * @ejb:transaction type="Supports"
 *
 * @ejb:data-object extends="hero.interfaces.AbstractData"
 *                  setdata="false"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnRole"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(r) FROM BnRole r"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnRole findByName(java.lang.String pName, java.lang.String pProjectId )" result-type-mapping="Remote"
 *     query="SELECT OBJECT(n) FROM BnRole n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnRoleLocal findByName(java.lang.String pName, java.lang.String pProjectId )" result-type-mapping="Local"
 *     query="SELECT OBJECT(n) FROM BnRole n WHERE n.name = ?1 AND n.bnProject.id = ?2"
 *     transaction-type="Supports"
 *
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnRole"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnRoleLight"
 *
 * @jboss.persistence
 *    table-name="BnRole"
 *    create-table="true"
 *    remove-table="false"
 * 
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnRoleBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnRole's id.
   *
   * @return Returns an int representing the id of this BnRole.
   * @ejb:interface-method view-type="both"
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnRole's id.
   *
   * @param pId The id of this BnRole. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );

   /**
   * Retrieve the BnRole's Description.
   *
   * @return Returns an int representing the Name of this BnRole.
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="description"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getDescription();

   /**
   * Set the BnRole's Description.
   *
   * @param pDescription The Description of this BnRole.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setDescription( String pDescription );

   /**
   * Retrieve the BnRole's Name.
   *
   * @return Returns an int representing the Name of this BnRole.
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the BnRole's Name.
   *
   * @param pName The Name of this BnRole.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String pName );


    /**
     * @returns the users of the role
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Roles" role-name="is-role-of" multiple="yes"
     * @jboss.relation-table create-table="true" remove-table="false"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "userRoleIdFK"
     */
    public abstract Collection getBnUsers() ;

    /**
     * Set the Roles of this project
     *
     * @param pUsers users of the  role
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnUsers( Collection pUsers);

    /**
     * @returns the nodes of the role
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="NodeRole" role-name="is-role-of-node" multiple="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     */
    public abstract Collection getBnNodes() ;

    /**
     * Set the Roles of this project
     *
     * @param pNodes nodes of the  role
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnNodes( Collection pNodes);

    /**
     * @returns the project of the role
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="ProjectRoles" role-name="is-role-of-project" cascade-delete="yes"
     * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "projectRoleIdFK"
      */
    public abstract BnProjectLocal getBnProject() ;

    /**
     * Set the Roles of this project
     *
     * @param pProject project of the  role
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProject( BnProjectLocal pProject);

 	/** Retrieve the RoleMapper for the role.
	 *
	 * @return Returns the role mapper.
	 * @ejb:interface-method view-type="both"
	 * @ejb:relation name="mapper-role" role-name="role2mapper"
	 * @ejb:value-object match="*"
	 *       relation="external"
	 *       aggregate="hero.interfaces.BnRoleMapperValue"
	 *       aggregate-name="BnRoleMapper"
	 *       
     * @jonas.field-mapping
     *    key-jdbc-name="id"
     *    
	 * @ejb:transaction type="Supports"
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "roleMapperRoleIdFK"
	 **/
   public abstract BnRoleMapperLocal getBnRoleMapper() ;

   /**
	* Set the Roles of this mapper
	*
	* @param pMapper mapper of the  role
	* @ejb:transaction type="Required"
	**/
   public abstract void setBnRoleMapper( BnRoleMapperLocal pMapper);



    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnRoleValue getBnRoleValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnRoleValue(BnRoleValue v);

   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnRole based on the supplied BnRole Value Object.
   *
   * @param pRole The data used to create the BnRole.
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnRolePK ejbCreate( BnRoleValue pRole )
      throws
         EJBException,
         CreateException
   {
       this.setId(hero.interfaces.BnRoleUtil.generateGUID(this));
       return null;
   }

   public void ejbPostCreate( BnRoleValue pRole )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnRoleValue(pRole);
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }
}
