package hero.entity;

import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserPK;
import hero.interfaces.BnUserValue;
import hero.interfaces.InvalidValueException;

import java.sql.Date;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnUser
 *
 * @author Fran?ois Charoy
 * @version $Revision: 1.15 $
 *
 * @ejb:bean name="BnUser"
 *           display-name="BnUser Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnUser"
 *           local-jndi-name="ejb/hero/BnUser_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 * The following tag is required to make JBoss guarantee database consistency
 * when removing entities involved in relations.
 * @ejb:transaction type="Required"
 *
 * @ejb:data-object extends="hero.interfaces.AbstractData"
 *                  setdata="false"
 *
 * @ejb.pk generate="true" method-intf="remote"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnUser"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(u) FROM BnUser u"
 *     transaction-type="Supports"
 *
 * @ejb:finder signature="hero.interfaces.BnUser findByName(java.lang.String pName )" result-type-mapping="Remote"
 *     query="SELECT Object(u) From BnUser as u Where u.name = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="hero.interfaces.BnUserLocal findByName(java.lang.String pName )" result-type-mapping="Local"
 *     query="SELECT Object(u) From BnUser as u Where u.name = ?1"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnUser"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnUserLight"
 *
 * @jboss.persistence
 *    table-name="BnUser"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/

public abstract class BnUserBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnUser's id.
   *
   * @return Returns an int representing the id of this BnUser.
   *
   * @ejb:pk-field
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *      match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the BnUser's id.
   *
   * @param pId The id of this BnUser. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );

   /**
   * Retrieve the BnUser's Name.
   *
   * @return Returns an int representing the Name of this BnUser.
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="name"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getName();

   /**
   * Set the BnUser's Name.
   *
   * @param uName The Name of this BnUser.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setName( String uName );

   /**
   * Retrieve the BnUser's Password.
   *
   * @return Returns an int representing the Password of this BnUser.
   *
   * @ejb:interface-method view-type="local"
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="password"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getPassword();

   /**
   * Set the BnUser's Password.
   *
   * @param pPassword The Password of this BnUser
   * @ejb:transaction type="Required"
   **/
   public abstract void setPassword( String pPassword );

   /**
   * Retrieve the BnUser's Email.
   *
   * @return Returns an String representing the Email of this BnUser.
   * @ejb:interface-method view-type="both"
   *
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="email"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getEmail();

   /**
   * Set the BnUser's Email.
   *
   * @param pEmail The Email of this BnUser.  Is set at creation time.
   * @ejb:persistence column-name="email"
   * @ejb:interface-method view-type="local"
   * @ejb:transaction type="Required"
   **/
   public abstract void setEmail( String pEmail );

   /**
   * Retrieve the BnUser's Jabber address.
   *
   * @return Returns an String representing the Jabber address of this BnUser.
   * @ejb:interface-method view-type="both"
   *
   * @ejb.value-object
   *    match="light"
   *
   * @ejb:persistence column-name="jabber"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getJabber();

   /**
   * Set the BnUser's Jabber address.
   *
   * @param pJabber The Jabber address of this BnUser.  Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setJabber( String pJabber );

    /**
     * @return Returns the creation date of this BnUser
     *
     * @ejb:interface-method view-type="local"
     * @ejb.value-object
     *    match="light"
     *
     * @ejb:persistence column-name="creationdate"
     * @ejb:transaction type="Supports"
     **/
    public abstract Date getCreationDate();

    /**
     * Specify the creation date of this BnUser
     *
     * @param pCreationDate Date of the creation of this BnUser
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setCreationDate( Date pCreationDate );

    /**
     * @return Returns the modification date of this BnUser
     *
     * @ejb.value-object
     *    match="light"
     * @ejb:persistence column-name="modificationdate"
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Supports"
     **/
    public abstract Date getModificationDate();

    /**
     * Specify the modification date of this BnUser
     *
     * @param pModificationDate Date of the modification of this BnUser
     * @ejb:interface-method view-type="local"
     * @ejb:transaction type="Required"
     **/
    public abstract void setModificationDate( Date pModificationDate );

    /**
     * @returns the project on which this user works
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Users" role-name="is-user-of" target-multiple="yes"
     *
     * @ejb.value-object
     *       match="normal"
     *       type="Collection"
     *       relation="external"
     *       aggregate="hero.interfaces.BnProjectLightValue"
     *       aggregate-name="BnProject"
     *       members="hero.interfaces.BnProjectLocal"
     *       members-name="BnProject"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "userProjectIdFK"
     */
    public abstract Collection getBnProjects() ;


    /**
     * Set the Users of this project
     *
     * @param pProjects projects of the  user
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProjects( Collection pProjects);

    /**
     * @returns the roles of the user
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="Roles" role-name="has-role" multiple="no"
     *
     * @ejb.value-object
     *       match="normal"
     *       type="Collection"
     *       relation="external"
     *       aggregate="hero.interfaces.BnRoleValue"
     *       aggregate-name="BnRole"
     *       members="hero.interfaces.BnRoleLocal"
     *       members-name="BnRole"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "rolesUserIdFK"
     */
    public abstract Collection getBnRoles() ;

    /**
     * Set the roles of the user
     *
     * @param pRoles roles of the user
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnRoles( Collection pRoles);

    /**
     * @returns the authorisation roles of the user (for ejb security)
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="AuthRoles" role-name="has-authrole" multiple="yes"
     *
     * @ejb.value-object
     *       match="normal"
     *       type="Collection"
     *       relation="external"
     *       aggregate="hero.interfaces.BnAuthRoleValue"
     *       aggregate-name="BnAuthRole"
     *       members="hero.interfaces.BnAuthRoleLocal"
     *       members-name="BnAuthRole"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "BnAuthRole_id"
     */
    public abstract Collection getBnAuthRoles() ;

    /**
     * Set the authorisation roles of the user (for ejb security)
     *
     * @param pRoles roles of the user
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnAuthRoles( Collection pRoles);

    /**
     * @returns the nodes created by the user
     * @ejb:interface-method view-type="local"
     * @ejb:relation name="AgentCreator" role-name="is-node-creator" multiple="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     */
    public abstract Collection getCreatedBnAgents() ;

    /**
     * Set the user created agents
     *
     * @param pAgents Agents created by the user
     * @ejb:transaction type="Required"
     **/
    public abstract void setCreatedBnAgents( Collection pAgents);

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnUserValue getBnUserValue();

    /**
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setBnUserValue(BnUserValue v);

    /**
     * @returns the light value of the bean (without relationships)
     * @ejb.interface-method view-type=both
     * @ejb:transaction type="Supports"
     */
    public abstract BnUserLightValue getBnUserLightValue();

    /**
     * @return the properties of the user
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="userprp" role-name="is-property-of"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation-table create-table="true" remove-table="false"
     **/
    public abstract Collection getBnProperties();

    /**
     * Set the properties of the user
     * @ejb:interface-method view-type="both"
     * @param pPrp the properties of the user
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnProperties(Collection pPrp);




   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a BnUser based on the supplied BnUser Value Object.
   *
   * @param pUser The data used to create the BnUser.
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnUserPK ejbCreate( BnUserValue pUser )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      // This is only possible in CMPs. Otherwise return a valid PK.
      this.setId(hero.interfaces.BnUserUtil.generateGUID(this));

      this.setCreationDate(new java.sql.Date(new java.util.Date().getTime()));
      this.setModificationDate(new java.sql.Date(new java.util.Date().getTime() ));
      return null;
   }

   public void ejbPostCreate( BnUserValue pUser )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnUserValue(pUser);
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }
}
