package hero.entity;

import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserPropertyPK;
import hero.interfaces.BnUserPropertyValue;
import hero.interfaces.InvalidValueException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * The Entity bean represents a BnUserProperty
 *
 * @author Oriol Montala
 * @version $Revision: 1.9 $
 *
 * @ejb:bean name="BnUserProperty"
 *           display-name="BnUser Entity"
 *           type="CMP"
 *           jndi-name="ejb/hero/BnUserProperty"
 *           local-jndi-name="ejb/hero/BnUserProperty_L"
 *           view-type="both"
 *           cmp-version="2.x"
 *           reentrant="true"
 *
 *
 * @ejb:transaction type="Supports"
 *
 *
 * @ejb.pk generate="true" method-intf="both"
 *
 * @jonas.bean
 * 	   lock-policy="container-read-committed"
 * @jonas.jdbc-mapping
 *     jndi-name="bonita"
 *     jdbc-table-name="BnUserProperty"
 *
 * @jonas.shared false
 *
 * @ejb:finder signature="java.util.Collection findAll()"
 *     query="SELECT object(u) FROM BnUserProperty u"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByUser(java.lang.String userId)" view-type="local"
 *     query="SELECT OBJECT(p) From BnUserProperty p Where p.bnUser.id = ?1"
 *     transaction-type="Supports"
 * @ejb:finder signature="java.util.Collection findByTheKey(java.lang.String userId, java.lang.String pkey)" view-type="local"
 *     query="SELECT OBJECT(p) From BnUserProperty p Where p.bnUser.id = ?1 AND p.theKey=?2"
 *     transaction-type="Supports"
 *
 *
 * @ejb.value-object
 *    match="*"
 *    name="BnUserProperty"
 *
 * @ejb.value-object
 *    match="light"
 *    name="BnUserPropertyLight"
 *
 * @jboss.persistence
 *    table-name="BnUserProperty"
 *    create-table="true"
 *    remove-table="false"
 *
 * @jboss.container-configuration name="Standard CMP 2.x EntityBean for Bonita"
 **/
public abstract class BnUserPropertyBean
   implements EntityBean
{

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   public EntityContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------


   // -------------------------------------------------------------------------
   // Properties (Getters/Setters)
   // -------------------------------------------------------------------------

   /**
   * Retrieve the BnUserProperty's id.
   *
   * @return Returns an int representing the id of this BnUserProperty.
   *
   * @ejb:pk-field
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="id"
   * @jonas.cmp-field-jdbc-mapping
   *    field-name="id"
   *    jdbc-field-name="id"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getId();

   /**
   * Set the Property's id.
   *
   * @param pId The id of this Property. Is set at creation time.
   * @ejb:transaction type="Required"
   **/
   public abstract void setId( String pId );



    /** Retrieve the BnUser.
     *
     * @return Returns a BnUserLocal.
     * @ejb:interface-method view-type="both"
     * @ejb:relation name="userprp" role-name="property-by" cascade-delete="yes"
     * @ejb:transaction type="Supports"
     *
     * @jboss.relation
     *    related-pk-field = "id"
     *    fk-column = "userProperyIdFK"
     **/
    public abstract BnUserLocal getBnUser();

    /**
     * Set the BnUser.
     *
     * @ejb:interface-method view-type="local"
     * @param pUser The BnUser of this Property.
     * @ejb:transaction type="Required"
     **/
    public abstract void setBnUser( BnUserLocal pUser );



   /**
   * Retrieve the Key value.
   *
   * @return Returns an String representing the Key value of this Property.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="thekey"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheKey();

   /**
   * Set the key value.
   * @ejb:interface-method view-type="both"
   * @param key The Key of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheKey( String key );

   /**
   * Retrieve the Value value.
   *
   * @return Returns an String representing the Value value of this Property.
   *
   * @ejb:interface-method view-type="both"
   * @ejb.value-object
   *    match="light"
   * @ejb:persistence column-name="thevalue"
   * @ejb:transaction type="Supports"
   **/
   public abstract String getTheValue();

   /**
   * Set the Value value.
   * @ejb:interface-method view-type="both"
   * @param value The Value value of this Property.
   * @ejb:transaction type="Required"
   **/
   public abstract void setTheValue( String value );

    /**
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract BnUserPropertyValue getBnUserPropertyValue();

    /**
     * @ejb.interface-method
     */
    public abstract void setBnUserPropertyValue(BnUserPropertyValue v);



   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------

   /**
   * Create a Property based on the supplied Property Value Object.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnUserPropertyPK ejbCreate(BnUserPropertyValue prp )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
      this.setId(hero.interfaces.BnUserPropertyUtil.generateGUID(this));
      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void ejbPostCreate(BnUserPropertyValue prp )
   {
	   // Assign value object in ejbPostCreate to apply XDoclet best
	   // practice. If not done, there will be an error in JBoss if
	   // the Value Object has relations.
       this.setBnUserPropertyValue(prp);
   }

   
   /**
   * Create a Property based on the supplied  key  and value.
   *
   *
   * @throws InvalidValueException If one of the values are not correct,
   *                               this will not roll back the transaction
   *                               because the caller has the chance to
   *                               fix the problem and try again
   * @throws EJBException If no new unique ID could be retrieved this will
   *                      rollback the transaction because there is no
   *                      hope to try again
   * @throws CreateException Because we have to do so (EJB spec.)
   *
   * @ejb:create-method view-type="both"
   **/
   public BnUserPropertyPK ejbCreate(String key, String value )
      throws
         InvalidValueException,
         EJBException,
         CreateException
   {
       this.setTheKey(key);
       this.setTheValue(value);
       this.setId(hero.interfaces.BnUserPropertyUtil.generateGUID(this));

      // This is only possible in CMPs. Otherwise return a valid PK.
      return null;
   }

   public void setEntityContext( EntityContext lContext )
   {
      mContext = lContext;
   }

   public void unsetEntityContext()
   {
      mContext = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbLoad()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbRemove()
      throws
         RemoveException
   {
   }

}
