package hero.entity;
import hero.interfaces.Constants;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnEdgeLocal;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceException;
import hero.util.BonitaServiceValue;
import hero.util.HeroException;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaProjectValue;

import java.util.*;

public class OrEdgeState extends EdgeState {

	private static OrEdgeState nd;

	public OrEdgeState() {
	}

	public static OrEdgeState getInstance() {
		if (nd == null) {
			nd = new OrEdgeState();
		}
		return nd;
	}

	public int state(BnNodeLocal node) {
		Collection edgesInstance = new ArrayList();
		Collection edgesModel = new ArrayList();

		try{
			edgesInstance = BonitaServiceValue.getNode(node.getBnProject(),node.getName()).getInEdges();
		}catch (Exception e) {e.printStackTrace();}

		// Instances consistency (may be some activities are not yet created in the instance)
		String nodeName = node.getName();
		if (isInstance(node.getName()))
			nodeName = getModel(nodeName);
		try{
			BonitaProjectValue model = BonitaProjectLocator.getInstance().getModelValue(this.getModel(node.getBnProject().getName()), node.getBnProject().getVersion());
			if (node.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
				edgesModel = BonitaServiceValue.getNode(model.getName(), model.getVersion(), nodeName).getInEdges();
			else
				edgesModel = BonitaServiceValue.getNodeFromCache(model,nodeName).getInEdges();
		}catch (Exception e) {edgesModel=new ArrayList();}

		if ((edgesInstance.size() > 0) || (edgesModel.size() > 0)) {
			edgesInstance = concatEdges(edgesInstance, edgesModel);
		}

		Iterator i = edgesInstance.iterator();
		int max = 0;
		int result = Constants.Nd.EDGEINITIAL;
		while (i.hasNext())
		{
			BonitaEdgeValue edge = (BonitaEdgeValue) i.next();

			if(edge.getState() == Constants.Ed.DEAD && max <= 0)
			{
				max = 0;
				result = Constants.Nd.CANCEL;
			}
			else if(edge.getState() == Constants.Ed.INITIAL && max <= 1)
			{
				max = 1;
				result = Constants.Nd.EDGEINITIAL;
			}
			else if(edge.getState() == Constants.Ed.ANTICIPATING && max <= 2)
			{
				max = 2;
				result = Constants.Nd.ANTACTIVE;
			}
			else if(edge.getState() == Constants.Ed.ACTIVE && max <= 3)
			{
				max = 3;
				result = Constants.Nd.ACTIVE;

				// As the highest possible value is reached
				break;
			}
		}

		return result;
	}

	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}

	/**
	 * Concatenates model edges that doesn't exist in the instances edges (without repetition)
	 *
	 * @param edgesInstance
	 * @param edgesModel
	 * @return
	 */
	private Collection concatEdges(Collection edgesInstance, Collection edgesModel) {
		Collection result = new ArrayList();
		boolean found = false;

		result = edgesInstance;

		Iterator it = edgesModel.iterator();
		while (it.hasNext()) {
			found = false;
			BonitaEdgeValue el = (BonitaEdgeValue)it.next();
			String nodeName = el.getInNode();
			Iterator instanceEdgesIt = edgesInstance.iterator();
			while(instanceEdgesIt.hasNext()) {
				BonitaEdgeValue insEd = (BonitaEdgeValue) instanceEdgesIt.next();
				String node = insEd.getInNode();
				if (this.isInstance(node)) {
					node = this.getModel(node);
				}
				if (nodeName.equals(node)) {
					found = true;
					break;
				}
			}
			if (!found) {
				result.add(el);
			}
		}
		return result;
	}

}
