package hero.ra;

/*
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
 --------------------------------------------------------------------------
 * $Id$
 *
 --------------------------------------------------------------------------
 */

import hero.session.AdminSessionBean;

import java.util.ArrayList;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import org.apache.log4j.Logger;


/**
 * This Resource Adapater is generic and automatically registers in the first
 * instantiated JMX MBean Server a class specified by the "ObjectClass"
 * Configuration Property of ra.xml. This class must implement the
 * <tt>BonitaService</tt> interface. 
 * 
 * @author Brice Revenant
 */
public class JmxRegisterResourceAdapter implements ResourceAdapter
{
	/** Used to log events */
	private Logger log = Logger.getLogger(JmxRegisterResourceAdapter.class);
	
	/** Instance to be registered in the MBean Server */
	private BonitaService object = null;
	
	/** Class to be instantiated. Specified in ra.xml. */
	private String objectClass = null;
	
	/** Object Name used during registration. Specified in ra.xml. */
	private String objectName = null;
		
	/** Reference to the MBean Server in which to register the instance */
	private MBeanServer server = null;
	
	
	public void endpointActivation(MessageEndpointFactory arg0,
	                               ActivationSpec arg1)
	                               throws ResourceException
	{	
	}

	public void endpointDeactivation(MessageEndpointFactory arg0,
	                                 ActivationSpec arg1)
	{
	}
	
	/**
	 * Retrieves the class to be instantiated
	 * 
	 * @return String representing the class to be instantiated
	 */
	public String getObjectClass()
	{
		return objectClass;
	}

	/**
	 * Retrieves the Object Name to be used during registration in the MBean
	 * Server
	 * 
	 * @return String representing the Object Name to be used during
	 *         registration in the MBean Server
	 */
	public String getObjectName()
	{
		return objectName;
	}
	
	public XAResource[] getXAResources(ActivationSpec[] arg0)
	                                   throws ResourceException
	{
		return null;
	}
	
	/**
	 * Default constructor.
	 * Gets a reference to the first MBean Server instantiated
	 */
	public JmxRegisterResourceAdapter()
	{
		ArrayList servers = MBeanServerFactory.findMBeanServer(null);
		server = (MBeanServer) servers.get(0);
	}

	/**
	 * Sets the class to be instantiated
	 * 
	 * @param objectClass String representing the class to be instantiated
	 */
	public void setObjectClass(String objectClass)
	{
		this.objectClass = objectClass;
	}

	/**
	 * Sets the Object Name to be used during registration in the MBean
	 * Server
	 * 
	 * @param objectName representing the Object Name to be used during
	 *                   registration in the MBean Server
	 */
	public void setObjectName(String objectName)
	{
		this.objectName = objectName;
	}
	
	public void start(BootstrapContext arg0)
	                  throws ResourceAdapterInternalException
	{
		try
		{
			object = (BonitaService) Class.forName(objectClass).newInstance();
			server.registerMBean(object, new ObjectName(getObjectName()));
			object.startService();
			
			log.debug("The following Bonita MBean is registered : "
			         + object.getClass().getName());
		}
		catch(Exception e)
		{
			throw new ResourceAdapterInternalException(
				"Error while starting the Resource Adapter : " +
				e.getMessage());
		}
	}

	public void stop()
	{
		try
		{
			object.stopService();
			server.unregisterMBean(new ObjectName(getObjectName()));
			object = null;
			
			log.debug("The following Bonita MBean is unregistered : " + objectName);
		}
		catch(Exception e)
		{
			log.error(
				"An exception occured while stopping the Resource Adapter : "
				+ e);
		}
	}
}
