package hero.ra.jabber;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
 --------------------------------------------------------------------------
 * $Id$
 *
 --------------------------------------------------------------------------
 */

import com.echomine.jabber.*;
import com.echomine.net.*;

import java.util.HashMap;
import java.util.List;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

import hero.ra.BonitaService;


public class Jabber implements BonitaService, JabberMBean
{
	/** Password used for the Jabber session */
	private String password = null;
	
	/** Port used for the Jabber session */
	private int port = 0 ;
	
	/** Host used for the Jabber session */
	private String host = null;
	
	/** Jabber Session */
	private JabberSession jabberSession;
	
	/** Jabber Context */
	private JabberContext jabberContext;
	
	/** User name used for the Jabber session */
	private String user = null;

	/**
	 * Connects to the Jabber server based on User, Password, Host and Port
	 * class attributes.
	 */
	private void connect()
	{
		// Initialize the Jabber context
		jabberContext = new JabberContext(user, password, host);
		
		// Create an instance of the Jabber session factory
		com.echomine.jabber.Jabber jabber = new com.echomine.jabber.Jabber();
		
		// Create the new session
		jabberSession = jabber.createSession(jabberContext);
		
		// Connect to the server
		try
		{
			jabberSession.connect(host, port);
		}
		catch(Exception e)
		{
			System.err.println(" Jabber service -> Connection Failed:"
			                   + e.getMessage());
		}

		// Connect to the Jabber server
		try
		{
			jabberSession.getUserService().login();
		}
		catch(Exception e)
		{
			System.err.println(" Jabber service -> Login Failed:"
			                   + e.getMessage());
		}

	}
	
	/**
	 * Invoked when the service is stopped
	 */
	public void stopService()
	{
		/*
		 * The session may is null when the Jabber feature is turned off or if
		 * a problem occured during initialization
		 */ 
		if(jabberSession != null)
		{
			try
			{
				jabberSession.disconnect();
			}
			catch(Exception e)
			{
				System.err.println(" Jabber service -> Disconnect Failed:"
		                           + e.getMessage());
			}
			
			jabberSession = null;
		}
	}

	/**
	 * Invoked when the service is started
	 */
	public void startService() throws Exception
	{
		// Retrieve the MBean Server
		ObjectName confObjName = new ObjectName("bonita:type=bonitaservice,name=Config");
		List list = MBeanServerFactory.findMBeanServer(null);
		MBeanServer mServer = (MBeanServer)list.iterator().next();
		
		/*
		 * MBean Attributes are retrieved by name so that Configuration classes
		 * do not need to be packaged in the RAR.
		 */
		boolean enabled = ((Boolean) mServer.getAttribute(confObjName, "JabberEnabled")).booleanValue();
		
		// If the configuration indicates that the Jabber feature is on
		if(enabled)
		{
			// Retrieve configuration attributes.
			password = (String) mServer.getAttribute(
			               confObjName,
			               "JabberPassword");
			port     = ((Integer) mServer.getAttribute(
			               confObjName,
			               "JabberPort")).intValue();
			host     = (String) mServer.getAttribute(
			               confObjName,
			               "JabberHost");
			user     = (String) mServer.getAttribute(
			               confObjName,
			               "JabberUser");
			
			// Connect to the Jabber service
			connect();
		}
	}

	/**
	 * Sends a Jabber message to the specified JID
	 */
	public synchronized boolean sendMessage(String JID,
	                                        String resource,
	                                        String subject,
	                                        String messageBody)
	{
		try
		{
			JabberChatMessage msg =
			    new JabberChatMessage(JabberChatMessage.TYPE_NORMAL);
			msg.setSubject(subject);
			msg.setBody(messageBody);
			msg.setTo(JID);
			jabberSession.sendMessage(msg);
			return true;
		}
		catch(Exception e)
		{
			System.err.println(" Jabber service -> Send Message Failed:"
			                   + e.getMessage());
		}
		
		return false;
	}

	/**
	 * Registers a new user to Jabber server
	 */
	public synchronized boolean registerUser(String login,
	                                         String password,
	                                         String email,
	                                         String name)
	{
		boolean status = true;
		jabberSession.disconnect();
		com.echomine.jabber.Jabber jabber = new com.echomine.jabber.Jabber();
		JabberSession session = jabber.createSession(jabberContext);
		
		try
		{
			session.connect(host, port); 
			HashMap fields = new HashMap();
			fields.put("username", login);
			fields.put("password", password);
			fields.put("email", email);
			fields.put("name", name);

			try
			{
				session.getUserService().register(
				    jabberContext.getServerName(),
				    fields);
			}
			catch(JabberMessageException ex)
			{ 
				status = false;
				System.err.println(" Jabber service ->" + ex);
			}
			
			session.disconnect();
			connect();
			return status;
		}
		catch(Exception e)
		{
			System.err.println(" Jabber service ->" + e);
		}
		
		connect();
		
		return false;
	}

	/**
	 * Add to Roster
	 */
	public synchronized boolean addToRoster(String login,
	                                        String password,
	                                        String contactJID,
	                                        String contactName,
	                                        String contactGroup)
	{
		boolean status = true;
		jabberSession.disconnect();
		com.echomine.jabber.Jabber jabber = new com.echomine.jabber.Jabber();
		JabberContext context = new JabberContext(login,password,host);
		JabberSession session = jabber.createSession(context);
		
		try
		{
			session.connect(host, port); 
			session.getUserService().login();
			
			try
			{
				session.getRosterService().addToRoster(contactJID,
				                                       contactName,
				                                       contactGroup,
				                                       true);
				session.getPresenceService().subscribe(contactJID);
			}
			catch(Exception ex)
			{ 
				status=false;
				System.err.println(" Jabber service ->" + ex);
			}
			
			session.disconnect();
			connect();
			
			return status;
		}
		catch(Exception e)
		{
			System.err.println(" Jabber service ->" + e);
		}
		
		connect();
		return false;
	}
}
