package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* AllProjectsSessionEJB.java - 
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.Constants;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.StrutsProjectValue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;




/**
 * Session Bean Template
 *
 * @ejb:bean name="AllProjectsSession"
 *           display-name="AllProjectsSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/AllProjectsSession"
 *           local-jndi-name="ejb/hero/AllProjectsSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 * @ejb.permission unchecked="yes" 
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="AllProjectsSession"
 *      jndi-name="ejb/hero/AllProjectsSession"
 * 
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class AllProjectsSessionBean implements SessionBean, EventConstants  {


   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
  
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
   * Get all projects
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Supports"
   *
   **/        
	   public Collection getProjects() throws HeroException {
	   BnProjectLocalHome pHome;
	   ArrayList projects = new ArrayList();

	   try {
		   pHome=hero.interfaces.BnProjectUtil.getLocalHome();
	   } catch (javax.naming.NamingException be) {
		   throw new HeroException(be.getMessage());
	   }
	   try {
		   Collection allProjects= pHome.findAll();
		   Iterator i = allProjects.iterator();
		   while(i.hasNext()) {
		   BnProjectLocal pvalue = (BnProjectLocal)i.next();
		   StrutsProjectValue spv = new StrutsProjectValue();
		   spv.setName(pvalue.getName());
		   spv.setCreator(pvalue.getCreator());
		   spv.setState(hero.interfaces.Constants.Pj.projectStateName[pvalue.getState()]);
		   spv.setNact(pvalue.getBnNodes().size());
		   spv.setNusers(pvalue.getBnUsers().size());
		   projects.add(spv);
		   }		
		   return((Collection)projects);

	   } catch (FinderException fe) {
		   throw new HeroException(fe.getMessage());}
	   }
	   
   /**
	* Get all projects
	*
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getProjects(int offset, int numrows) throws HeroException {
	   	BnProjectLocalHome pHome;
	   	ArrayList projects = new ArrayList();

	   	try {
		   pHome=hero.interfaces.BnProjectUtil.getLocalHome();
	   	} catch (javax.naming.NamingException be) {
		   throw new HeroException(be.getMessage());
	   	}
	   	try {
		    Collection allProjects= pHome.findAll();
			int cnt = 0;
			int add = 0;
		    Iterator i = allProjects.iterator();
		    while(i.hasNext()) {
		    	BnProjectLocal pvalue = (BnProjectLocal)i.next();
				if (cnt >= offset) {
					StrutsProjectValue spv = new StrutsProjectValue();
					spv.setName(pvalue.getName());
					spv.setCreator(pvalue.getCreator());
					spv.setState(hero.interfaces.Constants.Pj.projectStateName[pvalue.getState()]);
					spv.setNact(pvalue.getBnNodes().size());
					spv.setNusers(pvalue.getBnUsers().size());
					projects.add(spv);
					add++;
					if (add == numrows) break;
				}
				cnt++;
		    }		
	 		return((Collection)projects);
	   	} catch (FinderException fe) {
 		   throw new HeroException(fe.getMessage());
	   	}
       }
	   
   /**
	* Get number of total entities
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	public int getListCnt(String type) throws HeroException {
	   	BnProjectLocalHome pHome;
	   	Collection list = null;
		int cnt = 0;
		
		if (type.equals(Constants.Pj.GETPROJECTS)) {
			try {
				pHome=hero.interfaces.BnProjectUtil.getLocalHome();
			} catch (javax.naming.NamingException be) {
				throw new HeroException(be.getMessage());
			}
			try {
				list = pHome.findAll();
				Iterator i = list.iterator();
				while(i.hasNext()) {
					i.next();
					cnt++;
				}
			} catch (FinderException fe) {
				throw new HeroException(fe.getMessage());
			}
		}
		return (cnt);
	}
	
   /**
   * Create the All Projects Session Bean
   *
   * @throws CreateException 
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
    }
    
    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }
    
    public void ejbRemove() {
    }
    
    public void ejbActivate() {
    }
    
    public void ejbPassivate() {
    }
    
}
