/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BnLdapBean.java,v 1.8 2006/11/14 10:37:48 blachonm Exp $
 *
--------------------------------------------------------------------------
 */

package hero.session;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.*;
import java.text.MessageFormat;
// added for jndi/ldap
import javax.naming.*;
import javax.naming.directory.*;

// added for Mejb manage
import javax.rmi.PortableRemoteObject;
import hero.util.HeroException;
// JMX
import javax.management.ObjectName;
import javax.management.InstanceNotFoundException;
import javax.management.Query;

// J2EE management imports
import javax.management.j2ee.ManagementHome;
import javax.management.j2ee.Management;

// For bonita Beans
import java.rmi.RemoteException;
import hero.interfaces.UserRegistrationLocalHome;
import hero.interfaces.UserRegistrationLocal;
import hero.interfaces.UserRegistrationUtil;

import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionUtil;

import hero.interfaces.UserSessionLocalHome;
import hero.interfaces.UserSessionLocal;
import hero.interfaces.UserSessionUtil;

import hero.util.HeroException;
import javax.ejb.CreateException;


/**
 * @ejb:bean name="BnLdap"
 *           display-name="BnLdap Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/BnLdap"
 *           local-jndi-name="ejb/hero/BnLdap_L"
 *
 * @ejb:ejb-ref ejb-name="BnLdap"
 *              ref-name="myhero/BnLdap"
 *
 * @ejb.ejb-external-ref 
 *       view-type="remote"
 *       ref-name="ejb/mgmt/MEJB"
 *       type="Session"
 *       home="javax.management.j2ee.ManagementHome"
 *       remote="javax.management.j2ee.Management"
 *   
 * @jonas.ejb-ref ejb-ref-name="ejb/mgmt/MEJB"       
 *       jndi-name="ejb/mgmt/MEJB" 
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin"
 * @ejb:transaction type="Required"
 * @ejb:transaction-type type="Container"
 * @jonas.bean
 *      ejb-name="BnLdap"
 *      jndi-name="ejb/hero/BnLdap"
 *
 * @jboss:ejb-ref-jndi ref-name="mgmt/MEJB"
 *                     jndi-name="ejb/mgmt/MEJB"
 *                     
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class BnLdapBean implements SessionBean {

    private String ldapRealmName = null;
    private SessionContext sessionContext = null;
    
        String baseDn = "";
        String initialContextFactory = ""; 
        String providerUrl = ""; 
        String authenticationMode = "";	 
        String securityAuthentication = "";		 	
		String securityPrincipal = "";
		String securityCredentials = "";
        String referral = "";
        String roleDN = ""; 
        String roleNameAttribute = ""; 
        String roleSearchFilter = ""; 
        String userDN = ""; 
        String userPasswordAttribute = ""; 
        String userRolesAttribute = "";
        String userSearchFilter = "";
    
    /** The directory server connection */
    private DirContext dirContext = null;
    
	/**
	* Internal Enterprise Java Beans method. 
	**/   
        public void setSessionContext(SessionContext ctx) {
        sessionContext = ctx;
        
        Context initialContext = null;
            try{
                initialContext = new InitialContext();
        } catch (Exception e) {e.printStackTrace();
            System.err.println("Cannot get initial context for JNDI: " + e);
        } 

        // --------------
        // Access to MEJB
        // --------------

        // Connecting to the MEJB home through JNDI
        ManagementHome mgmtHome = null;
        try {
            mgmtHome = (ManagementHome) PortableRemoteObject.narrow(initialContext.lookup("java:comp/env/ejb/mgmt/MEJB"), ManagementHome.class);
        }
        catch (Exception e) {e.printStackTrace();
            System.err.println("Cannot lookup java:comp/env/ejb/mgmt/MEJB: " + e);
            return;
        }            
        // Management bean creation
        Management mgmt = null;
        try {
            mgmt = mgmtHome.create();
            System.out.println("MEJB created");
        }
        catch (Exception e) {e.printStackTrace();
            System.err.println("Error creating MEJB: " + e );
        }            

        // -------------------------------------------------------
        // Access to the J2EEDomain MBean for ldap securityfactory
        // -------------------------------------------------------

        System.out.println("Access the J2EEDomain MBean");
		ObjectName objectName  = null;
        
        // get the default domain
        String defaultDomain ="";
        try {
        defaultDomain = mgmt.getDefaultDomain();
        }
        catch (Exception e) {
            System.err.println("Cannot create MEJB: " + e );
        }
        
        //Check that there's only one Ldap realm defined in the JOnAS configuration
        try {
        ObjectName searchpattern = new ObjectName(defaultDomain + ":type=securityfactory,subtype=ldap,*");
        Query q = new Query();
        Set managedObject = mgmt.queryNames(searchpattern,null);       
         
		if ((managedObject.size()) > 1) {
			System.err.println("Error detected in JOnAS configuration:");
			System.err.println("More than 1 Ldap realms has been defined");
			Iterator itr = managedObject.iterator();
			while (itr.hasNext()) {
				ObjectName name = (ObjectName)itr.next();
				System.out.println(name.toString());
			}  
		throw new RemoteException("More than 1 Ldap realms has been defined");
		         
		} // end if
		else if ((managedObject.size()) == 0) {
		System.err.println("Error detected in JOnAS configuration:");
		System.err.println("No ldap configuration has been defined.");	
		throw new RemoteException("0 Ldap realms has been defined");
		}
		else {
			// The right case ! One ldap config defined !
			Iterator itr = managedObject.iterator();
			while (itr.hasNext()) {
				objectName = (ObjectName)itr.next();
				System.out.println("1 Ldap realm name detected in JOnAS configuration:");
				System.out.println(objectName.toString());
        		
			}		
		}
		
        }catch (Exception e) {
            System.err.println("Error checking ldap objectName: " + e );
        }        
               
	if (objectName != null){
        // Get the values of attributes of the the MBean for ldap securityfactory       
        try {
            boolean exists = mgmt.isRegistered(objectName);
            if (exists) {
//System.out.println("J2EEDomain managed object registered in the current MBean server");
                
                //BaseDN
	 	if (mgmt.getAttribute(objectName, "BaseDN")!=null) baseDn = mgmt.getAttribute(objectName, "BaseDN").toString();
	 	else baseDn ="";
//System.out.println("BaseDN = " + baseDn);
	 	
	 	// InitialContextFactory
	 	if (mgmt.getAttribute(objectName, "InitialContextFactory")!=null) initialContextFactory = mgmt.getAttribute(objectName, "InitialContextFactory").toString();
	 	else initialContextFactory ="";
//System.out.println("initialContextFactory = " + initialContextFactory);
	 	
	 	//ProviderUrl
	 	if (mgmt.getAttribute(objectName, "ProviderUrl")!=null) providerUrl = mgmt.getAttribute(objectName, "ProviderUrl").toString();
	 	else providerUrl ="";
//System.out.println("providerUrl = " + providerUrl);
	 	
	 	//SecurityAuthentication
	 	if (mgmt.getAttribute(objectName, "SecurityAuthentication")!=null) securityAuthentication = mgmt.getAttribute(objectName, "SecurityAuthentication").toString();
	 	else securityAuthentication ="";
//System.out.println("securityAuthentication = " + securityAuthentication);
	 		 	 		 	
		//SecurityPrincipal
	 	if (mgmt.getAttribute(objectName, "SecurityPrincipal")!=null) securityPrincipal = mgmt.getAttribute(objectName, "SecurityPrincipal").toString();
	 	else securityPrincipal ="";
//System.out.println("securityPrincipal = " + securityPrincipal);
	 			
		//SecurityCredentials
	 	if (mgmt.getAttribute(objectName, "SecurityCredentials")!=null) securityCredentials = mgmt.getAttribute(objectName, "SecurityCredentials").toString();
	 	else securityCredentials ="";
//System.out.println("securityCredentials = " + securityCredentials);		
		
	 	// RoleDN
	 	if (mgmt.getAttribute(objectName, "RoleDN")!=null) roleDN = mgmt.getAttribute(objectName, "RoleDN").toString();
	 	else roleDN ="";
//System.out.println("roleDN = " + roleDN);
	 	
	 	//RoleNameAttribute
	 	if (mgmt.getAttribute(objectName, "RoleNameAttribute")!=null) roleNameAttribute = mgmt.getAttribute(objectName, "RoleNameAttribute").toString();
	 	else roleNameAttribute ="";
//System.out.println("roleNameAttribute = " + roleNameAttribute);	 
	 	
	 	//RoleSearchFilter		 		 	
	 	if (mgmt.getAttribute(objectName, "RoleSearchFilter")!=null) roleSearchFilter = mgmt.getAttribute(objectName, "RoleSearchFilter").toString();
	 	else roleSearchFilter ="";
//System.out.println("roleSearchFilter = " + roleSearchFilter);
	 	
	 	//UserDN
	 	if (mgmt.getAttribute(objectName, "UserDN")!=null) userDN = mgmt.getAttribute(objectName, "UserDN").toString();
	 	else userDN ="";
//System.out.println("userDN = " + userDN);
	 	
	 	//UserPasswordAttribute
	 	if (mgmt.getAttribute(objectName, "UserPasswordAttribute")!=null) userPasswordAttribute = mgmt.getAttribute(objectName, "UserPasswordAttribute").toString();
	 	else userPasswordAttribute ="";
//System.out.println("userPasswordAttribute = " + userPasswordAttribute);
	 	
	 	//UserRolesAttribute
	 	if (mgmt.getAttribute(objectName, "UserRolesAttribute")!=null) userRolesAttribute = mgmt.getAttribute(objectName, "UserRolesAttribute").toString();
	 	else userRolesAttribute ="";
//System.out.println("userRolesAttribute = " + userRolesAttribute);
	 	
	 	//UserSearchFilter
	 	if (mgmt.getAttribute(objectName, "UserSearchFilter")!=null) userSearchFilter = mgmt.getAttribute(objectName, "UserSearchFilter").toString();
	 	else userSearchFilter ="";
//System.out.println("userSearchFilter = " + userSearchFilter);
	 	
	 	// optional : Referral   (parameter value = "throw")
	 	if (mgmt.getAttribute(objectName, "Referral")!=null) referral = mgmt.getAttribute(objectName, "Referral").toString();
	 	else referral ="";
//System.out.println("referral = " + referral);	 	
            }
            else {
                System.out.println("Can't find J2EEDomain managed object having the above name in the current MBean server");
                return;
            }
        }
        catch (Exception e) {
            System.err.println("Error when using the J2EEDomain managed object: " + e);
            return;
        }        
	}
            
	}    

	/**
	 * Import users from the LDAP used for authentication by the Application Server 
	 * to Bonita database.
	 * For each user found in LDAP:
	 * - if it doesn't exists in bonita database then this user is created
	 * - if it exists in bonita database then the mail property is updated.
	 * It requires that the ldap resource within the AS. is configured and activated. 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	**/	 
	public void importLdapUsers()throws HeroException{
		

	   String userId ="";
	   String email = "";
	   UserRegistrationLocalHome userRegistrationh=null;
	   UserRegistrationLocal userRegistration=null;
	   
	   UserSessionLocalHome userSessionh=null;
	   UserSessionLocal userSession=null;	   

	   // Initialize the InitialContext passed on the
	   //  connection parameters
	   Hashtable env = new Hashtable();
    	
	   env.put(Context.INITIAL_CONTEXT_FACTORY, initialContextFactory);
	   env.put(Context.PROVIDER_URL, providerUrl);
	   env.put(Context.SECURITY_AUTHENTICATION, securityAuthentication); 
	   env.put(Context.SECURITY_PRINCIPAL, securityPrincipal); 
	   env.put(Context.SECURITY_CREDENTIALS, securityCredentials);  	
	   env.put(Context.REFERRAL, referral);
	   
		 try
		   {
		   dirContext = new InitialDirContext(env);
		   }
	   catch (NamingException e)
		   {
				   System.err.println("cannot initialize the initialContext of naming" + e);
				   throw new EJBException("cannot initialize the initialContext of naming");
		   }
		   
		// Get users into ldap and Bonita database to be able to compare ...
		Map map = null;
		Collection bnUsersCol = null;
	   try {
		map = getTreeMapUidMailLdap();
	   } catch (Exception e){
	   throw new HeroException("Error getting users from LDAP " + e.getMessage());
	  } 
	    try {  
		bnUsersCol = getAllSortedBnUsers();	   				  
	   } catch(Exception e){
	   	throw new HeroException("Error getting users from bonita database " + e.getMessage());
	   }
	
			   	   
	   try {
		userRegistrationh = (UserRegistrationLocalHome)UserRegistrationUtil.getLocalHome();
		userRegistration = userRegistrationh.create();
		 } catch (Exception e){
				   e.printStackTrace();
				   System.err.println("Error creating userRegistration Bean" + e);
		 }
		 
		try { 
		userSessionh = (UserSessionLocalHome)UserSessionUtil.getLocalHome();
		userSession = userSessionh.create();
		
		 } catch (Exception e){
				  e.printStackTrace();
				  System.err.println("Error creating userSession Bean" + e);
		 }
      
        try { 
        // Look for each entry the Ldap if it exists in Bonita DB
     	Set entries = map.entrySet();
     	Iterator iterator = entries.iterator();
     	  
     	  while (iterator.hasNext()) {
       		Map.Entry entry = (Map.Entry)iterator.next();
       		userId = entry.getKey().toString();
       	  	email = entry.getValue().toString();
       	  	if (!bnUsersCol.contains(userId)){
       	  		// create the user in Bonita DB.
				userRegistration.userCreate(userId, userId, email);
       	  		// debug
       	  		System.out.println("Create user in Bonita DB: " + entry.getKey() + " / " + entry.getValue());
       	  	}
       	  	else {
       	  		System.out.println( "User already into Bonita database then set the mail of this user : " + entry.getKey() + " / " + entry.getValue());
       	  		// set/update the mail attribute
				userSession.setUserMail(userId, email);
				// suppress it from bnUsersCol to prevent removal as done after
				bnUsersCol.remove(userId);
			}	   
     	  } 
     	  
	   // Prevent removing mandatory/default users for the Bonita DB.
	   if (bnUsersCol.contains("admin")) bnUsersCol.remove("admin");
	   if (bnUsersCol.contains("admin2")) bnUsersCol.remove("admin2");
	   if (bnUsersCol.contains("nobody")) bnUsersCol.remove("nobody");
	   
	   // Remaining users in the bnUsersCol list doesnt exist in Ldap then delete these.	      	  
     	  Iterator it=bnUsersCol.iterator();
     	  while (it.hasNext()) {
     	  	userId = (String)it.next();
     	  	//debug
     	  	System.out.println( "Deleting user in Bonita DB: " + userId);
     	  	//delete user from bonita DB
     	  	try {
				userRegistration.deleteUser(userId);
		    } catch (HeroException e) {  
			  System.out.println( "Cannot remove user: " + userId + e.getMessage());			
     	    }
     	  }
     	// remove the beans 
	  	userRegistration.remove();
		userSession.remove();
       
	  } catch (Exception e) {
	  	e.printStackTrace();
	    System.err.println("Error executing import of users from ldap into Bonita DB. " + e);
	  }

	   // Disconnect from the directory server
		try { dirContext.close(); } catch (Exception e) {};
		
	 }

	/**
	 * Get the members of a given group into the ldap server.
	 * It requires that the ldap resource within the AS. is configured and activated. 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	**/		
	public Collection getGroupMembers(String groupName) throws HeroException{	
	  	
	  // Initialize the InitialContext passed on the
	  //  connection parameters
	  Hashtable env = new Hashtable();
    	
	  env.put(Context.INITIAL_CONTEXT_FACTORY, initialContextFactory);
	  env.put(Context.PROVIDER_URL, providerUrl);
	  env.put(Context.SECURITY_AUTHENTICATION, securityAuthentication); 
	  env.put(Context.SECURITY_PRINCIPAL, securityPrincipal); 
	  env.put(Context.SECURITY_CREDENTIALS, securityCredentials);  	
	  env.put(Context.REFERRAL, referral);
		try
		  {
		  dirContext = new InitialDirContext(env);
		  }
	  catch (NamingException e)
		  {
				  System.err.println("cannot initialize the initialContext of naming" + e);
				  throw new EJBException("cannot initialize the initialContext of naming");
		  }	 
		ArrayList al = new ArrayList();		  

		// Retreiving groups in LDAP is supposed to be based on:
		//    - LDAP objectclass = groupOfUniqueNames
		//    - LDAP attribute containing members = uniquemember
		//    - It is not recursive: the groups inside a group are not taken into account
		String dn = "";
		String filter = "(&(objectclass=groupOfUniqueNames)(" + roleNameAttribute + "=" + groupName + "))";
		// Specify the ids of the attributes to return and the scope of the search
		SearchControls constraints = new SearchControls();
		String[] attrID1s = {roleNameAttribute};
		String[] attrID2s = {"uniquemember"};
		
		
		constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
		//constraints.setSearchScope(SearchControls.OBJECT_SCOPE) ;
		// constraints.setSearchScope(SearchControls.ONELEVEL_SCOPE) ;	     
	    String subTree= roleDN + "," + baseDn;
		NamingEnumeration ne = null;
		int taille = 0;
		
		// Check if the group exist in the LDAP	
		try {
		constraints.setReturningAttributes(attrID1s);
		ne = dirContext.search(subTree, filter, constraints); 		
		}catch (NamingException e) {
		  throw new HeroException("Error searching groups in LDAP " + e);
		}			  
		try {
		  while (ne.hasMore()){    
			SearchResult sr = (SearchResult)ne.next();
			Attributes attrs = sr.getAttributes();
			// logs trace
			System.out.println("LDAP group searching : " + attrs.toString());	
						  
			taille = attrs.size();
		  }
		} catch (NamingException e) {
			throw new HeroException("Error dirContext.search" + e);
		}
		
		
		if ( taille == 0 ) {
			// logs trace
			System.err.println("The group " + groupName + " does not exist in the LDAP");			
			throw new HeroException("The group " + groupName + " does not exist in the LDAP ");
		}
		else {				
				try
				  {	
					constraints.setReturningAttributes(attrID2s);    
			    	ne = dirContext.search(subTree, filter, constraints);  
				  }catch (NamingException e){
				  	throw new HeroException ("Error searching groups in LDAP " + e);
				  }
				  
				// get the members 
				try {
			
				while (ne.hasMore()){    
						SearchResult sr = (SearchResult)ne.next();
						//get the members (dn)
						Attributes attrs = sr.getAttributes();
						  				 			
					for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
						Attribute attr = (Attribute)ae.next();
						// logs trace
						System.out.println("attribute: " + attr.getID());
		
						//print each value 
						for (NamingEnumeration e = attr.getAll(); e.hasMore();){
						  dn = (String) e.next();
						  
						  // logs trace
						  System.out.println("    dn: " + dn);
						  
						  // dn : cn = X, subtree 
						  // 3 cases:
						  //    1- if subtree = UserDn then get the User Id
						  //    2- if subtree = RoleDn then the member is also a group, then call recusively the getGroupMembers ??
						  //    3- if subtree != UserDn and != RoleDn then return null	
						  //   => At now we suppose that we are always in case 1
						  //get the uid of the dn's ldap object
						  String userId = getUserIdFromDn(dn) ;
						  
						  // logs trace
						  System.out.println("    uid: " +userId);	
						  
						  if (!"".equals(userId)){
						  	al.add(userId);				
						  }
						}				 
					}													  	
				  } // end while
				ne.close();	    	
				 } catch (NamingException e){
							e.printStackTrace();
							throw new HeroException("Error searching userId from group members in LDAP");
				}
		} // end else
		return al;
	}

	/**
	 * Get the uid giving a partial DN (without Base suffix) as input.
	 * It requires that the ldap resource within the AS. is configured and activated. 
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	**/		
	public String getUserIdFromPartialDn(String partialDN) throws HeroException{
			//Initialize the InitialContext passed on the
		   //  connection parameters
		   Hashtable env = new Hashtable();
	    	
		   env.put(Context.INITIAL_CONTEXT_FACTORY, initialContextFactory);
		   env.put(Context.PROVIDER_URL, providerUrl);
		   env.put(Context.SECURITY_AUTHENTICATION, securityAuthentication); 
		   env.put(Context.SECURITY_PRINCIPAL, securityPrincipal); 
		   env.put(Context.SECURITY_CREDENTIALS, securityCredentials);  	
		   env.put(Context.REFERRAL, referral);
		   
			 try
			   {
			   dirContext = new InitialDirContext(env);
			   }
		   catch (NamingException na)
			   {		   			
					throw new HeroException("cannot initialize the initialContext of naming" + na.getMessage(), na);
			   }
		   
			ArrayList al = new ArrayList();
	        String userId = "";
			String filter = "uid=*";	
			NamingEnumeration ne = null;		    
			// Specify the ids of the attributes to return 
			//          and the scope of the search
			SearchControls constraints = new SearchControls();
			String[] attrIDs = {"uid"};
			constraints.setReturningAttributes(attrIDs);
			//constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
			constraints.setSearchScope(SearchControls.OBJECT_SCOPE) ;
		
		   // Search for objects that have those matching attributes
		   try {
//debug
System.out.println("BnLdapBean/getUserIdFromPartialDn:  partialDN = " + partialDN + " baseDn = " + baseDn );
			   ne = dirContext.search(partialDN + "," + baseDn, filter, constraints);			    
				while (ne.hasMore())
				  {
				  SearchResult sr = (SearchResult)ne.next();      
				  //String dn = sr.getName() + ", " + userDN + "," + baseDn; 
				  Attributes attrs = sr.getAttributes();
				  Attribute attr = attrs.get("uid");
				  userId = (String)attr.get();		  		  
				  }
				ne.close();
		   } catch (NamingException na)
			{
						throw new HeroException("Error searching directory :" + na.getMessage(), na);
			}
			return userId;
		  } 
	
	/***************************************************************************
	                     getSortedMapUidMailLdap   
	         Target: Get user "uid" and "mail" and return a sortedMap
	****************************************************************************/  	
	private Map getTreeMapUidMailLdap() {
	    TreeMap treeMap = new TreeMap();	
	  try
	    {

	    // using the User Search Filter as jonas-realm_1_0.dtd : "uid={0}" 
	    MessageFormat form = new MessageFormat(userSearchFilter);
	    // Get all user by replacing the parameter {0} by * 
	    Object[] testArgs = {"*"};
	    String filter = form.format(testArgs);
	    
	    // logs trace
	    System.out.println("Filter used to get user in Ldap: " + filter);
	    
	    // Specify the ids of the attributes to return 
	    //          and the scope of the search
	    SearchControls constraints = new SearchControls();
	    
	    // get the attribut name from "User serach Filter" ie: uid={0}	    
	    int indSep = userSearchFilter.indexOf('=');
            StringBuffer sb = new StringBuffer(userSearchFilter);
            String userParameter = sb.substring(0,indSep);
            
        // logs trace
 	    System.out.println("UserParameter from userSearchFilter = " + userParameter);
	    
	    String[] attrIDs = {userParameter,"mail"};
	    constraints.setReturningAttributes(attrIDs);
	    
	    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
	    
	    // logs trace
	    // Set SearchControl for "CountLimit" and "TimeLimit" for information
	    System.out.println("Naming CountLimit = " +  constraints.getCountLimit());
	    System.out.println("Naming TimeLimit = " +  constraints.getTimeLimit());
	
	    // Search for objects that have those matching attributes
	    NamingEnumeration ne = dirContext.search(userDN + "," + baseDn, filter, constraints);   
	    String userId ="";
	    String userMail ="";
	    Attribute attr = null;	    
	    while (ne.hasMore())
	      {
	      SearchResult sr = (SearchResult)ne.next();      

	       
	      Attributes attrs = sr.getAttributes();
	      // uid -> key
	      attr = attrs.get(userParameter);
	      userId = (String)attr.get();
	      // mail -> value
	      if (attrs.get("mail") != null) {
	      	        attr = attrs.get("mail");
	      		userMail = (String)attr.get();
	      	}
	      else {
	      	userMail = "";
	      }
	      	      
	      treeMap.put(userId,userMail);
	      }
	    ne.close();
	    
	    // logs trace
	    System.out.println("------------------" + userParameter + " and mail found un Ldap -------------------------");
	    printUidMailCollection(treeMap);	    
	    } // end try
	  catch (SizeLimitExceededException e) 
	    {
	    	// logs trace
	  		System.err.println(e);
			System.err.println("");	  		
	  		System.err.println("Sizelimit Exceeded from the LDAP server.");
	  		System.out.println("Please contact the ldap administrator to extend this limit,");
	    	System.out.println("or reduce the scope of the user searching.");
	    	System.err.println("");
	    	e.printStackTrace();	    		
	    }
	  catch (NamingException e)
	    {
	    	e.printStackTrace();
	        System.err.println("Error reading directory" + e);
	        throw new EJBException("Error reading directory");
	    }
	   	    return treeMap;
	  }
 	
	
 
	/***************************************************************************
	                     getUidFromLdap   
	         Target: Get user uid and return a sorted collection
	****************************************************************************/  	
	private Collection getUidFromLdap() {
	    TreeSet sorter = new TreeSet();	
	  try
	    {

	    // using the User Search Filter as jonas-realm_1_0.dtd : "uid={0}" 
	    MessageFormat form = new MessageFormat(userSearchFilter);
	    // Get all user by replacing the parameter {0} by * 
	    Object[] testArgs = {"*"};
	    String filter = form.format(testArgs);
	    
	    // logs trace
	    System.out.println("Filter used to get user in Ldap: " + filter);
	    
	    // Specify the ids of the attributes to return 
	    //          and the scope of the search
	    SearchControls constraints = new SearchControls();
	    
	    // get the attribut name from "User serach Filter" ie: uid={0}	    
	    int indSep = userSearchFilter.indexOf('=');
            StringBuffer sb = new StringBuffer(userSearchFilter);
            String userParameter = sb.substring(0,indSep);
            
        // logs trace
 	    System.out.println("UserParameter from userSearchFilter = " + userParameter);
	    
	    String[] attrIDs = {userParameter};
	    constraints.setReturningAttributes(attrIDs);
	    
	    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
	    
	    // logs trace
	    // Set SearchControl for "CountLimit" and "TimeLimit" for information
	    System.out.println("Naming CountLimit = " +  constraints.getCountLimit());
	    System.out.println("Naming TimeLimit = " +  constraints.getTimeLimit());
	
	   // Search for objects that have those matching attributes
	   NamingEnumeration ne = dirContext.search(userDN + "," + baseDn , filter, constraints);   
	    
	    while (ne.hasMore())
	      {
	      SearchResult sr = (SearchResult)ne.next();      
	      //String dn = sr.getName() + ", " + userDN + "," + baseDn; 
	      Attributes attrs = sr.getAttributes();
	      Attribute attr = attrs.get(userParameter);
	      String userId = (String)attr.get();
	      sorter.add(userId);
	      // logs trace
	      System.out.println(userId);
	      }
	    ne.close();
	    
	    // logs trace
	    System.out.println("------------------" + userParameter + " found in Ldap -------------------------");
	    
	    printUidCollection(sorter);	    
	    } // end try
	  catch (SizeLimitExceededException e) 
	    {
	  		System.err.println(e);
	  		System.err.println("Sizelimit Exceeded from the LDAP server.");
	  		System.out.println("-> Please contact the ldap administrator to extend this limit.");
	    		System.out.println("-> or reduce the scope of the user searching.");
	    		System.err.println("Stack of the Error:");
	    		e.printStackTrace();	    		
	    }
	  catch (NamingException e)
	    {
	    	        e.printStackTrace();
	                System.err.println("Error reading directory" + e);
	                throw new EJBException("Error reading directory");
	    }
	   	    return sorter;
	  }
 
  	
	/***************************************************************************
	     getAttributesFromUser   
	     ex. to get attributes in naming (ldap) service 
	****************************************************************************/  	
	private Collection getAttributesFromUser(String userID) {
	
	  try
	    {
	    ArrayList al = new ArrayList();
	    
	    // using the User Search Filter as jonas-realm_1_0.dtd : "uid={0}" 
	    MessageFormat form = new MessageFormat(userSearchFilter);
	    Object[] testArgs = {userID};
	    String filter = form.format(testArgs);	
	        
	    // logs trace
	    System.out.println("Filter to get the users name in LDAP: " + filter);
	    
	    // Specify the ids of the attributes to return 
	    //          and the scope of the search
	    SearchControls constraints = new SearchControls();
	    String[] attrIDs = {"sn", "givenname", "cn", "mail"};
	    constraints.setReturningAttributes(attrIDs);
	    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
	
	   // Search for objects that have those matching attributes
	   NamingEnumeration ne = dirContext.search(userDN + "," + baseDn, filter, constraints);   
	    
	    while (ne.hasMore())
	      {
	      SearchResult sr = (SearchResult)ne.next();      
	      //String dn = sr.getName() + ", " + userDN + "," + baseDn; 
	      Attributes a = sr.getAttributes();
	      al.add(a);
	      }
	    ne.close();
	    
	    // logs trace
	    printUserAttributes(al);
	    
	    return al;
	    
	    }
	  catch (NamingException e)
	    {
	    	        e.printStackTrace();
	                System.err.println("Error reading directory" + e);
	                throw new EJBException("Error reading directory");
	    }
	  }  	 
    

	private String getUserIdFromDn(String dn) throws HeroException{
	
	  try
		{
	
		ArrayList al = new ArrayList();
	    
        String userId = "";
		//String filter = "dn=" + dn;
		String filter = "uid=*";	    
		// debug
		System.out.println("Filter to identify the user to get it's attributes: " + filter);
	    
		// Specify the ids of the attributes to return 
		//          and the scope of the search
		SearchControls constraints = new SearchControls();
		String[] attrIDs = {"uid"};
		constraints.setReturningAttributes(attrIDs);
		//constraints.setSearchScope(SearchControls.SUBTREE_SCOPE) ;
		constraints.setSearchScope(SearchControls.OBJECT_SCOPE) ;
	
	   // Search for objects that have those matching attributes
	   //NamingEnumeration ne = dirContext.search(userDN + "," + baseDn, filter, constraints);   
	   NamingEnumeration ne = dirContext.search(dn, filter, constraints);	
		while (ne.hasMore())
		  {          
		  SearchResult sr = (SearchResult)ne.next();      
		  //String dn = sr.getName() + ", " + userDN + "," + baseDn; 
		  Attributes attrs = sr.getAttributes();
		  Attribute attr = attrs.get("uid");
		  userId = (String)attr.get();		  		  
		  }
		ne.close();
		return userId;
	    
		}
	  catch (NamingException e)
		{
                    System.err.println("ERROR getting Id from dn = " + dn  + " - " + e);
					throw new HeroException("Error reading directory");
		}
	  }  	 
     

		        	
	/**********************************************
	     get all users in bonita database
	**********************************************/
     private Collection getAllSortedBnUsers() throws HeroException{
     TreeSet sorter = new TreeSet();	
	Collection unSortedList = null;
	try {
	ProjectSessionLocalHome projecth = (ProjectSessionLocalHome)ProjectSessionUtil.getLocalHome();
	ProjectSessionLocal projectsession = projecth.create();
	// get users in bonita database
	unSortedList = projectsession.getAllUsers();
	
	Iterator it = unSortedList.iterator();
	while (it.hasNext()){
		sorter.add(it.next());
	}
	} catch (Exception e){
    	        e.printStackTrace();
                System.err.println("Error in BnLdapBean for getAllSortedBnUsers " + e);
                throw new HeroException("Error in BnLdapBean for getAllSortedBnUsers" + e.getMessage());
        }
    // logs trace
	System.out.println("------------------ Bonita registered users -------------------------");        
        printUidCollection(sorter);
        return sorter;
     }

	/**
	 * Creates the BnLdap Bean. This method is the first one to invoke in order to
	 * use BnLdap API. 
	 * The client user of the API must be authentified otherwise an exception will be thrown
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void  ejbCreate() {
	}


	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbActivate() {
	}


	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbPassivate() {
	// Nothing to do for this simple example
	}


	/**
	* Internal Enterprise Java Beans method. 
	**/
	public void ejbRemove() {
	// Nothing to do for this simple example
	}


    /****************************************************************
          Utility:     printUidCollection   Only to print out !
     ***************************************************************/
    private static void printUidCollection(Collection co){
          // Example for searching user attributes in ldap giving the "uid" attribute  (not unique !)
          try {          
          Collection c = co;
  	  Iterator it = c.iterator();
    	 Attributes a=null;
    	 while (it.hasNext()) {
    	 	//a = (Attributes)it.next();
	 	//printAttrs(a);
	 	System.out.println( (String)(it.next()) );
	 }// end while
	 } catch (Exception e) {
	    System.err.println("erreur Client pour la recherche des uid dans ldap : " + e);
	    System.exit(2);
	 }    	  
   }   

    /****************************************************************
          Utility:     printUidMailCollection return TreeMap
     ***************************************************************/
    private static void printUidMailCollection(Map map){
          // Example for searching user attributes in ldap giving the "uid" attribute  (not unique !)
          try { 
     	  Set entries = map.entrySet();
     	  Iterator iterator = entries.iterator();
     	  while (iterator.hasNext()) {
       		Map.Entry entry = (Map.Entry)iterator.next();
       		System.out.println(entry.getKey() + " : " + entry.getValue());
     	  }        
	  } catch (Exception e) {
	    System.err.println("erreur Client pour la recherche des uid dans ldap : " + e);
	    System.exit(2);
	  }    	  
   }
    
              
    /****************************************************************
            Utility:     printUserAttributes
     ***************************************************************/
    private static void printUserAttributes(Collection co){
          // Example for searching user attributes in ldap giving the "uid" attribute  (not unique !)
          try {          
          Collection c = co;
  	  Iterator it = c.iterator();
    	 Attributes a=null;
    	 System.out.println("------------------ search attributes -------------------------");
    	 while (it.hasNext()) {
    	 	a = (Attributes)it.next();
	 	printAttrs(a);
	 }// end while
	 } catch (Exception e) {
	    System.err.println("erreur chez le client pour la recherche des attributs dans ldap: " + e);
	    System.exit(2);
	 }    	  
   }


    /****************************************************************
                Utility : printGroupMember
     ***************************************************************/
    private static void printGroupMember(Collection co){
         // Example for searching mmber of a goup in ldap
         try {
         Attributes a=null;
         // recup d'une array list of SearchResult
         Collection c = co;
  	 Iterator it = c.iterator();
  	 System.out.println("");
  	
    	 while (it.hasNext()) {
    	    SearchResult sr = (SearchResult)it.next();
    	    // recup du nom du name et des uniquemember du groupe
    	      	System.out.println("");
    	    	System.out.println("------- Group name = " + sr.getName());
            	a = sr.getAttributes();    	   
                printAttrs(a);
	 } // end while
	 } catch (Exception e) {
	    System.err.println("Client error pour la recherche de groupes dans ldap : " + e);
	    System.exit(2);
	 }    
      }
    /***********************************
    Utility for printing   for debug ...
    ************************************/
    private static void printAttrs(Attributes attrs) {
	if (attrs == null) {
	    System.out.println("No attributes");
	} else {
	    /* Print each attribute */
	    try {
		for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
		    Attribute attr = (Attribute)ae.next();
		    System.out.println("attribute: " + attr.getID());

		    /* print each value */
		    for (NamingEnumeration e = attr.getAll(); e.hasMore();
			 System.out.println("value: " + e.next()))
			;
		}
	    } catch (NamingException e) {
		e.printStackTrace();
	    }
	}
    }

    	
} // end class
