package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* AllProjectsSessionEJB.java -
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.DeadlineSessionLocal;
import hero.interfaces.DeadlineSessionLocalHome;
import hero.interfaces.DeadlineSessionUtil;
import hero.util.TimerData;

import hero.util.HeroException;
import java.util.Date;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.TimedObject;
import javax.ejb.Timer;
import javax.ejb.TimerService;


/**
 * Session Bean Template
 *
 * @ejb:bean name="DeadlineEjbTimerSession"
 *           display-name="DeadlineEjbTimerSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/DeadlineEjbTimerSession"
 *           local-jndi-name="ejb/hero/DeadlineEjbTimerSession_L"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission unchecked="yes"
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="DeadlineEjbTimerSession"
 *      jndi-name="ejb/hero/DeadlineEjbTimerSession"
 *
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class DeadlineEjbTimerSessionBean implements SessionBean, TimedObject {

   private SessionContext mContext;

		/**
		 * Start the EJB timers for node deadlines .
		 *
		 * @param name the name of the node
		 * @param p_id id of the project
		 * @ejb:interface-method view-type="both"
		 * @ejb:transaction type="Required"
		 * @throws HeroException
		 *
		 **/
		   public void startDeadlineEjbTimers(String nodeName, String p_id) throws HeroException{

					try {
						BnNodeLocalHome lh = BnNodeUtil.getLocalHome();
						BnNodeLocal node = lh.findByName(nodeName,p_id);

						String type = null;
						Timestamp lNext=null;

						// For absolute deadline
						if (node.getDeadlines().size()!=0) {
//debug
//System.out.println("startDeadlines : 	Nb of absolute deadlines = " + 	node.getDeadlines().size());
							Collection co = node.getDeadlines();
							Iterator it = co.iterator();

							while (it.hasNext()){
								// the co contains Date elements
								Timestamp date = (java.sql.Timestamp)it.next();
								type=node.getName() + ":" + node.getBnProject().getName()+":"+date.getTime();
								lNext = date;
								// add notification for each deadline in the collection

								TimerService ts = mContext.getTimerService();
								// addNotification(lNext, type, node);
						         // Create the timer with the date
								TimerData info = new TimerData(type, lNext.getTime());
						        Timer timer = ts.createTimer(date, info);
						        //info.onStart(timer);
//debug
//System.out.println("startDeadlines for absolute deadlines: lnext = " + lNext.toString() + " type = " + type + " node = " + node.getName());
							}
						}
						// For relative deadline
						if (node.getRelativeDeadlines().size()!=0) {
//debug
//System.out.println("startDeadlines : 	Nb of relative deadlines = " + 	node.getRelativeDeadlines().size());
							Collection co = node.getRelativeDeadlines();
							Iterator it = co.iterator();

							while (it.hasNext()){
								// the co contains Date elements
								java.sql.Timestamp date = (java.sql.Timestamp)it.next();
								type=node.getName() + ":" + node.getBnProject().getName()+":"+date.getTime();
								//create EJB timers
								lNext=new Timestamp(new Date().getTime()+date.getTime());
								TimerService ts = mContext.getTimerService();
								TimerData info = new TimerData(type, lNext.getTime());
						        Timer timer = ts.createTimer(lNext, info);
						        //call method onStart:    not used !
						        //info.onStart(timer);
//debug
//System.out.println("startDeadlines for relative deadlines: lnext = " + lNext.toString() + " type = " + type + " node = " + node.getName());
							}
						}


						} catch (Exception e) {
							throw new HeroException("ProjectSessionBean.startDeadlinesEjbTimers: " + e.getMessage());
						}
			   }

			/**
			 * Stop the EJB timer for node deadline .
			 *
			 * @param  projectName the name of the project
			 * @param  nodeName the name of the node
			 * @param date date
			 * @ejb:interface-method view-type="both"
			 * @ejb:transaction type="Required"
			 * @throws HeroException
			 *
			 **/
		   public void stopDeadlineEjbTimer(String projectName, String nodeName, long date) throws HeroException{
			      try {
			         TimerService ts = mContext.getTimerService();
			         Collection timers = ts.getTimers();
			         Iterator it = timers.iterator();
			         String type=nodeName + ":" + projectName + ":" + date;
			         while (it.hasNext()) {
			            Timer t = (Timer)it.next();

			            if (isCalled(t, type)) {
			               TimerData td = (TimerData)t.getInfo();
			               td.onStop(t);
			            }
			         }
			      } catch (Exception e) {
			    	  throw new HeroException("ProjectSessionBean.stopDeadlineEjbTimer: " + e.getMessage());
			      }
			   }

		   /**
			 * timeout is called after time duration within timer
			 *
			 * @param  Timer timer from the EJB container-managed timer service
			 * @ejb:interface-method view-type="both"
			 * @ejb:transaction type="Required"
			 * @throws HeroException
			 *
			 **/
		   public void ejbTimeout(Timer timer) {
				DeadlineSessionLocalHome dlHome = null;
				DeadlineSessionLocal dlSession = null;
				try {
					dlHome = DeadlineSessionUtil.getLocalHome();
				} catch (javax.naming.NamingException ne) {
					System.out.println("Error: DeadlineEjbTimerSessionBean: getting DeadlineSessionLocalHome");
				}
				try {
					dlSession = dlHome.create();
				} catch (CreateException he) {
					System.out.println("Error: DeadlineEjbTimerSessionBean: creating DeadlineSessionLocal");
				}
		      Object info = timer.getInfo();
//debug
//System.out.println("====> timeoutDeadlineEjbTimer:    has been called !!!!");
		      if (info != null && info instanceof TimerData) {
		         TimerData td = (TimerData)info;
		         	/*
					try {
						td.onTimeout(timer);
					} catch (HeroException e) {
						e.printStackTrace();
					}
					*/
			 		try{
						String type = (String)td.getType();
						String projectName = type.substring(type.indexOf(":")+1);
						projectName = projectName.substring(0,projectName.indexOf(":"));
						String nodeName = type.substring(0,type.indexOf(":"));
						dlSession.executeHookDeadline(type, projectName, nodeName);
					}
					catch(Exception hh)
					{
						hh.printStackTrace();
					}
		      }
		   }

			protected boolean isCalled(Timer t, String timerType) throws HeroException{
				  try {
				       Object info = t.getInfo();
				       if (info != null && info instanceof TimerData) {
				            TimerData td = (TimerData)info;
				            if (td.getType().equals(timerType)) {
				               return true;
				            }
				         }
				      } catch (Exception e) {
				    	  throw new HeroException("DeadlineEjbTimerSession: Error in isCalled: " + e.getMessage());
				      }
				      return false;
				   	}


    /**
     * Retrieves current timers
     *
     * @return Collection of <tt>TimerData</tt> objects
     * @ejb:interface-method view-type="both"
     **/
    public Collection getTimers() {
      Collection ret = new ArrayList();

      // Iterates through all timers
      Collection timers = mContext.getTimerService().getTimers();
      for(Iterator iterator = timers.iterator() ; iterator.hasNext() ; ) {
        Serializable info = ((Timer) iterator.next()).getInfo();
        
        // Check the Timer info data to increase robustness
        if(info instanceof TimerData) {
          ret.add(info);
        }
      }

      return ret;
    }
      
    /**
     * Remove on going timers for a particular process
     *
     * @ejb:interface-method view-type="both"
     **/
    public void removeProjectTimers(String projectName)throws HeroException {
		 Collection timers = mContext.getTimerService().getTimers();
	      for(Iterator iterator = timers.iterator() ; iterator.hasNext() ; ) {
			  Timer t = (Timer) iterator.next();
	          TimerData td = (TimerData)t.getInfo();
	          // Check the Timer info data to increase robustness
			  if (td.getType().matches(".*" + ":" + projectName + ":" + ".*")) {
				td.onStop(t);
			}
		}
    }
      
	
    public void ejbCreate() throws CreateException {
    }

    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }

    public void ejbRemove() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

}
