package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* AllProjectsSessionEJB.java -
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.Constants;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceValue;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.HeroHookException;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaProjectValue;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;




/**
 * Session Bean Template
 *
 * @ejb:bean name="DeadlineSession"
 *           display-name="DeadlineSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/DeadlineSession"
 *           local-jndi-name="ejb/hero/DeadlineSession_L"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission unchecked="yes"
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="DeadlineSession"
 *      jndi-name="ejb/hero/DeadlineSession"
 *
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class DeadlineSessionBean implements SessionBean, EventConstants  {


   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------

   // -------------------------------------------------------------------------
   // Members
   // -------------------------------------------------------------------------

   private SessionContext mContext;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------

   /**
   * Get all projects
   *
   * @ejb.permission unchecked="yes"
   * @ejb:interface-method view-type="both"
   * @ejb:transaction type="Required"
   *
   **/
	   public void executeHookDeadline(String type, String projectName, String nodeName) throws HeroException {
		   BnProjectLocal mProject = null;
			BonitaProjectValue model;
			BonitaNodeValue node;
			Collection hooks;

			try
			{
				BnProjectLocalHome pHome = BnProjectUtil.getLocalHome();
				mProject=pHome.findByName(projectName);
			}
			catch(Exception ple)
			{
				ple.printStackTrace();
			}

			try
			{
				BonitaNodeValue nodeLocal;
				if (mProject.getType().equals(Constants.Pj.INSTANCE))
				{
					model = BonitaProjectLocator.getInstance().getModelValue(this.getModel(mProject.getName()), this.getModel(mProject.getVersion()));
					node = BonitaServiceValue.getNodeFromCache(model,nodeName);

					if (nodeName.matches(".*_instance.*"))
						nodeName=nodeName.substring(0,nodeName.indexOf("_instance"));
					nodeLocal = BonitaServiceValue.getNodeFromCache(model,nodeName);
				}
				else
				{
					nodeLocal = BonitaServiceValue.getNode(mProject,nodeName);
				}

				hooks = nodeLocal.getHooks();
				for (Iterator i = hooks.iterator(); i.hasNext();)
				{
					BonitaHookValue hk = (BonitaHookValue) i.next();
					if (hk.getEvent().equalsIgnoreCase(hero.interfaces.Constants.Nd.ONDEADLINE))
					{
						hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), hero.interfaces.Constants.Nd.ONDEADLINE, hk.getType());
						h.execute(this, hero.interfaces.Constants.Nd.ONDEADLINE, mProject.getBnNode(nodeLocal.getName()));
					}
				}
			}
			catch(Exception hh)
			{
				mContext.setRollbackOnly();
				throw new HeroHookException("Cannot execute hook "+hh.getMessage());
			}
	   }
//	 Get the name of the project model of this instance
		private String getModel(String instanceName) {
			int i = instanceName.indexOf("_instance");
			if (i!=-1)
				return (instanceName.substring(0, i));
			else
				return(instanceName); // for cooperative processes
		}

   /**
   * Create the All Projects Session Bean
   *
   * @throws CreateException
   *
   * @ejb.permission unchecked="yes"
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
    }

    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }

    public void ejbRemove() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

}
