package hero.session;
/*
 * 01/02/2004 - 17:04:13
 *
 * ProjectSessionEJB.java -
 * Copyright (C) 2004 Ecoo Team
 * valdes@loria.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import hero.entity.EdgeState;
import hero.entity.NodeState;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnIterationLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSessionLocal;
import hero.interfaces.UserSessionLocalHome;
import hero.interfaces.UserSessionUtil;
import hero.util.BonitaConfig;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceException;
import hero.util.BonitaServiceValue;
import hero.util.EngineException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.HeroHookException;
import hero.util.BonitaParsing;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import hero.interfaces.BnNodePerformerAssignLocal;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaInterHookValue;
import hero.util.values.BonitaIterationValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaProjectValue;
import hero.util.values.BonitaPropertyValue;
import hero.interfaces.Constants;
import hero.interfaces.Constants.Nd;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import bsh.Interpreter;

/**
 * Session Bean Template
 *
 * @ejb:bean name="Engine"
 *           display-name="Engine Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/Engine"
 *           local-jndi-name="ejb/hero/Engine_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Required"
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="Engine"
 *      jndi-name="ejb/hero/Engine"
 *
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class EngineBean implements SessionBean, Constants {

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger log = Logger.getLogger("log");

	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private JMSServicesSessionLocal jms;
	private boolean raiseError = false;

	private boolean endActivity=false;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------
	/**
	 * Terminates the project
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void terminate(String projectName)
	throws EngineException, HeroException {

		log.debug("Start terminate the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome pHome;
		BnProjectLocal pLocal;
		ProjectSessionLocalHome psHome;
		ProjectSessionLocal mPSession;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			pLocal = pHome.findByName(projectName);
			mPSession.initProject(projectName);
			if (pLocal.getType().equals(Constants.Pj.INSTANCE))
			{
				if (pLocal.getStatus().equals(Constants.Pj.HIDDEN))
					throw new EngineException("Project is hidden. You cannot terminate an instance of a hidden model !!");
			}
			else if(pLocal.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot terminate a hidden project !!");
			if (this.endActivity || nodesTerminated(pLocal.getBnNodes())) {
				pLocal.setState(hero.interfaces.Constants.Pj.TERMINATED);
				pLocal.setEndDate(new java.util.Date());
			} else {
				throw new EngineException("Project cannot be terminated. Some nodes are still active");
			}
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}

		log.debug(logProjectProperties("Properties after terminated: ", mPSession)+ ", project = " + projectName+" started by " + mContext.getCallerPrincipal().getName());
		log.debug("End terminate the project = " + projectName+" started by " + mContext.getCallerPrincipal().getName());
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void startActivity(String projectName, String nodeName)
	throws EngineException, HeroException {

		log.debug("Start startActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getType().equals(Constants.Pj.INSTANCE))
			{
				if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
					throw new EngineException("Project is hidden. You cannot start an activity of a hidden model !!");
			}
			else if(mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot start an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			/*if (!mPSession.isUserInNodeRole(nodeName) && !mPSession.isAdminOfProject(projectName))
				throw new HeroException("You are not allowed to start this activity ! ");*/
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		/*try {
			if (mProject.getType().equals(Constants.Pj.COOPERATIVE))
				mProject.setModel(BonitaProjectLocator.generateProjectValue(mProject.getName()));
		} catch (BonitaServiceException e) {e.printStackTrace();
		throw new HeroException(e.getMessage());
		}*/

		log.debug(logActivityProperties(nodeName, "Properties before start", mPSession)+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
	//	if ( (nd.getActivityPerformer() != null  && ( nd.getActivityPerformer()).equals(mContext.getCallerPrincipal().getName())) || ! (nd.getActivityPerformer() != null))
		//{
			execute(nodeName, hero.interfaces.Constants.Nd.START, mProject ,mPSession);
			if (mProject.getState()== hero.interfaces.Constants.Pj.INITIAL)
				mProject.setState(hero.interfaces.Constants.Pj.STARTED);
			if (raiseError == true)
			{
				raiseError = false;
				throw new HeroHookException("An error is produced during the execution of a BeforeStart hook...");
			}
		//}

		log.debug("End of start activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());

		log.debug(logActivityProperties(nodeName, "Properties after start", mPSession)+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public boolean terminateActivity(String projectName, String nodeName)
	throws EngineException, HeroException {

		log.debug("Start terminateActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		this.endActivity=false;
		boolean terminated=false;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getType().equals(Constants.Pj.INSTANCE))
			{
				if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
					throw new EngineException("Project is hidden. You cannot terminate an activity of a hidden model !!");
			}
			else if(mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot terminate an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		if (mProject.getBnNode(nodeName).getType()==Constants.Nd.SUB_PROCESS_NODE)
			mPSession.propagatesSubProcessProperties(mProject.getBnNode(nodeName));

		log.debug(logActivityProperties(nodeName, "Properties before terminate", mPSession)+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		execute(nodeName, hero.interfaces.Constants.Nd.TERMINATE, mProject, mPSession);
		if (raiseError == true)
		{
			raiseError = false;
			throw new HeroHookException("An error is produced during the execution of a hook...");
		}

		log.debug("End of terminate activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());

		log.debug(logActivityProperties(nodeName, "Properties after terminate", mPSession)+ ", activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		// Automatically terminates subProcess activity
		if (!mProject.getParent().equals(mProject.getName()))
		{
			if (mPSession.isTerminated())
			{
				try{
					if (mPSession != null)
						mPSession.remove();
				}catch(Exception re){}

				try{
					if (this.terminateActivity(BonitaParsing.getParentInstanceName(mProject.getParent()),BonitaParsing.getParentNodeName(mProject.getParent())))
					{
						UserSessionLocalHome ulHome = UserSessionUtil.getLocalHome();
						UserSessionLocal ul =ulHome.create();
						ul.terminate(BonitaParsing.getParentInstanceName(mProject.getParent()));
						ul.remove();
					}

				}catch(Exception ri){throw new HeroException("Error when deleting the instance= " + ri.getMessage());}

			}
			return terminated;
		}
		else{
			try{
				if (mPSession != null)
				{
					// test if the instance is termined
			 		if (mPSession.isTerminated() && mProject.getType().equals(Constants.Pj.INSTANCE))
					{
						mProject.setState(Constants.Pj.TERMINATED);
						this.terminate(projectName);
						terminated=true;
					}
					mPSession.remove();
				}
			}catch(Exception re){re.printStackTrace();}
		}
		if (this.endActivity)
		{
			this.terminate(projectName);
			this.endActivity=false;
			terminated=true;
		}
		return terminated;
	}

	/**
	 * Starts the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void cancelActivity(String projectName, String nodeName)
	throws EngineException, HeroException {

		log.debug("Start cancelActivity = " + nodeName +" of the project = " + projectName + " started by " + mContext.getCallerPrincipal().getName());
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			if (mProject.getType().equals(Constants.Pj.INSTANCE))
			{
				if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
					throw new EngineException("Project is hidden. You cannot cancel an activity of a hidden model !!");
			}
			else if(mProject.getStatus().equals(Constants.Pj.HIDDEN))
				throw new EngineException("Project is hidden. You cannot cancel an activity of a hidden project !!");
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			//this.getProjectModel();
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.CANCEL, mProject, mPSession);

		log.debug("End of cancel activity = " + nodeName + "of project = " +projectName+" started by " + mContext.getCallerPrincipal().getName());
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	/**
	 * Suspend the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void suspendActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		//log.debug("suspendActivity: nodeName=" + nodeName);
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.SUSPEND, mProject, mPSession);
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	/**
	 * Non_Automatic operation method that resume specify activity
	 */

	/**
	 * Resume the activity nodeName
	 * @param nodeName the name of the activity
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void resumeActivity(String projectName, String nodeName)
	throws EngineException, HeroException {
		//log.debug("resumeActivity: nodeName=" + nodeName);
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		execute(nodeName, hero.interfaces.Constants.Nd.RESUME, mProject, mPSession);
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	/**
	 * Active an External Agent
	 * @param agentName the name of the external agent
	 * @ejb:interface-method view-type="both"
	 *
	 **/
	public void activeAgent(String projectName, String agentName)
	throws EngineException, HeroException {
		//log.debug("activeAgent: agentName=" + agentName);
		BnAgentLocal ag;
		ProjectSessionLocalHome psHome;
		BnProjectLocalHome pHome;
		ProjectSessionLocal mPSession;
		BnProjectLocal mProject;
		try {
			psHome = ProjectSessionUtil.getLocalHome();
			pHome = BnProjectUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			mProject = pHome.findByName(projectName);
			mPSession = psHome.create();
			mPSession.setCaller(mContext.getCallerPrincipal().getName());
			mPSession.initProject(projectName);
			ag = mProject.getBnAgent(agentName);
			mPSession.setAgentState(ag, hero.interfaces.Constants.Ag.ACTIVE);
			Collection edges = ag.getOutEdges();
			if (!edges.isEmpty())
				modifyAgentEdges(edges, mProject,mPSession);
		} catch (FinderException fe) {
			throw new HeroException("Project does not exist " + projectName);
		} catch (CreateException he) {
			throw new HeroException("ProjectSession Error" + projectName);
		}
		try{
			if (mPSession != null)
				mPSession.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	// Utility method
	private void execute(String _node, int operation, BnProjectLocal mProject, ProjectSessionLocal mPSession) throws HeroException, EngineException {
		boolean statePropagation = true;

		//log.debug("execute: node="+ _node+" operation="+ hero.interfaces.Constants.Nd.operationName[operation]);

		BnNodeLocal nd;
		nd = mProject.getBnNode(_node);
		NodeState nodeState = NodeState.make(nd.getType(), nd.getAnticipable());
		int newState = nodeState.computeState(nd, operation);

		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new EngineException("Bad state transition " + _node);
		}

		if (newState != nd.getState()) {

			Collection edges = nd.getOutBnEdges();
			if (edges.isEmpty() && isInstance(mProject.getName()) && newState != Constants.Nd.INITIAL){
				if (!instanceWasModified(nd, mProject)) {
					this.generateOutNodes(nd, mProject, mPSession);
					if (newState == Constants.Nd.DEAD)
						mPSession.propagateNodeProperties(_node);
					edges = nd.getOutBnEdges();
				}
			}

			if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
				if (jms!=null)
					jms.sendProjectEvent(EventConstants.TERMINATED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORETERMINATE, mProject);
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.AFTERTERMINATE, mProject);
				mPSession.propagateNodeProperties(_node);
			} else if (newState == hero.interfaces.Constants.Nd.EXECUTING) {
				if (nd.getType()== hero.interfaces.Constants.Nd.SUB_PROCESS_NODE)
					mPSession.propagatesParentProperties(_node);
				if (jms!=null)
					jms.sendProjectEvent(EventConstants.START,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,	hero.interfaces.Constants.Nd.BEFORESTART, mProject);
			} else if (newState == hero.interfaces.Constants.Nd.ANTICIPATING) {
				if (jms!=null)
					jms.sendProjectEvent(EventConstants.ANTICIPATING,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node,hero.interfaces.Constants.Nd.ANTICIPATE, mProject);
			} else if (newState == hero.interfaces.Constants.Nd.DEAD) {
				mPSession.propagateNodeProperties(_node);
				if (jms!=null)
					jms.sendProjectEvent(EventConstants.CANCELED,mProject.getName(),_node,mContext.getCallerPrincipal().getName());
				this.hookExecute(_node, hero.interfaces.Constants.Nd.ONCANCEL, mProject);
			}

			if (activityStarted(newState)) {
				this.hookExecute(_node,hero.interfaces.Constants.Nd.AFTERSTART, mProject);
			}

			// ---------- Activities iteration ----------
			if ((newState == hero.interfaces.Constants.Nd.TERMINATED) && (mPSession.getIterationExist(nd.getName()))) {
				Collection nodeIterations = mPSession.getIterations(_node);
				Iterator iterationsIt = nodeIterations.iterator();
				BnIterationLocal iteration;
				String iterationCondition;

				// Search the first condition of the iterations that is true
				do {
					iteration = (BnIterationLocal) iterationsIt.next();
					iterationCondition = iteration.getCondition();
				} while ((iterationsIt.hasNext()) && ((evaluateCondition(iterationCondition, nd)) == Constants.CONDITION_FALSE));

				if ((evaluateCondition(iterationCondition, nd)) == Constants.CONDITION_TRUE) {
					// No state propagation thru transitions if an iteration starts
					statePropagation = false;

					BnNodeLocal toNode = mProject.getBnNode(iteration.getToNode());

					this.setIterationPathInitialModel(nd, toNode, mProject, mPSession);
					mPSession.setNodeState(toNode,hero.interfaces.Constants.Nd.READY);

					if ( toNode.getBnNodePerformerAssign() != null) {
						this.performAssign(toNode);
					}

					if ((toNode.getType() == Constants.Nd.AND_JOIN_NODE) || (toNode.getType() == Constants.Nd.OR_JOIN_NODE)) {
						this.hookExecute(toNode.getName(), hero.interfaces.Constants.Nd.ONREADY, mProject);
					}

					if ((toNode.getType() == Constants.Nd.AND_JOIN_AUTOMATIC_NODE)
							|| (toNode.getType() == Constants.Nd.OR_JOIN_AUTOMATIC_NODE)
							|| (toNode.getType() == Constants.Nd.SUB_PROCESS_NODE)) {
						this.execute(toNode.getName(), Constants.Nd.START, mProject, mPSession);
					}

				} else {
					mPSession.setNodeState(nd, newState);
				}
			} else {  // ---------- End activities iteration -----------
				mPSession.setNodeState(nd, newState);
				mPSession.setNodeExecutor(_node);
			}

			try {
				if (newState == hero.interfaces.Constants.Nd.READY && nd.getBnNodePerformerAssign() != null)
					this.performAssign(nd);
				if (newState==hero.interfaces.Constants.Nd.READY)
					this.hookExecute(_node,	hero.interfaces.Constants.Nd.ONREADY, mProject);
			} catch (Exception e){
				e.printStackTrace();
				throw new HeroException("Error when calling the performAssign(): "+e.getMessage());
			}

			if (statePropagation) {
				// Propagate the new state to the connected nodes. Subsequence automatic activities will be automatically
				//executed by the Engine
				edges = nd.getOutBnEdges();
				if (!instanceWasModified(nd, mProject))
				{
					if (newState == Constants.Nd.DEAD)
						mPSession.propagateNodeProperties(_node);
					edges = nd.getOutBnEdges();
				}
				modifyNodeEdges(newState, edges, mProject, mPSession);
			}
		}

		if (newState == hero.interfaces.Constants.Nd.TERMINATED) {
			if (nd.getName().equals("BonitaEnd")) {
				this.endActivity=true;
			}
		}

		if ((newState == hero.interfaces.Constants.Nd.EXECUTING) && (nd.getType() == hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE)) {
			this.execute(_node, hero.interfaces.Constants.Nd.TERMINATE, mProject, mPSession);
		}
	}

	private void performAssign(BnNodeLocal nd) throws  HeroException {
		try {
			BnNodePerformerAssignLocal performerAssign = null;
			String creator = mContext.getCallerPrincipal().getName();
			performerAssign = nd.getBnNodePerformerAssign();
			hero.performerAssign.PerformerAssign h = hero.performerAssign.PerformerAssign.make(performerAssign.getName(), performerAssign.getType());
			h.execute(this, performerAssign.getType(), nd, creator);

		} catch (HeroException he) {
			he.printStackTrace();
			throw new HeroException("Error in performerAssignment. " + he.getMessage());
		}
	}

	/**
	 * Method that consults new node state and then propagates the state
	 * through its out edges. Supposes that the model is defined correctly
	 *
	 * @param nodeState
	 * @param edges
	 * @param mPSession
	 * @throws HeroException
	 * @throws EngineException
	 */
	private void modifyNodeEdges(int nodeState, Collection edges, BnProjectLocal mProject, ProjectSessionLocal mPSession)
	throws HeroException, EngineException {
		int newEdgeState;
		Iterator i = edges.iterator();

		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((nodeState != hero.interfaces.Constants.Nd.TERMINATED) && (nodeState != hero.interfaces.Constants.Nd.DEAD)) {
				newEdgeState =hero.interfaces.Constants.edgeTransition[Constants.CONDITION_TRUE][nodeState];

				if (edge.getState() != newEdgeState) {
					mPSession.setEdgeState(edge, newEdgeState);
					edgeChanged(edge, mProject, mPSession);
				}
			} else {
				newEdgeState =hero.interfaces.Constants.edgeTransition[evaluateCondition(edge.getCondition(),edge.getInBnNode())][nodeState];

				if (edge.getState() != newEdgeState) {
					mPSession.setEdgeState(edge, newEdgeState);

					edgeChanged(edge, mProject, mPSession);
				}
			}
		}
	}

	/**
	 * Gets, in a recursive way, the nodes inside the iteration<br>
	 * Extra entry points are no returned (because they are not inside the iteration)<br>
	 * It works with model's defined objects because may be, some of them, have not been declared yet<br>
	 *
	 * @param nodesInIterationModel initially it is an empty list and, at the end, contains the nodes inside the iteration
	 * @param currentModelNode
	 * @param toIterationModelNode
	 * @return true if current node is inside the iteration and false otherwise
	 * @throws HeroException
	 */
	private boolean getNodesInsideIterationModel(ArrayList nodesInIterationModel, BonitaNodeValue currentModelNode, BonitaNodeValue toIterationModelNode, BonitaProjectValue model) throws HeroException
	{
		boolean pathExists = false;

		if (currentModelNode.equals(toIterationModelNode)) {
			nodesInIterationModel.add(currentModelNode);
			return true;
		} else {
			Collection inEdges = currentModelNode.getInEdges();
			Iterator it = inEdges.iterator();
			while (it.hasNext()) {
				BonitaEdgeValue nextEdge = (BonitaEdgeValue) it.next();
				BonitaNodeValue nextNode = BonitaServiceValue.getNodeFromCache(model, nextEdge.getInNode());
				if (nodesInIterationModel.contains(nextNode)) {
					pathExists = true;
				} else {
					if (getNodesInsideIterationModel(nodesInIterationModel, nextNode, toIterationModelNode, model)) {
						pathExists = true;
					}
				}
			}
		}
		if (pathExists) {
			if (!nodesInIterationModel.contains(currentModelNode)) {
				nodesInIterationModel.add(currentModelNode);
			}
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Reinitializes 'instantiated' activities inside an iteration and their output edges as well<br>
	 *
	 * @param fromIterationNode
	 * @param toIterationNode
	 * @param mPSession
	 * @throws HeroException
	 */
	private void setIterationPathInitialModel(BnNodeLocal fromIterationNode, BnNodeLocal toIterationNode, BnProjectLocal mProject, ProjectSessionLocal mPSession) throws HeroException
	{
		ArrayList nodeListModel = new ArrayList();
		ArrayList nodeListInstance = new ArrayList();
		boolean correct;
		String fromIterationNodeModelName;
		String toIterationNodeModelName;
		BonitaProjectValue model;

		try {
			model = BonitaProjectLocator.getInstance().getModelValue(BonitaParsing.getModel(mProject.getName()), BonitaParsing.getModel(mProject.getVersion()));
		} catch (BonitaServiceException e) {
			e.printStackTrace();
			throw new HeroException(e.getMessage());
		}
		// Get model names. (toIteration could not be instantiated yet -> work with model)
		if (this.isInstance(fromIterationNode.getName())) {
			fromIterationNodeModelName = BonitaParsing.getModel(fromIterationNode.getName());
		} else {
			fromIterationNodeModelName = fromIterationNode.getName();
		}
		if (this.isInstance(toIterationNode.getName())) {
			toIterationNodeModelName = BonitaParsing.getModel(toIterationNode.getName());
		} else {
			toIterationNodeModelName = toIterationNode.getName();
		}
		BonitaNodeValue fromIterationNodeModel = BonitaServiceValue.getNodeFromCache(model, fromIterationNodeModelName);
		BonitaNodeValue toIterationNodeModel = BonitaServiceValue.getNodeFromCache(model, toIterationNodeModelName);
		correct = this.getNodesInsideIterationModel(nodeListModel, fromIterationNodeModel, toIterationNodeModel, model);
		if (!correct) {
			throw new HeroException("Doesn't exist a path in iteration : " + fromIterationNode.getName() + "--->" + toIterationNode.getName());
		}

		// Chek which nodes exist in the current mPSession
		// Firstly we set everything inside the iteration to INITIAL (nodes & edges)
		Iterator it = nodeListModel.iterator();
		while (it.hasNext()) {

			BonitaNodeValue modelNode = (BonitaNodeValue) it.next();
			//String instanceNodeName = this.nodeConvertion(modelNode.getName(), mProject);

			String instanceNodeName = modelNode.getName();
			String version = this.nodeSubProcess(instanceNodeName, mProject);

			if ((modelNode.getType() == Nd.SUB_PROCESS_NODE) && (!version.equals(instanceNodeName)))
				this.initSubProcess(version);

			if ((modelNode.getType() == Nd.SUB_PROCESS_NODE) && (!version.equals(instanceNodeName)) || (this.nodeExist(instanceNodeName, mPSession))) {
				BnNodeLocal nodeLocal = mProject.getBnNode(instanceNodeName);
				mPSession.setNodeState(nodeLocal, Constants.Nd.INITIAL);
				Collection outEdges = nodeLocal.getOutBnEdges();
				Iterator it2 = outEdges.iterator();
				while (it2.hasNext()) {
					BnEdgeLocal outEdge = (BnEdgeLocal) it2.next();
					mPSession.setEdgeState(outEdge, Constants.Ed.INITIAL);
					BnNodeLocal outNode = outEdge.getOutBnNode();
					if ((outNode.getState() == Constants.Nd.DEAD) || (outNode.getState() == Constants.Nd.ANTICIPABLE) || (outNode.getState() == Constants.Nd.ANTICIPATING)) {
						this.propagateInitialStateThruNodes(outNode, mPSession);
					}
				}
				nodeListInstance.add(nodeLocal);
			}
		}

		// After that, it is necessary to consider if there are another extra entry points in the iteration that changes some node's state
		// This in done later because it needs be sure that any edge inside the iteration is ACTIVE
		it = nodeListInstance.iterator();
		int operation;
		while (it.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) it.next();

			// Computes next operation over the node
			EdgeState edgeState = EdgeState.make(node.getType());
			operation = edgeState.state(node);

			// If node type is: AND_JOIN or AND_JOIN_AUTOMATIC evaluates their in edge states to see if someone is DEAD -> executes a CANCEL over the node
			if ((node.getType() == Constants.Nd.AND_JOIN_NODE) || (node.getType() == Constants.Nd.AND_JOIN_AUTOMATIC_NODE)) {
				if (operation == Constants.Nd.CANCEL) {
					this.execute(node.getName(), Constants.Nd.CANCEL, mProject, mPSession);
				}
			}

			// If node type is: OR_JOIN or OR_JOIN_AUTOMATIC evaluates their in edge states to see if someone is ACTIVE -> node is set to READY
			else if ((node.getType() == Constants.Nd.OR_JOIN_NODE) || (node.getType() == Constants.Nd.OR_JOIN_AUTOMATIC_NODE)) {
				if (operation == Constants.Nd.ACTIVE) {
					mPSession.setNodeState(node, Constants.Nd.READY);

					if (node.getType() == Constants.Nd.OR_JOIN_NODE) {
						this.hookExecute(node.getName(), hero.interfaces.Constants.Nd.ONREADY, mProject);
					} else { // It is an OR_JOIN_AUTOMATIC then it executes a START operation
						this.execute(node.getName(), Constants.Nd.START, mProject, mPSession);
					}
				}
			}
		}
	}

	private void propagateInitialStateThruNodes(BnNodeLocal node, ProjectSessionLocal mPSession) throws HeroException {
		mPSession.setNodeState(node, Constants.Nd.INITIAL);
		Collection outEdges = node.getOutBnEdges();
		Iterator it = outEdges.iterator();
		while (it.hasNext()) {
			BnEdgeLocal outEdge = (BnEdgeLocal) it.next();
			mPSession.setEdgeState(outEdge, Constants.Ed.INITIAL);
			BnNodeLocal outNode = outEdge.getOutBnNode();
			if ((outNode.getState() == Constants.Nd.DEAD) || (outNode.getState() == Constants.Nd.ANTICIPABLE) || (outNode.getState() == Constants.Nd.ANTICIPATING)) {
				this.propagateInitialStateThruNodes(outNode, mPSession);
			}
		}
	}

	/**
	 * Method that propagate the AgentEdges state
	 */
	private void modifyAgentEdges(Collection edges, BnProjectLocal mProject, ProjectSessionLocal mPSession)
	throws HeroException, EngineException {
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			mPSession.setAgentEdgeState(edge,hero.interfaces.Constants.Ed.ANTICIPATING);
			BnNodeLocal nd = edge.getOutBnNode();
			EdgeState edgeState = EdgeState.make(nd.getType());
			int operation = edgeState.state(nd);
			execute(nd.getName(), hero.interfaces.Constants.Nd.ANTACTIVE, mProject, mPSession);
		}
	}

	private void edgeChanged(BnEdgeLocal edge, BnProjectLocal mProject, ProjectSessionLocal mPSession)
	throws HeroException, EngineException {
		BnNodeLocal nd = edge.getOutBnNode();
		if (nd.getState() != hero.interfaces.Constants.Nd.DEAD) {
			EdgeState edgeState = EdgeState.make(nd.getType());
			int operation = edgeState.state(nd);
			execute(nd.getName(), operation, mProject,mPSession);
		}
	}

	private boolean activityStarted(int state) throws HeroException {
		return (
				state == hero.interfaces.Constants.Nd.ANTICIPATING
				|| state == hero.interfaces.Constants.Nd.EXECUTING);
	}

	private void hookExecute(String nodeName, String eventName, BnProjectLocal mProject)
	throws HeroException, HeroHookException {
		Collection hooks;
		Collection interHooks;
		BnNodeLocal node;
		try {
			node = mProject.getBnNode(nodeName);
			BonitaProjectValue model = BonitaProjectLocator.getInstance().getModelValue(BonitaParsing.getModel(mProject.getName()), BonitaParsing.getModel(mProject.getVersion()));

			//if (nodeName.matches(".*_instance.*"))
				//nodeName=nodeName.substring(0,nodeName.indexOf("_instance"));

			BonitaNodeValue nodeLocal;
			if (mProject.getType().equals(Constants.Pj.COOPERATIVE))
				nodeLocal = BonitaServiceValue.getNode(mProject,nodeName);
			else
				nodeLocal = BonitaServiceValue.getNodeFromCache(model,nodeName);

			hooks = nodeLocal.getHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BonitaHookValue hk = (BonitaHookValue) i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType());
					log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}

			interHooks = nodeLocal.getInterHooks();
			for (Iterator i = interHooks.iterator(); i.hasNext();) {
				BonitaInterHookValue hk = (BonitaInterHookValue)i.next();
				if (hk.getEvent().equalsIgnoreCase(eventName)) {
					hero.hook.Hook h = hero.hook.Hook.make(hk.getName(), eventName, hk.getType(), hk.getScript());
					log.debug("hookExecute, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName());
					h.execute(this, eventName, node);
				}
			}
		}catch(HeroHookException hh){
			log.error("hookExecute ERROR, eventName = " + eventName+" nodeName = "+nodeName+" projectName = "+mProject.getName()+" Error message: "+hh.getMessage());
			if (eventName.equals(hero.interfaces.Constants.Nd.AFTERSTART) || eventName.equals(hero.interfaces.Constants.Nd.BEFORETERMINATE)
					|| eventName.equals(hero.interfaces.Constants.Nd.ANTICIPATE) || eventName.equals(hero.interfaces.Constants.Nd.ONCANCEL)) {
				mContext.setRollbackOnly();
				throw new HeroHookException("Cannot execute hook "+hh.getMessage());
			}
			else
				raiseError = true;
		}catch (Exception e){e.printStackTrace();}
	}

	private boolean nodesTerminated(Collection nodes) {
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) i.next();
			int state = node.getState();
			if (state != hero.interfaces.Constants.Nd.TERMINATED
					&& state != hero.interfaces.Constants.Nd.EXPIRED
					&& state != hero.interfaces.Constants.Nd.DEAD)
				return false;
		}
		return true;
	}

	public int evaluateCondition(String condition, BnNodeLocal node) {
		try {
			if (condition != null) {
				condition = "import hero.interfaces.BnNodeLocal;\n"
					+ "boolean evalCondition (Object node) {\n\n\n"
					+ "return("
					+ condition
					+ ");"
					+ "}";
				Interpreter i = new Interpreter();

				// BnNode properties mapping

				Collection prop = node.getBnProperties();
				for (Iterator props = prop.iterator(); props.hasNext();) {
					BnNodePropertyLocal pl = (BnNodePropertyLocal) props.next();
					i.set(pl.getTheKey(), pl.getTheValue());
				}

				//Project properties mapping
				BnProjectLocal project = node.getBnProject();
				Collection projectProp = project.getBnProperties();
				for (Iterator props=projectProp.iterator();props.hasNext();) {
					BnProjectPropertyLocal pP=(BnProjectPropertyLocal)props.next();
					i.set(pP.getTheKey(),pP.getTheValue());
				}
				i.set("node",node);
				i.eval(condition);
				i.eval("result=evalCondition(node)");
				if (((Boolean) i.get("result")).booleanValue() == true)
					return (Constants.CONDITION_TRUE);
				else
					return (Constants.CONDITION_FALSE);
			} else
				return (Constants.CONDITION_TRUE);
		} catch (Exception t) {
			t.printStackTrace();
			return (Constants.CONDITION_FALSE);
		}
	}

	private String logActivityProperties(String nodeName, String prelog, ProjectSessionLocal mPSession)
	throws HeroException {
		String thelog = prelog;
		synchronized(mPSession){
			Collection np = mPSession.getNodeProperties(nodeName);
			Iterator i = np.iterator();
			if (np.size() != 0) {
				while (i.hasNext()) {
					thelog = thelog + " key= '";
					BnNodePropertyValue property = (BnNodePropertyValue) i.next();
					thelog = thelog + property.getTheKey();
					thelog = thelog + "' value = '" + property.getTheValue() + "'";
				}
			} else
				thelog = prelog + " No property";
		}
		return thelog;
	}
	private String logProjectProperties(String prelog, ProjectSessionLocal mPSession) throws HeroException {
		String thelog = prelog;
		synchronized(mPSession){
			Collection np = mPSession.getProperties();
			Iterator i = np.iterator();
			if (np.size() != 0) {
				while (i.hasNext()) {
					thelog = thelog + " key= '";
					BnProjectPropertyValue property =
						(BnProjectPropertyValue) i.next();
					thelog = thelog + property.getTheKey();
					thelog = thelog + "' value = '" + property.getTheValue() + "'";
				}
			} else
				thelog = prelog + " No property";
		}
		return thelog;
	}

	// Init activities of the subProcess project
	private void initSubProcess(String subProcess) throws HeroException {
		ProjectSessionLocalHome psh;
		ProjectSessionLocal psj;
		BnProjectLocalHome ph;
		BnProjectLocal pj;
		try{
			psh = ProjectSessionUtil.getLocalHome();
			psj = psh.create();
			psj.initProject(subProcess);

			ph = BnProjectUtil.getLocalHome();
			pj = ph.findByName(subProcess);
			Collection nodes = pj.getBnNodes();
			Iterator nds = nodes.iterator();
			while (nds.hasNext())
			{
				BnNodeLocal node = (BnNodeLocal)nds.next();
				if (node.getType() == Nd.SUB_PROCESS_NODE)
					this.initSubProcess(node.getReference());
				psj.setNodeState(node,hero.interfaces.Constants.Nd.INITIAL);

				Collection outEdges = node.getOutBnEdges();
				Iterator it2 = outEdges.iterator();
				while (it2.hasNext()) {
					BnEdgeLocal edge = (BnEdgeLocal) it2.next();
					psj.setEdgeState(edge, Constants.Ed.INITIAL);
				}
			}
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
		try{
			psj.remove();
		}catch(Exception re){throw new HeroException(re.getMessage());}
	}

	private void generateOutNodes(BnNodeLocal nd, BnProjectLocal mProject, ProjectSessionLocal mPSession) throws HeroException {
		// Searches the current node in the model to know their out edges and nodes
		BonitaNodeValue parentNode;
		String instName=null;

		BonitaProjectValue model;
		// Getting the model data from the cache, the version number is passed as a parameter
		try {
			model = BonitaProjectLocator.getInstance().getModelValue(BonitaParsing.getModel(mProject.getName()), mProject.getVersion());
		} catch (BonitaServiceException e1) {
			e1.printStackTrace();
			throw new HeroException(e1.getMessage());
		}

		// Getting the node Value Object from cache
		parentNode = BonitaServiceValue.getNodeFromCache(model,nd.getName());

        // Outgoing nodes creation
		Collection outEdges = parentNode.getOutEdges();
		Iterator edges = outEdges.iterator();
		while (edges.hasNext()) {
			BonitaEdgeValue edge = (BonitaEdgeValue) edges.next();
			BonitaNodeValue node = BonitaServiceValue.getNodeFromCache(model,edge.getOutNode());
			String nodeName = node.getName();

			if (node.getType() == Nd.SUB_PROCESS_NODE && mProject.getType().equals(Constants.Pj.INSTANCE)) // Node is a subprocess
			{
				String subProcessName=BonitaParsing.getSubProcessName(node.getReference());
				String subProcessVersion=BonitaParsing.getSubProcessVersion(node.getReference());
				try {
					// Creates an instance of the subProcess pointed by the node to be created
					BnProjectLocalHome opHome = BnProjectUtil.getLocalHome();
					instName = mPSession.instantiateProject(subProcessName, subProcessVersion);
					this.initSubProcess(instName);

					BnProjectLocal newInstance = opHome.findByName(instName);
					// The new instance refers to the instance in which the node subProcess is going to be created
					// The format of the parent column is InstanceName+"_node-"+nodeName
					newInstance.setParent(nd.getBnProject().getName()+"_node-"+nodeName);
					mPSession.initProject(nd.getBnProject().getName());

				} catch (Exception e) {
					throw new HeroException(e.getMessage());
				}
			}
			if (!this.nodeExist(nodeName, mPSession))
			{
				mPSession.copyNode(nodeName, node);
                // Copy Node Properties (default value)
                Collection nodeProp = node.getProperties();
                Iterator nodeProps = nodeProp.iterator();
                while (nodeProps.hasNext())
                {
                    BonitaPropertyValue ndProp = (BonitaPropertyValue)nodeProps.next();
                    mPSession.setNodePropertyPossibleValues(nodeName,ndProp.getKey(),ndProp.getPossibleValues());
                    mPSession.setNodeProperty(nodeName,ndProp.getKey(),ndProp.getValue(),ndProp.getPropagate());
                }
				if (node.getType() == Nd.SUB_PROCESS_NODE)
				{
					BnNodeLocal newNode = mProject.getBnNode(nodeName);
					if (!newNode.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
						newNode.setReference(instName);
				}
			}

			mPSession.copyEdge(nodeName,nd, edge);
		}

		// Iterations between nodes
		Collection modelIterations = model.getIterations();
		Iterator iter = modelIterations.iterator();
		while (iter.hasNext()) {
			BonitaIterationValue modelIteration = (BonitaIterationValue) iter.next();
			String modelFromNodeName = modelIteration.getFromNode();

			String instanceFromNodeName = modelFromNodeName;
			if (instanceFromNodeName.equals(nd.getName())) {
				BonitaNodeValue modelToNode = BonitaServiceValue.getNodeFromCache(model, modelIteration.getToNode());
				String instanceToNodeName = modelToNode.getName();
					// Adds toNode to the mPSession if it has not been defined yet and it is not a SUB_PROCESS
				    if (!this.nodeExist(instanceToNodeName, mPSession)) {
				    	if ((modelToNode.getType() == Nd.SUB_PROCESS_NODE)) {
				    		try{
							String subProcessName=BonitaParsing.getSubProcessName(modelToNode.getReference());
							String subProcessVersion=BonitaParsing.getSubProcessVersion(modelToNode.getReference());

							BnProjectLocalHome opHome = BnProjectUtil.getLocalHome();
							instName = mPSession.instantiateProject(subProcessName, subProcessVersion);
							this.initSubProcess(instName);
							BnProjectLocal newInstance = opHome.findByName(instName);
							newInstance.setParent(nd.getBnProject().getName()+"_node-"+modelToNode);
							mPSession.initProject(nd.getBnProject().getName());
				    	} catch (Exception e) {
							throw new HeroException(e.getMessage());
						}
						}
						mPSession.copyNode(instanceToNodeName, modelToNode);
					}
					try {
						mPSession.copyIteration(instanceFromNodeName, instanceToNodeName, modelIteration.getCondition());
					} catch (Exception it) {
						//throw new HeroException("Error in generateOutNodes(): copyIteration() fail\n   Message: " + it.getMessage());
					} // maybe the iteration was already created
				}
		}
	}

	private boolean isInstance(String name) {
		return (name.matches(".*_instance.*"));
	}

	private String nodeSubProcess(String name, BnProjectLocal mProject) {
		Collection nodes = mProject.getBnNodes();
		Iterator nds = nodes.iterator();
		while (nds.hasNext())
		{
			BnNodeLocal node = (BnNodeLocal)nds.next();
			if ((node.getName()).matches(name))
				return node.getReference();
		}
		return name;
	}

	private boolean nodeExist(String node,ProjectSessionLocal mPSession) throws HeroException{
		return(mPSession.getNodesNames().contains(node));
	}

	private boolean instanceWasModified(BnNodeLocal node, BnProjectLocal mProject) throws HeroException{
		try{
		BonitaProjectValue model = BonitaProjectLocator.getInstance().getModelValue(BonitaParsing.getModel(mProject.getName()), BonitaParsing.getModel(mProject.getVersion()));
		String name = node.getName();
		if (this.isInstance(name))
			name = BonitaParsing.getModel(node.getName());
		if (BonitaServiceValue.getNodeFromCache(model,name)==null)
			return true;
		return false;
		}catch(Exception e){e.printStackTrace();throw new HeroException(e.getMessage());}
	}

	/**
	 *
	 * @throws CreateException
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
		try{
			BonitaConfig bonitaConfig = BonitaConfig.getInstance();
			if (bonitaConfig.getJms())
			{
				JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
				jms=home.create();
			}
		}catch(Exception e){throw new CreateException(e.getMessage());}
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

}
