package hero.session;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import hero.interfaces.*;
import hero.util.HeroException;


/**
 * Encapsulates the retrival of DB data
 *
 * @author Andreas Schaefer
 * @version $Revision: 1.5 $
 *
 * @ejb:bean name="InsertAuthRoleUser"
 *           display-name="Makes inserts in the AuthRole_User table"
 *           type="Stateless"
 *           jndi-name="ejb/hero/InsertAuthRoleUser"
 *           local-jndi-name="ejb/hero/InsertAuthRoleUser_L"
 *
 *
 * @ejb:transaction-type type="Container"
 *
 * @jonas.bean
 *      ejb-name="InsertAuthRoleUser"
 *      jndi-name="ejb/hero/InsertAuthRoleUser"
 * @ejb.permission unchecked="yes" 
 * @ejb.security-identity run-as="SuperAdmin"
 *
 * @ejb.resource-ref res-ref-name="jdbc/myDS"
 *      res-type="javax.sql.DataSource"
 *      res-auth="Application"
 * @jonas.resource
 *      res-ref-name="jdbc/myDS"
 *      jndi-name="bonita"
 *      
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 */
public class InsertAuthRoleUserBean implements SessionBean
{
   private SessionContext mContext;

 /**
	* Makes inserts in User table
	*
	*
	* @throws RemoteException 
	*
	* @ejb:interface-method view-type="remote"
	* @ejb:transaction type="Required"
	**/
   public void initializeUser(String name,String password,String email) {
       
	Context lContext;
	BnUserLocalHome uhome = null;
	BnUserLocal user;	
	BnUserValue uv;

	try {
		uhome = hero.interfaces.BnUserUtil.getLocalHome();
		try {
			BnUserLocal us = uhome.findByName(name);
			throw new HeroException("BnUser " + name + " already exist in the system ");
		} catch (javax.ejb.FinderException nn) {
			user = uhome.create(new BnUserValue());
			uv=user.getBnUserValue();
			//uv.setId(id);
			uv.setName(name);
			uv.setPassword(password);
			uv.setEmail(email);
			uv.setCreationDate(null);
			uv.setModificationDate(null);
			uv.setJabber(null);
			user.setBnUserValue(uv);
		}
	} catch (java.lang.Exception e) {
		e.printStackTrace(System.out);   
	}

   }

 /**
	* Makes inserts in User table
	*
	*
	* @throws RemoteException 
	*
	* @ejb:interface-method view-type="remote"
	* @ejb:transaction type="Required"
	**/
   public void initializeAuthRole(String name,String rolegroup) {
       
	Context lContext;
	BnAuthRoleLocalHome arhome = null;
	BnAuthRoleLocal arole;	
	try {
		arhome = hero.interfaces.BnAuthRoleUtil.getLocalHome();
		try {
			BnAuthRoleLocal us = arhome.findByName(name);
			throw new HeroException("BnAuthRole " + name + " already exist in the system ");
		} catch (javax.ejb.FinderException nn) {
			BnAuthRoleValue arv = new BnAuthRoleValue();
			arv.setName(name);
			arv.setBnRoleGroup(rolegroup);
			arole = arhome.create(arv);			
		}
	} catch (java.lang.Exception e) {
		e.printStackTrace(System.out);   
	}

   }

   /**
	* Makes inserts in AuthRole_User table
	*
	* @throws RemoteException 
	*
	* @ejb:interface-method view-type="remote"
	* @ejb:transaction type="Required"
	**/
   public void initialize(String User,String AuthRole) {
       
	Context lContext;
	BnUserLocalHome uhome = null;
	BnAuthRoleLocalHome arhome = null;
	BnUserLocal user;
	BnAuthRoleLocal authRole;
	try {
		uhome = hero.interfaces.BnUserUtil.getLocalHome();
		arhome = hero.interfaces.BnAuthRoleUtil.getLocalHome();
		user=uhome.findByName(User);
		authRole=arhome.findByName(AuthRole);
        
        if (!authRole.getBnUsers().contains(user))
			authRole.getBnUsers().add(user);
		if (!user.getBnAuthRoles().contains(authRole))
			user.getBnAuthRoles().add(authRole);
	
	} catch (java.lang.Exception e) {
		e.printStackTrace(System.out);
	}

   }

   /**
	* Create the Session Bean
	*
	* @throws CreateException 
	*
	* @ejb:create-method view-type="remote"
	**/
   public void ejbCreate() throws CreateException {}


   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------
   
   public void setSessionContext( SessionContext aContext ) throws EJBException
   {
	  mContext = aContext;
   }
   
   public void ejbActivate()
	  throws EJBException
   {
   }
   
   public void ejbPassivate()
	  throws EJBException
   {
   }

   public void ejbRemove()
	  throws EJBException
   {
   }

}
