package hero.session;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: JMSServicesSessionBean.java,v 1.14 2005/10/17 15:40:34 brice Exp $
*
--------------------------------------------------------------------------
*/

import hero.util.EventConstants;
import hero.util.BonitaServiceException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import javax.jms.TopicConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TextMessage;
import javax.jms.JMSException;
import javax.jms.Session;

import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Enumeration;

import hero.util.BonitaServiceLocator;
import hero.util.HeroException;

/**
 * Session Bean Template
 *
 * @ejb:bean name="JMSServicesSession"
 *           display-name="JMSServices Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/JMSServicesSession"
 *           local-jndi-name="ejb/hero/JMSServicesSession_L"
 *
 * @ejb:transaction type="Required"
 * @ejb.permission unchecked="yes" 
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="JMSServicesSession"
 *      jndi-name="ejb/hero/JMSServicesSession"
 * 
 * @jboss.container-configuration name="Standard Stateful SessionBean for Bonita"
 **/

public class JMSServicesSessionBean implements SessionBean, EventConstants {

	private TopicConnectionFactory topicConnectionFactory = null;
	private Topic topic = null;

	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------  

	/**
	* Send messages, varying text slightly.
	*
	* @param evt Hashtable that includes the message
	* @param type Jms type
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void send(Hashtable evt, String type) throws Exception {
		try{
			
			TopicSession topicSession = null;
			TopicConnection topicConnection = null;
			TopicPublisher topicPublisher = null;
			TextMessage message = null;
			StringBuffer subs = new StringBuffer("");
			
			try {
				topicConnection = topicConnectionFactory.createTopicConnection();
				topicSession = topicConnection.createTopicSession(false,Session.AUTO_ACKNOWLEDGE);
			} catch (JMSException e) {
				System.out.println("Exception occurred: " + e.toString());
				throw new CreateException(e.getMessage());
			}
			
			try {
				topicPublisher = topicSession.createPublisher(topic);
				message = topicSession.createTextMessage();
				Enumeration keys = evt.keys();
				while (keys.hasMoreElements()) {
					String key = (String) keys.nextElement();
					String value = (String) evt.get(key);
					message.setStringProperty(key, value);
					subs.append(" " + key + " = '" + value + "' AND ");
					evt.put(key, value);
				}
				message.setJMSType(type);
				
				String res = subs.toString();
				String result = res.substring(0, res.length() - 4);
				message.setText(result);
				
				topicPublisher.publish(message);
				topicSession.close();
				topicConnection.close();
				
			} catch (JMSException e) {
				System.out.println("Exception occurred: " + e.toString());
				throw new Exception(e.getMessage());
			}		
		} catch (HeroException he){}// remove project consistency
	}

	/**
	* Send Node Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param nodeName The name of the node
	* @param type The type of the node
	* @param state The state of the node
	* @param userName The name of the user
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendNodeEvent(String event,String projectName,String nodeName,int type,int state,String userName)
	{
	try{		
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(NODENAME, nodeName);
		evt.put(NODETYPE, (new Integer(type)).toString());
		evt.put(NODESTATE, (new Integer(state)).toString());
		evt.put(USERNAME, userName);
		this.send(evt, NODE);
	}catch(Exception e){e.printStackTrace();}
	}
	/**
	* Send Edge Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param edgeName The name of the edge
	* @param nodeIn The name of the in node
	* @param nodeOut The name of the out node
	* @param userName The name of the user
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendEdgeEvent(String event,String projectName,String edgeName,String nodeIn,String nodeOut,String userName)
	{
	 try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(EDGENAME, edgeName);
		evt.put(NODEIN, nodeIn);
		evt.put(NODEOUT, nodeOut);
		evt.put(USERNAME, userName);
		this.send(evt, EDGE);
	  }catch(Exception e){e.printStackTrace();}
	}
	/**
	* Send Project Event.
	* 
	* @param event The name of the event
	* @param projectName The name of the project
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendProjectEvent(String event, String projectName)
	{
     try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		this.send(evt, PROJECT);
	  }catch(Exception e){e.printStackTrace();}
	}
	
	/**
	* Send Project Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param userName The name of the user
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/

	public void sendProjectEvent(String event,String projectName,String userName)
	{
	try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(USERNAME, userName);
		this.send(evt, PROJECT);
	}catch(Exception e){e.printStackTrace();}
	}
	/**
	* Send Project Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param nodeName The name of the node
	* @param userName The name of the user
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendProjectEvent(String event,String projectName,String nodeName,String userName)
	{
	try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(NODENAME, nodeName);
		evt.put(USERNAME, userName);
		this.send(evt, PROJECT);
	}catch(Exception e){e.printStackTrace();}
	}
	
	/**
	* Send Iteration Event.
	* 
	* @param event The name of the event
	* @param projectName The name of the project
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendIterationEvent(String event, String projectName, String from, String to)
	{
	 try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(FROM, from);
		evt.put(TO, to);
		this.send(evt, ITERATION);
	  }catch(Exception e){e.printStackTrace();}
	}
	
	/**
	* Send User Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param userName The name of the user
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendUserEvent(String event,String projectName,String userName)
	{
     try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(USERNAME, userName);
		this.send(evt, USER);
	 }catch(Exception e){e.printStackTrace();}
	}
	/**
	* Send Role Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param roleName The name of the role
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendRoleEvent(String event,String projectName,String roleName)
	{
	  try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(ROLENAME, roleName);
		this.send(evt, ROLE);
	  }catch(Exception e){e.printStackTrace();}
	}
	/**
	* Send User Role Event.
	*
	* @param event The name of the event
	* @param projectName The name of the project
	* @param userName The name of the user
	* @param roleName The name of the role
	* @ejb.permission unchecked="yes" 
	* @ejb:interface-method view-type="both"
	*
	**/
	public void sendUserRoleEvent(String event,String projectName,String userName,String roleName)
 	{
	  try{
		Hashtable evt = new Hashtable();
		evt.put(EVENT, event);
		evt.put(PROJECTNAME, projectName);
		evt.put(USERNAME, userName);
		evt.put(ROLENAME, roleName);
		this.send(evt, USERROLE);
	  }catch(Exception e){e.printStackTrace();}
	}

	private void initJMS() throws BonitaServiceException {		
		BonitaServiceLocator serviceLocator = BonitaServiceLocator.getInstance();
		topicConnectionFactory = (TopicConnectionFactory) serviceLocator.getResource(BonitaServiceLocator.Services.TOPIC_CONNECTION_FACTORY);
		topic = (Topic)serviceLocator.getResource(BonitaServiceLocator.Services.TOPIC);
	}

	/**
	* Create the All Projects Session Bean
	*
	* @throws CreateException 
	*
	* @ejb.permission unchecked="yes" 
	* @ejb:create-method view-type="both"
	**/

	public void ejbCreate() throws CreateException {
		try
		{
			initJMS();
		}
		catch (BonitaServiceException e)
		{
			throw new CreateException(e.getMessage());
		}
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	public void ejbRemove() throws EJBException, RemoteException {
	}

	public void ejbActivate() throws EJBException, RemoteException {
		try
		{
			initJMS();
		}
		catch(BonitaServiceException e)
		{ 
			throw new EJBException(e);
		}
	}

	public void ejbPassivate() throws EJBException, RemoteException {
		topicConnectionFactory = null;
		topic = null;
	}

}
