package hero.session;

import hero.interfaces.BnNextNumberLocal;
import hero.interfaces.BnNextNumberLocalHome;
import hero.interfaces.InvalidValueException;
import hero.util.HeroException;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.ServiceUnavailableException;
import javax.sql.DataSource;

/**
 * Encapsulates the retrival of DB data
 *
 * @author Andreas Schaefer
 * @version $Revision: 1.7 $
 *
 * @ejb:bean name="SequenceGenerator"
 *           display-name="Generates unique Identifier for an Entity"
 *           type="Stateless"
 *           jndi-name="ejb/hero/SequenceGenerator"
 *           local-jndi-name="ejb/hero/SequenceGenerator_L"
 * @ejb.permission role-name="BONITAUSER,SuperAdmin"
 *
 * @ejb:transaction type="Supports"
 * 
 * @jonas.bean
 *      ejb-name="SequenceGenerator"
 *      jndi-name="ejb/hero/SequenceGenerator"
 * 
 * 
 * @ejb.resource-ref res-ref-name="jdbc/myDS"
 *      res-type="javax.sql.DataSource"
 *      res-auth="Application"
 * @jonas.resource
 *      res-ref-name="jdbc/myDS"
 *      jndi-name="bonita"
 * 
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 */
public class SequenceGeneratorBean
   implements SessionBean
{

   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------

   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------

   private SessionContext mContext;
   // Only for test purposes
   private int mNextNumber = 0;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
    * Delivers the next sequence number from the given Sequence Name
    *
    * @param pSequenceName Name of the Sequence Generator
    *
    * @return Next sequence number
    *
    * @throws RemoteException 
    *
    * @ejb:interface-method view-type="local"
    * @ejb:transaction type="Required"    
    **/
   public int getNextNumber( String pSequenceName ) throws HeroException
   {	   
	   BnNextNumberLocalHome nHome=null;
	   try {
		   nHome = hero.interfaces.BnNextNumberUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		
		try{
			BnNextNumberLocal nLocal = nHome.findByName(pSequenceName);
			nLocal.setMax(nLocal.getMax()+1);
			return(nLocal.getMax());
		} catch (javax.ejb.FinderException pe) {
			try{
				BnNextNumberLocal nLocal = nHome.create(pSequenceName);
				return(nLocal.getMax());
			} catch (CreateException ce) {
				throw new HeroException(ce.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	         
   }

   /**
    * Create the Session Bean
    *
    * @throws CreateException 
    *
    * @ejb:create-method view-type="remote"
    **/
   public void ejbCreate()
      throws
         CreateException
   {
   }

   /**
    * Describes the instance and its content for debugging purpose
    *
    * @return Debugging information about the instance and its content
    * @ejb:transaction type="Supports"
    **/
   public String toString()
   {
      return "SequenceGeneratorBean [ " + " ]";
   }

   // -------------------------------------------------------------------------
   // Framework Callbacks
   // -------------------------------------------------------------------------
   
   public void setSessionContext( SessionContext aContext )
      throws EJBException
   {
      mContext = aContext;
   }
   
   public void ejbActivate()
      throws EJBException
   {
   }
   
   public void ejbPassivate()
      throws EJBException
   {
   }

   public void ejbRemove()
      throws EJBException
   {
   }

}
