package hero.session;
/*
* 02/01/2002 - 15:24:07
*
* UserRegistrationEJB.java - 
* Copyright (C) 2002 Ecoo Team
* valdes@loria.fr
* 
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
import hero.interfaces.BnAuthRoleLocal;
import hero.interfaces.BnAuthRoleLocalHome;
import hero.interfaces.BnAuthRoleUtil;
import hero.interfaces.BnAuthRoleValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserPropertyLocalHome;
import hero.interfaces.BnUserPropertyUtil;
import hero.interfaces.BnUserUtil;
import hero.interfaces.BnUserValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.InvalidValueException;
import hero.util.HeroException;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

/**
 * The User Registration Bean, is an stateless session bean that provides the API to create users in 
 * Bonita database (new user account). This API allows some functionalities oriented to user
 * configuration: set authorization roles, set Bonita roles, set preferences... 
 * 
 * <br><br>
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the User Registration files:<br><br>
 * 
 * import hero.interfaces.UserRegistrationLocalHome;<br>
 * import hero.interfaces.UserRegistrationLocal;<br>
 * import hero.interfaces.UserRegistrationHome;<br>
 * import hero.interfaces.UserRegistration;<br>
 * import hero.interfaces.UserRegistrationUtil;<br>
 *
 * <br>
 * Now, it is time to create the User Registration instance:<br>  
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * 
 * hero.interfaces.UserRegistrationLocalHome userh = (UserRegistrationLocalHome)hero.interfaces.UserRegistrationUtil.getLocalHome();<br>
 * hero.interfaces.UserRegistrationLocal user = userh.create();<br><br>
 * 
 * or like this if you use remote interfaces:<br>  
 * <br>
 *
 * hero.interfaces.UserRegistrationHome userh = (UserRegistrationHome)hero.interfaces.UserRegistrationUtil.getHome();<br>
 * hero.interfaces.UserRegistration user = userh.create();
 * <br>
 * <br>
 * Now, we can create a new user account by using userCreate methods (with or without the instant
 * messaging address).<br><br>
 * 
 *   user.userCreate(name,password,email);<br>
 *   or 
 *   user.userCreate(name,password,email,jabber);<br>
 *   
 * 
 * @ejb:bean name="UserRegistration"
 *           display-name="UserRegistration Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserRegistration"
 *           local-jndi-name="ejb/hero/UserRegistration_L"
 *
 * @ejb:ejb-ref ejb-name="UserRegistration"
 *              ref-name="myhero/UserRegistration"
 * @ejb:transaction type="Required"
 * @ejb:transaction-type type="Container"
 * @ejb.permission unchecked="yes"  
 * @ejb.security-identity run-as="SuperAdmin"
 * @jonas.bean
 *      ejb-name="UserRegistration"
 *      jndi-name="ejb/hero/UserRegistration"
 *
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 *
 **/

public class UserRegistrationBean implements SessionBean {
  
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;

    private BnUserLocalHome userhome;
    private BnAuthRoleLocalHome rolehome;

   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  
 
	/**
	 * Create the role BONITAUSER if it doesn't exist
	 * 
	 * @throws HeroException, (catch : FinderException)
	 */
	public void assertUserSecurityRoleExists() throws HeroException {

		String role = hero.interfaces.Constants.USER_SECURITY_ROLE;
		BnAuthRoleLocal _role;

		try {        	
			_role = rolehome.findByName(role);
		} catch (javax.ejb.FinderException fe) {
			roleCreate(role,role);
		}
	}

   /**
   * Create the User Session Bean. This method is the first one to invoke in order to
   * creates a new user account.
   *
   * @throws CreateException 
   *
   * @ejb:create-method view-type="both"
   **/
    public void ejbCreate() throws CreateException {
    }
    
    /**
     * Creates user account. You have to call this method after "create" call (this API is an stateles
     * session bean). This method is oriented to Bonita external users configuration
     * @param name name of the new user
     * @ejb:interface-method view-type="local"
     * @throws HeroException 
     *
     **/
    public BnUserLocal userCreate(String name) throws HeroException {
		try {
			BnUserLocal ul = userhome.findByName(name);
			throw new HeroException("User "+ name+ " already exist ");
		} catch (javax.ejb.FinderException nn) {
		try{
		   BnUserValue newUser=new BnUserValue();
	       newUser.setName(name);
	       BnUserLocal user = userhome.create(newUser);
         assertUserSecurityRoleExists();
	       this.setUserRole(name,hero.interfaces.Constants.USER_SECURITY_ROLE);
	       return user;
		} catch (InvalidValueException ie) {
	       throw new HeroException(ie.getMessage());
		} catch (CreateException f) {
	       throw new HeroException(name+" is already exist");
		}
     }
    }

    /**
     * Creates user account. You have to call this method after "create" call (this API is an stateles
     * session bean).
     * @param name name of the new user
     * @param password password of the new user
     * @param email email of the new user
     * @ejb:interface-method view-type="both"
     * @throws HeroException 
     *
     **/
    public void userCreate(String name, String password, String email) throws HeroException {
		try {
			BnUserLocal ul = userhome.findByName(name);
			throw new HeroException("User "+ name+ " already exist ");
		} catch (javax.ejb.FinderException nn) {
		try{
		   BnUserValue newUser=new BnUserValue();
	       newUser.setName(name);
	       newUser.setPassword(password);
	       newUser.setEmail(email);
	       BnUserLocal user = userhome.create(newUser);
         assertUserSecurityRoleExists();
	       this.setUserRole(name,hero.interfaces.Constants.USER_SECURITY_ROLE);
		} catch (InvalidValueException ie) {
	       throw new HeroException(ie.getMessage());
		} catch (CreateException f) {
	       throw new HeroException(name+" is already exist");
		}
     }
    }

    /**
     * Creates user account with an instant messaging address. You have to call this method after "create" call (this API is an stateles
     * session bean).
     * @param name name of the new user
     * @param password password of the new user
     * @param email email of the new user
     * @param jabber jabber address of the new user
     * @ejb:interface-method view-type="both"
     * @throws HeroException 
     *
	**/ 
	 public void userCreate(String name,String password,String email,String jabber)	throws HeroException {
		try {
			BnUserLocal ul = userhome.findByName(name);
			throw new HeroException("User " + name + " already exist ");
		} catch (javax.ejb.FinderException nn) {
			try {
				BnUserValue newUser = new BnUserValue();
				newUser.setName(name);
				newUser.setPassword(password);
				newUser.setEmail(email);
				newUser.setJabber(jabber);
				BnUserLocal user = userhome.create(newUser);
        assertUserSecurityRoleExists();
				this.setUserRole(name, hero.interfaces.Constants.USER_SECURITY_ROLE);
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			} catch (CreateException f) {
				throw new HeroException(name + " is already exist");
			}
		}
	}
	/**
	 * Detete a user from Bonita database. If the user is included in active projects this
	 * methods throws an exception.
	 *
	 * @param userName the name of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	**/
	public void deleteUser(String userName) throws HeroException {		
	
		try {
			BnUserLocal user = userhome.findByName(userName);
			Collection projects = user.getBnProjects();
			Iterator i = projects.iterator();
			while (i.hasNext())
			{
				BnProjectLocal project = (BnProjectLocal)i.next();
				if (project.getState()!=hero.interfaces.Constants.Pj.TERMINATED)
					throw new HeroException("Cannot remove user: This user takes part in some workflow projects ");
			}
			user.remove();
		} catch (FinderException fe) {
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {
			throw new HeroException(rm.getMessage());
		}
	}

    /**
     * Creates a new authorization role to the system. This kind of role is used to control the user
     * access to different APIs. You have to call this method after "create" call (this API is an stateles
     * session bean).
     * @param name name of the new role
     * @param roleGroup the name of the group
     * @ejb:interface-method view-type="both"
     * @throws HeroException
     *
     **/
    public void roleCreate(String name, String roleGroup) throws HeroException {
	try {
	    BnAuthRoleValue newRole=new BnAuthRoleValue();
	    newRole.setName(name);
	    newRole.setBnRoleGroup(roleGroup);
	    BnAuthRoleLocal role = rolehome.create(newRole);
	} catch (CreateException f) {
	    throw new HeroException(name+" is already exist");
	}
    }

    /**
     * Set a new authorization role to the user. You have to call this method after "create" call
     * (this API is an stateles session bean).
     * @param username name of the user
     * @param rolename name of the role
     * @ejb:interface-method view-type="both"
     * @throws HeroException
     *
     **/
    public void setUserRole(String userName, String roleName) throws HeroException {
	try {
	    BnUserLocal user = userhome.findByName(userName);
	    BnAuthRoleLocal role = rolehome.findByName(roleName);
	    Collection roles = user.getBnAuthRoles();
	    roles.add(role);
	} catch (Exception f) {
	    throw new HeroException(userName+" Error in setUserAuthRole...");
	}
    }

    /**
     * Set a new property to the user. User properties will be used to define user preferences.
     * You have to call this method after "create" call (this API is an stateles session bean).
     * 
     * @param userName the name of the user
     * @param key the key of user property
     * @param value the value of user property
     * @ejb:interface-method view-type="both"
     * @throws HeroException
     *
     **/
    public void setUserProperty(String userName, String key, String value) throws HeroException {

	BnUserLocalHome userhome;
	BnUserLocal mUser;
	BnUserPropertyLocalHome pHome;
	BnUserPropertyLocal propertyLocal=null;

	try {
	    userhome=BnUserUtil.getLocalHome();
	    mUser=userhome.findByName(userName);
	
	    try {
		pHome=BnUserPropertyUtil.getLocalHome();
	    } catch (javax.naming.NamingException ne) {
		throw new HeroException(ne.getMessage());
	    }
	    Collection c=null;
	    try {
		c=pHome.findByTheKey(mUser.getId(),key);
	    } catch (FinderException fe) {
		throw new HeroException(fe.getMessage());
	    } 	
	    
	    if (!c.isEmpty()){	    
		
		propertyLocal=(BnUserPropertyLocal)(c.toArray())[0];
		propertyLocal.setTheValue(value);
		
	    }else{
		try{
		    propertyLocal=pHome.create(key,value);
		    propertyLocal.setBnUser(mUser);
		    
		} catch(InvalidValueException ie){
		    throw new HeroException(ie.getMessage()); 	
		} catch (javax.ejb.CreateException ce) {
		    throw new HeroException(ce.getMessage());
		}     	    
	    }      
/*
	    hero.interfaces.UserSessionLocalHome usersessionhome = (UserSessionLocalHome)hero.interfaces.UserSessionUtil.getLocalHome();
	    UserSessionLocal user = usersessionhome.create();
	    user.setUserProperty(key,value);*/

	} catch (Exception f) {
	    throw new HeroException("Error in setUserProperty");
	}
    }

	/**
	* Internal Enterprise Java Beans method. 
	**/
	
    public void setSessionContext(javax.ejb.SessionContext context) throws RemoteException {
	this.mContext=context;
	try {
	    userhome = BnUserUtil.getLocalHome();
	    rolehome = BnAuthRoleUtil.getLocalHome();
	}catch(Exception e){
	    throw new RemoteException(e.getMessage());
	}
    }

	/**
	* Internal Enterprise Java Beans method. 
	**/
    public void unsetSessionContext() throws RemoteException {
    }
    
	/**
	* Internal Enterprise Java Beans method. 
	**/
    public void ejbRemove() throws java.rmi.RemoteException {
    }
    
	/**
	* Internal Enterprise Java Beans method. 
	**/
    public void ejbActivate() throws java.rmi.RemoteException {
    }
    
	/**
	* Internal Enterprise Java Beans method. 
	**/
    public void ejbPassivate() throws java.rmi.RemoteException {
    }

}
