package hero.session;

import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.interfaces.UserRegistrationLocal;
import hero.interfaces.UserRegistrationLocalHome;
import hero.interfaces.UserRegistrationUtil;
import hero.util.HeroException;
import hero.user.UserBase;
import hero.user.UserBaseException;
import hero.user.UserBaseService;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;


/**
 * @ejb:bean name="UserService"
 * 		display-name="UserService Bean" 
 * 		type="Stateless"
 * 		transaction-type="Container" 
 * 		jndi-name="ejb/hero/UserService"
 * 		local-jndi-name="ejb/hero/UserService_L"
 * 
 * @ejb:ejb-ref ejb-name="UserService" 
 * 		ref-name="myhero/UserService"
 * @ejb:transaction-type type="Container" 
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin"
 * 
 * @jonas.bean 
 * 		ejb-name="UserService" 
 * 		jndi-name="ejb/hero/UserService"
 *
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class UserServiceBean implements SessionBean {
	SessionContext mContext;
	
	/**
	 * Create the User Session Bean. This method is the first one to invoke in order to
	 * creates a new user account.
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}
	
	/**
	 * @ejb.interface-method
	 * @ejb.transaction type="Supports"
	 * @param name
	 */	
	public BnUserLocal findUserLocal(String name) throws HeroException {
		BnUserLocalHome uHome;
		try {
			uHome=BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			return(uHome.findByName(name));
		} catch (FinderException f){			
			return(registerUser(name));				 
		}
	}
	
	/**
	 * @ejb.interface-method
	 * @ejb.transaction type="Supports"
	 * @param name
	 */	
	public BnUserLocal findUser(String name) throws HeroException {
		BnUserLocalHome uHome;
		try {
			uHome=BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {		
			findUserInBase(name);	
		} catch (Exception ne){
			throw new HeroException(ne.getMessage());
		}	
		try {		
			return(uHome.findByName(name));
		} catch (Exception f){
			return(registerUser(name));				 
		}
	}
	
	/**
	 * @ejb.interface-method
	 * @ejb.transaction type="Supports"
	 * @param name
	 */	
	public Collection findUsers() throws HeroException {
		BnUserLocalHome uHome;
		try {
			uHome=BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			return(findUsersInBase());			
		} catch (Exception f){
			throw new HeroException(f.getMessage());
		}
	}
	
	/**
	 * @ejb.interface-method
	 * @ejb.transaction type="Supports"
	 * @param name
	 */	
	public Map getUserInfos(String name) throws HeroException {
		return(findUserInBase(name));
	}
		
	private Map findUserInBase(String id) throws HeroException{
		Collection ucol=UserBaseService.getInstance().getUserBases();
		Iterator i=ucol.iterator();
		Map uinfos=null;
		while (i.hasNext()) {
			UserBase ub=(UserBase)i.next();
			try {
				uinfos=ub.getUserInfos(id);
				break;
			} catch (UserBaseException u) {
				throw new HeroException(u.getMessage());
			}
		}
		return uinfos;
	}

	private Collection findUsersInBase() throws HeroException{
		Collection ucol=UserBaseService.getInstance().getUserBases();
		Iterator i=ucol.iterator();		
		Collection uinfos = new ArrayList();
		while (i.hasNext()) {
			UserBase ub=(UserBase)i.next();
			try {
				uinfos=ub.getUsers();
				break;
			} catch (UserBaseException u) {
				throw new HeroException(u.getMessage());
			}
		}
		return uinfos;
	}
	
	private BnUserLocal registerUser(String name) throws HeroException {
		try{
			UserRegistrationLocalHome urh=UserRegistrationUtil.getLocalHome();
			UserRegistrationLocal ur=urh.create();
			BnUserLocal user = ur.userCreate(name);
			ur.remove();
			return(user);
		}catch(Exception e){throw new HeroException(e.getMessage());}
		
	}
		
	/**
	 * Internal Enterprise Java Beans method.
	 */
	public void setSessionContext(javax.ejb.SessionContext context)
			throws RemoteException {
		this.mContext = context;
	}
	/**
	 * Internal Enterprise Java Beans method.
	 */
	public void unsetSessionContext() throws RemoteException {
	}
	/**
	 * Internal Enterprise Java Beans method.
	 */
	public void ejbRemove() throws java.rmi.RemoteException {
	}
	/**
	 * Internal Enterprise Java Beans method.
	 */
	public void ejbActivate() throws java.rmi.RemoteException {
	}
	/**
	 * Internal Enterprise Java Beans method.
	 */
	public void ejbPassivate() throws java.rmi.RemoteException {
	}
}