package hero.session;
/*
 * 1Z/01/2004 - 11:58:09
 *
 * UserSoapXML.java - 
 * Copyright (C) 2003 Ecoo Team
 * valdes@loria.fr
 * 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserPropertyLocal;
import hero.interfaces.BnUserValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSessionLocal;
import hero.interfaces.UserServiceLocal;
import hero.interfaces.UserServiceLocalHome;
import hero.util.HeroException;
import hero.util.ProjectNodes;
import hero.util.Projects;
import hero.util.UserProperties;
import hero.util.EventConstants;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;


import org.apache.log4j.Category;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;

/**
 * UserSessionXML API based on UserSession API but oriented to web services 
 * calls from a non-Java language. Get methods of this API returns
 * XML.
 *
 * @ejb:bean name="UserSoapXML"
 *           display-name="UserSoapXML Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/UserSoapXML"
 *
 * @ejb:ejb-ref ejb-name="BnUser"
 *              ref-name="myhero/BnUser"
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin"
 *
 * @jonas.bean
 *      ejb-name="UserSoapXML"
 *      jndi-name="ejb/hero/UserSoapXML"
 * 
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 * 
 * @copyright INRIA
 * @author Miguel Valdes
 * 
 **/

public class UserSoapXMLBean implements SessionBean, EventConstants {
	
	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Category trace = Category.getInstance(UserSessionBean.class);
	
	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------
	
	private SessionContext mContext;
	
	// -------------------------------------------------------------------------
	// XML Get Methods (Castor Data Binding)
	// -------------------------------------------------------------------------
	
	/**
	 * Get user project list. Workflow processes associated to this user
	 * @return a Collection of BnProjectLightValue objects - the projects list of the user
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getProjectList() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		Collection pjts = new ArrayList();
		Projects pb = new Projects();
		Collection pva;
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			Collection projects = null;
			projects = mUser.getBnProjects();
			BnUserValue uv = mUser.getBnUserValue();
			hero.interfaces.BnProjectLightValue[] pv = uv.getBnProjects();
			pva = new ArrayList(Arrays.asList(uv.getBnProjects()));
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
					mContext.getCallerPrincipal().getName() + " does not exists");
		}
		
		try {
			pb.setProject(pva);
			StringWriter sw = new StringWriter();
			Marshaller.marshal(pb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Obtain all user activities from specific project (ready and anticipable state)
	 * @param projectName the name of the project
	 * @return a String Collection - the list of todo activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getToDoList(String projectName) throws HeroException {
			trace.debug("parameter: projectName=" + projectName + "start by " + mContext.getCallerPrincipal().getName());
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
					"BnProject " + projectName + " does not exists");
		}
		try {
			ready =
				nodehome.findUserReady(
						project.getId(),
						mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			ready = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			result.add(nd.getBnNodeLightValue());
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	/**
	 * Obtain all user activities from specific project (ready and anticipable state)
	 * @param projectName the name of the project
	 * @return a String Collection - the list of todo activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getToDoList(String projectName, int offset, int numrows) throws HeroException {
		trace.debug("parameter: projectName=" + projectName + "start by " + mContext.getCallerPrincipal().getName());
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection ready;
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
					"BnProject " + projectName + " does not exists");
		}
		try {
			ready =
				nodehome.findUserReady(
					project.getId(),
					mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {
			ready = new ArrayList();
		}
		int cnt = 0;
		int add = 0;
		ArrayList result = new ArrayList();
		for (Iterator i = ready.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (cnt >= offset) {
				result.add(nd.getBnNodeLightValue());
				add++;
				if (add == numrows) break;
			}
			cnt++;
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getActivityList(String projectName) throws HeroException {
			trace.debug("parameter: projectName=" + projectName + "start by " + mContext.getCallerPrincipal().getName());
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection executing;		
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
					"BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(
						project.getId(),
						mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {			
			executing = new ArrayList();
		}
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			result.add(nd.getBnNodeLightValue());
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	/**
	 * Obtain all user activities from specific project (executing and anticipating state)
	 *
	 * @param projectName the name of the project
	 * @return a String Collectio - the list of active activities ot the user for a project
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getActivityList(String projectName, int offset, int numrows) throws HeroException {
		trace.debug("parameter: projectName=" + projectName + "start by " + mContext.getCallerPrincipal().getName());
		ProjectNodes nb = new ProjectNodes();
		BnProjectLocalHome projhome;
		BnProjectLocal project;
		BnNodeLocalHome nodehome;
		Collection executing;		
		try {
			projhome = BnProjectUtil.getLocalHome();
			nodehome = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			project = projhome.findByName(projectName);
		} catch (FinderException fe) {
			throw new HeroException(
				"BnProject " + projectName + " does not exists");
		}
		try {
			executing =
				nodehome.findUserExecuting(
					project.getId(),
					mContext.getCallerPrincipal().getName());
		} catch (FinderException fe) {			
			executing = new ArrayList();
		}
		int cnt = 0;
		int add = 0;
		ArrayList result = new ArrayList();
		for (Iterator i = executing.iterator(); i.hasNext();) {
			BnNodeLocal nd = (BnNodeLocal) i.next();
			if (cnt >= offset) {
				result.add(nd.getBnNodeLightValue());
				add++;
				if (add == numrows) break;
			}
			cnt++;
		}
		try {
			nb.setNodes(result.toArray());
			StringWriter sw = new StringWriter();
			Marshaller.marshal(nb, sw);
			return (sw.toString());
		} catch (MarshalException ex1) {
			throw new HeroException(ex1.getMessage());
		} catch (ValidationException ex2) {
			throw new HeroException(ex2.getMessage());
		}
	}
	
  /**
	* Get number of total entities
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
   **/        
    public int getListCnt(String projectName, String type) throws HeroException {
       trace.debug("parameter: projectName=" + projectName + ", type=" + type + "start by " + mContext.getCallerPrincipal().getName());
	   Collection list = null;
	   int cnt = 0;
	   BnProjectLocal project;
	   BnNodeLocalHome nodehome;
	   BnProjectLocalHome projhome;

	   try {
			if (type.equals(Constants.Pj.GETACTIVITYLIST) ||
					type.equals(Constants.Pj.GETTODOLIST)) {
				nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
				projhome = hero.interfaces.BnProjectUtil.getLocalHome();
				System.out.println("Antes del findByName...");
				project = projhome.findByName(projectName);
				System.out.println("Despues del findByName...");
				list = nodehome.findUserExecuting(project.getId(),
						mContext.getCallerPrincipal().getName());
			}
			Iterator it = list.iterator();
		   	while (it.hasNext()) {
				it.next();
				cnt++;
			}
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		return cnt;
	   }
	   
	/**
	 * Get the user light value
	 * 
	 * @return a BnUserLightValue object - get user information
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public BnUserLightValue getUserLightValue() throws HeroException{
		try{
			trace.debug("start by " + mContext.getCallerPrincipal().getName());
			BnUserLocalHome userhome = hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser = userhome.findByName(mContext.getCallerPrincipal().getName());
			BnUserLightValue ulv= mUser.getBnUserLightValue();
			UserServiceLocalHome ushome = null;
			try {
				ushome = hero.interfaces.UserServiceUtil.getLocalHome();
				UserServiceLocal ul = ushome.create();
				Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
				ulv.setEmail((String)infos.get("email"));
				ulv.setPassword((String)infos.get("password"));
				ulv.setJabber((String)infos.get("jabber"));
				return ulv;
			} catch (Exception ce) {
				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(mContext.getCallerPrincipal().getName() + " does not exists " + fe.getMessage());
			throw new HeroException(mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
	
	/**
	 * Get the user name
	 * 
	 * @return a String - get user name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUser() {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		return mContext.getCallerPrincipal().getName();
	}
		
	/**
	 * Get the mail of this user from Bonita database.
	 *
	 * @param userName the name of the user
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getUserMail() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		UserServiceLocalHome ushome = null;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("email"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}				
	}
	
		
	/**
	 * Get the user password
	 *
	 * @return a String - get user password
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserPassword() throws HeroException{
		UserServiceLocalHome ushome = null;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("password"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}		
	}
	
	/**
	 * Get the user jabber address
	 *
	 * @return a String - get user jabber address
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 **/
	public String getUserJabber() throws HeroException{
		UserServiceLocalHome ushome = null;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			Map infos = ul.getUserInfos(mContext.getCallerPrincipal().getName());
			return ((String)infos.get("jabber"));
		} catch (Exception ce) {
			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}		
	}
	
	/**
	 * Get User properties
	 *
	 * @return a BnUserPropertyValue Collection - list of user properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public String getUserProperties() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		Collection prp = new ArrayList();
		BnUserPropertyLocal pl;
		UserProperties ub = new UserProperties();
		try {
			BnUserLocalHome userhome =
				hero.interfaces.BnUserUtil.getLocalHome();
			BnUserLocal mUser =
				userhome.findByName(mContext.getCallerPrincipal().getName());
			Collection pls = mUser.getBnProperties();
			
			for (Iterator i = pls.iterator(); i.hasNext();) {
				pl = (BnUserPropertyLocal) i.next();
				prp.add(pl.getBnUserPropertyValue());
			}
			
			try {
				ub.setProperty(prp);
				StringWriter sw = new StringWriter();
				Marshaller.marshal(ub, sw);
				return (sw.toString());
			} catch (MarshalException ex1) {
				throw new HeroException(ex1.getMessage());
			} catch (ValidationException ex2) {
				throw new HeroException(ex2.getMessage());
			}
			
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			throw new HeroException(
					mContext.getCallerPrincipal().getName() + " does not exists");
		}
	}
    	
	
	/**
	 * Create the BnUser Session Bean
	 *
	 * @throws CreateException 
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {
	}
	
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}
	
	public void ejbRemove() {
	}
	
	public void ejbActivate() {
	}
	
	public void ejbPassivate() {
	}
	
	// utility methods
	private boolean checkRights(ProjectSessionLocal ps) throws HeroException {
		// check that current user is a member of the project
		return ps.containsUser(mContext.getCallerPrincipal().getName());
	}
}


