package hero.session;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import hero.interfaces.Constants.Nd;
import hero.interfaces.Constants.Pj;
import hero.interfaces.Constants.PerformerAssigment;
import hero.interfaces.Constants.Hook;
import hero.interfaces.Constants.Mapper;
import hero.interfaces.BnLdap;
import hero.interfaces.BnLdapHome;
import hero.interfaces.BnLdapUtil;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.ProjectSessionLocalHome;
import hero.interfaces.ProjectSessionLocal;

import hero.xpdl.XPDLDTDs;
import hero.xpdl.XPDLDataField;
import hero.xpdl.XPDLFormalParameter;
import hero.xpdl.XPDLHook;
import hero.xpdl.XPDLInitiatorMapper;
import hero.xpdl.XPDLSchemas;
import hero.xpdl.XPDLActivity;
import hero.xpdl.XPDLParticipant;
import hero.xpdl.XPDLApplication;
import hero.xpdl.XPDLTransition;
import hero.xpdl.XPDLWFProcess;
import hero.xpdl.XPDLConstants;
import hero.xpdl.BonitaTree;
import hero.xpdl.JEntityResolver;

import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.XPDLException;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.helpers.XMLFilterImpl;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;


/**
 * Session Bean Template
 *
 * @ejb:bean name="XPDLSessionImport"
 *           display-name="XPDLSessionImport Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/XPDLSessionImport"
 *           local-jndi-name="ejb/hero/XPDLSessionImport_L"
 *
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="Admin,user,SuperAdmin"
 * @jonas.bean
 *      ejb-name="XPDLSessionImport"
 *      jndi-name="ejb/hero/XPDLSessionImport"
 *
 *
 **/

public class XPDLSessionImportBean implements SessionBean,XPDLConstants {



	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------

	private SessionContext mContext;
	private ProjectSessionLocalHome pHome;
	private BnUserLocalHome userhome;


	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	/**
	 * getProjectWithVersion
	 *
	 * @ejb.permission unchecked="yes"
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public String openMainDocumentAndReturnProject(String xpdlRepository, String fileName, String xpdlString) throws XPDLException
	{
		String projectWithVersion = openMainDocument(xpdlRepository, fileName, xpdlString,false);
		return projectWithVersion;
	}

	/**
	 *        Parsing the xpdl file
	 * @param String xpdlRepository - The path of the repository
	 * @param String fileName  - The name the xpdl file
	 * @param String xpdlString - The xpdl file as string
	 * @param boolean traceDebug  - set debug trace
     * @return projectWithVersion - the project name with the version
     *
	 * @ejb.permission unchecked="yes"
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

	public String openMainDocument(String xpdlRepository, String fileName, String xpdlString, boolean traceDebug) throws XPDLException
	{
//debug
//System.out.println("openMainDocument: File = " + xpdlRepository + fileName );
//System.out.println("openMainDocument: xpdlFileName = " +     xpdlFileName);


        Hashtable xpdlData;
        Hashtable actMapping;
        // Activity sets
        Hashtable activitySetMapping;
        // Participants : id / XPDLParticipant
        Hashtable partMapping;
        // For applications id / XPDLApplication
        Hashtable appliMapping;
        // DataFields : id / XPDLDataField
        Hashtable dataFieldMapping;
        // DataFields introduced to get same property order as FormEditor
        ArrayList array_dataFieldKey;
        // WF Process : id / XPDLWFProcess
        Hashtable wfProcessMapping;
        // For transitions
        Hashtable transitionMapping;
        // for process version
        String projNameWithVersion="";

        xpdlData = new Hashtable();
        actMapping = new Hashtable();
        activitySetMapping = new Hashtable();
        partMapping = new Hashtable();
        appliMapping = new Hashtable();
        dataFieldMapping = new Hashtable();
        array_dataFieldKey = new ArrayList();
        wfProcessMapping = new Hashtable();
        transitionMapping = new Hashtable();

        xpdlData.put("actMapping",actMapping);
        xpdlData.put("activitySetMapping",activitySetMapping);
        xpdlData.put("partMapping",partMapping);
        xpdlData.put("appliMapping",appliMapping);
        xpdlData.put("dataFieldMapping",dataFieldMapping);
        xpdlData.put("array_dataFieldKey",array_dataFieldKey);
        xpdlData.put("wfProcessMapping",wfProcessMapping);
        xpdlData.put("transitionMapping",transitionMapping);
        xpdlData.put("projNameWithVersion",projNameWithVersion);
		openDocument(xpdlRepository, fileName, xpdlString, xpdlData);

        //Call print debug if trace_debug == true
        if (traceDebug == true)
        printHashTables(xpdlData);

        return (String)xpdlData.get("projNameWithVersion");
	}


	private void openDocument(String xpdlRepository, String fileName, String xpdlString, Hashtable xpdlData) throws XPDLException
	{
		DocumentBuilderFactory dbf =  DocumentBuilderFactory.newInstance();
		dbf.setValidating(true);
		dbf.setAttribute("http://apache.org/xml/features/dom/include-ignorable-whitespace",	new Boolean(false));
		dbf.setAttribute("http://apache.org/xml/features/validation/schema",new Boolean(true));
		dbf.setAttribute("http://apache.org/xml/features/allow-java-encodings",new Boolean(true));
		dbf.setNamespaceAware(true);

		DocumentBuilder docBuilder = null;
		try {
			docBuilder = dbf.newDocumentBuilder();
			ErrorHandler eh = new XMLFilterImpl();
	        docBuilder.setErrorHandler(eh);
		} catch (ParserConfigurationException pce) {
			throw new XPDLException("Cannot build a document builder : " + pce.getMessage(), pce);
		}
		JEntityResolver jer = new JEntityResolver();
		jer.addSchemas(new XPDLSchemas());
		jer.addDtds(new XPDLDTDs());
		docBuilder.setEntityResolver(jer);

		InputStream inputStream = (InputStream)new ByteArrayInputStream(xpdlString.getBytes());

		Document document =	null;
		try {
			document = docBuilder.parse(inputStream);
		} catch (Exception e) {
			throw new XPDLException("Cannot parse XPDL :" + e.getMessage(), e);
		}

		Element  rootElement =  document.getDocumentElement();
		//NodeList LesFils = rootElement.getChildNodes();

		if (rootElement.hasChildNodes())
		{
			BonitaTree root=new BonitaTree(rootElement);
			BonitaTree external=getNode(root, EXTERNAL_PACKAGES);
			//External Packages definitions parsing
			if (external!=null){
				obtainExternalPackage(external, xpdlRepository, xpdlData);
			}

			//Get package definitions containing : Participants, Applications , DataFields
			// from the Package level
			obtainParticipant(getNode (root, PARTICIPANTS), PACKAGE_LEVEL, xpdlData);
			obtainApplication(getNode (root, APPLICATIONS), PACKAGE_LEVEL, xpdlData);
//debug
//System.out.println("****!!! Calling   obtainDataField in openDocument - projectName = PACKAGE"  );
			obtainDataField(getNode (root, DATAFIELDS),PACKAGE_LEVEL, xpdlData);

			//Workflow processes parsing
			obtainProject(root, xpdlData);
		}


	}

	/*
	 *   Get all defined External Packages name
	 *   and call openDocument() for each of them
	 */
	private void obtainExternalPackage(BonitaTree bonitaTree, String xpdlRepository, Hashtable xpdlData) throws XPDLException
	{

		if ((bonitaTree.Noeud.getNodeName()).equalsIgnoreCase(EXTERNAL_PACKAGES))
		{
// debug
//System.out.println("External Packages...");
			for(int j = 0; j<bonitaTree.NbFils; j++)
			{
				if ((bonitaTree.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(EXTERNAL_PACKAGE))
				{
					Element external = (Element)bonitaTree.Fils[j].Noeud;
					String href = external.getAttribute(EXTERNAL_PACKAGE_HREF);
					String xpdl_file = "";
//debug
//System.out.println("External Package="+href);
					try{
					String filename = xpdlRepository + File.separator + href;
				    java.io.BufferedReader buff = new java.io.BufferedReader(new java.io.FileReader(filename));
				    String line = "";
				    line = buff.readLine();
				    while ((line = buff.readLine()) != null)
				      xpdl_file = xpdl_file + line;

					} catch (Exception e) {
						throw new XPDLException("Error buffering the XPDL file : " + e.getMessage(), e);
					}
					// open the xpdl document
					openDocument(xpdlRepository, href, xpdl_file, xpdlData);
				}
			}
		}
	}

	/*
	 * Get all WorkFlow Processes and call createProject for all of them
	 */
	private void obtainProject(BonitaTree n, Hashtable xpdlData) throws XPDLException
	{
		BonitaTree completeTree = n;
		n = getNode(n,WORKFLOW_PROCESSES);
		if (n!=null)
		{
			int type = n.Noeud.getNodeType();
			String projectName="";

			if (type==Node.ELEMENT_NODE)
			{
				if ((n.Noeud.getNodeName()).equalsIgnoreCase(WORKFLOW_PROCESSES))
				{
					for(int j=0; j<n.NbFils ; j++)
					{
						if ((n.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(WORKFLOW_PROCESS))
						{
							// search for Version of the Workflow Process
							String projectVersion = null;
							Text versionValue = null;
							BonitaTree currentWf = n.Fils[j];
							NodeList listWFProcessNode = currentWf.Noeud.getChildNodes();
							int size = listWFProcessNode.getLength();
							// get RedefinableHeader tag
							if (size > 0) {
								Element elementRedefinableHeader = null;
								for (int i=0; i<size; i++){
								    int ty = listWFProcessNode.item(i).getNodeType();
								    String nodeName = listWFProcessNode.item(i).getNodeName();
									if (ty == (Node.ELEMENT_NODE) && (WORKFLOW_PROCESS_REDEFINABLEHEADER.equals(nodeName))){
//debug
//System.out.println("node name = " + listWFProcessNode.item(i).getNodeName() );
										elementRedefinableHeader = (Element)listWFProcessNode.item(i);
										NodeList listElementsVersion =null;
										listElementsVersion = elementRedefinableHeader.getElementsByTagName(WORKFLOW_PROCESS_VERSION);
										versionValue = (Text)listElementsVersion.item(0).getFirstChild();
									}
								}
								//if (versionValue.getNodeValue() != null){
								if (versionValue != null){
									projectVersion = versionValue.getNodeValue();
//debug
//System.out.println("getCreateSubProject: version = " + projectVersion );
								}
								else{
									projectVersion = EventConstants.DEFAULTPROJECTVERSION;
//debug
//System.out.println("getCreateSubProject: version = DEFAULT = " + projectVersion );
								}
							}

							//get project name
							Element element = (Element)n.Fils[j].Noeud;
							projectName=element.getAttribute(WORKFLOW_PROCESS_NAME);
							String projectId=element.getAttribute(WORKFLOW_PROCESS_ID);

							if (!projectExist(projectName, projectVersion))
							{
								if (!(projectName.equalsIgnoreCase("")))
								{


//Trace
System.out.println("*** Start of Workflow Process creation : " + projectName.toUpperCase());

									createProject(n.Fils[j], projectName, projectVersion, projectId, completeTree, xpdlData);
//Trace
System.out.println("*** End of Workflow Process creation : " + projectName.toUpperCase());
								}
							}
							else {
								// If this process is used as subprocess it may has been already created
								ArrayList ar_pn = new ArrayList();
                                Hashtable wfProcessMapping = (Hashtable)xpdlData.get("wfProcessMapping");
								Object[] tab = wfProcessMapping.keySet().toArray();
									for (int k=0;k<tab.length;k++){
//debug
//System.out.print("   - " + tab[k].toString());
										XPDLWFProcess xpdlWFProcess = (XPDLWFProcess)wfProcessMapping.get(tab[k]);
										ar_pn.add(xpdlWFProcess.getName() );
									}
								if ( !ar_pn.contains(projectName)){
									throw new XPDLException("Error: Project = " + projectName + " is already existing in the Bonita Workflow Engine. ");
								}
							}
						}
					}
				}
			}
		}
	}

	/*
	 *    Obtain the WorkFlow Process (used as subproject)
	 *    Call createProject if this project does not exist
	 *    Note :   this method is not called at now!
	 */
	private String getCreateSubProject(String id, BonitaTree root, Hashtable xpdlData) throws XPDLException
	{
		BonitaTree project = getNode(root, WORKFLOW_PROCESSES);
		int type = project.Noeud.getNodeType();

		if (type==Node.ELEMENT_NODE)
		{
			if ((project.Noeud.getNodeName()).equalsIgnoreCase(WORKFLOW_PROCESSES))
			{
				for(int j=0; j<project.NbFils ; j++)
				{
					if ((project.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(WORKFLOW_PROCESS))
					{
						// search for Version of the Workflow Process
						String projectVersion = null;
						Text versionValue = null;
						BonitaTree currentWf = project.Fils[j];
						NodeList listWFProcessNode = currentWf.Noeud.getChildNodes();
						int size = listWFProcessNode.getLength();
						// get RedefinableHeader tag
						if (size > 0) {
							Element elementRedefinableHeader = null;
							for (int i=0; i<size; i++){
							    int ty = listWFProcessNode.item(i).getNodeType();
							    String nodeName = listWFProcessNode.item(i).getNodeName();
								if (ty == (Node.ELEMENT_NODE) && (WORKFLOW_PROCESS_REDEFINABLEHEADER.equals(nodeName))){
//debug
//System.out.println("node name = " + listWFProcessNode.item(i).getNodeName( );
									elementRedefinableHeader = (Element)listWFProcessNode.item(i);
									NodeList listElementsVersion =null;
									listElementsVersion = elementRedefinableHeader.getElementsByTagName(WORKFLOW_PROCESS_VERSION);
									versionValue = (Text)listElementsVersion.item(0).getFirstChild();
								}
							}
							if (versionValue.getNodeValue() != null){
								projectVersion = versionValue.getNodeValue();
//debug
//System.out.println("getCreateSubProject: version = " + projectVersion );
							}
							else{
								projectVersion = EventConstants.DEFAULTPROJECTVERSION;
//debug
//System.out.println("getCreateSubProject: version = DEFAULT = " + projectVersion );
							}
						}

						// get project name
						Element element = (Element)project.Fils[j].Noeud;
						if (element.getAttribute(WORKFLOW_PROCESS_ID).equalsIgnoreCase(id))
						{
							String projectName=element.getAttribute(WORKFLOW_PROCESS_NAME);
							String projectId=element.getAttribute(WORKFLOW_PROCESS_ID);

							if (!projectExist(projectName, projectVersion))
							{
								if (!(projectName.equalsIgnoreCase("")))
								{
									// addSubProcessDefinition(completeTree,projectName);
//Trace
System.out.println("*** Start of Workflow Process creation (used as subprocess): " + projectName.toUpperCase());
									createProject(project.Fils[j],projectName,projectVersion, projectId, root, xpdlData);
//Trace
System.out.println("*** End of Workflow Process creation : " + projectName.toUpperCase());
								}
							}
							return (projectName);
						}
					}
				}
			}
		}
		return null;
	}

	/*
	 *   Create the Bonita Project with all Workflow Process definitions
	 *   This is the heart method of this file that call :
	 *      - obtainActivitySet, obtainCreateActivity, obtain transition
	 * 		- obtainCreateActivity, obtainActivityData
	 */
	private void createProject(BonitaTree project, String projectName, String projectVersion, String projectId, BonitaTree root, Hashtable xpdlData) throws XPDLException{
		// create and init project Bean object
		ProjectSessionLocal p = null;
		try {
			pHome = (ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome();
		} catch (NamingException na) {
			throw new XPDLException(na.getMessage(), na);
		}
		try {
		p=pHome.create();
		} catch (javax.ejb.CreateException cr) {
			throw new XPDLException(cr.getMessage(), cr);
		}
		try {
			if (projectVersion == null) {
			p.initModel(projectName);
            xpdlData.put("projNameWithVersion", projectName + "." + EventConstants.DEFAULTPROJECTVERSION);
			//this.projNameWithVersion = projectName + "." + EventConstants.DEFAULTPROJECTVERSION ;
			}
			else {
			p.initModelWithVersion(projectName, projectVersion);
            xpdlData.put("projNameWithVersion", projectName + "." + projectVersion);
			//this.projNameWithVersion = projectName + "." + projectVersion;
			}
		} catch (HeroException he) {
			throw new XPDLException(he.getMessage(), he);
		}

		// Get Participants , Applications, DataFields defined into the WorkFlow Process
		obtainParticipant(getNode (project, PARTICIPANTS), projectName, xpdlData);
		obtainApplication(getNode (project, APPLICATIONS), projectName, xpdlData);
		obtainDataField(getNode (project, DATAFIELDS), projectName, xpdlData);

		// Add properties to Bonita project with obtained datafields
		createProcessProperties(p, xpdlData);

		// Get hooks defined as external attributes into the Workflow Process
		XPDLWFProcess xpdlWFProcess = new XPDLWFProcess();
		xpdlWFProcess.setName(projectName);

		// set the hashtable for process
        Hashtable wfProcessMapping = (Hashtable)xpdlData.get("wfProcessMapping");
        wfProcessMapping.put(projectId,xpdlWFProcess);



		// Obtain the process hooks
		//xpdlWFProcess = obtainProcessHook(getFilsNode (project, EXTENDED_ATTRIBUTES), xpdlWFProcess);
		if (project!=null)
		{
			  BonitaTree extAttrs = null;
			  BonitaTree extAttr = null;

			    // List childs of WFProcess BonitaTree to search EXTENDED_ATTRIBUTES element
				for(int j = 0; j<project.NbFils; j++)
				{

					if ((project.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(EXTENDED_ATTRIBUTES)){
						extAttrs = project.Fils[j];
						for(int i = 0; i<extAttrs.NbFils; i++){
							extAttr = extAttrs.Fils[i];
							if ((extAttr.Noeud.getNodeName()).equalsIgnoreCase(EXTENDED_ATTRIBUTE)){
								// detect if it's a hook
								Element element = (Element)extAttr.Noeud;
								if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_HOOK)){
									// Obtain the hook if it exists from ExtAttr Bonita tree
									XPDLHook xpdlHook = new XPDLHook();
									obtainProcessHook(extAttr, xpdlWFProcess,xpdlHook);
								}
								// detect if it's an Initiator Mapper
								else if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_INITIATORMAPPER)){
									// Obtain the hook if it exists from ExtAttr Bonita tree
									XPDLInitiatorMapper xpdlInitiatorMapper = new XPDLInitiatorMapper();
									obtainInitiatorMapper(extAttr, xpdlWFProcess,xpdlInitiatorMapper);
								}

							}
						}
					}
				}
		}

		// Add hook to Bonita project
		try {
		addProcessHook2(project, projectName, p, xpdlWFProcess);
		} catch (HeroException he) {
			throw new XPDLException(he.getMessage(), he);
		}

		// Add Initiator Mapper to Bonita project
		try {
		addProcessInitiatorMapper(project, projectName, p, xpdlWFProcess);
		} catch (HeroException he) {
			throw new XPDLException(he.getMessage(), he);
		}

		// Get activitySet
		boolean hasActivitySet = obtainActivitySet(getNode (project, ACTIVITYSETS), p, xpdlData);
		if (hasActivitySet)
		{
			for(int k=0; k<project.NbFils ; k++) // Create activities located out of an activityset
			{
				if ((project.Fils[k].Noeud.getNodeName()).equalsIgnoreCase(ACTIVITIES))
					obtainCreateActivity(getNode (project.Fils[k], ACTIVITIES), p, root, xpdlData);
				else if ((project.Fils[k].Noeud.getNodeName()).equalsIgnoreCase(TRANSITIONS))
					obtainTransition(getNode (project.Fils[k], TRANSITIONS), p, false, null, xpdlData);
			}
		}
		else // This workflow process does not have activitySets
		// Get and add : Activities & Transitions
		{
			obtainCreateActivity(getNode (project, ACTIVITIES), p, root, xpdlData);
			obtainTransition(getNode (project, TRANSITIONS), p, false, null, xpdlData);
		}
		// add iteration
        Hashtable actMapping = (Hashtable)xpdlData.get("actMapping");
        Object[] tab = actMapping.keySet().toArray();

		for (int k=0;k<tab.length;k++){
			XPDLActivity xpdlActivity = (XPDLActivity)actMapping.get(tab[k]);

			if (xpdlActivity.getIteration().size()!=0 ) {
				try {
                    Hashtable iteration = (Hashtable)xpdlActivity.getIteration();
                    Object[] tabIteration = iteration.keySet().toArray();
                        for (int ite=0;ite<tabIteration.length;ite++){
                            p.addIteration(xpdlActivity.getName(), tabIteration[ite].toString(), (String)iteration.get(tabIteration[ite]));
                        }
					//p.addIteration(xpdlActivity.getName(), xpdlActivity.getIterationTo(), xpdlActivity.getIterationCondition());
				} catch (HeroException he) {
					throw new XPDLException("Error setting iteration On activity = " + xpdlActivity.getName() + he.getMessage(), he);
				}
			}
		}

	}

	/*
	 *    Create properties for Bonita project
	 *    defined in the Workflow Process or in the Package
	 */
	private void createProcessProperties(ProjectSessionLocal p, Hashtable xpdlData) throws XPDLException{
		String projectName = null;
		try {
		projectName = p.getName();
		} catch (HeroException he) {
			throw new XPDLException(he.getMessage(), he);
		}
//Debug
//System.out.println("createProcessProperties");
		//Modif Form Ed: Collection co = dataFieldMapping.values();
//System.out.println("*****> nb values = " + co.size());
		//Modif Form Ed: Iterator it = co.iterator();
		//Modif Form Ed: get the order of the key in dataFieldMapping
        ArrayList array_dataFieldKey = (ArrayList)xpdlData.get("array_dataFieldKey");
        Iterator it =array_dataFieldKey.iterator();


		while (it.hasNext()){
			//XPDLDataField xpdlDataField = (XPDLDataField)it.next();
			String key = (String)it.next();
			Hashtable dataFieldMapping = (Hashtable)xpdlData.get("dataFieldMapping");
            XPDLDataField xpdlDataField = (XPDLDataField)dataFieldMapping.get(key);

			// check the datafield definition level is package or the WFProcess
			String projectTarget=xpdlDataField.getProjectName();

//Debug
//System.out.println("createProcessProperties: xpdlDataField.getProjectName() = " + projectTarget);

			if (projectTarget.equals(projectName)||projectTarget.equals(PACKAGE_LEVEL)) {
				// check if the boolean PropertyActivity is false (not datafields for activities)
				if (!xpdlDataField.getPropertyActivity()){
						// STRING Basic Type
						if (BASICTYPE.equalsIgnoreCase(xpdlDataField.getDataType())){
							if ("String".equalsIgnoreCase(xpdlDataField.getBasicType())){
								try {
									p.setProperty(xpdlDataField.getName(), xpdlDataField.getInitialValue());
								} catch (HeroException he) {
									throw new XPDLException(he.getMessage(), he);
								}
							}
						}

						// For Enumeration type
						if (ENUMERATIONTYPE.equalsIgnoreCase(xpdlDataField.getDataType())){
							ArrayList ar = xpdlDataField.getEnumeration();
							ArrayList values = new ArrayList();
							Iterator it2 = ar.iterator();
							while (it2.hasNext()) {
									String val = (String)it2.next();
									values.add(val);
							}
//Debug
//System.out.println("createProcessProperties: create property Name = " + xpdlDataField.getName() + " Value = " + xpdlDataField.getInitialValue());
								try {
								p.setPropertyPossibleValues(xpdlDataField.getName(),values);
								if (!"".equals(xpdlDataField.getInitialValue()))
									p.setProperty(xpdlDataField.getName(),xpdlDataField.getInitialValue());
								} catch (HeroException he) {
									throw new XPDLException("Error setting the initial value of: " + xpdlDataField.getName() + "  " + he.getMessage(), he);
								}
						}
				}
			}
		}
	}

	/*
	 *   Add Initiator Mapper to Bonita project if it exists in the Workflow Process definition
	 */
	private void addProcessInitiatorMapper(BonitaTree tree, String projectName, ProjectSessionLocal p, XPDLWFProcess xpdlWFProcess) throws HeroException
	{
		if (xpdlWFProcess.getInitiatorMapper()!=null){
			XPDLInitiatorMapper xpdlInitiatorMapper = xpdlWFProcess.getInitiatorMapper();
			String s_initType = xpdlInitiatorMapper.getInitiatorMapperType();
			int i_initType = 0;
			if (EXTENDED_ATTRIBUTE_VALUE_LDAP.equalsIgnoreCase(s_initType)) i_initType = 0;
			if (EXTENDED_ATTRIBUTE_VALUE_CUSTUM.equalsIgnoreCase(s_initType)) i_initType = 1;
			p.addInitiatorMapper(xpdlInitiatorMapper.getInitiatorMapperName(), i_initType );
		}
	}

	/*
	 *   Add Process hook to Bonita project if it exists in the Workflow Process definition
	 *   Same hook events as Node hooks
	 *   replaced by the addProcessHook2  (When introduction of OnInstantiate event hook)
	 */
	private void addProcessHook(BonitaTree tree, String projectName, ProjectSessionLocal p, XPDLWFProcess xpdlWFProcess) throws HeroException
	{
		ArrayList ar = xpdlWFProcess.getXpdlHook();
		Iterator it_phook = ar.iterator();
		String hookName = "";
		while (it_phook.hasNext()){
					XPDLHook xpdlHook= (XPDLHook)it_phook.next();
					hookName = xpdlHook.getHookName();
					if (!"".equals(hookName)) {
//debug
//System.out.println(">>> Add Hook: Name = " + hookName + " EventName = " + xpdlHook.getHookEventName() + " Script = " + xpdlHook.getHookScript());

									String hookEventName ="";
									String hookEvent = "";

									if ( !"".equals(xpdlHook.getHookEventName()) ) {
											hookEventName = xpdlHook.getHookEventName();

											if (Nd.BEFORESTART.equalsIgnoreCase(hookEventName)) hookEvent = Nd.BEFORESTART;
											else if (Nd.AFTERSTART.equalsIgnoreCase(hookEventName)) hookEvent = Nd.AFTERSTART;
											else if (Nd.BEFORETERMINATE.equalsIgnoreCase(hookEventName)) hookEvent = Nd.BEFORETERMINATE;
											else if (Nd.AFTERTERMINATE.equalsIgnoreCase(hookEventName)) hookEvent = Nd.AFTERTERMINATE;
											else if (Nd.ONCANCEL.equalsIgnoreCase(hookEventName)) hookEvent = Nd.ONCANCEL;
											else if (Nd.ONREADY.equalsIgnoreCase(hookEventName)) hookEvent = Nd.ONREADY;
											else throw new HeroException("Syntax Error in Extended Attribut with Name = hook - " + hookEventName + " is Unknown hookEvent");
										}

										// By default AFTERSTART if the ext. attribute has not the complex type
									    // <HookEventName> afterStart </HookEventName>

									else {
										hookEvent = Nd.AFTERSTART;
									}
									if ( "".equals(xpdlHook.getHookScript()) ) {
										    // javaHook  with by default HookType = Constants.Hook.JAVA
											p.addHook(hookName, hookEvent, Hook.JAVA);
//debug
//System.out.println("Set JAVA Hook for Project : Name = " + xpdlWFProcess.getName() + " Hook Name = " + hookName + " Event = " + hookEvent );
										}
									else {
											// interHook  with by default HookType = Constants.Hook.BSINTERACTIVE
											String hookScript ="";
											hookScript = xpdlHook.getHookScript();

											 String script =
											 	"import hero.interfaces.BnProjectLocal;\n"
											 	+ "import hero.interfaces.BnNodeLocal;\n"
												//+ "afterStart (Object b,Object n) {\n\n\n"
												+ hookEvent + " (Object b,Object n) {\n\n\n"
												//+ "System.out.println(\"InteractiveBnNodee Hook test, node: \"+n.getName());"
												+ hookScript
												+ "}";
											 p.addInterHook(hookName,hookEvent,Hook.BSINTERACTIVE,script);
//debug
//System.out.println("Set INTER Hook for Project : Name = " + xpdlWFProcess.getName() + " Hook Name = " + hookName
//				+ " Event = " + hookEvent + " Script = \n" + script);
									}
					}
					else throw new HeroException("A hook within the project = " + projectName );
			}
	}

	/*
	 *   Add Process hook to Bonita project if it exists in the Workflow Process definition
	 *   Replace addProcessHook when introduction of OnInstantiate event hook
	 */
	private void addProcessHook2(BonitaTree tree, String projectName, ProjectSessionLocal p, XPDLWFProcess xpdlWFProcess) throws HeroException
	{
		ArrayList ar = xpdlWFProcess.getXpdlHook();
		Iterator it_phook = ar.iterator();
		String hookName = "";
		while (it_phook.hasNext()){
					XPDLHook xpdlHook= (XPDLHook)it_phook.next();
					hookName = xpdlHook.getHookName();
					if (!"".equals(hookName)) {
//debug
//System.out.println(">>> Add Hook: Name = " + hookName + " EventName = " + xpdlHook.getHookEventName() + " Script = " + xpdlHook.getHookScript());

									String hookEventName ="";
									String hookEvent = "";

									if ( !"".equals(xpdlHook.getHookEventName()) ) {
											hookEventName = xpdlHook.getHookEventName();

											if (Pj.ONINSTANTIATE.equalsIgnoreCase(hookEventName)) {
												hookEvent = Pj.ONINSTANTIATE;
												//javaHook  with by default HookType = Constants.Hook.JAVA
												p.addHook(hookName, hookEvent, Hook.JAVA);
//	debug
//System.out.println("Set JAVA Hook for Project : Name = " + xpdlWFProcess.getName() + " Hook Name = " + hookName + " Event = " + hookEvent );
											}

											else throw new HeroException("Syntax Error in Extended Attribut onto Hook Process - " + hookEventName + " is Unknown hookEvent"
													+ " within project = " + projectName);
										}

										// By default AFTERSTART if the ext. attribute has not the complex type
									    // <HookEventName> afterStart </HookEventName>

									else {
										throw new HeroException("Missing Process Hook Event within the project = " + projectName );
									}

					}
					else throw new HeroException("Missing Process Hook Name within the project = " + projectName );
			}
	}

	/*
	 *  Obtain definition for Participants, Applications, DataFields
	 *   NO MORE USED !
	 */
	private void addSubProcessDefinition(BonitaTree root, String projectName, Hashtable xpdlData) throws Exception{
		try {
			pHome = (ProjectSessionLocalHome) hero.interfaces.ProjectSessionUtil.getLocalHome();
		} catch (NamingException be) { throw new HeroException(be.getMessage(), be);}
		ProjectSessionLocal p=pHome.create();
		p.initModel(projectName);
		obtainParticipant(getNode (root, PARTICIPANTS), projectName, xpdlData);
		obtainApplication(getNode (root, APPLICATIONS), projectName, xpdlData);

//debug
//System.out.println("****!!! Calling   obtainDataField in addSubProcessDefinition - projectName = " + 	projectName );

		obtainDataField(getNode (root, DATAFIELDS),projectName, xpdlData);
	}

	/*
	 *  Get activity Set definitions
	 */
	private boolean obtainActivitySet(BonitaTree a, ProjectSessionLocal p, Hashtable xpdlData) throws XPDLException
	{
		boolean hasActivitySet=false;
		if (a!=null)
		{
			if ((a.Noeud.getNodeName()).equalsIgnoreCase(ACTIVITYSETS))
			{
				hasActivitySet=true;
				// For every existing ActivitySet
				for(int j = 0; j<a.NbFils; j++)
				{
					if ((a.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(ACTIVITYSET))
					{
						Element actSet = (Element)a.Fils[j].Noeud;
						obtainCreateActivity(getNode (a.Fils[j], ACTIVITIES), p,a,  xpdlData);
						// the search of the first and last activity of the activitySet is performed in obtainTransition()
						// Then boolean is set to true and activitySet Id is provided to the method
						obtainTransition(getNode (a.Fils[j], TRANSITIONS), p, true, actSet.getAttribute(ACTIVITYSETID), xpdlData);
					}
				}
			}
		}
		return hasActivitySet;
	}

	/*
	 *   Search for the first element with the given name
	 *   according a deep search algo
	 */
	private BonitaTree getNode (BonitaTree A, String name)
	{
		if (A.Noeud.getNodeName().equalsIgnoreCase(name)) return A;
		if (A.NbFils>0)
		{
			BonitaTree ReturnArbre = null;
			for (int i = 0; i < A.NbFils; i++)
			{
				ReturnArbre = getNode(A.Fils[i], name);
				if (ReturnArbre != null)
					return ReturnArbre;
			}
		}
		return null;
	}

	/*
	 *   Search for child element under the given tree
	 *
	 */
	private BonitaTree getFilsNode (BonitaTree A, String name)
	{
		if (A.Noeud.getNodeName().equalsIgnoreCase(name)) return A;
		if (A.NbFils>0)
		{
			BonitaTree ReturnArbre = null;
			for (int i = 0; i < A.NbFils; i++)
			{
				if ( A.Fils[i].Noeud.getNodeName().equalsIgnoreCase(name) ){
					return (A.Fils[i]);
				}
			}
		}
		return null;
	}

	private boolean projectExist(String projName, String projVersion)
	{
		try {
			BnProjectLocalHome pHome = (BnProjectLocalHome) hero.interfaces.BnProjectUtil.getLocalHome();
			// take care of the version
			pHome.findByNameVersion(projName, projVersion);
		} catch (Exception be) {return false;}
		return true;
	}

	/**
	 * printHashTables
	 * Utility method set in the bean interface in order to get debug info
	 *
	 * @ejb.permission unchecked="yes"
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/

public void printHashTables(Hashtable xpdlData) throws XPDLException{
		/*  For debug , print data of hash tables
		 *
		 * 	actMapping
		 *  activitySetMapping
		 *  partMapping
		 *  appliMapping
		 */

		System.out.println("");
		System.out.println("*-------- Debug informations ---------------------------");

		// PROCESS
		System.out.println("**** Hashtable for workflow Process: wfProcessMapping");
        //TODO
        Hashtable wfProcessMapping = (Hashtable)xpdlData.get("wfProcessMapping");
		Object[] tab = wfProcessMapping.keySet().toArray();
			for (int k=0;k<tab.length;k++){
				System.out.print("   - " + tab[k].toString());
				XPDLWFProcess xpdlWFProcess = (XPDLWFProcess)wfProcessMapping.get(tab[k]);
				// process name
				System.out.println(" " + xpdlWFProcess.getName() );
				// process hooks
				ArrayList ar = xpdlWFProcess.getXpdlHook();
				Iterator it_phook = ar.iterator();
				while (it_phook.hasNext()){
					XPDLHook xpdlHook= (XPDLHook)it_phook.next();
					System.out.println("     Hook: Name = " + xpdlHook.getHookName() + " EventName = "
							+ xpdlHook.getHookEventName() + " Script = " + xpdlHook.getHookScript());
				}
				// process initiator mapper
				XPDLInitiatorMapper xpdlInitiatorMapper= xpdlWFProcess.getInitiatorMapper();
				if (xpdlInitiatorMapper != null ) {
					System.out.println("     Initiator Mapper: Type = " + xpdlInitiatorMapper.getInitiatorMapperType() + " Name = " + xpdlInitiatorMapper.getInitiatorMapperName() );
					System.out.println("");
				}
			}

		// ACTIVITY
		System.out.println("**** HashMap for activities: actMapping");
        //TODO
        Hashtable actMapping = (Hashtable)xpdlData.get("actMapping");
		tab = actMapping.keySet().toArray();
			for (int k=0;k<tab.length;k++){
				System.out.print("   - Id=" + tab[k].toString());
				//System.out.println(" " +(String)actMapping.get(tab[k]));
				XPDLActivity xpdlActivity = (XPDLActivity)actMapping.get(tab[k]);
				// performer
				String performer = xpdlActivity.getPerformer();
				if ( "".equals(performer) ) performer = "InitialRole";
				System.out.println("   Name = " + xpdlActivity.getName()+ "  Performer = " + performer);
				// implementation
				String implementation = xpdlActivity.getImplementation();
				if ( "".equals(implementation) ){
					System.out.println("   Name = " + xpdlActivity.getName()+ "  Implementation = " + implementation);
				}
				// subflow version  (if subflow implementation)
				String subFlowVersion = xpdlActivity.getSubFlowVersion();
				if ( "".equals(subFlowVersion) ){
					System.out.println("   Name = " + xpdlActivity.getName()+ "  SubFlowVersion = " + subFlowVersion);
				}
				// Performer Assignment
				if ( !"".equals(xpdlActivity.getPerformerAssignName())){
					System.out.println("       . Performer Assignment: Name = " + xpdlActivity.getPerformerAssignName()
							+ " Type = " + xpdlActivity.getPerformerAssignType() + " Property = " + xpdlActivity.getPerformerAssignProperty() );
				}
				// array of properties and propagations
				if (!xpdlActivity.getProperties().isEmpty()){
					Iterator it_proper = xpdlActivity.getProperties().iterator();
					Iterator it_propag = xpdlActivity.getPropagations().iterator();
					while (it_proper.hasNext() && it_propag.hasNext()){
						System.out.print("       . Property: Name = " + (String) it_proper.next() );
						boolean b_propagation = ((Boolean) it_propag.next()).booleanValue();
						if (b_propagation) System.out.println(" propagation = Yes ");
						else System.out.println(" propagation = No ");
					}

				}
				// Hooks
				ArrayList ar = xpdlActivity.getXpdlHook();
				Iterator it_ahook = ar.iterator();
				while (it_ahook.hasNext()){
					XPDLHook xpdlHook= (XPDLHook)it_ahook.next();
					System.out.println("       . Hook: Name = " + xpdlHook.getHookName() + " EventName = "
							+ xpdlHook.getHookEventName() + " Script = " + xpdlHook.getHookScript());
				}
				// Relative Deadlines
				ArrayList ar_dl = xpdlActivity.getRelativeTimeDeadline();
				if (ar_dl.size()!= 0){
					Iterator it_dl = ar_dl.iterator();
					while (it_dl.hasNext()){
						Long condition = (Long)it_dl.next();
						System.out.println("       . Relative Deadline : " + condition.longValue());
					}
				}
				// Fixed Deadlines
				ar_dl = xpdlActivity.getFixedTimeDeadline();
				if (ar_dl.size()!= 0){
					Iterator it_dl = ar_dl.iterator();
					while (it_dl.hasNext()){
						Long condition = (Long)it_dl.next();
						System.out.println("       . Fixed Deadline : " + condition.longValue());
					}
				}
				// ActualParameters
				ArrayList ar_ap = xpdlActivity.getActualParameter();
				if (ar_ap.size()!= 0){
					Iterator it_ap = ar_ap.iterator();
					while (it_ap.hasNext()){
						String actualParameter = (String)it_ap.next();
						System.out.println("       . ActualParameter : " + actualParameter);
					}
				}
				// Iteration
				// old iteration printing
				/*
				String iteration = xpdlActivity.getIterationName();
				if ( !"".equals(iteration) ) {
					System.out.println("   Iteration Name = " + xpdlActivity.getName());
					//System.out.println("   Activity Name (From) = " + xpdlActivity.getName());
					System.out.println("   Iteration condition = " + xpdlActivity.getIterationCondition());
					System.out.println("   Iteration To = " + xpdlActivity.getIterationTo());
				}
				*/
				// new iteration printing
	            if (xpdlActivity.getIteration().size()!=0 ) {

	                    Hashtable iteration = (Hashtable)xpdlActivity.getIteration();
	                    Object[] tabIteration = iteration.keySet().toArray();
	                        for (int ite=0;ite<tabIteration.length;ite++){
	                            System.out.println("   Iteration To = " + tabIteration[ite].toString());
	                            System.out.println("   Iteration condition = " + (String)iteration.get(tabIteration[ite]));
	                        }
	            }
			}

		// ActivitySet
		System.out.println("");
		System.out.println("**** Hashtable for activity Set: activitySetMapping");
        //TODO
        Hashtable activitySetMapping = (Hashtable)xpdlData.get("activitySetMapping");
		tab = activitySetMapping.keySet().toArray();
			for (int y=0;y<tab.length;y++){
				System.out.print("   - " + tab[y].toString());
				System.out.println(" " +(String)activitySetMapping.get(tab[y]));
			}

		// Transitions
		System.out.println("");
		System.out.println("**** Hashtable for transition: transitionMapping");
        //TODO
        Hashtable transitionMapping = (Hashtable)xpdlData.get("transitionMapping");
		tab = transitionMapping.keySet().toArray();
			for (int y=0;y<tab.length;y++){
				//id
				System.out.print("   - " + tab[y].toString());
				// name, to, from, condition
				XPDLTransition xpdlTransition = (XPDLTransition)transitionMapping.get(tab[y]);
				System.out.println("       From =  " + xpdlTransition.getFrom() + " To = " + xpdlTransition.getTo());
				System.out.println("       Name= " + xpdlTransition.getName() + " condition = " + xpdlTransition.getCondition() );
			}

		// Participants
		System.out.println("");
		System.out.println("**** Hashtable for participant: partMapping");
//      TODO
        Hashtable partMapping = (Hashtable)xpdlData.get("partMapping");
		tab = partMapping.keySet().toArray();
			for (int y=0;y<tab.length;y++){
				System.out.print("   - " + tab[y].toString());
				XPDLParticipant xpdlPart = (XPDLParticipant)partMapping.get(tab[y]);
				System.out.println("      Name= " + xpdlPart.getName() + " Type= " + xpdlPart.getType()
				+ " RoleMapper= " + xpdlPart.getMapperName() + " RoleMapperType= " + xpdlPart.getMapperType() );
				System.out.println("      For ProjectName (or all project) = " + xpdlPart.getProjectName());
			}

		// Applications
		System.out.println("");
		System.out.println("**** Hashtable for Applications: appliMapping");
//      TODO
        Hashtable appliMapping = (Hashtable)xpdlData.get("appliMapping");
		tab = appliMapping.keySet().toArray();
			for (int y=0;y<tab.length;y++){
				//application id
				System.out.print("   - " + tab[y].toString());
				XPDLApplication xpdlAppli = (XPDLApplication)appliMapping.get(tab[y]);
				// application name
				System.out.println("  Name= " + xpdlAppli.getName() );
				// level package or project
				System.out.println("      For ProjectName (or all project) = " + xpdlAppli.getProjectName());
				// formal parameters set in hash table (key = Index, value = Id of the formal param)
				ArrayList formalParams = xpdlAppli.getFormalParam();
				if (!formalParams.isEmpty()){
					System.out.println("      FormalParameters :");
					Iterator it_fp = formalParams.iterator();
					while (it_fp.hasNext()) {
						XPDLFormalParameter xpdlFormalParam = (XPDLFormalParameter)it_fp.next();
						System.out.print("      Id = " + xpdlFormalParam.getId());
						System.out.print("\t\tIndex = " + xpdlFormalParam.getIndex());
						System.out.println("\tMode = " + xpdlFormalParam.getMode());
					}
				}
			}

		// DataFields
		System.out.println("**** Hashtable for DataField: dataFieldMapping");
//      TODO
        Hashtable dataFieldMapping = (Hashtable)xpdlData.get("dataFieldMapping");
		tab = dataFieldMapping.keySet().toArray();
			for (int y=0;y<tab.length;y++){
				System.out.println("   - " + tab[y].toString());
				XPDLDataField xpdlDataField = (XPDLDataField)dataFieldMapping.get(tab[y]);
				System.out.println("      Name= " + xpdlDataField.getName()
						+ " DataType = " + xpdlDataField.getDataType()
						+ " BasicType = " + xpdlDataField.getBasicType()
						+ " InitialValue = " + xpdlDataField.getInitialValue());
				if (xpdlDataField.getPropertyActivity()) System.out.println("      Property for activity = true");
				else System.out.println("      Property for activity = false");
				System.out.println("      For ProjectName(or all project) = " + xpdlDataField.getProjectName());
					if ("EnumerationType".equals(xpdlDataField.getDataType())){
						ArrayList ar_df = xpdlDataField.getEnumeration();
						Iterator it2 = ar_df.iterator();
						System.out.println("        Enumeration values:");
						while (it2.hasNext()) {
								String val = (String)it2.next();
								System.out.println("           . " + val);
						}
					}
			}
	}

	/*
	 * Internal utility method used for dev.
	 */
	private void printDomTree (BonitaTree Arb)
	{
		int i=0;
		String type;
		switch (Arb.Noeud.getNodeType())
		{
			case Node.ATTRIBUTE_NODE :
				type = "attribut";
			break;
			case Node.CDATA_SECTION_NODE :
				type = "CDATA"; break;
			case Node.COMMENT_NODE :
				type = "comment"; break;
			case Node.DOCUMENT_FRAGMENT_NODE :
				type = "document fragment"; break;
			case Node.DOCUMENT_NODE :
				type = "document"; break;
			case Node.DOCUMENT_TYPE_NODE :
				type = "document type"; break;
			case Node.ELEMENT_NODE :
				type = "node";
			if (Arb.Noeud.hasAttributes())
			{
				int length=Arb.Noeud.getAttributes().getLength();
				Attr attributes[] = new Attr[length];

				for (int j=0; j < length; j++)
				{
					attributes[j] = (Attr)Arb.Noeud.getAttributes().item(j);
					System.out.println("***************       Attr Name="+ attributes[j].getNodeName());
					System.out.println("***************       Attr Value="+ attributes[j].getNodeValue());
					System.out.println("----------------------------------------------------------");
				}
			}
			break;
			case Node.ENTITY_NODE :
				type = "entity"; break;
			case Node.ENTITY_REFERENCE_NODE :
				type = "entity reference"; break;
			case Node.NOTATION_NODE :
				type = "notation"; break;
			case Node.PROCESSING_INSTRUCTION_NODE :
				type = "processing instruction"; break;
			case Node.TEXT_NODE :
				type = "text"; break;
			default : type = "none";
		}
		System.out.println("type : " + type);
		System.out.println("noeud name  : "+Arb.Noeud.getNodeName());
		System.out.println("value : "+Arb.Noeud.getNodeValue());

		System.out.println("Nb Fils = "+Arb.NbFils);

		if (Arb.NbFils>0)
		{
			i=0;
			while (i<Arb.NbFils)
			{
				printDomTree (Arb.Fils[i++]);
			}
		}
	}

	/*
	 * To obtain the participant attribut definition
	 *    - Name
	 *    - Id
	 */

	private void obtainParticipant(BonitaTree tree, String projectName, Hashtable xpdlData) throws XPDLException{
		if (tree!=null)
		{
			if ((tree.Noeud.getNodeName()).equalsIgnoreCase(PARTICIPANTS))
			{
				for(int j = 0; j<tree.NbFils; j++)
				{
					if ((tree.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(PARTICIPANT))
					{
						XPDLParticipant xpdlPart = new XPDLParticipant();
						Element ele = (Element)tree.Fils[j].Noeud;
						//set the participant Id
						xpdlPart.setId(ele.getAttribute(PARTICIPANT_ID));

						//set the participant name
						if (!ele.getAttribute(PARTICIPANT_TYPE_TYPE).equalsIgnoreCase(PARTICIPANT_TYPE_SYSTEM))
							xpdlPart.setName(ele.getAttribute(PARTICIPANT_NAME));
						else {
							//partMapping.put(xpdlPart.getId(),"InitialRole");
							xpdlPart.setName("InitialRole");
						}
						//set the level (WFProcess or package) in which application is accessible : not needed !
						xpdlPart.setProjectName(projectName);

						// Put the XPDLParticipant Bean in the HashMap
                        Hashtable partMapping = (Hashtable)xpdlData.get("partMapping");
                        partMapping.put(xpdlPart.getId(),xpdlPart);

						if (!("".equalsIgnoreCase(xpdlPart.getName())))
							// get and add data for package participants
							obtainParticipantData(tree.Fils[j], xpdlPart);
					}
				}
			}
		}
	}

	/*
	 * To obtain the applications attribut definition
	 *    - Name
	 *    - Id
	 */

	private void obtainApplication(BonitaTree tree, String projectName, Hashtable xpdlData) throws XPDLException{
		if (tree!=null)
		{
			if ((tree.Noeud.getNodeName()).equalsIgnoreCase(APPLICATIONS))
			{
				for(int j = 0; j<tree.NbFils; j++)
				{
					if ((tree.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(APPLICATION))
					{
						XPDLApplication xpdlAppli = new XPDLApplication();
						Element ele = (Element)tree.Fils[j].Noeud;
						//set the application Id
						xpdlAppli.setId(ele.getAttribute(APPLICATION_ID));
						xpdlAppli.setName(ele.getAttribute(APPLICATION_NAME));

						//set the level (WFProcess or package) in which application is accessible : not needed
						xpdlAppli.setProjectName(projectName);

						// Put the XPDLApplication in the HashMap
                        Hashtable appliMapping = (Hashtable)xpdlData.get("appliMapping");
                        appliMapping.put(xpdlAppli.getId(),xpdlAppli);

						if (!"".equalsIgnoreCase(xpdlAppli.getName()))
							// get and add data for package Applications
							obtainApplicationData(tree.Fils[j], xpdlAppli);
					}
				}
			}
		}
	}

	/*
	 * To obtain the datafields attribut definition
	 *    - Name
	 *    - Id
	 */

	private void obtainDataField(BonitaTree tree, String projectName, Hashtable xpdlData) {
		if (tree!=null)
		{
			if ((tree.Noeud.getNodeName()).equalsIgnoreCase(DATAFIELDS))
			{
				for(int j = 0; j<tree.NbFils; j++)
				{
					if ((tree.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(DATAFIELD))
					{
						XPDLDataField xpdlDataField = new XPDLDataField();
						Element ele = (Element)tree.Fils[j].Noeud;
						//set the dataField: Id, name, projectName or PACKAGE_LEVEL
						xpdlDataField.setId(ele.getAttribute(DATAFIELD_ID));
						xpdlDataField.setName(ele.getAttribute(DATAFIELD_NAME));

						//set the level (WFProcess or package) in which application is accessible
						if ("".equals(xpdlDataField.getProjectName())) xpdlDataField.setProjectName(projectName);
//debug
//System.out.println("****!!!**** In obtainDataField - projectName = " + 	projectName + " datafield Name = " + xpdlDataField.getName());

						// Put the XPDLApplication in the HashMap
                        //TODO
                        Hashtable dataFieldMapping = (Hashtable)xpdlData.get("dataFieldMapping");
						dataFieldMapping.put(xpdlDataField.getId(),xpdlDataField);
						// Add the key in an ArrayList to get the same order as it is expected in the Form Ed.
                        //TODO
                        ArrayList array_dataFieldKey = (ArrayList)xpdlData.get("array_dataFieldKey");
						array_dataFieldKey.add(xpdlDataField.getId());

						if (!"".equalsIgnoreCase(xpdlDataField.getName()))
							// get and add data for package Applications
							obtainDataFieldData(tree.Fils[j], xpdlDataField);
					}
				}
			}
		}
	}

	/*
	 * To obtain participants definition:
	 *    - TYPE
	 *    - EXTENDED ATTRIBUTES
	 *    (But does not create role ou user in workflowProcess (this is done in activity when performer)
	 */
	private void obtainParticipantData(BonitaTree n, XPDLParticipant xpdlPart) throws XPDLException
	{
		int type=n.Noeud.getNodeType();
		switch (type)
		{
			case Node.ATTRIBUTE_NODE :
				break;

			case Node.ELEMENT_NODE :
				if (n.Noeud.getNodeName().equalsIgnoreCase(PARTICIPANT_TYPE))
				{

					Element ele = (Element)n.Noeud;
					//TYPE of participant
					xpdlPart.setType(ele.getAttribute(PARTICIPANT_TYPE_TYPE));
				}

				// EXTENDED ATTRIBUTES on paticipant for RoleMappers and for support to JaWE connexion to LDAP
				if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTES)) {
						for(int j = 0; j<n.NbFils; j++)
						{
							if ((n.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(EXTENDED_ATTRIBUTE))
							{
								Element ele = (Element)n.Fils[j].Noeud;
									if (ele.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_MAPPER)) {
										// construct the name of the mapper based on the roleName
										String mapperName = "mapper".concat(xpdlPart.getName());
										xpdlPart.setMapperName(mapperName);
										// LDAP
										if (ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE).equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_LDAP))
											xpdlPart.setMapperType(EXTENDED_ATTRIBUTE_VALUE_LDAP);
										// PROPERTIES
										else if ( ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE).equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_PROPERTIES) )
											xpdlPart.setMapperType(EXTENDED_ATTRIBUTE_VALUE_PROPERTIES);
										// CUSTOM
										else if ( ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE).equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_CUSTUM) )
											xpdlPart.setMapperType(EXTENDED_ATTRIBUTE_VALUE_CUSTUM);
									}
									// Classname is required for the CUSTOM Mapper (No Error raised if this extended attribute is not set !!!!
									// TBD !!
									if (ele.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_MAPPER_CLASSNAME)) {
										xpdlPart.setMapperClassName(ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
										// The mapper name is also the name of the Mapper class
										xpdlPart.setMapperName(ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
									}
									//Extended attributs supporting JaWE connexion to Ldap
									if (ele.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_DN)) {
										// Only Human type Partricipant needs to get uid from Dn (not Role type)
										if (PARTICIPANT_TYPE_HUMAN.equalsIgnoreCase(xpdlPart.getType())) {
											//get uid providing the DN to the BnLdap Bean
											BnLdapHome ldapHome = null;
											BnLdap bnLdap = null;
											String uid = "";
											try {
												ldapHome = BnLdapUtil.getHome();
											} catch (NamingException na){
												throw new XPDLException("Error creating BnLdap Home : " + na.getMessage(), na);
											}
											try {
												bnLdap = ldapHome.create();
											} catch (Exception e){
												throw new XPDLException("Error creating BnLdap : " + e.getMessage(), e);
											}
											try {
												uid = bnLdap.getUserIdFromPartialDn(ele.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
												} catch (Exception e){
													throw new XPDLException("Error getting uid with BnLdap Bean : " + e.getMessage(), e);
												}
											//replacing the participant full name by the got uid
											xpdlPart.setName(uid);
										}

									}
							}
						}
				  }

			break;
			case Node.TEXT_NODE :
				if ((n.Noeud.getNodeName()).equalsIgnoreCase(PARTICIPANT_DESCRIPTION))
					xpdlPart.setDescription(n.Noeud.getNodeValue());
			break;

			default :
				break;
		}

		for (int k=0; k<n.NbFils; k++)
		{
			obtainParticipantData(n.Fils[k], xpdlPart);
		}
		//return xpdlPart;
	}

	/*
	 *    Get activity definition and create the Bonita nodes with all components (deadlins, hooks, ....)
	 */
	private void obtainCreateActivity(BonitaTree a, ProjectSessionLocal p, BonitaTree root, Hashtable xpdlData) throws XPDLException
	{

		if (a!=null)
		{
			BonitaTree activity_type=null;
			String Sub_Id;
			if ((a.Noeud.getNodeName()).equalsIgnoreCase(ACTIVITIES))
			{
				for (int i = 0; i < a.NbFils; i++)
				{
					if ((a.Fils[i].Noeud.getNodeName()).equalsIgnoreCase(ACTIVITY))
					{
						XPDLActivity xpdlAct = new XPDLActivity();
						Element element = (Element)a.Fils[i].Noeud;
						// set Id and Name of the activity in xpdlAct Bean
						xpdlAct.setId(element.getAttribute(ACTIVITY_ID));
						xpdlAct.setName(element.getAttribute(ACTIVITY_NAME));

						// store xpdlAct Bean in a global hashtable (to be used for addNode, addedge , .....)
                        //TODO
                        Hashtable actMapping = (Hashtable)xpdlData.get("actMapping");
						actMapping.put(element.getAttribute(ACTIVITY_ID), xpdlAct);


						//*****************
						// SUBFLOW Activity
						//*****************
						if ((activity_type=getNode(a.Fils[i], IMPLEMENTATION_SUBFLOW))!=null)
						{
							String projectName = null;
                            String defaultProjectName = null;
                            String projectId = null;
							String projectVersion = null;
							xpdlAct.setImplementation(IMPLEMENTATION_SUBFLOW);
							Element element2 = (Element)activity_type.Noeud;
                            defaultProjectName = element2.getAttribute(SUBFLOW_ID);
                            /*  get the process name :
                             *      - either in the wfProcessMapping if it has been already found in an defined external package
                             *        (xpdl generated by other tools than proed and imported with bonita ant target:
                             *        ant import-xpdl -Duser=xxx -Dpasswd=yyy -Dxpdl=<xpdl full path>)
                             *      - or in the extended attribute from xpdl/proed
                             *        <ExtendedAttribute Name="SubFlowName" Value="My Subflow" />
                             *  By default projectName = projectId if it has not been found thru the 2 previous cases
                             */

                            // search if the subflow exists in the defined external package
                            projectId = element2.getAttribute(SUBFLOW_ID);
                            //TODO
                            Hashtable wfProcessMapping = (Hashtable)xpdlData.get("wfProcessMapping");
                            XPDLWFProcess xpdlWFProcess = (XPDLWFProcess)wfProcessMapping.get(projectId);
                            if ( xpdlWFProcess != null) {
                                projectName = xpdlWFProcess.getName();
                            }
                            // search if the name is defined in an extended attribut
                            else {
                                xpdlAct = obtainSubFlowName(a.Fils[i], xpdlAct);
                                projectName = xpdlAct.getSubFlowName();
                            }
                            if ("".equals(projectName) ){
                                projectName = defaultProjectName;
                            }

							// Search for sub-process definition inside this package and then create it
							// With actual designer (proed), the definition of the sub-process is never in the same package
							// as the main process (decision to be not full xpdl compliant with proed)
							// But keep this fonction in the import code if this changes in the FUTUR
                            /*
                             * projectName = getCreateSubProject(Sub_Id,root);
                             */

							// Get subflow version specified within an extended attribut
							xpdlAct = obtainSubFlowVersion(a.Fils[i], xpdlAct);
							projectVersion = xpdlAct.getSubFlowVersion();
//debug
//System.out.println("	******** SubFlowversion = " + projectVersion);
							if (projectName != null && projectVersion != null){
								if (projectExist(projectName, projectVersion)){

									//get all the XPDL elements of this activity
									xpdlAct=obtainActivityData(a.Fils[i], p, xpdlAct,  xpdlData);
		//debug
		//System.out.println("Adding subflow : xpdlAct.getName() = " + xpdlAct.getName() + " projectName = " + projectName);
										//if (projectName != null)
											try {
												p.addNodeSubProcess(xpdlAct.getName(), projectName, projectVersion);
												} catch (HeroException he) {
													he.printStackTrace();
													throw new XPDLException("Error adding SubProcess Node " + he.getMessage() + " ", he);
												}
										/*
										else {
										// case projectName == null
											try {
												if (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
													p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);
												else p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);

												//Set node non anticipable
												p.setNodeTraditional(xpdlAct.getName());
											} catch (HeroException he) {
												throw new XPDLException("Error adding SubProcess Node with projectName == null " + he.getMessage() + " ", he);
											}
										}
										*/
								}// end project exist
								else {
									// project has not been found in the engine database
									// project is not searched into the the current or external packages !!
									throw new XPDLException("Error : the process = " + projectName  + " with version = " + projectVersion + " used as sub process has not been found in the Workflow engine database");
								}
							}
							// projectName or projectVersion is null
							else{
									throw new XPDLException("Error: Wrong SubProcess Id syntax");
								}

						}

						//***************************
						// NO implementation Activity
						//***************************
						else if ((activity_type=getNode(a.Fils[i], IMPLEMENTATION_NO))!=null)
						{
							// get all the XPDL elements of this activity
							xpdlAct=obtainActivityData(a.Fils[i], p, xpdlAct, xpdlData);
							xpdlAct.setImplementation(IMPLEMENTATION_NO);

							try {
							if (xpdlAct.getMode().equalsIgnoreCase(MANUAL))
							{
								if   (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
									p.addNode(xpdlAct.getName(), Nd.OR_JOIN_NODE);
								else
									p.addNode(xpdlAct.getName(), Nd.AND_JOIN_NODE);
							}
							else if (xpdlAct.getMode().equalsIgnoreCase(AUTOMATIC))
							{
								if (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
									p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);
								else
									p.addNode(xpdlAct.getName(), Nd.AND_JOIN_AUTOMATIC_NODE);
							}
							// Set node non anticipable
							p.setNodeTraditional(xpdlAct.getName());

							} catch (HeroException he) {
								throw new XPDLException(he.getMessage(), he);
							}
							/*
							 *  Extended Bonita properties within activity
							 */

//debug
//System.out.println("Set the properties for Activity: " +	xpdlAct.getName());
							/*  properties ListArray contains id of the datafield (key of the Hashtable)
							 * A remplacer par la cration de l'un de ces type
							 */
							if (!xpdlAct.getProperties().isEmpty()){
								// get also the collection the propagation
//debug
//System.out.println(" xpdlAct.getProperties() is not empty");
								ArrayList ar_properties = xpdlAct.getProperties();

								//get also the collection the propagation
								ArrayList ar_propagations = xpdlAct.getPropagations();
								Iterator it_propagate = ar_propagations.iterator();

								Iterator it_prop = ar_properties.iterator();
								while (it_prop.hasNext()&& it_propagate.hasNext()){
									// get the datafield with the Id
                                    //TODO
                                    Hashtable dataFieldMapping = (Hashtable)xpdlData.get("dataFieldMapping");
									XPDLDataField xpdlDataField = (XPDLDataField)dataFieldMapping.get(it_prop.next());

									// get the propagation value: true or false
									boolean b_propagation = ((Boolean) it_propagate.next()).booleanValue();

									// STRING
									if (BASICTYPE.equalsIgnoreCase(xpdlDataField.getDataType())){
										if ("String".equalsIgnoreCase(xpdlDataField.getBasicType())){
											try {
												p.setNodeProperty(xpdlAct.getName(), xpdlDataField.getName(), xpdlDataField.getInitialValue(), b_propagation);
											} catch (HeroException he) {
												throw new XPDLException(he.getMessage(), he);
											}
										}
									}
									// ENUMERATION
									if (ENUMERATIONTYPE.equalsIgnoreCase(xpdlDataField.getDataType())){
										ArrayList ar = xpdlDataField.getEnumeration();
										ArrayList values = new ArrayList();
										Iterator it2 = ar.iterator();
										while (it2.hasNext()) {
												String val = (String)it2.next();
												values.add(val);
										}
//Debug
//System.out.println("createNodeProperties: create property Name = " + xpdlDataField.getName() + " Value = " + xpdlDataField.getInitialValue());
											//set the enum property
											try {
											p.setNodePropertyPossibleValues(xpdlAct.getName(), xpdlDataField.getName(),values);
											if (!"".equals(xpdlDataField.getInitialValue()))
												p.setNodeProperty(xpdlAct.getName(), xpdlDataField.getName(),xpdlDataField.getInitialValue(), b_propagation);
											} catch (HeroException he) {
												throw new XPDLException("Error setting the initial value of: " + xpdlDataField.getName() + "  " + he.getMessage(), he);
											}
									}
								}
							}

							/*   >>>>>>>>>>>>>>>>>>>>>
							 *   >>>>> DEADLINES <<<<<
							 *   <<<<<<<<<<<<<<<<<<<<<
							 */

							if (xpdlAct.getDeadline()) {
								// conditions could be either relative time:    in milliseconds (ex.: 60000)
								//                   or fixed time:      yy/MM/dd/hh/mm/ss (ex. 2005/01/21/14/00/00)
								// Separate the 2 types	of deadlines into 2 separate arraylists
								ArrayList condition =  xpdlAct.getDeadlineCondition();
								String regex = "/";
								Iterator it = condition.iterator();
								String[] tab;
								while(it.hasNext()){
									String cond = (String)it.next();
									tab = cond.split(regex);
									if (tab.length > 1){
										int year = (new Integer(tab[0])).intValue();
										int month = (new Integer(tab[1])).intValue();
										int date = (new Integer(tab[2])).intValue();
										int hour = (new Integer(tab[3])).intValue();
										int minute = (new Integer(tab[4])).intValue();
										int second = (new Integer(tab[5])).intValue();

										GregorianCalendar gc = new GregorianCalendar(year,month,date,hour,minute,second);
										long t = gc.getTimeInMillis();
										String time_s = (new Long(t)).toString();
										xpdlAct.setFixedTimeDeadline(time_s);
//debug
//System.out.println("Set Fixed time deadline with time = " + time_s );
									}
									else if (tab.length == 1) {
										xpdlAct.setRelativeTimeDeadline(cond);

//System.out.println("Set Relative deadline with time = " + cond);
									}
									else {
//debug
//System.out.println("the following deadline condition is not correct: " + cond);
										// raise exception ??
									}
								}
								try {
								// set Node relative deadline if it exists
								ArrayList fixedTimeDeadlines = xpdlAct.getFixedTimeDeadline();
								p.setNodeDeadlines(xpdlAct.getName(),fixedTimeDeadlines);
								// set Node fixed deadline if it exists
								ArrayList relativeTimeDeadlines = xpdlAct.getRelativeTimeDeadline();
								p.setNodeRelativeDeadlines(xpdlAct.getName(),relativeTimeDeadlines);

								//add Node Hook
								String exception = xpdlAct.getExceptionName();
								// Not used at now ArrayList execution =  xpdlAct.getDeadlineExecution();
								p.addNodeHook(xpdlAct.getName(),exception,hero.interfaces.Constants.Nd.ONDEADLINE, Hook.JAVA);
								} catch (HeroException he) {
									throw new XPDLException(he.getMessage(), he);
								}
							} // end deadlines

							/*   >>>>>>>>>>>>>>>>
							 *   >>>>> HOOK <<<<<
							 *   <<<<<<<<<<<<<<<<
							 */

					 		ArrayList ar = xpdlAct.getXpdlHook();
							Iterator it_phook = ar.iterator();
							String hookName = "";
							XPDLHook xpdlHook = null;
							while (it_phook.hasNext()){
								xpdlHook= (XPDLHook)it_phook.next();
								hookName = xpdlHook.getHookName();
								if (!"".equals(hookName)) {
//debug
//System.out.println(">>> Add Hook to activity : " + xpdlAct.getName() + "  HookName = " + hookName + " EventName = " + xpdlHook.getHookEventName() + " Script = " + xpdlHook.getHookScript());

														String hookEventName ="";
														String hookEvent = "";

														if ( !"".equals(xpdlHook.getHookEventName()) ) {
																hookEventName = xpdlHook.getHookEventName();

																if (Nd.BEFORESTART.equalsIgnoreCase(hookEventName)) hookEvent = Nd.BEFORESTART;
																else if (Nd.AFTERSTART.equalsIgnoreCase(hookEventName)) hookEvent = Nd.AFTERSTART;
																else if (Nd.BEFORETERMINATE.equalsIgnoreCase(hookEventName)) hookEvent = Nd.BEFORETERMINATE;
																else if (Nd.AFTERTERMINATE.equalsIgnoreCase(hookEventName)) hookEvent = Nd.AFTERTERMINATE;
																else if (Nd.ONCANCEL.equalsIgnoreCase(hookEventName)) hookEvent = Nd.ONCANCEL;
																else if (Nd.ONREADY.equalsIgnoreCase(hookEventName)) hookEvent = Nd.ONREADY;
																else throw new XPDLException("Syntax Error in Extended Attribut with Name = hook - " + hookEventName + " is Unknown hookEvent");
														}

															// By default AFTERSTART if the ext. attribute has not the complex type
														    // <HookEventName> afterStart </HookEventName>

														else {
															hookEvent = Nd.AFTERSTART;
														}





														if ( "".equals(xpdlHook.getHookScript()) ) {
															// javaHook  with by default HookType = Constants.Hook.JAVA
															try {
															p.addNodeHook(xpdlAct.getName(), xpdlHook.getHookName(), hookEventName, Hook.JAVA);
															} catch (HeroException he) {
																throw new XPDLException(he.getMessage(), he);
															}
//debug
//System.out.println("Set JAVA Hook for Activity : Name = " + xpdlAct.getName() + " Hook Name = " + xpdlHook.getHookName() + " Event = " + hookEventName );
														}
														else {
															// interHook  with by default HookType = Constants.Hook.BSINTERACTIVE
															String hookScript ="";
															hookScript = xpdlHook.getHookScript();

															String script =
																"import hero.interfaces.BnProjectLocal;\n"
																+ "import hero.interfaces.BnNodeLocal;\n"
																//+ "afterStart (Object b,Object n) {\n\n\n"
																+ hookEventName + " (Object b,Object n) {\n\n\n"
																//+ "System.out.println(\"InteractiveBnNodee Hook test, node: \"+n.getName());"
																+ hookScript
																+ "}";
															try {
															p.addNodeInterHook(xpdlAct.getName(),xpdlHook.getHookName(),hookEventName,Hook.BSINTERACTIVE,script);
															} catch (HeroException he) {
																throw new XPDLException(he.getMessage(), he);
															}
//debug
//System.out.println("Set INTER Hook for Activity : Name = " + xpdlAct.getName() + " Hook Name = " + xpdlHook.getHookName()
//		+ " Event = " + hookEventName + " Script = \n" + script);
														}
									} // end if
								    else throw new XPDLException("A Hook for activity : " + xpdlAct.getName() + " does not have HookName!");

								} //end while

								/*   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
								 *   >>>>> PERFORMER ASSIGNMENT <<<<<
								 *   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
								 */
								if ( !"".equals(xpdlAct.getPerformerAssignName()) ) {
									String perfAssignType = xpdlAct.getPerformerAssignType();
									int pAType = PerformerAssigment.CALLBACK;
//debug
//System.out.println("obtainCreateActivity: perfAssignType = " + perfAssignType );

									if (perfAssignType.equalsIgnoreCase(PERFORMER_ASSIGN_PROPERTY)) pAType = PerformerAssigment.PROPERTIES;
									else if (perfAssignType.equalsIgnoreCase(PERFORMER_ASSIGN_CALLBACK)) pAType = PerformerAssigment.CALLBACK;
//debug
//System.out.println("obtainCreateActivity/addNodePerformerAssign : name = " + xpdlAct.getPerformerAssignName() + " type = "
//		+ pAType + " property = " + xpdlAct.getPerformerAssignProperty());
									try {
									p.addNodePerformerAssign(xpdlAct.getName(), xpdlAct.getPerformerAssignName(),pAType,xpdlAct.getPerformerAssignProperty());
									} catch (HeroException he) {
										throw new XPDLException(he.getMessage(), he);
									}
								}

						} // end NO

						//*****************
						//	TOOL Activity
						//*****************
						else if ((activity_type=getNode(a.Fils[i], TOOL))!=null)
						{
							Element tool = (Element)activity_type.Noeud;
							String appId=tool.getAttribute(APPLICATION_ID);

							//get all the XPDL elements of this activity (and specifically: application Id, Actual to formal mapping)
							xpdlAct=obtainActivityData(a.Fils[i], p, xpdlAct, xpdlData);
							xpdlAct.setImplementation(IMPLEMENTATION_TOOL);

							try {
							if   (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
								p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);
							else
								p.addNode(xpdlAct.getName(), Nd.AND_JOIN_AUTOMATIC_NODE);

							// Set node non anticipable
							p.setNodeTraditional(xpdlAct.getName());
							} catch (HeroException he) {
								throw new XPDLException(he.getMessage(), he);
							}
							//TODO
                            Hashtable appliMapping = (Hashtable)xpdlData.get("appliMapping");
							XPDLApplication xpdlAppli = (XPDLApplication)appliMapping.get(appId);
							/*
							 * check the Application definition level is package or the given WFProcess
							 */
							String projectTarget=xpdlAppli.getProjectName();

								String applicationId = xpdlAppli.getId();
								String applicationNameMethode = xpdlAppli.getName();
								// get the application class by removing ".methode" from applicationNameMethode
								int index = 1;
								//Get applicationName from applicationNameMethode
								String applicationName = "";
								String[] tab;
								tab = applicationNameMethode.split("\\.");
//debug
//System.out.println("Java static methode : " + applicationNameMethode);
								for (int j=0;j<tab.length-1;j++){
//debug
//System.out.println("tab[" + j + "]= " + tab[j] );
									if (j!= tab.length-2)
										applicationName = applicationName + tab[j] +".";
									else
										applicationName = applicationName + tab[j];
								}

								//Get ActualParameters values and build the signature of the application with the these values
								ArrayList ar_ap = xpdlAct.getActualParameter();
								String signature = "";
								int nb = 0;
								String actualParameterValue = null;
								if (ar_ap.size()!= 0){
									Iterator it_ap = ar_ap.iterator();
									BnProjectPropertyValue bnProjectPropertyValue = null;
									while (it_ap.hasNext()){
										String actualParameterId = (String)it_ap.next();
										//System.out.println("       . ActualParameter : " + actualParameter);
										// actual parameter Name
                                        //TODO
                                        Hashtable dataFieldMapping = (Hashtable)xpdlData.get("dataFieldMapping");
										XPDLDataField xpdlDataFieldActualParam = (XPDLDataField)dataFieldMapping.get(actualParameterId);
										String actualParameterName = xpdlDataFieldActualParam.getName();
//debug
//System.out.println("ObtainCreateActivity : actualParameterName = " + 	actualParameterName);

										try {
											bnProjectPropertyValue = p.getProperty(actualParameterName);
										} catch (HeroException he) {
											throw new XPDLException("Error getting bnNodePropertyValue: " + he.getMessage(), he);
										}
										try {
										actualParameterValue = bnProjectPropertyValue.getTheValue();
										} catch (Exception e) {
											throw new XPDLException("Error getting actualParameterValue for : " + actualParameterName + "  " + e.getMessage(), e);
										}
										// Construct the signature of the application with the values of actualParameters
										// ex. of signature got: "toto.titi@bull.net","essai","ceci est un essai"
										// We have done the hypothesis:
										//    - no check on dataType => risks at execution time
										//    - the order of the formal parameters is the same as the actual one :
										//      This is implicitly done by JaWE in the mapping screen
										//      The Index , Id for Formal parameters has no role !!!
										//    - We do not check the mode which is implicitly IN !!
										if (nb == ar_ap.size()-1 )
										signature = signature + "\"" + actualParameterValue + "\"";
										else {
											signature = signature + "\"" + actualParameterValue + "\"" + ",";
											nb++;
										}
									}
//debug
//System.out.println("ObtainCreateActivity: application signature = " + signature );

								}

								String script ="import hero.interfaces.*;\n"
								 	+ "import hero.interfaces.BnNodeLocal;\n"
								 	+ "beforeTerminate (Object b,Object n) {\n"
								    + "import " + applicationName + ";\n";

// debug
//script = script + "System.out.println(\"interactive hook executed !!!\");\n";

								script = script + applicationNameMethode
								          + "(" + signature + ");}" ;
								try {
								p.addNodeInterHook(xpdlAct.getName(),"Hook_" + appId , Nd.BEFORETERMINATE,Hook.BSINTERACTIVE,script);
								} catch (HeroException he) {
									throw new XPDLException(he.getMessage(), he);
								}

//debug
// System.out.println("Set Tool for activity = " + xpdlAct.getName() + " InterHookName = Hook_" + appId);
						}

						//*****************
						// ROUTE Activity
						//*****************
						else if ((activity_type=getNode(a.Fils[i], ROUTE))!=null)
						{
							//get all the XPDL elements of this activity
							xpdlAct=obtainActivityData(a.Fils[i], p, xpdlAct, xpdlData);
							try {
							if   (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
								p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);
							else
								p.addNode(xpdlAct.getName(), Nd.AND_JOIN_AUTOMATIC_NODE);

							// Set node non anticipable
							p.setNodeTraditional(xpdlAct.getName());
							} catch (HeroException he) {
								throw new XPDLException(he.getMessage(), he);
							}
						}

						//*****************
						//	BLOCK Activity
						//*****************

						else if ((activity_type=getNode(a.Fils[i], BLOCK))!=null)
						{
							try {
								Element ele = (Element)activity_type.Noeud;
								String blockId=ele.getAttribute(BLOCK_ID);

								//get all the XPDL elements of this activity
								xpdlAct=obtainActivityData(a.Fils[i], p, xpdlAct, xpdlData);

								if (xpdlAct.getMode().equalsIgnoreCase(MANUAL))
								{
									if   (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
										p.addNode(xpdlAct.getName(), Nd.OR_JOIN_NODE);
									else
										p.addNode(xpdlAct.getName(), Nd.AND_JOIN_NODE);
								}
								else if (xpdlAct.getMode().equalsIgnoreCase(AUTOMATIC))
								{
									if (xpdlAct.getJoin().equalsIgnoreCase(MODE_TYPE_XOR))
										p.addNode(xpdlAct.getName(), Nd.OR_JOIN_AUTOMATIC_NODE);
									else
										p.addNode(xpdlAct.getName(), Nd.AND_JOIN_AUTOMATIC_NODE);
								}
								// Set node non anticipable
								p.setNodeTraditional(xpdlAct.getName());

								connectBlock(xpdlAct.getName(),blockId,p, xpdlData);
							} catch (HeroException he) {
								throw new XPDLException(he.getMessage(), he);
							}
						}
						//****************************************
						//	Other attributs for all activity types:
						//     - description
						//     - performer & role mapper
						//     - iteration
						//****************************************

						// *******   set the description of the activity *********
						//p.setNodeDescription((String)actMapping.get(xpdlAct.getId()), xpdlAct.getDescription());
						try {
						p.setNodeDescription(xpdlAct.getName(), xpdlAct.getDescription());
						} catch (HeroException he) {
							throw new XPDLException(he.getMessage(), he);
						}

						// set the PERFORMER of the activity if it exists
						String participant_type = null;
						if (!("".equalsIgnoreCase(xpdlAct.getPerformer()))) {
							//try {
								//get XPDLParticipant Bean with the Id
                                //TODO
                                Hashtable partMapping = (Hashtable)xpdlData.get("partMapping");
								XPDLParticipant xpdlPart = (XPDLParticipant)partMapping.get(xpdlAct.getPerformer());
								// Get the participant type
								if (xpdlPart != null){
									participant_type = xpdlPart.getType();
								}
								else{
									throw new XPDLException("Error: participant definition not defined for the performer = " + xpdlAct.getPerformer() );
								}

								// performer == HUMAN  => add user to the project
								// This will be also an role that will be added to the activity
								if (participant_type.equalsIgnoreCase(PARTICIPANT_TYPE_HUMAN)) {
									// test if the user already exists in the project
									try {
										Collection co = p.getUsers();
										Iterator it = co.iterator();
										ArrayList ar = new ArrayList();
										while (it.hasNext()){
											//BnUserLocal user = (BnUserLocal)it.next();
											String userName = (String)it.next();
											//ar.add(user.getName());
											ar.add(userName);
										}

										if ( !ar.contains(xpdlPart.getName())){
											p.addUser(xpdlPart.getName());
										}
									} catch (HeroException he) {
										throw new XPDLException(he.getMessage(), he);
									}
								}

								// performer == (ROLE || OU)
								else if ( participant_type.equalsIgnoreCase(PARTICIPANT_TYPE_ROLE)||participant_type.equalsIgnoreCase(PARTICIPANT_TYPE_ORGANIZATIONAL_UNIT)){
									try {
										//test if the role already exists in the project
										Collection co = p.getRoles();
										Iterator it = co.iterator();
										ArrayList ar = new ArrayList();
										while (it.hasNext()){
											BnRoleLocal role = (BnRoleLocal)it.next();
											ar.add(role.getName());
										}
										if ( !ar.contains(xpdlPart.getName())){
											p.addRole(xpdlPart.getName(),xpdlPart.getDescription());
										}
									} catch (HeroException he) {
										throw new XPDLException(he.getMessage(), he);
									}
									/*
									 *  adding role mapper if extended attributs for role mapping
									 */
									if ( !"".equals(xpdlPart.getMapperName()) ){
										try {
											if (xpdlPart.getMapperType().equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_LDAP)) {
											   	p.addRoleMapper(xpdlPart.getName(),xpdlPart.getMapperName(),Mapper.LDAP);
											}
											else if ( xpdlPart.getMapperType().equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_PROPERTIES) ){
												p.addRoleMapper(xpdlPart.getName(),xpdlPart.getMapperName(),Mapper.PROPERTIES);
											}
											else if ( xpdlPart.getMapperType().equalsIgnoreCase(EXTENDED_ATTRIBUTE_VALUE_CUSTUM) ){
												//in this case MapperClassName = MapperName
												p.addRoleMapper(xpdlPart.getName(),xpdlPart.getMapperName(),Mapper.CUSTOM);
											}
										} catch (HeroException he) {
											throw new XPDLException("Error setting RoleMappper for Role = " + xpdlPart.getName() + " " + he.getMessage(), he);
										}
									}
								}
								// set the role for the Node (activity) except for the Role : InitialRole (case of performer type = SYSTEM)
								if (!participant_type.equalsIgnoreCase(PARTICIPANT_TYPE_SYSTEM)){
//debug
//System.out.println("Set the role: " + xpdlPart.getName() + " to activity: " + xpdlAct.getName());
									try {
										//p.setNodeRole((String)actMapping.get(xpdlAct.getId()), xpdlPart.getName() );
										p.setNodeRole(xpdlAct.getName(), xpdlPart.getName() );
									} catch (HeroException he) {
										throw new XPDLException(he.getMessage(), he);
									}
								}
						}// end set performer
					}
				}
			}
		}
	}

	/*
	 * Connect Block Activity with its corresponding activity set
	 */
	private void connectBlock(String name, String blockId, ProjectSessionLocal p, Hashtable xpdlData) throws HeroException
	{
		try {
        //TODO
        Hashtable activitySetMapping = (Hashtable)xpdlData.get("activitySetMapping");
		String block=(String)activitySetMapping.get(blockId);
		int index = block.indexOf(ACT_SET_MAP_SEPARATOR);
		String actLast = block.substring(0,index);
		String actFirst = block.substring(index+ACT_SET_MAP_SEPARATOR_LENGTH);
		// Transition between block activity and the first one of the activitySet
		p.addEdge(name,actFirst);
		// add iteration between the last activity of the activitySet and the BlockActivity
		p.addIteration(actLast,name,"false");
		} catch (HeroException he)
		{
			throw new HeroException("Error in connectBlock:  " + he.getMessage(),he);
		}

		catch (Exception e)
		{
			throw new HeroException("Error in connectBlock:  " + e.getMessage(),e);
		}
	}

	/*
	 *   Obtain attributes for Application:
	 *      - Formal parameters
	 */
	private XPDLApplication obtainApplicationData(BonitaTree n, XPDLApplication xpdlAppli) throws XPDLException
	{

		int type=n.Noeud.getNodeType();
		switch (type)
		{
			case Node.ATTRIBUTE_NODE :
				break;

			case Node.ELEMENT_NODE :
				if (n.Noeud.getNodeName().equalsIgnoreCase(FORMAL_PARAMETERS)) {
						for(int j = 0; j<n.NbFils; j++)
						{
							if ((n.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(FORMAL_PARAMETER))
							{
								Element ele = (Element)n.Fils[j].Noeud;
								//set XPDLFormalParameter Bean into xpdlAppli Bean
								XPDLFormalParameter xpdlFormalParam = new XPDLFormalParameter();
								xpdlFormalParam.setId(ele.getAttribute(FORMAL_PARAMETER_ID));
								xpdlFormalParam.setIndex(ele.getAttribute(FORMAL_PARAMETER_INDEX));
								xpdlFormalParam.setMode(ele.getAttribute(FORMAL_PARAMETER_MODE));
								xpdlAppli.setFormalParam(xpdlFormalParam);
							}
						}
				  }
			break;

			default :
				break;
		}

		for (int k=0; k<n.NbFils; k++)
		{
			obtainApplicationData(n.Fils[k], xpdlAppli);
		}
		return xpdlAppli;
	}

		/*
		 *   Obtain attributes for DataField:
		 *      - dataType
		 * 		- basicType
		 * 		- initial value
		 */
		private XPDLDataField obtainDataFieldData(BonitaTree n, XPDLDataField xpdlDataField)
		{
			int type=n.Noeud.getNodeType();
			switch (type)
			{
				case Node.ATTRIBUTE_NODE :
					break;

				case Node.ELEMENT_NODE :
					// get BasicType of Type STRING
					if ((n.Noeud.getNodeName().equalsIgnoreCase(BASICTYPE)))
					{
									xpdlDataField.setDataType(n.Noeud.getNodeName());
									Element ele = (Element)n.Noeud;
									xpdlDataField.setBasicType(ele.getAttribute(DATATYPE_TYPE));
					}
					else if ((n.Noeud.getNodeName().equalsIgnoreCase(ENUMERATIONTYPE)))
					{
					// Get EnumerationType
						xpdlDataField.setDataType(n.Noeud.getNodeName());
						for(int j = 0; j<n.NbFils; j++)
						{
							if ((n.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(ENUMERATIONVALUE))
							{
								Element ele = (Element)n.Fils[j].Noeud;
								xpdlDataField.setEnumeration(ele.getAttribute(ENUMERATIONVALUENAME));
							}
						}
					}
					else if ((n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTES)))
					{
						for(int j = 0; j<n.NbFils; j++){
							if ((n.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(EXTENDED_ATTRIBUTE))
							{
								Element ele = (Element)n.Fils[j].Noeud;
								// set the boolean "propertyActivity" to true (means this property is reserved for activity)
								if (ele.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_PROPERTY_ACTIVITY)){
//debug
//System.out.println("obtainDataFieldData: set activityproperty for property = " + xpdlDataField.getName() );
									xpdlDataField.setPropertyActivity();
								}
        					}
						}
					}
				break;

				case Node.TEXT_NODE :
					// Get initial Value attribut
					if ((n.Noeud.getParentNode().getNodeName()).equalsIgnoreCase(INITIALVALUE))
					{
						xpdlDataField.setInitialValue(n.Noeud.getNodeValue());

					}
				break;

				default :
					break;
			}

			for (int k=0; k<n.NbFils; k++)
			{
				obtainDataFieldData(n.Fils[k], xpdlDataField);
			}
			return xpdlDataField;
		}

		/*
		 *   Get the hook if it exists from External Attribute Bonita tree
		 *   giving the xpdlWFProcess bean
		 *
		 */
		private XPDLWFProcess obtainProcessHook(BonitaTree n, XPDLWFProcess xpdlWFProcess, XPDLHook xpdlHook)
		{
			int type = n.Noeud.getNodeType();
			switch (type) {
				case Node.ATTRIBUTE_NODE :
					break;
				case Node.ELEMENT_NODE :

					// EXTENDED ATTRIBUTE : Properties & Hooks
					if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE))
					{
						Element element = (Element)n.Noeud;
						// Ex. for Process HOOKS
						/*
						 *   based on :
						 *    <ExtendedAttribute
						 *             Name="hook" Value="hero.hook.TestHook">
						 *    </ExtendedAttribute>
						 *
						 *    And optionaly other complexe type can be added:
		                 *        <HookEventName>
		                 *           BEFORETERMINATE
		                 *        </HookEventName>
		                 *
		                 * 	      <HookScript>
		                 *           System.out.println("interactive hook executed !!!");
		                 *        </HookScript>
		                 *
						 */
						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_HOOK)) {
							//xpdlHook = new XPDLHook();
							xpdlHook.setHookName(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							xpdlWFProcess.setXpdlHook(xpdlHook);
//debug
//System.out.println("obtainProcessHook: Bonita Project = " + xpdlWFProcess.getName() + " extended attribut Name = Hook  Value =  "
//	+ xpdlHook.getHookName());
						}

						else if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_INITIATORMAPPER)) {
							//xpdlHook = new XPDLHook();
							xpdlHook.setHookName(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							xpdlWFProcess.setXpdlHook(xpdlHook);

//debug
//System.out.println("obtainProcessHook: Bonita Project = " + xpdlWFProcess.getName() + " extended attribut Name = InitiatorMapper  Value =  "
//	+ xpdlHook.getHookName());
						}

					}
				break;

				case Node.TEXT_NODE :
					// Hook Event Name
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(HOOKEVENTNAME)){
						xpdlHook.setHookEventName(n.Noeud.getNodeValue());
//debug
//System.out.println("obtainProcessHook: Bonita Project = " + xpdlWFProcess.getName() + " Hook EventName = " +  xpdlHook.getHookEventName());
					}
					// Hook Script
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(HOOKSCRIPT)){
						xpdlHook.setHookScript(n.Noeud.getNodeValue());
//debug
//System.out.println("obtainProcessHook: Bonita Project = " + xpdlWFProcess.getName() + " Hook Script = \n" +  xpdlHook.getHookScript());
					}
				break;

				default :
					break;
			}
			for (int i=0; i<n.NbFils; i++)
			{
				obtainProcessHook(n.Fils[i], xpdlWFProcess,xpdlHook);
			}
			return xpdlWFProcess;
		}

		/*
		 *   Get the subflow version from External Attribute of the activity with
		 *   SubFlow implementation
		 *
		 */
		private XPDLActivity obtainSubFlowVersion(BonitaTree n, XPDLActivity xpdlAct)
		{
			int type = n.Noeud.getNodeType();
			switch (type) {
				case Node.ATTRIBUTE_NODE :
					break;
				case Node.ELEMENT_NODE :

					// EXTENDED ATTRIBUTE
//debug
//System.out.println("**** obtainSubFlowVersion: node name = " + n.Noeud.getNodeName())	;

					if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE))
					{
						Element element = (Element)n.Noeud;
						// Ex. for subFlow version
						/*
						 *   <ExtendedAttribute Name="SubFlowVersion" Value="1.0" />
						 */
						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_SUBFLOW_VERSION)) {
							xpdlAct.setSubFlowVersion(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							break;
						}

					}

				default :
					break;
			}
			for (int i=0; i<n.NbFils; i++)
			{
				obtainSubFlowVersion(n.Fils[i], xpdlAct);
			}
			return xpdlAct;
		}

        /*
         *   Get the subflow name from External Attribute of the activity with
         *   SubFlow implementation
         *
         */
        private XPDLActivity obtainSubFlowName(BonitaTree n, XPDLActivity xpdlAct)
        {
            int type = n.Noeud.getNodeType();
            switch (type) {
                case Node.ATTRIBUTE_NODE :
                    break;
                case Node.ELEMENT_NODE :
                    // EXTENDED ATTRIBUTE
                    if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE))
                    {
                        Element element = (Element)n.Noeud;
                        // Ex. for subFlow name
                        /*
                         *   <ExtendedAttribute Name="SubFlowName" Value="childProcess" />
                         */
                        if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_SUBFLOW_NAME)) {
                            xpdlAct.setSubFlowName(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
                            break;
                        }
                    }

                default :
                    break;
            }
            for (int i=0; i<n.NbFils; i++)
            {
                obtainSubFlowName(n.Fils[i], xpdlAct);
            }
            return xpdlAct;
        }

		/*
		 *   Get the initiator Mapper if it exists from External Attribute Bonita tree
		 *   giving the xpdlWFProcess bean
		 *
		 */
		private XPDLWFProcess obtainInitiatorMapper(BonitaTree n, XPDLWFProcess xpdlWFProcess, XPDLInitiatorMapper xpdlInitiatorMapper)
		{

			int type = n.Noeud.getNodeType();
			switch (type) {
				case Node.ATTRIBUTE_NODE :
					break;
				case Node.ELEMENT_NODE :

					// EXTENDED ATTRIBUTE : Properties & Hooks
					if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE))
					{
						Element element = (Element)n.Noeud;
		                 /*
		                 *  Ex. for Process Initiator Mapper
		                 *
		                 *  <ExtendedAttribute
		                 *          Name="initiatorMapper" Value="Custom" or "LDAP">
                         *     <CustomInitiatorClass>
                         *        hero.initiatorMapper.CustomGroupMembers
                         *     </CustomInitiatorClass>
                         *  </ExtendedAttribute>
						 */


						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_INITIATORMAPPER)) {
							xpdlInitiatorMapper.setInitiatorMapperType(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							xpdlWFProcess.setInitiatorMapper(xpdlInitiatorMapper);

//debug
//System.out.println("obtainInitiatorMapper: Bonita Project = " + xpdlWFProcess.getName() + " extended attribut Name = InitiatorMapper  Value =  "
//	+ xpdlInitiatorMapper.getInitiatorMapperType());
						}

					}
				break;

				case Node.TEXT_NODE :
					//Initiator Mapper Name
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE_INITIATOR_NAME)){
						xpdlInitiatorMapper.setInitiatorMapperName(n.Noeud.getNodeValue());
//debug
//System.out.println("obtainInitiatorMapper: Bonita Project = " + xpdlWFProcess.getName() + " Initiator Mapper Name = " +  xpdlInitiatorMapper.getInitiatorMapperName());
					}
				break;

				default :
					break;
			}
			for (int i=0; i<n.NbFils; i++)
			{
				obtainInitiatorMapper(n.Fils[i], xpdlWFProcess,xpdlInitiatorMapper);
			}
			return xpdlWFProcess;
		}

	/*
	 *   Get activity definition
	 *   (Join, deadlines, ext. attrib (properties, hooks), start mode, description, performer)
	 */
	private XPDLActivity obtainActivityData(BonitaTree n, ProjectSessionLocal p, XPDLActivity xpdlAct, Hashtable xpdlData) throws XPDLException
	{

			int type = n.Noeud.getNodeType();
			XPDLHook xpdlHook = null;
			switch (type) {
				case Node.ATTRIBUTE_NODE :
					break;
				case Node.ELEMENT_NODE :
					// Split
					if ((n.Noeud.getNodeName().equalsIgnoreCase(ACTIVITY_SPLIT)))
					{
						Element element = (Element)n.Noeud;
						if (element.getAttribute(TYPE).equalsIgnoreCase(MODE_TYPE_XOR))
							xpdlAct.setSplit(MODE_TYPE_XOR);
						else if (element.getAttribute(TYPE).equalsIgnoreCase(MODE_TYPE_AND))
							xpdlAct.setSplit(MODE_TYPE_AND);
					}
					// Join
					else if (n.Noeud.getNodeName().equalsIgnoreCase(ACTIVITY_JOIN))
					{
						Element element = (Element)n.Noeud;
						if (element.getAttribute(TYPE).equalsIgnoreCase(MODE_TYPE_XOR))
							xpdlAct.setJoin(MODE_TYPE_XOR);
						else if (element.getAttribute(TYPE).equalsIgnoreCase(MODE_TYPE_AND))
							xpdlAct.setJoin(MODE_TYPE_AND);
					}
					// deadlines Execution
					else if (n.Noeud.getNodeName().equalsIgnoreCase(ACTIVITYDEADLINE))
					{
						xpdlAct.setDeadline(true);
						Element element = (Element)n.Noeud;
						if (element.getAttribute(DEADLINE_EXECUTION).equalsIgnoreCase(ASYNCHR))
							xpdlAct.setDeadlineExecution(ASYNCHR);
						else if (element.getAttribute(DEADLINE_EXECUTION).equalsIgnoreCase(SYNCHR))
							xpdlAct.setDeadlineExecution(SYNCHR);
					}
					// EXTENDED ATTRIBUTE
					else if (n.Noeud.getNodeName().equalsIgnoreCase(EXTENDED_ATTRIBUTE))
					{
						Element element = (Element)n.Noeud;
						//	>>>>>>> Bonita PROPERTIES <<<<<<<<
						/*    based on:
						 *       - extended attributes reference datafields attributes defined within the process
						 *       (these datafields have an extended attributes indicating that the datafield is defined for activities)
						 *       ex.:   <ExtendedAttribute Name="property" Value="datafield_wfp_for_act_4"
						 * 				<Propagated Value="No" />
						 * 				/ExtendedAttribute>
						 *        (other possibility is: <Propagated Value="yes" /> )
						 */
						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_PROPERTY)) {
							xpdlAct.setProperties(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
//debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + " extended attribut Name = Property  Value = "
//	+ element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
						}
						// >>>>>>> Bonita HOOKS <<<<<<<<
						/*
						 *   based on :
						 *    <ExtendedAttribute Name="hook" Value="hero.hook.TestHook"> </ExtendedAttribute>
						 *
						 *    And furthermore other complexe type can be added:
		                 *        <HookEventName>BEFORETERMINATE</HookEventName>
		                 * 	      <HookScript>
		                 *           System.out.println("interactive hook executed !!!");
		                 *        </HookScript>
						 */

						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_HOOK)) {
							// set the XPDLHook Bean the activity Bean
							xpdlHook = new XPDLHook();
							xpdlAct.setXpdlHook(xpdlHook);
							xpdlHook.setHookName(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));

//	debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + " extended attribut Name = Hook  Value =  "
//+ element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							}

						// >>>>>>> Bonita Iteration <<<<<<<<
						/*
						 *   based on :
						 *    <ExtendedAttribute Name="iteration" Value="data_process.equals("riri")"> </ExtendedAttribute>
						 *
						 *    complexe type must added:
		                 *        <from>activity_from</from>
		                 * 		  <to>activity_to</to>
						 */

						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_ITERATION)) {

							// indicate that iteration exists on the activity
							//xpdlAct.setIterationName("Iteration");

							// set the "From" parameter of the iteration
							//xpdlAct.setIterationFrom(xpdlAct.getName());
							// set the "Condition" parameter
							if ( (element.getAttribute(EXTENDED_ATTRIBUTE_VALUE) != null)) {
							    //record the iteration condition
							    xpdlAct.setTempIterationCondition(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
							}
//	debug
//System.out.println("obtainActivityData: Iteration Activity From = " + xpdlAct.getName() + " Condition =  "
// + element.getAttribute(EXTENDED_ATTRIBUTE_VALUE)) ;
						}

                        //>>>>>>> Bonita Performer Assignment <<<<<<<<
							/*
							 * <ExtendedAttribute Name="PerformerAssign" Value="property">
							 * 		<Property>valideur</property>
							 * </ExtendedAttribute>
							 * or
							 * <ExtendedAttribute Name="PerformerAssign" Value="Callback">
							 * 		<Callback>hero.performAssignSelectActor</Callback>
							 * </ExtendedAttribute>
							 */
						if (element.getAttribute(EXTENDED_ATTRIBUTE_NAME).equalsIgnoreCase(EXTENDED_ATTRIBUTE_NAME_PERFORMER_ASSIGN)) {
							xpdlAct.setPerformerAssignType(element.getAttribute(EXTENDED_ATTRIBUTE_VALUE));
						}

					}
					else {
					// MANUAL
						if (n.Noeud.getNodeName().equalsIgnoreCase(MANUAL) && n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(STARTMODE))
							xpdlAct.setMode(MANUAL);
					// AUTOMATIC
						if (n.Noeud.getNodeName().equalsIgnoreCase(AUTOMATIC) && n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(STARTMODE))
							xpdlAct.setMode(AUTOMATIC);
					}

				break;

				case Node.TEXT_NODE :
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(ACTIVITY_DESCRIPTION))
						xpdlAct.setDescription(n.Noeud.getNodeValue());
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(ACTIVITY_PERFORMER))
						xpdlAct.setPerformer(n.Noeud.getNodeValue());
					// propagated property
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(PROPAGATED_PROPERTY)){
//debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + "obtained propagation attribut !!!!!!");
						if ("Yes".equalsIgnoreCase(n.Noeud.getNodeValue())){
							xpdlAct.setPropagations(new Boolean(true));
//debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + "propagated = Yes !!!!!!");
						}
						else if ("No".equalsIgnoreCase(n.Noeud.getNodeValue())) xpdlAct.setPropagations(new Boolean(false));
						else xpdlAct.setPropagations(new Boolean(false));
					}

					//deadline Condition
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(DEADLINECONDITION))
						xpdlAct.setDeadlineCondition(n.Noeud.getNodeValue());
					//deadline Exception
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(EXCEPTIONNAME))
						if (!"".equals(n.Noeud.getNodeValue())) xpdlAct.setExceptionName(n.Noeud.getNodeValue());
					// Hook Event Name
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(HOOKEVENTNAME)){
						    // get the last element of xpdlHooks (ArrayList) and set HookEventName to the XPDLHook Bean
							xpdlHook = (XPDLHook)xpdlAct.getXpdlHook().get(xpdlAct.getXpdlHook().size()-1);
							xpdlHook.setHookEventName(n.Noeud.getNodeValue());
//debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + " set hookEvent = "
//+ xpdlHook.getHookEventName() + " for HookName = " + xpdlHook.getHookName());
					}
					// Hook Script
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(HOOKSCRIPT)){
							//get the last element of xpdlHooks (ArrayList) and set HookScript to the XPDLHook Bean
							xpdlHook = (XPDLHook)xpdlAct.getXpdlHook().get(xpdlAct.getXpdlHook().size()-1);
							xpdlHook.setHookScript(n.Noeud.getNodeValue());
//debug
//System.out.println("obtainActivityData: Activity = " + xpdlAct.getName() + " set hookScript = "
//+ xpdlHook.getHookScript() + " for HookName = " + xpdlHook.getHookName());
					}
					/*
					// Iteration From
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(ITERATIONFROM))
						xpdlAct.setIterationFrom(n.Noeud.getNodeValue());
					*/
					// Iteration To
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(ITERATIONTO)) {
						//xpdlAct.setIterationTo(n.Noeud.getNodeValue());
						// the new set  for iteration
                        xpdlAct.setIteration(n.Noeud.getNodeValue(), xpdlAct.getTempIterationCondition());
// debug
//System.out.println("obtainActivityData:  iterationFrom = " + xpdlAct.getIterationFrom()
//		+ " iterationTo = " + xpdlAct.getIterationTo());
					}

				 	// Performer Assignment Property
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(PERFORMER_ASSIGN_PROPERTY)){
						//property name
						xpdlAct.setPerformerAssignProperty(n.Noeud.getNodeValue());
						//build the performer assignment name
						xpdlAct.setPerformerAssignName("perf_assign_" + n.Noeud.getNodeValue());
					}

					// Performer Assignment Callback
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(PERFORMER_ASSIGN_CALLBACK)){
						xpdlAct.setPerformerAssignProperty("");
						xpdlAct.setPerformerAssignName(n.Noeud.getNodeValue());
					}

					// ActualParameters
					if (n.Noeud.getParentNode().getNodeName().equalsIgnoreCase(ACTUALPARAMETER)){
						//property name
						xpdlAct.setActualParameter(n.Noeud.getNodeValue());
					}
				break;

				default :
					break;
			}
			for (int i=0; i<n.NbFils; i++)
			{
				obtainActivityData(n.Fils[i], p, xpdlAct, xpdlData);
			}

		return xpdlAct;

	}

	private void obtainTransition(BonitaTree a, ProjectSessionLocal p, boolean inActivitySet, String ActivitySetId, Hashtable xpdlData) throws XPDLException
	{
		if (a!=null)
		{
			String inst_mod=null;
			if ((a.Noeud.getNodeName()).equalsIgnoreCase(TRANSITIONS))
			{
				//Set used in case we are into an activitySet to get the First and Last Activity
				Set setAll = new HashSet();
				Set setTo = new HashSet();
				Set setFrom = new HashSet();

				for(int j = 0; j<a.NbFils; j++)
				{
					if ((a.Fils[j].Noeud.getNodeName()).equalsIgnoreCase(TRANSITION))
					{
						XPDLTransition xpdlTransition = new XPDLTransition();
						String edgeTo=null; String edgeFrom=null; String edgeName=null; String condition=null;
						// get the attributs of the node: Transition in an NamedNodeMaps
						NamedNodeMap atts = a.Fils[j].Noeud.getAttributes();

						for (int i = 0; i < atts.getLength(); i++)
						{
							Node att = atts.item(i);
							if (att.getNodeName().equalsIgnoreCase(TRANSITION_NAME)){
								edgeName=att.getNodeValue();
								xpdlTransition.setName(att.getNodeValue());
							}

							else if (att.getNodeName().equalsIgnoreCase(TRANSITION_TO)){
								edgeTo=att.getNodeValue();
								xpdlTransition.setTo(att.getNodeValue());
								if (inActivitySet){
									setTo.add(edgeTo);
									setAll.add(edgeTo);
								}
							}

							else if (att.getNodeName().equalsIgnoreCase(TRANSITION_FROM)) {
								edgeFrom=att.getNodeValue();
								xpdlTransition.setFrom(att.getNodeValue());
								if (inActivitySet){
									setFrom.add(edgeFrom);
									setAll.add(edgeFrom);
								}
							}

							else if (att.getNodeName().equalsIgnoreCase(TRANSITION_ID)){
								xpdlTransition.setId(att.getNodeValue());
							}
						}


					    // get the Condition
						// For ex: <Condition Type="CONDITION">datafield_id_1.equals("tarte");</Condition>
						BonitaTree condTree = getNode(a.Fils[j],TRANSITION_CONDITION);
						// Get of the condition set on the transition
						if (condTree != null){
							if (condTree.Noeud.hasChildNodes()){
								condition = condTree.Noeud.getFirstChild().getNodeValue();
								xpdlTransition.setCondition(condition);
// Trace
//System.out.println("Condition = " + condition);
							}
						}
						//Add to TransitionMapping the XPDLTransition bean
                        //TODO
                        Hashtable transitionMapping = (Hashtable)xpdlData.get("transitionMapping");
						transitionMapping.put(xpdlTransition.getId(),xpdlTransition);

						if ( (edgeFrom!=null) && (edgeTo!=null) ) {
// Traces for transition
//System.out.println("InNode: "+edgeFrom);
//System.out.println("OutNode: "+edgeTo);
//System.out.println("");
							// add transition into Bonita
							//inst_mod=p.addEdge((String)actMapping.get(edgeFrom), (String)actMapping.get(edgeTo))
                            //TODO
                            Hashtable actMapping = (Hashtable)xpdlData.get("actMapping");
							XPDLActivity xpdlActivity =  (XPDLActivity)actMapping.get(edgeFrom);
							String actNameFrom = xpdlActivity.getName();
							xpdlActivity =  (XPDLActivity)actMapping.get(edgeTo);
							String actNameTo = xpdlActivity.getName();
							try {
								inst_mod=p.addEdge(actNameFrom, actNameTo);
							} catch (HeroException he) {
								throw new XPDLException("Error adding edge from activity: " +actNameFrom + " activity: " + actNameTo + " " + he.getMessage(), he);
							}
							try {
								// add condition on the transition
								if (condition!=null){
									//set condition on the transition
									p.setEdgeCondition(inst_mod,condition);
								}
							} catch (HeroException he) {
								throw new XPDLException("Error setting edge condition from activity: " +actNameFrom + " activity: " + actNameTo + " " + he.getMessage(), he);
							}
						}
					}
				}

				// set the activitySetMapping with the Sets if we are in Activity Set (ActivitySet == true)
				if (inActivitySet && (ActivitySetId != null)){
					Set setAllTemp = new HashSet();
					String firstAct = null;
					String lastAct = null;

					// get the first Activity of the Activity Set
					setAllTemp.addAll(setAll);
					setAllTemp.removeAll(setTo);
					Iterator itTo = setAllTemp.iterator();
					while (itTo.hasNext()) {
						firstAct = (String)itTo.next();
					}
					if (firstAct == null){
						throw new XPDLException("Error getting first activity of activitySet with Id = " + ActivitySetId);
					}

					//get the last Activity of the Activity Set
					setAllTemp.addAll(setAll);
					setAllTemp.removeAll(setFrom);
					Iterator itFrom = setAllTemp.iterator();
					while (itFrom.hasNext()) {
						lastAct = (String)itFrom.next();
					}
					if (lastAct == null){
						throw new XPDLException("Error getting last activity of activitySet with Id = " + ActivitySetId);
					}
                    //TODO
                    Hashtable activitySetMapping = (Hashtable)xpdlData.get("activitySetMapping");
					activitySetMapping.put(ActivitySetId,lastAct + ACT_SET_MAP_SEPARATOR + firstAct);
//debug
//System.out.println("XPDLSessionBean:obtainTransition	: ActivitySetId = " + ActivitySetId + " lastAct = " + lastAct + " firstAct = " + firstAct);
				}
			}
		}
	}

	/**
	 * Create the ArbreXPDLSession Bean
	 *
	 * @throws CreateException
	 *
	 * @ejb.permission unchecked="yes"
	 * @ejb:create-method view-type="both"
	 **/

	public void ejbCreate() throws CreateException {
	}

	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}

	public void ejbRemove() {
	}

	public void ejbActivate() {
	}

	public void ejbPassivate() {
	}

}
