package hero.util;

/*
*
* SoapClient.java - 
* Copyright (C) 2003 Ecoo Team
* valdes@loria.fr
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import org.apache.axis.client.Service;

import hero.interfaces.Constants;

import org.apache.axis.client.Stub;

import hero.net.UserSession.UserSession_PortType;
import hero.net.UserSession.UserSessionService;
import hero.net.UserSession.UserSessionServiceLocator;

import hero.net.ProjectSession.ProjectSession_PortType;
import hero.net.ProjectSession.ProjectSessionService;
import hero.net.ProjectSession.ProjectSessionServiceLocator;


public class BonitaClient {
	// end points
	private final String userSession = "UserSession";
	private final String projectSession = "ProjectSession";
	
	//private ProjectSoapXML projectSoapClient;
	private ProjectSession_PortType projectSessionClient;
	private UserSession_PortType userSessionClient;
	
	// Network information
	private String host;
	private int port;
	private String baseUrl = "/bonita_ws/services/";
	
	// User information
	private String userName = null;
	private String userPassword = null;

	// manage transport to soap server...
	private Service service;
	private URL defaultUri;
	private URL userUri;
	private URL projectUri;
	//private URL projectXMLUri;

	// Persitance 
	private String currentProject;
	private boolean permission = false;

	//====================================================
	private void createUserSoapCall() throws Exception {
		Stub stub = ((Stub) userSessionClient);
		stub.setUsername(this.userName);
		stub.setPassword(this.userPassword);
	}
	private void createProjectSoapCall() throws Exception {
		Stub stub = ((Stub) projectSessionClient);
		stub.setUsername(this.userName);
		stub.setPassword(this.userPassword);
		stub.setMaintainSession(true);
	}

	//=========== Local ==================================
	public void login(String user, String pass) throws Exception{
		this.userName = user;
		this.userPassword = pass;
						
		UserSessionService serviceUS = new UserSessionServiceLocator();
		userSessionClient = serviceUS.getUserSession(userUri);
	
		ProjectSessionService servicePSS = new ProjectSessionServiceLocator();
		projectSessionClient = servicePSS.getProjectSession(projectUri);
			
		this.createUserSoapCall();
		this.createProjectSoapCall();
	}
	public String getUser() throws Exception {
		return (this.userName);
	}
	public String getProjectName() {
		return this.currentProject;
	}
	public boolean hasPermission() {
		return permission;
	}

	//====================================================

	public BonitaClient(String host, int port, String urlBase) throws Exception {
		this.host = host;
		this.port = port;
		this.baseUrl = urlBase;
		// set all the possible endpoint
		userUri = new URL("http://" + host + ":" + port + baseUrl + userSession);
		projectUri = new URL("http://" + host + ":" + port + baseUrl + projectSession);	
	}

	// SOAP ===============================================

	public void initProject(String managerProject) throws Exception {
		this.currentProject = managerProject;				
		projectSessionClient.initProject(currentProject);
		
		// Check the permission		
		permission = isInRole(userName, "admin");
	}
	
	public void initModel(String managerProject) throws Exception {
		this.currentProject = managerProject;				
		projectSessionClient.initModel(currentProject);
		
		// Check the permission		
		permission = isInRole(userName, "admin");
	}
	
	public void initProject(String src, String dest) throws Exception {
		this.login(this.userName,this.userPassword);
		projectSessionClient.initProject(src,dest);
		// Set the local var
		this.currentProject = dest;
	}

	public void instantiateProject(String src) throws Exception {
		this.login(this.userName,this.userPassword);
		projectSessionClient.instantiateProject(src);
	}
		
	public boolean openProject(String projectName) throws Exception {
		initProject(projectName);
		return true;
	}
	public boolean createProject(String projectName) throws Exception {
		if (exists(projectName)) {
			return false;
		}
		initProject(projectName);
		return true;
	}
	
	public boolean createModel(String modelName) throws Exception {
		if (exists(modelName)) {
			return false;
		}
		initModel(modelName);
		return true;
	}

	public boolean cloneProject(String projectSrc, String projectDest) throws Exception {
		if (exists(projectDest)) {
			return false;
		}
		initProject(projectSrc, projectDest);
		return true;
	}

	private boolean exists(String name) throws Exception {
		Collection projects = getProjects();
		Iterator i = projects.iterator();
		while (i.hasNext()) {
			String prj = (String) i.next();
			if (prj.equalsIgnoreCase(name)) {
				return true;
			}
		}
		return false;
	}

	public Collection getProjects() throws Exception {
		ArrayList result = new ArrayList();
		Object[] projects = userSessionClient.getProjectListNames();
		Iterator i = ((java.util.List) Arrays.asList(projects)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return result;
	}
	
	public Collection getProjectsList() throws Exception {
		ArrayList result = new ArrayList();
		Object[] projects = userSessionClient.getProjectList();
		Iterator i = ((java.util.List) Arrays.asList(projects)).iterator();
		while (i.hasNext()) {
			result.add((hero.net.UserSession.BnProjectLightValue)i.next());
		}
		return result;
	}
	
	public Collection getProjectInstances(String projectName) throws Exception {
		ArrayList result = new ArrayList();
		Object[] instances = userSessionClient.getProjectInstancesNames(projectName);
		Iterator i = ((java.util.List) Arrays.asList(instances)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return result;
	}

	public Collection getInstances() throws Exception {
		ArrayList result = new ArrayList();
		Object[] instances = userSessionClient.getInstancesListNames();
		Iterator i = ((java.util.List) Arrays.asList(instances)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return result;
	}
	
	public boolean containsNode(String value) throws Exception {
		Collection nodes = getNodes();

		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			String nodeName = (String) i.next();
			if (value.equalsIgnoreCase(nodeName)) {
				return true;
			}
		}
		return false;
	}

	public String getType() throws Exception {
		return (projectSessionClient.getType());
	}
	
	public String getStatus() throws Exception {
		return (projectSessionClient.getStatus());
	}
	
	public void setStatus(String status) throws Exception {
		if (status.equals(Constants.Pj.ACTIVE))
			projectSessionClient.activeProcess();
		else
			projectSessionClient.hideProcess();
	}
	
	
	public Collection getNodes() throws Exception {
		ArrayList result = new ArrayList();
		Object[] nodes = projectSessionClient.getNodesNames();
		Iterator i = ((java.util.List) Arrays.asList(nodes)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return (result);
	}
	public Collection getEdges() throws Exception {
		ArrayList result = new ArrayList();
		Object[] edges = projectSessionClient.getEdgesNames();

		Iterator i = ((java.util.List) Arrays.asList(edges)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}

		return result;
	}

	public void addNodeProperty(String node, String key,String value, boolean propagate) throws Exception {
		
		    projectSessionClient.setNodeProperty( node,key,value, propagate);		
		}
		
	public void addIteration(String from, String to, String condition) throws Exception {
		projectSessionClient.addIteration(from,to,condition);
	}
	
	public Collection getIterations() throws Exception {
		ArrayList result = new ArrayList();
		Object[] ite = projectSessionClient.getIterations();
		
		Iterator i = ((java.util.List) Arrays.asList(ite)).iterator();
		while (i.hasNext()) {
			result.add((hero.net.ProjectSession.BnIterationLightValue)i.next());
		}

		return result;
	}
		
	public void addProjectProperty(String key,String value) throws Exception {
		projectSessionClient.setProperty(key,value);
	}
	
	public void addNodeInterHook(String node, String hook, String event, String value) throws Exception {
		
		    projectSessionClient.addNodeInterHook(node,hook,event,6,value);		    
	}
	
	public void addProjectInterHook(String hook, String event, String value) throws Exception {	
		projectSessionClient.addInterHook(hook,event,6,value);		    
	}
	
	public void addNode(String name, String type) throws Exception {
		int nodeType = 0;
		if (type.equals("AND JOIN"))
			nodeType = Constants.Nd.AND_JOIN_NODE;
		if (type.equals("OR JOIN"))
			nodeType = Constants.Nd.OR_JOIN_NODE;
		if (type.equals("AND JOIN AUTO"))
			nodeType = Constants.Nd.AND_JOIN_AUTOMATIC_NODE;
		if (type.equals("OR JOIN AUTO"))
			nodeType = Constants.Nd.OR_JOIN_AUTOMATIC_NODE;
		if (type.equals("SUB PROCESS"))
					nodeType = Constants.Nd.SUB_PROCESS_NODE;
		projectSessionClient.addNode(name, nodeType);

	}
	
	public void addNodeSubProcess(String name, String projectName) throws Exception {
		projectSessionClient.addNodeSubProcess(name,projectName);
	}

	public boolean deleteNode(String node) throws Exception {
		projectSessionClient.deleteNode(node);
		return true;
	}
	
	public boolean deleteProject(String projectName) throws Exception {
		if (projectName.matches(".*_instance.*"))
			userSessionClient.removeInstance(projectName);
		else
			userSessionClient.removeProject(projectName);
		return true;
	}


	public void deleteEdges(Object[] edges) throws Exception {
		for (int i = 0; i < edges.length; i++) {
			projectSessionClient.deleteEdge((String) edges[i]);
		}
	}

	public String addEdge(String src, String dest) throws Exception {
		String name = null;
		name = projectSessionClient.addEdge(src,dest);
		return name;
	}

	public String setEdgeCondition(String edgeName, String condition) throws Exception {
		projectSessionClient.setEdgeCondition(edgeName,condition);
		return edgeName;
	}
	
	public String getEdgeCondition(String edgeName) throws Exception {
			return(projectSessionClient.getEdgeCondition(edgeName));
	}

	public String getState(String name) throws Exception {
		int state = getNodeState(name);
		switch (state) {
			case Constants.Nd.INITIAL :
				return ("Initial");
			case Constants.Nd.READY :
				return ("Ready");
			case Constants.Nd.ANTICIPABLE :
				return ("Anticipable");
			case Constants.Nd.ANTICIPATING :
				return ("Anticipating");
			case Constants.Nd.EXECUTING :
				return ("Executing");
			case Constants.Nd.EXECUTED :
				return ("Executed");
			case Constants.Nd.TERMINATED :
				return ("Terminated");
			case Constants.Nd.ANT_SUSPENDED :
				return ("Ant_suspended");
			case Constants.Nd.EXEC_SUSPENDED :
				return ("Exec_suspended");
		}
		return ("");
	}

	public int getNodeState(String name) throws Exception {
		return (projectSessionClient.getNodeState(name));
	}
	
	public int getNodeType(String name) throws Exception {
			return (projectSessionClient.getNodeType(name));
		}

	public boolean getNodeAnticipable(String name) throws Exception {
		return (projectSessionClient.getNodeAnticipable(name));
	}
	
	public String getNodeInterHookValue(String nodeName, String hookName) throws Exception {
		return (projectSessionClient.getNodeInterHookValue(nodeName,hookName));
	}
	
	public void setNodeInterHookValue(String nodeName, String hookName, String value) throws Exception {
		projectSessionClient.setNodeInterHookValue(nodeName,hookName,value);
	}
	
	public String getProjectInterHookValue(String hookName) throws Exception {
		return (projectSessionClient.getInterHookValue(hookName));
	}
	
	public void setProjectInterHookValue(String hookName, String value) throws Exception {
		projectSessionClient.setInterHookValue(hookName,value);
	}
	
	public String getNodeDescription(String nodeName) throws Exception {
		return (projectSessionClient.getNodeDescription(nodeName));
	}
	
	public void setNodeDescription(String nodeName, String description) throws Exception {
		projectSessionClient.setNodeDescription(nodeName,description);
	}

	public void setNodeTraditional(String nodeName) throws Exception {
		projectSessionClient.setNodeTraditional(nodeName);
	}

	public void setNodeAnticipable(String nodeName) throws Exception {
		projectSessionClient.setNodeAnticipable(nodeName);
	}

	public String getEdgeInNode(String name) throws Exception {
		return (projectSessionClient.getEdgeInNode(name));
	}

	public String getEdgeOutNode(String name) throws Exception {
		return (projectSessionClient.getEdgeOutNode(name));
	}

	public boolean isActive(String name) throws Exception { //Only used by WFmanager.updateStatus
		int state = getNodeState(name);
		switch (state) {
			case Constants.Nd.INITIAL :
				return false;
			case Constants.Nd.READY :
				return false;
			case Constants.Nd.ANTICIPABLE :
				return false;
			case Constants.Nd.ANTICIPATING :
				return true;
			case Constants.Nd.EXECUTING :
				return true;
			case Constants.Nd.EXECUTED :
				return false;
			case Constants.Nd.TERMINATED :
				return true;
			case Constants.Nd.ANT_SUSPENDED :
				return false;
			case Constants.Nd.EXEC_SUSPENDED :
				return false;
		}
		return false;
	}

	public String getType(String name) throws Exception {
		int type = projectSessionClient.getNodeType(name);
		switch (type) {
			case Constants.Nd.AND_JOIN_NODE :
				return ("And Join");
			case Constants.Nd.OR_JOIN_NODE :
				return ("Or Join");
			case Constants.Nd.AND_JOIN_AUTOMATIC_NODE :
				return ("And Join Automatic");
			case Constants.Nd.OR_JOIN_AUTOMATIC_NODE :
				return ("Or Join Automatic");
			case Constants.Nd.SUB_PROCESS_NODE :
				return ("Sub Procss");
		}
		return ("");
	}

	public String getRole(String name) throws Exception {
		return(projectSessionClient.getNodeRoleName(name));
	}

	public void setNodeRole(String node, String newRole) throws Exception {
		projectSessionClient.setNodeRole(node,newRole);
	}
				

	public String getDeadline(String name) throws Exception {
		return(projectSessionClient.getNodeDeadline(name));
	}
	
	public void setDeadline(String node, long dl) throws Exception {
		projectSessionClient.setNodeDeadline(node,dl);   
	}

	public Collection getRoles() throws Exception {
		ArrayList result = new ArrayList();
		Object[] roles = projectSessionClient.getRolesNames();
		Iterator i = ((java.util.List) Arrays.asList(roles)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return (result);
	}

	public Collection getUsersInProject() throws Exception {
		ArrayList result = new ArrayList();
		Object[] users = projectSessionClient.getUsers();
		Iterator i = ((java.util.List) Arrays.asList(users)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return (result);
	}

	public Collection getAllUsers() throws Exception {
		ArrayList result = new ArrayList();
		Object[] users = projectSessionClient.getAllUsers();
		Iterator i = ((java.util.List) Arrays.asList(users)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return (result);
	}

	public void addUser(String name) throws Exception {
		projectSessionClient.addUser(name);
	}
	public void addRole(String name, String desc) throws Exception {
		projectSessionClient.addRole(name,desc);
	}
	public void setUserRole(String name, String role) throws Exception {
		projectSessionClient.setUserRole(name,role);
	}
	
	public void addRoleMapper(String role, String mapper, String type) throws Exception {
		int mapperType=0;
		if (type.equals(Constants.Mapper.PROPERTIESMAPPER))
			mapperType=1;
		if (type.equals(Constants.Mapper.CUSTOMMAPPER))
			mapperType=2;
		projectSessionClient.addRoleMapper(role,mapper,mapperType);
	}
	
	public void addNodePerformerAssigment(String node, String name, String type, String property) throws Exception {
		int performerType=0;
		if (type.equals(Constants.PerformerAssigment.PROPERTIESPA))
			performerType=1;
		projectSessionClient.addNodePerformerAssign(node, name, performerType, property);
	}

	public boolean isInRole(String name, String role) throws Exception {
		Object[] projects = projectSessionClient.getUserRolesInProjectNames(name);

		Iterator i = ((java.util.List) Arrays.asList(projects)).iterator();
		while (i.hasNext()) {
			String roleName = (String) i.next();
			if (roleName.equals(role))
				return true;
		}
		return false;
	}

	public Collection getToDoList(String project) throws Exception {
		ArrayList result = new ArrayList();
		Object[] activity = userSessionClient.getToDoList(project);
		Iterator i = ((java.util.List) Arrays.asList(activity)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return result;
	}
	
	public Collection getActivityList(String project) throws Exception {
		ArrayList result = new ArrayList();
		Object[] activity = userSessionClient.getActivityList(project);
		Iterator i = ((java.util.List) Arrays.asList(activity)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}
		return result;
	}

	
	public Object[] getProjectNodes() throws Exception {
	        return(projectSessionClient.getNodes());		    		   
		}
		
	public Collection getProperties() throws Exception {
		ArrayList result = new ArrayList();
		Object[] prop = projectSessionClient.getPropertiesKey();
		Iterator i = ((java.util.List) Arrays.asList(prop)).iterator();
		while (i.hasNext()) {
			result.add(i.next());
		}   		   
		return result;
	}	
	
	public void setEditNode(String node, String role, String description, long date) throws Exception {
		    projectSessionClient.setEditNode(node, role, description,date);
		}

	public void startActivity(String project, String activity) throws Exception {
		userSessionClient.startActivity(project,activity);
	}

	public void terminateActivity(String project, String activity) throws Exception {
		userSessionClient.terminateActivity(project,activity);
	}
	public void cancelActivity(String project, String activity) throws Exception {
		userSessionClient.cancelActivity(project,activity);
	}
	
	public void terminateProject(String project) throws Exception {
		userSessionClient.terminate(project);
	}
	public hero.net.ProjectSession.StrutsNodeValue getStrutsNode(String name) throws Exception {
		return(projectSessionClient.getStrutsNode(name));
	}	

}
