package hero.util;

/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: BonitaConfig.java,v 1.7 2006/05/23 13:41:51 brice Exp $
*
--------------------------------------------------------------------------
*/

import java.util.List;

import javax.management.Attribute;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

public class BonitaConfig {
	
	private ObjectName config;
	private MBeanServer server;	
	private static BonitaConfig bc = null; // Singleton pattern

	public BonitaConfig() throws HeroException{
		try{
			config = new ObjectName("bonita:type=bonitaservice,name=Config");
			List list = MBeanServerFactory.findMBeanServer(null);
			server = (MBeanServer) list.iterator().next();			
		} catch (Exception e) {e.printStackTrace();
			throw new HeroException(e.getMessage());
		}		
	}
	
	public static BonitaConfig getInstance() throws HeroException{
		if ( bc == null) {
			bc = new BonitaConfig();
		}
		return bc;
	}
	
	public void setJms(boolean jms) throws HeroException
	{
		try {
			server.invoke(config, "updateJms", new Object[] {new Boolean(jms)},new String[] {"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setHistoric(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateHistoric", new Object[] { value},new String[] {"".getClass().getName()});		
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public void setLogLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateLogLevel", new Object[] {value},new String[] {"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public void setTraceLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateTraceLevel", new Object[] {value},new String[] {"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}		
  
	public void setProperty(String key, String value) throws HeroException
	{
		try {
			server.invoke(config, "updateProperty", new Object[] {key,value}, new String[] {"".getClass().getName(),"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
   
	public boolean getJms() throws HeroException
	{
		try {
			return (((Boolean)server.getAttribute(config,"Jms")).booleanValue());
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public String getHistoric() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"Historic"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	public String getLogLevel() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"LogLevel"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	public String getTraceLevel() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"TraceLevel"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
  
	public String getProperty(String key) throws HeroException
	{
		try {
			return ((String)server.invoke(config, "getProperty", new Object[] {key}, new String[] {"".getClass().getName()}));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
}