/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: BonitaServiceLocator.java,v 1.5 2005/09/11 09:41:58 brice Exp $
 *
--------------------------------------------------------------------------
 */

package hero.util;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Map;
import java.util.HashMap;

import javax.rmi.PortableRemoteObject;
import javax.mail.Session;

public class BonitaServiceLocator {
    private static BonitaServiceLocator msl;
    private Map cache;
    private static Context initial;

   
    private BonitaServiceLocator() throws BonitaServiceException {
	try {
	    initial=new InitialContext();
	    cache = new HashMap();
	} catch (javax.naming.NamingException ne) {
	    throw new BonitaServiceException(ne.getMessage());
	}
    }

    static public BonitaServiceLocator getInstance() throws BonitaServiceException {

	if (msl==null) {
	    msl=new BonitaServiceLocator();
	}
	return msl;
    }

    // Services constant inner class - service objects

    public class Services {
	   final public static int TOPIC_CONNECTION_FACTORY=0;
	   final public static int TOPIC=1;
	   final public static int MAIL_SERVICE=2;
    }

    // JMS related constants
    // NB: It was decided not to change the name of the factory in
    // JBoss configuration to match JOnAS name, hence the test.
    final static String TOPIC_CONNECTION_FACTORY_NAME = ServerType.isJonas() ? "TCF" : "java:/XAConnectionFactory";
	final static String TOPIC_NAME = "testTopic";

    // Mail configuration
    final static String MAIL_SERVICE_NAME = "Mail";    

    // Returns the JNDI name for the required service
    

    static private  String getServiceName(int service) {
	  switch (service) {
	    case Services.TOPIC_CONNECTION_FACTORY: return TOPIC_CONNECTION_FACTORY_NAME;
		case Services.TOPIC: return TOPIC_NAME;
		case Services.MAIL_SERVICE: return MAIL_SERVICE_NAME;
	  }
	return null;
    }

    // returns a EJBHome for the given service, using the JNDI name
    // and the Class for the EJBHome

    public Object getResource(int s) throws BonitaServiceException {
	Object resource=null;
	try {
		String serv = getServiceName(s);
		if (cache.containsKey(serv)) {
		  resource = (Object) cache.get(serv);
		} else {
	       Object objref = initial.lookup(serv);
	       resource=objref;
		   cache.put(serv, resource);
		}
	} catch (NamingException ex) {
	    throw new BonitaServiceException("ServiceException");
	} catch (Exception ex) {
	    System.out.println("ServiceLocator Exception");
	    ex.printStackTrace();
	}

	return resource;
    }   

	public Object getMailSession(int s) throws BonitaServiceException {
	Object mail=null;
	try {
		String serv = getServiceName(s);
		if (cache.containsKey(serv)) {
		  mail = (Object) cache.get(serv);
		} else {
		   Object objref = initial.lookup(serv);
		   mail= PortableRemoteObject.narrow(objref,Session.class);
		   cache.put(serv, mail);
		}
	} catch (NamingException ex) {
		throw new BonitaServiceException("ServiceException");
	} catch (Exception ex) {
		System.out.println("ServiceLocator Exception");
		ex.printStackTrace();
	}

	return mail;
	}   
  
}
