/**
*
* Bonita
* Copyright (C) 1999 Bull S.A.
* Bull 68 route de versailles  78434 Louveciennes Cedex France
* Further information: bonita@objectweb.org
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
*
--------------------------------------------------------------------------
* $Id: BonitaServiceValue.java,v 1.5 2006/08/17 15:30:54 mvaldes Exp $
*
--------------------------------------------------------------------------
*/
package hero.util;

import java.io.Serializable;

import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaInterHookValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaPerformerValue;
import hero.util.values.BonitaProjectValue;
import hero.util.values.BonitaPropertyValue;
import hero.interfaces.Constants;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

public final class BonitaServiceValue implements Serializable {
    
    public static BonitaNodeValue getNodeFromCache(BonitaProjectValue pv, String nodeName){
    	Collection nodes = pv.getNodes();
		Iterator nodesC = nodes.iterator();		
		while (nodesC.hasNext())
		{
			BonitaNodeValue nValue = (BonitaNodeValue)nodesC.next();
			if (nValue.getName().equals(nodeName))						
				return nValue; 
		}
		return null;
    }
    
    public static BonitaNodeValue getNode(BnProjectLocal pl, String nodeName) throws HeroException
    {    
    	try {
    	BnNodeLocal nd = pl.getBnNode(nodeName);
    	return (BonitaServiceValue.generateNode(nd,pl));
    	
    }catch (Exception e){throw new HeroException(e.getMessage());}
    }
    
    public static BonitaNodeValue getNode(String projectName, String projectVersion, String nodeName) throws HeroException
    {    
    	try {
    	BnProjectLocalHome ph = hero.interfaces.BnProjectUtil.getLocalHome();
    	BnProjectLocal pl = ph.findByNameVersion(projectName, projectVersion);
    	BnNodeLocal nd = pl.getBnNode(nodeName);
    	return (BonitaServiceValue.generateNode(nd,pl));
    	
    }catch (Exception e){throw new HeroException(e.getMessage());}
    }
    
    public static Collection getProjectHooks(BnProjectLocal pl) throws HeroException
    {        	
    	Collection projectHooks = new ArrayList();
    	Collection hooks = pl.getBnHooks();
    	Iterator prHooks = hooks.iterator();
    	while (prHooks.hasNext())
    	{    		
    		BnProjectHookLocal ph = (BnProjectHookLocal)prHooks.next();
    		BonitaHookValue hook = new BonitaHookValue();
    		hook.setName(ph.getName());
    		hook.setEvent(ph.getEvent());
    		hook.setType(ph.getType());
    		projectHooks.add(hook);				
    	}    			   	
    	return projectHooks;
    }
    
    public static Collection getProjectInterHooks(BnProjectLocal pl) throws HeroException
    {        	
    	Collection projectHooks = new ArrayList();
    	Collection hooks = pl.getBnInterHooks();
    	Iterator prHooks = hooks.iterator();
    	while (prHooks.hasNext())
    	{    		
    		BnProjectInterHookLocal ph = (BnProjectInterHookLocal)prHooks.next();
    		BonitaInterHookValue hook = new BonitaInterHookValue();
    		hook.setName(ph.getName());
    		hook.setEvent(ph.getEvent());
    		hook.setType(ph.getType());
    		hook.setScript(ph.getScript());
    		projectHooks.add(hook);			
    	}    			   	
    	return projectHooks;
    }
    
    public static BonitaNodeValue generateNode(BnNodeLocal nd, BnProjectLocal pl) throws HeroException
    {
     try{
    	BonitaNodeValue nv = new BonitaNodeValue();
    	
    	BonitaNodeValue node = new BonitaNodeValue();
		node.setProjectName(pl.getName());		
		node.setName(nd.getName());
		node.setType(nd.getType());
		if (nd.getType()==Constants.Nd.SUB_PROCESS_NODE)
			node.setReference(nd.getReference());
		
		// Node inEdges			
		Collection inEdges = nd.getInBnEdges();
		Iterator inEdgesI = inEdges.iterator();
		Collection nodeInEdges = new ArrayList();
		while(inEdgesI.hasNext())
		{
			BnEdgeLocal el = (BnEdgeLocal)inEdgesI.next();
			BonitaEdgeValue inEdge = new BonitaEdgeValue();
			inEdge.setName(el.getName());
			inEdge.setCondition(el.getCondition());
			inEdge.setState(el.getState());		
			inEdge.setInNode(el.getInBnNode().getName());
			inEdge.setOutNode(el.getOutBnNode().getName());
			nodeInEdges.add(inEdge);
		}
		node.setInEdges(nodeInEdges);
		
		// Node outEdges			
		Collection outEdges = nd.getOutBnEdges();
		Iterator outEdgesI = outEdges.iterator();
		Collection nodeOutEdges = new ArrayList();
		while(outEdgesI.hasNext())
		{
			BnEdgeLocal el = (BnEdgeLocal)outEdgesI.next();
			BonitaEdgeValue outEdge = new BonitaEdgeValue();
			outEdge.setName(el.getName());
			outEdge.setCondition(el.getCondition());
			outEdge.setState(el.getState());		
			outEdge.setInNode(el.getInBnNode().getName());
			outEdge.setOutNode(el.getOutBnNode().getName());
			nodeOutEdges.add(outEdge);
		}
		node.setOutEdges(nodeOutEdges);
				
		// Node Hooks
		Collection nodeHooks = pl.getBnNode(nd.getName()).getBnHooks();
		Iterator nodeHooksI = nodeHooks.iterator();
		Collection nodeHks = new ArrayList();
		while(nodeHooksI.hasNext())
		{
			BnNodeHookLocal ph = (BnNodeHookLocal)nodeHooksI.next();
			BonitaHookValue hook = new BonitaHookValue();
			hook.setName(ph.getName());
			hook.setEvent(ph.getEvent());
			hook.setType(ph.getType());
			nodeHks.add(hook);				
		}
		node.setHooks(nodeHks);
		
		// Node InterHooks
		Collection nodeInterHooks = pl.getBnNode(nd.getName()).getBnInterHooks();
		Iterator nodeInterHooksI = nodeInterHooks.iterator();
		Collection nodeInterHks = new ArrayList();
		while(nodeInterHooksI.hasNext())
		{
			BnNodeInterHookLocal pih = (BnNodeInterHookLocal)nodeInterHooksI.next();
			BonitaInterHookValue interHook = new BonitaInterHookValue();
			interHook.setName(pih.getName());
			interHook.setEvent(pih.getEvent());
			interHook.setType(pih.getType());
			interHook.setScript(pih.getScript());
			nodeInterHks.add(interHook);				
		}
		node.setInterHooks(nodeInterHks);	
		
		return node;
     }catch (Exception e){throw new HeroException(e.getMessage());}
    }
}
