package hero.xpdl;

import java.util.HashMap;
import java.util.Map;
import java.net.URL;


/**
 * This class defines the declarations of defaults DTDs used for XPDL language
 * @author Florent Benoit
 */
public class XPDLDTDs implements DTDs {

    /**
     * List of default dtds
     */
    private static final String[] DEFAULT_DTDS = new String[] {
        "xpdl.dtd",
    };

    /**
     * List of default publicId
     */
    private static final String[] DEFAULT_DTDS_PUBLIC_ID = new String[] {
        "bonitaXPDL",
    };


    /**
     * Map where mapping publicId/dtds are stored
     */
    private static HashMap dtdsMapping = null;

    /**
     * Build a new object for DTDs handling
     */
    public XPDLDTDs() {
        dtdsMapping = new HashMap();
        addMapping(DEFAULT_DTDS, DEFAULT_DTDS_PUBLIC_ID);
    }

    /**
     * Gets the mapping between publicIds and DTDs
     * @return the mapping between publicIds and DTDs
     */
    public Map getMapping() {
        return dtdsMapping;
    }



    /**
     * Add to the list of DTDS the given dtds/publicId
     * @param dtds array of dtds
     * @param publicIds array of publicIds
     */
    protected void addMapping(String[] dtds, String[] publicIds) {
        if (dtds.length != publicIds.length) {
            throw new IllegalStateException("SEVER ERROR !!! Number of dtds is different of the number of PublicId !!! check the source code");
        }

        URL url = null;
        for (int i = 0; i < dtds.length; i++) {
            url = XPDLDTDs.class.getResource("/etc/xml/xpdl/" + dtds[i]);
            if (url == null) {
                throw new IllegalStateException("'" + dtds[i] + "' was not found in the current classloader !");
            }
            dtdsMapping.put(publicIds[i], url.toString());
        }
    }

}
