package hero.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.commons.EmptyVisitor;

public class RemoveMethod extends Task {

	private File classdir = null; 
	private String method = null;
	
	public void setClassdir(File classdir) {
		this.classdir = classdir;
	}

	public void setMethod(String method) {
		this.method = method;
	}
	
	public void execute() throws BuildException {
	    System.out.println("Removing method starting with '" + method + "' from the classes in folder : " + classdir);
	    File[] fileList = classdir.listFiles();
		InputStream is;
		ClassReader cr;
		ClassWriter cw;
		FileOutputStream fos;
		for(int i=0; i<fileList.length; i++) {
	    	try {
				is = new FileInputStream(fileList[i]);
				cr = new ClassReader(is);
		        cw = new ClassWriter(true);
		        RemoveMethodAdapter cv = new RemoveMethodAdapter(cw, method);
		        cr.accept(cv, false);
		        byte[] byteCodeClass = cw.toByteArray();
		        fos = null;
		        String fName = fileList[i].toString();
		        fos = new FileOutputStream(fName);
		        fos.write(byteCodeClass);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
	    }
		System.out.println("Done!");
	}
}

class RemoveMethodAdapter extends ClassAdapter implements Opcodes {
	private String method = null;
	
	public RemoveMethodAdapter(ClassVisitor cv, String method) {
	    super(cv);
	    this.method = method;
	}
	
	/*
	 * Method that manipulates class methods
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitMethod(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	public MethodVisitor visitMethod(
	        final int access,
	        final String name,
	        final String desc,
	        final String signature,
	        final String[] exceptions)
	    {
	        if (name.startsWith(method)) {
	            return new EmptyVisitor();
	        } else {
	            return cv.visitMethod(access, name, desc, signature, exceptions);
	        }
	    }
	
	/*
	 * Method that manipulates class atributes
	 * 
	 * @see org.objectweb.asm.ClassVisitor#visitField(int, java.lang.String, java.lang.String, java.lang.String, java.lang.Object)
	 */
	public FieldVisitor visitField(
	        final int access,
	        final String name,
	        final String desc,
	        final String signature,
	        final Object value)
	    {
	        if ("myString".equals(name)) {
	            return cv.visitField(ACC_PUBLIC, "newMyString", desc, signature, value);
	        } else {
	            return cv.visitField(access, name, desc, signature, value);
	        }
	    }
}
