package hero.util;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
 --------------------------------------------------------------------------
 * $Id$
 *
 --------------------------------------------------------------------------
 */

import java.net.MalformedURLException;
import java.net.URI;
import java.util.List;
import java.util.Properties;

import javax.management.Attribute;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import org.objectweb.carol.util.configuration.ConfigurationRepository;
import org.objectweb.carol.util.configuration.ProtocolConfiguration;

/**
 * This class is currently JOnAS specific. 
 */
public class BonitaConfigClient {
	
	private ObjectName config;
    private MBeanServerConnection server;
	//private MBeanServer server;	
	private static BonitaConfigClient bc = null; // Singleton pattern

	public BonitaConfigClient() throws HeroException{	
		try{
			ProtocolConfiguration protocolConfiguration = ConfigurationRepository.getCurrentConfiguration();
			String sCarolURL =  protocolConfiguration.getProviderURL();			
			URI carolURL = new URI(sCarolURL);
			String host= carolURL.getHost();
			int portNb = carolURL.getPort();
			String port = String.valueOf(portNb);
			String url = null;
			url = "service:jmx:rmi:///jndi/rmi://"+host+":" + port + "/jrmpconnector_jonas";
			//tmp = tmp + "url="+url+" ";

			/*
			CarolCurrentConfiguration carolConfig = CarolCurrentConfiguration.getCurrent();
			Properties props = carolConfig.getRMIProperties("jrmp");
			String sCarolURL = props.getProperty(Context.PROVIDER_URL);
			URI carolURL = new URI(sCarolURL);
			String host= carolURL.getHost();
			int portNb = carolURL.getPort();
			String port = String.valueOf(portNb);
			String url = null;
			
			url = "service:jmx:rmi:///jndi/rmi://"+host+":" + port + "/jrmpconnector_jonas";*/
			// Try to connect to the MBeanServer
			JMXServiceURL connURL = null;
			try {
				connURL = new JMXServiceURL(url);
			} catch (MalformedURLException e) {
				throw new HeroException("Can't create JMXServiceURL with string: " + url);
			}
			JMXConnector connector = null;
			try {
				connector = JMXConnectorFactory.newJMXConnector(connURL, null);
			} catch (MalformedURLException e1) {
				throw new HeroException("there is no provider for the protocol in " + url);
			} catch (java.io.IOException e) {
				throw new HeroException("Connector client cannot be made because of a communication problem (used URL: " + url + ")");
			}
			try {   	    	
				connector.connect(null);
				server = connector.getMBeanServerConnection();    		
			} catch (java.io.IOException ioe) {
				throw new HeroException("connection could not be made because of a communication problem");
			}		
			
			config = new ObjectName("bonita:type=bonitaservice,name=Config");					
		} catch (Exception e) {e.printStackTrace();
		throw new HeroException(e.getMessage());
		}		
	}
	
	public static BonitaConfigClient getInstance() throws HeroException{
		if ( bc == null) {
			bc = new BonitaConfigClient();
		}
		return bc;
	}
	
	public void setJms(boolean jms) throws HeroException
	{
		try {
			server.invoke(config, "updateJms", new Object[] {new Boolean(jms)},new String[] {"boolean"});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public void setHistoric(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateHistoric", new Object[] { value},new String[] {"".getClass().getName()});		
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public void setLogLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateLogLevel", new Object[] {value},new String[] {"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public void setTraceLevel(String value) throws HeroException
	{
		try {
			server.invoke(config, "updateTraceLevel", new Object[] {value},new String[] {"".getClass().getName()});
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
	
	public boolean getJms() throws HeroException
	{
		try {
			return (((Boolean)server.getAttribute(config,"Jms")).booleanValue());
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}
	
	public String getHistoric() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"Historic"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	public String getLogLevel() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"LogLevel"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}

	public String getTraceLevel() throws HeroException
	{
		try {
			return ((String)server.getAttribute(config,"TraceLevel"));
		} catch (Exception e) {
			throw new HeroException(e.getMessage());
		}
	}	
}