package hero.struts.forms;

import java.util.AbstractCollection;

import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for Activity.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>role</b> - The name of the hook.  [REQUIRED]
 * <li><b>deadline</b> - The event for this hook.  [REQUIRED]
 * <li><b>description</b> - The hook type [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */

public final class ActivityForm extends ActionForm  {


// =================================================== Instance Variables


	/**
	 * The activity role
	 */
	private String role = null;


	/**
	 * The activity deadline
	 */

	private String deadline = null;

	/**
	 * The activity description
	 */

         private String description = null;

	/**
	 * Type atribute
	 */

         private String type = null;

	/**
	 * Anticipable atribute
	 */

         private String anticipable = null;

	/**
	 * Automatic atribute
	 */

         private String automatic = null;

	// =========================================================== Properties


	/**
	 * Return the activity role
	 */
	public String getRole() {

	return (this.role);

	}

	/**
	 * Set the activity role
	 *
	 */
	public void setRole(String role) {

	this.role = role;

	}

	/**
	 * Return the activity deadline
	 */
	public String getDeadline() {
		if(this.deadline.length()>10)
			return (this.deadline.substring(0,10));
		else
			return (this.deadline);

	}

	/**
	 * Set the activity deadline
	 *
	 */
	public void setDeadline(String deadline) {

	this.deadline =deadline;

	}

	/**
	 * Return the activity description
	 */
	public String getDescription() {

	return (this.description);

	}

	/**
	 * Set the activity description
	 *
	 */
	public void setDescription (String description) {

	this.description = description;

	}

	/**
	 * Return the activity type
	 */
	public String getType() {

	return (this.type);

	}

	/**
	 * Set the activity type
	 *
	 */
	public void setType (String type) {
	this.type = type;
	}

	/**
	 * Return the anticipable value
	 */
	public String getAnticipable() {

	return (this.anticipable);

	}

	/**
	 * Set anticipable value
	 *
	 */
	public void setAnticipable (String anticipable) {

	this.anticipable = anticipable;

	}

	/**
	 * Return the automatic value
	 */
	public String getAutomatic() {

	return (this.automatic);

	}

	/**
	 * Set automatic value
	 *
	 */
	public void setAutomatic (String automatic) {

	this.automatic = automatic;

	}

	// --------------------------------------------------------- Public Methods


	/**
	 * Reset all properties to their default values.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public void reset(ActionMapping mapping, HttpServletRequest request) {

		this.role = null;
		this.deadline = null;
		this.description = null;
		this.type = null;
	}

	/**
	 * Validate the properties that have been set from this HTTP request,
	 * and return an <code>ActionErrors</code> object that encapsulates any
	 * validation errors that have been found.  If no errors are found, return
	 * <code>null</code> or an <code>ActionErrors</code> object with no
	 * recorded error messages.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {

	    ActionErrors errors = new ActionErrors();

	    if (role == null || role.length()==0)
			errors.add("role",
				   new ActionError("error.role.required"));
	    if (deadline == null || deadline.length()==0)
		errors.add("deadline",
			   new ActionError("error.deadline.required"));
	    if (description == null || description.length()==0)
		errors.add("description",
		new ActionError("error.description.required"));

	return (errors);

	}
}
