/*
 * EdgeForm.java
 */
package hero.struts.forms;


import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for the edge.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>nodeIn</b> - The nodeIn of the edge.  [REQUIRED]
 * <li><b>nodeOut</b> - The nodeOut of the edge.  [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */


public final class EdgeForm extends ActionForm  {

    // --------------------------------------------------- Instance Variables

    /**
     * The name of the in node
     */
    private String nodeIn = null;

    /**
     * The name of the out node
     */
    private String nodeOut = null;


    // ----------------------------------------------------------- Properties

    /**
     * Get the nodeIn
     *@return String 
     */
    public String getNodeIn() {
    	return (nodeIn);
    }

    /**
     * Set the nodeIn.
     * @param nodeIn
     */
    public void setNodeIn(String nodeIn) {
        this.nodeIn = nodeIn;
    }

    /**
     * Get the nodeOut
     *@return String 
     */
    public String getNodeOut() {
    	return (nodeOut);
    }

    /**
     * Set the nodeOut.
     * @param nodeOut
     */
    public void setNodeOut(String nodeOut) {
        this.nodeOut = nodeOut;
    }
    
    // --------------------------------------------------------- Public Methods

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */

    public void reset(ActionMapping mapping, HttpServletRequest request) {
	this.nodeIn = null;
        this.nodeOut = null;
        
    }

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public ActionErrors validate(ActionMapping mapping,
                                 HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();
	if (nodeIn == null || nodeIn.length()==0)
	    errors.add("nodeIn",
		       new ActionError("error.nodeIn.required"));
	if (nodeOut == null || nodeOut.length()==0)
	    errors.add("nodeOut",
		       new ActionError("error.nodeOut.required"));

    	return (errors);
    }



}

