/*
 * LoginForm.java
 */
package hero.struts.forms;


import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for the user profile page.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>username</b> - The username.  [REQUIRED]
 * <li><b>password</b> - The password.  [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */


public final class LoginForm extends ActionForm  {
    private String userName = null;
    private String password = null;
   
    /**
     * Get the userName
     *@return String 
     */
    public String getUserName() {
    	return (userName);
    }

    /**
     * Set the userName.
     * @param userName 
     */
    public void setUserName(String newUserName) {
        userName = newUserName;
    }

    /**
     * Get the password
     *@return String 
     */
    public String getPassword() {
    	return (password);
    }

    /**
     * Set the password.
     * @param password 
     */
    public void setPassword(String newPassword) {
        password = newPassword;
    }


    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public void reset(ActionMapping mapping, HttpServletRequest request) {
	userName = null;
        password = null;
        
    }

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public ActionErrors validate(ActionMapping mapping,
                                 HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();
	    if( userName == null || userName.length()==0 ){
            errors.add("userName",new ActionError("error.userName.required"));
	    }

	    if( password == null || password.length()==0 ){
            errors.add("password",new ActionError("error.password.required"));
	    }
    	return (errors);
    }



}

