package hero.struts.forms;

import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;

/**
 * Form bean for property.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>key</b> - The property key.  [REQUIRED]
 * <li><b>value</b> - The property value.  [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */

public final class PropertyForm extends ActionForm  {

	// --------------------------------------------------- Instance Variables

	/**
	 * The property key.
	 */
	private String key = null;


	/**
	 * The property value
	 */
	private String value = null;


	// ----------------------------------------------------------- Properties

	/**
	 * Return the property key.
	 */
	public String getKey() {

	return (this.key);
	}

	/**
	 * Set the property key.
	 *
	 * @param key The new property key
	 */
	public void setKey(String key) {

		this.key = key;
	}

	/**
	 * Return the property value.
	 */
	public String getValue() {

	return (this.value);
	}

	/**
	 * Set the property value.
	 *
	 * @param value The new property value
	 */
	public void setValue(String value) {

		this.value = value;
	}


	// --------------------------------------------------------- Public Methods


	/**
	 * Reset all properties to their default values.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public void reset(ActionMapping mapping, HttpServletRequest request) {

		this.key = null;
		this.value = null;
	}

	/**
	 * Validate the properties that have been set from this HTTP request,
	 * and return an <code>ActionErrors</code> object that encapsulates any
	 * validation errors that have been found.  If no errors are found, return
	 * <code>null</code> or an <code>ActionErrors</code> object with no
	 * recorded error messages.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {

	    ActionErrors errors = new ActionErrors();

	    if (key == null || key.length()==0)
		errors.add("key", new ActionError("error.key.required"));
	    if (value == null || value.length()==0)
	    errors.add("value", new ActionError("error.value.required"));

	return (errors);

	}
}
