/*
 * RoleForm.java
 */
package hero.struts.forms;


import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for the role.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>name</b> - The role name.  [REQUIRED]
 * <li><b>groupName</b> - The group of this role.  [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */


public final class RoleForm extends ActionForm  {

    // --------------------------------------------------- Instance Variables

    /**
     * The name of the role
     */
    private String name = null;

    /**
     * The name of the group
     */
    private String groupName = null;


    // ----------------------------------------------------------- Properties

    /**
     * Get the name
     *@return String 
     */
    public String getName() {
    	return (name);
    }

    /**
     * Set the name.
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Get the group name
     *@return String 
     */
    public String getGroupName() {
    	return (groupName);
    }

    /**
     * Set the group name
     * @param groupName
     */
    public void setGroupName(String groupName) {
        this.groupName = groupName;
    }
    
    // --------------------------------------------------------- Public Methods

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */

    public void reset(ActionMapping mapping, HttpServletRequest request) {
	this.name = null;
        this.groupName = null;
        
    }

    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public ActionErrors validate(ActionMapping mapping,
                                 HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();
	if (name == null || name.length()==0)
	    errors.add("name",
		       new ActionError("error.name.required"));
	if (groupName == null || groupName.length()==0)
	    errors.add("groupName",
		       new ActionError("error.groupname.required"));

    	return (errors);
    }



}

