package hero.struts.forms;

import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for the BnUser.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>name</b> - The name of the user.  [REQUIRED]
 * <li><b>password</b> - The password for this user.  [REQUIRED]
 * <li><b>email</b> - The email [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */

public final class UserForm extends ActionForm  {


// =================================================== Instance Variables


	/**
	 * The user name
	 */
	private String name = null;


	/**
	 * The password of the user
	 */

	private String password = null;

	/**
	 * The email of the user
	 */

         private String email = null;

	/**
	 * The jabber address of the user
	 */

         private String jabber = null;


	// =========================================================== Properties


	/**
	 * Return the name
	 */
	public String getName() {

	return (this.name);

	}

	/**
	 * Set the name
	 *
	 */
	public void setName(String name) {

	this.name = name;

	}

	/**
	 * Return the password
	 */
	public String getPassword() {

	return (this.password);

	}

	/**
	 * Set the password
	 *
	 */
	public void setPassword(String password) {

	this.password = password;

	}

	/**
	 * Return the email
	 */
	public String getEmail() {

	return (this.email);

	}

	/**
	 * Set the email
	 *
	 */
	public void setEmail(String email) {

	this.email = email;

	}

	/**
	 * Return the jabber address
	 */
	public String getJabber() {

	return (this.jabber);

	}

	/**
	 * Set the jabber address
	 *
	 */
	public void setJabber(String jabber) {

	this.jabber = jabber;

	}


	// --------------------------------------------------------- Public Methods


	/**
	 * Reset all properties to their default values.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public void reset(ActionMapping mapping, HttpServletRequest request) {

		this.name = null;
		this.password = null;
		this.email = null;
		this.jabber = null;
	}

	/**
	 * Validate the properties that have been set from this HTTP request,
	 * and return an <code>ActionErrors</code> object that encapsulates any
	 * validation errors that have been found.  If no errors are found, return
	 * <code>null</code> or an <code>ActionErrors</code> object with no
	 * recorded error messages.
	 *
	 * @param mapping The mapping used to select this instance
	 * @param request The servlet request we are processing
	 */
	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {

	    ActionErrors errors = new ActionErrors();

	    if (name == null || name.length()==0)
			errors.add("name",
				   new ActionError("error.username.required"));
	    if (password == null || password.length()==0)
		errors.add("password",
			   new ActionError("error.password.required"));
	    if (email == null || email.length()==0)
		errors.add("email",
		new ActionError("error.email.required"));
	    if (jabber == null || jabber.length()==0)
		errors.add("jabber",
		new ActionError("error.jabber.required"));

	return (errors);

	}
}
