/*
 * UserRoleForm.java
 */
package hero.struts.forms;


import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;


/**
 * Form bean for the node.  This form has the following fields,
 * with default values in square brackets:
 * <ul>
 * <li><b>user</b> - The username.  [REQUIRED]
 * <li><b>role</b> - The user role.  [REQUIRED]
 * </ul>
 *
 * @author Miguel Valdes Faura
 * @version $Revision: 1.1 $ $Date: 2004/07/30 14:57:57 $
 */


public final class UserRoleForm extends ActionForm  {
    
    // --------------------------------------------------- Instance Variables

    /**
     * The name of the user
     */
    private String user = null;

    /**
     * The role of the user
     */
    private String role = null;


    
    // ----------------------------------------------------------- Properties

    /**
     * Get the user
     *@return String 
     */
    public String getUser() {
    	return (user);
    }

    /**
     * Set the user
     * @param user
     */
    public void setUser(String user) {
        this.user = user;
    }

    /**
     * Get the role
     *@return String 
     */
    public String getRole() {
    	return (role);
    }

    /**
     * Set the role
     * @param role
     */
    public void setRole(String role) {
        this.role = role;
    }


    // --------------------------------------------------------- Public Methods

    /**
     * Reset all properties to their default values.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public void reset(ActionMapping mapping, HttpServletRequest request) {
	this.user = null;
	this.role = null;
    }


    /**
     * Validate the properties that have been set from this HTTP request,
     * and return an <code>ActionErrors</code> object that encapsulates any
     * validation errors that have been found.  If no errors are found, return
     * <code>null</code> or an <code>ActionErrors</code> object with no
     * recorded error messages.
     *
     * @param mapping The mapping used to select this instance
     * @param request The servlet request we are processing
     */
    public ActionErrors validate(ActionMapping mapping,
                                 HttpServletRequest request) {
        ActionErrors errors = new ActionErrors();
	    if (user == null || user.length()==0)
			errors.add("user",
				   new ActionError("error.user.required"));
	    if (role == null || role.length()==0)
			errors.add("role",
				   new ActionError("error.role.required"));

    	return (errors);
    }



}

