package hero.session;
/*
 * 02/01/2002 - 15:24:07
 *
 * ProjectSessionEJB.java -
 * Copyright (C) 2002 Ecoo Team
 * valdes@loria.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import hero.entity.EdgeState;
import hero.entity.NodeState;
import hero.interfaces.BnAgentEdgeLocal;
import hero.interfaces.BnAgentEdgeLocalHome;
import hero.interfaces.BnAgentEdgeValue;
import hero.interfaces.BnAgentLocal;
import hero.interfaces.BnAgentLocalHome;
import hero.interfaces.BnAgentValue;
import hero.interfaces.BnEdgeLocal;
import hero.interfaces.BnEdgeLocalHome;
import hero.interfaces.BnEdgeUtil;
import hero.interfaces.BnEdgeValue;
import hero.interfaces.BnNodeHookLocal;
import hero.interfaces.BnNodeHookLocalHome;
import hero.interfaces.BnNodeHookUtil;
import hero.interfaces.BnNodeInterHookLocal;
import hero.interfaces.BnNodeInterHookValue;
import hero.interfaces.BnNodeInterHookLocalHome;
import hero.interfaces.BnNodeInterHookUtil;
import hero.interfaces.BnNodeLightValue;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodePropertyLocal;
import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnNodePropertyLocalHome;
import hero.interfaces.BnNodePropertyUtil;
import hero.interfaces.BnNodeUtil;
import hero.interfaces.BnNodeValue;
import hero.interfaces.BnIterationLocal;
import hero.interfaces.BnIterationLocalHome;
import hero.interfaces.BnIterationUtil;
import hero.interfaces.BnIterationLightValue;
import hero.interfaces.BnProjectHookLocal;
import hero.interfaces.BnProjectHookLocalHome;
import hero.interfaces.BnProjectHookUtil;
import hero.interfaces.BnProjectInterHookLocal;
import hero.interfaces.BnProjectInterHookLocalHome;
import hero.interfaces.BnProjectInterHookUtil;
import hero.interfaces.BnProjectLightValue;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPropertyLocal;
import hero.interfaces.BnProjectPropertyLocalHome;
import hero.interfaces.BnProjectPropertyUtil;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.BnProjectUtil;
import hero.interfaces.BnProjectValue;
import hero.interfaces.BnRoleLocal;
import hero.interfaces.BnRoleLocalHome;
import hero.interfaces.BnRoleUtil;
import hero.interfaces.BnRoleValue;
import hero.interfaces.BnUserLightValue;
import hero.interfaces.BnUserLocal;
import hero.interfaces.BnUserLocalHome;
import hero.interfaces.BnUserUtil;
import hero.interfaces.DeadlineEjbTimerSessionLocal;
import hero.interfaces.DeadlineEjbTimerSessionLocalHome;
import hero.interfaces.DeadlineEjbTimerSessionUtil;
import hero.interfaces.EngineLocal;
import hero.interfaces.EngineLocalHome;
import hero.interfaces.InvalidValueException;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSessionLocal;

import hero.interfaces.UserServiceLocal;
import hero.interfaces.UserServiceLocalHome;
import hero.interfaces.BnInitiatorMapperLocal;
import hero.interfaces.BnInitiatorMapperLocalHome;
import hero.interfaces.BnInitiatorMapperUtil;

import hero.util.BonitaParsing;
import hero.util.BonitaConfig;
import hero.util.BonitaProjectLocator;
import hero.util.BonitaServiceException;
import hero.util.DeleteEdgeException;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.AdminException;
import hero.util.AccessException;
import hero.util.NodeExecutingException;
import hero.util.ServerType;
import hero.util.StrutsEdgeValue;
import hero.util.StrutsNodeValue;
import hero.util.values.BonitaEdgeValue;
import hero.util.values.BonitaHookValue;
import hero.util.values.BonitaIterationValue;
import hero.util.values.BonitaNodeValue;
import hero.util.values.BonitaProjectValue;
import hero.util.values.BonitaPropertyValue;
import hero.util.values.BonitaRoleValue;
import hero.util.values.BonitaUserValue;
import hero.interfaces.JMSServicesSessionUtil;
import hero.interfaces.JMSServicesSessionLocalHome;
import hero.interfaces.JMSServicesSessionLocal;

import hero.interfaces.BnRoleMapperLocal;
import hero.interfaces.BnRoleMapperLocalHome;
import hero.interfaces.BnRoleMapperUtil;

import hero.interfaces.BnNodePerformerAssignLocal;
import hero.interfaces.BnNodePerformerAssignLocalHome;
import hero.interfaces.BnNodePerformerAssignUtil;
import hero.interfaces.BnNodePerformerAssignValue;
import hero.interfaces.Constants.Nd;

import java.rmi.RemoteException;
import java.util.ArrayList;

import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.naming.Context;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;

/**
 *
 * Project Session Bean is an stateful session bean that provides the interface for workflow process
 * definition. By means of this API we can create workflow processes, project activities, transitions between activities ,
 * users and roles, properties, hooks, external agents...<br><br>
 *
 * A Bonita Workflow process is composed by the activities (nodes) and the connections between these
 * activities (edges). For each project, the administrator/s set the list of responsibles to execute these
 * activites (users and roles). In the same way, the Project Session Bean API offers a great number of
 * functionalities for working with activities behaviors (hooks) and workflow relevant data (properties).<br><br>
 *
 * This API can be used on different contexts depending on the needs of your application, so you can instantiate
 * an existing project in order to perform set, get, update and remove operations, you can create a new workflow
 * project (by instantiating a project that does not exists), you can clone an existing project or creates a
 * new workflow instante of a project.
 *
 * <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
 * <br>
 * First of all you have to import the Project Session files:<br>
 * <br>
 * import hero.interfaces.ProjectSessionLocalHome;<br>
 * import hero.interfaces.ProjectSessionLocal;<br>
 * import hero.interfaces.ProjectSessionHome;<br>
 * import hero.interfaces.ProjectSession;<br>
 * import hero.interfaces.ProjectSessionUtil;<br>
 * <br>
 * Now, it is time to create the Project Session instance:<br>
 * <br>
 * Like this if you want to use local interfaces:<br><br>
 * ProjectSessionLocalHome projecth = (ProjectSessionLocalHome)hero.interfaces.ProjectSessionUtil.getLocalHome();<br>
 * ProjectSessionLocal projectsession = projecth.create();<br>
 * <br>
 * or like this if you use remote interfaces:<br>
 * <br>
 * ProjectSessionHome userh = (ProjectSessionHome)hero.interfaces.ProjectSessionUtil.getHome();<br>
 * ProjectSession projectsession = projecth.create();<br>
 * <br>
 * <br>
 * - If you want to use Project Session API for a specific project, you have to init this project.<br><br>
 * Depending of the workflow process type you want, use:<br><br>
 *
 * projectsession.initProject("yourProject"); // for cooperative projects<br>
 * <br>or<br>
 * projectsession.initModel("yourProject"); // for workflow models<br>
 * <br>
 * - In order to clone an existing project, you have to call the initProject method:<br><br>
 * projectsession.initProject("Project","CloneProject");<br><br>
 *
 * - If you want to instantiate a project, you have to do the following:<br><br>
 * projectsession.instantiateProject("Project");<br><br>
 *
 * Now you can use Project Session methods...
 *
 *
 * @ejb:bean name="ProjectSession"
 *           display-name="ProjectSession Bean"
 *           type="Stateful"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/ProjectSession"
 *           local-jndi-name="ejb/hero/ProjectSession_L"
 *           reentrant="true"
 *
 * @ejb:ejb-ref	ejb-name="ProjectSession"
 *              	view-type="remote"
 *              	ref-name="ProjectSession"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 *
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin"
 * @jonas.bean
 *      ejb-name="ProjectSession"
 *      jndi-name="ejb/hero/ProjectSession"
 *
 * @jboss.container-configuration name="Standard Stateful SessionBean for Bonita"
 *
 * @copyright INRIA
 * @author Miguel Valdes
 *
 **/

public class ProjectSessionBean implements SessionBean, EventConstants{

	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger trace =	Logger.getLogger(ProjectSessionBean.class);
	private static final Logger thelog = Logger.getLogger("log");

	// -------------------------------------------------------------------------
	// Members
	// -------------------------------------------------------------------------

	private SessionContext mContext;

	private BnProjectLocalHome pHome;
	private BnProjectLocal mProject=null;
	private JMSServicesSessionLocal jms = null;
	private DeadlineEjbTimerSessionLocal dlSession;
	private String caller=null;

	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------

	// -------------------------------------------------------------------------
	// Add business methods
	// -------------------------------------------------------------------------

	/**
	 * Add a user to this project (This user must exist at bonita database)
	 *
	 * @param userName The name of the user to associate to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addUser(String userName) throws HeroException {

		trace.debug(" Parameter: name=" + userName + " process name = "+this.getName()+ " started by " + this.getCaller());
		UserServiceLocalHome ushome;
		UserServiceLocal ul;
		if (!isAdminOfProject())
		{
			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnRoleLocal mRole;
		BnUserLocal mUser;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			ul = ushome.create();
			mUser = ul.findUser(userName);
		} catch (Exception fe) {

			trace.error("User does not exist " + fe.getMessage());
			throw new HeroException("User does not exist");
		}
		Collection cUsers = mProject.getBnUsers();
		if (!cUsers.contains(mUser))
		{
			cUsers.add(mUser);
			if (!userName.equals(Constants.ADMIN))
				this.addRole(userName, "BnUser BnRole");
			this.setUserRole(userName, userName);
			if (jms!=null)
				jms.sendProjectEvent(ADDUSERPROJECT,this.getName(),userName);
		}
	}

	/**
	 * Add a node to the project. This method creates a node with the corresponding
	 * node type and assign to it the hero.interfaces.Constants.InitialRole role.<br><br>
	 *
	 * Possible node types are:<br>
	 * <br>
	 * 	hero.interfaces.Constants.Nd.AND_JOIN_NODE // Manual and join node <br>
	 *	hero.interfaces.Constants.Nd.OR_JOIN_NODE // Manual or join node <br>
	 *	hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE // Automatic and join node <br>
	 *	hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE // Automatic or join node <br>
	 *
	 * @param name The name of the node to add to the project
	 * @param nodeType The type of the node to add to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void addNode(String name, int nodeType) throws HeroException {

		trace.debug("parameter: name= " + name + " nodeType= " + name +  " process name = "+this.getName()+ " started by " + this.getCaller());
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}

		try {
			BnNodeLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("Node "+ name+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			try {
				if (name.equals(""))
					throw new HeroException("Node name can not be empty...");
				BnNodeLocal nd = nHome.create(name, nodeType);
				// Models are not executable
				if (mProject.getType().equals(Constants.Pj.MODEL))
					nd.setState(Constants.Nd.INITIAL);
				BnNodeValue ndata = nd.getBnNodeValue();
				ndata.setCreationDate(new java.util.Date());
				nd.setBnNodeValue(ndata);

				Collection cNodes = mProject.getBnNodes();
				cNodes.add(nd);
				//nd.setBnProject(mProject);

				String creator = this.getCaller();
				nd.setCreator(creator);
				nd.setExecutor(creator); // Only for JOnAS consistency
				BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
				BnRoleLocal role = rHome.findByName(hero.interfaces.Constants.INITIALROLE,mProject.getId());
				nd.setBnRole(role);

				// Tell the world
				if (jms!=null)
					jms.sendNodeEvent(ADDNODE,this.getName(),name,nodeType,ndata.getState(),this.getCaller());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {fe.printStackTrace();

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {

				trace.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
		}
	}


	/**
	 * Add a subProcess node to the project. This method creates the subProject from an existing
	 * project and creates the node associated to it. The type of created node is
	 * hero.interfaces.Constants.Nd.SUB_PROCESS_NODE
	 *
	 * @param name The name of the node to add to the project
	 * @param projectName the name of the subProcess
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void addNodeSubProcess(String name, String projectName) throws HeroException {
		this.addNodeSubProcess(name, projectName, EventConstants.DEFAULTPROJECTVERSION);
	}

	/**
	 * Add a subProcess node to the project. This method creates the subProject from an existing
	 * project and creates the node associated to it. The type of created node is
	 * hero.interfaces.Constants.Nd.SUB_PROCESS_NODE
	 *
	 * @param name The name of the node to add to the project
	 * @param projectName the name of the subProcess
	 * @param projectVersion the version of the subProcess
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void addNodeSubProcess(String name, String projectName, String projectVersion) throws HeroException {

		trace.debug("parameters : name=" + name + " project=" + projectName +  " version = " + projectVersion + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try{
			// A cooperative project can only add subProcess activities pointing to another cooperative project
			if (mProject.getType().equals(Constants.Pj.COOPERATIVE))
			{
				BnProjectLocalHome pHome;
				try {
					pHome = hero.interfaces.BnProjectUtil.getLocalHome();
				} catch (javax.naming.NamingException be) {
					trace.error(be.getMessage());
					throw new HeroException(be.getMessage());
				}

				try {
					BnProjectLocal pl = pHome.findByName(projectName);
					pl.setParent(mProject.getName()+"_node-"+name);
					if (!pl.getType().equals(Constants.Pj.COOPERATIVE))
						throw new HeroException("Cooperatives projects can only reference to cooperative subprocesses !");
					} catch (javax.ejb.FinderException nn) {
						throw new HeroException("This cooperative project does not exist or is not a cooperative one !");
				}
			}

			this.addNode(name,Constants.Nd.SUB_PROCESS_NODE);
			mProject.setParent(mProject.getName());
			BnNodeLocal nd = mProject.getBnNode(name);
			nd.setReference(projectName+"_version"+projectVersion);

		}catch (Exception ans){
			ans.printStackTrace();
			trace.error("Error in addNodeSubProcess " + ans.getMessage());
			throw new HeroException("Error in addNodeSubProcess");
		}
	}


	/**
	 * Add an Agent to the Project
	 *
	 * @param name The name of the Agent to add to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addAgent(String name) throws HeroException {

		trace.debug("parameter: name=" + name +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}

		try {
			BnAgentLocal nf = nHome.findByName(name, mProject.getId());
			throw new HeroException("BnAgent "+ name+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {

			trace.error(nn.getMessage());

			try {
				BnAgentLocal ag = nHome.create(name);
				BnAgentValue adata = ag.getBnAgentValue();
				adata.setCreationDate(new java.util.Date());
				ag.setBnAgentValue(adata);
				Collection cAgents = mProject.getBnAgents();
				cAgents.add(ag);
				String creator = this.getCaller();
				BnUserLocalHome uHome =	hero.interfaces.BnUserUtil.getLocalHome();
				BnUserLocal user = uHome.findByName(creator);
				ag.setCreator(user);

			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			} catch (FinderException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (javax.naming.NamingException ne) {

				trace.error(ne.getMessage());
				throw new HeroException(ne.getMessage());
			}
		}
	}

	/**
	 * Add an edge to the project. Creates a transition between two nodes by specifying
	 * the name of this edge. <strong> No more implemented </strong>
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addEdge(String name, String in, String out)
	throws HeroException {

		trace.error("parameters: name=" + name + " in=" + in + " out=" + out +  " process name = " + this.getName()+ " started by " + this.getCaller());
		throw new HeroException("no more implemented");
	}

	/**
	 * Add an edge between two nodes of the project [in] ---- [out]. This operation refresh
	 * the out node state. If new edge forms a cycle an exception is throwed.
	 *
	 * @param in the name of the in Node
	 * @param out the name of the out Node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 * @throws HeroException
	 *
	 **/
	public String addEdge(String in, String out) throws HeroException {
		ArrayList visited = new ArrayList();

		trace.debug("parameters: in=" + in + " out:" + out +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnNodeLocal mInNode;
		BnNodeLocal mOutNode;
		BnEdgeLocal mEdge;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mInNode = nHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {

			trace.error("Out Node does not exist " + fe.getMessage());
			throw new HeroException("Out Node does not exist");
		}
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}

		if (checkStructuralConstraints(visited, in, out))
			throw new hero.util.EdgeCycleException("This edge add a cycle");

		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnNode(mInNode);
					mEdge.setOutBnNode(mOutNode);

					if (jms!=null)
						jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),mInNode.getName(),mOutNode.getName(),this.getCaller());

				} catch (InvalidValueException ie) {

					trace.error(ie.getMessage());
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {

					trace.error(ce.getMessage());
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException("Cannot add edge. BnEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException("Cannot add edge. " + out + " is already started");
		}

		// update the current graph state
		int edgeState = hero.interfaces.Constants.edgeTransition[Constants.CONDITION_TRUE][mInNode.getState()];
		this.setEdgeState(mEdge, edgeState);
		/*if (mOutNode.getState()==Constants.Nd.READY)
		 this.setNodeState(mOutNode, Constants.Nd.INITIAL);*/

		EdgeState edgeStateClass = EdgeState.make(mOutNode.getType());
		int newOperation = edgeStateClass.state(mOutNode);
		// It doesn't consider CANCEL operation. CANCEL propagation is done by the EngineBean
		if (newOperation != Constants.Nd.CANCEL) {
			NodeState nodeState = NodeState.make(mOutNode.getType(), mOutNode.getAnticipable());
			int newState = nodeState.computeState(mOutNode, newOperation);

			if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
				throw new HeroException("AddEdge generated a bad Node transition");
			}
			this.setNodeState(mOutNode, newState);
		}
		return mEdge.getName();
	}

	/**
	 * Add an AgentEdge to the Project [in] ---- [out]
	 *
	 * @param in the name of the in Agent
	 * @param out the name of the out Node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 * @throws HeroException
	 *
	 **/
	public String addAgentEdge(String in, String out) throws HeroException {

		trace.debug("parameters: in=" + in + " out:" + out +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnAgentEdgeLocalHome eHome;
		BnNodeLocalHome nHome;
		BnAgentLocalHome aHome;
		BnAgentLocal mInAgent;
		BnNodeLocal mOutNode;
		BnAgentEdgeLocal mEdge;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			aHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mInAgent = aHome.findByName(in, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In Agent does not exist " + fe.getMessage());
			throw new HeroException("In Agent does not exist");
		}
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mOutNode = nHome.findByName(out, mProject.getId());
		} catch (FinderException fe) {

			trace.error("Out Node does not exist " + fe.getMessage());
			throw new HeroException("Out Node does not exist");
		}
		try {
			eHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		if (checkDynamics(out)) { // check state of Nodes
			if (checkDuplicateAgentEdge(in, out)) {
				try {
					mEdge = eHome.create();
					mEdge.setBnProject(mProject);
					mEdge.setInBnAgent(mInAgent);
					mEdge.setOutBnNode(mOutNode);

				} catch (InvalidValueException ie) {

					trace.error(ie.getMessage());
					throw new HeroException(ie.getMessage());
				} catch (CreateException ce) {

					trace.error(ce.getMessage());
					throw new HeroException(ce.getMessage());
				}
			} else {
				throw new hero.util.DuplicatedEdgeException("Cannot add agentEdge. AgentEdge between nodes already exist");
			}
		} else {
			throw new hero.util.NodeStartedException(
					"Cannot add agentEdge. " + out + " is already started");
		}

		// update the current graph state
		this.setAgentEdgeState(mEdge, hero.interfaces.Constants.Ed.INITIAL);

		EdgeState edgeState = EdgeState.make(mOutNode.getType());
		int newOperation = edgeState.state(mOutNode);

		NodeState nodeState = NodeState.make(mOutNode.getType(), mOutNode.getAnticipable());
		int newState = nodeState.computeState(mOutNode, newOperation);

		if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
			throw new HeroException("AddEdge generated a bad Node transition");
		}
		this.setNodeState(mOutNode, newState);
		return mEdge.getName();
	}

	/**
	 * Add a role to the project. Creates a role within this project. The role is specific of this project
	 *
	 * @param name the name of the role
	 * @param description the description of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 * @throws HeroException
	 *
	 **/
	public void addRole(String roleName, String description)
	throws HeroException {
		BnRoleLocalHome roleh;

		trace.debug("parameters: roleName=" + roleName + " description=" + description +  " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnRoleLocal nf = roleh.findByName(roleName, mProject.getId());
		} catch (javax.ejb.FinderException nn) {
			BnRoleValue rv = new BnRoleValue();
			rv.setName(roleName);
			rv.setDescription(description);
			try {
				BnRoleLocal rl = roleh.create(rv);
				mProject.getBnRoles().add(rl);
				//rl.setBnProject(mProject);
				if (jms!=null)
					jms.sendRoleEvent(ADDROLE,this.getName(),roleName);
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Add a new iteration between two nodes. This methods sets an iteration which is stopped when
	 * the lastNode property "iterate" is false.
	 *
	 * @param from the name of the first node
	 * @param to the name of the last node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 * @throws HeroException
	 *
	 **/
	public void addIteration(String from, String to, String condition) throws HeroException {
		ArrayList visited = new ArrayList();

		trace.debug("parameters: from=" + from + " to=" + to + " condition=" + condition + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject()) {
			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (this.checkStructuralConstraints(visited, from, to))	{
			this.copyIteration(from, to, condition);
		}
		else {
			throw new HeroException("This iteration is not possible...");
		}
	}


	/**
	 * Internal method used in processes instantiation.
	 *
	 * @param from
	 * @param to
	 * @param condition
	 * @throws HeroException
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 */
	public void copyIteration(String from, String to, String condition) throws HeroException {
		BnIterationLocalHome iteh;

		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			iteh.findByFromTo(from, to, mProject.getId());
			throw new HeroException("Iteration already exist between: " + from + " and " + to + " nodes, in project " + mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			BnIterationLightValue iv = new BnIterationLightValue();
			iv.setFromNode(from);
			iv.setToNode(to);
			iv.setCondition(condition);
			try {
				BnIterationLocal il = iteh.create(iv);
				mProject.getBnIterations().add(il);
				if (jms!=null) {
					jms.sendIterationEvent(ADDITERATION,this.getName(), from, to);
				}
			} catch (Exception ce) {
				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Delete an iteration between two nodes.
	 *
	 * @param from the name of the first node
	 * @param to the name of the last node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 * @throws HeroException
	 *
	 **/
	public void deleteIteration(String from, String to) throws HeroException {

		trace.debug("parameters: from=" + from + " to=" + to +" process name = " + this.getName()+ " started by " + this.getCaller());
		BnIterationLocalHome iteh;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			BnIterationLocal it = iteh.findByFromTo(from, to, mProject.getId());
			it.remove();
		} catch (javax.ejb.FinderException nn) {

			trace.error("Iteration does not exist: "+ from +" and "+to+" nodes, in project "+ mProject.getName());
			throw new HeroException("Iteration does not exist: "+ from +" and "+to+" nodes, in project "+ mProject.getName());
		} catch (javax.ejb.RemoveException rn) {

			trace.error("Error when deleting activity iteration: "+ from +" and "+to+" nodes, in project "+ mProject.getName());
			throw new HeroException("Error when deleting activity iteration: "+ from +" and "+to+" nodes, in project "+ mProject.getName());
		}
	}

	/**
	 * Add hook to project. Add an existing hook file to the project. This hook type uses a
	 * Java file loaded at run time. The hookName represents the class java file to be loaded by the system at run time.
	 * These classes must be in the application server classpath to be correctly executed.<br>
	 *
	 * The possible values of the argumets are:<br><br>
	 * <strong>eventName:<br></strong>
	 *	hero.interfaces.Constants.Pj.ONINSTANTIATE<br><br>
	 *	hero.interfaces.Constants.Pj.ONTERMINATE<br><br>
	 *
	 * @param hookName the name of the hook
	 * @param eventName the event
	 * @param hookType the type
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addHook(String hookName, String eventName, int hookType)
	throws HeroException {

		trace.debug("parameters: hookName=" + hookName + " eventName=" + eventName +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal pf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("BnProject hook "+ hookName	+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException pn) {
			try {
				BnProjectHookLocal h =	pHome.create(hookName, eventName, hookType);
				h.setBnProject(mProject);
			} catch (CreateException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * Add hook to a node. Add an existing hook file to the node. This type of hook use a
	 * Java file loaded at run time. The hookName represents the class java file to be loaded by the system at run time.
	 * These classes must be in the application server classpath to be correctly executed.<br>
	 * The possible values of the arguments are:<br><br>
	 * <strong>eventName:<br></strong>
	 * 	hero.interfaces.Constants.Nd.BEFORESTART<br>
	 *      hero.interfaces.Constants.Nd.AFTERSTART<br>
	 *	hero.interfaces.Constants.Nd.BEFORETERMINATE<br>
	 *	hero.interfaces.Constants.Nd.AFTERTERMINATE<br>
	 *	hero.interfaces.Constants.Nd.ONCANCEL<br>
	 *	hero.interfaces.Constants.Nd.ANTICIPATE<br><br>
	 *	hero.interfaces.Constants.Nd.ONDEADLINE<br><br>
	 *	hero.interfaces.Constants.Nd.ONREADY<br><br>
         *
	 *<br><br>
	 * <strong>hookType:<br></strong>
	 * 	hero.interfaces.Constants.Hook.JAVA<br><br>
	 *      hero.interfaces.Constants.Hook.BSINTERACTIVE<br><br>
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addNodeHook(String nodeName,String hookName,String eventName,int hookType)	throws HeroException {

		trace.debug("parameters: node="+ nodeName+ " hookName="+ hookName+ " eventName="	+ eventName +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("BnNode hook "+ hookName+ " already exist in node "+ nodeName);
		} catch (javax.ejb.FinderException nn) {
			try {
				BnNodeHookLocal h = pHome.create(hookName, eventName, hookType);
				h.setBnNode(nd);
			} catch (CreateException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	}

    /**
     * Add/Update a mapper for the INITIATOR role. This type of mapper use a
     * Java  file loaded at run time.
     *
     * @param mapperName the name of the mapper
     * @param mapperType the type of the mapper
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     * @throws HeroException
     *
     **/
    public void addInitiatorMapper(String mapperName,int mapperType) throws HeroException {

            trace.debug("addInitiatorMapper/parameters: projectName="+ mProject.getName()+ " mapperName="+ mapperName+ " mapperType="+ mapperType+ " process name = " + this.getName()+ " started by " + this.getCaller());

            BnInitiatorMapperLocalHome mapperh;
            BnInitiatorMapperLocal ppLocal = null;
            BnProjectLocal project = null;
            if (!isAdminOfProject())
            {

                    trace.error("Only the Admin of the project is allowed to do this operation... ");
                    throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
            }
            try {
                    mapperh = BnInitiatorMapperUtil.getLocalHome();
            } catch (javax.naming.NamingException ne) {

                    trace.error(ne.getMessage());
                    throw new HeroException(ne.getMessage());
            }
            // if the project has not already an initiator mapper put one. Otherwise update it.
            ppLocal = mProject.getBnInitiatorMapper();
            if (ppLocal != null){
                    ppLocal.setName(mapperName);
                    ppLocal.setType(mapperType);
            }
            else {
                    try {
                            ppLocal = mapperh.create(mapperName, mapperType);
                            ppLocal.setBnProject(mProject);
                    } catch (CreateException fe) {

                            trace.error(fe.getMessage());
                            throw new HeroException(fe.getMessage());
                    } catch (InvalidValueException ie) {

                            trace.error(ie.getMessage());
                            throw new HeroException(ie.getMessage());
                    }
            }
    }


	/**
	 * Add/Update an existing mapper to the role. This type of mapper use a
	 * Java  file loaded at run time.
	 *
	 * @param roleName the name of the role
	 * @param mapperName the name of the mapper
	 * @param mapperType the type of the mapper
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addRoleMapper(String roleName,String mapperName,int mapperType)	throws HeroException {
		// logs trace

		trace.debug("parameters: roleName="+ roleName+ " mapperName="+ mapperName+ " mapperType="+ mapperType+ " process name = " + this.getName()+ " started by " + this.getCaller());

		BnRoleMapperLocalHome mapperh;
		BnRoleLocalHome roleh;
		BnRoleMapperLocal ppLocal = null;
		BnRoleLocal role = null;
		if (!isAdminOfProject())
		{
			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			mapperh = BnRoleMapperUtil.getLocalHome();
			roleh = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		// Get objet role local with findByName from BnRoleBean
		try {
			role = roleh.findByName(roleName, mProject.getId());
		} catch (FinderException pfe) {

			trace.error(pfe.getMessage());
			throw new HeroException("The bnRole does not exist for role : " + roleName);
		}
		// if the role has not already a mapper put one. Otherwise update it.
		ppLocal = role.getBnRoleMapper();
		if (ppLocal != null){
			ppLocal.setName(mapperName);
			ppLocal.setType(mapperType);
		}
		else {
			try {
				ppLocal = mapperh.create(mapperName, mapperType);
				ppLocal.setBnRole(role);
			} catch (CreateException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * Add/Update an existing performerAssign to the node. This type of performerAssign use a
	 * Java  file loaded at run time.
	 *
	 * @param nodeName the name of the node
	 * @param performerAssignName the name of the performerAssign
	 * @param performerAssignType the type of the performerAssign
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addNodePerformerAssign(String nodeName,String performerAssignName,int performerAssignType, String propertyName)	throws HeroException {
		// logs trace

		trace.debug("addNodePerformerAssign: node = "+ nodeName+ " performerAssignName = "+ performerAssignName);

		BnNodePerformerAssignLocalHome performerAssignh;
		BnNodeLocalHome nodeh;
		BnNodePerformerAssignLocal ppLocal = null;
		BnNodeLocal node = null;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			performerAssignh = BnNodePerformerAssignUtil.getLocalHome();
			nodeh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		// Get object node local with findByName from BnNodeBean
		try {
			node = nodeh.findByName(nodeName, mProject.getId());
		} catch (FinderException pfe) {
			throw new HeroException("The bnNode does not exist for node : " + nodeName);
		}
		// if the node has not already a performerAssign put one. Otherwise update it.
		ppLocal = node.getBnNodePerformerAssign();
		if (ppLocal != null){
			ppLocal.setName(performerAssignName);
			ppLocal.setType(performerAssignType);
			ppLocal.setPropertyName(propertyName);
		}
		else {
			try {
				ppLocal = performerAssignh.create(performerAssignName, performerAssignType, propertyName);
				ppLocal.setBnNode(node);
			} catch (CreateException fe) {
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {
				throw new HeroException(ie.getMessage());
			}
		}
	}
	
	/**
	 * Remove the performer assign defined for this node
	 *
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 * @throws RemoveException 
	 * @throws EJBException 
	 *
	 **/
	public void removeNodePerformerAssign(String nodeName)	throws HeroException, EJBException, RemoveException {
		// logs trace
		trace.debug("removeNodePerformerAssign: node = "+ nodeName+ " performerAssignName = ");

		BnNodePerformerAssignLocalHome performerAssignh;
		BnNodeLocalHome nodeh;
		BnNodePerformerAssignLocal ppLocal = null;
		BnNodeLocal node = null;
		if (!isAdminOfProject())
		{
			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			performerAssignh = BnNodePerformerAssignUtil.getLocalHome();
			nodeh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		// Get object node local with findByName from BnNodeBean
		try {
			node = nodeh.findByName(nodeName, mProject.getId());
		} catch (FinderException pfe) {
			throw new HeroException("The bnNode does not exist for node : " + nodeName);
		}
		// if the node has a performer assign remove it.
		ppLocal = node.getBnNodePerformerAssign();
		if (ppLocal != null){
			ppLocal.remove();
		}		
	}

	/**
	 * Add interhook to a node. Set a new hook to this node by setting the eventName, hookType and
	 * script values.<br><br>
	 *
	 * The possible values of the argumets are:<br><br>
	 * <strong>eventName:<br></strong>
	 * 	hero.interfaces.Constants.Nd.BEFORESTART<br>
	 *  hero.interfaces.Constants.Nd.AFTERSTART<br>
	 *	hero.interfaces.Constants.Nd.BEFORETERMINATE<br>
	 *	hero.interfaces.Constants.Nd.AFTERTERMINATE<br>
	 *	hero.interfaces.Constants.Nd.ONCANCEL<br>
	 *	hero.interfaces.Constants.Nd.ANTICIPATE<br><br>
	 *	hero.interfaces.Constants.Nd.ONDEADLINE<br><br>
	 *	hero.interfaces.Constants.Nd.ONREADY<br><br>
	 *
	 * <strong>hookType:<br></strong>
	 *
	 *  hero.hook.Hook.BSINTERACTIVE<br><br>
	 *
	 * <strong>script:<br></strong>
	 * The beanshell source code associated to this hook.<br><br>
	 *
	 * <strong>The following sample code illustrates how to use this method:<br><br></strong>
	 *
	 *  ...<br>
	 *  String script = "import hero.interfaces.*;\n"<br>
	 * 		            + "import hero.interfaces.BnNodeLocal;\n"<br>
	 *			        + "beforeStart (Object b,Object n) {\n\n\n"<br>
	 *					+ "System.out.println(\"Hook Test\");\n"<br>
	 *			        + "}";<br>
	 *  <strong>yourProjectSessionInstance</strong>.addNodeInterHook("NodeName","Hook_Name",Constants.Nd.BEFORESTART,hero.hook.Hook.BSINTERACTIVE,script);
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addNodeInterHook(String nodeName,String hookName,String eventName,int hookType,String value)
	throws HeroException {

		trace.debug("parameters: node="+ nodeName+ " hookName="+ hookName+ " eventName="+ eventName+ " value="+ value +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}

		BnNodeLocal nd = mProject.getBnNode(nodeName);
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnNodeInterHookLocal nf = pHome.findByName(nd.getId(), hookName);
			throw new HeroException("Interactive node hook "+ hookName+ " already exist in node "+ nodeName);
		} catch (javax.ejb.FinderException nn) {
			try {
				BnNodeInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnNode(nd);
			} catch (CreateException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * Add interhook to the project. Creates new hook associated to all project activites.
	 * See the {@link #addNodeInterHook(String,String,String,int,String) addNodeInterHook} method in order
	 * to identify arguments values.
	 *
	 * @param hookName the name of the hook
	 * @param eventName the name of the event
	 * @param hookType the type of the hook
	 * @param script the value of the script
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void addInterHook(String hookName,String eventName,int hookType,String value)
	throws HeroException {

		trace.debug("parameters: hookName="+ hookName+ " eventName="+ eventName+ " value="+ value +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal nf = pHome.findByName(mProject.getId(), hookName);
			throw new HeroException("Interactive hook "+ hookName+ " already exist in project "+ mProject.getName());
		} catch (javax.ejb.FinderException nn) {
			try {
				BnProjectInterHookLocal h = pHome.create(hookName, eventName, hookType, value);
				h.setBnProject(mProject);
			} catch (CreateException fe) {

				trace.error(fe.getMessage());
				throw new HeroException(fe.getMessage());
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			}
		}
	}

	/**
	 * Active a workflow process (model/cooperative)
	 *
	 * @param name the name of the workflow process
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void activeProcess() throws HeroException {
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
        if ((mProject.getStatus().equals(Constants.Pj.HIDDEN))||(mProject.getStatus().equals(Constants.Pj.UNDEPLOYED))) {
			mProject.setStatus(Constants.Pj.ACTIVE);
            try{
                if (mProject.getType().equals(Constants.Pj.MODEL))
                    BonitaProjectLocator.getInstance().cleanModelValue(this.getName(), this.getVersion());
            }catch(BonitaServiceException se){throw new HeroException (se.getMessage());}
        }
		else
			throw new HeroException("This project is already ACTIVE...");
	}

	/**
	 * Hide a workflow process (model/cooperative).
	 * This method also clean the associated cache to this model.
	 *
	 * @param name the name of the workflow process
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void hideProcess() throws HeroException {
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (mProject.getStatus().equals(Constants.Pj.ACTIVE))
		{
			mProject.setStatus(Constants.Pj.HIDDEN);
			try{
				if (mProject.getType().equals(Constants.Pj.MODEL))
					BonitaProjectLocator.getInstance().cleanModelValue(this.getName(), this.getVersion());
			}catch(BonitaServiceException se){throw new HeroException (se.getMessage());}
		}

		else
			throw new HeroException("This projects is already HIDDEN...");
	}

    /**
     * Undeploy a workflow process (model/cooperative).
     * This method does not clean the associated cache to this model.
     *
     * @param name the name of the workflow process
     *
     * @ejb:interface-method view-type="both"
     * @ejb:transaction type="Required"
     * @throws HeroException
     *
     **/
    public void unDeployProcess() throws HeroException {
        if (!isAdminOfProject())
        {

            trace.error("Only the Admin of the project is allowed to do this operation... ");
            throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
        }
        if (mProject.getStatus().equals(Constants.Pj.ACTIVE))
        {
            mProject.setStatus(Constants.Pj.UNDEPLOYED);
            try{
                if (mProject.getType().equals(Constants.Pj.MODEL))
                    BonitaProjectLocator.getInstance().cleanModelValue(this.getName(), this.getVersion());
            }catch(BonitaServiceException se){throw new HeroException (se.getMessage());}
        }
    }
	// -------------------------------------------------------------------------
	// Delete business methods
	// -------------------------------------------------------------------------

	/**
	 * Delete a node from the project. If this node is in executing state, the
	 * method throws an execption
	 *
	 * @param name the name of the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNode(String name) throws HeroException {
		// check deletion conditions

		trace.debug("parameter: name=" + name +  " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		if (nd.isExecuting()) {
			throw new NodeExecutingException("Cannot delete node. It is executing");
		}
		/*		if (nd.isTerminated()) {
		 throw new NodeTerminatedException("Cannot delete node. It is terminated");
		 }
		 if (nd.isCancelled()) {
		 throw new NodeCancelledException("Cannot delete node. It is cancelled");
		 }*/
		if (this.getNodeType(name)==Constants.Nd.SUB_PROCESS_NODE)
		{
			try {
				BnProjectLocal plocal = pHome.findByNameVersion(name,EventConstants.DEFAULTPROJECTVERSION);
				Collection nodes = plocal.getBnNodes();
				Iterator nds = nodes.iterator();
				while (nds.hasNext())            // Verifies if the project has any subProcesses
				{
					BnNodeLocal nod = (BnNodeLocal)nds.next();
					if (nod.getType()==Constants.Nd.SUB_PROCESS_NODE)
						throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
				}
				plocal.remove();
				if (jms!=null)
					jms.sendProjectEvent(DELETEPROJECT, name);
			} catch (RemoveException pre) {

				trace.error("Cannot remove project " + pre.getMessage());
				throw new HeroException("Cannot remove project " + pre.getMessage());
			} catch (FinderException pfe) {

				trace.error("Cannot find project " + pfe.getMessage());
				throw new HeroException("Cannot find project " + pfe.getMessage());
			}
		}

		Collection edgesIn = nd.getInBnEdges();
		Object[] edai = edgesIn.toArray();
		for (int i = 0; i < edai.length; i++) {
			edgeDeletion((BnEdgeLocal) edai[i]);
		}
		Collection edgesOut = nd.getOutBnEdges();
		Object[] edao = edgesOut.toArray();
		for (int i = 0; i < edao.length; i++) {
			edgeDeletion((BnEdgeLocal) edao[i]);
		}
		try {
			int state = nd.getState();
			int type = nd.getType();
			nd.remove();
			if (jms!=null)
				jms.sendNodeEvent(DELETENODE,this.getName(),name,type,state,this.getCaller());
		} catch (RemoveException re) {

			trace.error("Cannot remove node " + re.getMessage());
			throw new HeroException("Cannot remove node " + re.getMessage());
		}
	}

	/**
	 * Delete a node with type subProcess from the project. If this node is in executing state, the
	 * method throws an execption
	 *
	 * @param name the name of the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteSubProcessNode(String name, String version) throws HeroException {
		// check deletion conditions

		trace.debug("parameter: name=" + name +  " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		if (nd.isExecuting()) {
			throw new NodeExecutingException("Cannot delete node. It is executing");
		}
		/*		if (nd.isTerminated()) {
		 throw new NodeTerminatedException("Cannot delete node. It is terminated");
		 }
		 if (nd.isCancelled()) {
		 throw new NodeCancelledException("Cannot delete node. It is cancelled");
		 }*/
		if (this.getNodeType(name)==Constants.Nd.SUB_PROCESS_NODE)
		{
			try {
				BnProjectLocal plocal = pHome.findByNameVersion(name, version);
				Collection nodes = plocal.getBnNodes();
				Iterator nds = nodes.iterator();
				while (nds.hasNext())            // Verifies if the project has any subProcesses
				{
					BnNodeLocal nod = (BnNodeLocal)nds.next();
					if (nod.getType()==Constants.Nd.SUB_PROCESS_NODE)
						throw new HeroException("Error in delete project: You have to delete subProcesses first !!");
				}
				plocal.remove();
				if (jms!=null)
					jms.sendProjectEvent(DELETEPROJECT, name);
			} catch (RemoveException pre) {

				trace.error("Cannot remove project " + pre.getMessage());
				throw new HeroException("Cannot remove project " + pre.getMessage());
			} catch (FinderException pfe) {

				trace.error("Cannot find project " + pfe.getMessage());
				throw new HeroException("Cannot find project " + pfe.getMessage());
			}
		}

		Collection edgesIn = nd.getInBnEdges();
		Object[] edai = edgesIn.toArray();
		for (int i = 0; i < edai.length; i++) {
			edgeDeletion((BnEdgeLocal) edai[i]);
		}
		Collection edgesOut = nd.getOutBnEdges();
		Object[] edao = edgesOut.toArray();
		for (int i = 0; i < edao.length; i++) {
			edgeDeletion((BnEdgeLocal) edao[i]);
		}
		try {
			int state = nd.getState();
			int type = nd.getType();
			nd.remove();
			if (jms!=null)
				jms.sendNodeEvent(DELETENODE,this.getName(),name,type,state,this.getCaller());
		} catch (RemoveException re) {

			trace.error("Cannot remove node " + re.getMessage());
			throw new HeroException("Cannot remove node " + re.getMessage());
		}
	}

	/**
	 * Delete an edge to the project. If out node of this edge is in executing, terminated or cancelled state, the
	 * method throws an execption.
	 *
	 * @param name the name ot the edge
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteEdge(String name) throws HeroException {
		// check deletion conditions

		trace.debug("parameter: name=" + name +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocalHome eh;
		BnEdgeLocal ed;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			eh = BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ed = eh.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("edge does not exists " + fe.getMessage());
			throw new HeroException("edge does not exists " + name);
		}
		String nodeIn = ed.getInBnNode().getName();
		String nodeOut = ed.getOutBnNode().getName();
		edgeDeletion(ed);
		if (jms!=null)
			jms.sendEdgeEvent(DELETEEDGE,this.getName(),name,nodeIn,nodeOut,this.getCaller());
	}

	/** Actually deletes an edge and restore the state of the outgoing node
	 *  <strong>Not a business method</strong>
	 **/

	public void edgeDeletion(BnEdgeLocal ed) throws HeroException {
		BnNodeLocal nd = ed.getOutBnNode();

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (nd.isExecuting()) {
			throw new DeleteEdgeException("Cannot delete edge. Out node executing");
		}
		/*	if (nd.isTerminated()) {
		 throw new DeleteEdgeException("Cannot delete edge. Out node terminated");
		 }
		 if (nd.isCancelled()) {
		 throw new DeleteEdgeException("Cannot delete edge. Out node cancelled");
		 }*/
		try {
			ed.remove();

			EdgeState edgeState = EdgeState.make(nd.getType());
			int newOperation = edgeState.state(nd);

			NodeState nodeState = NodeState.make(nd.getType(), nd.getAnticipable());
			int newState = nodeState.computeState(nd, newOperation);

			if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
				throw new HeroException("AddEdge generated a bad BnNode transition");
			}
			if (!mProject.getType().equals(Constants.Pj.MODEL))
				this.setNodeState(nd, newState);
		} catch (RemoveException e) {

			trace.error("deleteNode failed " + e.getMessage());
			throw new HeroException("deleteNode failed " + e.getMessage());
		}
	}


	/**
	 * Detete a hook. Delete a Java or aTCL hook of this project. </strong>
	 *
	 * @param hookName the name of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteHook(String hookName) throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectHookLocalHome pHome;
		BnProjectHookLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnProjectHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectHookLocal ph =	pHome.findByName(mProject.getId(), hookName);
			ph.remove();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a node hook. Delete a Java or a TCL hook of this node. </strong>
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNodeHook(String nodeName, String hookName)
	throws HeroException {

		trace.debug("parameters: nodename=" + nodeName + "Hookname=" + hookName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeHookLocalHome pHome;
		BnNodeHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnNodeHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeHookLocal nh = pHome.findByName(nd.getId(), hookName);
			nh.remove();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a node interHook.
	 *
	 * @param nodeName the name of the node
	 * @param hookName the name of the interHook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNodeInterHook(String nodeName, String interHookName)
	throws HeroException {

		trace.debug("parameters: nodename=" + nodeName + "InerhookName="  + interHookName  +  " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeInterHookLocalHome pHome;
		BnNodeInterHookLocal ppLocal;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnNodeInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			BnNodeInterHookLocal nh =
				pHome.findByName(nd.getId(), interHookName);
			nh.remove();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Detete a interHook of this project. This method deletes this hook from all projects nodes.
	 *
	 * @param hookName the name of the hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteInterHook(String interHookName) throws HeroException {
		BnProjectInterHookLocalHome pHome;
		BnProjectInterHookLocal ppLocal;

		trace.debug("parameter: interhookName=" + interHookName + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnProjectInterHookUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			BnProjectInterHookLocal ph =
				pHome.findByName(mProject.getId(), interHookName);
			ph.remove();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a role mapper.
	 *
	 * @param roleName the name of the node
	 * @param roleName name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteRoleMapper(String roleName) throws HeroException {

		trace.debug("parameter: roleName=" + roleName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnRoleLocalHome pHome;
		BnRoleMapperLocal mapperL;
		BnRoleLocal roleL;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			roleL = pHome.findByName(roleName, mProject.getId());
			mapperL = roleL.getBnRoleMapper();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		try {
			mapperL.remove();
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a role (and the Role mapper if it exists).
	 *
	 * @param roleName the name of the node
	 * @param roleName name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteRole(String roleName) throws HeroException {

		trace.debug("parameter: roleName=" + roleName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnRoleLocalHome pHome;
		BnRoleMapperLocal mapperL;
		BnRoleLocal roleL;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}

		try {
			roleL = pHome.findByName(roleName, mProject.getId());
			mapperL = roleL.getBnRoleMapper();
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		try {
			if (mapperL != null) mapperL.remove();
			roleL.remove();
		} catch (RemoveException rm) {

			trace.error(rm.getMessage());
			throw new HeroException(rm.getMessage());
		}
	}

	/**
	 * Delete a property of the project. Deletes the project property associated to this key.
	 *
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteProperty(String key) throws HeroException {

		trace.debug("parameter: key=" + key + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
				ppLocal.remove();
			} catch (RemoveException rm) {

				trace.error(rm.getMessage());
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	/**
	 * Delete a property of a node. Deletes the node property associated to this key
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void deleteNodeProperty(String nodeName, String key)
	throws HeroException {

		trace.debug("parameters: node=" + nodeName + " key=" + key + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {
			try {
				propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
				propertyLocal.remove();
			} catch (RemoveException rm) {

				trace.error(rm.getMessage());
				throw new HeroException(rm.getMessage());
			}

		} else {
			throw new HeroException("Property not found :" + key);
		}
	}

	// -------------------------------------------------------------------------
	// Set business methods
	// -------------------------------------------------------------------------

	/**
	 * Set a trace level of the actual project.
	 *
	 * @param level the trace level
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setTraceLevel(String level)	throws HeroException {

		trace.debug("parameters: level=" + level + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (level.equals(Constants.INFO))
			trace.setLevel(Level.INFO);
		else if (level.equals(Constants.DEBUG))
			trace.setLevel(Level.DEBUG);
		else if (level.equals(Constants.ERROR))
			trace.setLevel(Level.ERROR);
	}

	/**
	 * Set a log level of the actual project.
	 *
	 * @param level the log level
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setLogLevel(String level)	throws HeroException {

		trace.debug("parameters: level=" + level + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (level.equals(Constants.INFO))
			thelog.setLevel(Level.INFO);
		else if (level.equals(Constants.DEBUG))
			thelog.setLevel(Level.DEBUG);
		else if (level.equals(Constants.ERROR))
			thelog.setLevel(Level.ERROR);
	}


	/**
	 * Set a role to this user. This role is assigned to the user in the context of
	 * the actual project.
	 *
	 * @param userName the name of the user
	 * @param roleName the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setUserRole(String userName, String roleName)
	throws HeroException {

		trace.debug("parameters: user=" + userName + " role=" + roleName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnRoleLocalHome roleh;
		BnUserLocalHome userh;
		UserServiceLocalHome ushome;
		UserServiceLocal ul;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			roleh = BnRoleUtil.getLocalHome();
			userh = BnUserUtil.getLocalHome();
			ushome= hero.interfaces.UserServiceUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			ul = ushome.create();
			BnUserLocal user = ul.findUser(userName);

			BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			user.getBnRoles().add(role);
			if (jms!=null)
				jms.sendUserRoleEvent(SETUSERROLE,this.getName(),userName,roleName);
		} catch (Exception fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Sets the role of an activity. Changes the role of this activity
	 *
	 * @param name the name of the activity
	 * @param role the name of the role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeRole(String name, String role) throws HeroException {

		trace.debug("parameters: name=" + name + " role=" + role + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnRoleLocalHome rh;
		BnNodeLocalHome nh;
		BnNodeLocal nl;
		BnRoleLocal rl;
		if (!isAdminOfProject())
		{
			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			rh = BnRoleUtil.getLocalHome();
			nh = BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			nl = nh.findByName(name, mProject.getId());
		} catch (FinderException fe) {
			trace.error("BnNode does not exists " + fe.getMessage());
			throw new HeroException("BnNode does not exists " + name);
		}
		try {
			rl = rh.findByName(role, mProject.getId());
		} catch (FinderException fe) {
			trace.error("BnRole does not exists " + fe.getMessage());
			throw new HeroException("BnRole does not exists " + role);
		}
		nl.setBnRole(rl);
		if (jms!=null)
			jms.sendProjectEvent(CHANGEROLE, this.getName());
	}

	/**
	 * Set edit node changes. Only for GraphEditor client
	 *
	 * @param name of the node
	 * @param role of the node
	 * @param description of the node
	 * @param deadline of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void setEditNode(String node, String role, String description, long deadline) throws HeroException {

		trace.debug("setEditNode:  node=" + node + " role=" + role +" description=" + description + " deadline=" + deadline);
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		this.setNodeDeadline(node,deadline);
		this.setNodeDescription(node,description);
		this.setNodeRole(node,role);
	}

	/**
	 * Set the edge condition. This condition is evaluated at run-time in order to perform activity transition.
	 * To set the condition you have to use beanshell language (Lightweight Scripting for Java).<br>
	 * <strong>The next line illustrates the use of setEdgeCondition method:<br><br></strong>
	 *
	 * ...<br>
	 * String condition = "partial_status.equals(\"ok\")";<br>
	 * <strong>yourProjectSessionInstance</strong>.setEdgeCondition("edgeName",condition);<br><br>
	 *
	 * The engine evaluates on the fly the condition by using the If statement. In the previous sample,
	 * if the value of the property "partial_status" is equals to "ok" the result of this condition is true.
	 *
	 * @param edge edge name
	 * @param condition condition of the edge
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setEdgeCondition(String edge, String condition)
	throws HeroException {

		trace.debug("parameters: name=" + edge + " condition=" + condition + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnEdgeLocal e = mProject.getBnEdge(edge);
		e.setCondition(condition);
	}

	/**
	 * Set the node deadline. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param name the name of the node
	 * @param date date of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 * @deprecated replaced by setDeadlines(String name, Collection co)
	 *
	 **/
	public void setNodeDeadline(String name, long date) throws HeroException {

		trace.debug("parameters: name=" + name + " deadline=" + date + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		if (new java.sql.Date(date).before(new Date()))
			throw new HeroException("This date is before the current date !!");

		//Call setNodeDeadlines()
		ArrayList ar = new ArrayList();
		ar.add(new Long(date));
		setNodeDeadlines(name, ar);
	}

	/**
	 * Set a collection of deadlines for the node.
	 * Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param name the name of the node
	 * @param co collection of dates of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeDeadlines(String name, Collection co) throws HeroException {
		Iterator it = co.iterator();
		//for deadline based on EJB timers
		DeadlineEjbTimerSessionLocalHome dlHome;
		DeadlineEjbTimerSessionLocal dlSession;
		try {
			dlHome = DeadlineEjbTimerSessionUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			dlSession = dlHome.create();

	} catch (CreateException he) {
		throw new HeroException("setNodeDeadlines Error creating DeadlineEjbTimerSessionLocal");
	}

		trace.debug("parameters: name=" + name + " collection of deadlines " + " process name = " + this.getName()+ " started by " + this.getCaller());
		while (it.hasNext()){
			trace.debug("deadline = " + ((Long)it.next()).longValue() );
		}
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		// set the collection of deadlines
		it = co.iterator();
		ArrayList ar = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);
		while (it.hasNext()){
			long d = ((Long)it.next()).longValue();
			java.sql.Timestamp dt = new java.sql.Timestamp(d);
			long t = new Date().getTime();
			java.sql.Timestamp ct = new java.sql.Timestamp(t);
			if (dt.before(ct))
				throw new HeroException("The date of the deadline : " + dt.toString() + " is before the current date !!");
			ar.add(dt);
		}
		nd.setDeadlines(ar);

		if (nd.getState()== Constants.Nd.READY && nd.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
			//BonitaDeadline.getInstance().startDeadlines(name, mProject.getId());
			dlSession.startDeadlineEjbTimers(name, mProject.getId());


		String projectName = this.getName();
		if (jms!=null)
			jms.sendNodeEvent(SETNODEDEADLINES,projectName,name,nd.getType(),nd.getState(),this.getCaller());

	}

	/**
	 * Set a collection of relativeDeadlines for the node.
	 *
	 * @param name the name of the node
	 * @param co collection of dates of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeRelativeDeadlines(String name, Collection co) throws HeroException {
		Iterator it = co.iterator();

		//for deadline based on EJB timers
		DeadlineEjbTimerSessionLocalHome dlHome;
		DeadlineEjbTimerSessionLocal dlSession;
		try {
			dlHome = DeadlineEjbTimerSessionUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			dlSession = dlHome.create();

	} catch (CreateException he) {
		throw new HeroException("setNodeRelativeDeadlines Error creating DeadlineEjbTimerSessionLocal");
	}

		trace.debug("parameters: name=" + name + " collection of relativeDeadlines " + " process name = " + this.getName()+ " started by " + this.getCaller());
		while (it.hasNext()){
			trace.debug("deadline = " + ((Long)it.next()).longValue() );
		}
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		// set the collection of relativeDeadlines
		it = co.iterator();
		ArrayList ar = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);
		while (it.hasNext()){
			long d = ((Long)it.next()).longValue();
			java.sql.Timestamp dt = new java.sql.Timestamp(d);
			ar.add(dt);
		}
		nd.setRelativeDeadlines(ar);

		if (nd.getState()== Constants.Nd.READY && nd.getBnProject().getType().equals(Constants.Pj.COOPERATIVE))
			//BonitaDeadline.getInstance().startDeadlines(name, mProject.getId());
			dlSession.startDeadlineEjbTimers(name, mProject.getId());


		String projectName = this.getName();
		if (jms!=null)
			jms.sendNodeEvent(SETNODEDEADLINES,projectName,name,nd.getType(),nd.getState(),this.getCaller());

	}

	/**
	 * Set the node relative deadline. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param name the name of the node
	 * @param date date of the deadline
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 * @deprecated replaced by setRelativeDeadlines(String name, Collection co)
	 *
	 **/
	public void setNodeRelativeDeadline(String name, long date) throws HeroException {

		trace.debug("parameters: name=" + name + " deadline=" + date + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		//Call setNodeRelativeDeadlines()
		ArrayList ar = new ArrayList();
		ar.add(new Long(date));
		setNodeRelativeDeadlines(name, ar);
	}

	/**
	 * Set the node description. Node description represents, explicity, execution related information
	 * of this task.
	 *
	 * @param name the name of the node
	 * @param descriptio description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeDescription(String name, String description)
	throws HeroException {

		trace.debug("parameters: name="+ name+ " description="+ description + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setDescription(description);
	}

	/**
	 * Set the node hook value.
	 *
	 * @param name name of the node
	 * @param hook hookName of the node
	 * @param value new script value of this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeInterHookValue(String node, String hook, String value)
	throws HeroException {

		trace.debug("parameters: node="+ node+ " hook="+ hook+ " value="+ value+ " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}

	/**
	 * Set the hook value. This value is the new hook script associated to all project nodes
	 *
	 * @param hook hookName of the node
	 * @param value new script value of this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setInterHookValue(String hook, String value)
	throws HeroException {

		trace.debug("parameters:  hook=" + hook + " value=" + value+ " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				hk.setScript(value);
			}
		}
	}

	/**
	 * Unset a user role in this project.
	 *
	 * @param userName name of the user
	 * @param roleName name of the role
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void unsetUserRole(String userName, String roleName) throws HeroException{

		trace.debug("parameters: userName=" + userName + " roleName=" + roleName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnUserLocal mUser;
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnRoleLocal mRole;
		UserServiceLocalHome ushome;
		UserServiceLocal ul;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		try {
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			ul = ushome.create();
			mUser = ul.findUser(userName);
		} catch (Exception fe) {

			trace.error("User does not exist " + fe.getMessage());
			throw new HeroException("User does not exist");
		}

		Collection userRoles = this.getUserRolesInProject(userName);
		Iterator i = userRoles.iterator();
		while (i.hasNext())
		{
			BnRoleValue role = (BnRoleValue)i.next();
			if (role.getName().equals(roleName))
			{
				try {
					mRole = rHome.findByName(role.getName(),mProject.getId());
					mUser.getBnRoles().remove(mRole);
				} catch (javax.ejb.FinderException fe) {

					trace.error("Role does not exist " + fe.getMessage());
					throw new HeroException("Role does not exist");
				}
			}
		}

		if (jms!=null)
			jms.sendUserRoleEvent(UNSETUSERROLE,this.getName(),userName,roleName);
	}

	/**
	 * Unset a user for this project
	 *
	 * @param userName The name of the user
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void unsetUser(String userName) throws HeroException {

		trace.debug(" Parameter: name=" + userName + " process name = "+this.getName()+ " started by " + this.getCaller());
		UserServiceLocalHome ushome;
		UserServiceLocal ul;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnUserLocal mUser;
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			ul = ushome.create();
			mUser = ul.findUser(userName);
		} catch (Exception fe) {

			trace.error("User does not exist " + fe.getMessage());
			throw new HeroException("User does not exist");
		}
		Collection cUsers = mProject.getBnUsers();
		if (cUsers.contains(mUser))
			mProject.getBnUsers().remove(mUser);
		else
		{

			trace.error("User does not exist in the project...");
			throw new HeroException("User does not exist in the project");
		}

	}

	/**
	 * Set the node in traditional mode. When a node is traditional the anticipable attribute is false. This
	 * method must be used if you want to execute this activity in a traditional model.
	 *
	 * @param name name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeTraditional(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(false);
	}

	/**
	 * Set the node in anticipable mode.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeAnticipable(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setAnticipable(true);
	}

	/**
	 * Set the node in automatic  mode. The responsible of the activity execution is now the engine.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 * @deprecated replaced by setNodeType
	 *
	 **/
	public void setNodeAutomatic(String name) throws HeroException {}

	/**
	 * Set the node type. Change the current type of the node (if node is not executing).<br><br>
	 *
	 * Possibles node types are:<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE<br>
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeType(String name, int type) throws HeroException {

		trace.debug("parameters: name=" + name + " type" + type + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		nd.setType(type);
	}

	/**
	 * Set a property of a node. A property is a pair key/value representing workflow relevant data. By using
	 * propagate argument we can specify if we want to propagate this property.
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @param propagate if it is a propagation property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeProperty(String nodeName,String key,String value,boolean propagate)
	throws HeroException {

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;
		if (!isAdminOfProject())
		{
			if (mProject.getName().matches(".*_instance.*") && !isUserInNodeRole(nodeName))
			{

				trace.error("You are not allowed to set/update properties of this node... ");
				throw new HeroException("You are not allowed to set/update properties of this node... ");
			}
		}

		trace.debug("parameters: node="+ nodeName+ " key="+ key+ " value="+ value + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
//			Check the value of an enumerated property contains only possible values
			boolean allcontains = false;
			if (propertyLocal.getPossibleValues() != null) {
				Collection ar_pv = propertyLocal.getPossibleValues();
				String current = value;
				SortedSet sort_pv =
				    new TreeSet (new Comparator () {
				      public int compare (Object a, Object b) {
				      	if ( (a.toString().length()) < (b.toString().length()) )
				        return 1;
				      	else if ( (a.toString().length()) > (b.toString().length()) )
				      		return -1;
				      	else {
				      		int re = a.toString().compareTo(b.toString());
				      		if ( re > 0 ) return 1;
				      		else if ( re < 0 ) return -1;
				      		else return 0;
				      	}
				      }
				      });

				Iterator it_pv = ar_pv.iterator();
				while (it_pv.hasNext()){
					String val = (String)it_pv.next();
					sort_pv.add(val);
				}
				boolean found = true;
				boolean not_end = true;
				while (found && not_end){
					Iterator it_check = sort_pv.iterator();
					found = false;
					while (it_check.hasNext()){
						String val = (String)it_check.next();
						if (current.startsWith(val)){
							found = true;
							current = current.replaceFirst(val, "");
							if (current.length()!=0) {
								current = current.substring(1);
							}
							if (current.length()==0) {
								allcontains = true;
								not_end = false;
							}
							break;
						}
					}
				}
			}
			if (propertyLocal.getPossibleValues() == null || allcontains){
				propertyLocal.setTheValue(value);
			}
			else
				throw new HeroException("Error: This value is not one of possibles values defined for the property = "+key);

			//propertyLocal.setTheValue(value);
			propertyLocal.setPropagate(propagate);
		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);
				propertyLocal.setPropagate(propagate);
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Set a property of a node. A property is a pair key/value representing workflow relevant data. With
	 * this method the property is propagated within others nodes.
	 *
	 * @param nodeName the name of the node
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void setNodeProperty(String nodeName, String key, String value)
	throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		if (!isAdminOfProject())
		{
			if (mProject.getName().matches(".*_instance.*") && !isUserInNodeRole(nodeName))
			{

				trace.error("You are not allowed to set/update properties of this node... ");
				throw new HeroException("You are not allowed to set/update properties of this node... ");
			}
		}

		trace.debug("parameters: node="+ nodeName+ " key="+ key+ " value="+ value + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];

			//Check the value of an enumerated property contains only possible values
			boolean allcontains = false;
			if (propertyLocal.getPossibleValues() != null) {
				Collection ar_pv = propertyLocal.getPossibleValues();
				String current = value;
				SortedSet sort_pv =  new TreeSet (new Comparator () {
				      public int compare (Object a, Object b) {
				      	if ( (a.toString().length()) < (b.toString().length()) )
				        return 1;
				      	else if ( (a.toString().length()) > (b.toString().length()) )
				      		return -1;
				      	else {
				      		int re = a.toString().compareTo(b.toString());
				      		if ( re > 0 ) return 1;
				      		else if ( re < 0 ) return -1;
				      		else return 0;
				      	}
				      }
				      });

				Iterator it_pv = ar_pv.iterator();
				while (it_pv.hasNext()){
					String val = (String)it_pv.next();
					sort_pv.add(val);
				}
				boolean found = true;
				boolean not_end = true;
				while (found && not_end){
					Iterator it_check = sort_pv.iterator();
					found = false;
					while (it_check.hasNext()){
						String val = (String)it_check.next();
						if (current.startsWith(val)){
							found = true;
							current = current.replaceFirst(val, "");
							if (current.length()!=0) {
								current = current.substring(1);
							}
							if (current.length()==0) {
								allcontains = true;
								not_end = false;
							}
							break;
						}
					}
				}
			}

			//if (propertyLocal.getPossibleValues() == null || propertyLocal.getPossibleValues().contains(value))
			if (propertyLocal.getPossibleValues() == null || allcontains)
				propertyLocal.setTheValue(value);
			else
				throw new HeroException("Error: This value is not one of possibles values defined for the property = "+key);


		} else {
			try {
				propertyLocal = pHome.create(key, value);
				propertyLocal.setBnNode(nd);

			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Set a property of the project. A property is a pair key/value representing workflow relevant data. This
	 * methods creates a new property that can be accessed from all activites of the project.
	 *
	 * @param key the key of the property
	 * @param value the name of the property
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setProperty(String key, String value) throws HeroException {


		trace.debug("parameters: key=" + key + " value=" + value + " process name = " + this.getName()+ " started by " + this.getCaller());
		// MBL: change and may be error ????
		//if (!isInProject(this.getCaller()))
		if (!isInProject(this.getName(), this.getVersion()) )
		{

			trace.error("You don't have access to this project, only project users are allowed to set/update properties !!");
			throw new AdminException("You don't have access to this project, only project users are allowed to set/update properties !!");
		}
		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal ppLocal = null;

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}

		if (!c.isEmpty()) {

			ppLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			//Check the value of an enumerated property contains only possible values
			boolean allcontains = false;
			if (ppLocal.getPossibleValues() != null) {
				Collection ar_pv = ppLocal.getPossibleValues();
				String current = value;
				SortedSet sort_pv =
				    new TreeSet (new Comparator () {
				      public int compare (Object a, Object b) {
				      	if ( (a.toString().length()) < (b.toString().length()) )
				        return 1;
				      	else if ( (a.toString().length()) > (b.toString().length()) )
				      		return -1;
				      	else {
				      		int re = a.toString().compareTo(b.toString());
				      		if ( re > 0 ) return 1;
				      		else if ( re < 0 ) return -1;
				      		else return 0;
				      	}
				      }
				      });

				Iterator it_pv = ar_pv.iterator();
				while (it_pv.hasNext()){
					String val = (String)it_pv.next();
					sort_pv.add(val);
				}
				boolean found = true;
				boolean not_end = true;
				while (found && not_end){
					Iterator it_check = sort_pv.iterator();
					found = false;
					while (it_check.hasNext()){
						String val = (String)it_check.next();
						if (current.startsWith(val)){
							found = true;
							current = current.replaceFirst(val, "");
							if (current.length()!=0) {
								current = current.substring(1);
							}
							if (current.length()==0) {
								allcontains = true;
								not_end = false;
							}
							break;
						}
					}
				}
			}
			if (ppLocal.getPossibleValues() == null ||allcontains )
				ppLocal.setTheValue(value);
			else
				throw new HeroException("Error: This value is not one of possibles values defined for the property = "+key);

		} else {
			try {
				ppLocal = pHome.create(key, value);
				ppLocal.setBnProject(mProject);

			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Set property possible values for a specific node. Values argument represent the values which are accepted
	 * as possible property values.
	 *
	 * @param nodeName the name of the node
	 * @param key property key
	 * @param values possible values
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodePropertyPossibleValues(String nodeName, String key, Collection values)
	throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;
		if (!isAdminOfProject())
		{
			if (mProject.getName().matches(".*_instance.*") && !isUserInNodeRole(nodeName))
			{
				trace.error("You are not allowed to set/update properties of this node... ");
				throw new HeroException("You are not allowed to set/update properties of this node... ");
			}
		}

		trace.debug("parameters: node="+ nodeName+ " key="+ key+ " values="+ values + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (c.isEmpty()) {
			try {
				propertyLocal = pHome.create(key, values);
				propertyLocal.setBnNode(nd);
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * This method allows users to change dynamically the possible values defined for this property.
	 * Only used for enumeration types.
	 * A default value is mandatory for this method.
	 *
	 * @param nodeName the name of the node
	 * @param key property key
	 * @param values possible values
	 * @param defaultValues Collection of default values
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void updateNodePropertyPossibleValues(String nodeName, String key, Collection values, Collection defaultValues)
	throws HeroException {
		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal;
		if (!isAdminOfProject())
		{
			if (mProject.getName().matches(".*_instance.*") && !isUserInNodeRole(nodeName))
			{

				trace.error("You are not allowed to set/update properties of this node... ");
				throw new HeroException("You are not allowed to set/update properties of this node... ");
			}
		}

		trace.debug("parameters: node="+ nodeName+ " key="+ key+ " values="+ values+" defaultValue "+ defaultValues + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		try {
			pHome = BnNodePropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c;
		try {
			c= pHome.findByTheKey(nd.getId(), key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnNodePropertyLocal) (c.toArray())[0];
			//Check the value of an enumerated property contains only possible values
			Iterator it_defaultValues = defaultValues.iterator();
			boolean allcontains = true;
			String defaultValue = "";
			String currentValue ="";
			while (it_defaultValues.hasNext()){
				currentValue = (String)it_defaultValues.next();
				if (values.contains(currentValue)){
					defaultValue = defaultValue + " " + currentValue;
				}
				else {
					allcontains = false;
					break;
				}
			}
			if (propertyLocal.getPossibleValues() == null || allcontains){
				propertyLocal.setPossibleValues(values);
				if (!"".equals(defaultValue)) defaultValue = defaultValue.substring(1);
				propertyLocal.setTheValue(defaultValue);
			}
			else
				throw new HeroException("Error: value in the collection of defaultValues is not one of possibles values defined for the property...");
		}
		else
			throw new HeroException("An Error occurs when updating possible values...");
	}

	/**
	 * Set property possible values. Values argument represent the values which are accepted
	 * as possible property values.
	 *
	 * @param key property key
	 * @param values possible values
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setPropertyPossibleValues(String key, Collection values)
	throws HeroException {
		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal propertyLocal;
		//MBL: change and may be error ????
		//if (!isInProject(this.getCaller()))
		if (!isInProject(this.getName(), this.getVersion()) )
		{

			trace.error("You don't have access to this project, only project users are allowed to set/update properties !!");
			throw new AdminException("You don't have access to this project, only project users are allowed to set/update properties !!");
		}

		trace.debug("parameters: key="+ key+ " values="+ values + " process name = " + this.getName()+ " started by " + this.getCaller());

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c = null;
		try {
			c = pHome.findByTheKey(mProject.getId(),key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (c.isEmpty()) {
			try {
				propertyLocal = pHome.create(key, values);
				propertyLocal.setBnProject(mProject);
			} catch (InvalidValueException ie) {

				trace.error(ie.getMessage());
				throw new HeroException(ie.getMessage());
			} catch (javax.ejb.CreateException ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}


	/**
	 * This method allows users to change dynamically the possible values defined for this property.
	 * Only used for enumeration types.
	 * A default value is mandatory for this method.
	 *
	 * @param key property key
	 * @param values possible values
	 * @param defaultValues Collection of default values
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void updatePropertyPossibleValues(String key, Collection values, Collection defaultValues)
	throws HeroException {
		BnProjectPropertyLocalHome pHome;
		BnProjectPropertyLocal propertyLocal;
		//MBL:   change and may be error ????
		//if (!isInProject(this.getCaller()))
		if (!isInProject(this.getName(), this.getVersion()) )
		{

			trace.error("You don't have access to this project, only project users are allowed to set/update properties !!");
			throw new AdminException("You don't have access to this project, only project users are allowed to set/update properties !!");
		}

		trace.debug("parameters: key="+ key+ " values="+ values + " defaultValue="+ defaultValues +" process name = " + this.getName()+ " started by " + this.getCaller());

		try {
			pHome = BnProjectPropertyUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		Collection c;
		try {
			c= pHome.findByTheKey(mProject.getId(),key);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
		if (!c.isEmpty()) {
			propertyLocal = (BnProjectPropertyLocal) (c.toArray())[0];
			//Check the value of an enumerated property contains only possible values
			Iterator it_defaultValues = defaultValues.iterator();
			boolean allcontains = true;
			String defaultValue = "";
			String currentValue ="";
			while (it_defaultValues.hasNext()){
				currentValue = (String)it_defaultValues.next();
				if (values.contains(currentValue)){
					defaultValue = defaultValue + " " + currentValue;
				}
				else {
					allcontains = false;
					break;
				}
			}
			if (propertyLocal.getPossibleValues() == null || allcontains){
				propertyLocal.setPossibleValues(values);
				if (!"".equals(defaultValue)) defaultValue = defaultValue.substring(1);

				propertyLocal.setTheValue(defaultValue);
			}
			else
				throw new HeroException("Error: value in the collection of defaultValues is not one of possibles values defined for the property...");
		}
		else
			throw new HeroException("An Error occurs when updating possible values...");
	}

	// -------------------------------------------------------------------------
	// Set Methods only accessibles by the workflow Engine
	// -------------------------------------------------------------------------

	/**
	 * Execute a node hook.
	 *
	 * @param nodeName the name of the node
	 * @param event the event
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public void hookEvent(String nodeName, String event) throws HeroException {

		trace.debug("parameters: node=" + nodeName + " event=" + event + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		Collection hooks;
		BnNodeLocal node;

		node = mProject.getBnNode(nodeName);
		hooks = node.getBnHooks();
		for (Iterator i = hooks.iterator(); i.hasNext();) {
			BnNodeHookLocal hk = (BnNodeHookLocal) i.next();
			if (hk.getEvent().equalsIgnoreCase(event)) {
				hero.hook.Hook h =
					hero.hook.Hook.make(hk.getName(), event, hk.getType());
				h.execute(this, event, node);
			}
		}
	}


	/**
	 * Propagate node properties. Propagates all node properties to connected nodes.
	 * @param nodeName the name of the Node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void propagateNodeProperties(String nodeName) throws HeroException {

		trace.debug("parameter: name=" + nodeName + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodePropertyLocalHome pHome;
		BnNodePropertyLocal propertyLocal = null;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection c = nd.getBnProperties();
		Collection c2 = nd.getOutBnEdges();

		if ((!c.isEmpty()) && (!c2.isEmpty())) {
			BnNodePropertyLocal p;
			BnEdgeLocal ed;
			for (Iterator i = c.iterator(); i.hasNext();) {
				p = (BnNodePropertyLocal) i.next();
				for (Iterator i2 = c2.iterator(); i2.hasNext();) {
					ed = (BnEdgeLocal) i2.next();
					if (p.getPropagate())
					{
						this.setNodePropertyPossibleValues((ed.getOutBnNode()).getName(),p.getTheKey(),p.getPossibleValues());
						if (p.getTheValue()!=null || p.getPossibleValues() == null || p.getPossibleValues().size()==0)
							this.setNodeProperty((ed.getOutBnNode()).getName(),p.getTheKey(),p.getTheValue(),true);
					}
				}
			}
		}
	}


	/**
	 * Set the node state. Set start date of this node when starts the execution and the end date when node
	 * is terminated.
	 *
	 * @param name name of the node
	 * @param state new state of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeState(BnNodeLocal node, int state)	throws HeroException {

		trace.debug("parameters: name=" + node.getName() + " state=" + state + " process name = " + this.getName()+ " started by " + this.getCaller());

		node.setState(state);
		if (state == hero.interfaces.Constants.Nd.EXECUTING || state == hero.interfaces.Constants.Nd.ANTICIPATING) {
			node.setStartDate(new java.util.Date());
		}
		else if (state == hero.interfaces.Constants.Nd.TERMINATED) {
			if (!(node.getStartDate() !=null))
				node.setStartDate(new java.util.Date());
			node.setEndDate(new java.util.Date());
		}
		else if (state == hero.interfaces.Constants.Nd.DEAD)
		{
			if (!(node.getStartDate() !=null))
				node.setStartDate(new java.util.Date());
			node.setEndDate(new java.util.Date());
		}

		// Managing deadlines
		if (state == hero.interfaces.Constants.Nd.READY && (node.getDeadlines().size() != 0 || node.getRelativeDeadlines().size() != 0)) {
			dlSession.startDeadlineEjbTimers(node.getName(),node.getBnProject().getId());

		} else {
			// For absolute deadlines
			if ((state == hero.interfaces.Constants.Nd.TERMINATED || state == hero.interfaces.Constants.Nd.DEAD)
					&& node.getDeadlines().size() != 0) {
				Collection co = node.getDeadlines();
				Iterator it = co.iterator();
				while (it.hasNext()) {
					long date = ((java.sql.Timestamp) it.next()).getTime();
					if (date > new Date().getTime())
						;
					dlSession.stopDeadlineEjbTimer(mProject.getName(),node.getName(), date);
				}
			}
			// For relative deadlines
			if ((state == hero.interfaces.Constants.Nd.TERMINATED || state == hero.interfaces.Constants.Nd.DEAD)
					&& node.getRelativeDeadlines().size() != 0) {
				Collection co = node.getRelativeDeadlines();
				Iterator it = co.iterator();
				while (it.hasNext()) {
					long date = ((java.sql.Timestamp) it.next()).getTime();
					if (new Date().getTime() + date > new Date().getTime());
					dlSession.stopDeadlineEjbTimer(mProject.getName(),node.getName(), date);
				}
			}
		}
		if (jms!=null)
			jms.sendNodeEvent(SETNODESTATE,this.getName(),node.getName(),node.getType(),state,this.getCaller());
	}

	/**
	 * Set all initial roles to ready if node does not has in edges.
	 *
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodesReady() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		try{
			Collection nodes = mProject.getBnNodes();
			Iterator nds = nodes.iterator();
			while (nds.hasNext()) // first activities are initial
			{
				BnNodeLocal subNode = (BnNodeLocal)nds.next();
				if (this.getNodeInEdges(subNode.getName()).size()==0 && subNode.getState()==Constants.Nd.INITIAL
						&& (subNode.getType()==Constants.Nd.AND_JOIN_AUTOMATIC_NODE || subNode.getType()==Constants.Nd.OR_JOIN_AUTOMATIC_NODE))
				{
					this.setNodeState(subNode,Constants.Nd.READY);
					EngineLocalHome enghome;
					try {
						enghome = hero.interfaces.EngineUtil.getLocalHome();
						EngineLocal engine = enghome.create();
						engine.startActivity(mProject.getName(), subNode.getName());
					} catch (javax.naming.NamingException ne) {
						throw new HeroException(ne.getMessage());
					}
				}
				else if (this.getNodeInEdges(subNode.getName()).size()==0 && subNode.getState()==Constants.Nd.INITIAL)
					this.setNodeState(subNode,Constants.Nd.READY);
			}
		}catch (Exception ans){

			trace.error("Error in setNodeReady " + ans.getMessage());
			throw new HeroException("Error in setNodeReady");
		}
	}

	/**
	 * Propagate subProcess activity properties to subProcess
	 *
	 * @param node the name of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void propagatesParentProperties(String node) throws HeroException {

		trace.debug("parameter: nodeName=" + node + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectLocal pl = mProject;
		try{
			BnNodeLocal nd = mProject.getBnNode(node);
			Collection props = nd.getBnProperties();
			if (nd.getBnProject().getType().equals(Constants.Pj.INSTANCE))
					this.initProject(nd.getReference());
			else
				this.initProject(BonitaParsing.getSubProcessName(nd.getReference()));
			Iterator i = props.iterator();
			while (i.hasNext())
			{
				BnNodePropertyLocal prop = (BnNodePropertyLocal)i.next();
				this.setPropertyPossibleValues(prop.getTheKey(),prop.getPossibleValues());
				if (prop.getTheValue()!=null  || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
					this.setProperty(prop.getTheKey(),prop.getTheValue());
			}
			this.setNodesReady();
			mProject=pl;
		}catch (Exception ans){ans.printStackTrace();
		mProject=pl;

		trace.error("Error in propagatesParentProperties " + ans.getMessage());
		throw new HeroException("Error in propagatesParentProperties");}
	}

	/**
	 * Propagate subProcess properties to subProcess activity.
	 *
	 * @param node the name of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/

	public void propagatesSubProcessProperties(BnNodeLocal node) throws HeroException {

		trace.debug("parameter: nodeName=" + node.getName() + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectLocal pl = mProject;
		try{
			this.initProject(node.getReference());
			Collection props = this.getProperties();
			mProject = pl;
			Iterator i = props.iterator();
			while (i.hasNext())
			{
				BnProjectPropertyValue prop = (BnProjectPropertyValue)i.next();
				this.setNodePropertyPossibleValues(node.getName(),prop.getTheKey(),prop.getPossibleValues());
				if (prop.getTheValue()!=null || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
					this.setNodeProperty(node.getName(),prop.getTheKey(),prop.getTheValue(),true);
			}
		}catch (Exception ans){ans.printStackTrace();
		mProject=pl;

		trace.error("Error in propagatesSubProcessProperties " + ans.getMessage());
		throw new HeroException("Error in propagatesSubProcessProperties ");}
	}

	/**
	 * Set the edge state
	 *
	 * @param bean of the edge
	 * @param new state of the edge
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setEdgeState(BnEdgeLocal edge, int state)
	throws HeroException {

		trace.debug("parameters: name=" + edge.getName() + " state=" + state + " process name = " + this.getName()+ " started by " + this.getCaller());
		edge.setState(state);
		if (jms!=null)
			jms.sendEdgeEvent(SETEDGESTATE,this.getName(),edge.getName(),edge.getInBnNode().getName(),edge.getOutBnNode().getName(),this.getCaller());
	}

	/**
	 * Set the agent state
	 *
	 * @param agent bean of the Agent
	 * @param new state of the Agent
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setAgentState(BnAgentLocal agent, int state)
	throws HeroException {

		trace.debug("parameters: name=" + agent.getName() + " state=" + state + " state=" + state + " process name = " + this.getName()+ " started by " + this.getCaller());
		agent.setState(state);
	}

	/**
	 * Set the agentEdge state
	 *
	 * @param agentEdge bean of the BnAgentEdge
	 * @param new state of the BnAgentEdge
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setAgentEdgeState(BnAgentEdgeLocal agentEdge, int state)
	throws HeroException {

		trace.debug("parameters: name="+ agentEdge.getName()+ " state="+ state + " state=" + state + " process name = " + this.getName()+ " started by " + this.getCaller());
		agentEdge.setState(state);
	}

	/**
	 * Set the name of the node executor. Set the name of the user which is executing the activity.
	 *
	 * @param name the name of the node
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void setNodeExecutor(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		BnNodeLocal nd = mProject.getBnNode(name);
		String executor = this.getCaller();
		Collection users = mProject.getBnUsers();
		Iterator i = users.iterator();
		while (i.hasNext())
		{
			BnUserLocal ul = (BnUserLocal)i.next();
			if (ul.getName().equals(executor))
			{
				nd.setExecutor(executor);
				break;
			}
		}
		/*try {
		 user = uHome.findByName(executor);
		 } catch (FinderException fe) {

		 trace.error("BnUser " + executor + " does not exists " + fe.getMessage());
		 throw new HeroException("BnUser " + executor + " does not exists");
		 }
		 nd.setExecutor(user);*/
		if (jms!=null)
			jms.sendNodeEvent(SETNODEEXECUTOR,this.getName(),name,nd.getType(),nd.getState(),this.getCaller());
	}


	// -------------------------------------------------------------------------
	// Get business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the list of choices into the value of the enumerated property .
	 *
	 * @param value the value of the enumerated property
	 * @param possiblesValues collection of the enumerated property
	 * @return Collection of choices of the enumerated property into the value
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public ArrayList getChoices(String value, Collection possibleValues)
	throws HeroException {
		ArrayList ar_mv= new ArrayList();
		boolean allcontains = false;
		if (possibleValues != null) {
			String current = value;
			SortedSet sort_pv =
			    new TreeSet (new Comparator () {
			      public int compare (Object a, Object b) {
			      	if ( (a.toString().length()) < (b.toString().length()) )
			        return 1;
			      	else if ( (a.toString().length()) > (b.toString().length()) )
			      		return -1;
			      	else {
			      		int re = a.toString().compareTo(b.toString());
			      		if ( re > 0 ) return 1;
			      		else if ( re < 0 ) return -1;
			      		else return 0;
			      	}
			      }
			      });
			// get sorted possible values
			Iterator it_pv = possibleValues.iterator();
			while (it_pv.hasNext()){
				String val = (String)it_pv.next();
				sort_pv.add(val);
			}

			boolean found = true;
			boolean not_end = true;
			while (found && not_end){
				Iterator it_check = sort_pv.iterator();
				found = false;
				while (it_check.hasNext()){
					String val = (String)it_check.next();
					if (current.startsWith(val)){
						found = true;
						current = current.replaceFirst(val, "");
						ar_mv.add(val);
						if (current.length()!=0) {
							current = current.substring(1);
						}
						if (current.length()==0) {
							allcontains = true;
							not_end = false;
						}
						break;
					}
				}
			}

		}
		return ar_mv;
	}

	/**
	 * Get the node description. Node description represents, explicity, execution related information
	 * of this task.
	 *
	 * @param name of the node
	 * @return String - description of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeDescription(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getDescription());
	}

	/**
	 * Get the node hook value. This method returns the hook script associated to this node
	 *
	 * @param name the name of the node
	 * @param hook hookName of the node
	 * @return String - script associated to this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeInterHookValue(String node, String hook)
	throws HeroException {
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}
		String value = null;

		trace.debug("parameters: node=" + node + " hook=" + hook + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(node);
		Collection hooks = nd.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnNodeInterHookLocal hk = (BnNodeInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		return value;
	}

	/**
	 * Get the inter hook value script. This method returns the hook script associated to all project nodes.
	 *
	 * @param hook hook name
	 * @return String - script associated to this hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getInterHookValue(String hook) throws HeroException {
		String value = null;
		if (!isAdminOfProject())
		{

			trace.error("Only the Admin of the project is allowed to do this operation... ");
			throw new AdminException("Only the Admin of the project is allowed to do this operation... ");
		}

		trace.debug("parameter:  hook=" + hook + " process name = " + this.getName()+ " started by " + this.getCaller());
		Collection hooks = mProject.getBnInterHooks();
		for (Iterator ih = hooks.iterator(); ih.hasNext();) {
			BnProjectInterHookLocal hk = (BnProjectInterHookLocal) ih.next();
			if (hk.getName().equals(hook)) {
				value = hk.getScript();
			}
		}
		return value;
	}

	/**
	 * Get the ProjectValue (historique). Returns project information: project attributes, nodes, edges,
	 * hooks, properties...
	 *
	 * @return BnProjectValue object - project data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public BnProjectValue getDetails() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return mProject.getBnProjectValue();
	}

	/**
	 * Get the ProjectLightValue. Returns the basic project information: project attributes (without relationships)
	 *
	 * @return BnProjectLightValue object - project data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public BnProjectLightValue getLightDetails() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return mProject.getBnProjectLightValue();
	}

	/**
	 * Get ProjectName
	 * @return String - project name
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getName() throws HeroException {
		if (mProject!=null)
		{
			trace.debug(" process name = " + mProject.getName()+ " started by " + this.getCaller());
			return mProject.getName();
		}
		return null;
	}

	/**
	 * Get ProjectVersion
	 * @return String - project version
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getVersion() throws HeroException {
		if (mProject!=null)
		{
			trace.debug(" process name = " + mProject.getVersion()+ " started by " + this.getCaller());
			return mProject.getVersion();
		}
		return null;
	}

	/**
	 * Get creator of the project. The user that created the project
	 * @return String - project creator
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/

	public String getCreator() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return mProject.getCreator();
	}

	/**
	 * Get status of the project. Active/Hidden
	 * @return String - project status
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/

	public String getStatus() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return mProject.getStatus();
	}

	/**
	 * Get type of the project. Cooperative/Model/Instance
	 * @return String - project type
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/

	public String getType() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return mProject.getType();
	}


	/**
	 * Get all roles of the project
	 * @return Collection of BnRoleLocal objects - roles of the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRoles() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			result.add((BnRoleLocal) i.next());
		}
		return result;
	}

	/**
	 * Get all roles values of the project
	 * @return Collection of BnRoleValue objects - roles of the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRolesValue() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			BnRoleLocal rl =(BnRoleLocal) i.next();
			result.add(rl.getBnRoleValue());
		}
		return result;
	}

	/**
	 * Get the value of this role in the project
	 * @return BnRoleValue - role of the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnRoleValue getRoleValue(String roleName) throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		try {
			BnRoleLocalHome roleh = (BnRoleLocalHome)BnRoleUtil.getLocalHome();
			BnRoleLocal rolel = roleh.findByName(roleName, mProject.getId());
			return (rolel.getBnRoleValue());
		} catch (Exception pfe) {
			throw new HeroException("This Role does not exist in this project : " + roleName);
		}
	}

	/**
	 * Get performer assign of this node.
	 *
	 * @return BnNodePerformerAssignValue of this node
	 *
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public BnNodePerformerAssignValue getNodePerformerAssign(String nodeName)	throws HeroException {
		// logs trace

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocal node = null;

		// Get objet node local with findByName from BnNodeBean
		try {
			BnNodeLocalHome nodeh = (BnNodeLocalHome)BnNodeUtil.getLocalHome();
			node = nodeh.findByName(nodeName, mProject.getId());
		} catch (Exception pfe) {
			throw new HeroException("The bnNode does not exist for node : " + nodeName);
		}
		if (node.getBnNodePerformerAssign()!=null)
			return (node.getBnNodePerformerAssign().getBnNodePerformerAssignValue());
		else
			return null;
	}

	/**
	 * Get all roles names of the project
	 * @return String Collection - roles names of this projects
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRolesNames() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection roles = mProject.getBnRoles();
		ArrayList result = new ArrayList();
		Iterator i = roles.iterator();
		while (i.hasNext()) {
			result.add(((BnRoleLocal) i.next()).getName());
		}
		return result;
	}


	/**
	 * Get all user roles. This method returns all the roles for this user in differents projects.
	 *
	 * @param userName The name of the user
	 * @return Collection of BnRoleValue objects - roles of the user in differents projects
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRoles(String userName) throws HeroException {

		trace.debug("parameter: userName=" + userName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnUserLocalHome uHome;
		BnUserLocal user;
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {

			trace.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}
		Collection roles = user.getBnRoles();
		ArrayList result = new ArrayList();
		for (Iterator i = roles.iterator(); i.hasNext();) {
			BnRoleLocal rl = (BnRoleLocal) i.next();
			result.add(rl.getBnRoleValue());
		}
		return result;
	}

	/**
	 * Get user roles in the current project. Returns the roles of this user in the current project.
	 *
	 * @param userName The name of the user
	 * @return Collection of BnRoleValue objects - roles of the user in this project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRolesInProject(String userName)
	throws HeroException {

		trace.debug("parameter: userName=" + userName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {

			trace.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}
		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId() == up.getId())
					result.add(ur.getBnRoleValue());
			}
		}
		return result;
	}

	/**
	 * Get users matching with given role in the current project.
	 *
	 * @param roleName The name of the role
	 * @return Collection of String - users matching with this role
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUsersRole(String roleName)
	throws HeroException {

		trace.debug("parameter: roleName=" + roleName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnRoleLocalHome rHome;
		BnRoleLocal role;
		ArrayList result = new ArrayList();
		try {
			rHome = (BnRoleLocalHome) hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			role = rHome.findByName(roleName,mProject.getId());
		} catch (FinderException f) {

			trace.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}
		Collection usersR = role.getBnUsers();
		Iterator users = usersR.iterator();
		while (users.hasNext())
		{
			BnUserLocal user = (BnUserLocal) users.next();
			result.add(user.getName());
		}
		return result;
	}

	/**
	 * Get user roles in project names. Returns the role names of the user in the current project
	 *
	 * @param userName The name of the user
	 * @return String Collection - role names of this user in the current project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUserRolesInProjectNames(String userName)
	throws HeroException {

		trace.debug("parameter: userName=" + userName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnUserLocalHome uHome;
		BnUserLocal user;
		ArrayList result = new ArrayList();
		try {
			uHome = (BnUserLocalHome) hero.interfaces.BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			user = uHome.findByName(userName);
		} catch (FinderException f) {

			trace.error("FinderException " + f.getMessage());
			throw new HeroException("FinderException");
		}

		Collection userRoles = user.getBnRoles();
		Collection projectRoles = mProject.getBnRoles();
		for (Iterator i = userRoles.iterator(); i.hasNext();) {
			BnRoleLocal ur = (BnRoleLocal) i.next();
			for (Iterator j = projectRoles.iterator(); j.hasNext();) {
				BnRoleLocal up = (BnRoleLocal) j.next();
				if (ur.getId().equals(up.getId()))
					result.add(ur.getBnRoleValue().getName());
			}
		}
		return result;
	}

	/**
	 * Get node role. Returns the role information.
	 *
	 * @param nodeName The name of the node
	 * @return BnRoleLocal object - the role data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnRoleLocal getNodeRole(String nodeName) throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		return ((BnRoleLocal) nd.getBnRole());
	}

	/**
	 * Get the edge Condition. This condition is evaluated at run-time in order to perform activity transition.
	 * This condition follows the beanshell syntax (Lightweight Scripting for Java).<br>
	 *
	 * @param edge the edge/transition name
	 * @return String - edge condition
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeCondition(String edge)
	throws HeroException {

		trace.debug("parameter: name=" + edge+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocal e = mProject.getBnEdge(edge);
		return(e.getCondition());
	}

	/**
	 * Returns if the node is set to be executed in anticipated mode.
	 *
	 * @param name name of the node
	 * @return boolean - if anticipable node is set to this node.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public boolean getNodeAnticipable(String name) throws HeroException {

		trace.debug("parameter: name=" + name+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getAnticipable());
	}

	/**
	 * Get node role name. Obtains the role name of this node.
	 *
	 * @param nodeName The name of the node
	 * @return String - the role name of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeRoleName(String nodeName) throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		return (((BnRoleLocal) nd.getBnRole()).getName());
	}

	/**
	 * Get node deadline. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param nodeName The name of the node
	 * @return String - deadline of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 * @deprecated replaced by getNodeDeadlines(String name, Collection co)
	 *
	 **/
	public String getNodeDeadline(String nodeName) throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (nd.getDeadlines().size() != 0) {
			Collection co = nd.getDeadlines();
			Iterator it = co.iterator();
			Date date = null;
			while (it.hasNext()){
				date = (Date)it.next();
			}
			return (date.toString());
		}
		else
			return (null);
		/*
		 if (nd.getDeadline() != null)
		 return (nd.getDeadline().toString());
		 else
		 return (null);
		 */
	}

	/**
	 * Get collection of absolute deadlines for the node. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param nodeName The name of the node
	 * @return Collection - collection of deadlines of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeDeadlines(String nodeName) throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (nd.getDeadlines() != null)
			return (nd.getDeadlines());
		else
			return (null);
	}

	/**
	 * Get collection of relative deadlines for the node. Activity deadline is the lastest date in which the activity must be finished.
	 *
	 * @param nodeName The name of the node
	 * @return Collection - collection of deadlines of this node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeRelativeDeadlines(String nodeName) throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName+ " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		if (nd.getRelativeDeadlines() != null)
			return (nd.getRelativeDeadlines());
		else
			return (null);
	}

	/**
	 * Get the node Value. Returns node information.
	 *
	 * @param name the name of the Node
	 * @return BnNodeValue object - node data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodeValue getNodeValue(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		nv = mNode.getBnNodeValue();
		return nv;
	}

	/**
	 * Get the Agent Value.
	 *
	 * @param name the name of the Agent
	 * @return BnAgentValue object - agent data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnAgentValue getAgentValue(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnAgentLocalHome nHome;
		BnAgentLocal mAgent;
		BnAgentValue av;
		try {
			nHome = hero.interfaces.BnAgentUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mAgent = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In BnAgent does not exist " + fe.getMessage());
			throw new HeroException("In BnAgent does not exist");
		}
		av = mAgent.getBnAgentValue();
		return av;
	}

	/**
	 * Get the AgentEdge Value
	 *
	 * @param name the name of the BnAgentEdge
	 * @return BnAgentEdgeValue object - agentEdge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnAgentEdgeValue getAgentEdgeValue(String name)
	throws HeroException {

		trace.debug("parameter: name=" + name);
		BnAgentEdgeLocalHome nHome;
		BnAgentEdgeLocal mAgentEdge;
		BnAgentEdgeValue av;
		try {
			nHome = hero.interfaces.BnAgentEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mAgentEdge = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In BnAgentEdge does not exist " + fe.getMessage());
			throw new HeroException("In BnAgentEdge does not exist");
		}
		av = mAgentEdge.getBnAgentEdgeValue();

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return av;
	}

	/**
	 * Get the node Light Value. Returns node basic information.
	 *
	 * @param name the name of the Node
	 * @return BnNodeLightValue object - agentEdge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodeLightValue getNodeLightValue(String name)
	throws HeroException {

		trace.debug("parameter: name=" + name);
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeLightValue nv;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		nv = mNode.getBnNodeLightValue();
		return nv;
	}


	/**
	 * Get the edge value.
	 *
	 * @param name the name of the edge
	 * @return BnEdgeValue object - edge data.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnEdgeValue getEdgeValue(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocalHome eHome;
		BnEdgeLocal mEdge;
		BnEdgeValue ev;
		try {
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mEdge = eHome.findByName(name, mProject.getId());
		} catch (FinderException fe) {

			trace.error("BnEdge does not exist " + fe.getMessage());
			throw new HeroException("BnEdge does not exist");
		}
		ev = mEdge.getBnEdgeValue();
		return ev;
	}

	/**
	 * Get the node executor. Get the name of the user which is executing the activity.
	 *
	 * @param name the name of the node
	 * @return String - the user executor of this node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getNodeExecutor(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(name);
		//BnUserLocal ul = nd.getExecutor();
		//return (ul.getName());
		return nd.getExecutor();
	}

	/**
	 * Get the state of the node. The possible states for a node are:<br><br>
	 *
	 * hero.interfaces.Constants.Nd.INITIAL<br>
	 * hero.interfaces.Constants.Nd.READY<br>
	 * hero.interfaces.Constants.Nd.DEAD<br>
	 * hero.interfaces.Constants.Nd.ANTICIPABLE<br>
	 * hero.interfaces.Constants.Nd.ANTICIPATING<br>
	 * hero.interfaces.Constants.Nd.EXECUTING<br>
	 * hero.interfaces.Constants.Nd.TERMINATED<br>
	 *
	 * @param name of the node
	 * @return int - The state of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public int getNodeState(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getState());
	}

	/**
	 * Get the type of the node. Possibles node types are:<br><br>
	 *
	 * hero.interfaces.Constants.Nd.AND_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_NODE<br>
	 * hero.interfaces.Constants.Nd.AND_JOIN_AUTOMATIC_NODE<br>
	 * hero.interfaces.Constants.Nd.OR_JOIN_AUTOMATIC_NODE<br>
	 *
	 * @param name of the node
	 * @return int - The type of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public int getNodeType(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeLocal nd = mProject.getBnNode(name);
		return (nd.getType());
	}

	/**
	 * Get all users of the project. Obtains the user names of the project
	 * @return String Collection - user names in project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getUsers() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection users = mProject.getBnUsers();
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			result.add(((BnUserLocal) i.next()).getName());
		}
		return result;
	}

	/**
	 * Return all registered users. Get user names of Bonita System
	 * @return String Collection - all user names
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getAllUsers() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		UserServiceLocalHome ushome;
		UserServiceLocal ul;
		Collection users;
		try {
			ushome = (UserServiceLocalHome) hero.interfaces.UserServiceUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			ul = ushome.create();
			users = ul.findUsers();
		} catch (Exception fe) {

			trace.error("Error in find users " + fe.getMessage());
			throw new HeroException("Error in find users ");
		}
		ArrayList result = new ArrayList();
		for (Iterator i = users.iterator(); i.hasNext();) {
			Map user = (Map)i.next();
			result.add(user.get("name"));
		}
		return result;
	}

	/**
	 * Get all node names of the project
	 *
	 * @return String Collection - all node names
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodesNames() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());

		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	 * Get all edges names of the project
	 * @return String Collection - all edges names of the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getEdgesNames() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		//	Vector names = new Vector();
		ArrayList names = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal n = (BnEdgeLocal) i.next();
			names.add(n.getName());
		}
		return ((Collection) names);
	}

	/**
	 * Get edge in node. Returns the name of in node
	 * @param edgeName the name ot the edge
	 * @return String - the name of in node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeInNode(String edgeName) throws HeroException {

		trace.debug("parameter: edgeName" + edgeName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getInBnNode()).getName());
	}

	/**
	 * Get edge out node. Returns the name of out node
	 * @param edgeName the name ot the edge
	 * @return String - the name of out node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getEdgeOutNode(String edgeName) throws HeroException {

		trace.debug("parameter: edgeName" + edgeName + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnEdgeLocal e = mProject.getBnEdge(edgeName);
		return (((BnNodeLocal) e.getOutBnNode()).getName());
	}

	/**
	 * Get all in edges of the node
	 *
	 * @param name the name ot the node
	 * @return String Collection - get in edges of the node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeInEdges(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());

		Collection in = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesIn = nd.getInBnEdges();
		Iterator i = edgesIn.iterator();
		while (i.hasNext())
			in.add(((BnEdgeLocal) i.next()).getName());
		return (in);
	}

	/**
	 * Get all out edges of the node

	 * @param name the name ot the node
	 * @return String Collection - get out edges of the node
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeOutEdges(String name) throws HeroException {

		trace.debug("parameter: name=" + name + " process name = " + this.getName()+ " started by " + this.getCaller());

		Collection out = new ArrayList();
		BnNodeLocal nd = mProject.getBnNode(name);

		Collection edgesOut = nd.getOutBnEdges();
		Iterator i = edgesOut.iterator();
		while (i.hasNext())
			out.add(((BnEdgeLocal) i.next()).getName());
		return (out);
	}

	/**
	 * Get Node properties. Get a list of pair key/value properties associated to the node.
	 *
	 * @param nodeName the name of the node
	 * @return Collection of BnNodePropertyValue objects - all node properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeProperties(String nodeName) throws HeroException {

		trace.debug("parameter: name=" + nodeName + " process name = " + this.getName()+ " started by " + this.getCaller());
		Collection prp = new ArrayList();
		BnNodePropertyLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			prp.add(pl.getBnNodePropertyValue());
		}
		return (prp);
	}

	/**
	 * Get Node property value. Get a pair key/value properties associated to the node.
	 *
	 * @param nodeName the name of the node
	 * @param key Property key
	 * @return BnNodePropertyValue object - node property data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodePropertyValue getNodeProperty(String nodeName, String key) throws HeroException {

		trace.debug("parameters: node=" + nodeName + " key=" + key + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodePropertyLocal pl;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		Collection pls = nd.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodePropertyLocal) i.next();
			if (pl.getTheKey().equals(key))
				return (pl.getBnNodePropertyValue());

		}
		return (null);
	}

	/**
	 * Get property value of the project. Get a pair key/value properties associated to this project.
	 *
	 * @param key Property key
	 * @return BnProjectPropertyValue object - property data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnProjectPropertyValue getProperty(String key) throws HeroException {

		trace.debug("parameter: key=" + key + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnProjectPropertyLocal pl;
		Collection pls = mProject.getBnProperties();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnProjectPropertyLocal) i.next();
			if (pl.getTheKey().equals(key))
				return (pl.getBnProjectPropertyValue());
		}
		return (null);
	}

	/**
	 * Get Node hooks of the project.  Get hooks of the node. Method to get the list of Java or TCL hooks of the node.
	 *
	 * @param nodeName the name of the node
	 * @return Collection of BnNodeHookValue - all node hooks
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeHooks(String nodeName) throws HeroException {

		trace.debug("parameter: node=" + nodeName  + " process name = " + this.getName()+ " started by " + this.getCaller());
		Collection hk = new ArrayList();
		BnNodeHookLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodeHookLocal) i.next();
			hk.add(pl.getBnNodeHookValue());
		}
		return (hk);
	}

	/**
	 * Get Interactive Node hooks of the project
	 *
	 * @param nodeName the name of the node
	 * @return Collection of BnNodeInterHookValue objects - all node hooks
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getNodeInterHooks(String nodeName) throws HeroException {

		trace.debug("parameter: node=" + nodeName + " process name = " + this.getName()+ " started by " + this.getCaller());

		Collection hk = new ArrayList();
		BnNodeInterHookLocal pl;

		BnNodeLocal nd = mProject.getBnNode(nodeName);

		Collection pls = nd.getBnInterHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnNodeInterHookLocal) i.next();
			hk.add(pl.getBnNodeInterHookValue());
		}
		return (hk);
	}

	/**
	 * Get role mappers of the project.
	 *
	 * @return Collection of BnRoleMapperValue - all role mappers
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getRoleMappers() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());

		Collection cm = new ArrayList();
		BnRoleMapperLocal mapperL = null;
		BnRoleLocal roleL = null;
		Collection cr = mProject.getBnRoles();

		for (Iterator i = cr.iterator(); i.hasNext();) {
			roleL = (BnRoleLocal) i.next();
			// get the mapper if exists
			mapperL = roleL.getBnRoleMapper();
			if ( mapperL != null){
				cm.add(mapperL.getBnRoleMapperValue());
			}
		}
		return (cm);
	}

	/**
	 * Get Interactive hooks of the project. Returns hooks assigned to project nodes.
	 *
	 * @return Collection of ProjectInterHookValue - project inter hooks
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getInterHooks() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());

		Collection hk = new ArrayList();
		BnProjectInterHookLocal pl;
		Collection pls = mProject.getBnInterHooks();

		for (Iterator i = pls.iterator(); i.hasNext();) {
			pl = (BnProjectInterHookLocal) i.next();
			hk.add(pl.getBnProjectInterHookValue());
		}
		return (hk);
	}


	/**
	 * Get Node inter hook of the project
	 *
	 * @param nodeName the name of the node
	 * @param interHook name of the hooks
	 * @return BnNodeInterHookValue object - node hook data
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public BnNodeInterHookValue getNodeInterHook(String nodeName, String interHook) throws HeroException {

		trace.debug("parameters: node=" + nodeName + " key=" + interHook + " process name = " + this.getName()+ " started by " + this.getCaller());
		BnNodeInterHookLocal hl;
		BnNodeLocal nd = mProject.getBnNode(nodeName);
		Collection hls = nd.getBnInterHooks();

		for (Iterator i = hls.iterator(); i.hasNext();) {
			hl = (BnNodeInterHookLocal) i.next();
			if (hl.getName().equals(interHook))
				return (hl.getBnNodeInterHookValue());
		}
		return (null);
	}

	/**
	 * Get properties key of the project. A property is a pair key/value representing workflow relevant data. This
	 * methods obtains properties key of the project.
	 * @return Collection of String - all project properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getPropertiesKey() throws HeroException {

		trace.debug("started by " + this.getCaller());
		Collection prp = new ArrayList();
		BnProjectPropertyLocal ppl;

		Collection pls = mProject.getBnProperties();
		for (Iterator i = pls.iterator(); i.hasNext();) {
			ppl = (BnProjectPropertyLocal) i.next();
			prp.add(ppl.getBnProjectPropertyValue().getTheKey());
		}
		return (prp);
	}

	/**
	 * Get ALL project iterations.
	 *
	 * @return Collection of BnIterationLightValue - all project iterations
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Object[] getIterations() throws HeroException {

		trace.debug("started by " + this.getCaller());
		Collection ite = new ArrayList();
		BnIterationLocal itel;

		Collection itls = mProject.getBnIterations();
		for (Iterator i = itls.iterator(); i.hasNext();) {
			itel = (BnIterationLocal) i.next();
			BnIterationLightValue iv = (BnIterationLightValue)itel.getBnIterationLightValue();
			ite.add(iv);
		}
		return (ite.toArray());
	}

	/**
	 * Get nodes destinations of this iteration. <br>
	 * We can have more than 1 iteration starting in the same node.
	 *
	 * @return Collection
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getIterations(String from) throws HeroException {

		trace.debug("parameters: from=" + from+"started by " + this.getCaller());
		BnIterationLocalHome iteh;

		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			Collection iterations = iteh.findByFrom(mProject.getId(),from);
			return (iterations);
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			return (null);
		}
	}

	/**
	 * Verifies if this node starts one iteration or more iterations
	 *
	 * @return boolean
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public boolean getIterationExist(String from) throws HeroException {

		trace.debug("parameters: from=" + from+"started by " + this.getCaller());
		BnIterationLocalHome iteh;

		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			Collection iterations = iteh.findByFrom(mProject.getId(),from);
			if (iterations.isEmpty()) {
				return false;
			} else {
				return true;
			}
		} catch (FinderException fe) {
			return (false);
		}
	}

	/**
	 * Verifies if an iteration exist between both nodes
	 *
	 * @return boolean
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public boolean getIterationExist(String from, String to) throws HeroException {

		trace.debug("parameters: from=" + from+" to=" + to+" started by " + this.getCaller());
		BnIterationLocalHome iteh;

		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			iteh.findByFromTo(mProject.getId(),from, to);
		} catch (FinderException fe) {
			return (false);
		}
		return true;
	}

	/**
	 * Get iterations conditions <br>
	 * We can have more than 1 iteration starting in the same node.
	 *
	 * @return Collection - collection of conditions that starts in node from
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getIterationConditions(String from) throws HeroException {

		trace.debug("parameters: from=" + from+"started by " + this.getCaller());
		BnIterationLocalHome iteh;

		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
		try {
			Collection iterations = iteh.findByFrom(mProject.getId(),from);
			return (iterations);
		} catch (FinderException fe) {
			trace.error(fe.getMessage());
			return (null);
		}
	}

	/**
	 * Get properties of the project. A property is a pair key/value representing workflow relevant data. This
	 * method obtains properties of the project.
	 * @return Collection of ProjectPropertyValue - all project properties
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getProperties() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection prp = new ArrayList();
		BnProjectPropertyLocal ppl;

		Collection pls = mProject.getBnProperties();
		for (Iterator i = pls.iterator(); i.hasNext();) {
			ppl = (BnProjectPropertyLocal) i.next();
			prp.add(ppl.getBnProjectPropertyValue());
		}
		return (prp);
	}

	/**
	 * Get hooks of the project. Method to get the list of Java or TCL hooks of the project
	 *
	 * @return Collection of BnProjectHooksValue - all project hook
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getHooks() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		Collection hooks = new ArrayList();
		BnProjectHookLocal phl;

		Collection hks = mProject.getBnHooks();
		for (Iterator i = hks.iterator(); i.hasNext();) {
			phl = (BnProjectHookLocal) i.next();
			hooks.add(phl.getBnProjectHookValue());
		}
		return (hooks);
	}


	/**
	 * Get the project name of this instance
	 *
	 * @param instanceName the name of the instance
	 * @return String - the project name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getProjectNameOfInstance(String instanceName) throws HeroException {

		trace.debug("parameters: instanceName=" + instanceName + " process name = " + this.getName()+ " started by " + this.getCaller());
		if (instanceName.matches(".*"+"_instance"+".*")){
			int i = instanceName.indexOf("_instance");
			return(instanceName.substring(0,i));
		}
		else {
			throw new HeroException(instanceName+" is not a project instance...");
		}
	}

	/**
	 * Get the parent project of this project
	 *
	 * @return String - the parent project of this project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getParent() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		return(mProject.getParent());
	}

	/**
	 * Test if projects nodes are terminated
	 *
	 * @return boolean - the result of the test
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public boolean isTerminated() {
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal node = (BnNodeLocal) i.next();
			int state = node.getState();
			if (state != hero.interfaces.Constants.Nd.TERMINATED
					&& state != hero.interfaces.Constants.Nd.EXPIRED
					&& state != hero.interfaces.Constants.Nd.DEAD)
				return false;
		}
		return true;
	}

	// -------------------------------------------------------------------------
	// Struts business methods
	// -------------------------------------------------------------------------

	/**
	 * Get the StrutsNode. Returns node information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @param nodeName the name of the Node
	 * @return BnStrutsNodeValue object - node data.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public StrutsNodeValue getStrutsNode(String nodeName)
	throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocalHome nHome;
		BnNodeLocal n;
		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {

			trace.debug("getStrutsNode: nodeName=" + nodeName);
			n = nHome.findByName(nodeName, mProject.getId());

			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator());
			snv.setRole(n.getBnRole().getName());
			snv.setState(
					hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor());
			snv.setAnticipable(n.getAnticipable());
			/* Modif MBL
			 if (n.getDeadline() != null)
			 snv.setDeadline(n.getDeadline().toString());
			 */

			if (n.getDeadlines().size() != 0) {
				Collection co = n.getDeadlines();
				Iterator it = co.iterator();
				Date date = null;
				while (it.hasNext()){
					date = (Date)it.next();
				}
				snv.setDeadline(date.toString());
			}
			return (snv);
		} catch (FinderException fe) {

			trace.error("In BnNode does not exist " + fe.getMessage());
			throw new HeroException("In BnNode does not exist");
		}
	}


	/**
	 * Get the project nodes. Returns project nodes data.
	 *
	 * @return Array of StrutsNodeValue - all project nodes
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Object[] getNodes() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());
		//ProjectNodes nb = new ProjectNodes();
		ArrayList ns = new ArrayList();
		Collection nodes = new ArrayList();

		nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setProjectName(mProject.getName());
			snv.setName(n.getName());
			snv.setCreator(n.getCreator());
			snv.setRole(n.getBnRole().getName());
			snv.setState(Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(Constants.Nd.nodeTypeName[n.getType()]);
			if (n.getDescription() != null)
				snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor());
			snv.setAnticipable(n.getAnticipable());
			/* Modif MBL
			 if (n.getDeadline() != null)
			 snv.setDeadline(n.getDeadline().toString());
			 */
			if (n.getDeadlines().size() != 0) {
				Collection co = n.getDeadlines();
				Iterator it = co.iterator();
				Date date = null;
				while (it.hasNext()){
					date = (Date)it.next();
				}
				snv.setDeadline(date.toString());
			}

			ns.add(snv);
		}
		return (ns.toArray());
	}

	/**
	 * Get the StrutsNodes. Returns nodes information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @return Collection of StrutsNodeValue - all project nodes
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsNodes() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());

		//ctor strutsNodes = new Vector();
		ArrayList strutsNodes = new ArrayList();
		Collection nodes = mProject.getBnNodes();
		Iterator i = nodes.iterator();
		while (i.hasNext()) {
			BnNodeLocal n = (BnNodeLocal) i.next();
			StrutsNodeValue snv = new StrutsNodeValue();
			snv.setName(n.getName());
			snv.setCreator(n.getCreator());
			snv.setRole(n.getBnRole().getName());
			snv.setState(
					hero.interfaces.Constants.Nd.nodeStateName[n.getState()]);
			snv.setType(hero.interfaces.Constants.Nd.nodeTypeName[n.getType()]);
			snv.setDescription(n.getDescription());
			if (n.getExecutor() != null)
				snv.setExecutor(n.getExecutor());
			snv.setAnticipable(n.getAnticipable());
			/* modif MBL
			 if (n.getDeadline() != null)
			 snv.setDeadline(n.getDeadline().toString());
			 */
			if (n.getDeadlines().size() != 0) {
				Collection co = n.getDeadlines();
				Iterator it = co.iterator();
				Date date = null;
				while (it.hasNext()){
					date = (Date)it.next();
				}
				snv.setDeadline(date.toString());
			}
			strutsNodes.add(snv);
		}
		return ((Collection) strutsNodes);
	}

	/**
	 * Get the Edges of the Node. Returns node edges information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @param nodeName the name of the Node
	 * @return Collection of StrutsEdgeValue - all node edges
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsNodeEdges(String nodeName)
	throws HeroException {

		trace.debug("parameter: nodeName=" + nodeName + " process name = " + this.getName()+ " started by " + this.getCaller());

		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnNodeValue nv;

		//	Vector edges = new Vector();
		ArrayList edges = new ArrayList();

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			mNode = nHome.findByName(nodeName, mProject.getId());
		} catch (FinderException fe) {

			trace.error("In Node does not exist " + fe.getMessage());
			throw new HeroException("In Node does not exist");
		}
		Collection in = mNode.getInBnEdges();
		Collection out = mNode.getOutBnEdges();

		Iterator iin = in.iterator();
		while (iin.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			hero.interfaces.BnEdgeLocal actualEdge =(hero.interfaces.BnEdgeLocal) iin.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}

		Iterator iout = out.iterator();
		while (iout.hasNext()) {
			StrutsEdgeValue sev = new StrutsEdgeValue();
			BnEdgeLocal actualEdge = (BnEdgeLocal) iout.next();
			sev.setState(hero.interfaces.Constants.Ed.edgeStateName[actualEdge.getState()]);
			sev.setInNode(((BnNodeLocal) actualEdge.getInBnNode()).getName());
			sev.setOutNode(((BnNodeLocal) actualEdge.getOutBnNode()).getName());
			edges.add(sev);
		}
		return ((Collection) edges);
	}

	/**
	 * Get the StrutsEdges. Returns project edges information with a specific java bean structure (useful in
	 * struts web applications).
	 *
	 * @return Collection of StrutsEdgeValue - all project edges
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public Collection getStrutsEdges() throws HeroException {

		trace.debug(" process name = " + this.getName()+ " started by " + this.getCaller());

		//	Vector strutsEdges = new Vector();
		ArrayList strutsEdges = new ArrayList();
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal e = (BnEdgeLocal) i.next();
			StrutsEdgeValue sev = new StrutsEdgeValue();
			sev.setName(e.getName());
			sev.setState(
					hero.interfaces.Constants.Ed.edgeStateName[e.getState()]);
			sev.setInNode(e.getInBnNode().getName());
			sev.setOutNode(e.getOutBnNode().getName());
			strutsEdges.add(sev);
		}
		return ((Collection) strutsEdges);
	}

	/**
	 * Import all the nodes, hooks and edges of a project to the current project
	 *
	 * @param projectname the name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importProject(String projectName, String projectVersion) throws HeroException {

		trace.debug("parameter: project=" + projectName + " process name = " + this.getName()+ " started by " + this.getCaller());
		importProject(projectName, projectVersion, "");
	}

	//	-------------------------------------------------------------------------
	// ejbCreate and initProject business methods
	// -------------------------------------------------------------------------

	/**
	 * Import all the nodes, hooks, properties and edges of a project to the current project
	 *
	 * @param projectname the name of the project
	 * @param prefix the prefix of names for the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public void importProject(String projectName, String projectVersion, String prefix)
	throws HeroException {

		trace.debug("parameters: project=" + projectName + " prefix=" + prefix + " process name = " + this.getName()+ " started by " + this.getCaller());
		try {
			BnProjectLocalHome opHome =	hero.interfaces.BnProjectUtil.getLocalHome();
			//MBL changed
			BnProjectLocal oldProject = opHome.findByNameVersion(projectName, projectVersion);
			String actualProject = this.getName();
			//MBL changed
			String actualProjectVersion = this.getVersion();
			BnProjectValue pv = oldProject.getBnProjectValue();

			mProject.setType(oldProject.getType());
			mProject.setStatus(Constants.Pj.ACTIVE);

			// Project Properties (default value)
			Collection props = oldProject.getBnProperties();
			for (Iterator i = props.iterator(); i.hasNext();) {
				BnProjectPropertyLocal prop = (BnProjectPropertyLocal) i.next();
				this.setPropertyPossibleValues(prop.getTheKey(),prop.getPossibleValues());
				if (prop.getTheValue()!=null || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
				this.setProperty(prop.getTheKey(),prop.getTheValue());
			}

			//Project InterHooks
			Collection hooks = oldProject.getBnInterHooks();
			for (Iterator i = hooks.iterator(); i.hasNext();) {
				BnProjectInterHookLocal hk = (BnProjectInterHookLocal) i.next();
				this.addInterHook(hk.getName(),hk.getEvent(),hk.getType(),hk.getScript());
			}

			//Project Hooks
			Collection pHooks = oldProject.getBnHooks();
			for (Iterator i = pHooks.iterator(); i.hasNext();) {
				BnProjectHookLocal hk = (BnProjectHookLocal) i.next();
				this.addHook(hk.getName(),hk.getEvent(),hk.getType());
			}

			// Add role and roleMappers
			Collection roles = oldProject.getBnRoles();
			for (Iterator i = roles.iterator(); i.hasNext();)
			{
				BnRoleLocal ro = (BnRoleLocal)i.next();
				String roleName = ro.getName();
				// add mapper for the instance
				if (!containsRole(roleName))
					this.addRole(roleName, ro.getDescription());
				if (ro.getBnRoleMapper()!=null)
					this.addRoleMapper(ro.getName(),ro.getBnRoleMapper().getName(),ro.getBnRoleMapper().getType());
			}

			// Add the InitiatorMapper
			if (oldProject.getBnInitiatorMapper()!=null)
				this.addInitiatorMapper(oldProject.getBnInitiatorMapper().getName(),oldProject.getBnInitiatorMapper().getType());

			//	Copy not connected nodes
			Collection allNodes = oldProject.getBnNodes();
			Iterator no = allNodes.iterator();
			while (no.hasNext()) {
				BnNodeLocal node = (BnNodeLocal) no.next();
				if (node.getInBnEdges().isEmpty()&& node.getOutBnEdges().isEmpty()) {
					// New Instance if activity is a subProcess
					String newNodeName=node.getName();
					String instName=null;
					if (node.getType() == Constants.Nd.SUB_PROCESS_NODE) // First activity is a subProcess
						this.addNodeSubProcess(newNodeName, BonitaParsing.getSubProcessName(node.getReference()), BonitaParsing.getSubProcessVersion(node.getReference()));
					else
						this.addNode(newNodeName,node.getType());

					//this.addNode(newNodeName, node.getType());
					if (!node.getAnticipable())
						this.setNodeTraditional(newNodeName);

					// For deadlines
					if (node.getDeadlines().size()!=0){
						Collection co = node.getDeadlines();
						Iterator it = co.iterator();
						ArrayList ar = new ArrayList();
						while (it.hasNext()){
							Date date = (Date)it.next();
							ar.add(new Long(date.getTime()));
						}
						this.setNodeDeadlines(newNodeName,ar);
					}
					// For relative deadlines
					if (node.getRelativeDeadlines().size()!=0){
						Collection co = node.getRelativeDeadlines();
						Iterator it = co.iterator();
						ArrayList ar = new ArrayList();
						while (it.hasNext()){
							Date date = (Date)it.next();
							ar.add(new Long(date.getTime()));
						}
						this.setNodeRelativeDeadlines(newNodeName,ar);
					}
					this.setNodeRole(newNodeName,node.getBnRole().getName());
					this.setNodeDescription(newNodeName, node.getDescription());

					if (node.getBnNodePerformerAssign()!=null) {
						this.addNodePerformerAssign(newNodeName,node.getBnNodePerformerAssign().getName(),node.getBnNodePerformerAssign().getType(),node.getBnNodePerformerAssign().getPropertyName());
					}

					//Node InterHooks
					Collection nodeHooks = node.getBnInterHooks();
					for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
						BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
						try {
							this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
						} catch (Exception ih) {

							trace.error(ih.getMessage());
						}; // For subProcess Instance consistency
					}

					  // Node Properties
                    Collection nodeprops = node.getBnProperties();
                    for (Iterator i = nodeprops.iterator(); i.hasNext(); )
                    {
                          BnNodePropertyLocal prop = (BnNodePropertyLocal)i.next();
						  this.setNodePropertyPossibleValues(newNodeName,prop.getTheKey(),prop.getPossibleValues());
						  if (prop.getTheValue()!=null || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
							  this.setNodeProperty(newNodeName,prop.getTheKey(), prop.getTheValue(),false);
                    }

					//Node Hooks
					Collection nHooks = node.getBnHooks();
					for (Iterator j = nHooks.iterator(); j.hasNext();) {
						BnNodeHookLocal nh = (BnNodeHookLocal) j.next();
						try {
							this.addNodeHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType());
						} catch (Exception ih) {}; // For subProcess Instance consistency
					}
				}
			}

			Collection edges = oldProject.getBnEdges();
			Iterator e = edges.iterator();
			while (e.hasNext())
			{
				BnEdgeLocal ed = (BnEdgeLocal)e.next();
				BnNodeLocal nodeIn = ed.getInBnNode();
				BnNodeLocal nodeOut = ed.getOutBnNode();
				BnNodeLocal[] nodes = { nodeIn, nodeOut };
				String newNodeNameIn = null;
				String newNodeNameOut = null;
				for (int n = 0; n < nodes.length; n++) {
					String newNodeName = nodes[n].getName();
					try {
						if (nodes[n].getType() == Constants.Nd.SUB_PROCESS_NODE) // First activity is a subProcess
							this.addNodeSubProcess(newNodeName, BonitaParsing.getSubProcessName(nodes[n].getReference()), BonitaParsing.getSubProcessVersion(nodes[n].getReference()));
						else
							this.addNode(newNodeName,nodes[n].getType());
						if (!nodes[n].getAnticipable())
							this.setNodeTraditional(newNodeName);

						// For deadlines
						if (nodes[n].getDeadlines().size()!=0){
							Collection co = nodes[n].getDeadlines();
							Iterator it = co.iterator();
							ArrayList ar = new ArrayList();
							while (it.hasNext()){
								Date date = (Date)it.next();
								ar.add(new Long(date.getTime()));
							}
							this.setNodeDeadlines(newNodeName,ar);
						}
						// For relative deadlines
						if (nodes[n].getRelativeDeadlines().size()!=0){
							Collection co = nodes[n].getRelativeDeadlines();
							Iterator it = co.iterator();
							ArrayList ar = new ArrayList();
							while (it.hasNext()){
								Date date = (Date)it.next();
								ar.add(new Long(date.getTime()));
							}
							this.setNodeRelativeDeadlines(newNodeName,ar);
						}
						this.setNodeRole(newNodeName,nodes[n].getBnRole().getName());
						this.setNodeDescription(newNodeName,nodes[n].getDescription());

						if (nodes[n].getBnNodePerformerAssign()!=null) {
							this.addNodePerformerAssign(newNodeName,nodes[n].getBnNodePerformerAssign().getName(),nodes[n].getBnNodePerformerAssign().getType(),nodes[n].getBnNodePerformerAssign().getPropertyName());
						}

						//Node InterHooks
						BnNodeLocal nd = oldProject.getBnNode(nodes[n].getName());
						Collection nodeHooks = nd.getBnInterHooks();
						for (Iterator j = nodeHooks.iterator(); j.hasNext();) {
							BnNodeInterHookLocal nh = (BnNodeInterHookLocal) j.next();
							try {
								this.addNodeInterHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType(),nh.getScript());
							} catch (Exception ih) {

								trace.error(ih.getMessage());
							}; // For subProcess Instance consistency
						}

                        // Node Properties
                        Collection nodeprops = nodes[n].getBnProperties();
                        for (Iterator i = nodeprops.iterator(); i.hasNext(); )
                        {
                              BnNodePropertyLocal prop = (BnNodePropertyLocal)i.next();
							  this.setNodePropertyPossibleValues(newNodeName,prop.getTheKey(),prop.getPossibleValues());
							  if (prop.getTheValue()!=null || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
								  this.setNodeProperty(newNodeName, prop.getTheKey(), prop.getTheValue(), false);
                        }

						//Node Hooks
						Collection nHooks = nd.getBnHooks();
						for (Iterator j = nHooks.iterator(); j.hasNext();) {
							BnNodeHookLocal nh = (BnNodeHookLocal) j.next();
							try {
								this.addNodeHook(newNodeName,nh.getName(),nh.getEvent(),nh.getType());
							} catch (Exception ih) {}; // For subProcess Instance consistency
						}

					} catch (Exception an) {

						trace.error(an.getMessage());
					}; // Node already exist
					if (n == 0)
						newNodeNameIn = newNodeName;
					else
						newNodeNameOut = newNodeName;
				}

				String edgeName = this.addEdge(newNodeNameIn, newNodeNameOut);
				this.setEdgeCondition(edgeName, ed.getCondition());
			}

			// Project Iterations
			Collection ites = oldProject.getBnIterations();
			Iterator iterations = ites.iterator();
			while (iterations.hasNext())
			{
				BnIterationLocal ite = (BnIterationLocal)iterations.next();
				this.addIteration(ite.getFromNode(),ite.getToNode(),ite.getCondition());
			}

			// Users and roles

			BnUserLightValue[] users = pv.getBnUsers();
			int uIndex = 0;
			for (uIndex=0;uIndex<users.length;uIndex++)
			{
				BnUserLightValue userP = users[uIndex];
				try {
					this.addUser(userP.getName());
				} catch (Exception eUser) {

					trace.error(eUser.getMessage());
					if (jms!=null)
						jms.sendProjectEvent(ADDUSERPROJECT,mProject.getName(),userP.getName());
				}
				mProject = oldProject;
				Collection userRoles = this.getUserRolesInProjectNames(userP.getName());
				mProject = opHome.findByNameVersion(actualProject, actualProjectVersion);
				Iterator ir = userRoles.iterator();
				while (ir.hasNext())
					this.setUserRole(userP.getName(),(String) ir.next());
			}


		} catch (FinderException e) {

			trace.error("Cannot find project " + e.getMessage());
			throw new HeroException("Cannot find project " + e.getMessage());
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
	}

	/**
	 * Import Project instance. Import all the nodes, hooks, properties,edges and users of a project instant to the
	 * current project
	 *
	 * @param projectname the name of the project
	 * @param instance the instance name of the project
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 *
	 **/
	public String importInstance(BonitaProjectValue pv, String instance, String version, Hashtable initProperties)
	throws HeroException {
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		BnProjectLocalHome opHome = null;
		UserServiceLocalHome ushome;
		//for deadline based on EJB timers
		DeadlineEjbTimerSessionLocalHome dlHome;
		DeadlineEjbTimerSessionLocal dlSession;
		try {
			dlHome = DeadlineEjbTimerSessionUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			dlSession = dlHome.create();

	} catch (CreateException he) {
		throw new HeroException("importInstance: Error creating DeadlineEjbTimerSessionLocal");
	}

		trace.debug("parameter: project=" + pv.getName() + " started by " + this.getCaller());
		try {
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
			opHome = hero.interfaces.BnProjectUtil.getLocalHome();

			String creator = this.getCaller();
			mProject = opHome.create(instance, version, creator);
			mProject.setType(Constants.Pj.INSTANCE);
			mProject.setStatus(Constants.Pj.ACTIVE);

			instance=mProject.getName();
			version = mProject.getVersion();
			mProject.setParent(mProject.getName()); // Only for subprocesses

			BnUserLocal user = uhome.findByName(creator);
			mProject.getBnUsers().add(user);
			// role admin is added
			BnRoleValue rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.ADMIN);
			rv.setDescription("the admin role");
			BnRoleLocal role = rhome.create(rv);
			mProject.getBnRoles().add(role);
			// Set User Role
			user.getBnRoles().add(role);

			// role InitialRole is added
			rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.INITIALROLE);
			rv.setDescription("the initial role for nodes");
			role = rhome.create(rv);
			mProject.getBnRoles().add(role);

			if (!creator.equals(Constants.ADMIN))
			{
				// role username is added
				rv = new BnRoleValue();
				rv.setName(creator);
				rv.setDescription(creator+" user role");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				// Set User Role
				user.getBnRoles().add(role);
			}

			// Project Properties
			Collection propers = pv.getProperties();
			Iterator props = propers.iterator();
			while (props.hasNext())
			{
				BonitaPropertyValue prop = (BonitaPropertyValue) props.next();
				this.setPropertyPossibleValues(prop.getKey(),prop.getPossibleValues());
				if (prop.getValue()!=null || prop.getPossibleValues() == null || prop.getPossibleValues().size()==0)
					this.setProperty(prop.getKey(),prop.getValue());

			}

			// Default properties (if needed)
			Iterator initProps = initProperties.keySet().iterator();
			while (initProps.hasNext())
			{
				String key = (String) initProps.next();
				if (initProperties.get(key)!=null)
					this.setProperty(key,(String)initProperties.get(key));

			}

			// Project Roles
			Collection roles = pv.getRoles();
			Iterator allRoles = roles.iterator();
			while (allRoles.hasNext())
			{
				BonitaRoleValue ro = (BonitaRoleValue)allRoles.next();
				String roleName = ro.getName();
				// add mapper for the instance
				if (!containsRole(roleName))
					this.addRole(roleName, ro.getDescription());
				if (ro.getMapper()!=null)
					this.addRoleMapper(ro.getName(),ro.getMapper().getName(),ro.getMapper().getType());
			}

			//	First activity/ies
			Collection nds = pv.getNodes();
			Iterator nodes = nds.iterator();
			while (nodes.hasNext())
			{
				BonitaNodeValue nd = (BonitaNodeValue)nodes.next();
				if (nd.getInEdges().isEmpty())
				{
					String instName=null;
					String newNodeName = nd.getName();
					if (nd.getType() == Constants.Nd.SUB_PROCESS_NODE) // First activity is a subProcess
					{
						String subProcessName=BonitaParsing.getSubProcessName(nd.getReference());
						String subProcessVersion=BonitaParsing.getSubProcessVersion(nd.getReference());

						instName = this.instantiateProject(subProcessName, subProcessVersion);
						BnProjectLocal newInstance = opHome.findByName(instName);
						newInstance.setParent(instance+"_node-"+newNodeName);
						this.initProject(instance);

					}

					this.addNode(newNodeName,nd.getType());

					if (nd.getType() == Nd.SUB_PROCESS_NODE)
					{
						BnNodeLocal newNode = mProject.getBnNode(nd.getName());
						newNode.setReference(instName);
					}

					if (!nd.getAnticipable())
						this.setNodeTraditional(newNodeName);

					// For Deadlines
					if (nd.getDeadlines().size()!=0){
						Collection co = nd.getDeadlines();
						Iterator it = co.iterator();
						ArrayList ar = new ArrayList();
						while (it.hasNext()){
							Date date = (Date)it.next();
							ar.add(new Long(date.getTime()));
						}
						this.setNodeDeadlines(newNodeName,ar);
					}
					// For relative Deadlines
					if (nd.getRelativeDeadlines().size()!=0){
						Collection co = nd.getRelativeDeadlines();
						Iterator it = co.iterator();
						ArrayList ar = new ArrayList();
						while (it.hasNext()){
							Date date = (Date)it.next();
							ar.add(new Long(date.getTime()));
						}
						this.setNodeRelativeDeadlines(newNodeName,ar);
					}
					this.setNodeRole(newNodeName,nd.getRole());
					this.setNodeDescription(newNodeName,nd.getDescription());
					if (!nd.getProjectParent().equals(nd.getProjectName()))
						this.setNodeState(mProject.getBnNode(newNodeName),Constants.Nd.INITIAL);

					Collection nodeProp = nd.getProperties();
					Iterator nodeProps = nodeProp.iterator();
					while (nodeProps.hasNext())
					{
						BonitaPropertyValue ndProp = (BonitaPropertyValue)nodeProps.next();
						this.setNodePropertyPossibleValues(newNodeName,ndProp.getKey(),ndProp.getPossibleValues());
						if (ndProp.getValue()!=null || ndProp.getPossibleValues() == null || ndProp.getPossibleValues().size()==0)
							this.setNodeProperty(newNodeName,ndProp.getKey(),ndProp.getValue(),ndProp.getPropagate());
					}

					if (nd.getPerformer()!=null) {
						this.addNodePerformerAssign(newNodeName,nd.getPerformer().getName(),nd.getPerformer().getType(),nd.getPerformer().getPropertyName());
					}

					//	For Deadlines or relative deadlines
					if (nd.getDeadlines().size()!=0 || nd.getRelativeDeadlines().size()!=0)
						dlSession.startDeadlineEjbTimers(newNodeName, mProject.getId());

					// Project Iterations
					Collection iterations = pv.getIterations();
					Iterator iter = iterations.iterator();
					while (iter.hasNext())
					{
						BonitaIterationValue iteration = (BonitaIterationValue) iter.next();
						String from = iteration.getFromNode();
						if (newNodeName.matches(from+"_instance.*") || newNodeName.equals(from))
						{
							this.addIteration(newNodeName, newNodeName,iteration.getCondition());
							break;
						}
					}
					if ((nd.getType()==Constants.Nd.SUB_PROCESS_NODE || nd.getType()==Constants.Nd.AND_JOIN_AUTOMATIC_NODE || nd.getType()==Constants.Nd.OR_JOIN_AUTOMATIC_NODE)
							&& (nd.getProjectParent().equals(nd.getProjectName())))
					{
						EngineLocalHome enghome;
						try {
							enghome = hero.interfaces.EngineUtil.getLocalHome();
							EngineLocal engine = enghome.create();
							engine.startActivity(mProject.getName(), newNodeName);
						} catch (javax.naming.NamingException ne) {
							throw new HeroException(ne.getMessage());
						}
					}

				}
			}
			Collection users = pv.getUsers();
			Iterator usersI = users.iterator();
			while(usersI.hasNext())
			{
				BonitaUserValue userP = (BonitaUserValue)usersI.next();
				try {
					this.addUserLocal(userP.getName());
				} catch (Exception eUser) {

					trace.error(eUser.getMessage());
				}
				Collection userRoles = userP.getRoles();
				Iterator ir = userRoles.iterator();
				while (ir.hasNext())
				{
					String roleName = (String) ir.next();
					this.setUserRoleLocal(userP.getName(),roleName);
				}
			}
			mProject.setCreationDate(new Date());

			// mapper execution
			this.mapRole();

			this.executeOnReadyHook();
			if (jms!=null)
				jms.sendProjectEvent(ADDUSERPROJECT, instance, creator);
			return instance;
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (Exception ce) {ce.printStackTrace();

		trace.error("Instantiate process error " + ce.getMessage());
		throw new HeroException("Instantiate process error " + ce.getMessage());
		}
	}

	/**
	 * Test if the project with projectName name exists.
	 *
	 * @param projectName project name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public boolean existingProject(String projectName) throws HeroException {
		UserServiceLocalHome ushome = null;
		mProject=null;

		trace.debug("parameter: project=" + projectName + " started by " + this.getCaller());
		if (projectName.equals(""))
			throw new HeroException("Project name can not be empty...");
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();

			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(this.getCaller());
		} catch (Exception ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			pHome.findByNameVersion(projectName, EventConstants.DEFAULTPROJECTVERSION);
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			return false;

		}
		return true;
	}

	/**
	 * Test if the project with projectName and projectVersion name exists.
	 *
	 * @param projectName project name
	 * @param projectName project name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public boolean existingProject(String projectName, String projectVersion) throws HeroException {
		UserServiceLocalHome ushome = null;
		mProject=null;

		trace.debug("parameter: project=" + projectName + " started by " + this.getCaller());
		if (projectName.equals(""))
			throw new HeroException("Project name can not be empty...");
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();

			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(this.getCaller());
		} catch (Exception ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			pHome.findByNameVersion(projectName, projectVersion);
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			return false;

		}
		return true;
	}


	/**
	 * Creates a cooperative workflow or creates a pointer to the Project Session Bean for this cooperative/instance
	 * element. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param projectName project name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initProject(String projectName) throws HeroException {
		Context lContext;
		UserServiceLocalHome ushome = null;
		mProject=null;

		trace.debug("parameter: project=" + projectName + " started by " + this.getCaller());
		if (projectName.equals(""))
			throw new HeroException("Project name can not be empty...");
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();

			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(this.getCaller());
		} catch (Exception ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			//this.mProject = pHome.findByNameVersion(projectName, EventConstants.DEFAULTPROJECTVERSION);
			this.mProject = pHome.findByName(projectName);
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			try{
				// for compatibility set default version
				this.createProcess(projectName, EventConstants.DEFAULTPROJECTVERSION); // Create Process
				mProject.setType(Constants.Pj.COOPERATIVE);
				mProject.setStatus(Constants.Pj.ACTIVE);
			}catch (CreateException ce){

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}


	/**
	 * Executes the OnInstatiate hook associated to this process. This method can only be called before
	 * a workflow model is instantiated.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 **/
	public void executeProcessHook() throws HeroException {
		if (mProject.getType().equals(Constants.Pj.MODEL))
		{
			Collection pHooks = mProject.getBnHooks();
			Iterator i = pHooks.iterator();
			while (i.hasNext())
			{
				BnProjectHookLocal ph = (BnProjectHookLocal)i.next();
				if (ph.getEvent().equalsIgnoreCase(Constants.Pj.ONINSTANTIATE)) {
					hero.hook.ProcessHook h = hero.hook.ProcessHook.make(ph.getName(), ph.getEvent(), ph.getType());
					h.execute(this, ph.getEvent(), mProject);
				}
			}
		}
		else {
			throw new HeroException("This method is not available for workflow instances...");
		}
	}

	/**
	 * Executes the OnReady hook associated to the first activities of the process (those ones
	 * for which there is no ingoing transitions). This method must only be called after
	 * a workflow model is instantiated.
	 **/
	public void executeOnReadyHook() throws HeroException {

		try{
		BnNodeLocalHome nHome=null;
		if (mProject.getType().equals(Constants.Pj.INSTANCE))
		{
			performAssign();
			//	First activity/ies
//			BonitaProjectValue pv = BonitaProjectLocator.getInstance().getModelValue(this.getModel(this.getName()), this.getModel(this.getVersion()) );
			BonitaProjectValue pv = BonitaProjectLocator.getInstance().getModelValue(this.getModel(this.getName()), this.getVersion() );

			Collection nds = pv.getNodes();
			Iterator nodes = nds.iterator();
			while (nodes.hasNext())
			{
				BonitaNodeValue nd = (BonitaNodeValue)nodes.next();
				if (nd.getInEdges().isEmpty())
				{
					Collection hks = nd.getHooks();
					Iterator i = hks.iterator();
					while (i.hasNext())
					{
						BonitaHookValue bh = (BonitaHookValue)i.next();
						if (bh.getEvent().equalsIgnoreCase(Constants.Nd.ONREADY)) {
							hero.hook.Hook h = hero.hook.Hook.make(bh.getName(), bh.getEvent(), bh.getType());
							try {
								nHome = hero.interfaces.BnNodeUtil.getLocalHome();
							} catch (javax.naming.NamingException be) {
								trace.error(be.getMessage());
								throw new HeroException(be.getMessage());
							}
							try {
								BnNodeLocal nf = nHome.findByName(nd.getName(), mProject.getId());
								h.execute(this, bh.getEvent(), nf);
							} catch (javax.ejb.FinderException nn) {throw new HeroException("This node does not exist...");}
						}
					}
				}
			}
		}
		else
			throw new HeroException("This method is not available for workflow models...");
		}catch(Exception e){e.printStackTrace();throw new HeroException(e.getMessage());}
	}

	/**
	 * Creates a workflow process model or Init the Project Session Bean for this model. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param modelName model name
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initModel(String modelName) throws HeroException {
		Context lContext;
		UserServiceLocalHome ushome = null;
		mProject=null;

		trace.debug("parameter: project=" + modelName + " started by " + this.getCaller());
		if (modelName.equals(""))
			throw new HeroException("Project name can not be empty...");
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(this.getCaller());
		} catch (Exception ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			this.mProject = pHome.findByNameVersion(modelName, EventConstants.DEFAULTPROJECTVERSION);

		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			try{
				this.createProcess(modelName, EventConstants.DEFAULTPROJECTVERSION); // Create Process
				mProject.setType(Constants.Pj.MODEL);
				mProject.setStatus(Constants.Pj.ACTIVE);
			}catch (CreateException ce){

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}

	/**
	 * Creates a workflow process model or Init the Project Session Bean for this model. You have to call this method after "create" call. After that
	 * all methods of ProjectSession API are available.
	 *
	 * @param modelName model name
	 * @param modelVersion model version
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void initModelWithVersion(String modelName, String modelVersion) throws HeroException {
		Context lContext;
		UserServiceLocalHome ushome = null;
		mProject=null;

		trace.debug("parameter: project=" + modelName + " started by " + this.getCaller());
		if (modelName.equals(""))
			throw new HeroException("Project name can not be empty...");
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			ul.findUserLocal(this.getCaller());
		} catch (Exception ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		}
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			this.mProject = pHome.findByNameVersion(modelName, modelVersion);

		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			try{
				this.createProcess(modelName, modelVersion); // Create Process
				mProject.setType(Constants.Pj.MODEL);
				mProject.setStatus(Constants.Pj.ACTIVE);
			}catch (CreateException ce){

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
		}
	}


	/**
	 * init the Project Session Bean (clone project).You have to call this method after "create" call. After that
	 * the project is cloned and all methods of ProjectSession API are available.
	 *
	 * @throws HeroException
	 * @param oldProject the name of the project to clone
	 * @param newProject the name of the project clone.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws CreateException
	 **/
	public void initProject(String oldProject, String newProject)
	throws CreateException {
		try{

			trace.debug("parameters: oldProject="+ oldProject+ " newProject="+ newProject + " started by " + this.getCaller());
			Context lContext;
			BnUserLocalHome uhome = null;
			BnRoleLocalHome rhome = null;
			UserServiceLocalHome ushome;
			try {
				ushome = hero.interfaces.UserServiceUtil.getLocalHome();
				UserServiceLocal ul = ushome.create();
				ul.findUserLocal(this.getCaller());
			} catch (Exception ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
			if (oldProject.equals("") || newProject.equals(""))
				throw new CreateException("Project name can not be empty...");
			try {
				pHome = hero.interfaces.BnProjectUtil.getLocalHome();
				uhome = hero.interfaces.BnUserUtil.getLocalHome();
				rhome = hero.interfaces.BnRoleUtil.getLocalHome();
				mProject = pHome.findByNameVersion(newProject, EventConstants.DEFAULTPROJECTVERSION);
			} catch (javax.naming.NamingException ne) {

				trace.error(ne.getMessage());
				throw new CreateException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				// The project has to be created
				try {
					String creator = this.getCaller();
					mProject = pHome.create(newProject, EventConstants.DEFAULTPROJECTVERSION, creator);
					mProject.setParent(newProject);
					BonitaProjectValue pv = new BonitaProjectValue();
					pv.setName(newProject);

					BnUserLocal user = uhome.findByName(creator);
					mProject.getBnUsers().add(user);

					// role admin is added
					BnRoleValue rv = new BnRoleValue();
					rv.setName(hero.interfaces.Constants.ADMIN);
					rv.setDescription("the admin role");
					BnRoleLocal role = rhome.create(rv);
					mProject.getBnRoles().add(role);
					// Set User Role
					user.getBnRoles().add(role);

					// role InitialRole is added
					rv = new BnRoleValue();
					rv.setName(hero.interfaces.Constants.INITIALROLE);
					rv.setDescription("the initial role for nodes");
					role = rhome.create(rv);
					mProject.getBnRoles().add(role);

					if (!creator.equals(Constants.ADMIN))
					{
						// role username is added
						rv = new BnRoleValue();
						rv.setName(creator);
						rv.setDescription(creator+" user role");
						role = rhome.create(rv);
						mProject.getBnRoles().add(role);
						// Set User Role
						user.getBnRoles().add(role);
					}

					importProject(oldProject, EventConstants.DEFAULTPROJECTVERSION);
					mProject.setCreationDate(new Date());

					if (jms!=null)
						jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

				} catch (FinderException fie) {

					trace.error(fie.getMessage());
					throw new CreateException(fie.getMessage());
				} catch (InvalidValueException ie) {

					trace.error(ie.getMessage());
					throw new CreateException(ie.getMessage());
				} catch (HeroException he) {

					trace.error(he.getMessage());
					throw new CreateException(he.getMessage());
				}
			}
		}catch(HeroException traceEx){traceEx.printStackTrace();}
	}

	/**
	 * init the Project Session Bean (clone project).You have to call this method after "create" call. After that
	 * the project is cloned and all methods of ProjectSession API are available.
	 *
	 * @throws HeroException
	 * @param oldProject the name of the project to clone
	 * @param oldVersion the version of the project to clone
	 * @param newProject the name of the project clone.
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws CreateException
	 **/
	public void initProjectWithVersion(String oldProject, String oldVersion, String newProject)
	throws CreateException {
		try{

			trace.debug("parameters: oldProject="+ oldProject+ " newProject="+ newProject + " started by " + this.getCaller());
			Context lContext;
			BnUserLocalHome uhome = null;
			BnRoleLocalHome rhome = null;
			UserServiceLocalHome ushome;
			try {
				ushome = hero.interfaces.UserServiceUtil.getLocalHome();
				UserServiceLocal ul = ushome.create();
				ul.findUserLocal(this.getCaller());
			} catch (Exception ce) {

				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
			if (oldProject.equals("") || newProject.equals(""))
				throw new CreateException("Project name can not be empty...");
			try {
				pHome = hero.interfaces.BnProjectUtil.getLocalHome();
				uhome = hero.interfaces.BnUserUtil.getLocalHome();
				rhome = hero.interfaces.BnRoleUtil.getLocalHome();
				mProject = pHome.findByNameVersion(newProject, oldVersion);
			} catch (javax.naming.NamingException ne) {

				trace.error(ne.getMessage());
				throw new CreateException(ne.getMessage());
			} catch (javax.ejb.FinderException fe) {
				// The project has to be created
				try {
					String creator = this.getCaller();
					mProject = pHome.create(newProject, oldVersion, creator);
					mProject.setParent(newProject);
					BonitaProjectValue pv = new BonitaProjectValue();
					pv.setName(newProject);

					BnUserLocal user = uhome.findByName(creator);
					mProject.getBnUsers().add(user);

					// role admin is added
					BnRoleValue rv = new BnRoleValue();
					rv.setName(hero.interfaces.Constants.ADMIN);
					rv.setDescription("the admin role");
					BnRoleLocal role = rhome.create(rv);
					mProject.getBnRoles().add(role);
					// Set User Role
					user.getBnRoles().add(role);

					// role InitialRole is added
					rv = new BnRoleValue();
					rv.setName(hero.interfaces.Constants.INITIALROLE);
					rv.setDescription("the initial role for nodes");
					role = rhome.create(rv);
					mProject.getBnRoles().add(role);

					if (!creator.equals(Constants.ADMIN))
					{
						// role username is added
						rv = new BnRoleValue();
						rv.setName(creator);
						rv.setDescription(creator+" user role");
						role = rhome.create(rv);
						mProject.getBnRoles().add(role);
						// Set User Role
						user.getBnRoles().add(role);
					}

					//importProject(oldProject);
					importProject(oldProject, oldVersion);
					mProject.setCreationDate(new Date());

					if (jms!=null)
						jms.sendProjectEvent(ADDUSERPROJECT, newProject, creator);

				} catch (FinderException fie) {

					trace.error(fie.getMessage());
					throw new CreateException(fie.getMessage());
				} catch (InvalidValueException ie) {

					trace.error(ie.getMessage());
					throw new CreateException(ie.getMessage());
				} catch (HeroException he) {

					trace.error(he.getMessage());
					throw new CreateException(he.getMessage());
				}
			}
		}catch(HeroException traceEx){traceEx.printStackTrace();}
	}

	/**
	 * Project instance. You have to call this method after "create" call. After that
	 * you have an instance of the project and all methods of ProjectSession API are available.
	 *
	 * @param project the name of the project to instance
	 * @return String - the name of the project instance
  	 * @return Hashtable - default value for properties defined at process level
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public String instantiateProject(String project, Hashtable initProperties) throws CreateException {
		String instance=null;
		try{
			BonitaProjectValue pv = this.instantiateInit(project, EventConstants.DEFAULTPROJECTVERSION );
			instance=project+"_instance";
			if(ServerType.isJonas())
			{
				instance=((ProjectSessionLocal)mContext.getEJBLocalObject()).importInstance(pv, instance, EventConstants.DEFAULTPROJECTVERSION, initProperties);
			}
			else
			{
				/* JBoss does not support Stateful Session Bean reentrance
				 * even if the Thread is the same one. So we need to
				 * invoke the method directly. This *should* work as a
				 * transaction is already begun, as "Support" transaction
				 * attribute is switched to "Required" during deployment
				 * descriptor post processing.
				 */
				instance=this.importInstance(pv, instance, EventConstants.DEFAULTPROJECTVERSION, initProperties);
			}
			thelog.info("User: "+ this.getCaller() + " instantiates project: " + project + ". " +instance + " is created");
		}catch(HeroException traceEx){traceEx.printStackTrace();
			throw new CreateException(traceEx.getMessage());
		}
		return (instance);
	}

	/**
	 * Project instance. You have to call this method after "create" call. After that
	 * you have an instance of the project and all methods of ProjectSession API are available.
	 *
	 * @param project the name of the project to instance
	 * @param version the version of the project to instance
	 * @return String - the name of the project instance
  	 * @return Hashtable - default value for properties defined at process level
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public String instantiateProject(String project, String version, Hashtable initProperties) throws CreateException {
		String instance=null;
		try{
			BonitaProjectValue pv = this.instantiateInit(project, version);
			instance=project+"_instance";
			if(ServerType.isJonas())
			{
				instance=((ProjectSessionLocal)mContext.getEJBLocalObject()).importInstance(pv, instance, version, initProperties);
			}
			else
			{
				/* JBoss does not support Stateful Session Bean reentrance
				 * even if the Thread is the same one. So we need to
				 * invoke the method directly. This *should* work as a
				 * transaction is already begun, as "Support" transaction
				 * attribute is switched to "Required" during deployment
				 * descriptor post processing.
				 */
				instance=this.importInstance(pv, instance, version, initProperties);
			}
			thelog.info("User: "+ this.getCaller() + " instantiates project: " + project +" version: "+version+ ". " +instance + " is created");
		}catch(HeroException traceEx){traceEx.printStackTrace();
			throw new CreateException(traceEx.getMessage());
		}
		return (instance);
	}


	/**
	 * Project instance. You have to call this method after "create" call. After that
	 * you have an instance of the project and all methods of ProjectSession API are available.
	 *
	 * @param project the name of the project to instance
	 * @return String - the name of the project instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public String instantiateProject(String project) throws CreateException {
		String instance=null;
		try{
			BonitaProjectValue pv = this.instantiateInit(project, EventConstants.DEFAULTPROJECTVERSION);
			instance=project+"_instance";
			if(ServerType.isJonas())
			{
				instance=((ProjectSessionLocal)mContext.getEJBLocalObject()).importInstance(pv, instance, EventConstants.DEFAULTPROJECTVERSION, new Hashtable());
			}
			else
			{
				/* JBoss does not support Stateful Session Bean reentrance
				 * even if the Thread is the same one. So we need to
				 * invoke the method directly. This *should* work as a
				 * transaction is already begun, as "Support" transaction
				 * attribute is switched to "Required" during deployment
				 * descriptor post processing.
				 */
				instance=this.importInstance(pv, instance, EventConstants.DEFAULTPROJECTVERSION, new Hashtable());
			}
			thelog.info("User: "+ this.getCaller() + " instantiates project: " + project + ". " +instance + " is created");
		}catch(HeroException traceEx){traceEx.printStackTrace();
			throw new CreateException(traceEx.getMessage());
		}
		return (instance);
	}

	/**
	 * Project instance. You have to call this method after "create" call. After that
	 * you have an instance of the project and all methods of ProjectSession API are available.
	 *
	 * @param project the name of the project to instance
	 * @param version the version of the project to instance
	 * @return String - the name of the project instance
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public String instantiateProject(String project, String version) throws CreateException {
		String instance=null;
		try{
			BonitaProjectValue pv = this.instantiateInit(project, version);
			instance=project+"_instance";
			if(ServerType.isJonas())
			{
				instance=((ProjectSessionLocal)mContext.getEJBLocalObject()).importInstance(pv, instance, version, new Hashtable());
			}
			else
			{
				/* JBoss does not support Stateful Session Bean reentrance
				 * even if the Thread is the same one. So we need to
				 * invoke the method directly. This *should* work as a
				 * transaction is already begun, as "Support" transaction
				 * attribute is switched to "Required" during deployment
				 * descriptor post processing.
				 */
				instance=this.importInstance(pv, instance, version, new Hashtable());
			}
			thelog.info("User: "+ this.getCaller() + " instantiates project: " + project +" version: "+version+ ". " +instance + " is created");
		}catch(HeroException traceEx){traceEx.printStackTrace();
			throw new CreateException(traceEx.getMessage());
		}
		return (instance);
	}


	private BonitaProjectValue instantiateInit(String project, String version) throws CreateException {
		Context lContext;
		String instance=null;
		UserServiceLocalHome ushome;
		BonitaProjectValue pv=null;
		try{
			trace.debug("parameter: project=" + project + " started by " + this.getCaller());
			try {
				pHome = hero.interfaces.BnProjectUtil.getLocalHome();
				ushome = hero.interfaces.UserServiceUtil.getLocalHome();
				UserServiceLocal ul = ushome.create();
				ul.findUserLocal(this.getCaller());
			} catch (Exception ce) {
				trace.error(ce.getMessage());
				throw new HeroException(ce.getMessage());
			}
			if (project.equals(""))
				throw new CreateException("Project name can not be empty !!");
			if (project.matches(".*_instance.*"))
				throw new CreateException("You cannot instantiate an instance !!");
			try {
				try {
					pHome = hero.interfaces.BnProjectUtil.getLocalHome();
					mProject = pHome.findByNameVersion(project, version);
				} catch (javax.naming.NamingException ne) {
					throw new BonitaServiceException(ne.getMessage());
				} catch (javax.ejb.FinderException fe) {throw new BonitaServiceException(fe.getMessage());}

				pv = BonitaProjectLocator.getInstance().getModelValue(project, version);

				if (!pv.getType().equals(Constants.Pj.MODEL))
					throw new BonitaServiceException("This process is not instantiable. Only model processes can be instantiated !!");

				if (pv.getStatus().equals(Constants.Pj.HIDDEN))
					throw new BonitaServiceException("This model is not ACTIVE. Activate the model before instantiating it !!");

			} catch (BonitaServiceException e) {
				throw new CreateException(e.getMessage());
			}
		}catch(HeroException traceEx){traceEx.printStackTrace();
			throw new CreateException(traceEx.getMessage());
		}
		return pv;
	}

	/**
	 * Creates the Project Session Bean. This method is the first one to invoke in order to
	 * use ProjectSession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException
	 *
	 * @ejb:create-method view-type="both"
	 **/

	public void ejbCreate() throws CreateException {
	}

	/**
	 * Internal Enterprise Java Beans method.
	 **/
	public void ejbPostCreate(String oldProject, String newProject)
	throws CreateException, RemoteException {
	}

	/**
	 * Internal Enterprise Java Beans method.
	 **/
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
		try{
			BonitaConfig bonitaConfig = BonitaConfig.getInstance();
			if (bonitaConfig.getJms())
			{
				JMSServicesSessionLocalHome home = JMSServicesSessionUtil.getLocalHome();
				this.jms=home.create();
			}

			// To use Ejb timers
			DeadlineEjbTimerSessionLocalHome dlHome;
			try {
				dlHome = DeadlineEjbTimerSessionUtil.getLocalHome();
			} catch (javax.naming.NamingException ne) {
				throw new HeroException(ne.getMessage());
			}
			try {
				this.dlSession = dlHome.create();
			} catch (CreateException he) {
				throw new HeroException("Execute: Error to create DeadlineEjbTimerSessionLocal");
			}

		}catch(Exception e){throw new EJBException(e.getMessage());}
	}

	/**
	 * Internal Enterprise Java Beans method.
	 **/

	public void ejbRemove() {
	}

	/**
	 * Internal Enterprise Java Beans method.
	 **/

	public void ejbActivate() {
	}

	/**
	 * Internal Enterprise Java Beans method.
	 **/

	public void ejbPassivate() {
	}

	// Utility methods


/*
	private void createProcess(String projectName)throws HeroException, CreateException
	{
		trace.info("User: "+ this.getCaller() + " creates workflow process: " + projectName);
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new CreateException(ne.getMessage());
		}
		// The project has to be created
		try {
			String creator = this.getCaller();
			mProject = pHome.create(projectName, creator);
			mProject.setParent(projectName);
			BonitaProjectValue pv = new BonitaProjectValue();
			pv.setName(projectName);

			BnUserLocal user = uhome.findByName(creator);
			mProject.getBnUsers().add(user);

			// role admin is added
			BnRoleValue rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.ADMIN);
			rv.setDescription("the admin role");
			BnRoleLocal role = rhome.create(rv);
			mProject.getBnRoles().add(role);

			// Set User Role
			user.getBnRoles().add(role);

			// role InitialRole is added
			rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.INITIALROLE);
			rv.setDescription("the initial role for nodes");
			role = rhome.create(rv);
			mProject.getBnRoles().add(role);

			if (!creator.equals(Constants.ADMIN))
			{
				// role username is added
				rv = new BnRoleValue();
				rv.setName(creator);
				rv.setDescription(creator+" user role");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				// Set User Role
				user.getBnRoles().add(role);
			}

			mProject.setCreationDate(new Date());

			if (jms!=null)
				jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
		} catch (FinderException fie) {

			trace.error(fie.getMessage());
			throw new HeroException(fie.getMessage());
		} catch (InvalidValueException ie) {

			trace.error(ie.getMessage());
			throw new HeroException(ie.getMessage());
		} catch (Exception he) {

			trace.error(he.getMessage());
			throw new HeroException(he.getMessage());
		}
	}
*/
	private void createProcess(String projectName, String projectVersion)throws HeroException, CreateException
	{
		trace.info("User: "+ this.getCaller() + " creates workflow process: name = " + projectName + " version = " + projectVersion);
		BnUserLocalHome uhome = null;
		BnRoleLocalHome rhome = null;
		try {
			pHome = hero.interfaces.BnProjectUtil.getLocalHome();
			uhome = hero.interfaces.BnUserUtil.getLocalHome();
			rhome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new CreateException(ne.getMessage());
		}
		// The project has to be created
		try {
			String creator = this.getCaller();
			mProject = pHome.create(projectName, projectVersion, creator);
			mProject.setParent(projectName);
			BonitaProjectValue pv = new BonitaProjectValue();
			pv.setName(projectName);

			BnUserLocal user = uhome.findByName(creator);
			mProject.getBnUsers().add(user);

			// role admin is added
			BnRoleValue rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.ADMIN);
			rv.setDescription("the admin role");
			BnRoleLocal role = rhome.create(rv);
			mProject.getBnRoles().add(role);

			// Set User Role
			user.getBnRoles().add(role);

			// role InitialRole is added
			rv = new BnRoleValue();
			rv.setName(hero.interfaces.Constants.INITIALROLE);
			rv.setDescription("the initial role for nodes");
			role = rhome.create(rv);
			mProject.getBnRoles().add(role);

			if (!creator.equals(Constants.ADMIN))
			{
				// role username is added
				rv = new BnRoleValue();
				rv.setName(creator);
				rv.setDescription(creator+" user role");
				role = rhome.create(rv);
				mProject.getBnRoles().add(role);
				// Set User Role
				user.getBnRoles().add(role);
			}

			mProject.setCreationDate(new Date());

			if (jms!=null)
				jms.sendProjectEvent(ADDUSERPROJECT, projectName, creator);
		} catch (FinderException fie) {

			trace.error(fie.getMessage());
			throw new HeroException(fie.getMessage());
		} catch (InvalidValueException ie) {

			trace.error(ie.getMessage());
			throw new HeroException(ie.getMessage());
		} catch (Exception he) {

			trace.error(he.getMessage());
			throw new HeroException(he.getMessage());
		}
	}


	private boolean isInProject(String projectName, String projectVersion) throws HeroException{
		if (mContext.getCallerPrincipal().getName().equals(Constants.SECURITY_ROLE))
			return true;
		BnProjectLocalHome phome = null;
		BnProjectLocal plocal = null;
		try {
			phome = hero.interfaces.BnProjectUtil.getLocalHome();
			plocal = phome.findByNameVersion(projectName, projectVersion);
			Collection users = plocal.getBnUsers();
			Iterator usrs = users.iterator();
			while (usrs.hasNext())
			{
				if (((BnUserLocal)usrs.next()).getName().equals(this.getCaller()))
					return true;
			}
			return false;

		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (FinderException fie) {return true;} // The project does not exist
	}


	/**
	 * Verifies whether the user is admin of this project/instance
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public boolean isAdminOfProject() throws HeroException{
		if (mContext.getCallerPrincipal().getName().equals(Constants.SECURITY_ROLE))
			return true;
		Collection projects = this.getUserRolesInProjectNames(this.getCaller());
		Iterator i = projects.iterator();
		while (i.hasNext()) {
			String roleName = (String) i.next();
			if (roleName.equals(Constants.ADMIN))
				return true;
		}
		return false;
	}

	private boolean isProjectHidden() throws HeroException{
		if (mProject.getStatus().equals(Constants.Pj.HIDDEN))
			return true;
		else
			return false;
	}

	/**
	 * Returns whether the user belongs to the activity role
	 *
	 * @param nodeName the name of the node
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws CreateException
	 **/
	public boolean isUserInNodeRole(String nodeName) throws HeroException{
		BnNodeLocalHome nhome = null;
		BnNodeLocal nlocal = null;
		try {
			nhome = hero.interfaces.BnNodeUtil.getLocalHome();
			nlocal = nhome.findByName(nodeName,mProject.getId());
			if (nlocal.getActivityPerformer() !=null)
				return (this.getCaller().equals(nlocal.getActivityPerformer()));
			else
			{
			Collection users = nlocal.getBnRole().getBnUsers();

			Iterator i = users.iterator();
			while (i.hasNext()) {
				BnUserLocal user = (BnUserLocal) i.next();
				String userName = user.getName();
				if (userName.equals(this.getCaller()))
					return true;
			}
			}
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		} catch (FinderException fie) {
			throw new HeroException(fie.getMessage());
		}
		return false;
	}

	private boolean checkDynamics(String out) throws HeroException {
		BnNodeLocal nodeout = mProject.getBnNode(out);
		if ((nodeout.getState() == hero.interfaces.Constants.Nd.INITIAL)
				|| (nodeout.getState() == hero.interfaces.Constants.Nd.READY)
				|| (nodeout.getState() == hero.interfaces.Constants.Nd.ANTICIPABLE))
			return true;
		return false;
	}

	// Execute RoleMappers for this instance. This method obtain the users list matching with a
	// specific role and add them to the instance. To improuve performance, this method uses the
	// adduserlocal and setUserRoleLocal methods below.
	private void mapRole() throws  HeroException {
		try {
			String creator = this.getCaller();
			Collection mappers = new ArrayList();
			Collection roles = mProject.getBnRoles();
			BnRoleMapperLocal mapper = null;
			Iterator it  = roles.iterator();
			it  = roles.iterator();
			while (it.hasNext()){
				BnRoleLocal role = (BnRoleLocal) it.next();
				mapper = role.getBnRoleMapper();
				if (mapper != null){
					hero.mapper.Mapper h = hero.mapper.Mapper.make(mapper.getName(), mapper.getType());
					Collection users = h.execute(this, mapper.getType(), role, creator);
					if (mapper.getType()==(Constants.Mapper.PROPERTIES))
						this.setUserRoleLocal(creator,role.getName());
					else
					{
						Iterator i = users.iterator();
						while (i.hasNext())
						{
							String userName = (String)i.next();
							this.addUserLocal(userName);
							this.setUserRoleLocal(userName,role.getName());
						}
					}
				}
			}
		} catch (HeroException he) {
			throw new HeroException(he.getMessage());
		}
	}

	private void addUserLocal(String userName) throws HeroException
	{
		BnUserLocalHome uHome;
		BnRoleLocalHome rHome;
		BnUserLocal mUser;
		UserServiceLocalHome ushome;
		try {
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			rHome = hero.interfaces.BnRoleUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {
			throw new HeroException(be.getMessage());
		}
		try {
			ushome = hero.interfaces.UserServiceUtil.getLocalHome();
			UserServiceLocal ul = ushome.create();
			mUser = ul.findUserLocal(userName);
		} catch (Exception ce) {
			throw new HeroException(ce.getMessage());
		}
		Collection cUsers = mProject.getBnUsers();
		if (!cUsers.contains(mUser))
		{
			cUsers.add(mUser);
			if (!userName.equals(Constants.ADMIN))
				this.addRole(userName, "BnUser BnRole");
			this.setUserRoleLocal(userName, userName);
		}
	}

	private void setUserRoleLocal(String userName, String roleName) throws HeroException
	{
		BnRoleLocalHome roleh;
		BnUserLocalHome userh;
		try {
			roleh = BnRoleUtil.getLocalHome();
			userh = BnUserUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}
		try {
			BnUserLocal user = userh.findByName(userName);
			BnRoleLocal role = roleh.findByName(roleName, mProject.getId());
			user.getBnRoles().add(role);
		} catch (Exception fe) {
			throw new HeroException(fe.getMessage());
		}

	}

	private void performAssign() throws  HeroException {
		// logs trace

		trace.debug("perform assign ");
		try {
			String creator = this.getCaller();
			Collection performers = new ArrayList();
			Collection nodes = mProject.getBnNodes();
			BnNodePerformerAssignLocal performerAssign = null;
			Iterator it  = nodes.iterator();

			it  = nodes.iterator();
			while (it.hasNext()){
				BnNodeLocal node = (BnNodeLocal) it.next();
				performerAssign = node.getBnNodePerformerAssign();
				if (performerAssign != null){
					hero.performerAssign.PerformerAssign h = hero.performerAssign.PerformerAssign.make(performerAssign.getName(), performerAssign.getType());
					h.execute(this, performerAssign.getType(), node, creator);
				}
			}
		} catch (HeroException he) {
			throw new HeroException(he.getMessage());
		}

	}

	/**
	 * Checks recursively if toIterationNode is reachable.
	 *
	 * @param visited
	 * @param currentNode
	 * @param toIterationNode
	 * @return 'true' if exists a path from fromIterationNode to toIterationNode. Otherwise it returns 'false'
	 * @throws HeroException
	 */
	private boolean checkStructuralConstraints(ArrayList visited, String currentNode, String toIterationNode) throws HeroException {
		if (currentNode.equals(toIterationNode)) {
			return true;
		}

		if (!visited.contains(currentNode)) {
			visited.add(currentNode);
		}

		BnNodeLocal currentBnNode = mProject.getBnNode(currentNode);
		Collection inEdges = currentBnNode.getInBnEdges();
		Iterator it = inEdges.iterator();
		while (it.hasNext()) {
			BnEdgeLocal nextEdge = (BnEdgeLocal) it.next();
			BnNodeLocal nextNode = nextEdge.getInBnNode();
			String nextNodeName  = nextNode.getName();
			if ((!visited.contains(nextNodeName)) && (checkStructuralConstraints(visited, nextNodeName, toIterationNode))) {
				return true;
			}
		}
		return false;
	}

	private boolean checkDuplicateEdge(String in, String out)
	throws HeroException {
		Collection edges = mProject.getBnEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnEdgeLocal edge = (BnEdgeLocal) i.next();
			if ((edge.getInBnNode().getName()).equals(in)
					&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}

	private boolean checkDuplicateAgentEdge(String in, String out)
	throws HeroException {
		Collection edges = mProject.getBnAgentEdges();
		Iterator i = edges.iterator();
		while (i.hasNext()) {
			BnAgentEdgeLocal edge = (BnAgentEdgeLocal) i.next();
			if ((edge.getInBnAgent().getName()).equals(in)
					&& (edge.getOutBnNode().getName()).equals(out))
				return false;
		}
		return true;
	}


	/**
	 * Copy node. Internal method used in processes instantiation.
	 *
	 * @param name of the new node
	 * @param node the BnNodeLocal object of the copied node.
	 * @param inNode the BnNodeLocal in node object of the new node.
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void copyNode(String nodeName, BonitaNodeValue node) throws HeroException {

		trace.debug("copyNode:  nodeName=" + nodeName + " node=" + node.getName()+ " inNode=");
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;
		BnUserLocalHome uHome;
		BnNodePerformerAssignLocal ppLocal;
		BnNodePerformerAssignLocalHome performerAssignh;

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
			uHome = hero.interfaces.BnUserUtil.getLocalHome();
			performerAssignh = BnNodePerformerAssignUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			if (nodeName.equals(""))
				throw new HeroException("Node name can not be empty...");

			//  Creates node and set the role
			BnNodeLocal nd = nHome.create(nodeName, node.getType());
			BnNodeValue ndata = nd.getBnNodeValue();
			ndata.setCreationDate(new java.util.Date());
			nd.setBnNodeValue(ndata);
			Collection cNodes = mProject.getBnNodes();
			cNodes.add(nd);

			nd.setCreator(node.getCreator());
			nd.setExecutor(node.getExecutor()); // Only for JOnAS consistency

			BnRoleLocalHome rHome = BnRoleUtil.getLocalHome();
			BnRoleLocal role = rHome.findByName(node.getRole(),mProject.getId());
			nd.setBnRole(role);

			if (!node.getAnticipable()) // Anticipable/traditional mode
			{
				nd.setAnticipable(false);
			}
			nd.setDescription(node.getDescription()); // Node description

			//For Deadlines
			if (node.getDeadlines().size()!=0)
				nd.setDeadlines(node.getDeadlines());
			//For relative Deadlines
			if (node.getRelativeDeadlines().size()!=0)
				nd.setRelativeDeadlines(node.getRelativeDeadlines());

			if (node.getPerformer() != null) // Node Performer assignment
			{
				ppLocal = performerAssignh.create(node.getPerformer().getName(),
						node.getPerformer().getType(),
						node.getPerformer().getPropertyName());
				ppLocal.setBnNode(nd);
			}
			// Tell the world
			if (jms!=null)
				jms.sendNodeEvent(ADDNODE,this.getName(),nodeName,nd.getType(),nd.getState(),this.getCaller());

		} catch (InvalidValueException ie) {

			trace.error(ie.getMessage());
			throw new HeroException(ie.getMessage());
		} catch (CreateException ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		} catch (javax.naming.NamingException ne) {

			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		}
	}

	/**
	 * Copy node edge. Internal method used in processes instantiation.
	 *
	 * @param name of the new node
	 * @param inNode the BnNodeLocal in node object of the new node.
	 * @param edge the BnEdgeLocal edge object from the project model.
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Required"
	 *
	 **/
	public void copyEdge(String nodeName, BnNodeLocal inNode, BonitaEdgeValue edge ) throws HeroException {

		trace.debug("copyEdge:  nodeName=" + nodeName + " inNode=" + inNode.getName()+" edge=" + edge.getName());
		BnEdgeLocalHome eHome;
		BnEdgeLocal mEdge;
		BnNodeLocalHome nHome;
		BnNodeLocal mNode;

		try {
			nHome = hero.interfaces.BnNodeUtil.getLocalHome();
			eHome = hero.interfaces.BnEdgeUtil.getLocalHome();
		} catch (javax.naming.NamingException be) {

			trace.error(be.getMessage());
			throw new HeroException(be.getMessage());
		}
		try {
			BnNodeLocal nd = nHome.findByName(nodeName, mProject.getId());

			try{
				mEdge = eHome.create();
				mEdge.setBnProject(mProject);
				mEdge.setInBnNode(inNode);
				mEdge.setOutBnNode(nd);
				mEdge.setCondition(edge.getCondition());

				int inState = Constants.Nd.READY;
				if (inNode.getState() == Constants.Nd.ANTICIPATING)
					inState = Constants.Nd.ANTICIPABLE;

				int edgeState = hero.interfaces.Constants.edgeTransition[Constants.CONDITION_TRUE][inState];
				this.setEdgeState(mEdge, edgeState);
				//	mEdge.setState(edgeState);

				EdgeState edgeStateClass = EdgeState.make(nd.getType());
				int newOperation = edgeStateClass.state(nd);

				// It doesn't consider CANCEL operation. CANCEL propagation is done by the EngineBean
				if (newOperation != Constants.Nd.CANCEL) {
					NodeState nodeState = NodeState.make(nd.getType(), nd.getAnticipable());
					int newState = nodeState.computeState(nd, newOperation);

					if (newState == hero.interfaces.Constants.Nd.BAD_TRANSITION) {
						throw new HeroException("AddEdge generated a bad Node transition");
					}
					this.setNodeState(nd, newState);
				}

				if (jms!=null)
					jms.sendEdgeEvent(ADDEDGE,this.getName(),mEdge.getName(),inNode.getName(),nd.getName(),this.getCaller());
			}catch (hero.util.NodeStartedException nse) {} // Instances execution consistency
		} catch (InvalidValueException ie) {

			trace.error(ie.getMessage());
			throw new HeroException(ie.getMessage());
		} catch (CreateException ce) {

			trace.error(ce.getMessage());
			throw new HeroException(ce.getMessage());
		} catch (FinderException fe) {

			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	}

	/**
	 * Test if the project contains this user
	 * @param userName The name of the user to associate to the project
	 * @return boolean - true is the user belongs to the project
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public boolean containsUser(String userName) throws HeroException {

		trace.debug("containsUser : name=" + userName);
		BnUserLightValue[] uv = mProject.getBnProjectValue().getBnUsers();
		for (int i = 0; i < uv.length; i++) {
			if (uv[i].getName().equalsIgnoreCase(userName))
				return true;
		}
		return false;
	}

	private boolean containsRole(String roleName) {
		BnRoleValue[] rv = mProject.getBnProjectValue().getBnRoles();
		for (int i = 0; i < rv.length; i++) {
			if (rv[i].getName().equalsIgnoreCase(roleName))
				return true;
		}
		return false;
	}

	private Collection getNodeInEdges(BnNodeLocal nd ){
		Collection edgeValues = new ArrayList();
		Collection edges = nd.getInBnEdges();
		Iterator nodeEdges = edges.iterator();
		while (nodeEdges.hasNext())
			edgeValues.add(((BnEdgeLocal)nodeEdges.next()).getBnEdgeValue());
		return edgeValues;

	}

	private void addIterationInstance(String from, String to, String condition) throws HeroException {
		BnIterationLocalHome iteh;
		try {
			iteh = BnIterationUtil.getLocalHome();
		} catch (javax.naming.NamingException ne) {
			throw new HeroException(ne.getMessage());
		}

		BnIterationLightValue iv = new BnIterationLightValue();
		iv.setFromNode(from);
		iv.setToNode(to);
		iv.setCondition(condition);
		try {
			BnIterationLocal il = iteh.create(iv);
			mProject.getBnIterations().add(il);
			if (jms!=null)
				jms.sendIterationEvent(ADDITERATION,this.getName(), from, to);
		} catch (Exception ce) {
			throw new HeroException(ce.getMessage());
		}
	}

	private String getModel(String instanceName) {
		int i = instanceName.indexOf("_instance");
		return (instanceName.substring(0, i));
	}

	/**
	 * Local method
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public String getCaller() {
		/*
		 * Previously, mContext.getCallerPrincipal() was invoked from
		 * ejbCreate(). This was not working in some application servers
		 * such as JBoss. For this reason, the caller field is set in this
		 * method on first call.
		 */
		if(this.caller == null) {
			this.setCaller(mContext.getCallerPrincipal().getName());
		}

		return this.caller;
	}


	/**
	 * Local method
	 * @ejb:interface-method view-type="local"
	 * @ejb:transaction type="Supports"
	 * @throws HeroException
	 *
	 **/
	public void setCaller(String user) {
		this.caller=user;
	}

	/**
	 * Checks: <br>
	 * 		- 1st: checks if iterations conditions are not empty <br>
	 * 		- 2nd: if we have more than one iteration starting in the same node -> checks that the iterations conditions are different <br>
	 *
	 * @throws HeroException
	 */
	private void checkIterationConstrains() throws HeroException {
		Collection projectIterations = mProject.getBnIterations();
		Iterator iterationsIt = projectIterations.iterator();
		while (iterationsIt.hasNext()) {
			BnIterationLocal iteration = (BnIterationLocal) iterationsIt.next();
			BnNodeLocal fromNodeLocal = mProject.getBnNode(iteration.getFromNode());
			BnNodeLocal toNodeLocal = mProject.getBnNode(iteration.getToNode());
			String iterationCondition = iteration.getCondition();

			// 1st: checks if iterations conditions are not empty
			if (iterationCondition.equals("")) {
				throw new HeroException("ERROR in checkIterationConstrains() : Condition in iteration " + fromNodeLocal.getName() + "--->" + toNodeLocal.getName() + " cannot be \"\" (nothing)");
			}

			// 2nd: checks if exists a path between from--->to
			ArrayList visited = new ArrayList();
			if (!this.checkStructuralConstraints(visited, fromNodeLocal.getName(), toNodeLocal.getName())) {
				throw new HeroException("ERROR in checkIterationConstrains() : There isn't any path between " + toNodeLocal.getName() + "--->" + fromNodeLocal.getName());
			}

			// 3rd: if we have more than one iteration starting in the same node -> checks that the iterations conditions are different
			Iterator anotherIterationsIt = projectIterations.iterator();
			while (anotherIterationsIt.hasNext()) {
				BnIterationLocal iterationToCompare = (BnIterationLocal) anotherIterationsIt.next();
				if ( !(iterationToCompare.isIdentical(iteration))									// If they are different iterations
						&& (iterationToCompare.getFromNode().equals(iteration.getFromNode())) 		// If they start in the same node
						&& (iterationToCompare.getCondition().equals(iterationCondition)) ) {	// and they have the same condition
					throw new HeroException("ERROR in checkIterationConstrains() : Two iterations starting in the same node (" + fromNodeLocal.getName() + "--->" + toNodeLocal.getName() + " & " + iterationToCompare.getFromNode() + "--->" + iterationToCompare.getToNode() + ") cannot have the same condition (" + iterationCondition + ")");
				}
			}
		}
	}

	/**
	 * Gets in a recursive way the nodes inside the iteration.<br>
	 * Extra entry points are no returned (because they are not inside the iteration)
	 *
	 * @param nodesInIteration initially is an empty list and at the end contains the nodes inside the iteration
	 * @param currentNode
	 * @param toIterationNode
	 * @return true if current node is inside the iteration and false otherwise
	 * @throws HeroException
	 */
	private boolean getNodesInsideIteration(ArrayList nodesInIteration, BnNodeLocal currentNode, BnNodeLocal toIterationNode) throws HeroException
	{
		boolean pathExists = false;

		if (currentNode.equals(toIterationNode)) {
			nodesInIteration.add(currentNode);
			return true;
		} else {
			Collection inEdges = currentNode.getInBnEdges();
			Iterator it = inEdges.iterator();
			while (it.hasNext()) {
				BnEdgeLocal nextEdge = (BnEdgeLocal) it.next();
				BnNodeLocal nextNode = nextEdge.getInBnNode();
				if (nodesInIteration.contains(nextNode)) {
					pathExists = true;
				} else {
					if (getNodesInsideIteration(nodesInIteration, nextNode, toIterationNode)) {
						pathExists = true;
					}
				}
			}
		}
		if (pathExists) {
			if (!nodesInIteration.contains(currentNode)) {
				nodesInIteration.add(currentNode);
			}
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Checks: <br>
	 *		- 1st: checks mandatory conditions over the out edges of the nodes that create and extra exit point from the iteration <br>
	 * 		- 2nd: checks mandatory conditions over the out edges from the node that starts the iteration are different from the iteration's condition <br>
	 * @throws HeroException
	 **/
	private void checkMandatoryIterationConditions() throws HeroException {
		Collection projectIterations = mProject.getBnIterations();
		Iterator iterationsIt = projectIterations.iterator();
		while (iterationsIt.hasNext()) {
			BnIterationLocal iteration = (BnIterationLocal) iterationsIt.next();
			ArrayList nodeList = new ArrayList();
			BnNodeLocal fromNodeLocal = mProject.getBnNode(iteration.getFromNode());
			BnNodeLocal toNodeLocal = mProject.getBnNode(iteration.getToNode());
			String iterationCondition = iteration.getCondition();

			// 1st: checks mandatory conditions over the out edges of the nodes that create and extra exit point from the iteration
			this.getNodesInsideIteration(nodeList, fromNodeLocal, toNodeLocal);

			// check if some out egde of the nodeList point to node that is not in the list (it's an extra exit point)
			Iterator nodeListIterator = nodeList.iterator();
			while (nodeListIterator.hasNext()) {
				BnNodeLocal node = (BnNodeLocal) nodeListIterator.next();
				Collection outEdges = node.getOutBnEdges();
				Iterator outEdgesIt = outEdges.iterator();
				while (outEdgesIt.hasNext()) {
					BnEdgeLocal edge = (BnEdgeLocal) outEdgesIt.next();
					BnNodeLocal outNode = edge.getOutBnNode();
					if (!nodeList.contains(outNode)) { // outNode is outside iteration
						// All out edges of the "inNode" has to have a condition
						Collection outNodeEdges = edge.getInBnNode().getOutBnEdges();
						Iterator outNodeEdgesIt = outNodeEdges.iterator();
						while (outNodeEdgesIt.hasNext()) {
							BnEdgeLocal nextEdge = (BnEdgeLocal) outNodeEdgesIt.next();
							String edgeTmpCondition = nextEdge.getCondition();
							if (edgeTmpCondition == null) {
								throw new HeroException("ERROR in checkMandatoryIterationConditions() : It has to be a condition in transition " + nextEdge.getInBnNode().getName() + "--->" + nextEdge.getOutBnNode().getName());
							} else if (edgeTmpCondition.equals("")) {
								throw new HeroException("ERROR in checkMandatoryIterationConditions() : Condition in transition " + nextEdge.getInBnNode().getName() + "--->" + nextEdge.getOutBnNode().getName() + " cannot be \"\" (nothing)");
							} else if (edgeTmpCondition.equals("true")) {
								throw new HeroException("ERROR in checkMandatoryIterationConditions() : Condition in transition " + nextEdge.getInBnNode().getName() + "--->" + nextEdge.getOutBnNode().getName() + " cannot be \"true\"");
							}
						}
					}
				}
			} // End 1st checking

			// 2nd: checks mandatory conditions over the out edges from the node that starts the iteration are different from the iteration's condition
			Collection outEdges = fromNodeLocal.getOutBnEdges();
			Iterator outEdgesIt = outEdges.iterator();
			while (outEdgesIt.hasNext()) {
				BnEdgeLocal nextEdge = (BnEdgeLocal) outEdgesIt.next();
				String edgeCondition = nextEdge.getCondition();
				if (edgeCondition.equals(iterationCondition)) {
					throw new HeroException("ERROR in checkMandatoryIterationConditions() : Condition in transition " + nextEdge.getInBnNode().getName() + "--->" + nextEdge.getOutBnNode().getName() + " cannot be equal to iteration condition");
				}
			} // End 2nd checking
		}
	}

	/**
	 * Checks if the project model is correctly defined.
	 *
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Required"
	 * @throws HeroException
	 **/
	public void checkModelDefinition() throws HeroException {
		this.checkIterationConstrains();
		this.checkMandatoryIterationConditions();
	}

}
