/*
 * Created on 3 mai 2004
 *
 */
package mc.formgenerator.bonita;

import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;
import hero.interfaces.UserSession;
import hero.interfaces.UserSessionHome;
import hero.interfaces.UserSessionUtil;

import java.util.*;

import org.w3c.dom.Document;

import hero.util.HeroException;

/**
 * Given instance document with properties, starts and ends an activity
 */
public class BonitaActivityExecutor {



	/**
	 * Default constructor
	 */
	public BonitaActivityExecutor() {
		
	}
	
	
	
	/**
	 * Allows to find, initialise and terminate an activity.
	 * @param document Document with the activity information (name, attributes and project name).
	 * @param mode the application mode: consumer or exploitant
	 * @throws LoginException
	 */
	public void actOnActivity(Document document, Vector req_parameters, boolean formsExists, String mode, String version) throws HeroException {

		//'parse' transforms a Document
		DocumentParser parse = new DocumentParser();

		DataActivity data = parse.parseActivity(document, req_parameters, formsExists);


		//get the project name to wich it belongs
		String projectName = data.getProjectName();


		//get the activity name	
		String activityName = data.getActivityName();


		//get the process properties
		HashMap projectTable = data.getProjectProperties();
		
		
		//get activity properties
		HashMap activityTable = data.getActivityProperties();
		
		
		//terminate the activity thanks to its name and properties
		this.startActivity(projectName, activityName, projectTable, activityTable, mode, version);		
	}
	
	
	
	
	
	/**
	 * Thanks to its name, the activity is found and stopped. 
	 * Before, its attributes are set with the HashMap values.
	 * @param activityName Name of the activity to stop.
	 * @param projectName Name of the project to which it belongs.
	 * @param table HashMap with the activity attributes associated to their values
	 * @param mode the application mode: consumer or exploitant
	 * @throws LoginException
	 */
	private void startActivity(String projectName, String activityName, HashMap projectTable, HashMap activityTable, String mode, String version)
	throws HeroException {

		//The local home
		ProjectSessionHome projecth = null;

		//The bean
		ProjectSession project = null;
		
		//All the keys will be saved in a set
		Set collec;

		try {
			//Getting local home by using JNDI
			projecth = ProjectSessionUtil.getHome();

			//Creation of the project
			project = projecth.create();
				
			//project.initProjectWithVersion(projectName,version);
			project.initProject(projectName);
			
			//**************************************************************************
			//					SET THE PROJECT PROPERTIES
			//**************************************************************************

			collec = projectTable.keySet();

			//Set properties if they exist
			if (collec.size() > 0) {

				//For all the keys we have to retrieve the associated value
				Iterator it = collec.iterator();

				//variable which will stock the property name
				String cle="";
				
				while (it.hasNext()) {

					//Getting the values
					cle = (String) it.next();
					//Set the current process property
					project.setProperty(cle, (String) projectTable.get(cle));
				}
			}
			

			//**************************************************************************
			//					SET ACTIVITY PROPERTIES
			//**************************************************************************

			collec = activityTable.keySet();

			//Set properties if they exist
			if (collec.size() > 0) {

				//For all the keys we have to retrieve the associated value
				Iterator it = collec.iterator();

				//variable which will stock the property name
				String cle="";

				while (it.hasNext()) {

					//Getting the values
					cle = (String) it.next();
					//Set the current process property
					project.setNodeProperty(activityName, cle, (String) activityTable.get(cle));
				}
			}
			
			//**************************************************************************
			//					START THE ACTIVITY
			//**************************************************************************
			
			//The local home
			UserSessionHome userh = UserSessionUtil.getHome();
			//The bean
			UserSession user = userh.create();		
			//the session starts the activity
			user.startActivity(projectName, activityName);
			//the session terminate the activity
			user.terminateActivity(projectName, activityName);

			//**************************************************************************
			//					STOP THE ACTIVITY
			//**************************************************************************	
			
			//if(mode.equals("consumer"))
			    //this.endActivity(activityName, projectName);
			project.remove();
		} catch (Exception e) {
			try{
				if (project != null)
					project.remove();
			}catch(Exception re){
				System.out.println(re + " " + e.getMessage());
			}
			//System.out.println(e.getMessage());
			throw new HeroException (e.getMessage());
		}
	}

	
	


	public void endActivity (String activityName, String projectName) {
		try {
			
			//The local home
			UserSessionHome userh = UserSessionUtil.getHome();
		
			//The bean
			UserSession user = userh.create();
			
			
			//the session terminates the activity
			user.terminateActivity( projectName, activityName);
			
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
}
