/*
 * Created on 23 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.servlets;

import mc.formgenerator.servlets.bonita.RepositoryFacade;
import mc.formgenerator.servlets.util.*;

import hero.user.ReadEnv;
import hero.util.HeroException;

import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import mc.formgenerator.bonita.BonitaActivityAdapter;
import mc.formgenerator.bonita.BonitaActivityExecutor;
import mc.formgenerator.bonita.BonitaProjectAdapter;
import mc.formgenerator.bonita.BonitaProjectExecutor;
import mc.formgenerator.bonita.DocumentParser;

import org.apache.log4j.Category;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.xml.serialize.DOMSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.chiba.xml.xforms.Model;
import org.chiba.xml.xforms.config.Config;
import org.chiba.xml.xforms.exception.XFormsException;
import org.chiba.adapter.ChibaAdapter;


/**
 * Servlet that displays forms acording to projects/activities properties
 * and start/stop projects/activities execution. This servlet is a modification
 * of ChibaServlet.
 */
public class ServletBonitaForm extends HttpServlet {

	//ServletAdapter instance
	//private ServletAdapter servletAdapter = null;

	//Allow us to know if we are working with an activity or a project
	//private Boolean isActivity;

	//Name of the project we are working with
	//private String projectName = null;

	//Name of the activity we are working with
	//private String activityName = null;

	//Application mode consumer or exploitant
	//private String mode = null;

	//forward URL
	//private String forward = null;

    //init-params
    private static Category cat = Category.getInstance(ServletBonitaForm.class);
    //private static final String FORM_PARAM_NAME = "form";
    private static final String XSL_PARAM_NAME = "xslt";
    private static final String CSS_PARAM_NAME = "css";
    private static final String ACTIONURL_PARAM_NAME = "action_url";

    /*
     * It is not thread safe to modify these variables once the
     * init(ServletConfig) method has been called
     */
    // the absolute path to the Chiba config-file
    private String configPath = null;

    // the rootdir of this app; forms + documents fill be searched under this root
    private String contextRoot = null;

    // where uploaded files are stored
    private String uploadDir = null;

    private String stylesPath = null;

    /**
     * Returns a short description of the servlet.
     *
     * @return - Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Servlet Controller for Chiba Bonita's XForms Processor";
    }

    /**
     * Destroys the servlet.
     */
    public void destroy() {
    }


	/**
	 * Start the server side interpretation of the xforms document
	 * @param document the xforms document we have to interprete
	 * @param writer the servlet's writer
	 */
	private ServletAdapter xformProcessing(Document instance, PrintWriter writer, HttpServletRequest request,
			HttpServletResponse response, String actionURL, HttpSession session,
			ServletAdapter servletAdapter, Boolean isActivity, String projectName, String activityName,
			String projectVersion)
		throws 	TransformerConfigurationException, XFormsException,
		FileNotFoundException, TransformerException, IOException,
		URISyntaxException, SAXException,
		ParserConfigurationException, Exception {

		//hostUrl needed by ui.xsl
		String hostUrl = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort();

		//xmlBase needed by Chiba
		String xmlBase = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort()+ request.getContextPath() + "/";
		Document xformDocument = null;
		Document langInstance = null;

        String xslFile = request.getParameter(XSL_PARAM_NAME);
        String css = request.getParameter(CSS_PARAM_NAME);
		if(css == null){
			css = "web/styles/admin.css";
		}

		String realProjectName = projectName.replaceFirst("_instance.+","");

		String formPath = null;

		//instantiate RepositoryFacade to get form path (if exist)
		RepositoryFacade repositoryFacade = new RepositoryFacade(getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator+ "repository.xml");

		if (repositoryFacade == null){
			//Exception
			throw new IOException("Invalid session - Cannot retrieve repositoryFacade");
		}

		if(isActivity.booleanValue() == true){
			formPath = repositoryFacade.getActivityFormPath(realProjectName, activityName, projectVersion);
		}else{
			formPath = repositoryFacade.getProjectFormPath(realProjectName, projectVersion);
		}
		if (formPath!=null){
			// ****************************************************
			//xform file exist : use the xform to generate the html
			// ****************************************************

			//get file and parse it to xmlDocument
			String realFormPath = getServletConfig().getServletContext().getRealPath("") + formPath;
			xformDocument = getDocument(realFormPath);
			//set xmlbase
			Element xformRoot = xformDocument.getDocumentElement();
			xformRoot.setAttribute("xml:base",xmlBase);

			//lookup languages and create language instance
			String languagePath = null;
			// *** Activity ***
			if(isActivity.booleanValue() == true){
				//We are working with an activity
				languagePath = getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator + activityName + File.separator;
			// *** Project Instance ***
			}else{
				//We are working with a project
				languagePath = getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + projectVersion + File.separator;
			}

			String languages = request.getHeader("Accept-Language");
			String userLanguage = "en";
			if((languages != null) && (!languages.equals(""))){
				userLanguage = getUserLanguage(languages,languagePath);
			}

			langInstance = getDocument(languagePath + userLanguage + ".xml");

			//xformDocument = getDocument(realFormPath);

			//this.servletAdapter = setupServletAdapter(actionURL, session, formPath, xslFile, css);
			servletAdapter = setupServletAdapter(actionURL, hostUrl, session, xformDocument, realFormPath, xslFile, css);

		}else{
			//xform file does not exist. Generate xform document
			xformDocument = createXform(instance,xmlBase,projectName,activityName,projectVersion);
			servletAdapter = setupServletAdapter(actionURL, hostUrl, session, xformDocument, null, xslFile, css);
		}

		//Chiba launching
		//Build HTML page according to an activity or a project we are working with
		try{
			//System.out.println("INSTANCE: " + toString(instance));
			//System.out.println("XFORM DOCUMENT: " + toString(xformDocument));
			//if (langInstance!=null) System.out.println("LANG INSTANCE: " + toString(langInstance));
			//create instance for XForm
			if (formPath!=null){
				servletAdapter = updateContext(request, session, instance, langInstance, servletAdapter);
			}else{
				servletAdapter = updateContext(request, session, instance, servletAdapter);
			}

	        //add all request params that are not used by this servlet to the context map in ChibaBean
			servletAdapter = storeContextParams(request,servletAdapter);
	        servletAdapter.init();
	        //this.servletAdapter.executeHandler();
			servletAdapter.dispatch(null);
	        servletAdapter.buildUI(writer);
		} catch (Exception e) {
			//Error
			if (request.getParameter("forward")!=null) {
				String saveTime = saveErrors(e);
				response.sendRedirect(request.getParameter("forward")+"&processError=saved&processName="+projectName+"&saveTime="+saveTime);
			}
			else e.printStackTrace();
		}
		return (servletAdapter);
	}

	/**
	 * Action on doGet
	 * @param writer Response's writer
	 * @param request Request access
	 * @param response servlet response
	 * @throws TransformerConfigurationException
	 * @throws FileNotFoundException
	 * @throws XFormsException
	 * @throws TransformerException
	 * @throws IOException
	 */
	private void writingUI(PrintWriter writer, HttpServletRequest request,
			HttpServletResponse response, String actionURL, HttpSession session)
		throws 	LoginException, TransformerConfigurationException,
				FileNotFoundException, TransformerException,
				IOException, Exception{

		  //ServletAdapter instance
		  ServletAdapter servletAdapter = null;

		  //Allow us to know if we are working with an activity or a project
		  Boolean isActivity = null;

		  //Name of the project we are working with
		  String projectName = request.getParameter("projectName");

		  //Name of the activity we are working with
		  String activityName = request.getParameter("activityName");
		  
		  //The project version we are working with
		  String projectVersion = request.getParameter("version");

		  //Application mode consumer or exploitant
		  String mode = request.getParameter("mode");

		  //Document for server side interpretation
		  Document instance = null;

		  //Start or stop the activity
		  String action = request.getParameter("action");

		  //Project name parameter is present
		  if(projectName != null){

			//We are working with an activity
			if(activityName != null){

			    if (action != null){

			        if(action.equals("stop")){

						//Creation of executor
						BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor();

						//Terminate activity
						bonitaActivityExecutor.endActivity(activityName, projectName);

				        //Redirection to work list
						response.sendRedirect("/formgenerator/ServletWorkList");

						return;
			        }
				    else{
						isActivity = new Boolean(true);

						//Creation of adapter
						BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter();

						//Getting process informations via the adapter
						instance = bonitaActivityAdapter.getActivityData(projectName, activityName, projectVersion);
				    }
			    }
			    else{
					isActivity = new Boolean(true);

					//Creation of adapter
					BonitaActivityAdapter bonitaActivityAdapter = new BonitaActivityAdapter();

					//Getting process informations via the adapter
					instance = bonitaActivityAdapter.getActivityData(projectName, activityName, projectVersion);
			    }
			}
			else{

				//We are working with a project
				isActivity = new Boolean(false);

				//Creation of adapter
				BonitaProjectAdapter bonitaProjectAdapter = new BonitaProjectAdapter();
				//Getting process informations via the adapter
				instance = bonitaProjectAdapter.getProjectData(projectName, projectVersion);
			}
			DocumentParser parser = new DocumentParser();
			//If the document has properties

			if(parser.hasProperties(instance,isActivity)){
				//Finally starting of interpretation
				servletAdapter = xformProcessing(instance, writer, request, response, actionURL,
						session, servletAdapter, isActivity, projectName, activityName, projectVersion);
			}
			else{
				//case there's no properties: request parameters collection is null and formsExists is false)
				String newinstance = callExecutor(instance, null, false, isActivity, mode, projectVersion);
				if (request.getParameter("forward")!=null)
				{
					String forward = (String)request.getParameter("forward");
					if (newinstance != null) {
						int delim = forward.indexOf("processName=");
						if (delim >= 0) forward = forward.substring(0, delim+12) + newinstance;
					}
					if (projectVersion != null)
						forward = forward + "&version="+projectVersion;
					response.sendRedirect(forward);
			    }else{
	  				//Page redirection
					if(isActivity.booleanValue() == true){
					    if(mode.equals("consumer"))
					        //Redirection to todo list
							response.sendRedirect("/formgenerator/ServletActivityToDoList?projectName=" + projectName);
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator/ServletWorkList");
					}
					else{
					    if(mode.equals("consumer"))
					        //Redirection to instances projets
					        response.sendRedirect("/formgenerator/ServletInstancesProjects");
					    else
					        //Redirection to work list
							response.sendRedirect("/formgenerator/ServletWorkList");
					}
				}
			}
		  }
		  else
			//Message to user
			(new MessageServlet()).writeMessage(writer, "PARAMETER FOR projectName IS MISSING CANNOT PERFORM OPERATION");

		  //Stores the state of isActivity attribute in the session
		  session.setAttribute("formgenerator.isActivity", isActivity);

		  //Stores the servletAdapter
		  session.setAttribute("formgenerator.sAdapter", servletAdapter);

		  //Stores the projectname in the session
		  session.setAttribute("formgenerator.projectName", projectName);
		  
		  //Stores the projectVersion in the session
		  session.setAttribute("formgenerator.projectVersion", projectVersion);

		  //Stores the activity name in the session
		  session.setAttribute("formgenerator.activityName", activityName);

		  //Stores the application mode in the session
		  session.setAttribute("formgenerator.mode", mode);
	}


	/**
	 * Call the correct executor
	 * @param filledDocument document that will be use by the executor
	 * @param mode the application mode: consumer or exploitant
	 */
	private String callExecutor(Document filledDocument, Vector req_parameters, boolean formsExists, Boolean isActivity, String mode, String version)
		throws 	MalformedURLException, SAXException, IOException,
				RemoteException, NamingException, CreateException,
				HeroException, Exception {

		String instance = null;
		if(isActivity.booleanValue() == true){

			//Creation of executor
			BonitaActivityExecutor bonitaActivityExecutor = new BonitaActivityExecutor();

			//Terminate the activity
			bonitaActivityExecutor.actOnActivity(filledDocument, req_parameters, formsExists, mode, version);
		}
		else{

			//Creation of executor
			BonitaProjectExecutor bonitaProjectExecutor = new BonitaProjectExecutor();

			//Instanciate project
			instance = bonitaProjectExecutor.startBonitaProject(filledDocument, req_parameters, formsExists, version);
		}
		return instance;
	}




    /**
     * Initializes the servlet.
     *
     * @param config - the ServletConfig object
     * @throws javax.servlet.ServletException
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        //cat.info("--------------- initing ChibaView... ---------------");
        //read some params from web-inf
        contextRoot = getServletConfig().getServletContext().getRealPath("");
        if (contextRoot == null)
            contextRoot = getServletConfig().getServletContext().getRealPath(".");

        //get the relative path to the chiba config-file
        String path = getServletConfig().getInitParameter("chiba.config");

        //get the real path for the config-file
        if (path != null) {
            configPath = getServletConfig().getServletContext().getRealPath(path);
        }

        //get the path for the stylesheets
        //path = getServletConfig().getServletContext().getInitParameter("chiba.xforms.stylesPath");
        path = getServletConfig().getInitParameter("chiba.xforms.stylesPath");

        //get the real path for the stylesheets and configure a new StylesheetLoader with it
        if (path != null) {
            stylesPath = getServletConfig().getServletContext().getRealPath(path);
		    //cat.info("stylesPath: " + stylesPath);
        }

        //uploadDir = contextRoot	+ "/" + getServletConfig().getServletContext().getInitParameter("chiba.upload");
        //uploadDir = getServletConfig().getServletContext().getInitParameter("chiba.upload");
        uploadDir = getServletConfig().getInitParameter("chiba.upload");
        //Security constraint
        if (uploadDir != null) {
            if (uploadDir.toUpperCase().indexOf("WEB-INF") >= 0) {
                throw new ServletException("Chiba security constraint: uploadDir '" + uploadDir + "' not allowed");
            }
        }
    }




/**
	 * Servlet reaction on get method:
	 * Display the form generated for filling a valid XML file
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		  //set the content-type and character encoding for request and response
		  response.setContentType("text/html; charset=" + FormGeneratorConstant.CHARACTER_ENCODING);
		  request.setCharacterEncoding(FormGeneratorConstant.CHARACTER_ENCODING);

		  //Creating a session
		  HttpSession session = request.getSession(true);

		  //Name of the project we are working with
		  String projectName = request.getParameter("projectName");
		  
		  //The project version we are working with
		  String projectVersion = request.getParameter("version");

		  // get the user language
		  String userLanguage = request.getHeader("Accept-Language");

		  //write the 'forward' parameter in the session
		  if (request.getParameter("forward")!=null)
		    session.setAttribute("formgenerator.forward",request.getParameter("forward"));
		  else
		    session.removeAttribute("forward");

		  try {


			//Getting the writer
			PrintWriter out = response.getWriter();

            // build actionURL where forms are submitted to
            String actionURL = getActionURL(request, response);

			//Build HTML page according to an activity or a project we are working with
			writingUI(out, request, response, actionURL, session);

			//Closing writer
			out.close();

		  } catch (Exception e) {
			//Error
			if (request.getParameter("forward")!=null) {
				String saveTime = saveErrors(e);
				response.sendRedirect(request.getParameter("forward")+"&processError=saved&processName="+projectName+"&version="+projectVersion+"&saveTime="+saveTime);
			}
			else e.printStackTrace();

		  }
	}

    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected ServletAdapter updateContext(HttpServletRequest request, HttpSession session,Document instance,ServletAdapter servletAdapter) {
    	servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
		servletAdapter.setContextProperty("instance",instance);
		return(servletAdapter);
    }

    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected ServletAdapter updateContext(HttpServletRequest request, HttpSession session, Document formDocument, Document langDocument,ServletAdapter servletAdapter) {
    	servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    	servletAdapter.setContextProperty("instance",formDocument);
		servletAdapter.setContextProperty("lang",langDocument);
		return(servletAdapter);
    }


	/**
	 * Servlet reaction on post method:
	 * Submit information from user
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response)	throws ServletException, IOException
	{
		  String forward = null;
		  //Getting the current session
		  HttpSession session = request.getSession(true);

		  ServletAdapter servletAdapter = null;
		  String projectName = null;
		  String projectVersion = null;

		  try {
			response.setContentType("text/html; charset=" + FormGeneratorConstant.CHARACTER_ENCODING);
			request.setCharacterEncoding(FormGeneratorConstant.CHARACTER_ENCODING);

			//Getting servletAdapter from the session
			servletAdapter = (ServletAdapter)session.getAttribute("formgenerator.sAdapter");

			//Getting isActivity from the session
			Boolean isActivity = (Boolean)session.getAttribute("formgenerator.isActivity");
			//Set isActivity in the session
			session.setAttribute("formgenerator.isActivity", isActivity);

			//Getting projectName from the session
			projectName = (String)session.getAttribute("formgenerator.projectName");
			//Set the projectname in the session
			session.setAttribute("formgenerator.projectName", projectName);
			
//			Getting projectVersion from the session
			projectVersion = (String)session.getAttribute("formgenerator.projectVersion");
			//Set the projectversion in the session
			session.setAttribute("formgenerator.projectVersion", projectVersion);

			//Getting activityName from the session
			String activityName = (String)session.getAttribute("formgenerator.activityName");
			//Set the activityname in the session
			session.setAttribute("formgenerator.activityName", activityName);

			//Getting mode from the session
			String mode = (String)session.getAttribute("formgenerator.mode");

			//Getting forward from the session
			forward = (String)session.getAttribute("formgenerator.forward");

			//Set isActivity in the session
			session.setAttribute("formgenerator.forward", forward);

			if ((servletAdapter == null)||(isActivity == null)||(projectName == null)){
				//Exception
				throw new IOException("Invalid session - Cannot retrieve servletAdapter or ggf or isActivity or projectName in the session");
			}

			servletAdapter = updateContextPost(request, session, servletAdapter);

            //this.servletAdapter.executeHandler();
			servletAdapter.dispatch(null);

            // handle setRedirect <xforms:load show='replace'/>
            // and redirect from submission as well
            // NOTE - this needs to be checked *before* the this.getForwardMap()
            // as a submission handler may force a redirect
            if (servletAdapter.getRedirectUri() != null) {
                String redirectTo = servletAdapter.getRedirectUri();
                // todo: remove from session ?
                // shutdown processor
                servletAdapter.getChibaBean().shutdown();

                // send redirect (after encoding session id if required)
                response.sendRedirect(response.encodeRedirectURL(redirectTo));

                // remove redirect uri and terminate
                servletAdapter.setRedirect(null);
                return;
            }

            // handle forward <xforms:submission replace='all'/>
            Map forwardMap = servletAdapter.getForwardMap();
            InputStream forwardStream = (InputStream) forwardMap.get(ChibaAdapter.SUBMISSION_RESPONSE_STREAM);
            if (forwardStream != null) {
			        //get instance filled
					//Document filledDocument = (Document)this.servletAdapter.getChibaBean().getContainer().getModel("model").getInstance("instance").getInstanceDocument();
					Model model = (Model)servletAdapter.getChibaBean().lookup("model");
					Document filledDocument = (Document)model.getInstance("instance").getInstanceDocument();
		            if (filledDocument == null) {
		                throw new ServletException(Config.getInstance().getErrorMessage("session-invalid"));
		            }
		            //------ test if xform has been generated -----
		            String formPath = null;
		            boolean formsExists = false;
		            Vector v_req_params = null;

		            //instantiate RepositoryFacade to get form path (if exist)
	    				RepositoryFacade repositoryFacade = new RepositoryFacade(getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator+ "repository.xml");

	    				if (repositoryFacade == null){
	    					//Exception
	    					throw new IOException("Invalid session - Cannot retrieve repositoryFacade");
	    				}
	    				String realProjectName = projectName.replaceFirst("_instance.+","");
	    				if (isActivity.booleanValue() == true){
	    					formPath = repositoryFacade.getActivityFormPath(realProjectName, activityName, projectVersion);
	    				}else{
	    					formPath = repositoryFacade.getProjectFormPath(realProjectName, projectVersion);
	    				}
	    				if (formPath != null) {
	    					//xform has been generated and some properties may have constraints: hidden, readonly
	    					formsExists = true;
			            //Get parameters of the request (to be able latter in xxExecutor Bean
			            // remove hidden, readonly properties from the filledDocument)
			            Enumeration param_names = request.getParameterNames();
			            v_req_params = new Vector();

			            for (; param_names.hasMoreElements(); ) {
			                //Get the name of the request parameter
			                String name = (String)param_names.nextElement();
			                //remove prefix : d_  from for ex.: d_proc_str if xform generated or for ex. : d_C2 if no xform
			    				name = name.substring(2);
			    				v_req_params.add(name);
			            }
	    				}
					//Calling the executor
					String newinstance = this.callExecutor(filledDocument, v_req_params, formsExists, isActivity, mode, projectVersion);

					//redirect
					if ((forward!=null)&&!forward.equals("")){
						//redirection to forward
						if (newinstance != null) {
							int delim = forward.indexOf("processName=");
							if (delim >= 0) forward = forward.substring(0, delim+12) + newinstance;
						}
						if (projectVersion!=null)
							forward = forward + "&version="+projectVersion;
					    response.sendRedirect(forward);
					}else{
						if(isActivity.booleanValue() == true){

						    if(mode.equals("consumer"))
						        //Redirection to todo list
								response.sendRedirect("/formgenerator/ServletActivityToDoList?projectName=" + projectName);
						    else
						        //Redirection to work list
								response.sendRedirect("/formgenerator/ServletWorkList");
						}
						else{
							if (mode != null){
								if(mode.equals("consumer"))
						        //Redirection to instances projets
						        response.sendRedirect("/formgenerator/ServletInstancesProjects");
						    else
						        //Redirection to work list
								response.sendRedirect("/formgenerator/ServletWorkList");
							}
						}
					}
					/*
	                // todo: remove from session ?
	                // shutdown processor
	                this.servletAdapter.getChibaBean().shutdown();

	                // forward submission response
	                forwardResponse(forwardMap, response);

	                // remove forward response and terminate
	                this.servletAdapter.forward(null);
	                return;
	                */
            }

			//Set content type
			//response.setContentType("text/html");

			//Render result to output
			//this.chiba.runChiba(out);
			servletAdapter.buildUI(response.getWriter());
			response.getWriter().close();

		  } catch (Exception e) {
			//Error
			if (forward!=null) {
				String saveTime = saveErrors(e);
				response.sendRedirect(forward+"&processError=saved&processName="+projectName+"&saveTime="+saveTime);
			}
			else e.printStackTrace();
		  }
	}


    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected ServletAdapter updateContextPost(HttpServletRequest request, HttpSession session, ServletAdapter servletAdapter) {
    	servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    	return(servletAdapter);
    }


    /**
     * creates and configures the ServletAdapter which does the actual request processing.
     *
     * @param actionURL - the URL to submit to
     * @param session   - the Servlet session
     * @param formPath  - the relative location where forms are stored
     * @param xslFile   - the xsl file to use for transform
     * @param cssFile   - the CSS file to use for styling the output
     * @return ServletAdapter
     */
    private ServletAdapter setupServletAdapter(String actionURL, String hostUrl,
                                               HttpSession session,
                                               Document formDocument, String baseURI,
                                               String xslFile,
                                               String cssFile) throws XFormsException, URISyntaxException, Exception {
        //setup and configure the adapter
        ServletAdapter aAdapter = new ServletAdapter();
        aAdapter.setContextRoot(contextRoot);
        if ((configPath != null) && !(configPath.equals(""))) {
            aAdapter.setConfigPath(configPath);
        }

		//call getDocumentFromString to get a valid Chiba Document
		try {
			formDocument = getDocumentFromString(toString(formDocument));
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception("ServletBonitaForm.setupServletAdapter");
		}

		//aAdapter.setFormURI(new URI(this.contextRoot));
		aAdapter.setXFormsURI(this.contextRoot);
        //aAdapter.setFormDocument(formDocument);
		aAdapter.setXForms(formDocument);
        aAdapter.setStylesheetPath(stylesPath);
        aAdapter.setActionUrl(actionURL);
        aAdapter.setUploadDir(uploadDir);
		aAdapter.setHostUrl(hostUrl);

        if (xslFile != null) {
            aAdapter.setStylesheet(xslFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using xsl stylesheet: " + xslFile);
            }
        }
        if (cssFile != null) {
            aAdapter.setCSS(cssFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using css stylesheet: " + cssFile);
            }
        }

        Map servletMap = new HashMap();
        servletMap.put(ChibaAdapter.SESSION_ID, session.getId());
        aAdapter.setContextProperty(ChibaAdapter.SUBMISSION_RESPONSE, servletMap);

        return aAdapter;
    }

    /**
     * creates and configures the ServletAdapter which does the actual request processing.
     *
     * @param actionURL - the URL to submit to
     * @param session   - the Servlet session
     * @param formPath  - the relative location where forms are stored
     * @param xslFile   - the xsl file to use for transform
     * @param cssFile   - the CSS file to use for styling the output
     * @return ServletAdapter
     */
	/*
    private ServletAdapter setupServletAdapter(String actionURL,
                                               HttpSession session,
                                               String formPath,
                                               String xslFile,
                                               String cssFile) throws XFormsException, URISyntaxException {
        //setup and configure the adapter
        ServletAdapter aAdapter = new ServletAdapter();
        aAdapter.setContextRoot(contextRoot);
        if ((configPath != null) && !(configPath.equals(""))) {
            aAdapter.setConfigPath(configPath);
        }
        aAdapter.setFormPath(formPath);
		//aAdapter.setXFormsURI(formPath);
        aAdapter.setStylesheetPath(stylesPath);
        aAdapter.setActionUrl(actionURL);
        aAdapter.setUploadDir(uploadDir);

        if (xslFile != null) {
            aAdapter.setStylesheet(xslFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using xsl stylesheet: " + xslFile);
            }
        }
        if (cssFile != null) {
            aAdapter.setCSS(cssFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using css stylesheet: " + cssFile);
            }
        }

        Map servletMap = new HashMap();
        servletMap.put(ChibaAdapter.SESSION_ID, session.getId());
        aAdapter.setContextProperty(ChibaAdapter.SUBMISSION_RESPONSE, servletMap);

        return aAdapter;
    }
    */

    private ServletAdapter storeContextParams(HttpServletRequest request, ServletAdapter servletAdapter) {
        Enumeration params = request.getParameterNames();
        String s;
        while (params.hasMoreElements()) {
            s = (String) params.nextElement();
            //store all request-params we don't use in the context map of ChibaBean
            if (!(s.equals(XSL_PARAM_NAME) || s.equals(CSS_PARAM_NAME) || s.equals(ACTIONURL_PARAM_NAME))) {
                String value = request.getParameter(s);
                servletAdapter.setContextProperty(s, value);
                if (cat.isDebugEnabled()) {
                    cat.debug("added request param '" + s + "' added to context");
                }
            }
        }
        return(servletAdapter);
    }


	/**
	 * Return the action URL with jsession
	 * @param request : The request from user
	 * @param response : The response to its request
	 * @return a String object that represents the action URL
	 */
    private String getActionURL(HttpServletRequest request, HttpServletResponse response) {
        String defaultActionURL =
                request.getScheme()
                + "://"
                + request.getServerName()
                + ":"
                + request.getServerPort()
                + request.getContextPath()
                + request.getServletPath();
        String encodedDefaultActionURL = response.encodeURL(defaultActionURL);
        int sessIdx = encodedDefaultActionURL.indexOf(";jsession");
        String sessionId = null;
        if (sessIdx > -1) {
            sessionId = encodedDefaultActionURL.substring(sessIdx);
        }
        String actionURL = request.getParameter(ACTIONURL_PARAM_NAME);
        if (null == actionURL) {
            actionURL = encodedDefaultActionURL;
        } else if (null != sessionId) {
            actionURL += sessionId;
        }

        //cat.info("actionURL: " + actionURL);
        // encode the URL to allow for session id rewriting
        actionURL = response.encodeURL(actionURL);
        return actionURL;
    }

	private void writeFile(String formPath, Document formDocument)
	throws FileNotFoundException, IOException {
		FileOutputStream output = null;
		OutputFormat of = new OutputFormat("XML",FormGeneratorConstant.CHARACTER_ENCODING,true);
			of.setIndent(1);
		of.setIndenting(true);
		XMLSerializer serializer = null;
		try {
			output = new FileOutputStream(formPath);
			serializer = new XMLSerializer(output,of);
			serializer.asDOMSerializer();
			serializer.serialize(formDocument.getDocumentElement() );
			output.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			throw new FileNotFoundException("ServletBonitaForm.writeFile");
		} catch (IOException e) {
			e.printStackTrace();
			throw new IOException("ServletBonitaForm.writeFile");
		}
	}

	private void deleteFile(String filename) {
	    // Create a File object to represent the filename
	    File f = new File(filename);
	    // Make sure the file or directory exists and isn't write protected
	    if (!f.exists()) System.out.println("ERROR: ServletBonitaForm ==> deleteFile: no such file or directory: " + filename);
	    if (!f.canWrite()) System.out.println("ERROR: ServletBonitaForm ==> deleteFile: write protected: " + filename);
	    // If we passed all the tests, then attempt to delete it
	    boolean success = f.delete();
	    if (!success) System.out.println("ERROR: ServletBonitaForm ==> deleteFile: deletion failed");
	}

	private Document getDocument(String path) throws SAXException, IOException, ParserConfigurationException {

		//DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(false);
        factory.setAttribute("http://apache.org/xml/properties/dom/document-class-name",
                "org.chiba.xml.xforms.XFormsDocument");

		Document document = null;
		try {
			//get file and parse it to xmlDocument
			File xmlFile = new File(path);
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			document = documentBuilder.parse(xmlFile);
		} catch (SAXException e) {
			e.printStackTrace();
			throw new SAXException("ServletBonitaForm.getDocument");
		} catch (IOException e) {
			e.printStackTrace();
			throw new IOException("ServletBonitaForm.getDocument");
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new ParserConfigurationException("ServletBonitaForm.getDocument");
		}
		return document;
	}

	private Document getDocumentFromString(String theString)
	throws SAXException, IOException, ParserConfigurationException {

		//DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(false);
        factory.setAttribute("http://apache.org/xml/properties/dom/document-class-name",
                "org.chiba.xml.xforms.XFormsDocument");

		Document document = null;
		try {
			//get file and parse it to xmlDocument
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			document = documentBuilder.parse(new ByteArrayInputStream(theString.getBytes()));
		} catch (SAXException e) {
			e.printStackTrace();
			throw new SAXException("ServletBonitaForm.getDocumentFromString");
		} catch (IOException e) {
			e.printStackTrace();
			throw new IOException("ServletBonitaForm.getDocumentFromString");
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new ParserConfigurationException("ServletBonitaForm.getDocumentFromString");
		}
		return document;
	}

	private String getUserLanguage(String languages, String languagePath) {

		//Use a tokenizer to separate acceptable languages
	    StringTokenizer tokenizer = new StringTokenizer(languages, ",");
		String lang = null;
		boolean fileFound = false;
	    while (tokenizer.hasMoreTokens() && !fileFound) {
			lang = tokenizer.nextToken();
			int semi,dash;
			//Cut off any q-value that might come after a semi-colon
		    if ((semi = lang.indexOf(';')) != -1) {
		      lang = lang.substring(0, semi);
		    }
			//Trim any whitespace
		    lang = lang.trim();
			//A dash may separate the
		    //language from the country.
		    if ((dash = lang.indexOf('-')) != -1) {
				lang = lang.substring(0, dash);
		    }
			//try to find lang file
			File tmp = new File(languagePath + lang + ".xml");
			if (tmp.exists()) {
				fileFound = true;
			}
	    }
		if(!fileFound){
			lang = "en";
		}

		return lang;
	}

    private void forwardResponse(Map forwardMap, HttpServletResponse response) throws IOException {
        // fetch response stream
        InputStream responseStream = (InputStream) forwardMap.remove(ChibaAdapter.SUBMISSION_RESPONSE_STREAM);

        // copy header information
        Iterator iterator = forwardMap.keySet().iterator();
        while (iterator.hasNext()) {
            String name = iterator.next().toString();
            String value = forwardMap.get(name).toString();
            response.setHeader(name, value);
        }

        // copy stream content
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        for (int b = responseStream.read();
             b > -1;
             b = responseStream.read()) {
            outputStream.write(b);
        }

        // close streams
        responseStream.close();
        outputStream.close();
    }

	// Retourne un Document en String
	private static String toString(Document doc) throws Exception{

	    ByteArrayOutputStream os = new ByteArrayOutputStream();
	    OutputFormat outputFormat = new OutputFormat(doc);
	    outputFormat.setPreserveSpace(true);
	    outputFormat.setIndenting(true);
	    //outputFormat.setLineWidth(0);
	    XMLSerializer serializer = new XMLSerializer(os, outputFormat);
	    serializer.serialize(doc);

	    return new String(os.toByteArray());
	 }

	/**  13-05-2005
	 *   This method transform xmlInput to XForm instance.
	 *   <p/>
	 *
	 *   @param xmlInput XML Input to transform.
	 * @throws SAXException
	 * @throws IOException
	 */
	/*
	public Document getInstance(Document xmlInput){

		//get xmlInput root
		Element root = xmlInput.getDocumentElement();
		NodeList rootChilds = root.getChildNodes();

		//Instance Document creation with the root element called process
		DOMImplementation domImpl = new DOMImplementationImpl();
		Document instanceDocument = domImpl.createDocument(null, "process" , null);
		Element instanceRoot = instanceDocument.getDocumentElement();

		//project properties element creation
		Element projectProperties = instanceDocument.createElement("projectProperties");
		instanceRoot.appendChild(projectProperties);

		//activity properties element creation
		Element activityProperties = instanceDocument.createElement("activityProperties");
		instanceRoot.appendChild(activityProperties);

		//get xmlInput project and activity propeties nodes
		Node inputProjectProperties = rootChilds.item(0);
		NodeList propertiesChilds = inputProjectProperties.getChildNodes();

		Node inputActivityProperties = null;
		NodeList activityPropertiesChilds = null;
		if(this.activityName != null){
			inputActivityProperties = rootChilds.item(1);
			activityPropertiesChilds = activityProperties.getChildNodes();
		}

		int i; //i will be used also to create activity binds and properties if they exist

		//project properties
		for(i=0;i<propertiesChilds.getLength();i++){
				Node property = propertiesChilds.item(i);
				NodeList propertyChilds = property.getChildNodes();

				//get property name
				Element propertyNameElement = (Element)propertyChilds.item(0); //property name will be allways in the same position
				Text propertyNameText = (Text)propertyNameElement.getFirstChild();
				String propertyName = propertyNameText.getNodeValue();

				//get property value
				Element propertyValueElement = (Element)propertyChilds.item(1); //property value will be allways in the same position
				Text propertyValueText = (Text)propertyValueElement.getFirstChild();
				String propertyValue = propertyValueText.getNodeValue();

				//set property node in instance file
				Element propertyElement = instanceDocument.createElement(propertyName);
				Text propertyText = instanceDocument.createTextNode(propertyValue);
				propertyElement.appendChild(propertyText);
				projectProperties.appendChild(propertyElement);
		}

		//activity properties
		if(this.activityName!=null){
			for(int j=i,k=0;k<activityPropertiesChilds.getLength();j++,k++){
				Node property = activityPropertiesChilds.item(k);
				NodeList propertyChilds = property.getChildNodes();

				//get property name
				Element propertyNameElement = (Element)propertyChilds.item(0); //property name will be allways in the same position
				Text propertyNameText = (Text)propertyNameElement.getFirstChild();
				String propertyName = propertyNameText.getNodeValue();

				//get property value
				Element propertyValueElement = (Element)propertyChilds.item(2); //property type will be allways in the same position
				Text propertyValueText = (Text)propertyValueElement.getFirstChild();
				String propertyValue = propertyValueText.getNodeValue();

				//set property node in instance file
				Element propertyElement = instanceDocument.createElement(propertyName);
				Text propertyText = instanceDocument.createTextNode(propertyValue);
				propertyElement.appendChild(propertyText);
				activityProperties.appendChild(propertyElement);
			}
		}
		return instanceDocument;

	}
	*/

	/**  13-05-2005
	 *   This method creates xform and language documents.
	 *   <p/>
	 *
	 *   @param xmlInput xml input Document.
	 * @throws SAXException
	 * @throws IOException
	 */
	public Document createXform(Document xmlInput, String xmlBase, String projectName, String activityName, String projectVersion) throws Exception {
		//get xmlInput root
		Element root = xmlInput.getDocumentElement();
		NodeList rootChilds = root.getChildNodes();

		//XForm Document creation with the root element called html
		DOMImplementation domImpl = new DOMImplementationImpl();
		Document xformDocument = domImpl.createDocument(null, "html" , null);
		Element xformRoot = xformDocument.getDocumentElement();

		//Set xmlns...
		xformRoot.setAttribute("xml:base",xmlBase);
		xformRoot.setAttribute("xmlns","http://www.w3.org/2002/06/xhtml2");
		xformRoot.setAttribute("xmlns:xf","http://www.w3.org/2002/xforms");
		xformRoot.setAttribute("xmlns:xsd","http://www.w3.org/2001/XMLSchema");
		xformRoot.setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");

		//head
		Element head = xformDocument.createElement("head");
		xformRoot.appendChild(head);

		//tittle
		Element tittle = xformDocument.createElement("tittle");
		Text tittleText = xformDocument.createTextNode(projectName + "XForm");
		tittle.appendChild(tittleText);
		head.appendChild(tittle);

		//model
		Element model = xformDocument.createElement("xf:model");
		//model.setPrefix("");
		model.setAttribute("id","model");
		head.appendChild(model);

		//instances
		Element instance = xformDocument.createElement("xf:instance");
		instance.setAttribute("id","instance");
		instance.setAttribute("src","contextobject:$instance");
		model.appendChild(instance);

		//submission
		Element submission = xformDocument.createElement("xf:submission");
		submission.setAttribute("id","bValider");
		submission.setAttribute("action","");
		submission.setAttribute("method","post");
		submission.setAttribute("replace","all");
		model.appendChild(submission);

		//binds
		//bind element to clone and create new binds
		Element nameBind = xformDocument.createElement("xf:bind");
		nameBind.setAttribute("id","");
		nameBind.setAttribute("nodeset","");
		nameBind.setAttribute("readonly","");
		nameBind.setAttribute("required","");
		nameBind.setAttribute("type","");
		nameBind.setAttribute("relevant","");
		//nameBind.setAttribute("int:extattr","");

		//select choice binds
		//select choice bind element to clone and create new binds
		Element selectChoiceBind = xformDocument.createElement("xf:bind");
		selectChoiceBind.setAttribute("id","");
		selectChoiceBind.setAttribute("nodeset","");

		//project properties node will be always rootChilds.item(1)
		Node properties = rootChilds.item(1);
		NodeList propertiesChilds = properties.getChildNodes();

		//activity properties node will be always rootChilds.item(3)
		Node activityProperties = null;
		NodeList activityPropertiesChilds = null;
		if(activityName!=null){
			activityProperties = rootChilds.item(3);
			activityPropertiesChilds = activityProperties.getChildNodes();
		}

		//end head

		//body
		Element body = xformDocument.createElement("body");
		xformRoot.appendChild(body);

		//group
		Element group = xformDocument.createElement("xf:group");
		group.setAttribute("appearance","maximal");
		body.appendChild(group);

		//xform elements

		//label element to clone and create new labels
		Element defaultLabel = xformDocument.createElement("xf:label");

		//input element to clone and create new inputs
		Element defaultInput = xformDocument.createElement("xf:input");
		defaultInput.setAttribute("bind","");
		//defaultInput.setAttribute("ref","");
		defaultInput.setAttribute("extattr","");
		defaultInput.setAttribute("content","no");
		Element defaultInputLabel = xformDocument.createElement("xf:label");
		defaultInput.appendChild(defaultInputLabel);
		Element defaultInputAlert = xformDocument.createElement("xf:alert");
		Text defaultInputAlertText = xformDocument.createTextNode("");
		defaultInputAlertText.setNodeValue("Error!");
		defaultInputAlert.appendChild(defaultInputAlertText);
		defaultInput.appendChild(defaultInputAlert);

		//textarea element to clone and create new textareas
		Element defaultTextArea = xformDocument.createElement("xf:textarea");
		defaultTextArea.setAttribute("bind","");
		//defaultTextArea.setAttribute("ref","");
		defaultTextArea.setAttribute("extattr","");
		defaultTextArea.setAttribute("content","no");
		Element defaultTextAreaLabel = xformDocument.createElement("xf:label");
		defaultTextArea.appendChild(defaultTextAreaLabel);
		Element defaultTextAreaAlert = xformDocument.createElement("xf:alert");
		Text defaultTextAreaAlertText = xformDocument.createTextNode("");
		defaultTextAreaAlertText.setNodeValue("Error!");
		defaultTextAreaAlert.appendChild(defaultTextAreaAlertText);
		defaultTextArea.appendChild(defaultTextAreaAlert);

		//select element to clone and create new selects
		Element defaultSelect = xformDocument.createElement("xf:select1");
		defaultSelect.setAttribute("bind","");
		//defaultSelect.setAttribute("ref","");
		defaultSelect.setAttribute("extattr","");
		defaultSelect.setAttribute("content","no");
		Element defaultSelectLabel = xformDocument.createElement("xf:label");
		defaultSelect.appendChild(defaultSelectLabel);
		Element defaultSelectAlert = xformDocument.createElement("xf:alert");
		Text defaultSelectAlertText = xformDocument.createTextNode("");
		defaultSelectAlertText.setNodeValue("Error!");
		defaultSelectAlert.appendChild(defaultSelectAlertText);
		defaultSelect.appendChild(defaultSelectAlert);
		Element defaultSelectChoices = xformDocument.createElement("xf:choices");
		defaultSelect.appendChild(defaultSelectChoices);

		//select item element to clone and create new select items
		Element defaultSelectItem = xformDocument.createElement("xf:item");
		Element defaultSelectItemLabel = xformDocument.createElement("xf:label");
		defaultSelectItem.appendChild(defaultSelectItemLabel);
		Element defaultSelectItemValue = xformDocument.createElement("xf:value");
		defaultSelectItem.appendChild(defaultSelectItemValue);
		Text defaultSelectItemLabelText = xformDocument.createTextNode("");
		defaultSelectItemLabel.appendChild(defaultSelectItemLabelText);
		Text defaultSelectItemLabelValue = xformDocument.createTextNode("");
		defaultSelectItemValue.appendChild(defaultSelectItemLabelValue);

		//submit element to clone and create new inputs
		Element defaultSubmit = xformDocument.createElement("xf:submit");
		defaultSubmit.setAttribute("submission","");
		Element defaultSubmitLabel = xformDocument.createElement("xf:label");
		defaultSubmit.appendChild(defaultSubmitLabel);


		//project(or activity) name input will be allways the same
		Element projectNameLabel = (Element)defaultLabel.cloneNode(true);
		String realProjectName = projectName.replaceFirst("_instance.+","");
		projectNameLabel.setAttribute("id","projectname");
		Text projectNameLabelText = xformDocument.createTextNode(realProjectName);
		projectNameLabel.appendChild(projectNameLabelText);
		group.appendChild(projectNameLabel);

		//project version label
		Element projectVersionLabel = (Element)defaultLabel.cloneNode(true);
		projectVersionLabel.setAttribute("id","projectversion");
		Text projectVersionLabelText = xformDocument.createTextNode(projectVersion);
		projectVersionLabel.appendChild(projectVersionLabelText);
		group.appendChild(projectVersionLabel);
		
		//activity name label
		if(activityName!=null){
			Element activityNameLabel = (Element)defaultLabel.cloneNode(true);
			activityNameLabel.setAttribute("id","activityname");
			Text activityNameLabelText = xformDocument.createTextNode(activityName);
			activityNameLabel.appendChild(activityNameLabelText);
			group.appendChild(activityNameLabel);
		}

		int i; //i will be used also to create activity binds and properties if they exist

		//project properties
		for(i=0;i<propertiesChilds.getLength();i++){
				Element property = (Element)propertiesChilds.item(i);
				NodeList propertyChilds = property.getChildNodes();
				//get property name
				//Element propertyNameElement = (Element)propertyChilds.item(0); //property name will be allways in the same position
				//Text propertyNameText = (Text)propertyNameElement.getFirstChild();
				//String propertyName = propertyNameText.getNodeValue();
				String propertyName = property.getAttribute("id");

				//get element type
				Element propertyTypeElement = (Element)propertyChilds.item(1); //property type will be allways in the same position
				Text propertyTypeText = (Text)propertyTypeElement.getFirstChild();
				String propertyType = propertyTypeText.getNodeValue();

				if(propertyType.equalsIgnoreCase("select")){
					//build select node
					Element tmpSelect = (Element)defaultSelect.cloneNode(true);
					tmpSelect.setAttribute("extattr","project");
					tmpSelect.setAttribute("bind","b " + propertyName);
					//tmpSelect.setAttribute("ref","/*[1]/*[2]/*[" + (i+1) + "]/*[2]");
					//tmpSelect.setAttribute("ref",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label text
					Element tmpSelectLabel = (Element)tmpSelect.getFirstChild();
					Text tmpSelectLabelText = xformDocument.createTextNode(propertyName);
					tmpSelectLabel.appendChild(tmpSelectLabelText);

					//set items labels
					Element tmpSelectChoices = (Element)tmpSelect.getLastChild();
					Node propertyPossibleValues = propertyChilds.item(2); //possible values will be allways in the same position
					NodeList possibleValues = propertyPossibleValues.getChildNodes();

					for(int l=1;l<=possibleValues.getLength();l++){
						Element tmpItem = (Element)defaultSelectItem.cloneNode(true);
						//set item label
						Element tmpSelectItemLabel = (Element)tmpItem.getFirstChild();
						//tmpSelectItemLabel.setAttribute("ref","/*[1]/*[2]/*[" + (i+1) + "]/*[4]/*[" + l + "]");
						//tmpSelectItemLabel.setAttribute("ref",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/possible[" + l + "]");
						tmpSelectItemLabel.setAttribute("bind","b " + propertyName + " choice" + l);
						//set item value
						Element tmpSelectItemValue = (Element)tmpItem.getLastChild();
						//tmpSelectItemValue.setAttribute("ref","/*[1]/*[2]/*[" + (i+1) + "]/*[4]/*[" + l + "]");
						//tmpSelectItemValue.setAttribute("ref",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/possible[" + l + "]");
						tmpSelectItemValue.setAttribute("bind","b " + propertyName + " choice" + l);
						tmpSelectChoices.appendChild(tmpItem);

						//add element bind
						Element tmpBind = (Element)selectChoiceBind.cloneNode(true);
						tmpBind.setAttribute("id","b " + propertyName + " choice" + l);
						//tmpBind.setAttribute("nodeset","/*[1]/*[2]/*[" + i + "]/*[2]");
						tmpBind.setAttribute("nodeset",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/" + possibleValues.item(l-1).getNodeName() + "[" + l + "]");
						model.appendChild(tmpBind);
					}
					group.appendChild(tmpSelect);

				}else if (propertyType.equalsIgnoreCase("textarea")){
					//build textarea node
					Element tmpTextarea = (Element)defaultTextArea.cloneNode(true);
					tmpTextarea.setAttribute("extattr","project");
					tmpTextarea.setAttribute("bind","b " + propertyName);
					//tmpTextarea.setAttribute("ref","/*[1]/*[2]/*[" + (i+1) + "]/*[2]");
					//tmpTextarea.setAttribute("ref",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label attributes
					Element tmpTextareaLabel = (Element)tmpTextarea.getFirstChild();
					Text tmpTextareaLabelText = xformDocument.createTextNode(propertyName);
					tmpTextareaLabel.appendChild(tmpTextareaLabelText);

					group.appendChild(tmpTextarea);

				}else{
					//build input node
					Element tmpInput = (Element)defaultInput.cloneNode(true);
					tmpInput.setAttribute("extattr","project");
					tmpInput.setAttribute("bind","b " + propertyName);
					//tmpInput.setAttribute("ref","/*[1]/*[2]/*[" + (i+1) + "]/*[2]");
					//tmpInput.setAttribute("ref",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label attributes
					Element tmpInputLabel = (Element)tmpInput.getFirstChild();
					Text tmpInputLabelText = xformDocument.createTextNode(propertyName);
					tmpInputLabel.appendChild(tmpInputLabelText);

					group.appendChild(tmpInput);
				}
				//add element bind
				Element tmpBind = (Element)nameBind.cloneNode(true);
				tmpBind.setAttribute("id","b " + propertyName);
				//tmpBind.setAttribute("nodeset","/*[1]/*[2]/*[" + i + "]/*[2]");
				tmpBind.setAttribute("nodeset",properties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());
				tmpBind.setAttribute("readonly","false()");
				tmpBind.setAttribute("required","false()");
				tmpBind.setAttribute("type","string");
				tmpBind.setAttribute("relevant","true()");
				model.appendChild(tmpBind);
			//}
		}

		//activity properties
		if(activityName!=null){
			for(int j=i,k=0;k<activityPropertiesChilds.getLength();j++,k++){
				Element property = (Element)activityPropertiesChilds.item(k);
				NodeList propertyChilds = property.getChildNodes();

				//get property name
				//Element propertyNameElement = (Element)propertyChilds.item(0); //property name will be allways in the same position
				//Text propertyNameText = (Text)propertyNameElement.getFirstChild();
				//String propertyName = propertyNameText.getNodeValue();
				String propertyName = property.getAttribute("id");

				//get element type
				Element propertyTypeElement = (Element)propertyChilds.item(1); //property type will be allways in the same position
				Text propertyTypeText = (Text)propertyTypeElement.getFirstChild();
				String propertyType = propertyTypeText.getNodeValue();

				if(propertyType.equalsIgnoreCase("select")){
					//build select node
					Element tmpSelect = (Element)defaultSelect.cloneNode(true);
					tmpSelect.setAttribute("extattr","activity");
					tmpSelect.setAttribute("bind","b " + propertyName);
					//tmpSelect.setAttribute("ref","/*[1]/*[4]/*[" + (k+1) + "]/*[2]");
					//tmpSelect.setAttribute("ref",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label attributes
					Element tmpSelectLabel = (Element)tmpSelect.getFirstChild();
					Text tmpSelectLabelText = xformDocument.createTextNode(propertyName);
					tmpSelectLabel.appendChild(tmpSelectLabelText);

					//set items labels
					Element tmpSelectChoices = (Element)tmpSelect.getLastChild();
					Node propertyPossibleValues = propertyChilds.item(2); //possible values will be allways in the same position
					NodeList possibleValues = propertyPossibleValues.getChildNodes();

					for(int l=1;l<=possibleValues.getLength();l++){
						Element tmpItem = (Element)defaultSelectItem.cloneNode(true);
						//set item label
						Element tmpSelectItemLabel = (Element)tmpItem.getFirstChild();
						//tmpSelectItemLabel.setAttribute("ref","/*[1]/*[4]/*[" + (k+1) + "]/*[4]/*[" + l + "]");
						//tmpSelectItemLabel.setAttribute("ref",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/possible[" + l + "]");
						tmpSelectItemLabel.setAttribute("bind","b " + propertyName + " choice" + l);
						//set item value
						Element tmpSelectItemValue = (Element)tmpItem.getLastChild();
						//tmpSelectItemValue.setAttribute("ref","/*[1]/*[4]/*[" + (k+1) + "]/*[4]/*[" + l + "]");
						//tmpSelectItemValue.setAttribute("ref",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/possible[" + l + "]");
						tmpSelectItemValue.setAttribute("bind","b " + propertyName + " choice" + l);
						tmpSelectChoices.appendChild(tmpItem);

						//add element bind
						Element tmpBind = (Element)selectChoiceBind.cloneNode(true);
						tmpBind.setAttribute("id","b " + propertyName + " choice" + l);
						//tmpBind.setAttribute("nodeset","/*[1]/*[2]/*[" + i + "]/*[2]");
						tmpBind.setAttribute("nodeset",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyPossibleValues.getNodeName() + "/" + possibleValues.item(l-1).getNodeName() + "[" + l + "]");
						model.appendChild(tmpBind);
					}
					group.appendChild(tmpSelect);

				}else if (propertyType.equalsIgnoreCase("textarea")){
					//build textarea node
					Element tmpTextarea = (Element)defaultTextArea.cloneNode(true);
					tmpTextarea.setAttribute("extattr","activity");
					tmpTextarea.setAttribute("bind","b " + propertyName);
					//tmpTextarea.setAttribute("ref","/*[1]/*[4]/*[" + (k+1) + "]/*[2]");
					//tmpTextarea.setAttribute("ref",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label attributes
					Element tmpTextareaLabel = (Element)tmpTextarea.getFirstChild();
					Text tmpTextareaLabelText = xformDocument.createTextNode(propertyName);
					tmpTextareaLabel.appendChild(tmpTextareaLabelText);

					group.appendChild(tmpTextarea);

				}else{
					//build input node
					Element tmpInput = (Element)defaultInput.cloneNode(true);
					tmpInput.setAttribute("extattr","activity");
					tmpInput.setAttribute("bind","b " + propertyName);
					//tmpInput.setAttribute("ref","/*[1]/*[4]/*[" + (k+1) + "]/*[2]");
					//tmpInput.setAttribute("ref",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());

					//set label attributes
					Element tmpInputLabel = (Element)tmpInput.getFirstChild();
					Text tmpInputLabelText = xformDocument.createTextNode(propertyName);
					tmpInputLabel.appendChild(tmpInputLabelText);

					group.appendChild(tmpInput);
				}
				//add element bind
				Element tmpBind = (Element)nameBind.cloneNode(true);
				tmpBind.setAttribute("id","b " + propertyName);
				//tmpBind.setAttribute("nodeset","/*[1]/*[4]/*[" + k + "]/*[2]");
				tmpBind.setAttribute("nodeset",activityProperties.getNodeName() + "/" + property.getNodeName() + "[@id='" + propertyName + "']/" + propertyChilds.item(0).getNodeName());
				tmpBind.setAttribute("readonly","false()");
				tmpBind.setAttribute("required","false()");
				tmpBind.setAttribute("type","string");
				tmpBind.setAttribute("relevant","true()");
				model.appendChild(tmpBind);
			}
		}


		//project(or activity) submit will be allways the same
		Element submit = (Element)defaultSubmit.cloneNode(true);
		submit.setAttribute("submission","bValider");
		//set label value
		Element submitLabel = (Element)submit.getFirstChild();
		Text submitLabelText = xformDocument.createTextNode("Submit");
		submitLabel.appendChild(submitLabelText);
		group.appendChild(submit);

		//end xform elements
		//end group

		//call getDocumentFromString to build a valid Chiba's document
		try {
			String theString = toString(xformDocument);
			xformDocument = getDocumentFromString(theString);
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception("ServletBonitaForm.createXform");
		}

		return(xformDocument);
	}

	protected String getErrors (Throwable t) {
		String mes = "";
        if(t != null) {
        	if(t.getCause() != null) mes = mes + "\n\nCause:\n\n" + t.getCause();
            StackTraceElement[] aStacks = t.getStackTrace();
        	if (0 < aStacks.length) {
        		mes = mes + "\n\nDetail:\n\n";
        		for (int i = 0; i < aStacks.length; i++) {
        			mes = mes + aStacks[i].toString() + "\n";
        		}
        	}
        }
        return(mes);
	}

	// saveErrors:
	// since a trace stack may be too large to successfully transmit back to
	// a calling external application via a request object, we write the
	// stack and any other relevant error message information to a file
	// with specific delimiters. Currently, the delimiter is the 'Detail:'
	// string which represents the break between the error message and the
	// trace stack report. By saving the error data in a file, the calling
	// application can read the file in order to subsequently display the
	// error (if the 'processError' request attribute is set to 'saved'.
	protected String saveErrors (Throwable t) {
		String mes = "";
		long curTime = 0;
		String saveTime = "";
        if(t != null) {
        	if(t.getMessage() != null) mes = mes + t.getMessage();
            StackTraceElement[] aStacks = t.getStackTrace();
        	if (0 < aStacks.length) {
        		mes = mes + "\n\nDetail:\n\n";
        		for (int i = 0; i < aStacks.length; i++) {
        			mes = mes + aStacks[i].toString() + "\n";
        		}
        	}
        	//save the error so that a calling application can read it - if need be!
        	try {
        		java.util.Date curDate = new java.util.Date();
        		curTime = curDate.getTime();
        		saveTime = Long.toString(curTime);
        		ReadEnv re = new ReadEnv();
        		String saveErrorPath = re.getVariable("BONITA_HOME") + File.separator + ".formerr-" + saveTime;
        		java.io.BufferedWriter bw = new BufferedWriter(new FileWriter(saveErrorPath));
        		bw.write(mes);
        		bw.close();
        	}
        	catch (IOException io) {
        		io.printStackTrace();
        	}
        	catch (Exception e) {
        		e.printStackTrace();
        	}
        }
        return(saveTime);
	}

	private static String toXmlString(Document document) {
		try {
			OutputFormat outputFormat = new OutputFormat(document,FormGeneratorConstant.DEBUG_OUTPUT_FORMAT_ENCODING,true);
			outputFormat.setIndenting(true);
			outputFormat.setIndent(1);
			outputFormat.setPreserveSpace(true);
			StringWriter writer = new StringWriter();
			DOMSerializer serializer = new XMLSerializer(writer, outputFormat);
			serializer.serialize(document);
			writer.close();
			return writer.toString();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
}