/*
 * Created on 23 avr. 2004 by the Message Center Team
 *
 */
package mc.formgenerator.servlets;

import mc.formgenerator.servlets.bonita.RepositoryFacade;
import mc.formgenerator.servlets.util.*;

import hero.interfaces.*;
import hero.util.HeroException;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import mc.formgenerator.bonita.BonitaActivityAdapter;
import mc.formgenerator.bonita.BonitaActivityExecutor;
import mc.formgenerator.bonita.BonitaProjectAdapter;
import mc.formgenerator.bonita.BonitaProjectExecutor;
import mc.formgenerator.bonita.DocumentParser;

import org.apache.log4j.Category;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.chiba.xml.xforms.Model;
import org.chiba.xml.xforms.config.Config;
import org.chiba.xml.xforms.exception.XFormsException;
import org.chiba.adapter.ChibaAdapter;

/**
 * Servlet that will display XForm Look&Feel, without interaction with Bonita.
 * This servlet is a modification of ChibaServlet
 */
public class ServletLookAndFeel extends HttpServlet {

	//ServletAdapter instance
	private ServletAdapter servletAdapter = null;

	//Allow us to know if we are working with an activity or a project
	private Boolean isActivity;

	//Name of the project we are working with
	private String projectName = null;
	
	//Name of the project version we are working with
	private String projectVersion = null;

	//Name of the activity we are working with
	private String activityName = null;

	//Application mode consumer or exploitant
	private String mode = null;

	//forward URL
	private String forward = null;

    //init-params
    private static Category cat = Category.getInstance(ServletLookAndFeel.class);
    //private static final String FORM_PARAM_NAME = "form";
    private static final String XSL_PARAM_NAME = "xslt";
    private static final String CSS_PARAM_NAME = "css";
    private static final String ACTIONURL_PARAM_NAME = "action_url";

    /*
     * It is not thread safe to modify these variables once the
     * init(ServletConfig) method has been called
     */
    // the absolute path to the Chiba config-file
    private String configPath = null;

    // the rootdir of this app; forms + documents fill be searched under this root
    private String contextRoot = null;

    // where uploaded files are stored
    private String uploadDir = null;

    private String stylesPath = null;


    /**
     * Returns a short description of the servlet.
     *
     * @return - Returns a short description of the servlet.
     */
    public String getServletInfo() {
        return "Servlet Controller for Chiba Bonita's XForms Processor";
    }

    /**
     * Destroys the servlet.
     */
    public void destroy() {
    }


	/**
	 * Start the server side interpretation of the xforms document
	 * @param document the xforms document we have to interprete
	 * @param writer the servlet's writer
	 */
	private void xformProcessing(PrintWriter writer, HttpServletRequest request, String actionURL, HttpSession session)
		throws 	TransformerConfigurationException, XFormsException,
		FileNotFoundException, TransformerException, IOException, URISyntaxException{


		//We must get bonita project name
		this.projectName = request.getParameter("projectName");
		
		//We must get bonita project version name
		this.projectVersion = request.getParameter("projectVersion");

		//We must get bonita activity name eventually
		this.activityName = request.getParameter("activityName");

		if((this.activityName == null) || (this.activityName.equals(""))){
			this.isActivity = new Boolean(false);
		}else{
			this.isActivity = new Boolean(true);
		}

		//hostUrl needed by ui.xsl
		String hostUrl = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort();

		//xmlBase needed by Chiba
		String xmlBase = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort()+ request.getContextPath() + "/";
		Document xformDocument = null;
		Document langInstance = null;
		Document instance = null;

        String xslFile = request.getParameter(XSL_PARAM_NAME);  

        String css = request.getParameter(CSS_PARAM_NAME);
		if(css == null){
			css = "web/styles/admin.css";
		}

		String realProjectName = this.projectName.replaceFirst("_instance.+","");

		String formPath = null;

		//instantiate RepositoryFacade to get form path (if exist)
		RepositoryFacade repositoryFacade = new RepositoryFacade(getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator+ "repository.xml");

		if (repositoryFacade == null){
			//Exception
			throw new IOException("Invalid session - Cannot retrieve repositoryFacade");
		}

		if(this.isActivity.booleanValue() == true){
//System.out.println("WORKING WITH ACTIVITY");
			formPath = repositoryFacade.getActivityFormPath(realProjectName, this.activityName, this.projectVersion);
		}else{
//System.out.println("WORKING WITH PROJECT: " + realProjectName);
			formPath = repositoryFacade.getProjectFormPath(realProjectName, this.projectVersion);
		}

		if (formPath!=null){
			//xform file exist

			//get file and parse it to xmlDocument
			String realFormPath = getServletConfig().getServletContext().getRealPath("") + formPath;
			xformDocument = getDocument(realFormPath);
			//set xmlbase
			Element xformRoot = xformDocument.getDocumentElement();
			xformRoot.setAttribute("xml:base",xmlBase);

			//lookup languages and create language instance
			String languagePath = null;
			if(this.isActivity.booleanValue() == true){
				//We are working with an activity
				languagePath = getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.projectVersion + File.separator + this.activityName + File.separator;
			}else{
				//We are working with a project
				languagePath = getServletConfig().getServletContext().getRealPath("") + File.separator + "web" + File.separator + "xforms" + File.separator + realProjectName + File.separator + this.projectVersion + File.separator;
			}

			String languages = request.getHeader("Accept-Language");
			String userLanguage = "en";
			if((languages != null) && (!languages.equals(""))){
				userLanguage = getUserLanguage(languages,languagePath);
			}

			langInstance = getDocument(languagePath + userLanguage + ".xml");

			instance = getDocument(languagePath + "instance.xml");
			//xformDocument = getDocument(realFormPath);

			//this.servletAdapter = setupServletAdapter(actionURL, session, formPath, xslFile, css);
			this.servletAdapter = setupServletAdapter(actionURL, hostUrl, session, xformDocument, realFormPath, xslFile, css);

		}else{
			//xform file does not exist. Generate xform document
			System.out.println("ERROR: XForm file does not exist in repository");
		}

		//Chiba launching
		//Build HTML page according to an activity or a project we are working with
		try{
			//System.out.println("INSTANCE: " + toString(instance));
			//System.out.println("XFORM DOCUMENT: " + toString(xformDocument));
			//if (langInstance!=null) System.out.println("LANG INSTANCE: " + toString(langInstance));
			if (formPath!=null){
				updateContext(request, session, instance, langInstance);
			}else{
				System.out.println("ERROR: XForm file does not exist in repository");
			}

	        //add all request params that are not used by this servlet to the context map in ChibaBean
	        storeContextParams(request);
	        this.servletAdapter.init();
	        //this.servletAdapter.executeHandler();
			this.servletAdapter.dispatch(null);
	        this.servletAdapter.buildUI(writer);
		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}


    /**
     * Initializes the servlet.
     *
     * @param config - the ServletConfig object
     * @throws javax.servlet.ServletException
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        //cat.info("--------------- initing ChibaView... ---------------");
        //read some params from web-inf
        contextRoot = getServletConfig().getServletContext().getRealPath("");
        if (contextRoot == null)
            contextRoot = getServletConfig().getServletContext().getRealPath(".");

        //get the relative path to the chiba config-file
        String path = getServletConfig().getInitParameter("chiba.config");

        //get the real path for the config-file
        if (path != null) {
            configPath = getServletConfig().getServletContext().getRealPath(path);
        }

        //get the path for the stylesheets
        //path = getServletConfig().getServletContext().getInitParameter("chiba.xforms.stylesPath");
        path = getServletConfig().getInitParameter("chiba.xforms.stylesPath");

        //get the real path for the stylesheets and configure a new StylesheetLoader with it
        if (path != null) {
            stylesPath = getServletConfig().getServletContext().getRealPath(path);
            //cat.info("stylesPath: " + stylesPath);
        }

        //uploadDir = contextRoot	+ "/" + getServletConfig().getServletContext().getInitParameter("chiba.upload");
        //uploadDir = getServletConfig().getServletContext().getInitParameter("chiba.upload");
        uploadDir = getServletConfig().getInitParameter("chiba.upload");
        //Security constraint
        if (uploadDir != null) {
            if (uploadDir.toUpperCase().indexOf("WEB-INF") >= 0) {
                throw new ServletException("Chiba security constraint: uploadDir '" + uploadDir + "' not allowed");
            }
        }
    }




	/**
	 * Servlet reaction on get method:
	 * Display the form generated for filling a valid XML file
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {

		this.servletAdapter = null;

		//set the content-type and character encoding for request and response
		response.setContentType("text/html; charset=" + FormGeneratorConstant.CHARACTER_ENCODING);
		request.setCharacterEncoding(FormGeneratorConstant.CHARACTER_ENCODING);

		//Creating a session
		HttpSession session = request.getSession(true);

		// get the user language
		String userLanguage = request.getHeader("Accept-Language");

		//write the 'forward' parameter in the session
		if (request.getParameter("forward")!=null)
		    session.setAttribute("formgenerator.forward",request.getParameter("forward"));
		else
		    session.removeAttribute("forward");

		try {
			//Getting the writer
			PrintWriter out = response.getWriter();

            // build actionURL where forms are submitted to
            String actionURL = getActionURL(request, response);

			//Build HTML page according to an activity or a project we are working with
			//writingUI(out, request, response, actionURL, session);
			xformProcessing(out, request, actionURL, session);

			//Stores the state of isActivity attribute in the session
			session.setAttribute("formgenerator.isActivity", this.isActivity);

			//Stores the projectname in the session
			session.setAttribute("formgenerator.projectName", this.projectName);
			
			//Stores the project version in the session
			session.setAttribute("formgenerator.projectVersion", this.projectVersion);

			//Stores the activity name in the session
			session.setAttribute("formgenerator.activityName", this.activityName);

			//Stores the application mode in the session
			//session.setAttribute("formgenerator.mode", this.mode);

			//Stores the servletAdapter
			session.setAttribute("formgenerator.sAdapter", this.servletAdapter);

			//Closing writer
			out.close();

		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}


    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected void updateContext(HttpServletRequest request, HttpSession session, Document instance, Document langDocument) {
    	this.servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    	this.servletAdapter.setContextProperty("instance",instance);
		this.servletAdapter.setContextProperty("lang",langDocument);
    }


	/**
	 * Servlet reaction on post method:
	 * Submit information from user
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response)	throws ServletException, IOException
	{
		//Getting the current session
		HttpSession session = request.getSession(true);
		this.servletAdapter = null;

		try {
			//set the content-type and character encoding for request and response
			response.setContentType("text/html; charset=" + FormGeneratorConstant.CHARACTER_ENCODING);
			request.setCharacterEncoding(FormGeneratorConstant.CHARACTER_ENCODING);

			//Getting servletAdapter from the session
			this.servletAdapter = (ServletAdapter)session.getAttribute("formgenerator.sAdapter");

			//Getting isActivity from the session
			this.isActivity = (Boolean)session.getAttribute("formgenerator.isActivity");
			//Set isActivity in the session
			session.setAttribute("formgenerator.isActivity", this.isActivity);

			//Getting projectName from the session
			this.projectName = (String)session.getAttribute("formgenerator.projectName");
			//Set the projectname in the session
			session.setAttribute("formgenerator.projectName", this.projectName);

			//Getting projectVersion from the session
			projectVersion = (String)session.getAttribute("formgenerator.projectVersion");
			//Set the projectversion in the session
			session.setAttribute("formgenerator.projectVersion", projectVersion);
			
			//Getting activityName from the session
			this.activityName = (String)session.getAttribute("formgenerator.activityName");
			//Set the activityname in the session
			session.setAttribute("formgenerator.activityName", this.activityName);

			//Getting mode from the session
			//this.mode = (String)session.getAttribute("formgenerator.mode");

			//Getting forward from the session
			this.forward = (String)session.getAttribute("formgenerator.forward");

			//Set isActivity in the session
			session.setAttribute("formgenerator.forward", this.forward);

			if ((this.servletAdapter == null)||(this.isActivity == null)||(this.projectName == null)){
				//Exception
				throw new IOException("Invalid session - Cannot retrieve servletAdapter or ggf or isActivity or projectName in the session");
			}

			updateContextPost(request, session);

            //this.servletAdapter.executeHandler();
			this.servletAdapter.dispatch(null);

            // handle setRedirect <xforms:load show='replace'/>
            // and redirect from submission as well
            // NOTE - this needs to be checked *before* the this.getForwardMap()
            // as a submission handler may force a redirect
            if (this.servletAdapter.getRedirectUri() != null) {
                String redirectTo = this.servletAdapter.getRedirectUri();
                // todo: remove from session ?
                // shutdown processor
                this.servletAdapter.getChibaBean().shutdown();

                // send redirect (after encoding session id if required)
                response.sendRedirect(response.encodeRedirectURL(redirectTo));

                // remove redirect uri and terminate
                this.servletAdapter.setRedirect(null);
                return;
            }

            // handle forward <xforms:submission replace='all'/>
            Map forwardMap = this.servletAdapter.getForwardMap();
            InputStream forwardStream = (InputStream) forwardMap.get(ChibaAdapter.SUBMISSION_RESPONSE_STREAM);
            if (forwardStream != null) {
			        //get instance filled
					//Document filledDocument = (Document)this.servletAdapter.getChibaBean().getContainer().getModel("model").getInstance("instance").getInstanceDocument();
					Model model = (Model)this.servletAdapter.getChibaBean().lookup("model");
					Document filledDocument = (Document)model.getInstance("instance").getInstanceDocument();

		            if (filledDocument == null) {
		                throw new ServletException(Config.getInstance().getErrorMessage("session-invalid"));
		            }

					//Calling the executor
					//this.callExecutor(filledDocument);

					//redirect
					/*
					if ((this.forward!=null)&&!this.forward.equals("")){
						//redirection to forward
					    response.sendRedirect(this.forward);
					}else{
						if(this.isActivity.booleanValue() == true){

						    if(this.mode.equals("consumer"))
						        //Redirection to todo list
								response.sendRedirect("/formgenerator/ServletActivityToDoList?projectName=" + projectName);
						    else
						        //Redirection to work list
								response.sendRedirect("/formgenerator/ServletWorkList");
						}
						else{
						    if(this.mode.equals("consumer"))
						        //Redirection to instances projets
						        response.sendRedirect("/formgenerator/ServletInstancesProjects");
						    else
						        //Redirection to work list
								response.sendRedirect("/formgenerator/ServletWorkList");
						}
					}
					*/
					/*
	                // todo: remove from session ?
	                // shutdown processor
	                this.servletAdapter.getChibaBean().shutdown();

	                // forward submission response
	                forwardResponse(forwardMap, response);

	                // remove forward response and terminate
	                this.servletAdapter.forward(null);
	                return;
	                */
            }

			//Set content type
			//response.setContentType("text/html");

			//Render result to output
			//this.chiba.runChiba(out);
			this.servletAdapter.buildUI(response.getWriter());
			response.getWriter().close();

		} catch (Exception e) {
			//Error
			e.printStackTrace();
		}
	}


    /**
     * this method is responsible for passing all context information needed by the Adapter and Processor from
     * ServletRequest to ChibaContext.
     *
     * @param servletAdapter the ChibaAdapter to use
     * @param request        the ServletRequest
     * @param session        the ServletSession
     */
    protected void updateContextPost(HttpServletRequest request, HttpSession session) {
    	this.servletAdapter.setContextProperty(ServletAdapter.USERAGENT, request.getHeader("User-Agent"));
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SERVLET_REQUEST, request);
    	this.servletAdapter.setContextProperty(ServletAdapter.HTTP_SESSION_OBJECT, session);
    }


    /**
     * creates and configures the ServletAdapter which does the actual request processing.
     *
     * @param actionURL - the URL to submit to
     * @param session   - the Servlet session
     * @param formPath  - the relative location where forms are stored
     * @param xslFile   - the xsl file to use for transform
     * @param cssFile   - the CSS file to use for styling the output
     * @return ServletAdapter
     */
    private ServletAdapter setupServletAdapter(String actionURL, String hostUrl,
                                               HttpSession session,
                                               Document formDocument, String baseURI,
                                               String xslFile,
                                               String cssFile) throws XFormsException, URISyntaxException {
        //setup and configure the adapter
        ServletAdapter aAdapter = new ServletAdapter();
        aAdapter.setContextRoot(contextRoot);
        if ((configPath != null) && !(configPath.equals(""))) {
            aAdapter.setConfigPath(configPath);
        }

		//call getDocumentFromString to get a valid Chiba Document
		try {
			formDocument = getDocumentFromString(toString(formDocument));
		} catch (Exception e) {
			e.printStackTrace();
		}

		//aAdapter.setFormURI(new URI(this.contextRoot));
		aAdapter.setXFormsURI(this.contextRoot);
        //aAdapter.setFormDocument(formDocument);
		aAdapter.setXForms(formDocument);
        aAdapter.setStylesheetPath(stylesPath);
        aAdapter.setActionUrl(actionURL);
        aAdapter.setUploadDir(uploadDir);
		aAdapter.setHostUrl(hostUrl);

        if (xslFile != null) {
            aAdapter.setStylesheet(xslFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using xsl stylesheet: " + xslFile);
            }
        }
        if (cssFile != null) {
            aAdapter.setCSS(cssFile);
            if (cat.isDebugEnabled()) {
                cat.debug("using css stylesheet: " + cssFile);
            }
        }

        Map servletMap = new HashMap();
        servletMap.put(ChibaAdapter.SESSION_ID, session.getId());
        aAdapter.setContextProperty(ChibaAdapter.SUBMISSION_RESPONSE, servletMap);

        return aAdapter;
    }


    private void storeContextParams(HttpServletRequest request) {
        Enumeration params = request.getParameterNames();
        String s;
        while (params.hasMoreElements()) {
            s = (String) params.nextElement();
            //store all request-params we don't use in the context map of ChibaBean
            if (!(s.equals(XSL_PARAM_NAME) || s.equals(CSS_PARAM_NAME) || s.equals(ACTIONURL_PARAM_NAME))) {
                String value = request.getParameter(s);
                this.servletAdapter.setContextProperty(s, value);
                if (cat.isDebugEnabled()) {
                    cat.debug("added request param '" + s + "' added to context");
                }
            }
        }
    }


	/**
	 * Return the action URL with jsession
	 * @param request : The request from user
	 * @param response : The response to its request
	 * @return a String object that represents the action URL
	 */
    private String getActionURL(HttpServletRequest request, HttpServletResponse response) {
        String defaultActionURL =
                request.getScheme()
                + "://"
                + request.getServerName()
                + ":"
                + request.getServerPort()
                + request.getContextPath()
                + request.getServletPath();
        String encodedDefaultActionURL = response.encodeURL(defaultActionURL);
        int sessIdx = encodedDefaultActionURL.indexOf(";jsession");
        String sessionId = null;
        if (sessIdx > -1) {
            sessionId = encodedDefaultActionURL.substring(sessIdx);
        }
        String actionURL = request.getParameter(ACTIONURL_PARAM_NAME);
        if (null == actionURL) {
            actionURL = encodedDefaultActionURL;
        } else if (null != sessionId) {
            actionURL += sessionId;
        }

        //cat.info("actionURL: " + actionURL);
        // encode the URL to allow for session id rewriting
        actionURL = response.encodeURL(actionURL);
        return actionURL;
    }

	private Document getDocument(String path) {

		//DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(false);
        factory.setAttribute("http://apache.org/xml/properties/dom/document-class-name",
                "org.chiba.xml.xforms.XFormsDocument");

		Document document = null;
		try {
			//get file and parse it to xmlDocument
			File xmlFile = new File(path);
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			document = documentBuilder.parse(xmlFile);
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return document;
	}

	private Document getDocumentFromString(String theString) {

		//DocumentBuilderFactory factory = new org.apache.xerces.jaxp.DocumentBuilderFactoryImpl();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(false);
        factory.setAttribute("http://apache.org/xml/properties/dom/document-class-name",
                "org.chiba.xml.xforms.XFormsDocument");

		Document document = null;
		try {
			//get file and parse it to xmlDocument
			DocumentBuilder documentBuilder = factory.newDocumentBuilder();
			document = documentBuilder.parse(new ByteArrayInputStream(theString.getBytes()));
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
		return document;
	}

	private String getUserLanguage(String languages, String languagePath) {

		//Use a tokenizer to separate acceptable languages
	    StringTokenizer tokenizer = new StringTokenizer(languages, ",");
		String lang = null;
		boolean fileFound = false;
	    while (tokenizer.hasMoreTokens() && !fileFound) {
			lang = tokenizer.nextToken();
			int semi,dash;
			//Cut off any q-value that might come after a semi-colon
		    if ((semi = lang.indexOf(';')) != -1) {
		      lang = lang.substring(0, semi);
		    }
			//Trim any whitespace
		    lang = lang.trim();
			//A dash may separate the
		    //language from the country.
		    if ((dash = lang.indexOf('-')) != -1) {
				lang = lang.substring(0, dash);
		    }
			//try to find lang file
			File tmp = new File(languagePath + lang + ".xml");
			if (tmp.exists()) {
				fileFound = true;
			}
	    }
		if(!fileFound){
			lang = "en";
		}

		return lang;
	}

    private void forwardResponse(Map forwardMap, HttpServletResponse response) throws IOException {
        // fetch response stream
        InputStream responseStream = (InputStream) forwardMap.remove(ChibaAdapter.SUBMISSION_RESPONSE_STREAM);

        // copy header information
        Iterator iterator = forwardMap.keySet().iterator();
        while (iterator.hasNext()) {
            String name = iterator.next().toString();
            String value = forwardMap.get(name).toString();
            response.setHeader(name, value);
        }

        // copy stream content
        OutputStream outputStream = new BufferedOutputStream(response.getOutputStream());
        for (int b = responseStream.read();
             b > -1;
             b = responseStream.read()) {
            outputStream.write(b);
        }

        // close streams
        responseStream.close();
        outputStream.close();
    }

	// Retourne un Document en String
	private static String toString(Document doc) throws Exception{

	    ByteArrayOutputStream os = new ByteArrayOutputStream();
	    OutputFormat outputFormat = new OutputFormat(doc);
	    outputFormat.setPreserveSpace(true);
	    outputFormat.setIndenting(true);
	    //outputFormat.setLineWidth(0);
	    XMLSerializer serializer = new XMLSerializer(os, outputFormat);
	    serializer.serialize(doc);

	    return new String(os.toByteArray());
	 }


}